/**
 * ==================  angular-ios9-uiwebview.patch.js v1.1.0 ==================
 *
 * This patch works around iOS9 UIWebView regression that causes infinite digest
 * errors in Angular.
 *
 * The patch can be applied to Angular 1.2.0 – 1.4.5. Newer versions of Angular
 * have the workaround baked in.
 *
 * To apply this patch load/bundle this file with your application and add a
 * dependency on the "ngIOS9Patch" module to your main app module.
 *
 * For example:
 *
 * ```
 * angular.module('myApp', ['ngRoute'])`
 * ```
 *
 * becomes
 *
 * ```
 * angular.module('myApp', ['ngRoute', 'ngIOS9UIWebViewPatch'])
 * ```
 *
 *
 * More info:
 * - https://openradar.appspot.com/22186109
 * - https://github.com/angular/angular.js/issues/12241
 * - https://github.com/driftyco/ionic/issues/4082
 *
 *
 * @license AngularJS
 * (c) 2010-2015 Google, Inc. http://angularjs.org
 * License: MIT
 */
angular.module('ngIOS9UIWebViewPatch', ['ng']).config([
  '$provide',
  function ($provide) {
    $provide.decorator('$browser', [
      '$delegate',
      '$window',
      function ($delegate, $window) {
        if (isIOS9UIWebView($window.navigator.userAgent)) {
          return applyIOS9Shim($delegate);
        }
        return $delegate;
        function isIOS9UIWebView(userAgent) {
          return /(iPhone|iPad|iPod).* OS 9_\d/.test(userAgent) && !/Version\/9\./.test(userAgent);
        }
        function applyIOS9Shim(browser) {
          var pendingLocationUrl = null;
          var originalUrlFn = browser.url;
          browser.url = function () {
            if (arguments.length) {
              pendingLocationUrl = arguments[0];
              return originalUrlFn.apply(browser, arguments);
            }
            return pendingLocationUrl || originalUrlFn.apply(browser, arguments);
          };
          window.addEventListener('popstate', clearPendingLocationUrl, false);
          window.addEventListener('hashchange', clearPendingLocationUrl, false);
          function clearPendingLocationUrl() {
            pendingLocationUrl = null;
          }
          return browser;
        }
      }
    ]);
  }
]);
'use strict';
var app = angular.module('kidreports2App', [
    'ui.router',
    'ui.bootstrap',
    'ui.calendar',
    'ngResource',
    'com.2fdevs.videogular',
    'com.2fdevs.videogular.plugins.poster',
    'ngCookies',
    'ng-multiselecttree',
    'pascalprecht.translate',
    'ui.calendar',
    'ngIOS9UIWebViewPatch',
    'angularMoment',
    'ngSanitize',
    'pageslide-directive',
    'ngMaterial',
    'ngMessages'
  ]).config([
    '$stateProvider',
    '$urlRouterProvider',
    '$compileProvider',
    '$sceDelegateProvider',
    '$httpProvider',
    '$translateProvider',
    'translationsProvider',
    function ($stateProvider, $urlRouterProvider, $compileProvider, $sceDelegateProvider, $httpProvider, $translateProvider, translationsProvider) {
      //$compileProvider.debugInfoEnabled(false); // disable debug to improve production performance, change to true or comment out to collect debug info
      translationsProvider.configure($translateProvider);
      //.then(init);
      init();
      function init() {
        $sceDelegateProvider.resourceUrlWhitelist([
          'self',
          'http://iactivev.s3.amazonaws.com/**'
        ]);
        $httpProvider.defaults.withCredentials = true;
        //$compileProvider.imgSrcSanitizationWhitelist(/^\s*(https?|ftp|mailto|file|tel|content):/);
        $compileProvider.imgSrcSanitizationWhitelist(/^\s*(https?|ftp|mailto|file|tel|content|blob):|data:image|/);
        // note: you'll also have to do imgSrcSanitizationWhitelist if you want to use gene
        $compileProvider.aHrefSanitizationWhitelist(/^\s*(https?|ftp|mailto|file|tel|content):/);
        /////////////////////////////
        // Redirects and Otherwise //
        /////////////////////////////
        $urlRouterProvider.rule(function ($injector, $location) {
          var path = $location.path();
          // security check.  Force login if trying to go to any other page.
          var unauthorizedPages = [
              '/login',
              '/login/forgot-username',
              '/login/forgot-password'
            ];
          var pathRequiresAuth = !_.includes(unauthorizedPages, path);
          if (pathRequiresAuth && !$injector.get('$facility').getSync()) {
            return '/login';
          }
        });
        $urlRouterProvider.when('/home', '/');
        $stateProvider.state('activities', {
          url: '/activities',
          views: {
            'main@': {
              templateUrl: 'views/activities.html',
              controller: 'ActivitiesController',
              controllerAs: 'vm'
            }
          }
        }).state('home', {
          url: '',
          abstract: true,
          views: {
            'main@': {
              templateUrl: 'views/main.html',
              controller: 'NavCtrl'
            },
            'help@home': {
              templateUrl: 'views/help.html',
              controller: 'HelpCtrl'
            },
            'header@home': { templateUrl: 'views/navbar.html' },
            'messageArea@home': { templateUrl: 'views/footer.html' }
          }
        }).state('home.main', {
          url: '/',
          views: {
            'container@home': {
              templateUrl: 'views/topMenu.html',
              controller: 'TopmenuCtrl'
            },
            'footer@home': { templateUrl: 'views/attendanceCounter.html' },
            'activityfeed@home.main': { templateUrl: 'views/activityStream.html' },
            'mainmenu@home.main': { templateUrl: 'views/topMenuWrapper.html' },
            'memories@home.main': {
              templateUrl: 'views/album.html',
              controller: 'AlbumCtrl'
            },
            'customArea@home': { templateUrl: null }
          }
        }).state('parent-home', {
          url: '/parent-home',
          views: {
            'main@': {
              templateUrl: 'views/parent-home.html',
              controller: 'ParentHomeController',
              controllerAs: 'vm'
            }
          }
        }).state('memories', {
          url: '/memories',
          views: {
            'main@': {
              templateUrl: 'views/memories.html',
              controller: 'MemoriesController',
              controllerAs: 'vm'
            }
          }
        }).state('attendance', {
          url: '/attendance',
          params: {
            tab: null,
            mode: null
          },
          views: {
            'main@': {
              templateUrl: 'views/attendance-full.html',
              controller: 'AttendanceController',
              controllerAs: 'vm'
            }
          }
        }).state('attendanceOptions', {
          url: '/attendance/options',
          params: {
            room: null,
            people: null
          },
          views: {
            'main@': {
              templateUrl: 'views/attendance-options.html',
              controller: 'AttendanceOptionsController',
              controllerAs: 'vm'
            }
          }
        }).state('messageNew', {
          url: '/message/new',
          views: {
            'main@': {
              templateUrl: 'views/message-new.html',
              controller: 'MessageNewController'
            }
          }
        }).state('messageReply', {
          url: '/message/reply/:messageId',
          views: {
            'main@': {
              templateUrl: 'views/message-reply.html',
              controller: 'MessageReplyController'
            }
          }
        }).state('message', {
          url: '/message/:messageId',
          views: {
            'main@': {
              templateUrl: 'views/message.html',
              controller: 'MessageController'
            }
          }
        }).state('messages', {
          url: '/messages',
          views: {
            'main@': {
              templateUrl: 'views/messages.html',
              controller: 'MessagesController'
            }
          }
        }).state('home.main.tab', {
          url: '/tab',
          views: {},
          abstract: true
        }).state('home.main.tab.activityfeed', {
          url: '/activityfeed',
          views: {
            'activityfeed@home.main': { templateUrl: 'views/activityStreamInline.html' },
            'messages@home.main': { templateUrl: null },
            'memories@home.main': { templateUrl: null }
          }
        }).state('home.dash.nameToFace', {
          url: '/nameToFace',
          views: {
            'main@': {
              templateUrl: 'views/dashNameToFace.html',
              controller: 'NametofaceCtrl'
            }
          }
        }).state('home.dash.whereIs', {
          url: '/whereIs',
          views: {
            'main@': {
              templateUrl: 'views/whereIs.html',
              controller: 'WhereIsCtrl'
            }
          }
        }).state('home.dash.trackingSheet', {
          url: '/trackingSheet',
          views: {
            'main@': {
              templateUrl: 'views/cwDashNameToFace.html',
              controller: 'CWNametofaceCtrl'
            }
          }
        }).state('home.dash.trackingChart', {
          url: '/trackingChart/:trackingChartId',
          views: {
            'main@': {
              templateUrl: 'views/trackingChartDash.html',
              controller: 'TrackingChartCtrl'
            }
          }
        }).state('home.main.tab.messages', {
          url: '/memories',
          views: {
            'activityfeed@home.main': { templateUrl: null },
            'memories@home.main': { templateUrl: null }
          }
        }).state('home.refresh', {
          url: '/',
          views: {
            'container@home': {
              templateUrl: 'views/topMenu.html',
              controller: 'TopmenuCtrl'
            },
            'customArea@home': {
              templateUrl: 'views/mainTopBar.html',
              controller: 'TopmenubarCtrl'
            }
          }
        }).state('help', {
          url: '/help',
          views: {
            'main@': {
              templateUrl: 'views/help.html',
              controller: 'HelpCtrl'
            }
          }
        }).state('loginForgotUsername', {
          url: '/login/forgot-username',
          views: {
            'main@': {
              templateUrl: 'views/login-forgot-username.html',
              controller: 'LoginForgotUsernameController'
            }
          }
        }).state('loginForgotPassword', {
          url: '/login/forgot-password',
          views: {
            'main@': {
              templateUrl: 'views/login-forgot-password.html',
              controller: 'LoginForgotPasswordController'
            }
          }
        }).state('login', {
          url: '/login?u&f&k&go',
          views: {
            'main@': {
              templateUrl: 'views/login.html',
              controller: 'LoginController'
            }
          }
        }).state('logout', {
          url: '/logout',
          views: {
            'main@': {
              templateUrl: 'views/login.html',
              controller: 'LogoutCtrl'
            },
            'footer@login': { templateUrl: 'views/footer.html' }
          }
        }).state('home.action', {
          url: '/action',
          abstract: true
        }).state('home.action.selectRoom', {
          url: '/selectRoom',
          views: {
            'container@home': {
              templateUrl: 'views/pageSingleSelect.html',
              controller: 'RoomsCtrl'
            }
          }
        }).state('home.action.signInMain', {
          url: '/kiosk',
          views: {
            'header@home': { templateUrl: null },
            'container@home': {
              templateUrl: 'views/kiosk.html',
              controller: 'KioskCtrl'
            }
          }
        }).state('home.action.kioskMain', {
          url: '/kioskMain',
          views: {
            'header@home': { templateUrl: null },
            'container@home': {
              templateUrl: 'views/kioskmain.html',
              controller: 'KioskmainCtrl'
            },
            'container@home.action.kioskMain': {
              templateUrl: 'views/attendance.html',
              controller: 'AttendanceCtrl'
            }
          }
        }).state('home.action.kidPlanner', {
          url: '/kidPlanner',
          abstract: true,
          views: {
            'header@home': { templateUrl: null },
            'header@home.action.kidPlanner': { templateUrl: 'views/navbar.html' },
            'container@home': {
              templateUrl: 'views/kidplanner.html',
              controller: 'KidplannerCtrl'
            }
          }
        }).state('home.action.kidPlanner.main', { url: '/main' }).state('home.action.kidPlanner.eventDetails', {
          url: '/eventDetails',
          views: {
            'kidPlannerModal@home.action.kidPlanner': {
              templateUrl: 'views/event_full.html',
              controller: 'EventdetailsCtrl'
            }
          }
        }).state('home.action.kidPlanner.selectEvent', {
          url: '/selectEvent',
          views: {
            'kidPlannerModal@home.action.kidPlanner': {
              templateUrl: 'views/activityPageSelect.html',
              controller: 'EventsController'
            }
          }
        }).state('home.action.profilePlanner', {
          url: '/profilePlanner',
          views: {
            'container@home': {
              templateUrl: 'views/ProfilePlanner.html',
              controller: 'ProfileplannerCtrl'
            }
          }
        }).state('home.action.bulkPlanner', {
          url: '/bulkPlanner',
          views: {
            'header@home': { templateUrl: null },
            'container@home': {
              templateUrl: 'views/bulkPlanner.html',
              controller: 'BulkplannerCtrl'
            }
          }
        }).state('home.action.selectChild', {
          url: '/selectChild',
          views: {
            'main@': {
              templateUrl: 'views/childSelect.html',
              controller: 'ChildrenSingleController'
            },
            'customArea@home': {
              templateUrl: 'views/childSelectHeader.html',
              controller: 'ChildselectheaderCtrl'
            }
          }
        }).state('home.action.selectEvent', {
          url: '/selectEvent',
          views: {
            'main@': {
              templateUrl: 'views/activityPageSelect.html',
              controller: 'EventsController'
            }
          }
        }).state('home.action.eventDetails', {
          url: '/eventDetails',
          views: {
            'main@': {
              templateUrl: 'views/event_full.html',
              controller: 'EventdetailsCtrl'
            }
          }
        }).state('home.action.eventObservations', {
          url: '/eventObservations',
          views: {
            'container@home': {
              templateUrl: 'views/event_observations.html',
              controller: 'EventobservationsCtrl'
            }
          }
        }).state('home.action.selectChildren', {
          url: '/selectChildren',
          views: {
            'container@home': {
              templateUrl: 'views/childSelect.html',
              controller: 'ChildMultiSelectController'
            },
            'customArea@home': {
              templateUrl: 'views/childSelectHeader.html',
              controller: 'ChildselectheaderCtrl'
            }
          }
        }).state('home.action.pdf', {
          url: '/pdf',
          views: {
            'container@home': {
              templateUrl: 'views/integrations/pdf.html',
              controller: 'PDFCtrl'
            }
          }
        }).state('home.action.viewEvents', {
          url: '/viewEvents',
          views: {
            'container@home': {
              templateUrl: 'views/viewEvents.html',
              controller: 'VieweventsCtrl'
            }
          }
        }).state('home.action.moveRoom', {
          url: '/moveRoom',
          views: {
            'container@home': {
              templateUrl: 'views/moveRoom.html',
              controller: 'MoveroomCtrl'
            }
          }
        }).state('home.action.abc', {
          url: '/abc',
          parent: 'home',
          views: {
            'container@home': {
              templateUrl: 'views/integrations/abc.html',
              controller: 'AbcCtrl'
            }
          }
        }).state('home.action.coe', {
          url: '/coe',
          parent: 'home',
          views: {
            'container@home': {
              templateUrl: 'views/integrations/coe.html',
              controller: 'CoeCtrl'
            }
          }
        }).state('home.action.pbnj', {
          url: '/pbnj',
          parent: 'home',
          views: {
            'container@home': {
              templateUrl: 'views/integrations/pbnj.html',
              controller: 'PbnjCtrl'
            }
          }
        }).state('home.action.settings', {
          url: '/settings',
          views: {
            'main@': {
              templateUrl: 'views/settings.html',
              controller: 'SettingsCtrl'
            }
          }
        }).state('home.action.attendance', {
          url: '/attendance',
          views: {
            'container@home': {
              templateUrl: 'views/attendance.html',
              controller: 'AttendanceCtrl'
            },
            'customArea@home': {
              templateUrl: 'views/attendanceHeader.html',
              controller: 'AttendanceheaderCtrl'
            }
          }
        }).state('home.action.overview', {
          url: '/overview',
          views: {
            'container@home': {
              templateUrl: 'views/overview.html',
              controller: 'OverviewCtrl'
            },
            'customArea@home': {
              templateUrl: 'views/overviewHeader.html',
              controller: 'OverviewheaderCtrl'
            }
          }
        }).state('home.action.teacherSignIn', {
          url: '/teacherSignIn',
          views: {
            'container@home': {
              templateUrl: 'views/teacherSignIn.html',
              controller: 'AttendanceCtrl'
            },
            'customArea@home': {
              templateUrl: 'views/attendanceHeader.html',
              controller: 'AttendanceheaderCtrl'
            }
          }
        }).state('home.action.manageRoom', {
          url: '/manageRoom',
          views: {
            'container@home': {
              templateUrl: 'views/manageRoom.html',
              controller: 'ManageroomCtrl'
            }
          }
        }).state('home.dash.pickDraft', {
          url: '/pickDraft',
          views: {
            'dashPopup@home.dash': {
              templateUrl: 'views/pickDraft.html',
              controller: 'PickDraftController'
            }
          }
        }).state('home.dash.quicklinks', {
          url: '/quicklinks',
          views: {
            'main@': {
              templateUrl: 'views/quicklinks.html',
              controller: 'QuicklinksController'
            }
          }
        }).state('home.action.policies', {
          url: '/policies',
          views: {
            'container@home': {
              templateUrl: 'views/policies.html',
              controller: 'PoliciesCtrl'
            }
          }
        }).state('home.dash.policies', {
          url: '/policies/:storyId',
          views: {
            'dashPopup@home.dash': {
              templateUrl: 'views/policies.html',
              controller: 'PoliciesCtrl'
            }
          }
        }).state('home.action.editMyProfile', {
          url: '/editMyProfile',
          views: {
            'container@home': {
              templateUrl: 'views/editMyProfile.html',
              controller: 'EditmyprofileCtrl'
            }
          }
        }).state('home.dash.reminders', {
          url: '/reminders',
          views: {
            'main@': {
              templateUrl: 'views/reminders.html',
              controller: 'RemindersController'
            }
          }
        }).state('home.action.sendMessage', {
          url: '/sendMessage',
          views: {
            'container@home': {
              templateUrl: 'views/sendMessage.html',
              controller: 'SendmessageCtrl'
            }
          }
        }).state('home.action.learningStories', {
          url: '/learningstories/:storyId',
          views: {
            'container@home': {
              templateUrl: 'views/learningStories.html',
              controller: 'LearningStoriesCtrl'
            }
          }
        }).state('home.dash.learningStories', {
          url: '/learningstories/:storyId',
          views: {
            'dashPopup@home.dash': {
              templateUrl: 'views/learningStories.html',
              controller: 'LearningStoriesCtrl'
            }
          }
        }).state('home.action.takePhotoVideo', {
          url: '/takePhotoVideo',
          views: {
            'container@home': {
              templateUrl: 'views/takePhotoVideo.html',
              controller: 'MediaCtrl'
            }
          }
        }).state('home.dash', {
          url: '/dashboard',
          views: {
            'header@home': { templateUrl: null },
            'header@home.dash': { templateUrl: 'views/navbar.html' },
            'container@home': { templateUrl: 'views/dashboard.html' },
            'singleChildList@home.dash': {
              templateUrl: 'views/dashSingleChildSelect.html',
              controller: 'ChildrenSingleController'
            },
            'activitiesList@home.dash': {
              templateUrl: 'views/selectActivity.html',
              controller: 'EventsController'
            },
            'dashContent@home.dash': { templateUrl: 'views/activityStream.html' }
          }
        }).state('home.dash.event', {
          url: '/event',
          views: {
            'dashPopup@home.dash': {
              templateUrl: 'views/event_full.html',
              controller: 'EventdetailsCtrl'
            }
          }
        }).state('home.dash.eventObservations', {
          url: '/eventObservations',
          views: {
            'dashPopup@home.dash': {
              templateUrl: 'views/event_observations.html',
              controller: 'EventobservationsCtrl'
            }
          }
        }).state('home.dash.pictureevent', {
          url: '/pictureevent',
          views: {
            'dashPopup@home.dash': {
              templateUrl: 'views/photoVideo.html',
              controller: 'EventdetailsCtrl'
            }
          }
        }).state('home.dash.videoevent', {
          url: '/videoevent',
          views: {
            'dashPopup@home.dash': {
              templateUrl: 'views/photoVideo.html',
              controller: 'EventdetailsCtrl'
            }
          }
        }).state('home.dash.takePhotoVideo', {
          url: '/takePhotoVideo',
          views: {
            'dashPopup@home.dash': {
              templateUrl: 'views/takePhotoVideo.html',
              controller: 'MediaCtrl'
            }
          }
        }).state('home.dash.attendance', {
          url: '/attendance',
          views: {
            'dashPopup@home.dash': {
              templateUrl: 'views/dashAttendance.html',
              controller: 'AttendancedashCtrl'
            },
            'container@home.dash.attendance': {
              templateUrl: 'views/attendance.html',
              controller: 'AttendanceCtrl'
            },
            'customArea@home.dash.attendance': {
              templateUrl: 'views/attendanceHeader.html',
              controller: 'AttendanceheaderCtrl'
            }
          }
        }).state('home.dash.overview', {
          url: '/overview',
          views: {
            'dashPopup@home.dash': {
              templateUrl: 'views/dashOverview.html',
              controller: 'OverviewdashCtrl'
            },
            'container@home.dash.overview': {
              templateUrl: 'views/overview.html',
              controller: 'OverviewCtrl'
            },
            'customArea@home.dash.overview': {
              templateUrl: 'views/overviewHeader.html',
              controller: 'OverviewheaderCtrl'
            }
          }
        }).state('home.dash.teacherSignIn', {
          url: '/teacherSignIn',
          views: {
            'dashPopup@home.dash': {
              templateUrl: 'views/dashAttendance.html',
              controller: 'AttendancedashCtrl'
            },
            'container@home.dash.teacherSignIn': {
              templateUrl: 'views/teacherSignIn.html',
              controller: 'AttendanceCtrl'
            },
            'customArea@home.dash.teacherSignIn': {
              templateUrl: 'views/attendanceHeader.html',
              controller: 'AttendanceheaderCtrl'
            }
          }
        }).state('home.dash.sendMessage', {
          url: '/sendmessage',
          views: {
            'dashPopup@home.dash': {
              templateUrl: 'views/sendMessage.html',
              controller: 'SendmessageCtrl'
            }
          }
        }).state('home.dash.moveRoom', {
          url: '/moveRoom',
          views: {
            'dashPopup@home.dash': {
              templateUrl: 'views/moveRoom.html',
              controller: 'MoveroomCtrl'
            }
          }
        }).state('home.dash.activityPickerDialog', {
          url: '/activityPicker',
          views: {
            'dashPopup@home.dash': {
              templateUrl: 'views/activityPickerDialog.html',
              controller: 'NavCtrl'
            },
            'activitiesList@home.dash.activityPickerDialog': {
              templateUrl: 'views/selectActivity.html',
              controller: 'EventsController'
            }
          }
        }).state('home.dash.kidPlanner', {
          url: '/kidPlanner',
          views: {
            'header@home': { templateUrl: null },
            'header@home.dash': { templateUrl: 'views/navbar.html' },
            'dashPopup@home.dash': {
              templateUrl: 'views/kidplanner.html',
              controller: 'KidplannerCtrl'
            }
          }
        });
      }
    }
  ]);
app.run([
  '$appService',
  '$rootScope',
  '$native',
  '$online',
  '$backend',
  '$krCache',
  '$krConfig',
  '$ajax',
  '$messageBox',
  '$events',
  '$localdb',
  '$utils',
  '$timeout',
  '$appstate',
  '$facility',
  '$attendance',
  '$state',
  '$translate',
  function ($appService, $rootScope, $native, $online, $backend, $krCache, $krConfig, $ajax, $messageBox, $events, $localdb, $utils, $timeout, $appstate, $facility, $attendance, $state, $translate) {
    $native.setup();
    $(document).on('blur', 'input, textarea', function () {
      setTimeout(function () {
        window.scrollTo(document.body.scrollLeft, document.body.scrollTop);
      }, 0);
    });
    if ($native.isNative()) {
      $timeout(checkCache, 5000);
    } else {
    }
    if ($native.android()) {
      forge.event.backPressed.preventDefault(function () {
        forge.event.backPressed.addListener(function (closeApp, pauseApp) {
          $timeout(function () {
            if ($appstate.model.dashSelectChild) {
              var f = $appstate.model.dashSelectChild;
              $appstate.model.dashSelectChild = null;
              f();
            } else if ($appstate.model.imageurl != 'img/empty.png') {
              $appstate.model.imageurl = 'img/empty.png';
            } else if ($appstate.model.backButtonFunction) {
              var func = $appstate.model.backButtonFunction;
              console.log('backfunction set');
              $appstate.model.backButtonFunction = null;
              func();
            } else if (!$appstate.model.freezeBackButton && (!$state.is('home.dash') && !$state.is('home.main') && !$state.is('login'))) {
              $appstate.nav.back();
            } else if ($appstate.model.canClose) {
              var _f = $appstate.model.canClose;
              $appstate.model.canClose = null;
              _f.call();
            } else if ($appstate.isParent() && $state.is('home.main') && $appstate.model.parentmode != 'HOME') {
              console.log('last elseif');
              console.log('$appstate.isParent()');
              $timeout(function () {
                $appstate.model.parentmode = 'HOME';
              });  //if backButtonFunction is defined, set = to func, then call that function with func() after setting to null
            } else {
              pauseApp();
            }
          });
        }, function (err) {
          console.log(err);
        });
      }, function (err) {
        console.log(err);
      });
    }
    $rootScope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams) {
      if ($native.isNative())
        $timeout($native.resetiOSHeader, 500);
    });
    function checkCache() {
      var pingValid = false;
      if ($facility.getSync() && $facility.getSync().disableServerPing) {
        confirmCache();
      } else {
        var opts = {
            method: 'GET',
            url: $krConfig.getAjaxBase() + '/public/ping.json',
            params: {}
          };
        $ajax.ajax(opts, function (err, data) {
          //console.log("Got ping response");
          pingValid = true;
          if (err) {
            // offline
            confirmCache();
          } else {
          }
        });
        $timeout(function () {
          if (!pingValid) {
            confirmCache();
          }
        }, 2000);
      }
    }
    function confirmCache() {
      $krCache.getCache(null, function (cache) {
        if (cache && _.isArray(cache) && cache.length > 0)
          showMsg();
        else {
          $krCache.getCache('uploads', function (cache) {
            if (cache && _.isArray(cache) && cache.length > 0)
              showMsg();
          });
        }
      });
    }
    function showMsg() {
      $messageBox.open($translate.instant('OfflineEvents'), $translate.instant('OfflineEventsText'), { oktext: $translate.instant('Ok') });
    }
    if ($native.isNative()) {
      forge.event.appPaused.addListener(function (content) {
        restartUploads();  // force
                           //console.log("appPaused");
                           //$backend.cancelEventSave();
                           //if ( $appstate.isCacheEmpty() && $facility.getSync() && $facility.getSync().alwaysSyncOnResume ) {
                           //  $attendance.refreshAll(true);
                           //} else {
                           //     }
      }, function (err) {
      });
      forge.event.appResumed.addListener(function (content) {
        if ($facility.needsRefresh()) {
          $timeout(function () {
            $krCache.setCache([], 'errors');
            $krCache.setCache([], 'notifications');
            $appstate.refreshFacility();
          }, 10);
          return;
        }
        //console.log("appResumed cb");
        var isParent = $backend.getCurrentUser() && $backend.getCurrentUser().role == 'PARENT_ROLE';
        $appstate.model.isRefreshing = false;
        //setTimeout(checkCache, 4000);
        if ($appstate.isCacheEmpty() && $facility.getSync() && $facility.getSync().alwaysSyncOnResume) {
          $attendance.refreshAll(true);
        } else {
          restartUploads();
        }
        // check if it's tomorrow and reset state
        //console.log("checking last sync time");
        $localdb.get('lastSyncTime').then(function (t) {
          //console.log("lastSyncTime:"+t);
          if ($utils.isTomorrow(t)) {
            //console.log("AppResumed: isTomorrow == true");
            $timeout(function () {
              $krCache.setCache([], 'errors');
              $krCache.setCache([], 'notifications');
              $krCache.setCache([], 'drafts');
              $timeout($appstate.refreshFacility, 500);
            }, 50);
          } else if (isParent) {
            // force refresh for activity stream.
            var date = $appstate.model.currentDate || new Date();
            var kids = $appstate.model.observed;
            if (kids) {
              $appstate.refreshEvents(kids, date, true);
            }
          }
        }, function (err) {
          console.log(err);
        });
      }, function (err) {
        console.log(err);
      });
    }
    $rootScope.$on('onlineStateChange', function (evt, isOnline) {
      //console.log(evt + " online:" + isOnline);
      // if we are back online, and a facility is defined, start up the save timers.
      if (isOnline) {
        restartUploads();
      } else {
        $events.offline();
      }
    });
    function restartUploads() {
      if ($appstate.eventQueueSize() > 0) {
        $events.push();
      }
      if ($appstate.backendQueueSize() > 0) {
        $backend.saveEventInternal();
      }
      if ($appstate.mediaQueueSize() > 0) {
        // set all uploads to pending so they go thru...
        _.forEach($appstate.model.cachedMedia, function (item) {
          item.status = 'Pending';
        });
        $krCache.setCache($appstate.model.cachedMedia, 'uploads', $native.uploadInternal);
      }
    }
    function checkUploads() {
      restartUploads();
      $timeout(checkUploads, 60000);
    }
    $timeout(checkUploads, 60000);
    var ele = angular.element(document.querySelector('#onlineWarningDiv'))[0];
    if (ele) {
      ele.style.display = 'block';
    }
  }
]);
'use strict';
angular.module('kidreports2App').controller('LoginForgotPasswordController', LoginForgotPasswordController);
LoginForgotPasswordController.$inject = [
  '$scope',
  '$state',
  '$auth',
  '$uiAlert'
];
/**
 * @function LoginForgotPasswordController
 * @param {*} $scope 
 * @param {*} $state 
 * @param {*} $auth 
 * @param {*} $uiAlert 
 */
function LoginForgotPasswordController($scope, $state, $auth, $uiAlert) {
  $scope.email = '';
  $scope.lookup = function () {
    if ($scope.email == '') {
      $uiAlert.pushErrorAlert('Email could not be found.', 'LOGIN');
      $scope.navigateBack();
      return;
    }
    $auth.lookupUserPass(false, $scope.email).then(function (response) {
      if (response.error && response.error === 'true') {
        $uiAlert.pushErrorAlert(response.value, 'LOGIN');
      } else {
        $uiAlert.pushSuccessAlert(response.value, 'LOGIN');
      }
      $scope.navigateBack();
    }, function (err) {
      $uiAlert.pushErrorAlert(err, 'LOGIN');
    });
  };
  $scope.navigateBack = function () {
    $state.go('login');
  };
}
'use strict';
angular.module('kidreports2App').controller('LoginForgotUsernameController', LoginForgotUsernameController);
LoginForgotUsernameController.$inject = [
  '$scope',
  '$state',
  '$auth',
  '$uiAlert'
];
/**
 * @function LoginForgotUsernameController
 * @param {*} $scope 
 * @param {*} $state
 * @param {*} $auth 
 * @param {*} $uiAlert 
 */
function LoginForgotUsernameController($scope, $state, $auth, $uiAlert) {
  $scope.email = '';
  $scope.lookup = function () {
    if ($scope.email == '') {
      $uiAlert.pushErrorAlert('Email could not be found.', 'LOGIN');
      $scope.navigateBack();
      return;
    }
    $auth.lookupUserPass(true, $scope.email).then(function (response) {
      if (response.error) {
        $uiAlert.pushErrorAlert(response.value, 'LOGIN');
      } else {
        $uiAlert.pushSuccessAlert('Username sent.', 'LOGIN');
      }
      $scope.navigateBack();
    }, function (err) {
      $uiAlert.pushErrorAlert(err, 'LOGIN');
    });
  };
  $scope.navigateBack = function () {
    $state.go('login');
  };
}
'use strict';
angular.module('kidreports2App').controller('LoginController', LoginController);
LoginController.$inject = [
  '$rootScope',
  '$scope',
  '$appstate',
  '$state',
  '$backend',
  '$utils',
  '$learningstories',
  '$stateStandards',
  '$krConfig',
  '$messageBox',
  '$timeout',
  '$auth',
  '$facility',
  '$events',
  '$bootstrap',
  '$modal',
  '$localdb',
  '$stateParams',
  '$messages',
  '$native',
  'popover',
  '$krCache',
  'locale',
  'translations',
  '$eventAdapter',
  '$reminders',
  '$msg',
  '$translate',
  '$model',
  '$adapter',
  '$nav',
  '$uiAlert'
];
/**
   * @function LoginController
   * @author Joshua Belden <jcbelden@procaresoftware.com>
   * @constructor
   */
function LoginController($rootScope, $scope, $appstate, $state, $backend, $utils, $learningstories, $stateStandards, $krConfig, $messageBox, $timeout, $auth, $facility, $events, $bootstrap, $modal, $localdb, $stateParams, $messages, $native, popover, $krCache, locale, translations, $eventAdapter, $reminders, $msg, $translate, $model, $adapter, $nav, $uiAlert) {
  $scope.cancelled = false;
  $localdb.get('disableProfilePictureCaching').then(function (t) {
    $scope.model.disableProfilePictureCaching = t != null && (t == true || t == 'true');
  });
  // testing... disable for live deployment.
  $scope.native = $native.isNative();
  $model.settingsClickCount = 0;
  $scope.popup = { isOpen: false };
  $scope.togglePopup = function () {
    $model.settingsClickCount++;
    popover.show($scope);
  };
  $scope.navigateForgotUsername = function () {
    $state.go('loginForgotUsername');
  };
  $scope.navigateForgotPassword = function () {
    $state.go('loginForgotPassword');
  };
  $scope.model = $model;
  $localdb.get('disableNativeTimepicker').then(function (t) {
    $scope.model.disableNativeTimepicker = t != null && (t == true || t == 'true');
  });
  $scope.closeSettings = function () {
    $scope.shared.backdropClick();
    $localdb.set('disableNativeTimepicker', $scope.model.disableNativeTimepicker);
  };
  $scope.shared = $appstate;
  $scope.values = {
    presetUrls: [
      {
        label: 'KidReports Main',
        value: 'https://secure.kidreports.com'
      },
      {
        label: 'KidReports Stage',
        value: 'https://stage.kidreports.com'
      },
      {
        label: 'KidReports Test',
        value: 'https://test.kidreports.com'
      },
      {
        label: 'Other...',
        value: ''
      }
    ]
  };
  var url = $krConfig.getAjaxBase();
  if ($scope.native) {
    $scope.values.presetUrls.forEach(function (v) {
      if ($scope.values.server)
        return;
      if (url == '' && v.label == 'KidReports Main' || v.value == url)
        $scope.values.server = v;
    });
    if (!$scope.values.server) {
      $scope.values.server = $scope.values.presetUrls[$scope.values.presetUrls.length - 1];
      $scope.values.server.value = url;
    }
  } else {
    $scope.values.server = {
      label: '',
      value: ''
    };
  }
  // international hooks.
  var lang = locale.getLanguage();
  if (lang && lang.indexOf && lang.indexOf('pt') > -1) {
    // set language to portuguese
    // hack for now... fix when we add a third language
    translations.setLanguage(translations.getLanguages()[1]);
  }
  if (locale.getTimeZone() == 'Pacific/Auckland') {
    // set server to NZ
    $scope.values.server = $scope.values.presetUrls[1];
  }
  // first check for url params
  var q_username = $stateParams.u;
  var q_token = $stateParams.k;
  var q_go = $stateParams.go;
  var q_fac = $stateParams.f;
  if (q_token && q_username) {
    // first check if we have credentials, if we do and they aren't the same, force a logout.
    $auth.setCredentials(q_username, q_token).then(function () {
      if (q_fac) {
        $scope.isLoading = true;
        $scope.username = q_username;
        $facility.get().then(function (f) {
          if (f) {
            var usr = f.currentUser.username;
            // Before verifying if the user represented by the token matches the current facility user
            //  see if the user may be linked to the current facility.
            if (_.includes(f.currentUser.linkedIds, f.id)) {
              checkLogin();
            } else if (parseInt(q_fac, 10) != f.id || usr != q_username) {
              $facility.logout().then(function () {
                document.location.reload({ forceGet: true });
              });
            } else {
              checkLogin();
            }
          } else {
            checkLogin();
          }
        });
      } else {
        checkLogin();
      }
    });
  } else {
    checkLogin();
  }
  function filterChildrenByTeacher(facility, teacher) {
    function roomHasTeacher(room) {
      var ret = false;
      var i;
      if (room.teachers) {
        for (i = 0; i < room.teachers.length; i++) {
          if (room.teachers[i].id === teacher.id) {
            ret = true;
            break;
          }
        }
      }
      return ret;
    }
    var rooms = facility.rooms.filter(function (room) {
        return roomHasTeacher(room);
      });
    var ret = [];
    rooms.forEach(function (room) {
      ret = ret.concat(room.children);
    });
    return ret;
  }
  function cleanItem(item) {
    item.isIn = null;
    item.attendanceNotes = null;
    item.cachedAttendance = null;
    item.attendanceSet = null;
  }
  function login(isUpdate) {
    // Login completely hinges on successfully getting the facility
    $msg.msg($translate.instant('FetchingFacility'), true, 'info', 2000);
    $facility.get().then(function (facility) {
      if ($scope.cancelled) {
        cancel();
        return;
      }
      if ($scope.native && facility.currentUser.role != 'PARENT_ROLE') {
        $native.getFreeSpace(function (sizeInMB) {
          if (sizeInMB < 500)
            //alert(sizeInMB);
            $messageBox.open($translate.instant(''), $translate.instant('FreeSpaceWarning'), { oktext: $translate.instant('Ok') });
        });
      }
      $reminders.start();
      //$events.disableBulkUpdates(false);//facility.disableBulkUpdates);
      $events.disableBulkUpdates(facility.disableBulkUpdates);
      $native.setServerPingDisabled(facility.disableServerPing);
      $native.setUseMultiImageSelection(facility.enableMultiImageSelectInApp);
      var currentUser = facility.currentUser;
      $timeout(function () {
        // Sync today's events for the teacher's assigned rooms
        $localdb.get('lastSyncTime').then(function (t) {
          if ($utils.isTomorrow(t)) {
            $krCache.setCache([], 'errors');
            $krCache.setCache([], 'notifications');
            $krCache.setCache([], 'drafts');
            $appstate.model.isSynchingEvents = true;
            $appstate.model.isRefreshing = true;
            //var children = filterChildrenByTeacher(facility, currentUser);
            $events.purge();
            if (!$appstate.model.selectedRoom) {
              $appstate.model.selectedRoom = facility.rooms[0];
            }
            $localdb.set('lastSyncTime', new Date().getTime());
            $localdb.set('lastEventSynchTime', null);
            if (currentUser.role != 'PARENT_ROLE') {
              //                $eventAdapter.getEvents([], new Date(), true, null, $appstate.model.selectedRoom.id, false, false, false).then(function (d) {
              // after sync, update attendance for all kids
              facility.rooms.forEach(function (r) {
                if (r.teachers) {
                  r.teachers.forEach(function (t) {
                    cleanItem(t);
                  });
                }
                //                  });
                $facility.save().then(function () {
                  // update the attendance state
                  //                    $eventAdapter.getEvents($facility.getSync().observed, new Date(), false, null, null, false, true, false);
                  $appstate.model.isSynchingEvents = false;
                  $appstate.model.isRefreshing = false;  //$rootScope.$broadcast("eventSynchDone");
                });
              });
            } else {
              //                $eventAdapter.getEvents($facility.getSync().observed, new Date(), true, null, null, false, false, false).then(function() {
              $appstate.model.isSynchingEvents = false;
              $appstate.model.isRefreshing = false;  //                });
            }
          }
        }, function (err) {
          console.log(err);
        });
      }, 100);
      //  kiosk mode override
      $localdb.get('kioskMode').then(function (t) {
        if (t === 'true') {
          if (!$appstate.model.menuItems)
            $appstate.model.menuItems = facility.menuItems;
          $appstate.nav.startPath('signInKiosk');
        } else {
          doBootstrap();
        }
      });
      function doBootstrap() {
        // Check bootstrap logic
        $bootstrap.check($krConfig.version, isUpdate, currentUser).then(function () {
          if (q_go) {
            if (!$appstate.model.menuItems)
              $appstate.model.menuItems = facility.menuItems;
            $appstate.model.hideFrame = true;
            // The following checks are for paths that are launched from the admin.
            //  Certain pages hide features based on if the page is being rendered from admin.
            if (q_go === 'messageCenter') {
              $appstate.isAdminIntegration = true;
            }
            var editItem = 'editItem:';
            if (q_go.indexOf(editItem) > -1) {
              var id = q_go.substr(q_go.indexOf(editItem) + editItem.length);
              $events.getById(id).then(function (item) {
                $model.closeOnEventComplete = true;
                $model.currentEvent = $adapter.createEvent(item, $facility.getSync());
                if ($backend.getCurrentUser().role == 'PARENT_ROLE' && item.user.id != $backend.getCurrentUser().id) {
                  //** show parent popup
                  $eventAdapter.showEvent($adapter.processEvent(item));
                  //$model.currentEvent));
                  return;
                }
                $nav.startPath('viewEventDirect');
              });
            } else {
              $appstate.nav.startPath(q_go);
            }
          } else {
            // kick off msg initialization
            var token = $auth.getCredentialsSync().token;
            var creds = { 'Authorization': 'Bearer ' + token };
            $messages.setCredentials(creds).then(function (f) {
              $messages.get();
              $messages.getUnreadMessageCount().then(function (response) {
                $appstate.model.messageBadge = response.count;
              });
            });
            $learningstories.setCredentials(creds).then(function (f) {
              if ($facility.isLSEnabled())
                $learningstories.update($appstate.model);
            });
            // init state standards.
            $stateStandards.getFacility().then(function (facility) {
              $stateStandards.getPerformanceRatingLabels(facility.facilityId);
              if (facility.state && facility.state !== 'none') {
                $stateStandards.getStandards(facility.state);
              }
            });
            // invalidate room cache map
            $localdb.set('roomMap', []);
            // check if cache should be cleared or kept
            var formerUserRole = $localdb.get('formerUserRole').$$state.value;
            var formerUserDate = new moment($localdb.get('formerUserDate').$$state.value);
            var isFormerUserAvailable = formerUserRole && formerUserDate;
            if (isFormerUserAvailable) {
              var currentUserType = facility.currentUser.role;
              var formerUserType = formerUserRole === facility.currentUser.role;
              var currentDate = moment();
              var isCurrentUserAdminOrTeacher = currentUserType === 'ADMIN_ROLE' || currentUserType === 'TEACHER_ROLE';
              var isFormerUserAdminOrTeacher = formerUserType === 'ADMIN_ROLE' || formerUserType === 'TEACHER_ROLE';
              var isCurrentUserSimilar = isCurrentUserAdminOrTeacher && isFormerUserAdminOrTeacher;
              var isSameDay = currentDate.dayOfYear() === formerUserDate.dayOfYear();
              var isSameYear = currentDate.year() === formerUserDate.year();
              var isSameDate = isSameDay && isSameYear;
              // nuke localdb saved user state
              $localdb.set('formerUserRole', null);
              $localdb.set('formerUserDate', null);
            }
            var tooMuchStateChange = !(isCurrentUserSimilar && isSameDate);
            var shouldDatabaseBeCleared = tooMuchStateChange || !isFormerUserAvailable;
            if (shouldDatabaseBeCleared) {
              $events.cleardb();
            }
            $appstate.goHome(true);
          }
        });
      }
    }, function (err) {
      // Failed to get facility
      console.log(err);
      $appstate.msg($translate.instant('ErrorLoggingIn'), true, 'warning');
      $scope.isLoading = false;
    });
  }
  var usrname = '';
  function checkLogin() {
    // check for old way of caching username
    $localdb.get('username').then(function (un) {
      usrname = un && un != 'null' && un != null ? un : '';
      $auth.getCredentials().then(function (credentials) {
        $scope.isLoading = true;
        $scope.username = credentials.username;
        if (credentials.token) {
          login(true);
        } else {
          $scope.isLoading = false;
        }
      }, function (err) {
        // No credentials
        $scope.isLoading = false;
      });
    });
  }
  $scope.onClick = function () {
    $krConfig.setUrl($scope.values.server.value);
    if ($scope.isLoading) {
      $scope.cancelled = true;
      $scope.isLoading = false;
      return;
    }
    $scope.isLoading = true;
    $msg.msg($translate.instant('LoggingIn'), true, 'info', 2000);
    $auth.login($scope.username, $scope.password).then(function (credentials) {
      if (!$scope.cancelled) {
        login(usrname != '');
      } else {
        cancel();
      }
    }, function (err) {
      $scope.isLoading = false;
      var msg = '';
      if (err && (err.status === 401 || err.statusCode === '401')) {
        msg = $translate.instant('InvalidLoginCredentials');
      } else if (err && (err.status === 0 || err.statusCode === '0')) {
        msg = $translate.instant('CannotReachServers');
      }
      $appstate.msg($translate.instant('ErrorLoggingIn'), true, 'warning');
      if (msg != '') {
        $uiAlert.pushErrorAlert(msg, 'LOGIN');
        displayAlerts();
      }
    });
  };
  function cancel() {
    $scope.cancelled = false;
    $scope.isLoading = false;
  }
  $rootScope.$on('$stateChangeError', function ($event, current) {
    console.log('$stateChangeError');
    $scope.isLoading = false;
    $appstate.msg($translate.instant('ErrorLoggingIn'), true, 'warning');
    $state.go('login');
  });
  $rootScope.$on('$stateChangeSuccess', function (ev, to, toParams, from, fromParams) {
    $appstate.model.lastState = from;
  });
  $scope.getAlertClass = function (alert) {
    if (alert.type === $uiAlert.ALERT_TYPE_SUCCESS) {
      return 'alert alert-success';
    } else if (alert.type === $uiAlert.ALERT_TYPE_WARNING) {
      return 'alert alert-warning';
    } else if (alert.type === $uiAlert.ALERT_TYPE_ERROR) {
      return 'alert alert-danger';
    } else {
      return 'alert';
    }
  };
  function displayAlerts() {
    var alerts = $uiAlert.popAlertsFor('LOGIN');
    if (alerts) {
      $scope.alerts = alerts;
      $timeout(function () {
        $scope.alerts = null;
      }, 8000);
    }
  }
  displayAlerts();
}
;
'use strict';
angular.module('kidreports2App').controller('DashboardCtrl', [
  '$scope',
  '$rootScope',
  '$appstate',
  '$facility',
  '$backend',
  '$state',
  '$popup',
  '$utils',
  '$krConfig',
  '$native',
  '$adapter',
  '$krCache',
  '$timeout',
  '$messageBox',
  '$controller',
  '$translate',
  '$attendance',
  '$localdb',
  '$drawerService',
  '$modal',
  '$trackingService',
  function ($scope, $rootScope, $appstate, $facility, $backend, $state, $popup, $utils, $krConfig, $native, $adapter, $krCache, $timeout, $messageBox, $controller, $translate, $attendance, $localdb, $drawerService, $modal, $trackingService) {
    var facility = $facility.getSync();
    $scope.preventWriteActivity = facility.preventWriteActivity;
    if (facility.preventWriteActivity && !facility.preventWriteActivityWarned) {
      facility.preventWriteActivityWarned = true;
      if (facility.currentUser.role == 'PARENT_ROLE') {
        $messageBox.open('Account Disabled', 'This account is no longer active. You are able to view existing records but will be prevented from adding new records. If you have any questions please reach out to your center with any questions.', { oktext: 'Ok' }, null, 'img/icon/alert_solid.svg');
      } else {
        $messageBox.open('Account Disabled', 'This account is no longer active. You are able to view existing records but will be prevented from adding new records. If you have any questions please reach out to: support or implementations.', { oktext: 'Ok' }, null, 'img/icon/alert_solid.svg');
      }
    }
    $scope.isLite = $facility.isLite;
    $appstate.model.canClose = null;
    $appstate.model.dashboardMode = true;
    $scope.hideDailyReport = $facility.getSync().hideDailyReportButton;
    $scope.hideSignInKiosk = !hasSignInKiosk();
    $scope.saveFunc = null;
    if ($backend.getCurrentUser().role == 'PARENT_ROLE') {
      $state.go('home.main');
    }
    var func1 = $rootScope.$on('pushComplete', function (evt) {
        $timeout($scope.refreshEvents);
      });
    var func2 = $rootScope.$on('eventSaved', function (evt) {
        $timeout(function () {
          $scope.refreshEvents($appstate.model.isDashActivityEdit);
        }, 50);
      });
    var func3 = $scope.$watch('isSelectChild', function () {
        if ($scope.isSelectChild) {
          $appstate.dashSelectChild = $scope.closeDialog;
        } else {
          $appstate.dashSelectChild = null;
        }
      });
    $scope.setSelectChild = function (v, color) {
      $scope.isSelectChild = v;
      $scope.selectChildHeaderColor = color;
    };
    $scope.showSave = function () {
      return $scope.saveFunc !== null && $state.is('home.dash.event');
    };
    $scope.showChildSelect = function () {
      return !$scope.showSave() && !($state.is('home.dash.attendance') || $state.is('home.dash.kidPlanner') || $state.is('message') || $state.is('messages') || $state.is('messageNew') || $state.is('messageReply') || $state.is('home.dash.teacherSignIn') || $state.is('home.dash.nameToFace') || $state.is('home.dash.trackingSheet') || $state.is('home.dash.trackingChart') || $state.is('home.dash.overview') || $state.is('home.dash.learningStories') || $state.is('home.dash.policies') || $state.is('home.dash.eventObservations') || $state.is('home.dash.pickDraft'));
    };
    $scope.showCloseDialog = function () {
      return !($state.is('home.dash.nameToFace') || $state.is('message') || $state.is('messages') || $state.is('messageNew') || $state.is('messageReply') || $state.is('home.dash.trackingSheet') || $state.is('home.dash.trackingChart') || $state.is('home.dash.pickDraft'));
    };
    /**
      * Gets a cound of children that will be rendered on the page.
      * @return {number} An count of the children to be rendered on the page.
      */
    $scope.getVisibleChildCount = function () {
      return $scope.getChildrenInDashboard().length;
    };
    /**
      * Gets all children that should be rendered because of presence or selection.
      * @return {any[]} An array of children to be rendered on the page.
      */
    $scope.getChildrenInDashboard = function () {
      var includeAbsentIfSet = false;
      var includeDashboardedChildren = true;
      return $appstate.getVisibleChildren(includeAbsentIfSet, includeDashboardedChildren);
    };
    $scope.isPresent = function (child) {
      return $appstate.isPresent(child);
    };
    $scope.dashClick = function (item) {
      if (item.path.root == 'moveRoom') {
        $appstate.checkAttendance($appstate.model.selectedRoom.id, function (_item) {
          __finishDashClick(_item || item);
        }, item);
      } else {
        __finishDashClick(item);
      }
    };
    function __finishDashClick(item) {
      if (item.sref != null) {
        $state.go(item.sref);
      } else {
        $appstate.nav.onClick(item);
      }
    }
    $scope.dashboardTitle = 'Activity Log';
    // todo change title based on state.
    // need this on the scope for the view.
    $scope.fac = $facility.getSync();
    $scope.rooms = _.filter($scope.fac.rooms, function (item) {
      return item.name != $translate.instant('AllChildren') && item.name.indexOf('Todas as') == -1 && item.name != 'All Children' && !(item.props && item.props['moveRoomsOnly'] === 'true');
    });
    var f = $facility.getSync();
    $scope.photoVideoEnabled = true;
    if (f.props && f.props.photoVideoEnabled === 'false') {
      $scope.photoVideoEnabled = false;
    }
    $scope.sendPictureVideo = function (isPic) {
      var cat = $appstate.model.selectedRoom.category;
      var route = isPic ? 'pictureevent' : 'videoevent';
      var item = _.filter($facility.getSync().eventTypes, function (item) {
          return item.name.indexOf('Picture/Video') > -1 && item.categories[0].id == cat.id;
        })[0];
      var msg = null;
      if (!item || !$scope.photoVideoEnabled) {
        msg = $translate.instant('NotSupportedYetText');
      }
      if (msg) {
        $messageBox.open($translate.instant('NotSupportedYet'), msg, { oktext: 'Ok' });
        return;
      }
      newEvent(item, 'home.dash.' + route, true);
    };
    $scope.dailyReport = function () {
      var cat = $appstate.model.selectedRoom.category;
      var item = _.filter($facility.getSync().eventTypes, function (item) {
          return item.alertType == 'ALERT_ALL_EVENTS' && item.categories[0].id == cat.id;
        })[0];
      newEvent(item, 'home.dash.event');
    };
    function newEvent(item, state, isPicVideo) {
      $appstate.checkAttendance($appstate.model.selectedRoom.id, function (_item) {
        __finishEvent(_item || item, state, isPicVideo);
      }, item);
    }
    function __finishEvent(item, state, isPicVideo) {
      $appstate.model.currentEvent = $adapter.createNewEvent(item);
      $appstate.model.currentEvent.observed = $appstate.model.observed;
      if (isPicVideo && item.name.indexOf('Picture/Video') == -1) {
        state = 'home.dash.event';
      }
      if ($state.includes('home.dash')) {
        $state.go(state, {}, {
          location: true,
          inherit: true,
          relative: 'home.dash',
          notify: true
        });
      }
    }
    $scope.showChildDrawer = function (item) {
      event.cancelBubble = true;
      $drawerService.openChildProfile(item.id);
    };
    function hasTakenAttendance(roomId) {
      var room = $facility.findRoom(roomId);
      var found = false;
      room.children.forEach(function (c) {
        found = found || (c.attendanceSet || c.isIn != null);
      });
      return found;
    }
    $scope.toggleSelection = function (item) {
      if (item.selected) {
        var idx = -1;
        $appstate.model.observed.forEach(function (i, index) {
          if (i.id == item.id)
            idx = index;
        });
        if (idx != -1)
          $appstate.model.observed.splice(idx, 1);
      } else {
        $appstate.model.observed.push(item);
      }
      item.selected = !item.selected;
      event.preventDefault();
    };
    $scope.showProfile = function (item) {
      $popup.showProfile(item);
      event.cancelBubble = true;
    };
    $timeout(function () {
      $native.enhanceUI();
    }, 1000);
    $scope.showEvent = function (ele) {
      return !ele.eventType.hidden;
    };
    /**
     * Selects the specified kid and deselects all other children, regardless of presence or previous selection.
     * @param {any} kid - The child that was clicked in the UI that should be marked as selected.
     */
    $scope.pickKid = function (kid) {
      // Set the kid that was selected as the only one.
      $appstate.model.observed = [kid];
      // Deselect all of the other children
      if ($facility.getSync().observed) {
        $facility.getSync().observed.forEach(function (child) {
          child.selected = false;
        });
      }
      kid.selected = true;
      if (event) {
        event.cancelBubble = true;
      }
    };
    $appstate.pickKid = $scope.pickKid;
    /**
     * Toggles the selection of the specified child
     * @param {any} kid - The child that was clicked in the UI that should be toggled.
     */
    $scope.toggleKid = function (kid) {
      if (_.remove($appstate.model.observed, [
          'id',
          kid.id
        ]).length) {
        kid.selected = false;
      } else {
        kid.selected = true;
        $appstate.model.observed.push(kid);
      }
      if (event) {
        event.cancelBubble = true;
      }
    };
    /**
     * Scrolls the dash carousel left or right
     * @param {Integer} direction - -1 or 1 for left or right
     */
    $scope.stepKids = function (direction) {
      var containerWidth = $('#carouselScroll').width();
      var photoWidth = $('.dash-carousel-inner .dash-carousel-photo')[0].offsetWidth;
      var numChildrenAbleToDisplay = Math.floor(containerWidth / photoWidth);
      var currentScrollLeft = $('#carouselScroll').scrollLeft();
      var targetScrollLeft = Math.round(currentScrollLeft / photoWidth) * photoWidth + direction * numChildrenAbleToDisplay * photoWidth;
      var maxScrollLeft = $scope.getVisibleChildCount() * photoWidth - containerWidth;
      if (targetScrollLeft < 0)
        targetScrollLeft = 0;
      if (targetScrollLeft > maxScrollLeft)
        targetScrollLeft = maxScrollLeft;
      $scope.isScrolling = true;
      $('#carouselScroll').animate({ scrollLeft: targetScrollLeft }, 'fast', 'swing', function () {
        $scope.isScrolling = false;
      });
      if (event)
        event.cancelBubble = true;
    };
    $scope.carouselSelectAll = function () {
      $appstate.model.observed = [];
      var visibleKids = $scope.getChildrenInDashboard();
      if (visibleKids) {
        Array.prototype.push.apply($appstate.model.observed, visibleKids);
        visibleKids.forEach(function (child) {
          child.selected = true;
        });
      }
    };
    $scope.carouselSelectNone = function () {
      $appstate.model.observed = [];
      if ($facility.getSync().observed) {
        $facility.getSync().observed.forEach(function (child) {
          child.selected = false;
        });
      }
    };
    /**
     * Gets the appropriate text to display that describes how many children have been selected. This value has already been translated.
     * @return {string} A translated string value of the number of children currently selected.
     */
    $scope.getSelectedChildText = function () {
      var observed = $appstate.model.observed;
      if (!observed || observed.length == 0) {
        return $translate.instant('NoChildrenSelected');
      }
      if (observed.length == 1) {
        var child = observed[0];
        if (!child) {
          observed.splice(0, 1);
          return $translate.instant('NoChildrenSelected');
        } else {
          return child.firstName + ' ' + child.lastName;
        }
      }
      return $translate.instant('CountChildrenSelected').replace('&count&', observed.length);
    };
    $scope.multi = true;
    $appstate.model.multi = true;
    $scope.setMulti = function (multi) {
      $appstate.model.multi = multi;
      if (!multi) {
        var item = null;
        $facility.getSync().observed.forEach(function (i) {
          if (i.singleSelected == true)
            item = i;
          i.singleSelected = false;
        });
        if (item) {
          item.singleSelected = true;
          $appstate.model.observed = [item];
        }
      } else {
        // update the shared observer viewlist for the dashboard view.
        var selected = [];
        $facility.getSync().observed.forEach(function (item) {
          if (item.selected) {
            selected.push(item);
          }
        });
        $appstate.model.observed = selected;
      }
    };
    $appstate.model.gridView = true;
    $scope.isPopup = checkIsPopup();
    function checkIsPopup() {
      return !$state.is('home.dash');
    }
    // I don't see any usage of this method in this project.
    // Leaving the comment here so others see that it's been removed.
    // function checkPopup() {
    //   if ($scope.isPopup) {
    //     $timeout(function () {
    //       $scope.openModal();
    //     }, 20);
    //   }
    // }
    var func4 = $rootScope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams) {
        $scope.isPopup = checkIsPopup();
        $appstate.model.isRefreshing = false;
      });
    var theRm = null;
    if (!$appstate.model.selectedRoom) {
      theRm = $scope.rooms[0];
    } else {
      // link up
      $scope.rooms.forEach(function (r) {
        if (r.id == $appstate.model.selectedRoom.id)
          theRm = r;
      });
    }
    $scope.showStream = true;
    $scope.refreshEvents = function (forceStreamUpdate) {
      // refresh for the entire room, let the filter decide who to show.
      $appstate.refreshEvents($facility.getSync().observed, $appstate.model.currentDate || new Date(), null, true, null, function () {
        if (forceStreamUpdate) {
          $appstate.model.isDashActivityEdit = false;
        }
      });
    };
    if (theRm == null) {
      theRm = $scope.rooms[0];
    }
    var isFirstTime = true;
    var rmWatcher = $scope.$watch('selectedRoom', setSelectedRm);
    function setSelectedRm(newval, oldval) {
      var isSelectedRoomSame = newval.id === oldval.id;
      if (!isSelectedRoomSame && isFirstTime) {
        // De-select children in old room
        _.each($facility.getSync().observed, function (child) {
          child.selected = false;
        });
      }
      if (!isSelectedRoomSame || isFirstTime) {
        // NOTE: below block works without promise, but may cause problems with slow internet
        $appstate.setCurrentRoom($scope.selectedRoom);
        var didRefresh = refreshIfNew(false, newval);
        if (!didRefresh) {
          setTimeout(function () {
            var observed = $facility.getSync().observed;
            if (observed) {
              $appstate.model.observed = _.filter(observed, 'selected');
            }
          }, 50);
        }
      }
    }
    var refreshIfNew = function (forceUpdate, newRoom) {
      var roomMap = $localdb.get('roomMap').$$state.value;
      var firstTime = !(roomMap.indexOf(newRoom.id) > -1);
      if (firstTime || forceUpdate) {
        $attendance.refreshAll(true);
        $state.reload();
        // update room map to indicate which rooms have been cached
        roomMap.push(newRoom.id);
        $localdb.set('roomMap', roomMap);
        return true;
      }
      return false;
    };
    $scope.selectedRoom = theRm;
    $appstate.setCurrentRoom(theRm, true);
    var syncWatcher = $rootScope.$on('eventSynchDone', function () {
        $appstate.model.isRefreshing = false;
        syncWatcher();
        console.log('-------------------------------->>>Calling eventSyncDone........');
        $scope.refreshEvents();
      });
    $scope.getDashItemText = function (item) {
      if ($krConfig.isNZ() && item.path.root == 'portfolio')
        return 'Observations';
      else
        return $translate.instant(item.token);
    };
    $scope.closeDialog = function () {
      if ($scope.isSelectChild) {
        $scope.innerSelect = false;
        $scope.isSelectChild = false;
      } else {
        if ($appstate.canClose) {
          $appstate.canClose.call();
          $appstate.canClose = null;
        } else {
          $timeout(function () {
            $appstate.model.observed.forEach(function (o) {
              o.selected = true;
            });
          }, 50);
          $state.go('home.dash');
        }
      }
    };
    $scope.$on('$destroy', function () {
      func1();
      func2();
      func3();
      func4();
      rmWatcher();
    });
    $appstate.model.selectAll = false;
    // bad bad... (It would be awesome if someone could say why this is bad.)
    $appstate.dash = $scope;
    $appstate.getClass = function (token) {
      if (token == 'portfolio' && $appstate.model.viewPortfolio) {
        return 'portfolio-selected';
      }
      return '';
    };
    $appstate.getProfileImageUrl(false);
    /* Profile menu */
    $scope.navigateToMyProfile = function ($event) {
      $event.stopPropagation();
      $drawerService.openTeacherProfile();
    };
    $scope.navigateToEditDrafts = function () {
      $state.go('home.dash.pickDraft');
    };
    $scope.navigateToViewReminders = function () {
      $state.go('home.dash.reminders');
    };
    $scope.navigateToQuickLinks = function () {
      $state.go('home.dash.quicklinks');
    };
    $scope.signOut = function () {
      $appstate.nav.logout();
    };
    /* Side nav */
    /**
     * Navigates to Sign In Kiosk, if allowed
     */
    $scope.navigateToSignInKiosk = function () {
      $state.go('home.action.signInMain');
    };
    $scope.navigateToKidPlanner = function () {
      $state.go('home.action.kidPlanner.main');
    };
    $scope.navigateToSettings = function () {
      $state.go('home.action.settings');
    };
    /* Bottom nav bar */
    $scope.navigateToAttendance = function () {
      $state.go('attendance');
    };
    $scope.navigateToMoveRoom = function () {
      $appstate.checkAttendance($appstate.model.selectedRoom.id, function () {
        $state.go('attendance', {
          tab: 'In',
          mode: 'moveRoom'
        });
      });
    };
    $scope.navigateToMessages = function () {
      $state.go('messages');
    };
    $scope.navigateToMedia = function () {
      var cat = $appstate.model.selectedRoom.category;
      var item = _.filter($facility.getSync().eventTypes, function (item) {
          return item.name.indexOf('Picture/Video') > -1 && item.categories[0].id == cat.id;
        })[0];
      if (!item || !$scope.photoVideoEnabled) {
        $messageBox.open($translate.instant('NotSupportedYet'), $translate.instant('NotSupportedYetText'), { oktext: 'Ok' });
        return;
      }
      $modal.open({
        templateUrl: 'views/popups/chooseMediaSourcePopup.html',
        controller: 'ChooseMediaSourceControlller'
      });
    };
    $scope.navigateToTracking = function () {
      if ($trackingService.hasTrackingSheet()) {
        $state.go('home.dash.trackingSheet');
      } else {
        $state.go('home.dash.whereIs');
      }
    };
    /**
     * Checks in menu items if Sign In Kiosk is enabled
     * @returns {boolean}
     */
    function hasSignInKiosk() {
      var menuItems = _.get($facility.getSync(), 'menuItems');
      return !!_.find(menuItems, { name: 'Sign In Kiosk' });
    }
  }
]);
'use strict';
angular.module('kidreports2App').controller('TopmenubarCtrl', [
  '$scope',
  '$appstate',
  '$backend',
  '$state',
  function ($scope, $appstate, $backend, $state) {
    // todo
    $scope.switchToMain = function () {
      $appstate.model.showEventStream = false;
    };
    $scope.switchToList = function () {
      $appstate.model.showEventStream = true;
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('TopmenuCtrl', [
  '$rootScope',
  '$scope',
  'popover',
  '$appstate',
  '$facility',
  '$backend',
  '$native',
  '$state',
  '$prefs',
  '$adapter',
  '$localdb',
  '$utils',
  '$krCache',
  function ($rootScope, $scope, popover, $appstate, $facility, $backend, $native, $state, $prefs, $adapter, $localdb, $utils, $krCache) {
    $scope.parentRole = function () {
      return $backend.getCurrentUser().role === 'PARENT_ROLE';
    };
    $appstate.model.parentmode = 'HOME';
    $scope.getTabClass = function (m) {
      return m == $appstate.model.parentmode ? 'tabSelected' : '';
    };
    $scope.popup = { isOpen: false };
    $scope.togglePopup = function () {
      popover.show($scope);
    };
    $appstate.model.dashboardMode = false;
    $appstate.model.showEventStream = $backend.getCurrentUser().role === 'PARENT_ROLE';
    $appstate.model.loggedIn = true;
    // force restart of queue and uploads if we are online.
    $krCache.getCache(null, function (c) {
      if (c && _.isArray(c) && c.length > 0)
        $backend.saveEventInternal();
    });
    $krCache.getCache('uploads', function (c) {
      if (c && _.isArray(c) && c.length > 0)
        $native.uploadInternal();
    });
    //$backend.saveEventInternal();
    //$native.uploadInternal();
    // setup attendance
    $appstate.model.totalAttendance = $facility.getSync().totalAttendance != -1 ? $facility.getSync().totalAttendance : '--';
    $appstate.configureRooms();
    //var kids = $scope.parentRole() ? angular.copy($facility.getSync().children) : angular.copy(rm.children);
    //$appstate.msg("Logged In Successfully", true);
    $scope.navigateToMessages = function () {
      $state.go('messages');
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('RoomsCtrl', [
  '$scope',
  '$appstate',
  '$prefs',
  '$facility',
  '$backend',
  '$translate',
  function ($scope, $appstate, $prefs, $facility, $backend, $translate) {
    var user = $backend.getCurrentUser();
    if (user.role == 'PARENT_ROLE') {
      // transition now
      $appstate.nav.next();
    }
    $scope.status = $facility.getSync().name;
    $scope.items = $facility.getSync().rooms;
    if ($appstate.nav.getPath() == 'roomEvent' || $appstate.nav.getPath() == 'teacherSignIn' || $appstate.nav.getPath() == 'nameToFace') {
      $scope.items = _.filter($scope.items, function (item) {
        return item.name != $translate.instant('AllChildren');
      });
    }
    if ($appstate.nav.getPath() != 'moveRoom') {
      $scope.items = _.filter($scope.items, function (item) {
        return !(item.props && item.props['moveRoomsOnly'] === 'true');
      });
    }
    $scope.onClick = function (item) {
      $prefs.current.room = item;
      $appstate.setCurrentRoom(item);
      $appstate.nav.nextState = null;
      $appstate.nav.next();
    };
    return this;
  }
]);
'use strict';
angular.module('kidreports2App').controller('PickDraftController', PickDraftController);
PickDraftController.$inject = [
  '$scope',
  '$appstate',
  '$adapter',
  '$krCache',
  '$messageBox',
  '$translate',
  '$utils'
];
function PickDraftController($scope, $appstate, $adapter, $krCache, $messageBox, $translate, $utils) {
  $scope.shared = $appstate;
  $scope.deleteAllDrafts = function () {
    $messageBox.open($translate.instant('ConfirmDelete'), $translate.instant('DeleteAllDraftsConfirm'), {
      oktext: $translate.instant('Yes'),
      canceltext: $translate.instant('No')
    }, null, 'img/icon/delete.svg').result.then(function (selectedItem) {
      $scope.items = [];
      $krCache.setCache($scope.items, 'drafts');
    }, function () {
    });
  };
  $scope.deleteItem = function (item) {
    $messageBox.open($translate.instant('ConfirmDelete'), $translate.instant('DeleteDraftConfirm'), {
      oktext: $translate.instant('Yes'),
      canceltext: $translate.instant('No')
    }, null, 'img/icon/delete.svg').result.then(function (selectedItem) {
      $scope.items = _.filter($scope.items, function (i) {
        return item.draftId != i.draftId;
      });
      $krCache.setCache($scope.items, 'drafts');
    }, function () {
    });
  };
  $scope.edit = function (item) {
    $appstate.model.isDashActivityEdit = true;
    $appstate.viewEvent(item, false);
  };
  $scope.getTime = function (ts) {
    return $utils.getTimeString($utils.getTimePart(new Date(ts)));
  };
  function getDrafts() {
    var retVal = [];
    _.forEach($appstate.model.eventListByChild, function (childDraft) {
      _.forEach(childDraft, function (draft) {
        if (!draft.draftId) {
          return;
        }
        retVal.push(draft);
      });
    });
    return retVal;
  }
  $scope.items = getDrafts();
}
'use strict';
angular.module('kidreports2App').controller('WhereIsCtrl', [
  '$scope',
  '$state',
  '$appstate',
  '$facility',
  '$q',
  '$auth',
  '$krConfig',
  'moment',
  '$ajax',
  '$utils',
  '$translate',
  '$drawerService',
  '$trackingService',
  '$featureFlags',
  function ($scope, $state, $appstate, $facility, $q, $auth, $krConfig, moment, $ajax, $utils, $translate, $drawerService, $trackingService, $featureFlags) {
    $scope.toggleSearch = function () {
      $scope.searchInputShown = !$scope.searchInputShown;
    };
    $scope.openChildProfile = function (childId, $event) {
      $event.stopPropagation();
      $drawerService.openChildProfile(childId);
    };
    $scope.getCurrentUserPhotoPath = function () {
      var currentUser = $facility.getSync().currentUser;
      return $utils.createPhotoUrl($krConfig.s3Base, currentUser.photoPath);
    };
    $scope.filter = function () {
      var filteredChildren = $scope.allChildViewModels;
      filteredChildren = $scope.filterByRoom(filteredChildren);
      filteredChildren = $scope.filterByChildName(filteredChildren);
      $scope.viewModel.children = filteredChildren;
    };
    $scope.filterByRoom = function (children) {
      if (!$scope.selectedRoom || $scope.selectedRoom === 'All Children') {
        return children;
      }
      return _.filter(children, function (child) {
        return child.homeRoom.name === $scope.selectedRoom;
      });
    };
    $scope.filterByChildName = function (children) {
      if (!$scope.selectedChildName || $scope.selectedChildName === '') {
        return children;
      }
      return _.filter(children, function (child) {
        return child.firstName.toLowerCase().indexOf($scope.selectedChildName.toLowerCase()) > -1 || child.lastName.toLowerCase().indexOf($scope.selectedChildName.toLowerCase()) > -1;
      });
    };
    $scope.refresh = function () {
      init();
    };
    $scope.close = function () {
      $appstate.goHome();
    };
    function loadEventsAsync(children) {
      var q = $q.defer();
      var eventData = {};
      var options = {
          method: 'POST',
          url: $krConfig.getAjaxBase() + '/api/whereisbulk',
          headers: { 'Authorization': 'Bearer ' + $auth.getCredentialsSync().token },
          data: {
            ids: _.map(children, function (child) {
              return child.id;
            })
          }
        };
      $ajax.ajax(options, function (err, response) {
        if (err) {
          q.reject(err);
          return;
        }
        _.each(response, function (data, id) {
          var childData = [];
          _.each(data.attendance || [], function (data) {
            childData.push({
              type: 'attendance',
              event: data
            });
          });
          _.each(data.rooms || [], function (data) {
            childData.push({
              type: 'room',
              event: data
            });
          });
          if (childData.length > 0) {
            eventData[id] = _.sortBy(childData, 'event.timeOccurred');
          }
        });
        q.resolve(eventData);
      });
      return q.promise;
    }
    function buildChildViewModels(children, eventData) {
      var childViewModels = [];
      _.each(children, function (child) {
        var childEvents = eventData[child.id];
        if (childEvents) {
          var homeRoom = $facility.findRoom(child.homeRoomId);
          // Setup the defaults for the childs view model
          var model = {
              id: child.id,
              firstName: child.firstName,
              lastName: child.lastName,
              photoUrl: $utils.createPhotoUrl($krConfig.s3Base, child.photoPath),
              homeRoom: {
                id: homeRoom.id,
                name: homeRoom.name,
                time: new Date()
              },
              currentRoom: {
                id: homeRoom.id,
                name: homeRoom.name,
                time: new Date()
              }
            };
          // All check-in times should display the last time the child checked in.
          var lastAttendanceEvent = childEvents.filter(function (e) {
              return e.type === 'attendance' && e.event.eventValues.some(function (ev) {
                return ev.value === 'In';
              });
            }).reverse(function (e) {
              return e.event.timeOccurred;
            })[0];
          if (!lastAttendanceEvent) {
            // this child shouldn't be displayed if there's no check-in event.
            return;
          }
          var attendanceUseSeconds = $featureFlags.isFeatureEnabled('attendance-use-seconds');
          // default both the home room and current room's check-in time to the last attendance check-in. 
          // Any move room events will override this.
          model.homeRoom.time = attendanceUseSeconds ? moment(new Date(lastAttendanceEvent.event.timeOccurred)).format('hh:mm:ss A') : moment(new Date(lastAttendanceEvent.event.timeOccurred)).format('hh:mm A');
          model.currentRoom.time = attendanceUseSeconds ? moment(new Date(lastAttendanceEvent.event.timeOccurred)).format('hh:mm:ss A') : moment(new Date(lastAttendanceEvent.event.timeOccurred)).format('hh:mm A');
          var moveEvent = childEvents.filter(function (event) {
              return event.type === 'room';
            }).reverse()[0];
          if (moveEvent) {
            var targetRoom = moveEvent.event.eventValues[1];
            model.currentRoom = {
              id: targetRoom.id,
              name: targetRoom.value.replace('To:', ''),
              time: attendanceUseSeconds ? moment(new Date(moveEvent.event.timeOccurred)).format('hh:mm:ss A') : moment(new Date(moveEvent.event.timeOccurred)).format('hh:mm A')
            };
          }
          childViewModels.push(model);
        }
      });
      return childViewModels;
    }
    $scope.init = function () {
      $scope.loading = true;
      try {
        var facility = $facility.getSync();
        $scope.menuItems = $trackingService.populateMenuItems();
        $scope.menuItemSelectedIndex = _.findIndex($scope.menuItems, function (item) {
          return item.name === $translate.instant('WhereIs');
        });
        // Date for display purposes
        $scope.today = moment(new Date()).format('MMMM Do, YYYY');
        $scope.state = $state;
        // A collection of all children for the center. Searching and filtering children will modify the collection on the view model, not this collection.
        $scope.allChildViewModels = [];
        // View model designed for the html page
        $scope.viewModel = { children: [] };
        $scope.searchInputShown = false;
        // All available rooms for the facility.
        $scope.facility = facility;
        $scope.rooms = facility.rooms;
        // Models bound to search elements on the page.
        $scope.selectedChildName = '';
        $scope.selectedRoom = 'All Children';
        var children = $facility.getSync().children;
        loadEventsAsync(children).then(function (eventData) {
          var childViewModels = buildChildViewModels(children, eventData);
          $scope.allChildViewModels = childViewModels;
          $scope.viewModel.children = childViewModels;
          $scope.filterByRoom();
          $scope.errorRetrievingChildren = false;
          $scope.loading = false;
        }).catch(function (error) {
          $scope.errorRetrievingChildren = true;
          $scope.loading = false;
        });
      } catch (error) {
        $scope.errorRetrievingChildren = true;
        $scope.loading = false;
      }
    };
    $scope.init();
  }
]);
'use strict';
angular.module('kidreports2App').controller('ChildselectheaderCtrl', [
  '$scope',
  '$appstate',
  function ($scope, $appstate) {
    $scope.awesomeThings = [
      'HTML5 Boilerplate',
      'AngularJS',
      'Karma'
    ];
  }
]);
'use strict';
angular.module('kidreports2App').controller('ChildrenSingleController', ChildrenSingleController);
ChildrenSingleController.$inject = [
  '$scope',
  '$rootScope',
  '$appstate',
  '$krConfig',
  '$backend',
  '$state',
  '$facility',
  '$popup',
  '$translate',
  '$eventAdapter',
  '$utils'
];
function ChildrenSingleController($scope, $rootScope, $appstate, $krConfig, $backend, $state, $facility, $popup, $translate, $eventAdapter, $utils) {
  var func = $rootScope.$on('selectedRoomChange', function (newval, oldval) {
      if ($state.includes('home.dash')) {
        onRoomChange();
      }
    });
  $scope.$on('$destroy', function () {
    func();
  });
  $scope.setPresent = function (b) {
    $scope.showPresent = b;
    $appstate.model.showAbsentKids = !b;
  };
  $scope.showProfile = function (item) {
    $popup.showProfile(item);
  };
  $scope.getPhotoUrl = function (photoPath) {
    return $utils.createPhotoUrl($krConfig.s3Base, photoPath);
  };
  $scope.onClick = function (item) {
    $appstate.model.observed = [item];
    if ($state.includes('home.dash')) {
      $scope.items.forEach(function (i) {
        i.singleSelected = false;
      });
      item.singleSelected = true;
    } else {
      item.singleSelected = true;
      $appstate.nav.nextState = null;
      $appstate.nav.next();
    }
  };
  function getItems(user) {
    var items = [];
    if (user.role != 'PARENT_ROLE') {
      items = $facility.getSync().observed;
    } else {
      items = $facility.getSync().children;
    }
    items.forEach(function (item) {
      item.selected = false;
    });
    return items;
  }
  function onRoomChange() {
    var user = $backend.getCurrentUser();
    if (user.role != 'PARENT_ROLE' && $appstate.model.selectedRoom) {
      $scope.status = $translate.instant('RoomChildren').replace('&room&', $appstate.model.selectedRoom.name) + ':';
    } else {
      $scope.status = $translate.instant('ParentsChildren').replace('&name&', user.firstName);
    }
    $scope.getSelectedText = function () {
      return '';
    };
    $scope.items = [];
    $appstate.refresh = function (force) {
      $eventAdapter.getEvents(getItems(user), null, force, null, null, false, true, !force).then(function (data) {
        $scope.items = getItems(user);
        if ($scope.items && $state.includes('home.dash'))
          $scope.onClick($scope.items[0]);
      });
    };
    $appstate.refresh(false);
  }
  // INIT
  $scope.setPresent(true);
  onRoomChange();
}
'use strict';
angular.module('kidreports2App').controller('ChildMultiSelectController', ChildMultiSelectController);
function ChildMultiSelectController($scope, $rootScope, $appstate, $state, $facility, $translate, $eventAdapter, $timeout) {
  var roomFunc = $rootScope.$on('selectedRoomChange', function (newval, oldval) {
      if ($state.includes('home.dash') || $state.includes('home.action.kidPlanner.main'))
        onRoomChange();
    });
  function setItems() {
    if ($appstate.nav.getPath() == 'roomEvent') {
      $appstate.model.selectAll = false;
      $scope.toggleAll(true);
    } else {
      $appstate.model.selectAll = false;
      $facility.getSync().observed.forEach(function (item) {
        // unselect any absent kids to avoid any issues
        if ($appstate.isAbsent(item)) {
          item.selected = false;
          item.showInDashboard = item.selected;
        }
      });
    }
    $scope.items = $facility.getSync().observed;
    if ($appstate.model.lastState.name != 'home.action.selectRoom' && !$state.includes('home.dash')) {
      if ($facility.getSync().observed) {
        $facility.getSync().observed.forEach(function (item) {
          item.selected = !$appstate.isAbsent(item) && (item.selected || _.filter($appstate.model.observed, function (o) {
            return o.id == item.id;
          }).length > 0);
          item.showInDashboard = item.selected;
        });
        checkDirty();
      }
    } else {
      if ($facility.getSync().observed) {
        if (!$state.is('home.dash.moveRoom') && $state.includes('home.dash') && $appstate.pickKid && $facility.getSync().observed && $facility.getSync().observed[0]) {
          var kidArr = _.filter($facility.getSync().observed, function (k) {
              return $appstate.isPresent(k);
            });
          $appstate.model.observed = [];
        } else {
          $timeout(function () {
            if ($facility.getSync().observed.length === 0 || $appstate.getVisibleChildren().length == 0) {
              $appstate.model.observed = [];
            }
          }, 300);
        }
      }
    }
  }
  function onRoomChange(suppress) {
    if ($appstate.model.selectedRoom) {
      $scope.status = $appstate.model.selectedRoom.name + ' ' + $translate.instant('Children') + ':';
    }
    if (!suppress) {
      $appstate.refresh(false, $facility.getSync().observed);
    } else {
      setItems();
    }
    $appstate.toggle = $scope.toggleAll;
  }
  function checkDirty() {
    $appstate.model.dirty = _.filter($facility.getSync().observed, function (item) {
      return item.selected;
    }).length > 0;
    // update the shared observer viewlist for the dashboard view.
    var selected = [];
    $facility.getSync().observed.forEach(function (item) {
      if (item.selected) {
        selected.push(item);
      }
    });
    $appstate.model.observed = selected;
    $rootScope.$broadcast('observedChange');
  }
  $scope.$on('$destroy', function () {
    roomFunc();
  });
  $scope.setPresent = function (presentOnly) {
    $scope.showPresent = presentOnly;
    if (presentOnly) {
      $facility.getSync().observed.forEach(function (item) {
        // we don't need to check children that aren't selected for a presence state.
        if (item.selected) {
          item.selected = $appstate.isPresent(item);
        }
        // Regardless of selection or child presence, reset all dashboard status to the current selection.
        item.showInDashboard = item.selected;
      });
    }
    $timeout(function () {
      $appstate.model.showAbsentKids = !presentOnly;
      checkDirty();
    });
  };
  $scope.setPresent(false);
  /**
      * Gets the text that describes the number of children selected.
      * @return {string} A string representation of the number of children being selected.
      */
  $scope.getSelectedText = function () {
    var selectedChildren = _.filter($facility.getSync().observed, function (child) {
        return child.selected;
      });
    return ' (' + selectedChildren.length + ')';
  };
  // Need a copy since we're modifying the objects
  $appstate.refresh = function (force, items) {
    $eventAdapter.getEvents(items || $facility.getSync().observed, null, force, null, null, false, true, !force).then(setItems);
  };
  $scope.onClick = function (item) {
    item.selected = !item.selected;
    item.showInDashboard = item.selected;
    checkDirty();
  };
  $scope.toggleAll = function (b) {
    $facility.getSync().observed.forEach(function (item) {
      item.selected = b && !$appstate.isAbsent(item);
      item.showInDashboard = item.selected;
    });
    checkDirty();
  };
  $scope.forceSelectAll = function (b) {
    var items = $facility.getSync().observed;
    var allSelected = _.every(items, function (item) {
        return item.selected || $appstate.isAbsent(item) && !$scope.includeAbsent;
      });
    var newState = typeof b !== 'undefined' ? b : !allSelected;
    $facility.getSync().observed.forEach(function (item) {
      if ($scope.includeAbsent || !$appstate.isAbsent(item) || !newState) {
        item.selected = newState;
        item.showInDashboard = item.selected;
      }
    });
    $appstate.model.selectAll = newState;
    checkDirty();
  };
  $scope.next = function () {
    var selected = [];
    $facility.getSync().observed.forEach(function (item) {
      if (item.selected) {
        selected.push(item);
      }
    });
    if (selected.length == 0) {
      $appstate.msg($translate.instant('NoChildrenSelected'), true, 'warning');
      return;
    }
    $appstate.model.dirty = false;
    $appstate.model.observed = selected;
    $appstate.nav.nextState = null;
    $appstate.nav.next();
  };
  // orderBy custom comparator to show checked in kids first
  $scope.checkedInKidsFirst = function (kid) {
    return $appstate.isAbsent(kid);
  };
  $appstate.model.gridView = false;
  $appstate.nav.nextState = $scope.next;
  $scope.shared = $appstate;
  $scope.includeAbsent = false;
}
'use strict';
angular.module('kidreports2App').controller('ModalinstanceCtrl', [
  '$scope',
  '$appstate',
  '$modalInstance',
  'date',
  '$utils',
  function ($scope, $appstate, $modalInstance, date, $utils) {
    $scope.vals = { date: { value: date } };
    //$scope.scope = scope;
    $scope.ok = function () {
      //console.log($scope.vals.date);
      var v = $scope.vals.date.value;
      if (v.getTime)
        v = $utils.getTimePart(v);
      $modalInstance.close(v);
    };
    $scope.cancel = function () {
      $modalInstance.dismiss('cancel');
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('EmptyCtrl', [
  '$scope',
  '$appstate',
  function ($scope, $appstate) {
    $scope.awesomeThings = [
      'HTML5 Boilerplate',
      'AngularJS',
      'Karma'
    ];
  }
]);
'use strict';
angular.module('kidreports2App').controller('FilterquickviewCtrl', [
  '$scope',
  '$appstate',
  '$modalInstance',
  'facility',
  'room',
  '$prefs',
  function ($scope, $appstate, $modalInstance, facility, room, $prefs) {
    var cat = _.filter(facility.categories, function (cat) {
        return cat.id == room.category.id;
      })[0];
    $scope.items = [];
    facility.eventTypes.forEach(function (et) {
      if (et.categories[0].id === cat.id && et.hidden != true && et.enabled != false) {
        et.selected = _.filter($prefs.current.quickViewFilter, function (f) {
          return f.id == et.id;
        }).length > 0;
        $scope.items.push(et);
      }
    });
    $scope.ok = function () {
      $modalInstance.close(_.filter($scope.items, function (item) {
        return item.selected;
      }));
    };
    $scope.cancel = function () {
      $modalInstance.dismiss('cancel');
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('ChildprofileCtrl', [
  '$backend',
  '$scope',
  '$drawerService',
  '$timeout',
  '$facility',
  '$nav',
  '$state',
  '$ajax',
  '$appstate',
  '$utils',
  '$msg',
  '$translate',
  '$krConfig',
  '$auth',
  '$native',
  '$profile',
  '$image',
  function ($backend, $scope, $drawerService, $timeout, $facility, $nav, $state, $ajax, $appstate, $utils, $msg, $translate, $krConfig, $auth, $native, $profile, $image) {
    var facility = $facility.getSync();
    $scope.preventWriteActivity = facility.preventWriteActivity;
    $scope.child = _.find(facility.children, function (child) {
      return child.id === $drawerService.params.childId;
    });
    var user = facility.currentUser;
    $scope.isParent = user.role === 'PARENT_ROLE';
    $scope.showSetPhoto = user.role === 'TEACHER_ROLE' && user.canEditChildren === true;
    $scope.native = $native.isNative();
    $scope.setProfilePhoto = $profile.setProfilePhoto;
    if ($scope.child.birthday) {
      var bday = moment(new Date($scope.child.birthday));
      $scope.childBirthday = bday.format('MMMM D, YYYY');
      var ageDiff = moment.duration(moment().diff(bday))._data;
      $scope.childAge = ageDiff.years + ' year' + (ageDiff.years !== 1 ? 's ' : ' ') + ageDiff.months + ' month' + (ageDiff.months !== 1 ? 's' : '');
    } else {
      $scope.childBirthday = 'Unknown';
      $scope.childAge = 'Unknown';
    }
    function updateKidInfoForDisplay() {
      $scope.otherInfo = $scope.child.otherInfo ? $scope.child.otherInfo.replace(/\r\n/g, '<br/>').replace(/\n/g, '<br/>') : '';
      $scope.emergencyContact = $scope.child.emergencyContact ? $scope.child.emergencyContact.replace(/\r\n/g, '<br/>').replace(/\n/g, '<br/>') : '';
    }
    updateKidInfoForDisplay();
    $scope.showParentProfileDrawer = function (item) {
      $drawerService.openParentProfile(item.id);
    };
    $scope.closeDrawer = $drawerService.closeDrawer;
    $scope.editFields = {};
    $scope.startEdit = function (field) {
      $scope[field + 'EditMode'] = true;
      $scope[field + 'Prev'] = $scope[field];
      $scope.editFields[field + 'Edit'] = fixNewlines($scope[field]);
    };
    $scope.saveEdit = function (field) {
      $scope[field + 'EditMode'] = false;
      $scope.child[field] = $scope.editFields[field + 'Edit'];
      saveEdits();
      updateKidInfoForDisplay();
    };
    $scope.cancelEdit = function (field) {
      $scope[field + 'EditMode'] = false;
      $scope[field] = $scope[field + 'Prev'];
    };
    function fixNewlines(str) {
      if (str)
        return str.replace(/<br\/>/g, '\r\n');
      else
        return '';
    }
    function saveEdits() {
      $appstate.model.isRefreshing = true;
      // set up credentials
      var token = $auth.getCredentialsSync().token;
      var creds = { 'Authorization': 'Bearer ' + token };
      var obj = {
          id: $scope.child.id,
          otherInfo: $scope.child.otherInfo,
          emergencyContact: $scope.child.emergencyContact
        };
      var opts = {
          method: 'POST',
          url: $krConfig.getAjaxBase() + '/children/updateChild',
          headers: creds,
          data: obj
        };
      $ajax.ajax(opts, function (err, data) {
        $appstate.model.isRefreshing = false;
        if (err) {
          $utils.alert(JSON.stringify(err));
        } else {
          $msg.msg($translate.instant('Saved'));
          $facility.save();
        }
      });
    }
    ;
    $scope.child.parents.forEach(function (p) {
      $image.getImage(p.id, $utils.createPhotoUrl($krConfig.s3Base, p.photoPath)).then(function (d) {
        p.photoUrl = d;
      });
      p.relationText = getRelationText(p);
    });
    function getRelationText(parent) {
      if (!parent.parentType) {
        parent = $facility.findUser(parent.id);
      }
      if (!parent || !parent.parentType || parent.parentType == 'PARENT')
        return $translate.instant('ParentGuardian');
      else if (parent.parentType == 'GRANDPARENT')
        return $translate.instant('Grandparent');
      else if (parent.parentType == 'AUNT')
        return $translate.instant('AuntUncle');
      else if (parent.parentType == 'OTHER')
        return $translate.instant('OtherFamilyFriend');
      else
        return $translate.instant('ParentGuardian');
    }
    ;
  }
]);
'use strict';
angular.module('kidreports2App').controller('AttendanceheaderCtrl', [
  '$scope',
  '$state',
  '$facility',
  '$appstate',
  function ($scope, $state, $facility, $appstate) {
    if ($state.is('home.action.teacherSignIn') || $state.is('home.dash.teacherSignIn')) {
      $scope.canSetTime = !$facility.getSync().disableTeachersFromSettingTime;
    } else {
      $scope.canSetTime = true;
    }
    // Setup the margin if device has insets (eg. iPhone X)
    if (typeof forge !== 'undefined' && forge.layout) {
      forge.layout.getSafeAreaInsets(function (safeAreaInsets) {
        $scope.topPx = safeAreaInsets.top + 41;
      });
    }
    $scope.attendanceShowAll = false;
    $scope.toggleIn = function (item) {
      $appstate.toggleIn(item);
    };
    $scope.setTime = function (item) {
      $appstate.setTime();
    };
    $scope.getSelectedText = function () {
      return $appstate.getSelectedText();
    };
    $scope.show = function () {
      return $state.is('home.action.teacherSignIn') || $state.is('home.dash.teacherSignIn');
    };
    $scope.refreshAttendance = function (b) {
      $appstate.refresh(b);
    };
    var _showToggleAllButton = $facility.getSync().consolidatingRoomId != null;
    $scope.showToggleAllButton = function () {
      return _showToggleAllButton;
    };
    $scope.toggleAttendanceShowAll = function () {
      $scope.attendanceShowAll = !$scope.attendanceShowAll;
      $appstate.updateChildList($scope.attendanceShowAll);
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('TeachersigninCtrl', [
  '$scope',
  '$appstate',
  function ($scope, $appstate) {
  }
]);
'use strict';
angular.module('kidreports2App').controller('AttendanceCtrl', [
  '$scope',
  '$appstate',
  '$krConfig',
  '$modal',
  '$utils',
  '$backend',
  '$state',
  '$adapter',
  '$localdb',
  'moment',
  '$messageBox',
  '$events',
  '$facility',
  '$attendance',
  '$rootScope',
  '$timeout',
  '$translate',
  '$eventAdapter',
  function ($scope, $appstate, $krConfig, $modal, $utils, $backend, $state, $adapter, $localdb, moment, $messageBox, $events, $facility, $attendance, $rootScope, $timeout, $translate, $eventAdapter) {
    var isTeacher = false;
    var title = $translate.instant('Attendance');
    if ($state.is('home.action.teacherSignIn') || $state.is('home.dash.teacherSignIn')) {
      isTeacher = true;
      title = $translate.instant('TeacherSignIn');
      $scope.status = 'All Teachers';
      $scope.canSetTime = !$facility.getSync().disableTeachersFromSettingTime;
    } else {
      $scope.status = $appstate.model.selectedRoom.name + ' Children:';
      $scope.canSetTime = true;
    }
    // Grab the margin necessary for non-iPhone X devices
    if (typeof forge !== 'undefined' && forge.layout) {
      forge.layout.getSafeAreaInsets(function (safeAreaInsets) {
        $scope.autoSendMargin = safeAreaInsets.top > 20 ? safeAreaInsets.top + 35 : safeAreaInsets.top + 16;
      });
    }
    var isKiosk = $state.is('home.action.kioskMain');
    $scope.title = title;
    $appstate.model.title = title;
    $appstate.model.inVal = 'In';
    $scope.showAllKids = false;
    $scope.getSelectedText = function () {
      var size = _.filter($scope.items, function (i) {
          return !i.clean;
        }).length;
      return ' (' + size + ')';
    };
    // Need a copy since we're modifying the objects
    //$scope.items = angular.copy($facility.getSync().observed);
    var d = new Date();
    // todo revert for app release
    var allowBackdate = $facility.getSync().allowAttendanceBackdate;
    if (allowBackdate) {
      var d2 = $appstate.model.currentDate || new Date();
      d.setMonth(d2.getMonth());
      d.setFullYear(d2.getFullYear());
      d.setDate(d2.getDate());
    }
    $scope.dateValue = d.toLocaleDateString();
    $scope.notToday = !moment(d).isSame(moment(), 'day');
    $scope.value = $utils.getTimePart(d);
    $scope.allrooms = _.filter($facility.getSync().rooms, function (item) {
      return item.name != $translate.instant('AllChildren');
    });
    var roomId = $appstate.model.selectedRoom.id;
    function setupItems() {
      if (isTeacher) {
        // first find all teachers...
        $scope.items = _.filter($facility.getSync().users, function (u) {
          return u.role != 'PARENT_ROLE' && u.enabled;
        });
        console.log($scope.items);
        // then filter by room
        $scope.items = _.filter($scope.items, function (u) {
          return _.filter($appstate.model.selectedRoom.teachers, function (t) {
            return t.id == u.id && !t.removeFromSignIn;
          }).length > 0;
        });
      } else {
        if (!$scope.showAllKids)
          $scope.items = angular.copy($facility.getSync().observed);
      }
      _.each($scope.items, function (item) {
        item.clean = true;
        item.selected = false;
      });
      console.log($scope.items);
    }
    $scope.showAttendance = true;
    var oldRefresh = $appstate.refresh;
    $appstate.refresh = function (b, hideRoomId) {
      _.each($scope.items, function (item) {
        item.current = {};
      });
      $appstate.model.inVal = 'In';
      $attendance.refresh($scope.items, d, isTeacher, b, false, hideRoomId).then(updateItemState);
    };
    $scope.$on('$destroy', function () {
      $appstate.refresh = oldRefresh;
    });
    function updateItemState() {
      setupItems();
      _.each($scope.items, function (item) {
        item.current = {
          isIn: item.isIn,
          time: item.time,
          timeVal: item.timeVal
        };
      });
    }
    $scope.autosend = $facility.getSync().sendWhenCheckingOut;
    setupItems();
    $appstate.refresh(isTeacher || isKiosk);
    $appstate.model.dirty = false;
    $scope.toggleIn = function (item) {
      function toggleItem(i, val) {
        if (i.isIn == val)
          return;
        if (typeof val != 'undefined')
          i.isIn = val;
        else
          i.isIn = i.isIn ? !i.isIn : true;
        // set to null if they haven't been checked in yet.
        if (i.originalState == null && i.isIn == false)
          i.isIn = null;
        i.timeVal = $scope.value;
        i.time = $utils.getTimeString(i.timeVal);
        i.clean = !i.clean;
        i.isInDisplay = i.isIn;
      }
      if (item) {
        toggleItem(item, !item.isIn);
        $appstate.model.dirty = true;
        $rootScope.$broadcast('attendanceDirty', item);
      } else {
        $scope.items.forEach(function (item) {
          toggleItem(item, $appstate.model.inVal === 'In');
        });
        $appstate.model.inVal = $appstate.model.inVal === 'In' ? 'Out' : 'In';
      }
    };
    function isSigningOut() {
      var isOut = false;
      _.each($scope.items, function (item) {
        isOut = isOut || !item.isIn && !item.clean;
      });
      return isOut;
    }
    $scope.save = function () {
      var invalid = [];
      // check if any items are set to invalid times and update them
      _.each($scope.items, function (item) {
        var isInSet = item.current.isIn != null && typeof item.current.isIn != 'undefined';
        if (isInSet && (item.current.isIn != item.isIn && item.current.timeVal >= item.timeVal)) {
          var str = 'Uh oh. ' + item.firstName + ' was Signed ' + (item.current.isIn ? 'In' : 'Out') + ' at ' + item.current.time + '. Please make sure ' + item.firstName + ' is Signed ' + (!item.current.isIn ? 'In' : 'Out') + ' at a time after ' + item.current.time;
          invalid.push(str);
        }
      });
      if (invalid.length > 0) {
        $messageBox.open($translate.instant('AttendanceWarning'), invalid.join('<hr/>'), { oktext: $translate.instant('Ok') });
        $appstate.nav.nextState = $scope.save;
        return;
      }
      if (isSigningOut() && !isTeacher) {
        var attendanceCB = 'attendanceSignOutCB';
        $localdb.get(attendanceCB).then(function (ocb) {
          if (ocb == false || ocb == null) {
            $messageBox.open($translate.instant('Caution'), $translate.instant('AttendanceSignOut'), {
              oktext: $translate.instant('Yes'),
              canceltext: $translate.instant('No'),
              showOneTimeCb: true
            }).result.then(function (ocbValue) {
              $localdb.set(attendanceCB, ocbValue);
              confirmedAttendance();
            }, function (err) {
              $appstate.nav.nextState = $scope.save;
              return;
            });
          } else if (ocb == true) {
            confirmedAttendance();
          } else {
            $appstate.nav.nextState = $scope.save;
            return;
          }
        });
      } else {
        confirmedAttendance();
      }
    };
    function confirmedAttendance() {
      //var d = $utils.createDateFromDTStrings(, $scope.value);
      //var ts = $utils.createGsonTSFromDTStrings($utils.getDatePart(d), $scope.value);
      $appstate.model.dirty = false;
      //$appstate.model.isRefreshing = true;
      var forId = $facility.getSync().observer && $facility.getSync().observer.id;
      var handleRoomTransitions = $scope.showAllKids;
      $backend.saveAttendance($scope.items, $appstate.model.selectedCategory, d, function (err, data) {
        //$appstate.model.isRefreshing = false;
        if (err) {
          console.log(err);
          $messageBox.open(title, $translate.instant('ErrorSaving') + ' ' + title, { oktext: $translate.instant('Ok') });
          $appstate.msg($translate.instant('ErrorSaving') + ' ' + title, true, 'danger');
        } else {
          var type = 'Saved';
          if (data.offline) {
            type = 'Cached';
          }
          if (!isTeacher) {
            $appstate.model.currentRoomAttendance = _.filter($scope.items, function (i) {
              return i.isIn == true;
            }).length;
          }
          $eventAdapter.updateAttendanceState($scope.items, isTeacher, true);
          $appstate.msg($translate.instant('AttendanceSavedSuccess'), true);
          $scope.items.forEach(function (i) {
            i.clean = true;
            i.selectedChildRoomId = i.selectedChildRoom ? i.selectedChildRoom.id : null;
            delete i.selectedChildRoom;
          });
          $facility.save().then(function () {
            $timeout(function () {
              $rootScope.$broadcast('eventSaved');
            }, 10);
          });
          // Now kick off room transitions if applicable
          if (handleRoomTransitions) {
            ///
            //$timeout(function() {
            handleRoomMovement(title, type);
          } else {
            finishAttendance(title, type);
          }
        }
      }, isTeacher, roomId, forId);
    }
    function handleRoomMovement(title, type) {
      var rooms = [];
      var roomToTime = {};
      $scope.items.forEach(function (kid) {
        // only add kids to the list if they are signing in and they aren't selecting the current room
        if (kid.isIn === true && kid.current.isIn != true && kid.selectedChildRoomId != null && kid.selectedChildRoomId != kid.roomId) {
          if (!roomToTime[kid.selectedChildRoomId]) {
            roomToTime[kid.selectedChildRoomId] = {};
            rooms.push(kid.selectedChildRoomId);
          }
          if (!roomToTime[kid.selectedChildRoomId][kid.timeVal]) {
            roomToTime[kid.selectedChildRoomId][kid.timeVal] = [];
          }
          roomToTime[kid.selectedChildRoomId][kid.timeVal].push(kid);
        }
      });
      var rmCount = 0;
      rooms.forEach(function (rmId) {
        var timeToKids = roomToTime[rmId];
        for (var timeVal in timeToKids) {
          if (timeToKids.hasOwnProperty(timeVal)) {
            var ids = _.map(timeToKids[timeVal], function (kid) {
                return kid.id;
              });
            var date = new Date($utils.createDateFromDTStrings($utils.getDatePart(new Date()), timeVal).getTime() + 60000);
            var _timeMoved = $utils.createGsonCompatibleTimestamp(date);
            $backend.moveRoom(rmId, ids, function (err, data) {
              if (err) {
                console.log(err);
                //alert(err);
                $appstate.msg('Error moving room!', true, 'danger');
              } else {
                rmCount++;
                // clean up the room.  Move all kids from the old to new room in mem, then update their category to match the new room
                timeToKids[timeVal].forEach(function (kid) {
                  var oldRm = $facility.findRoom(kid.roomId), newRoom = $facility.findRoom(rmId);
                  $facility.moveRoom(kid, oldRm, newRoom, true);
                  // set that the room has been cached, so it doesn't get overridden by local events before we sync to server.
                  kid.cachedRoom = true;
                });
                // save facility when all moves done.
                if (rmCount == rooms.length) {
                  $timeout(function () {
                    $facility.save().then(function () {
                      var type = data.offline ? 'Cached' : 'Saved';
                      // force reselection of first kid since the selected child will have moved.
                      $timeout(function () {
                        $appstate.setCurrentRoom($appstate.model.selectedRoom);
                      }, 50);
                      $appstate.msg($translate.instant('RoomTransition') + ' ' + $translate.instant(type), true);
                      //$appstate.goHome(true);
                      //$appstate.nav.goHome();
                      finishAttendance(title, type);
                    });
                  });
                }
              }
            }, _timeMoved);
          }
        }
      });
    }
    function finishAttendance(title, type) {
      if ($state.includes('home.dash')) {
        $appstate.goHome(true);
      } else {
        $messageBox.open(title, title + ' ' + $translate.instant(type) + ' ' + $translate.instant('Successfully'), { oktext: $translate.instant('Ok') });
        $appstate.nav.nextState = $scope.save;
      }
    }
    $appstate.nav.nextState = $scope.save;
    $scope.setItemTime = function (item, preventBubble) {
      $scope.setTime(item);
      if (preventBubble) {
        event.preventDefault();
        event.cancelBubble = true;
      }
    };
    $scope.updateChildList = function (all) {
      if (all) {
        var allrooms = _.filter($facility.getSync().rooms, function (item) {
            return item.name == $translate.instant('AllChildren');
          });
        if (allrooms && allrooms[0] && allrooms[0].children) {
          var list = _.map(allrooms[0].children, function (c) {
              return $facility.findChild(c.id);
            });
          $scope.items = _.sortBy(angular.copy(list), 'firstName', 'lastName');
        }
      } else {
        $scope.items = angular.copy($facility.getSync().observed);
      }
      _.each($scope.items, function (item) {
        item.clean = true;
        item.selected = false;
      });
      $scope.showAllKids = all;
      $appstate.refresh(true, true);
    };
    var ctrl = this;
    $scope.setTime = function (item) {
      var modalInstance = $modal.open({
          templateUrl: 'views/popups/setTimePopup.html',
          controller: 'ModalinstanceCtrl',
          resolve: {
            date: function () {
              return $scope.value;
            }
          }
        });
      modalInstance.result.then(function (d) {
        // d = d + ":00";
        //console.log("Got date:" + d);
        //normalize to military
        if (d.indexOf('AM') > -1 || d.indexOf('PM') > -1) {
          d = $utils.getMilTime(d);
        }
        $scope.value = d;
        if (item)
          updateTime(item, d);
        else
          $scope.items.forEach(function (i) {
            updateTime(i, d);
          });
      }, function () {
      });
    };
    function updateTime(item, d) {
      if (!item.clean) {
        item.timeVal = d;
        item.time = $utils.getTimeString(item.timeVal);
      }
    }
    function _close() {
      if ($appstate.model.dirty) {
        $messageBox.open($translate.instant('UnsavedChanges'), $translate.instant('UnsavedChangesMessage'), {
          canceltext: $translate.instant('YesDiscardChanges'),
          oktext: $translate.instant('Cancel')
        }).result.then(function (selectedItem) {
          $timeout(function () {
            $appstate.canClose = _close;
          }, 50);
        }, function () {
          $appstate.canClose = null;
          $appstate.goHome();
        });
      } else {
        $appstate.goHome();
      }
    }
    $scope.getAttendanceNotes = function (item) {
      var str = '';
      if (item.clean || !item.isIn && item.attendanceNotes) {
        if (item.attendanceNotes) {
          str = ' ' + item.attendanceNotes;
          if (item.attendanceNotes.indexOf('(') > -1) {
            str = ' ' + item.attendanceNotes.substr(0, item.attendanceNotes.indexOf('('));
          }
        }
      } else {
        str = ' ' + $scope.model.selectedRoom.name;
      }
      return str;
    };
    $appstate.canClose = _close;
    $scope.close = _close;
    $appstate.setTime = $scope.setTime;
    $appstate.toggleIn = $scope.toggleIn;
    $appstate.getSelectedText = $scope.getSelectedText;
    $appstate.close = $scope.close;
    $appstate.updateChildList = $scope.updateChildList;
  }
]);
'use strict';
angular.module('kidreports2App').controller('EventsController', EventsController);
EventsController.$inject = [
  '$scope',
  '$rootScope',
  '$appstate',
  '$state',
  '$adapter',
  '$krConfig',
  '$utils',
  '$facility',
  '$timeout',
  '$krCache',
  '$backend',
  '$kidplanner',
  '$translate',
  '$location'
];
function EventsController($scope, $rootScope, $appstate, $state, $adapter, $krConfig, $utils, $facility, $timeout, $krCache, $backend, $kidplanner, $translate, $location) {
  $scope.currentPath = $location.path();
  var isParent = $backend.getCurrentUser().role === 'PARENT_ROLE';
  var f = $facility.getSync();
  if (f.props && f.props.photoVideoEnabled === 'false') {
    $scope.photoVideoEnabled = false;
  }
  $scope.shared = $appstate;
  $scope.filterDailyReport = $state.includes('home.dash');
  $scope.model = $appstate.model;
  $scope.photoVideoEnabled = true;
  var watch = $rootScope.$on('selectedRoomChange', function (newval, oldval) {
      onChange();
    });
  var watch2 = $rootScope.$on('kidPlannerDateChange', function (newval, oldval) {
      onChange();
    });
  function onChange() {
    var title = 'No Children Selected';
    if ($appstate.model.observed)
      title = $appstate.model.observed.length == 1 ? $appstate.model.observed[0].name : $appstate.model.observed.length + ' Children';
    if ($kidplanner.planInSession)
      $scope.status = $translate.instant('AddPlanFor') + ' ' + $kidplanner.planDate;
    else
      $scope.status = $translate.instant('EventSelectionForChild').replace('&name&', title);
    var cat = isParent ? $appstate.model.observed && $appstate.model.observed[0].category : $appstate.model.selectedCategory || $appstate.model.selectedRoom.category || $appstate.model.observed && $appstate.model.observed[0].category;
    function prepIcon(eventType) {
      if (eventType.icon == null) {
        eventType.icon = '/';
      }
    }
    // items is an array of group wrapper objects, each having a groupType and an items subarray
    $scope.classicItems = [];
    $scope.items = {};
    var items = _.filter($facility.getSync().eventTypes, function (r) {
        return $facility.allowEventType(r);
      });
    items.forEach(function (et) {
      if (et.categories[0] && cat && et.categories[0].id === cat.id && et.hidden != true) {
        if (et.name.indexOf('Picture/Video') > -1) {
          if (!$scope.photoVideoEnabled)
            return;
          else if ($facility.isLite()) {
            et.translatedname = 'Picture';
          } else {
            et.translatedname = $translate.instant('picturevideo');
          }
        }
        if (et.alertType == 'ALERT_ALL_EVENTS' && (!$facility.isLite() || window.innerWidth >= 600))
          return;
        et.iconUrl = $utils.createPhotoUrl($krConfig.s3Base, et.icon, $utils.getDefault(et));
        $scope.classicItems.push(et);
        var gt = $krConfig.isNZ() && et.groupName.toLowerCase() == 'portfolio' ? 'Observations' : et.groupName;
        var arr = $scope.items[gt] || [];
        arr.push(et);
        $scope.items[gt] = arr;
      }
    });
    // pull out daily report and pop to bottom
    var el = null;
    var idx = -1;
    $scope.classicItems = _.filter($scope.classicItems, function (i) {
      if (i.alertType == 'ALERT_ALL_EVENTS' && el == null) {
        el = i;
        return false;
      } else
        return true;
    });
    if (el) {
      $scope.classicItems.push(el);
    }
    var arr = [];
    angular.forEach($scope.items, function (v, k) {
      arr.push({
        groupName: k,
        items: v,
        baseGroupName: v[0].baseGroupName.replace(' ', '')
      });
    });
    $scope.items = _.sortBy(arr, function (el) {
      // groups should be in this order: DailyRoutine, Learning/(Portfolio), Meals, Administration
      if (el.baseGroupName === 'DailyRoutine')
        return 0;
      else if (el.baseGroupName === 'Portfolio')
        return 1;
      else if (el.baseGroupName === 'Meals')
        return 2;
      else if (el.baseGroupName === 'Administrative')
        return 3;
      else
        return 4;
    });
    _.each($scope.items, function (w) {
      w.items = _(w.items).chain().sortBy('name').sortBy('dailyReportOrder').value();
    });
  }
  $scope.$on('$destroy', function () {
    watch();
    watch2();
  });
  $scope.onClick = function (item) {
    $appstate.checkAttendance($appstate.model.selectedRoom.id, function (_item) {
      $timeout(function () {
        _onClick(_item || item);
      }, 0);
    }, item);
  };
  function _onClick(item) {
    $appstate.model.currentEvent = $adapter.createNewEvent(item);
    $appstate.model.currentEvent.observed = $appstate.model.observed;
    // first check if this is a nap event for a kid who already has one, and load that one if so
    if ($appstate.isNapEvent($appstate.model.currentEvent)) {
      $krCache.getCache('drafts', function (c) {
        var cache = c && _.isArray(c) ? c : [];
        /** 3 cases to check for:
           * 1.  Draft for single kid, selection is same kid -> edit event
           * 2.  Draft for several kids, selection is same group of kids -> edit event for all kids
           * 3.  Draft for several kids, selection is one of kids in draft -> edit event for just that kid
           * 4.  All other cases, create new draft.
           */
        cache.forEach(function (draft) {
          if (draft.eventType.id == $appstate.model.currentEvent.eventType.id) {
            if (kidsMatch(draft, $appstate.model.currentEvent)) {
              // break out to editing the existing one.
              $appstate.viewDraft(draft, isParent);
            } else if (kidsMatch(draft, $appstate.model.currentEvent, true)) {
              var lEvent = $adapter.createLegacyEvent(draft);
              $adapter.processEvent(lEvent);
              lEvent.eventType = $facility.findEventType(lEvent.eventType.id);
              lEvent.user = $facility.findUser(lEvent.user.id);
              lEvent.child = $facility.findChild($appstate.model.currentEvent.observed[0].id);
              //$appstate.viewEvent(lEvent, isParent);
              $appstate.model.currentEvent = $adapter.createEvent(lEvent, $facility.getSync());
            }
          }
        });
        finish();
      });
    } else {
      finish();
    }
  }
  function finish() {
    if ($state.includes('home.dash')) {
      $state.go('home.dash.event', {}, {
        location: true,
        inherit: true,
        relative: 'home.dash',
        notify: true
      });
    } else {
      $appstate.model.isRefreshing = false;
      $appstate.nav.next();
    }
  }
  function kidsMatch(draft, curr, checkIncluded) {
    var diff = _.filter(draft.observed, function (e1) {
        var found = false;
        curr.observed.forEach(function (e2) {
          found = found || e2.id == e1.id;
        });
        return !found;
      });
    return diff.length == 0 && curr.observed.length == draft.observed.length || draft.observed.length == diff.length + 1 && checkIncluded;
  }
  // INIT
  onChange();
}
'use strict';
angular.module('kidreports2App').controller('ParentPopupController', ParentPopupController);
ParentPopupController.$inject = [
  '$scope',
  '$facility',
  '$backend',
  '$appstate',
  '$modalInstance',
  'event',
  '$utils',
  '$model',
  '$timeout',
  '$stateStandards'
];
function ParentPopupController($scope, $facility, $backend, $appstate, $modalInstance, event, $utils, $model, $timeout, $stateStandards) {
  var facility = $facility.getSync();
  var currentUser = $backend.getCurrentUser();
  var isParent = currentUser.role == 'PARENT_ROLE';
  $scope.item = event;
  $scope.showLinks = !$facility.getSync().disableParentLinks || !isParent;
  $scope.translateDate = $utils.translateDate;
  $stateStandards.getPerformanceRatingLabels(facility.id).then(function (result) {
    $scope.progressLevels = [
      result['statestandards.labels.not-observed'],
      result['statestandards.labels.emerging'],
      result['statestandards.labels.progressing'],
      result['statestandards.labels.skilled']
    ];
  });
  $scope.ok = function () {
    $timeout(checkClose, 10);
    $modalInstance.close();
  };
  $scope.cancel = function () {
    $timeout(checkClose, 10);
    $modalInstance.dismiss('cancel');
  };
  $scope.getEventIcon = function (group) {
    return $appstate.getGroupIcon(group, false);
  };
  function checkClose() {
    if ($model.closeOnEventComplete) {
      window.top.postMessage('closeKRMobileNoReload', '*');
    }
  }
}
'use strict';
angular.module('kidreports2App').controller('ManageroomCtrl', [
  '$scope',
  '$appstate',
  '$native',
  '$state',
  '$online',
  '$messageBox',
  function ($scope, $appstate, $native, $state, $online, $messageBox) {
    $scope.attendance = function () {
      // short circuit for offline mode
      // if ( !$online.state() ) {
      //   $messageBox.open("Offline Mode", "Please try this again when you are online.", { oktext: "Ok"  });
      //   return;
      // }
      $appstate.nav.startPath('attendance');
    };
    $scope.teacherSignIn = function () {
      // short circuit for offline mode
      if (!$online.state()) {
        $messageBox.open('Offline Mode', 'Please try this again when you are online.', { oktext: 'Ok' });
        return;
      }
      $appstate.nav.startPath('teacherSignIn');
    };
    $scope.moveRoom = function () {
      // short circuit for offline mode
      //if ( !$online.state() ) {
      //  $messageBox.open("Offline Mode", "Please try this again when you are online.", { oktext: "Ok"  });
      //  return;
      // }
      $appstate.nav.startPath('moveRoom');
    }  // todo
;
  }
]);
'use strict';
angular.module('kidreports2App').controller('CoeCtrl', [
  '$scope',
  '$appstate',
  '$native',
  function ($scope, $appstate, $native) {
  }
]);
'use strict';
angular.module('kidreports2App').controller('PbnjCtrl', [
  '$scope',
  '$appstate',
  '$native',
  '$timeout',
  '$state',
  '$pbnj',
  '$http',
  '$utils',
  function ($scope, $appstate, $native, $timeout, $state, $pbnj, $http, $utils) {
    //$scope.username = "Gary";
    //$scope.password = "Akavit1";
    // $scope.vidsrc = "http://admin:128367@361764.peanutbutterandjellytv.com:10510/snapshot.jpg?uniq="+Math.random();
    //$scope.vidsrc = "http://admin:128367@361764.peanutbutterandjellytv.com:10510/stream.jpg";
    //http://518615.peanutbutterandjellytv.com/Cam6/key=538451
    $scope.cams = [];
    //    $scope.cams.push({
    //      name: "Test Cam",
    //      //url: "http://361764.peanutbutterandjellytv.com:10510/stream.jpg"
    //      url:  "http://518615.peanutbutterandjellytv.com/Cam24?key=654659"
    //    });
    $scope.vidsrc = 'http://admin:admin@123156.peanutbutterandjellytv.com:13000/stream.jpg';
    $scope.login = function () {
      $appstate.model.isRefreshing = true;
      $pbnj.getCamList($scope.username, $scope.password, function (list) {
        //$scope.$apply(function() {
        $scope.cams = list;
        $appstate.model.isRefreshing = false;  //});
      });
    };
    function buildSrc(url) {
      // url = url.replace("snapshot.jpg", "stream.jpg")
      // url = (url.indexOf("http://") != -1) ? url.substring(7, url.length) : url;
      if (url.indexOf('http://') < 0)
        return 'http://' + url;  // $scope.username + ":" + $scope.password + "@" + url;
      else
        return url;
    }
    $scope.intervals = [];
    for (var i = 1; i < 6; i++) {
      $scope.intervals.push({
        name: i + ' Seconds',
        interval: i * 1000
      });
    }
    $scope.interval = $scope.intervals[1];
    $scope.running = false;
    function refreshDroid() {
      var item = $scope.item;
      $scope.running = true;
      //var url = item.url.replace("stream.jpg", "snapshot.jpg");
      // convert a url like this: http://518615.peanutbutterandjellytv.com/Cam24?key=654659
      // into this: http://518615.peanutbutterandjellytv.com/video.jpg?ch=24
      var url = item.url;
      var ch = url.match('Cam([0-9]+)');
      var key = url.match('key=([0-9]+)');
      url = url.substring(0, url.indexOf(ch[0]));
      ch = ch[1];
      url = url + 'video.jpg?ch=' + ch + '&' + key[0] + '&uniq=' + Math.random();
      var src = buildSrc(url);
      //      $http({method: 'GET', url: src}).
      //        success(function(data, status, headers, config) {
      //          var el = document.getElementById("pbnjvidimg");
      //            if ( el ) {
      //
      //              //var arr = new Uint8Array(data);
      //
      //              var rw =btoa(unescape(encodeURIComponent( data )));
      //
      //              // Convert the int array to a binary string
      //              // We have to use apply() as we are converting an *array*
      //              // and String.fromCharCode() takes one or more single values, not
      //              // an array.
      //              //var raw = String.fromCharCode.apply(null,arr);
      //
      //              // This works!!!
      //              // var b64=btoa(raw);
      //              var dataURL="data:image/jpeg;base64,"+rw;
      //              el.src = dataURL;
      //              if ( $state.is('home.action.pbnj') ) {
      //                $timeout(function() {refreshDroid()}, 0);
      //              }
      //            }
      //          }).
      //        error(function(data, status, headers, config) {
      //          // called asynchronously if an error occurs
      //          // or server returns response with an error status.
      //        });
      var el = document.getElementById('pbnjvidimg');
      //console.log(src);
      if (el) {
        el.src = src;
        if ($state.is('home.action.pbnj')) {
          setTimeout(function () {
            refreshDroid();
          }, $scope.interval.interval);
        } else
          $scope.running = false;
      }  // keep it going until we change pages.
    }
    function useDroid() {
      //return $native.isNative() && $native.android();
      return false;
    }
    $scope.isAndroid = useDroid();
    $scope.selectCam = function (item) {
      $scope.item = item;
      if ($native.isNative()) {
        forge.media.videoPlay(item.url, function () {
        }, function (err) {
        });
      } else {
        $utils.alert('Not Supported');  //$scope.vid = { url : item.url };
                                        //document.getElementById("pbnjvidimg").src = item.url;
                                        //$scope.pbnjvidimg = item.url;
                                        //$appstate.vidurl = url;
      }  //if ( useDroid() && !$scope.running ) {
         //$scope.droidVidSrc = item.url;
         //document.getElementById("pbnjdroidvidimg").src = item.url;
         //refreshDroid();
         //} else if ( !useDroid() ) {
         //        document.getElementById("pbnjvidimg").src = item.url;
         //$scope.vidsrc = item.url;
         // }
         //      } else {
         //        $scope.vidsrc = buildSrc(item.url);
         //        //console.log($scope.vidsrc);
         //      }
    }  //    $scope.watch("selectedCam", function() {
       //      $scope.selectCam($scope.selectedCam);
       //    });
       // dummy it up for now for demo purposes.
       // $scope.selectCam($scope.cams[0]);
;
  }
]);
'use strict';
angular.module('kidreports2App').controller('SettingsCtrl', [
  '$scope',
  '$appstate',
  '$native',
  '$state',
  '$messageBox',
  '$krCache',
  '$online',
  '$rootScope',
  '$backend',
  '$localdb',
  '$events',
  '$krConfig',
  '$ajax',
  'translations',
  '$translate',
  '$facility',
  '$image',
  '$auth',
  '$http',
  '$timeout',
  '$utils',
  '$featureFlags',
  function ($scope, $appstate, $native, $state, $messageBox, $krCache, $online, $rootScope, $backend, $localdb, $events, $krConfig, $ajax, translations, $translate, $facility, $image, $auth, $http, $timeout, $utils, $featureFlags) {
    function init() {
      $scope.isParent = $backend.getCurrentUser().role == 'PARENT_ROLE';
      $localdb.get('dashboard').then(function (db) {
        $scope.dashboard = db != null && db != 'null' && db != 'false' && db != false;
      });
      $localdb.get('showMediaInStream').then(function (media) {
        $scope.showMediaInStream = media === 'false' || media === false ? false : true;
      });
      //$scope.profilePictureCount = $image.getProfilePicCount();
      //$image.getProfilePicSize().then(function (s) {
      //  $scope.profilePictureSize = s;
      //});
      var currentUser = $facility.getSync().currentUser;
      if (currentUser.legacyRole == 'SYSTEM_ADMIN_ROLE' || currentUser.legacyRole == 'COMPANY_ADMIN_ROLE') {
        $scope.flags = $featureFlags.getAllFlags();
        $scope.isFeatureEnabled = function (featureKey) {
          return $featureFlags.isFeatureEnabled(featureKey);
        };
      }
    }
    init();
    $scope.clearProfilePictures = function () {
      $image.removeAllProfilePics().then(function () {
        $utils.alert('All Pics Removed!');
        init();
      });
    };
    $localdb.get('disableProfilePictureCaching').then(function (t) {
      $scope.disableProfilePictureCaching = t != null && (t == true || t == 'true');
    });
    $scope.currLang = translations.getCurrent();
    $scope.languages = translations.getLanguages();
    $scope.$watch('currLang', function () {
      translations.setLanguage($scope.currLang);
    });
    $scope.fac = $facility.getSync();
    $scope.isNative = $native.isNative();
    $scope.save = function () {
      $appstate.model.dirty = false;
      $localdb.set('dashboard', $scope.dashboard);
      $localdb.set('showMediaInStream', $scope.showMediaInStream);
      $localdb.set('disableProfilePictureCaching', $scope.disableProfilePictureCaching);
      $scope.model.disableProfilePictureCaching = $scope.disableProfilePictureCaching;
      $appstate.model.showMediaInStream = $scope.showMediaInStream;
      $appstate.model._dashboard = $scope.dashboard;
      $appstate.nav.goHome();
    };
    $scope.clearCache = function () {
      $messageBox.open($translate.instant('ConfirmDelete'), $translate.instant('EventDeleteConfirm'), {
        oktext: $translate.instant('Yes'),
        canceltext: $translate.instant('No')
      }).result.then(function () {
        $krCache.setCache([]);
      }, function () {
      });
    };
    $scope.clearNotices = function () {
      $krCache.setCache([], 'notifications');
    };
    $scope.clearUploads = function () {
      $messageBox.open($translate.instant('ConfirmDelete'), $translate.instant('UploadDeleteConfirm'), {
        oktext: $translate.instant('Yes'),
        canceltext: $translate.instant('No')
      }).result.then(function () {
        $krCache.setCache([], 'uploads');
      }, function () {
      });
    };
    $scope.restart = function (item, ajax) {
      if (ajax) {
      } else {
        item.status = 'Pending';
        $krCache.setCache($appstate.model.cachedMedia, 'uploads');
      }
    };
    $scope.getEventStatus = function (evt) {
      if (evt._deleted)
        return $translate.instant('PendingDelete');
      if (evt.id)
        return $translate.instant('PendingUpdate');
      else
        return $translate.instant('PendingSave');
    };
    $scope.deleteItem = function (item, ajax) {
      if (!item.info) {
      } else {
        item.status = 'Deleted';
        if (ajax) {
          $krCache.setCache($appstate.model.cachedItems);
        } else {
          $krCache.setCache($appstate.model.cachedMedia, 'uploads');
        }
      }
    };
    $scope.testConnection = function () {
      $appstate.model.isRefreshing = true;
      $timeout(_realtest);
    };
    function _realtest() {
      var canvas = angular.element(document.querySelector('#canvas'))[0];
      var ratio = 1;
      // window.devicePixelRatio || 1;
      canvas.width = 10000;
      canvas.height = 6000;
      canvas.getContext('2d').scale(ratio, ratio);
      var context = canvas.getContext('2d');
      context.fillStyle = 'red';
      context.fillRect(0, 0, canvas.width, canvas.height);
      context.fillStyle = 'blue';
      context.fillRect(canvas.width / 2, canvas.height / 2, canvas.width / 2, canvas.height / 2);
      var imgsrc = angular.copy($appstate.model.signaturePad.toDataURL());
      var startping = new Date().getTime();
      var opts = {
          method: 'GET',
          url: $krConfig.getAjaxBase() + '/public/ping.json',
          params: {}
        };
      $ajax.ajax(opts, function (err, data) {
        $utils.log('Got ping');
        if (err) {
          $appstate.model.isRefreshing = false;
          $utils.alert('Error Sending Ping to Server');
        } else {
          var endping = new Date().getTime();
          opts = {
            method: 'GET',
            url: $krConfig.getAjaxBase() + '/public/ping2.json',
            params: {}
          };
          $ajax.ajax(opts, function (err, data) {
            $utils.log('Got painting');
            if (err) {
              $appstate.model.isRefreshing = false;
              $utils.alert('Error Downloading Test File');
            } else {
              var endget = new Date().getTime();
              var token = $auth.getCredentialsSync();
              var opts = {
                  method: 'POST',
                  url: $krConfig.getMediaBase() + '/data/uploadMedia',
                  params: {
                    key: null,
                    type: 'PHOTO',
                    token: token.token
                  },
                  data: imgsrc,
                  preserveDataFormat: true
                };
              opts.params.filename = 'TEMP.png';
              $http(opts).then(function (d) {
                $utils.log('sent data');
                $appstate.model.isRefreshing = false;
                var endpost = new Date().getTime();
                var pingtime = endping - startping;
                var dltime = endget - endping;
                var ultime = endpost - endget;
                $utils.alert('Ping: ' + pingtime + 'ms, 1MB Download: ' + dltime + 'ms, 1MB Upload: ' + ultime + 'ms');
              }, function (err) {
                $appstate.model.isRefreshing = false;
                $utils.alert('Error Uploading Test File');
              });
            }
          });
        }
      });
    }
    $appstate.model.isUploading = $native.uploadThreadRunning();
    $scope.startTimer = function () {
      $appstate.model.isUploading = true;
      $native.uploadInternal();  //$scope.uploadTimerRunnng
    };
    $scope.startEvtTimer = function () {
      if ($appstate.model.isSavingAjax) {
        $appstate.model.isSavingAjax = false;
        $backend.cancelEventSave();
      } else {
        $appstate.model.isSavingAjax = true;
        $backend.saveEventInternal();
      }
      $events.push();  //$scope.uploadTimerRunnng
    };
    var ajaxCompleteWatch = $rootScope.$on('ajaxComplete', function (evt, v) {
        $appstate.model.isSavingAjax = false;
      });
    var ajaxRunningWatch = $rootScope.$on('ajaxRunning', function (evt, v) {
        $appstate.model.isSavingAjax = true;
      });
    var uploadsCompleteWatch = $rootScope.$on('uploadsComplete', function (evt, v) {
        $appstate.model.isUploading = false;
      });
    var uploadsRunningWatch = $rootScope.$on('uploadsRunning', function (evt, v) {
        $appstate.model.isUploading = true;
      });
    $appstate.nav.nextState = $scope.save;
    $scope.online = $online.state();
    var onlineWatch = $rootScope.$on('onlineStateChange', function (evt, isOnline) {
        //console.log(evt + " online:" + isOnline);
        $scope.$apply(function () {
          $scope.online = isOnline;
        });
      });
    $scope.$on('$destroy', function () {
      ajaxCompleteWatch();
      ajaxRunningWatch();
      uploadsCompleteWatch();
      uploadsRunningWatch();
      onlineWatch();
    });
    $scope.getLength = function (hash) {
      return Object.size(hash);
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('SendmessageCtrl', [
  '$scope',
  '$appstate',
  '$backend',
  '$state',
  '$online',
  '$messageBox',
  '$ajax',
  '$krConfig',
  '$facility',
  '$translate',
  '$timeout',
  function ($scope, $appstate, $backend, $state, $online, $messageBox, $ajax, $krConfig, $facility, $translate, $timeout) {
    $scope.message = {};
    if ($state.includes('home.dash') && !($appstate.model.observed && $appstate.model.observed.length > 0)) {
      $appstate.msg($translate.instant('NoChildrenSelected'), true, 'warning');
      $state.go('home.dash');
    }
    function goHome() {
      $messageBox.open($translate.instant('OfflineMode'), $translate.instant('OfflineText'), { oktext: $translate.instant('Ok') }).result.then(function (selectedItem) {
        $appstate.nav.goHome();
      }, function () {
        $appstate.nav.goHome();
      });
    }
    // short circuit for offline mode
    if (!$online.state()) {
      goHome();
    } else {
      var online = true;
      if ($facility.getSync() && $facility.getSync().disableServerPing) {
        // check if we are "online" but we can't ping the server
        var opts = {
            method: 'GET',
            url: $krConfig.getAjaxBase() + '/public/ping.json',
            params: {}
          };
        var pingTrue = false;
        $appstate.model.isRefreshing = true;
        $ajax.ajax(opts, function (err, data) {
          $appstate.model.isRefreshing = false;
          pingTrue = true;
          if (err) {
            goHome();
          } else {
            online = true;
          }
        });
        setTimeout(function () {
          if (!pingTrue) {
            $scope.$apply(function () {
              $appstate.model.isRefreshing = false;
              goHome();
            });
          }
        }, 2000);
      }
    }
    $scope.readOnly = false;
    $scope.send = function () {
      $scope.readOnly = true;
      $appstate.model.isRefreshing = true;
      $timeout(realSend);
    };
    function realSend() {
      //alert("Send Message with Body:"+$scope.message.body+", subj:"+$scope.message.subject);
      $scope.message.childIds = [];
      // $appstate.model.observed.forEach(function (item) {
      //   $scope.message.childIds.push(item.id);
      // })
      //
      $facility.getSync().observed.forEach(function (item) {
        if (item.selected)
          $scope.message.childIds.push(item.id);
      });
      $backend.sendMessage($scope.message, function (err, data) {
        $appstate.model.isRefreshing = false;
        $scope.readOnly = false;
        if (err) {
          //alert(err);
          console.log(err);
          $appstate.msg($translate.instant('ErrorSendingMsg'), true);
        } else {
          var type = data.offline ? 'Cached' : 'Sent';
          console.log('sent msg');
          $appstate.msg($translate.instant('MessageSentSuccess'), true);
          //alert("Message Sent Succesfully!");
          $appstate.nav.goHome();
        }
      });
    }
  }
]);
'use strict';
angular.module('kidreports2App').controller('MoveroomCtrl', [
  '$scope',
  '$q',
  '$appstate',
  '$backend',
  '$facility',
  '$state',
  '$messageBox',
  '$translate',
  '$timeout',
  '$localdb',
  function ($scope, $q, $appstate, $backend, $facility, $state, $messageBox, $translate, $timeout, $localdb) {
    $scope.rooms = _.filter($facility.getSync().rooms, function (item) {
      return item.name != $translate.instant('AllChildren') && item.id != $appstate.model.selectedRoom.id;
    });
    $scope.selected = $scope.rooms[0];
    var OneTimeCBKey = 'oneTimeCbVal';
    $scope.isDash = $state.includes('home.dash');
    if ($scope.isDash && !($appstate.model.observed && $appstate.model.observed.length > 0)) {
      $appstate.msg($translate.instant('NoChildrenSelected'), true, 'warning');
      $state.go('home.dash');
    }
    function moveDone(type) {
      $appstate.msg($translate.instant('RoomTransition') + ' ' + $translate.instant(type), true);
      $appstate.nav.goHome();
    }
    function hasTakenAttendance(roomId) {
      var room = $facility.findRoom(roomId);
      var found = false;
      room.children.forEach(function (c) {
        found = found || (c.attendanceSet || c.isIn != null);
      });
      return found;
    }
    function checkOneTimeCb() {
      var d = $q.defer();
      $localdb.get(OneTimeCBKey).then(function (ocb) {
        //console.log("ocb get " + ocb);
        //console.log("$scope.Onetimecb get " + $scope.Onetimecb);
        d.resolve(ocb);
      }, function (err) {
        d.reject(err);
      });
      return d.promise;
    }
    $scope.moveRoom = function () {
      var selectedChildren = $appstate.model.observed;
      var s = '';
      _.each(selectedChildren, function (child, idx) {
        if (selectedChildren.length == 1 || idx == selectedChildren.length - 1) {
          s += child.name + ' ';
        } else if (idx == selectedChildren.length - 2) {
          s += child.name + ' and ';
        } else if (idx < selectedChildren.length - 1 && selectedChildren.length > 2) {
          s += child.name + ', ';
        }
      });
      checkOneTimeCb().then(function (ocb) {
        if (ocb == false || ocb == null) {
          $messageBox.open($translate.instant('Caution'), $translate.instant('RoomChange') + ' ' + s + $translate.instant('From') + ' ' + $appstate.model.selectedRoom.name + ' ' + $translate.instant('To') + ' ' + $scope.selected.name + '.', {
            oktext: $translate.instant('Yes'),
            canceltext: $translate.instant('No'),
            showOneTimeCb: true
          }).result.then(function (ocbValue) {
            $localdb.set(OneTimeCBKey, ocbValue);
            console.log('ocb val set ' + ocbValue);
            s = '';
            confirmedMove();
          });
        } else if (ocb == true) {
          confirmedMove();
        } else {
        }
      });
    };
    function confirmedMove() {
      var currAttendance = hasTakenAttendance($appstate.model.selectedRoom.id) || false;
      var newAttendance = hasTakenAttendance($scope.selected.id) || false;
      if (currAttendance != newAttendance) {
        var msg;
        if (currAttendance && !newAttendance) {
          msg = $translate.instant('AttendanceWarningMsg1');
        } else {
          msg = $translate.instant('AttendanceWarningMsg2');
        }
        $messageBox.open($translate.instant('AttendanceWarning'), msg + ' ' + $translate.instant('Continue'), {
          oktext: $translate.instant('Yes'),
          canceltext: $translate.instant('No')
        }).result.then(function (selectedItem) {
          doMove();
        }, function () {
        });
      } else {
        doMove();
      }
    }
    function doMove() {
      var ids = [];
      $appstate.model.observed.forEach(function (item) {
        ids.push(item.id);
      });
      $backend.moveRoom($scope.selected.id, ids, function (err, data) {
        if (err) {
          console.log(err);
          //alert(err);
          $appstate.msg('Error moving room!', true, 'danger');
        } else {
          // clean up the room.  Move all kids from the old to new room in mem, then update their category to match the new room
          var oldRm = $appstate.model.selectedRoom, newRoom = $scope.selected;
          $appstate.model.observed.forEach(function (kid) {
            $facility.moveRoom(kid, oldRm, newRoom, true);
            // set that the room has been cached, so it doesn't get overridden by local events before we sync to server.
            kid.cachedRoom = true;
          });
          $facility.save().then(function () {
            var type = data.offline ? 'Cached' : 'Saved';
            if ($state.includes('home.dash')) {
              // force reselection of first kid since the selected child will have moved.
              $timeout(function () {
                $appstate.setCurrentRoom($appstate.model.selectedRoom);
              }, 50);
              moveDone(type);
            } else {
              $messageBox.open($translate.instant('RoomTransition') + ' ' + $translate.instant(type), $translate.instant('RoomTransitionMsg'), {
                oktext: $translate.instant('Yes'),
                canceltext: $translate.instant('No')
              }).result.then(function (selectedItem) {
                //$state.go("home.action.selectEvent");
                $appstate.nav.back();
              }, function () {
                moveDone(type);
              });
            }
          });
        }
      });
    }
  }
]);
'use strict';
angular.module('kidreports2App').controller('MediaCtrl', [
  '$scope',
  '$appstate',
  '$native',
  '$translate',
  '$facility',
  function ($scope, $appstate, $native, $translate, $facility) {
    $scope.photo = function () {
      if (!$native.isNative()) {
        //alert("Only supported on native devices!");
        $appstate.msg($translate.instant('NativeOnly'), true, 'warning');
        return;
      }
      $native.image(function (err, files, urls) {
        if (err) {
          //alert(err);
          console.log(err);
          $appstate.msg($translate.instant('ErrorSavingPhoto'), true, 'danger');
        } else {
          _.each(urls, function (url) {
            $native.saveToLibrary(url);
          });
          $appstate.nav.goHome();
        }
      }, _.extend({ source: 'camera' }, $facility.getImageParameters()));
    };
    $scope.video = function () {
      if (!$native.isNative()) {
        //alert("Only supported on native devices!");
        $appstate.msg($translate.instant('NativeOnly'), true, 'warning');
        return;
      }
      $native.video(function (err, file, url) {
        if (err) {
          //alert(err);
          console.log(err);
          $appstate.msg($translate.instant('ErrorSavingVideo'), true, 'danger');
        } else {
          $native.saveToLibrary(url);
          $appstate.nav.goHome();
        }
      }, {
        source: 'camera',
        videoDuration: $facility.getVideoLength()
      });
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('CreatemediaCtrl', [
  '$scope',
  '$appstate',
  '$native',
  function ($scope, $appstate, $native) {
    // This view acts like a dialog
    $appstate.nav.dialog();
    $scope.continueNav = function () {
      $appstate.nav.next();
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('VieweventsCtrl', [
  '$scope',
  '$appstate',
  '$events',
  '$utils',
  '$adapter',
  '$backend',
  '$native',
  '$krConfig',
  '$ajax',
  '$online',
  '$messageBox',
  '$state',
  '$facility',
  '$timeout',
  '$eventAdapter',
  function ($scope, $appstate, $events, $utils, $adapter, $backend, $native, $krConfig, $ajax, $online, $messageBox, $state, $facility, $timeout, $eventAdapter) {
    var child = $appstate.model.observed[0];
    $scope.status = child.name;
    function goHome() {
      $messageBox.open($translate.instant('OfflineMode'), $translate.instant('OfflineText'), { oktext: $translate.instant('Ok') }).result.then(function (selectedItem) {
        $appstate.nav.goHome();
      }, function () {
        $appstate.nav.goHome();
      });
    }
    // short circuit for offline mode
    if (!$online.state()) {
      goHome();
    } else {
      var online = true;
      if ($facility.getSync() && $facility.getSync().disableServerPing) {
        // check if we are "online" but we can't ping the server
        var opts = {
            method: 'GET',
            url: $krConfig.getAjaxBase() + '/public/ping.json',
            params: {}
          };
        var pingTrue = false;
        $appstate.model.isRefreshing = true;
        $ajax.ajax(opts, function (err, data) {
          pingTrue = true;
          if (err) {
            $appstate.model.isRefreshing = false;
            goHome();
          } else {
            online = true;
          }
        });
        setTimeout(function () {
          if (!pingTrue) {
            $scope.$apply(function () {
              $appstate.model.isRefreshing = false;
              goHome();
            });
          }
        }, 10000);
      }
    }
    // Always start with today
    $scope.items = [];
    $scope.images = [];
    $scope.videos = [];
    $scope.onClick = function (item) {
      $appstate.model.currentEvent = $adapter.createEvent(item, $facility.getSync());
      $appstate.nav.next();
    };
    // The date scope variable needs to be watched because it is being modified from the DOM (via the directive)
    $scope.$watch('date', function (newval, oldval) {
      newval += 'T12:00:00';
      //console.log(newval);
      var date = Date.fromISO(newval);
      //new Date(newval);
      if (isNaN(date))
        return;
      //console.log("date value changed, setting new val");
      //console.log(date);
      $appstate.model.currentDate = date;
      $appstate.model.isRefreshing = true;
      $eventAdapter.getEvents([child], date, false, null, null, false, false, true).then(function (data) {
        $scope.items = data.events;
        $appstate.model.isRefreshing = false;
      });
    });
    var dayMillis = 24 * 60 * 60 * 1000;
    function updateDate(offset) {
      $scope.date = $utils.getDatePart(new Date($appstate.model.currentDate.setTime($appstate.model.currentDate.getTime() + offset)));
    }
    $scope.forwardDate = function () {
      updateDate(dayMillis);
    };
    $scope.forwardWeek = function () {
      updateDate(7 * dayMillis);
    };
    $scope.backDate = function () {
      updateDate(-dayMillis);
    };
    $scope.backWeek = function () {
      updateDate(-7 * dayMillis);
    };
    // set the date after defining the watch
    $scope.date = $utils.getDatePart($appstate.model.currentDate || new Date());
    $timeout(function () {
      $native.enhanceUI();
    }, 1000);
    $scope.showEvent = function (ele) {
      return !ele.eventType.hidden;
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('EventdetailsCtrl', [
  '$scope',
  '$appstate',
  '$compile',
  '$native',
  '$utils',
  '$backend',
  '$events',
  '$adapter',
  '$messageBox',
  '$state',
  '$krCache',
  '$krConfig',
  '$modal',
  '$model',
  '$rootScope',
  '$timeout',
  '$facility',
  '$auth',
  '$kidplanner',
  'locale',
  '$defaults',
  '$base64',
  '$ajax',
  '$http',
  '$reminders',
  '$stateStandards',
  '$templateMgr',
  '$translate',
  function ($scope, $appstate, $compile, $native, $utils, $backend, $events, $adapter, $messageBox, $state, $krCache, $krConfig, $modal, $model, $rootScope, $timeout, $facility, $auth, $kidplanner, locale, $defaults, $base64, $ajax, $http, $reminders, $stateStandards, $templateMgr, $translate) {
    var isParent = $backend.getCurrentUser().role === 'PARENT_ROLE';
    $scope.isParent = isParent;
    // State Standards Variables:
    $scope.facility = false;
    $scope.existingStandards = [];
    $scope.eventStandardSelections = [];
    $scope.displayStandards = !isParent && _.get($appstate, 'model.currentEvent.eventType.stateStandardsDisplay') !== 'NOT_SHOWN';
    $scope.eventType = $appstate.model.currentEvent.eventType.translatedname || $appstate.model.currentEvent.eventType.name;
    // On instantiation, get standards information and check if there are existing standards on the event:
    getStandardsInfo();
    checkExistingStandards();
    $scope.kidplanner = $kidplanner;
    $scope.enableAutoApplyPlans = $facility.getSync().enableAutoApplyPlans;
    $scope.isLite = $facility.isLite;
    $scope.addingMedia = false;
    $scope.rooms = _.filter($facility.getSync().rooms, function (item) {
      return $appstate.model.selectedRoom != null && item.name != $translate.instant('AllChildren') && item.id != $appstate.model.selectedRoom.id;
    });
    $scope.selected_items = {};
    $scope.eventGroupIcon = $appstate.getGroupIcon($appstate.model.currentEvent.eventType.group, true);
    $scope.eventGroupColor = $appstate.getGroupColor($appstate.model.currentEvent.eventType.group);
    /** Functions **/
    /**
     * @function checkExistingStandards()
     * @description Checks if an existing event is being edited, and if there are existing state standards
     */
    function checkExistingStandards() {
      var stateStandards = _.get($appstate, 'model.currentEvent.stateStandards');
      if (stateStandards) {
        $scope.existingStandards = angular.copy(stateStandards);
      }
    }
    /**
     * @function getStandardsInfo()
     * @description Retrieves state standards
     * @returns {void}
     */
    function getStandardsInfo() {
      $stateStandards.getFacility().then(function (facility) {
        if (facility.state && facility.state !== 'None') {
          $scope.standardsFacility = facility;
          $scope.toggleTab('notes');
        }
        $scope.toggleTab('notes');
      });
    }
    /**
     * @function printTime()
     * @param ms
     * @returns {string}
     */
    function printTime(ms) {
      var hr = parseInt(ms / (1000 * 60 * 60), 10);
      ms = ms - hr * 1000 * 60 * 60;
      var min = parseInt(ms / (1000 * 60), 10);
      ms = ms - min * 1000 * 60;
      var sec = parseInt(ms / 1000, 10);
      return $utils.zeroPad(hr) + ':' + $utils.zeroPad(min) + ':' + $utils.zeroPad(sec);
    }
    /**
     * @function updateTimerValue()
     */
    function updateTimerValue() {
      if ($scope.napTimerStarted) {
        $scope.nowTime += 1000;
        $scope.timerValue = printTime($scope.nowTime - $scope.startTime);
        $appstate.model.currentEvent.properties.nowTime = $scope.nowTime;
        $timeout(updateTimerValue, 1000);
      }
    }
    /** Functions on Scope **/
    /**
     * @function closeSelectRoom()
     */
    $scope.closeSelectRoom = function () {
      $scope.showSelectRoom = false;
    };
    /**
     * @function deselectAll()
     */
    $scope.deselectAll = function () {
      $scope.selectRoomModel = [];
    };
    /**
     * @function isNewKidPlan()
     * @description Returns true/false whether plan is new
     * @returns {boolean}
     */
    $scope.isNewKidPlan = function () {
      return $kidplanner.currentPlan && !$kidplanner.currentPlan.id;
    };
    /**
     * @function saveSelectedRooms()
     */
    $scope.saveSelectedRooms = function () {
      $scope.selected_items = angular.copy($scope.selectRoomModel);
      $scope.showSelectRoom = false;
    };
    /**
     * @function selectAll()
     * @description Selects all in room model
     * @returns {void}
     */
    $scope.selectAll = function () {
      for (var i = 0; i < $scope.rooms.length; i++) {
        $scope.selectRoomModel[$scope.rooms[i].id] = true;
      }
    };
    /**
     * @function showSelectRoomDialog()
     */
    $scope.showSelectRoomDialog = function () {
      $scope.showSelectRoom = true;
      $scope.selectRoomModel = angular.copy($scope.selected_items);
    };
    $appstate.model.currentEvent.properties = $appstate.model.currentEvent.properties || {};
    $scope.timerValue = '00:00:00';
    var d = new Date().getTime();
    $scope.startTime = $appstate.model.currentEvent.properties.startTime || new Date(d).getTime();
    $scope.nowTime = new Date(d).getTime();
    $scope.napTimerStarted = false;
    $scope.isKidPlan = $kidplanner.planInSession;
    $scope.signatureRequired = $appstate.model.currentEvent.eventType.signatureRequired && $appstate.model.currentEvent.eventType.signatureRequired == true;
    /**
     * @function napTimer()
     * @param start
     * @param internal
     */
    $scope.napTimer = function (start, internal) {
      if (start && !$scope.napTimerStarted) {
        $scope.napTimerStarted = true;
        $appstate.model.currentEvent.properties.startTime = $scope.startTime;
        $appstate.model.currentEvent.properties.napTimerStarted = true;
        $appstate.model.currentEvent.properties.stoppedManually = false;
        $timeout(updateTimerValue, 0);
        if (!internal) {
          $timeout(function () {
            handleClick(true);
          }, 500);
        }
      } else if ($scope.napTimerStarted) {
        $scope.napTimerStarted = false;
        $appstate.model.currentEvent.properties.napTimerStarted = false;
        $appstate.model.currentEvent.properties.stoppedManually = true;
        $adapter.updateNapEndTime($appstate.model.currentEvent);
      }
    };
    /**
     * @function resetTimer()
     */
    $scope.resetTimer = function () {
      $scope.nowTime = new Date(d).getTime();
      $scope.startTime = new Date(d).getTime();
      $scope.timerValue = '00:00:00';
      $scope.napTimerStarted = false;
      $appstate.model.currentEvent.properties.napTimerStarted = false;
      $appstate.model.currentEvent.properties.nowTime = 0;
      delete $appstate.model.currentEvent.properties.startTime;
    };
    //KR965?
    $scope.getStatus = function () {
      var title = $appstate.model.currentEvent.observed.length + ' ' + $translate.instant('Children');
      if ($appstate.model.currentEvent.observed.length === 1 && $appstate.model.currentEvent.observed[0]) {
        var kid = $facility.findChild($appstate.model.currentEvent.observed[0].id);
        title = kid.firstName + ' ' + kid.lastName;
      }
      if ($kidplanner.planInSession) {
        return $translate.instant('Planning') + ':' + $appstate.model.currentEvent.eventType.name;
      } else {
        var name = $appstate.model.currentEvent.eventType.translatedname || $appstate.model.currentEvent.eventType.name;
        if ($appstate.model.currentEvent.eventType.alertType == 'ALERT_ALL_EVENTS') {
          name = $translate.instant('DailyReport');
        }
        return $appstate.model.currentEvent.eventType.groupName;  //return $translate.instant('EventForChild').replace('&event&', name).replace('&name&', title);
      }
    };
    $scope.isNapEvent = function (evt) {
      return $appstate.isNapEvent($appstate.model.currentEvent);
    };
    if ($scope.isNapEvent() && $appstate.model.currentEvent.properties.napTimerStarted) {
      //$scope.startTime = $appstate.model.currentEvent.properties.startTime;
      //$scope.nowTime = new Date().getTime();
      $scope.napTimer(true, true);
    }
    $scope.status = $scope.getStatus();
    $scope.isOldEvent = function () {
      var maxDaysOld = 30;
      var msPerDay = 86400000;
      var eventDateTime = $appstate.model.currentEvent.timestamp;
      if (typeof $appstate.model.currentEvent.timestamp == 'string') {
        eventDateTime = new Date($appstate.model.currentEvent.timestamp);
      }
      var eventDay = Math.trunc(eventDateTime.getTime() / msPerDay);
      var curDay = Math.trunc(new Date().getTime() / msPerDay);
      return curDay - eventDay > maxDaysOld;
    };
    var observedWatcher = $rootScope.$on('observedChange', function (newval, oldval) {
        $appstate.model.currentEvent.observed = $appstate.model.observed;
        $scope.status = $scope.getStatus();
      });
    $scope.$on('$destroy', function () {
      observedWatcher();
    });
    function hasGetUserMedia() {
      return !!(navigator.getUserMedia || navigator.webkitGetUserMedia || navigator.mozGetUserMedia || navigator.msGetUserMedia);
    }
    $scope.hasGetUserMedia = true;
    //hasGetUserMedia();
    $appstate.dirty = true;
    var token = $auth.getCredentialsSync();
    var media = $appstate.model.currentEvent.eventType.supportedMedia;
    $scope.native = $native.isNative();
    $scope.photoEnabled = ($scope.native || $scope.hasGetUserMedia) && (media == 'ALLOW_PHOTO' || media == 'ALLOW_ALL');
    $scope.videoEnabled = ($scope.native || $scope.hasGetUserMedia) && (media == 'ALLOW_VIDEO' || media == 'ALLOW_ALL');
    $scope.hasMedia = $appstate.model.currentEvent.images != null && $appstate.model.currentEvent.images.length > 0 || $appstate.model.currentEvent.videos != null && $appstate.model.currentEvent.videos.length > 0;
    $scope.setImage = function (element) {
      $scope.setFile(element, true);
    };
    $scope.setVideo = function (element) {
      $scope.setFile(element, false);
    };
    $scope.setFile = function (element, isImage) {
      $scope.$apply(function () {
        $scope.files = [];
        var uploadType = isImage ? 'PHOTO' : 'VIDEO';
        for (var i = 0; i < element.files.length; i++) {
          var file = element.files[i];
          var k = $utils.generateUUID();
          if (isImage) {
            if (file.size > 3000000) {
              $appstate.setModalMessage();
              $appstate.msg('Image is larger than 3mb and may not save properly.', true);
            }
            var reader = new FileReader();
            reader.onloadend = function () {
              var imgsrc = angular.copy(reader.result);
              if (!isImage)
                imgsrc = btoa(imgsrc);
              var opts = {
                  method: 'POST',
                  url: $krConfig.getMediaBase() + '/data/uploadMedia',
                  params: {
                    key: k,
                    type: uploadType,
                    token: token.token
                  },
                  data: angular.copy(imgsrc),
                  preserveDataFormat: true
                };
              opts.params.filename = file.name;
              var info = 'Uploaded File for ' + $appstate.model.currentEvent.eventType.name + ' Event: Added @ ' + $utils.getTimeString($utils.getTimePart(new Date()));
              $backend.cacheAjax(opts, 'uploadCB', info);
              if (isImage) {
                // DEV-1293: Adds the local flag to enable viewing immediately (without a facility ID embedded in URL):
                $appstate.model.currentEvent.images.push({
                  key: k,
                  isNew: true,
                  url: imgsrc,
                  local: true
                });
              } else {
                // DEV-1293: Adds the local flag to enable viewing immediately (without a facility ID embedded in URL):
                $appstate.model.currentEvent.videos.push({
                  key: k,
                  isNew: true,
                  url: '',
                  local: true
                });
              }
            };
            if (isImage)
              reader.readAsDataURL(file);  //            else reader.readAsBinaryString(file);
          } else {
            var formData = new FormData();
            formData.append('file', file);
            var hType = undefined;
            var opts = {
                method: 'POST',
                url: $krConfig.getMediaBase() + '/data/uploadMedia',
                params: {
                  key: k,
                  type: uploadType,
                  token: token.token
                },
                data: formData,
                contentType: hType,
                headers: { 'Content-Type': hType },
                transformRequest: angular.identity
              };
            var info = 'Uploaded File for ' + $appstate.model.currentEvent.eventType.name + ' Event: Added @ ' + $utils.getTimeString($utils.getTimePart(new Date()));
            $appstate.model.loading = true;
            $appstate.setModalMessage('Uploading...');
            $http(opts).success(function (data, status, headers, config) {
              $appstate.model.loading = false;
              $appstate.setModalMessage();
              $appstate.msg('Uploaded Succesfully', true);
            }).error(function (data, status, headers, config) {
              console.log('error:' + data);
              $appstate.model.loading = false;
              $appstate.setModalMessage();
              $appstate.msg('Error Uploading', true, 'warning');
            });
            //$backend.cacheAjax(opts, 'uploadCB', info);
            // DEV-1293: Adds the local flag to enable viewing immediately (without a facility ID embedded in URL):
            $appstate.model.currentEvent.videos.push({
              key: k,
              isNew: true,
              url: '',
              local: true
            });
          }
        }
      });
    };
    function getCreds(type) {
      return '/data/uploadMedia?type=' + type + '&token=' + token.token;
    }
    $scope.picture = function () {
      $scope.processing = true;
      $native.image(imgCallback, $facility.getImageParameters());
    };
    var imgCallback = function (err, files, urls) {
      if (err) {
        //alert(err);
        $scope.$apply(function () {
          $scope.processing = false;
        });
      } else {
        var total = files.length;
        var count = 0;
        var keys = [];
        _.each(files, function (file, idx) {
          var key = $utils.generateUUID();
          keys.push(key);
          var url = urls[count];
          $timeout(function () {
            // DEV-1293: Adds the local flag to enable viewing immediately (without a facility ID embedded in URL):
            $appstate.model.currentEvent.images.push({
              key: key,
              isNew: true,
              url: url,
              local: true
            });
            $scope.processing = false;
          }, 200);
          count++;
          if (count == total) {
            $native.uploadMulti(files, urls, function (err, data) {
              if (err) {
                //alert('Upload failed');
                console.log(err);
                $appstate.msg('Upload failed!', true, 'danger');
                _.each(files, function () {
                  $appstate.model.currentEvent.images.pop();
                });
              } else {
                //alert('startedUpload:'+url);
                $appstate.msg('Upload Started!', true);
              }
            }, getCreds('PHOTO'), null, null, keys);
          }
        });
      }
    };
    $scope.video = function () {
      $scope.processing = true;
      $native.video(function (err, file, url) {
        if (err) {
          //alert(err);
          $scope.$apply(function () {
            $scope.processing = false;
          });
        } else {
          var key = $utils.generateUUID();
          $scope.$apply(function () {
            // DEV-1293: Adds the local flag to enable viewing immediately (without a facility ID embedded in URL):
            $appstate.model.currentEvent.videos.push({
              key: key,
              isNew: true,
              url: url,
              local: true
            });
            $scope.processing = false;
          });
          $native.upload(file, url, function (err, data) {
            if (err) {
              console.log(err);
              $appstate.msg('Upload failed!', true, 'danger');
              $appstate.model.currentEvent.videos.pop();
            } else {
            }
          }, getCreds('VIDEO'), null, null, key);
        }
      }, { videoDuration: $facility.getVideoLength() });
    };
    function removeEle(arr, ele) {
      var idx = -1;
      for (var i = 0; i < arr.length; i++) {
        if (arr[i] == ele)
          idx = i;
      }
      if (idx != -1) {
        arr.splice(idx, 1);
      }
      return arr;
    }
    function removeMedia(arr, url) {
      var obj = null;
      arr.forEach(function (e) {
        if (e == url)
          obj = e;
      });
      if (obj != null)
        return removeEle(arr, obj);
      return arr;
    }
    $scope.removeVideo = function (img) {
      //$scope.$apply(function(){
      $appstate.model.currentEvent.videos = removeMedia($appstate.model.currentEvent.videos, img);  //});
    };
    $scope.removeImage = function (img) {
      //$scope.$apply(function(){
      $appstate.model.currentEvent.images = removeMedia($appstate.model.currentEvent.images, img);  //});
    };
    $scope.isCollapsed = false;
    $scope.tabShowing = 'notes';
    $scope.toggleTab = function (tab, skipScroll) {
      var old = $scope.tabShowing;
      if ($scope.isCollapsed || tab != old) {
        $scope.isCollapsed = false;
        $scope.tabShowing = tab;
      } else {
        //if ( tab == old ) {
        $scope.isCollapsed = true;
        $timeout(function () {
          $scope.tabShowing = null;
        }, 200);
      }
      $timeout(function () {
        var ele = angular.element(document.querySelector('.eventsContent'))[0];
        ele.scrollTop = ele.scrollHeight;
      }, 50);
    };
    //    $timeout(function() {
    //      $scope.toggleTab('notes', true)
    //    }, 250);
    $scope.reminderIntervals = $reminders.getIntervals();
    $scope.reminderValues = {
      inVal: $scope.reminderIntervals[7],
      atVal: { value: $utils.getTimePart(new Date(new Date().getTime() + 2000 * 60 * 60)) }
    };
    $scope.reminderTab = 'in';
    $scope.saveReminder = function () {
      var now = new Date().getTime();
      var str = $appstate.model.currentEvent.eventType.name + ' ' + $translate.instant('For') + ' ' + _.reduce($appstate.model.currentEvent.observed, function (m, v) {
          return m + v.firstName + ' ' + v.lastName + ', ';
        }, '');
      str = str.substring(0, str.length - 2);
      var time = 0;
      if ($scope.reminderTab === 'at') {
        var d = new Date($utils.createDateFromDTStrings($scope.values.date, $utils.getMilTime($scope.reminderValues.atVal.value)));
        time = d.getTime();
      } else {
        // inVal is in minutes, compute to millis and offset from now.
        time = new Date().getTime() + 1000 * 60 * $scope.reminderValues.inVal.v;
      }
      $reminders.addReminder(str, time, now);
      $appstate.msg($translate.instant('ReminderSaved'), true);
      $scope.toggleTab('reminders');
    };
    var isEdit = $appstate.model.currentEvent.observers.length > 0;
    $scope.isEdit = isEdit;
    $scope.observerLabel = $translate.instant(isParent ? 'Parent' : 'Teacher');
    // Get the right observer if coming from an existing event
    var useParents = isParent;
    $scope.readOnly = false;
    if (isEdit) {
      observer = _.filter($facility.getSync().users, function (observer) {
        return observer.id === $appstate.model.currentEvent.observers[0].id;
      })[0];
      if (!observer)
        observer = $appstate.model.currentEvent.observers[0];
      if (isParent && observer.role == 'TEACHER_ROLE') {
        $scope.observerLabel = 'Teacher';
        $scope.readOnly = true;
      } else if (!isParent && observer.role == 'PARENT_ROLE') {
        $scope.observerLabel = 'Parent';
        useParents = true;
        $scope.readOnly = true;
      }
    } else if (isParent) {
      var lang = locale.getLanguage();
      var isPT = lang && lang.indexOf && lang.indexOf('pt') > -1;
      // check if we have a saved template
      var template = $defaults.get($appstate.model.currentEvent);
      if (template && !isPT) {
        $appstate.model.currentEvent.eventValues = angular.copy(template.eventValues);
        $appstate.model.currentEvent.eventValues.forEach(function (ev) {
          if (ev.value == null || ev.value == '')
            return;
          if (ev.ett.templateType == 'TIME_INPUT') {
            ev.value = $utils.getMilTime(ev.value);
          } else if (ev.ett.templateType == 'NUMERIC_SELECT' || ev.ett.templateType == 'SINGLE_SELECT') {
            ev.value.selected = _.filter(ev.value.choices, function (c) {
              return c.name == ev.value.selected.name;
            })[0];
          }
        });
        $appstate.model.currentEvent.notes = angular.copy(template.notes);
      }
    }
    if ($facility.getSync().preventWriteActivity) {
      $scope.readOnly = true;
    }
    var observer;
    if (useParents) {
      var compare = $backend.getCurrentUser().id;
      if (isEdit) {
        var kid = $appstate.model.currentEvent.observed[0];
        if (kid.parents) {
          $scope.observers = kid.parents;
        } else {
          $scope.observers = $facility.findChild(kid.id).parents;
        }
        compare = $appstate.model.currentEvent.observers[0].id;
      } else {
        $scope.observers = $appstate.model.observed[0].parents;
      }
      observer = _.filter($scope.observers, function (obs) {
        return obs.id == compare;
      })[0];
    } else {
      var rm = $appstate.model.selectedRoom;
      if (rm) {
        $scope.observers = rm.teachers;
      }
      observer = $backend.getCurrentUser();
      // filter out inactive ones.
      $scope.observers = _.filter($scope.observers, function (a) {
        var u = $facility.findUser(a.id);
        return u && !u.deleted && !u.archived && u.enabled;
      });
      // add current user if not in the room.
      if (_.filter($scope.observers, function (obs, idx) {
          return obs.id == observer.id;
        }).length == 0) {
        $scope.observers.push(observer);
      }
    }
    var idx = -1;
    if (isEdit) {
      observer = $appstate.model.currentEvent.observers[0];
    }
    $scope.observers.forEach(function (obs, obsIdx) {
      if (obs.id == observer.id)
        idx = obsIdx;
      obs.fullName = obs.firstName + ' ' + obs.lastName;
    });
    observer = idx != -1 ? $scope.observers[idx] : observer;
    $scope.values = {
      notes: $appstate.model.currentEvent.notes,
      observer: $appstate.model.currentEvent.observer || observer,
      date: $utils.getDatePart($appstate.model.currentEvent.timestamp),
      time: { value: $utils.getTimePart($appstate.model.currentEvent.timestamp) },
      sigName: ''
    };
    $scope.observers.forEach(function (ob) {
      if (ob.id == $scope.values.observer.id) {
        $scope.values.observer = ob;
      }
    });
    //    if (isEdit) $scope.values.observer = $backend.findUser($scope.values.observer.id);
    $scope.editNotes = $appstate.model.currentEvent.eventType.allowNotes === true;
    $scope.items = [];
    $scope.bind = function (element) {
      function createElement(index) {
        var item = $scope.items[index];
        var v = $templateMgr.getTemplate(item.ett.id);
        if (v) {
          return v;
        } else {
          //item.continueControl = $scope.continueControl.execute;
          var template = '<div class="template-wrapper" <%= item.type %> item="items[<%= index %>]" ></div>';
          // TODO Optimize so these are compiled only once
          var compiled = _.template(template);
          //if ( $utils.isAndroid5() && item.type == 'time' ) item.type = 'datetime-local';
          var r = compiled({
              item: item,
              index: index
            });
          $templateMgr.setTemplate(item.ett.id, r);
          return r;
        }
      }
      // First clear out the element
      element.html('');
      for (var i = 0; i < $scope.items.length; i++) {
        var html = createElement(i);
        var el = $compile(html)($scope);
        element.append(el);
      }
    };
    function addEventDetails(event) {
      $scope.items = event.eventValues;
    }
    addEventDetails($appstate.model.currentEvent);
    $scope.isDraft = $appstate.model.currentEvent.draftId != null;
    $scope.deleteEvent = function () {
      // only allow drafts for now
      if ($scope.isDraft) {
        $krCache.getCache('drafts', function (c) {
          var cache = c && _.isArray(c) ? c : [];
          var item = $appstate.model.currentEvent;
          var idx = -1;
          var match = _.filter(c, function (i, count) {
              var b = false;
              if (item.draftId == i.draftId) {
                b = true;
                idx = count;
              }
              return b;
            })[0];
          if (match) {
            // if we found a match, return true if child IDs match and there is one of each.
            if (item.observed.length == 1 && match.observed.length == 1 && item.observed[0].id == match.observed[0].id) {
              cache = _.filter(cache, function (i) {
                return match.draftId != i.draftId;
              });
            } else {
              // Otherwise, if the draft has this child or children in it, remove it and update.
              match.observed = _.filter(match.observed, function (o) {
                return _.filter(item.observed, function (obs) {
                  return obs.id != o.id;
                }).length > 0;
              });
            }
            $krCache.setCache(cache, 'drafts', function () {
              $timeout(function () {
                $rootScope.$broadcast('eventSaved');
              }, 100);
            });
          }
        });
        $appstate.msg($translate.instant('DraftDeleted'), true);
        $appstate.nav.nextState = null;
        $appstate.nav.next();
      }
    };
    function isDefault(value, ett) {
      if (!ett.templateValues || ett.templateValues.length && ett.templateValues.length == 0)
        return true;
      var def = _.filter(ett.templateValues, function (tv) {
          return tv.isDefault;
        });
      if (!def || def.length == 0)
        def = ett.templateValues[0];
      else
        def = def[0];
      return value.selected.name == def.value;
    }
    function findInvalidInputs() {
      var s = null;
      $appstate.model.currentEvent.eventValues.forEach(function (ev) {
        if (ev.ett.templateType == 'NUMERIC_SELECT' && ev.value.amount == 0 && !isDefault(ev.value, ev.ett)) {
          if (s == null) {
            s = 'It looks like you\'re trying to save ' + ev.ett.name + ' information and didn\'t enter how much. Are you sure you want to continue?';
          }
        }
      });
      //    * It looks like you're trying to save bottle information and didn't enter how many ounces/ml. Are you sure you want to continue?
      //
      return s;
    }
    function getEventDate(propName) {
      var event = $appstate.model.currentEvent;
      var date = null;
      event.eventValues.forEach(function (ev) {
        if (ev.type == 'TIME_INPUT' && ev.name.toLowerCase().indexOf(propName) > -1) {
          var time = ev.value;
          date = new Date($utils.createDateFromDTStrings($scope.values.date, time));
        }
      });
      return date;
    }
    ;
    function validate(cb) {
      /**
       * initial validation:
       * 1.  Check if any single select w/number ETs were changed from the default but no value entered in numeric portion
       * 2.  Check if backdated or future dated and warn.
       *
       */
      var dt = new Date($utils.createDateFromDTStrings($scope.values.date, $scope.values.time.value));
      var now = new Date();
      var subj, msg;
      var startDate = getEventDate('start');
      var endDate = getEventDate('end');
      var isEndBeforeStart = new Date(startDate) > new Date(endDate);
      if (isEndBeforeStart) {
        subj = $translate.instant('EndBeforeStartSubject');
        msg = $translate.instant('EndBeforeStartMessage');
      } else if (dt.getDate() != now.getDate() || dt.getFullYear() != now.getFullYear() || dt.getMonth() != now.getMonth()) {
        var isBackdate = now.getTime() > dt.getTime();
        var backDateType = isBackdate ? $translate.instant('BackdateWarning') : $translate.instant('PostdateWarning');
        subj = backDateType;
        if (isBackdate) {
          msg = $translate.instant('BackdateMessage');
        } else {
          msg = $translate.instant('PostdateMessage');
        }
      } else if ($scope.signatureRequired && (!$appstate.model.signaturePad.isEmpty() || $scope.values.sigName && $scope.values.sigName != '')) {
        msg = 'You have not saved your signature.  Are you sure you want to continue?';
        subj = 'Unsaved Signature';
      } else {
        var invalidInputs = findInvalidInputs();
        if (invalidInputs) {
          subj = 'Missing Information!';
          msg = invalidInputs;
        }
      }
      if (subj && msg) {
        var messageBox;
        if (isEndBeforeStart) {
          messageBox = $messageBox.open(subj, msg, {}).result.then(function (selectedItem) {
            cb(false);
          });
        } else {
          messageBox = $messageBox.open(subj, msg, {
            oktext: $translate.instant('Yes'),
            canceltext: $translate.instant('No')
          }).result.then(function (selectedItem) {
            //$state.go("home.action.selectEvent");
            cb(true);
          }, function () {
            cb(false);
          });
        }
      } else {
        cb(true);
      }
    }
    function validateSigPad() {
      if ($appstate.model.signaturePad.isEmpty()) {
        $messageBox.open('Empty Signature', 'Please input your signature before saving!', { oktext: 'Ok' });
        return false;
      }
      if (!$scope.values.sigName || $scope.values.sigName == '') {
        $messageBox.open('Empty Name', 'Please print your name before saving!', { oktext: 'Ok' });
        return false;
      }
      return true;
    }
    function handleClick(isDraft) {
      if ($scope.readOnly) {
        $appstate.nav.nextState = $scope.onClick;
        return;
      }
      if ($scope.processing)
        return;
      // lock the save button to prevent multiple clicks.
      $scope.processing = true;
      $timeout(function () {
        validate(function (success) {
          if (success) {
            _save(isDraft);
          } else {
            $appstate.nav.nextState = $scope.onClick;
            $scope.processing = false;
          }
        });
      }, 10);
    }
    function _save(isDraft) {
      if (!$appstate.model.currentEvent.observed || $appstate.model.currentEvent.observed.length === 0 && $appstate.nav.getPath() != 'kidPlannerAdd') {
        $appstate.msg($translate.instant('NoChildrenSelected'), true, 'warning');
        return;
      }
      $appstate.nav.nextState = null;
      if ($appstate.updateHeight)
        $appstate.updateHeight(false);
      // update event values for android 5.0.2 bug
      _.each($appstate.model.currentEvent.eventValues, function (ev) {
        if (ev.value != null && ev.value.getTime && ev.type == 'TIME_INPUT') {
          ev.value = $utils.getTimePart(ev.value);
        }
      });
      // Add selected state standards:
      $appstate.model.currentEvent.stateStandards = angular.copy($scope.eventStandardSelections);
      $appstate.model.currentEvent.showAbsentKids = $appstate.model.showAbsentKids;
      var napping = false;
      if ($scope.isNapEvent() && ($scope.napTimerStarted || $appstate.model.currentEvent.properties.nowTime > $scope.startTime)) {
        // stop our timer
        $scope.napTimerStarted = false;
        napping = true;
        if (!isDraft)
          $appstate.model.currentEvent.properties.napTimerStarted = false;
      }
      $krCache.getCache('drafts', function (c) {
        var cache = c && _.isArray(c) ? c : [];
        var txt = isEdit ? 'Edit' : 'Add';
        $appstate.model.currentEvent.notes = $scope.values.notes;
        $appstate.model.currentEvent.timestamp = $utils.createGsonTSFromDTStrings($scope.values.date, $scope.values.time.value);
        $appstate.model.currentEvent.observers = [$scope.values.observer];
        if ($kidplanner.planInSession) {
          $kidplanner.extraRooms = $scope.selected_items;
          // todo pull out selected extra rooms, store them on the $kidplanner service
          $appstate.nav.nextState = $kidplanner.nextState;
          $appstate.nav.next();
          return;
        }
        $appstate.model.currentEvent.clientKey = $appstate.model.currentEvent.clientKey || $utils.generateUUID();
        if (isDraft) {
          let newDraftIdRequired = false;
          if ($appstate.model.currentEvent.draftId != null) {
            var retDrafts = [];
            _.forEach($appstate.model.eventListByChild, function (child) {
              _.forEach(child, function (draft) {
                if (!draft.draftId) {
                  return;
                }
                if (draft.draftId === $appstate.model.currentEvent.draftId) {
                  retDrafts.push(draft);
                }
              });
            });
            if (retDrafts.length > 1) {
              _.forEach(retDrafts, function (retDraft) {
                if (!$appstate.model.currentEvent.observed.includes(retDraft.child)) {
                  newDraftIdRequired = true;
                }
              });
            }
          }
          let oldDraftId = null;
          if (newDraftIdRequired) {
            oldDraftId = $appstate.model.currentEvent.draftId;
            $appstate.model.currentEvent.draftId = null;  //clearing draftId so that a new one can be generated, and not erase old drafts.
          }
          // generate a uuid for in mem lookups
          $appstate.model.currentEvent.draftId = $appstate.model.currentEvent.draftId || $utils.generateUUID();
          cache = _.filter(cache, function (i) {
            return i.draftId != $appstate.model.currentEvent.draftId;
          });
          // copy by hand
          var cleanDraft = {
              draftId: $appstate.model.currentEvent.draftId,
              clientKey: $appstate.model.currentEvent.clientKey,
              eventValues: angular.copy($appstate.model.currentEvent.eventValues),
              id: $appstate.model.currentEvent.id,
              images: angular.copy($appstate.model.currentEvent.images),
              videos: angular.copy($appstate.model.currentEvent.videos),
              name: $appstate.model.currentEvent.name,
              notes: $appstate.model.currentEvent.notes,
              properties: angular.copy($appstate.model.currentEvent.properties),
              timestamp: $appstate.model.currentEvent.timestamp,
              stateStandards: $appstate.model.currentEvent.stateStandards,
              showAbsentKids: $appstate.model.currentEvent.showAbsentKids
            };
          cleanDraft.eventType = { id: $appstate.model.currentEvent.eventType.id };
          cleanDraft.observed = [];
          $appstate.model.currentEvent.observed.forEach(function (obs) {
            cleanDraft.observed.push({ id: obs.id });
          });
          cleanDraft.observers = [];
          $appstate.model.currentEvent.observers.forEach(function (obs) {
            cleanDraft.observers.push({ id: obs.id });
          });
          //updating draft by removing children who should now have their own draft.
          if (oldDraftId != null) {
            $krCache.getCache('drafts', function (c) {
              var cache = c && _.isArray(c) ? c : [];
              var match = _.filter(c, function (i, count) {
                  var b = false;
                  if (oldDraftId === i.draftId) {
                    b = true;
                  }
                  return b;
                })[0];
              if (match) {
                var temp = [];
                match.observed.forEach(function (matchObs) {
                  let toAdd = true;
                  // assume true and should be added
                  cleanDraft.observed.forEach(function (cleanDraftObs) {
                    if (matchObs.id === cleanDraftObs.id) {
                      toAdd = false;
                    }
                  });
                  if (toAdd) {
                    temp.push(matchObs);
                  }
                });
                match.observed = temp;
                $krCache.setCache(cache, 'drafts', function () {
                  $timeout(function () {
                    $rootScope.$broadcast('eventSaved');
                  }, 100);
                });
              }
            });
          }
          cache.push(cleanDraft);
          $krCache.setCache(cache, 'drafts');
          $appstate.model.currentEvent.clientKey = $appstate.model.currentEvent.draftId;
          $events.saveDraft($appstate.model.currentEvent);
          $timeout(function () {
            $rootScope.$broadcast('eventSaved');
          }, 100);
          if ($state.includes('home.dash')) {
            $appstate.msg($translate.instant('DraftSaved'), true);
            $appstate.nav.next();
          } else {
            var msg = $translate.instant('DraftSavedMsg').replace('&type&', $translate.instant(txt));
            $messageBox.open($translate.instant('DraftSaved'), msg, {
              oktext: $translate.instant('Yes'),
              canceltext: $translate.instant('No')
            }).result.then(function (selectedItem) {
              //$state.go("home.action.selectEvent");
              $appstate.nav.back();
            }, function () {
              $appstate.msg($translate.instant('DraftSaved'), true);
              $appstate.nav.next();
            });
          }
        } else {
          // -------------------- Save event ----------------------------
          // only update nap timer if not editing a saved event.
          if (napping && $appstate.model.currentEvent.id == 0 && !$appstate.model.currentEvent.properties.stoppedManually) {
            $adapter.updateNapEndTime($appstate.model.currentEvent);
            $appstate.model.currentEvent.properties.nowTime = 0;
            delete $appstate.model.currentEvent.properties.startTime;
          }
          var draftId = $appstate.model.currentEvent.draftId;
          $appstate.model.currentEvent.draftId = null;
          if (!$appstate.model.currentEvent.properties)
            $appstate.model.currentEvent.properties = {};
          $appstate.model.currentEvent.properties.oldDraftId = draftId;
          $events.saveEvent($appstate.model.currentEvent, false, $model.closeOnEventComplete);
          // Check if parent and save as template if so.
          if (isParent) {
            var template = {};
            template.eventValues = angular.copy($appstate.model.currentEvent.eventValues);
            template.eventType = { id: $appstate.model.currentEvent.eventType.id };
            template.observed = [{ id: $appstate.model.currentEvent.observed[0].id }];
            template.notes = angular.copy($appstate.model.currentEvent.notes);
            $defaults.save(template);
          }
          // check for draft id and remove
          if (draftId) {
            var draft = _.filter(cache, function (i) {
                return i.draftId == draftId;
              })[0];
            // see if this draft was originally a multi select but we are only editing one kid for now
            if (draft) {
              if (draft.observed.length > $appstate.model.currentEvent.observed.length) {
                draft.observed = _.filter(draft.observed, function (o) {
                  var keep = _.filter($appstate.model.currentEvent.observed, function (curr) {
                      return curr.id == o.id;
                    }).length == 0;
                  return keep;
                });
              } else {
                cache = _.filter(cache, function (i) {
                  return i.draftId != draftId;
                });
              }
            }
            $krCache.setCache(cache, 'drafts', function () {
              // broadcast after drafts updated
              $timeout(function () {
                $rootScope.$broadcast('eventSaved');
              }, 750);
            });
          } else {
            // broadcast now
            $timeout(function () {
              $rootScope.$broadcast('eventSaved');
            }, 750);
          }
          var title = $translate.instant('EventSaved');
          if ($appstate.nav.getPath() === 'viewEventDirect') {
            if ($model.closeOnEventComplete) {
              $rootScope.$on('remoteEventSaveComplete', function () {
                window.top.postMessage('closeKRMobile', '*');
              });
              $appstate.model.loading = true;
              return;
            }
            $appstate.msg(title, true);
            $appstate.nav.next();
          } else {
            var msg = 'Your event was saved. Would you like to ' + (isEdit ? 'edit' : 'add') + ' another?';
            if ($state.includes('home.dash')) {
              $appstate.msg(title, true);
              $appstate.nav.next();
            } else {
              $messageBox.open(title, msg, {
                oktext: $translate.instant('Yes'),
                canceltext: $translate.instant('No')
              }).result.then(function (selectedItem) {
                //$state.go("home.action.selectEvent");
                $appstate.nav.back();
              }, function () {
                $appstate.msg(title, true);
                $appstate.nav.next();
              });
            }
          }
        }  // -------------------- END OF SAVE EVENT ----------------------------
      });
    }
    ;
    $scope.onClick = handleClick;
    $scope.preview = function () {
      var modalInstance = $modal.open({
          templateUrl: 'views/popups/previewPopup.html',
          controller: 'PreviewpopupCtrl',
          resolve: {
            kids: function () {
              return $appstate.model.observed;
            },
            name: function () {
              return $appstate.model.currentEvent.eventType.name;
            }
          }
        });
      modalInstance.result.then(function (d) {
      }, function () {
      });
    };
    function isDirty() {
      if (dirtyState.dirtyevent === cleanState.cleanevent) {
        if (dirtyState.dirtytime == cleanState.cleantime) {
          if (dirtyState.dirtyobserver == cleanState.cleanobserver) {
            if (dirtyState.dirtynotes == cleanState.cleannotes) {
              return false;
            } else {
              return true;
            }
          } else {
            return true;
          }
        } else {
          return true;
        }
      } else {
        return true;
      }
      ;  //      if ( isEdit ) {
         //        return true;
         //      } else {
         //        // dirty if they have added a photo or video
         //        return ($appstate.model.currentEvent.images && $appstate.model.currentEvent.images.length &&
         //          $appstate.model.currentEvent.images.length > 0) || ($appstate.model.currentEvent.videos &&
         //          $appstate.model.currentEvent.videos.length && $appstate.model.currentEvent.videos.length > 0);
         //      }
    }
    //KR-965
    function _close() {
      dirtyState = {
        dirtyevent: JSON.stringify($appstate.model.currentEvent),
        dirtytime: angular.copy($scope.values.time.value),
        dirtyobserver: angular.copy($scope.values.observer.fullName),
        dirtynotes: angular.copy($scope.values.notes)
      };
      if (isDirty()) {
        $messageBox.open($translate.instant('UnsavedEvent'), $translate.instant('EventNotSavedDiscard'), {
          canceltext: $translate.instant('YesDiscardChanges'),
          oktext: $translate.instant('Cancel')
        }).result.then(function (selectedItem) {
          $timeout(function () {
            $appstate.canClose = _close;
          }, 50);
        }, function () {
          $appstate.canClose = null;
          $appstate.goHome();
        });
      } else {
        $appstate.goHome();
      }
    }
    $appstate.canClose = _close;
    $scope.close = _close;
    $appstate.nav.nextState = $scope.onClick;
    //  $timeout(function() {
    //    var myEl = angular.element( document.querySelector( '#eventNotesTextArea' ) )[0];
    //    myEl.focus();
    //  }, 100);
    $scope.isPhotoVideo = false;
    var waittime = $scope.native ? 300 : 300;
    if ($state.is('home.dash.pictureevent')) {
      $scope.isPhotoVideo = true;
      setTimeout(function () {
        $scope.$apply(function () {
          if ($scope.native)
            $scope.picture();
          else {
            $('#fileimg')[0].click();
          }
        });
      }, waittime);
    } else if ($state.is('home.dash.videoevent')) {
      $scope.isPhotoVideo = true;
      //$timeout($scope.video(), 300);
      setTimeout(function () {
        $scope.$apply(function () {
          if ($scope.native)
            $scope.video();
          else {
            $('#filevid')[0].click();
          }
        });
      }, waittime);
    }
    //$rootScope.$on("windowResize", sigCanvasResize);
    function sigCanvasResize() {
      //if ( $native.isNative() ) return;
      var canvas = angular.element(document.querySelector('#canvas'))[0];
      var ratio = 1;
      // window.devicePixelRatio || 1;
      canvas.width = window.innerWidth * 0.8;
      //canvas.offsetWidth * ratio;
      canvas.height = canvas.width / 3;
      //canvas.offsetHeight * ratio;
      canvas.getContext('2d').scale(ratio, ratio);
    }
    //    if ( $scope.signatureRequired ) {
    //      $timeout(sigCanvasResize, 250);
    //    }
    /// Sig canvas stuff
    $scope.clearSignature = function () {
      $appstate.model.signaturePad.clear();
      $scope.values.sigName = '';
    };
    $scope.values.sigName = '';
    $scope.cancelSignature = function () {
      $scope.clearSignature();
      $scope.toggleSignaturePad(false);
    };
    $scope.saveSignature = function () {
      if (!validateSigPad())
        return;
      var canvas = angular.element(document.querySelector('#canvas'))[0];
      var context = canvas.getContext('2d');
      context.fillStyle = 'darkred';
      context.font = '12px Arial';
      context.fillText('Signed By: ' + $scope.values.sigName, 10, 40);
      context.fillText('Date: ' + new Date(), 10, 20);
      var imgsrc = angular.copy($appstate.model.signaturePad.toDataURL());
      var k = $utils.generateUUID();
      var opts = {
          method: 'POST',
          url: $krConfig.getMediaBase() + '/data/uploadMedia',
          params: {
            key: k,
            type: 'PHOTO',
            token: token.token,
            sig: 'true'
          },
          data: angular.copy(imgsrc),
          preserveDataFormat: true
        };
      // $appstate.model.currentEvent is current event that you're working with
      var info = 'Signature for ' + $appstate.model.currentEvent.eventType.name + ' Event: Added @ ' + $utils.getTimeString($utils.getTimePart(new Date()));
      $backend.cacheAjax(opts, 'uploadCB', info);
      // DEV-1293: Adds the local flag to enable viewing immediately (without a facility ID embedded in URL):
      $appstate.model.currentEvent.images.push({
        key: k,
        isNew: true,
        url: imgsrc,
        local: true
      });
      $scope.clearSignature();
      $scope.toggleSignaturePad(false);
    };
    $scope.showSigPad = false;
    $scope.toggleSignaturePad = function (b) {
      if (b)
        sigCanvasResize();
      $scope.showSigPad = b;
    };
    var cleanState;
    var dirtyState;
    // A bit of a hack - we need a way to intercept the actual dom attach event
    //if ($native.type() == "ANDROID") {
    $timeout(function () {
      //Create Clean
      ///
      cleanState = {
        cleanevent: JSON.stringify($appstate.model.currentEvent),
        cleantime: angular.copy($scope.values.time.value),
        cleanobserver: angular.copy($scope.values.observer.fullName),
        cleannotes: angular.copy($scope.values.notes)
      };
      if (!$utils.isAndroid5())
        $native.enhanceUI();
    }, 500);
    $rootScope.$broadcast('EventDetailsRendered');
  }
]);
'use strict';
angular.module('kidreports2App').controller('PreviewpopupCtrl', [
  '$scope',
  '$appstate',
  '$modalInstance',
  '$adapter',
  '$backend',
  'kids',
  'name',
  '$facility',
  '$events',
  function ($scope, $appstate, $modalInstance, $adapter, $backend, kids, name, $facility, $events) {
    var k2 = kids;
    //angular.copy(kids);
    k2.forEach(function (kid) {
      if (!kid.events) {
        $events.getEvents([kid], null, function (err, data) {
          if (!err) {
            kid.events = data.events;
            process(kid);
          }
        });
      } else
        process(kid);
    });
    function process(kid) {
      kid.events.forEach(function (event) {
        $adapter.processEvent(event);
        if (!event.eventType.name)
          event.eventType.name = $facility.findEventType(event.eventType.id).name;
        if (!event.user.lastName)
          event.user.lastName = $facility.findUser(event.user.id).lastName;
      });
    }
    $scope.vals = {
      kids: k2,
      name: name,
      curr: 1
    };
    $scope.ok = function () {
      //console.log($scope.vals.date);
      $modalInstance.close($scope.vals);
    };
    $scope.cancel = function () {
      $modalInstance.dismiss('cancel');
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('DaterangeCtrl', [
  '$scope',
  '$appstate',
  '$modalInstance',
  'startdate',
  'enddate',
  function ($scope, $appstate, $modalInstance, startdate, enddate) {
    $scope.vals = {
      startdate: startdate,
      enddate: enddate,
      date: startdate
    };
    $scope.isDateRangeMode = false;
    $scope.setMode = function (b) {
      $scope.isDateRangeMode = b;
    };
    $scope.ok = function () {
      //console.log($scope.vals.date);
      if (!$scope.isDateRangeMode) {
        $scope.vals.startdate = $scope.vals.date;
        $scope.vals.enddate = $scope.vals.date;
      }
      $modalInstance.close($scope.vals);
    };
    $scope.cancel = function () {
      $modalInstance.dismiss('cancel');
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('NavCtrl', [
  '$scope',
  '$appstate',
  '$location',
  '$backend',
  '$state',
  '$facility',
  '$localdb',
  '$msg',
  '$messages',
  '$rootScope',
  '$prefs',
  '$popup',
  '$timeout',
  '$adapter',
  '$native',
  'translations',
  '$translate',
  'popover',
  '$quicklinks',
  '$utils',
  '$policies',
  '$appService',
  '$model',
  function ($scope, $appstate, $location, $backend, $state, $facility, $localdb, $msg, $messages, $rootScope, $prefs, $popup, $timeout, $adapter, $native, translations, $translate, popover, $quicklinks, $utils, $policies, $appService, $model) {
    $scope.lastEventSyncTime = $model.lastSynchTime;
    //    $ionicPopover.fromTemplateUrl('views/popups/childPopover.html', {
    //      scope: $scope,
    //    }).then(function (popover) {
    //      $scope.popover = popover;
    //
    // DEV-1000: Redirect to `parent-home` if role is parent:
    if ($appService.getUserRole() === 'PARENT_ROLE') {
      $state.go('parent-home');
    }
    var setHeaderView = function () {
      $scope.hideHeaderUIView = $state.includes('home.action.kidPlanner');
    };
    setHeaderView();
    var fStatechange = $rootScope.$on('$stateChangeSuccess', setHeaderView);
    $scope.$on('$destroy', fStatechange);
    $scope.model = $appstate.model;
    $scope.isParent = $backend.getCurrentUser().role == 'PARENT_ROLE';
    $scope.vid = { url: '' };
    $scope.showProfile = function () {
      //alert('show child profile');
      if ($appstate.backdropClick)
        $appstate.backdropClick();
      $popup.showProfile($appstate.model.observed[$scope.model.currChild]);  //$scope.popover.hide();
    };
    $scope.setVidUrl = function (url) {
      //      $appstate.vidurl = url;
      //      $scope.vid.url = url;
      //event.preventDefault();
      //event.cancelBubble = true;
      if ($native.isNative()) {
        forge.media.videoPlay(url, function () {
        }, function (err) {
        });
      } else {
      }
    };
    function backState() {
      if ($state.is('home.action.selectEvent') && !$appstate.model.parentDropoff || $state.is('home.action.eventDetails') && $appstate.model.parentDropoff) {
        $appstate.goHome();
      } else {
        $appstate.nav.back();
        $appstate.nav.backState = backState;
      }
    }
    $scope.getBGStyle = function () {
      return { 'background-image': 'url("' + $appstate.model.observed[$scope.model.currChild].photoUrl + '")' };
    };
    $scope.recordOther = function () {
      if ($appstate.backdropClick)
        $appstate.backdropClick();
      $appstate.model.observed = [$appstate.model.observed[$scope.model.currChild]];
      $appstate.model.observed[0].singleSelected = true;
      $appstate.model.parentDropoff = false;
      $appstate.nav.startPath('parentChildEvent', true);
      $appstate.nav.nextState = null;
      $appstate.nav.backState = backState;
      $appstate.nav.next();
    };
    $appstate.model.isSettingChild = false;
    $scope.setIndex = function (idx, evt) {
      if (idx === $scope.model.currChild)
        return;
      else {
        $appstate.model.isSettingChild = true;
        //var tmp = $appstate.eventListByDate;
        //$appstate.model.eventListByDate = [];
        $scope.model.currChild = idx;
        evt.cancelBubble = true;
        evt.stopPropagation();
        //$ionicScrollDelegate.$getByHandle('activityFeedScroll').scrollTop();
        $timeout(function () {
          //$appstate.model.eventListByDate = tmp;
          $appstate.model.isSettingChild = false;
        }, 50);
      }
    };
    $scope.isChildSelected = function (idx) {
      return idx === $scope.model.currChild ? 'childSelected' : 'childNotSelected';
    };
    var hasArrivalInfo = _.filter($facility.getSync().eventTypes, function (item) {
        return item.name.indexOf('Arrival Info') !== -1;
      }).length > 0;
    var lookup = 'Drop-Off Information';
    if (hasArrivalInfo) {
      $scope.dropOffText = 'Record Arrival Info';
      lookup = 'Arrival Info';
    } else {
      var hasArrivalInformation = _.filter($facility.getSync().eventTypes, function (item) {
          return item.name.indexOf('Arrival Information') !== -1;
        }).length > 0;
      if (hasArrivalInformation) {
        $scope.dropOffText = 'Record Arrival Info';
        lookup = 'Arrival Information';
      } else if (translations.getCurrent().value == 'pt-br') {
        $scope.dropOffText = 'Aviso para a escola';
        lookup = 'Aviso para a escola';
      } else {
        $scope.dropOffText = 'RecordDropOff';
      }
    }
    $scope.showParentRecordEvent = function () {
      return !($facility.hideParentRecordEvent() && $scope.isParent);
    };
    $scope.recordDropOff = function () {
      if ($appstate.backdropClick)
        $appstate.backdropClick();
      $appstate.model.observed = [$appstate.model.observed[$scope.model.currChild]];
      $appstate.model.parentDropoff = true;
      $appstate.model.observed[0].singleSelected = true;
      $appstate.nav.startPath('parentChildEvent', true);
      $appstate.nav.nextState = null;
      $appstate.nav.backState = backState;
      var cat = $appstate.model.observed[0].category;
      var item = _.filter($facility.getSync().eventTypes, function (item) {
          return item.name.indexOf(lookup) !== -1 && item.categories[0].id == cat.id;
        })[0];
      if (item) {
        $appstate.model.currentEvent = $adapter.createNewEvent(item);
        $appstate.model.currentEvent.observed = $appstate.model.observed;
        $appstate.nav.currentIndex = 1;
        $appstate.nav.next();
      }
    };
    $scope.model.currChild = 0;
    $scope.updateThumb = function (cnt) {
      $scope.model.currChild = $scope.model.currChild + cnt;
      if ($scope.model.currChild < 0)
        $scope.model.currChild = $appstate.model.observed.length - 1;
      if ($scope.model.currChild >= $appstate.model.observed.length)
        $scope.model.currChild = 0;
    };
    $scope.slideHasChanged = function (index) {
      $scope.model.currChild = index;
    };
    $scope.filterParentKid = $appstate.filters.filterParentKid;
    $scope.backDisabled = true;
    $scope.forwardDisabled = true;
    $scope.message = '';
    $scope.hideMessage = true;
    var fac = $facility.getSync();
    $scope.menuItems = fac.menuItems;
    var count = 1;
    $scope.menuItems.forEach(function (i) {
      i.id = count++;
    });
    $scope.sideitems = [];
    $scope.mainitems = [];
    $scope.dashitems = [];
    var usr = $backend.getCurrentUser();
    var role = usr.role;
    function capitalize(s) {
      return s.charAt(0).toUpperCase() + s.slice(1);
    }
    var helpIDX = 0;
    var hasQuicklinks = false, hasPolicies = false;
    $scope.menuItems.forEach(function (item) {
      item.iconUrl = 'img/icons/' + item.icon;
      if ($facility.isLite()) {
        // filter out tracking charts
        if (item.sref && (item.sref.indexOf('trackingChart') > -1 || item.sref.indexOf('trackingSheet') > -1)) {
          return;
        } else if (item.path.root == 'kidPlanner') {
          return;
        }
      }
      item.token = capitalize(item.path.root);
      if ((item.main === undefined || item.main == true) && item.allowedRoles.contains(role) && !usr.restrictedActions.contains(item.path.root))
        $scope.mainitems.push(item);
      if ((item.side === undefined || item.side == true) && item.allowedRoles.contains(role) && !usr.restrictedActions.contains(item.path.root))
        $scope.sideitems.push(item);
      if (item.dash !== undefined && item.dash == true && item.allowedRoles.contains(role) && !usr.restrictedActions.contains(item.path.root))
        $scope.dashitems.push(item);
      if (item.path.root == 'help')
        helpIDX = $scope.sideitems.length - 1;
      if (item.path.root == 'quicklinks')
        hasQuicklinks = true;
      if (item.path.root == 'policies')
        hasPolicies = true;
    });
    if (!$appstate.model.menuItems)
      $appstate.model.menuItems = $scope.menuItems;
    $appstate.model.sideItems = $scope.sideitems;
    if (!hasQuicklinks && fac.quicklinks && fac.quicklinks.length && !$facility.isLite()) {
      var _quicklink = $quicklinks.getMenuItem();
      _quicklink.iconUrl = 'img/icons/' + _quicklink.icon;
      _quicklink.token = _quicklink.name;
      //capitalize(_quicklink.path.root);
      $scope.sideitems.splice(helpIDX, 0, _quicklink);
      $appstate.model.menuItems.splice(helpIDX, 0, _quicklink);
    }
    if (!hasPolicies && fac.enablePolicyModule && fac.policies && fac.policies.length && !$facility.isLite()) {
      var _policy = $policies.getMenuItem();
      _policy.iconUrl = 'img/icons/' + _policy.icon;
      _policy.token = capitalize(_policy.path.root);
      $scope.sideitems.splice(helpIDX, 0, _policy);
      $appstate.model.menuItems.splice(helpIDX, 0, _policy);
    }
    var cls = 'height33';
    var items = $scope.mainitems.length;
    if (items < 5)
      cls = 'height50';
    else if (items > 6 && items < 9)
      cls = 'height25';
    else if (items > 8 && items < 11)
      cls = 'height20';
    // else we need to rethink the main menu beyond 10 main buttons.
    if (items % 2 != 0) {
      var optItem = _.filter($scope.menuItems, function (item) {
          return item.optional || false;
        })[0];
      if (optItem && optItem.allowedRoles.contains(role) && !usr.restrictedActions.contains(optItem.path.root)) {
        $scope.mainitems.push(optItem);
      }
    }
    if (typeof forge !== 'undefined' && forge.layout) {
      forge.layout.getSafeAreaInsets(function (safeAreaInsets) {
        $scope.mainitems.forEach(function (item) {
          // If the top offset is greater than 20 (tested for iPhone 8), then
          // we need to reduce the margin slightly to fit the screen/notch.
          item.heightCls = safeAreaInsets.top > 20 ? 'height29' : 'height33';
        });
      });
    }
    // todo move these into the controller, and have each controller register itself with a service so the lookup is abstracted away.
    $scope.navForward = function () {
      return $state.is('home.action.selectChildren') || $state.is('home.action.attendance') || $state.is('home.action.settings') || $state.is('home.action.eventDetails') || $state.is('home.action.teacherSignIn') || $state.is('home.action.nameToFace');
    };
    $scope.expandEnabled = function () {
      return $state.is('home.action.selectChildren') || $state.is('home.action.selectChild');
    };
    $scope.multiSelectEnabled = function () {
      return $state.is('home.action.selectChildren');
    };
    $scope.goHome = function () {
      $appstate.goHome(null, true);
    };
    $scope.settings = function () {
      $appstate.nav.startPath('settings');
    };
    $scope.back = function () {
      $appstate.nav.back();
    };
    $scope.next = function () {
      $appstate.nav.next();
    };
    $scope.backDisabled = function () {
      // DEV-1025: If in main KidPlanner window, enable back button:
      if ($location.path() === '/action/kidPlanner/main') {
        return false;
      } else {
        return !$appstate.nav.current;
      }
    };
    $appstate.model.dashitems = $scope.dashitems;
    var u = $backend.getCurrentUser();
    $scope.name = u.firstName + ' ' + u.lastName;
    $localdb.get('dashboard').then(function (dashboard) {
      var db = dashboard != null && dashboard != 'null' && dashboard != 'false' && dashboard != false;
      $appstate.model._dashboard = db;
    });
    $scope.checkItem = function (item) {
      var p = $scope.showParentRecordEvent();
      if (!p && (item.path.root.toLowerCase().indexOf('childevent') > -1 || item.path.root.toLowerCase().indexOf('viewevent') > -1)) {
        return false;
      }
      var dashboard = $appstate.model._dashboard;
      if ((dashboard || $state.includes('home.dash')) && !item.sideWhenDash)
        return false;
      return true;
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('RootCtrl', [
  '$scope',
  '$appstate',
  '$rootScope',
  '$messageBox',
  '$location',
  '$utils',
  '$backend',
  '$krConfig',
  '$native',
  '$facility',
  '$modal',
  '$prefs',
  '$timeout',
  '$online',
  '$krCache',
  '$state',
  '$localdb',
  '$facebook',
  '$attendance',
  '$reminders',
  '$events',
  'translations',
  '$translate',
  '$image',
  '$mdSidenav',
  '$refreshService',
  function ($scope, $appstate, $rootScope, $messageBox, $location, $utils, $backend, $krConfig, $native, $facility, $modal, $prefs, $timeout, $online, $krCache, $state, $localdb, $facebook, $attendance, $reminders, $events, translations, $translate, $image, $mdSidenav, $refreshService) {
    //    $rootScope.$on('$routeChangeError', function ($event, current) {
    //      console.log("$routeChangeError")
    //      $appstate.nav.dialog();
    //      $location.path('/login');
    //    });
    $scope.currentLanguage = translations.getCurrent();
    $scope.toggleLanguage = function () {
      var idx = -1;
      _.each(translations.getLanguages(), function (l, index) {
        if (l.value === $scope.currentLanguage.value)
          idx = index;
      });
      idx++;
      if (idx >= translations.getLanguages().length)
        idx = 0;
      translations.setLanguage(translations.getLanguages()[idx]);
      $scope.currentLanguage = translations.getCurrent();
    };
    $scope.online = $online.state();
    var onlineWatch = $rootScope.$on('onlineStateChange', function (evt, isOnline) {
        //console.log(evt + " online:" + isOnline);
        $scope.$apply(function () {
          $scope.online = isOnline;
        });
      });
    $scope.shared = $appstate;
    $scope.attendanceRefresh = function () {
      if ($scope.cacheEmpty()) {
        $attendance.refreshAll().then(function () {
          $state.reload();
        }, function (err) {
          console.log(err);  // Error: "It broke"
        });
      } else {
        $messageBox.open('Offline Events', 'Not all events have uploaded to the servers. Please try again when complete', { oktext: 'Ok' });
        $events.push();
        $backend.saveEventInternal();
        $native.uploadInternal();
      }
    };
    var attendanceWatch = $rootScope.$on('attendanceChange', function (evt) {
        $scope.$apply(function () {
          $facility.getSync().observed.forEach(function (o, idx) {
            var count = -1;
            $appstate.model.observed.forEach(function (picked, pickedIdx) {
              if (picked.id == o.id && o.attendanceSet && o.isIn != true)
                count = pickedIdx;
            });
            if (count != -1) {
              $appstate.model.observed.splice(count, 1);
            }
          });
        });
      });
    $scope.goHome = function () {
      $appstate.goHome();
    };
    /**
     * Triggers a configuration refresh (clears cache), showing confirmation if pending outbound items
     * @returns {void}
     */
    $scope.refreshFacility = function () {
      var items = $refreshService.checkAllStorage();
      if (items && items.length) {
        // If items in cache, show confirmation:
        var subject = $refreshService.getConfirmationSubject();
        var message = $refreshService.getConfirmationMessage();
        $messageBox.open(subject, message, {
          oktext: $translate.instant('Yes'),
          canceltext: $translate.instant('No')
        }).result.then($appstate.refreshFacility);
      } else {
        // If no items in cache, perform refresh:
        $appstate.refreshFacility();
      }
    };
    // check if any of the given events match the selection
    $scope.hasActivities = function (evtList) {
      var show = false;
      evtList.forEach(function (evt) {
        show = show || $scope.filterTemplateEvents(evt) && $scope.filterSelectedKids(evt) && $appstate.checkEventFilter(evt);
      });
      return show;
    };
    var queueWatch = $rootScope.$on('eventQueued', function (evt, queuedEvt) {
        if (!$appstate.model.eventList)
          $appstate.model.eventList = [];
        if (!queuedEvt)
          return;
        // if we have a match then remove the item.
        var remove = null;
        var draftId = queuedEvt.draftId || queuedEvt.properties.oldDraftId;
        $appstate.model.eventList.forEach(function (e, idx) {
          if (e.clientKey == queuedEvt.clientKey || e.id == queuedEvt.id && queuedEvt.id != 0 || draftId && e.draftId == draftId && e.child.id == queuedEvt.child.id) {
            remove = idx;
          }
        });
        if (remove) {
          $appstate.model.eventList.splice(remove, 1);
        }
        $appstate.model.eventList.push(queuedEvt);
        $appstate.eventAdapter.rebuildLists();
      });
    $scope.isDash = function () {
      return $state.includes('home.dash');
    };
    $scope.version = $krConfig.version;
    $scope.reloadVersion = $krConfig.reloadVersion;
    function cacheChange(evt, cacheName) {
      if (!$appstate)
        $appstate = {};
      $krCache.getCache(null, function (c) {
        var items = c && _.isArray(c) ? c : [];
        setTimeout(function () {
          $rootScope.$apply(function () {
            $appstate.model.cachedItems = items;
          });
        }, 50);
      });
      $krCache.getCache('uploads', function (c) {
        var media = c && _.isArray(c) ? c : [];
        setTimeout(function () {
          $rootScope.$apply(function () {
            $appstate.model.cachedMedia = media;
          });
        }, 50);
      });
      $krCache.getCache('notifications', function (c) {
        setTimeout(function () {
          $rootScope.$apply(function () {
            $appstate.model.notices = c && _.isArray(c) ? c : [];
          });
        }, 50);
      });
    }
    cacheChange(null, null);
    $scope.offlineMessage = function () {
      if (!$scope.online && $scope.cacheEmpty()) {
        return $translate.instant('OfflineModeEnabled');  //} else if ($events.bulkUpdatesEnabled()) {
                                                          //  return $translate.instant("EventsSending");
      } else {
        return $translate.instant('EventsWaiting').replace('&events&', $appstate.cacheSize());
      }
    };
    var cacheWatch = $rootScope.$on('cacheChange', cacheChange);
    var reloadWatch = $rootScope.$on('reload.message', function (evt, msg) {
        $messageBox.open('Update Available!', msg, { oktext: 'Ok' }).result.then(function (selectedItem) {
          $native.restart();
        }, function () {
          $native.restart();
        });
        $appstate.msg('KidReports App Updated!', true);
      });
    $scope.$on('$destroy', function () {
      attendanceWatch();
      onlineWatch();
      queueWatch();
      cacheWatch();
      reloadWatch();
    });
    //$scope.title = "KidReports 2.0";
    $scope.toggleQuickView = function () {
      $prefs.current.showQuickView = !$prefs.current.showQuickView;
    };
    $scope.getIconUrl = function (evt) {
      var et = evt.eventType;
      return $utils.createPhotoUrl($krConfig.s3Base, et.icon, $utils.getDefault(et));
    };
    $appstate.getIconUrl = $scope.getIconUrl;
    $scope.prefs = $prefs.current;
    //$scope.matchName = "";
    $scope.hasAnyEvents = function (v, idx, d) {
      if (typeof idx === 'number') {
        var list = _.filter(_.filter(_.filter(v || $appstate.model.eventList, $scope.filterTemplateEvents), $scope.filterSelectedKids), $appstate.checkEventFilter);
        list = _.filter(list, function (evt) {
          return $appstate.model.observed && $appstate.model.observed[idx].id == evt.child.id && (d && evt.dateTimeOccurred.toDateString() == d.toDateString());
        });
        return list.length > 0;
      } else {
        return _.filter(_.filter(_.filter(v || $appstate.model.eventList, $scope.filterTemplateEvents), $scope.filterSelectedKids), $appstate.checkEventFilter).length > 0;
      }
    };
    $scope.filterTemplateEvents = $appstate.filters.filterTemplateEvents;
    $scope.filterSelectedKids = $appstate.filters.filterSelectedKids;
    $scope.toggleSelectAll = function () {
      if ($appstate.toggle) {
        $appstate.model.selectAll = !$appstate.model.selectAll;
        $appstate.toggle.call(this, $appstate.model.selectAll);
      }
    };
    $scope.snapOpts = { disable: 'right' };
    $scope.mainRefresh = function () {
      if ($appstate.refresh) {
        $appstate.refresh(true);  //
      }
    };
    $localdb.get('showMediaInStream').then(function (media) {
      $appstate.model.showMediaInStream = media === 'false' || media === false ? false : true;
    });
    $scope.showFilterPopup = function () {
      var modalInstance = $modal.open({
          templateUrl: 'views/popups/filterQuickView.html',
          controller: 'FilterquickviewCtrl',
          resolve: {
            room: function () {
              var rm = $appstate.model.selectedRoom ? $appstate.model.selectedRoom.children[0].roomId : $facility.getSync().observed[0].roomId;
              return $facility.findRoom(rm);
            },
            facility: function () {
              return $facility.getSync();
            }
          }
        });
      modalInstance.result.then(function (d) {
        $prefs.current.quickViewFilter = d;
        $appstate.model.filterApplied = d && d.length > 0 ? 'filterApplied' : '';
      }, function () {
        $prefs.current.quickViewFilter = [];
        $appstate.model.filterApplied = '';
      });
    };
    $scope.cacheEmpty = $appstate.isCacheEmpty;
    window.onresize = function () {
      if ($native.isNative()) {
        $timeout(function () {
          $rootScope.$broadcast('windowResize');  //$utils.alert("resize");
        }, 400);
      } else {
        $rootScope.$broadcast('windowResize');
      }
    };
    $scope.getMainHeight = function () {
    };
    //$scope.$watch('shared.model.eventList', function (evt, isOnline) {
    //  $appstate.model.eventListByChild = $appstate.getEventsByChild();
    //  $appstate.model.eventListByGroup = $appstate.getEventsByGroup();
    //  $appstate.model.eventListByDate = $appstate.getEventsByDate();
    //  $appstate.model.pageSize = 50;//parseInt(window.innerHeight / 40, 10);
    //});
    $scope.setImageUrl = function (url) {
      $appstate.model.imageurl = url;
      $appstate.model.backButtonFunction = function () {
        $timeout(function () {
          $appstate.model.imageurl = 'img/empty.png';
        });
      };
      event.preventDefault();
      event.cancelBubble = true;
    };
    $scope.fb = function (item, evt) {
      event.preventDefault();
      event.cancelBubble = true;
      //var url = item ? item.url : $appstate.model.imageurl;
      //getMessage(function(msg) {
      //if ( msg !== null )
      $facebook.sharePhoto(url);  //});
                                  //var msg = event ? event.child.firstName : $appstate.currentMedia
    };
    $scope.fbVid = function (item, evt) {
      event.preventDefault();
      event.cancelBubble = true;
      var url = item ? item.url : $appstate.model.vidurl;
      console.log(url);
      //var msg = event ? event.child.firstName : $appstate.currentMedia;
      //      getMessage(function(msg) {
      $facebook.shareVideo(url, null, item.thumbnail);  //      });
    };
    $scope.isLite = $facility.isLite;
    function getMessage(cb) {
      var modalInstance = $modal.open({
          templateUrl: 'views/popups/facebookStatusPopup.html',
          controller: 'FacebookstatusCtrl'
        });
      modalInstance.result.then(function (msg) {
        cb(msg);
      });
    }
    $scope.fbLogout = function () {
      $facebook.logout();
    };
    $scope.streamSaveFile = function (item) {
      event.preventDefault();
      event.cancelBubble = true;
      forge.file.saveURL(item.url, function () {
        $appstate.msg('Saved File');
      }, function (err) {
        console.log(err);
      });
    };
    $scope.showMsg = function () {
      $utils.alert('Clicked on your name!');
    };
    $scope.getImageUrl = function () {
      return 'url(' + $appstate.model.imageurl + ')';
    };
    $appstate.getProfileImageUrl = function (force) {
      var currentUser = $facility.getSync().currentUser;
      $image.getImage(currentUser.id, $utils.createPhotoUrl($krConfig.s3Base, currentUser.photoPath), force).then(function (d) {
        $scope.profilePhotoUrl = d;
      });
    };
    $scope.sideNav = { isOpen: false };
    $scope.toggleSideNav = function ($event) {
      $scope.sideNav.isOpen = !$scope.sideNav.isOpen;
      if ($event) {
        $event.stopPropagation();
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('FacebookstatusCtrl', [
  '$scope',
  '$modalInstance',
  function ($scope, $modalInstance) {
    $scope.vals = { status: '' };
    $scope.ok = function () {
      //console.log($scope.vals.date);
      $modalInstance.close($scope.vals.status);
    };
    $scope.cancel = function () {
      $modalInstance.close(null);
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('EditdeleteCtrl', [
  '$scope',
  '$appstate',
  '$messageBox',
  '$events',
  '$modalInstance',
  'item',
  'canEdit',
  'canDelete',
  'isParent',
  function ($scope, $appstate, $messageBox, $events, $modalInstance, item, canEdit, canDelete, isParent) {
    $scope.item = item;
    $scope.canEdit = canEdit;
    $scope.canDelete = canDelete;
    $scope.isParent = isParent;
    $scope.width = $scope.canEdit && $scope.canDelete ? '33%' : $scope.canEdit || $scope.canDelete ? '50%' : '100%';
    $scope.ok = function () {
      $modalInstance.close();
    };
    $scope.cancel = function () {
      $modalInstance.dismiss('cancel');
    };
    $scope.viewItem = function (item) {
      $modalInstance.close();
      $appstate.showEvent(item);
    };
    $scope.editItem = function (item) {
      $modalInstance.close();
      $appstate.viewEvent(item, $scope.isParent);
    };
    $scope.deleteItem = function (item) {
      $modalInstance.close();
      //$appstate.viewEvent(item, isParent);
      if ($scope.canDelete) {
        $messageBox.open('Delete Item', 'This will permanently delete the event.  This cannot be undone.  Are you sure you wish to continue?', {
          oktext: 'Yes',
          canceltext: 'No'
        }).result.then(function (selectedItem) {
          $events.deleteEvent(item, function (err, data) {
            if (err) {
              console.log(err);
              $messageBox.open('Error', 'An error occurred, the event was not deleted!', { oktext: 'Ok' });
            } else {
              $messageBox.open('Item Deleted', 'Event Deleted Successfully!', { oktext: 'Ok' });
              $appstate.refreshEvents($appstate.model.observed);
            }
          });
        }, function () {
          $appstate.msg('Delete Cancelled');
        });
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').factory('$popup', [
  '$modal',
  '$appstate',
  '$krConfig',
  '$native',
  '$backend',
  '$rootScope',
  '$utils',
  '$online',
  '$messageBox',
  '$facility',
  '$auth',
  '$timeout',
  '$translate',
  function ($modal, $appstate, $krConfig, $native, $backend, $rootScope, $utils, $online, $messageBox, $facility, $auth, $timeout, $translate) {
    var _scope = {};
    _scope.ok = function () {
      _scope.modal.close();  //.then(_scope.modal.remove);
    };
    _scope.getRelationText = function (p) {
      var parent = $facility.findUser(p.id);
      if (parent.parentType == 'PARENT')
        return $translate.instant('ParentGuardian');
      else if (parent.parentType == 'GRANDPARENT')
        return $translate.instant('Grandparent');
      else if (parent.parentType == 'AUNT')
        return $translate.instant('AuntUncle');
      else if (parent.parentType == 'OTHER')
        return $translate.instant('OtherFamilyFriend');
      else
        return $translate.instant('ParentGuardian');
    };
    _scope.native = $native.isNative();
    _scope.setProfilePhoto = function (obj) {
      //var un = localStorageService.get('username');
      //var pw = localStorageService.get('password');
      var token = $auth.getCredentialsSync();
      var creds = '/data/uploadMedia?type=PHOTO&profileId=' + obj.id + '&token=' + token.token;
      // short circuit for offline mode
      if (!$online.state()) {
        $messageBox.open('Offline Mode', 'Please try this again when you are online.', { oktext: 'Ok' });
        return;
      }
      $native.singleimage(function (err, file, url) {
        if (err) {
        } else {
          $timeout(function () {
            obj.photoUrl = url;  //"img/loading.gif";
          }, 10);
          $rootScope.$on('uploadSuccess', function (evt, resp) {
            var params = resp.params;
            //console.log(resp);
            if (params && params.id == obj.id) {
              //console.log("calling getprofileUrl");
              $backend.getProfileUrl(params.id, function (evt, data) {
                var data = angular.fromJson(data);
                _scope.$apply(function () {
                  obj.photoPath = data.path;
                  obj.photoUrl = $utils.createPhotoUrl($krConfig.s3Base, obj.photoPath);
                  //alert('Upload successful');
                  $appstate.msg('Profile Upload Successful!', true);
                });
              });
            }
          });
          $native.upload(file, url, null, creds, null, { id: obj.id });
        }
      }, $facility.getImageParameters());
    };
    function showIonic(child) {
      _scope.child = child;
      var user = $backend.getCurrentUser();
      _scope.setPhoto = user.role === 'TEACHER_ROLE' && user.canEditChildren === true;
      _scope.child.birthday = _scope.child.birthday != null && _scope.child.birthday.length > 0 ? _scope.child.birthday.split(' ').splice(0, 3).join(' ') : '';
      _scope.otherInfo = child.otherInfo ? child.otherInfo.replace('\r\n', '<br/>').replace('\n', '<br/>') : '';
      _scope.emergencyContact = child.emergencyContact ? child.emergencyContact.replace('\r\n', '<br/>').replace('\n', '<br/>') : '';
      //_scope.setPhoto = true; // todo check current user for permissions.
      _scope.child.parents.forEach(function (p) {
        p.photoUrl = $utils.createPhotoUrl($krConfig.s3Base, p.photoPath);
      });
      $ionicModal.fromTemplateUrl('views/popups/profilePopup.html', {
        scope: _scope,
        animation: 'slide-in-up'
      }).then(function (modal) {
        _scope.modal = modal;
        modal.show();
      });
    }
    function showBootstrap(item) {
      $appstate.canClose = _scope.ok;
      _scope.modal = $modal.open({
        templateUrl: 'views/popups/profilePopup.html',
        controller: 'ChildprofileCtrl',
        resolve: {
          child: function () {
            return item;
          }
        }
      });
    }
    return {
      showProfile: function (item) {
        //showIonic(item);
        showBootstrap(item);
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').factory('$online', [
  '$rootScope',
  function ($rootScope) {
    var _online = true;
    function _setState(s) {
      _online = s;
      $rootScope.$broadcast('onlineStateChange', _online);
    }
    if (typeof forge != 'undefined') {
      forge.event.connectionStateChange.addListener(function () {
        _setState(forge.is.connection.connected());
      }, function (err) {
        console.log(err);
      });
    } else {
      _setState(true);
      window.addEventListener('online', function () {
        _setState(true);
      }, true);
      window.addEventListener('offline', function () {
        _setState(false);
      }, true);
    }
    return {
      setState: function (state) {
        _setState(state);
      },
      state: function () {
        return _online;
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').factory('$krConfig', [
  '$localdb',
  function ($localdb) {
    // use relative path for web view
    var ajax = '';
    //ajax = "https://secure.kidreports.com";
    // use absolute url for native
    if (typeof forge != 'undefined') {
      ajax = 'https://secure.kidreports.com';
      //ajax = "http://10.0.0.209:9001";
      //ajax = "http://192.168.16.139:9001";
      $localdb.get('serverUrl').then(function (v) {
        if (v && v != '')
          ajax = v;
      });
    }
    return {
      clientID: 'com.kidreports.client.app',
      clientKey: 'webNativeClientApp',
      version: '2.20.0',
      reloadVersion: '',
      ajaxBase: ajax,
      mediaBase: ajax,
      s3Base: 'https://s3.amazonaws.com/kidreports-media/',
      token: null,
      width: window.innerWidth,
      height: window.innerHeight,
      getAjaxBase: function () {
        //ajax = "http://127.0.0.1:9001";
        //ajax = "http://192.168.16.64:9001";
        //ajax = "http://192.168.0.14:9001";
        //ajax = "http://192.168.16.159:9001";
        return ajax;
      },
      getMediaBase: function () {
        //return "http://192.168.16.159:3000";
        //return this.mediaBase;//'http://localhost:3000'
        return ajax;
      },
      isNZ: function () {
        return ajax.indexOf('co.nz') > -1;
      },
      setUrl: function (url) {
        if (typeof forge != 'undefined') {
          ajax = url;
          $localdb.set('serverUrl', url);
        }
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').factory('$prefs', function () {
  var prefs = {
      current: {
        autoLogin: false,
        showQuickView: false,
        loadingEvents: false,
        quickViewFilter: []
      },
      matchesQuickViewFilter: function (evt) {
        if (prefs.current.quickViewFilter.length == 0)
          return true;
        else {
          var arr = _.filter(prefs.current.quickViewFilter, function (et) {
              return et.name == evt.eventType.name;
            });
          return arr.length > 0;
        }
      }
    };
  return prefs;
});
'use strict';
angular.module('kidreports2App').factory('$base64', function () {
  var keyStr = 'ABCDEFGHIJKLMNOP' + 'QRSTUVWXYZabcdef' + 'ghijklmnopqrstuv' + 'wxyz0123456789+/' + '=';
  return {
    encode: function (input) {
      var output = '';
      var chr1, chr2, chr3 = '';
      var enc1, enc2, enc3, enc4 = '';
      var i = 0;
      do {
        chr1 = input.charCodeAt(i++);
        chr2 = input.charCodeAt(i++);
        chr3 = input.charCodeAt(i++);
        enc1 = chr1 >> 2;
        enc2 = (chr1 & 3) << 4 | chr2 >> 4;
        enc3 = (chr2 & 15) << 2 | chr3 >> 6;
        enc4 = chr3 & 63;
        if (isNaN(chr2)) {
          enc3 = enc4 = 64;
        } else if (isNaN(chr3)) {
          enc4 = 64;
        }
        output = output + keyStr.charAt(enc1) + keyStr.charAt(enc2) + keyStr.charAt(enc3) + keyStr.charAt(enc4);
        chr1 = chr2 = chr3 = '';
        enc1 = enc2 = enc3 = enc4 = '';
      } while (i < input.length);
      return output;
    },
    decode: function (input) {
      var output = '';
      var chr1, chr2, chr3 = '';
      var enc1, enc2, enc3, enc4 = '';
      var i = 0;
      // remove all characters that are not A-Z, a-z, 0-9, +, /, or =
      var base64test = /[^A-Za-z0-9\+\/\=]/g;
      if (base64test.exec(input)) {
        console.log('There were invalid base64 characters in the input text.\n' + 'Valid base64 characters are A-Z, a-z, 0-9, \'+\', \'/\',and \'=\'\n' + 'Expect errors in decoding.');
      }
      input = input.replace(/[^A-Za-z0-9\+\/\=]/g, '');
      do {
        enc1 = keyStr.indexOf(input.charAt(i++));
        enc2 = keyStr.indexOf(input.charAt(i++));
        enc3 = keyStr.indexOf(input.charAt(i++));
        enc4 = keyStr.indexOf(input.charAt(i++));
        chr1 = enc1 << 2 | enc2 >> 4;
        chr2 = (enc2 & 15) << 4 | enc3 >> 2;
        chr3 = (enc3 & 3) << 6 | enc4;
        output = output + String.fromCharCode(chr1);
        if (enc3 != 64) {
          output = output + String.fromCharCode(chr2);
        }
        if (enc4 != 64) {
          output = output + String.fromCharCode(chr3);
        }
        chr1 = chr2 = chr3 = '';
        enc1 = enc2 = enc3 = enc4 = '';
      } while (i < input.length);
      return output;
    }
  };
});
'use strict';
angular.module('kidreports2App').factory('$krCache', [
  '$rootScope',
  '$timeout',
  '$localstorage',
  function ($rootScope, $timeout, $localstorage) {
    var isForge = false;
    //typeof(forge) != 'undefined';
    // use some semaphores to prevent concurrent access due to callback threading
    var _isSetting = {};
    //var _isGetting = false;
    // disable if not debugging
    var isDebug = true;
    var _self = {
        addToCache: function (obj, cacheName, key, cb) {
          _self.getCache(cacheName, function (c) {
            var cache = c && _.isArray(c) ? c : [];
            if (cache.push) {
              if (obj) {
                if (obj.length)
                  cache = cache.concat(obj);
                else
                  cache.push(obj);
              }
            } else if (key)
              cache[key] = obj;
            _self.setCache(cache, cacheName);
            if (cb)
              cb();
          }, false);
        },
        setCache: function (cache, cacheName, cb) {
          cacheName = cacheName || 'requestCache';
          if (_isSetting[cacheName]) {
            $timeout(function () {
              _self.setCache(cache, cacheName, cb);
            }, 10);
            return;
          }
          _isSetting[cacheName] = true;
          if (isForge) {
            forge.prefs.set(cacheName, cache, function () {
              $rootScope.$broadcast('cacheChange', cacheName);
              _isSetting[cacheName] = false;
              if (cb)
                cb();
            }, function (err) {
              _isSetting[cacheName] = false;
              console.log(err);
            });
          } else {
            $timeout(function () {
              $localstorage.set(cacheName, cache).then(function () {
                $rootScope.$broadcast('cacheChange', cacheName);
                _isSetting[cacheName] = false;
                if (cb)
                  cb();
              }, function (err) {
                _isSetting[cacheName] = false;
                console.log(err);
              });
            }, 1);
          }
        },
        getCache: function (cacheName, cb, scopeApply) {
          cacheName = cacheName || 'requestCache';
          if (_isSetting[cacheName]) {
            $timeout(function () {
              _self.getCache(cacheName, cb, scopeApply);
            }, 10);
            return;
          }
          if (isForge) {
            forge.prefs.get(cacheName, function (data) {
              if (_isSetting[cacheName]) {
                $timeout(function () {
                  _self.getCache(cacheName, cb, scopeApply);
                }, 10);
                return;
              }
              if (scopeApply == false)
                cb(data);
              else
                $rootScope.$apply(function () {
                  //_isGetting = false;
                  cb(data);
                });
            }, function (err) {
              console.log(err);
              cb(null);
            });
          } else {
            $timeout(function () {
              $localstorage.get(cacheName).then(function (d) {
                if (_isSetting[cacheName]) {
                  $timeout(function () {
                    _self.getCache(cacheName, cb, scopeApply);
                  }, 10);
                  return;
                }
                //_isGetting = false;
                cb(d);
              });
            }, 1);
          }  //      },
             //      setValue: function(key, value, cb) {
             //        if ( isForge ) {
             //          forge.prefs.set(key, value, function() {
             //            if ( cb ) cb(null);
             //          }, function(err) {
             //            if ( cb ) cb(err);
             //          });
             //        } else {
             //          _self.setCache(value, key);
             //          if ( cb ) cb(null);
             //        }
             //      },
             //      getValue: function(key, cb) {
             //        if ( isForge ) {
             //          forge.prefs.get(key, function(data) {
             //            cb(null, data);
             //          }, function(err) {
             //            cb(err, null);
             //          });
             //        } else {
             //          cb(null, _self.getCache(key));
             //        }
        }
      };
    return _self;
  }
]);
'use strict';
angular.module('kidreports2App').factory('$ajax', [
  '$http',
  '$rootScope',
  '$krConfig',
  '$online',
  '$krCache',
  '$base64',
  '$auth',
  '$utils',
  function ($http, $rootScope, $krConfig, $online, $krCache, $base64, $auth, $utils) {
    var realAjax = null;
    function fixErr(err) {
      var obj = {
          date: new Date(),
          type: 'ajax'
        };
      if (err.config && err.config.url) {
        obj.err = {
          url: err.config.url,
          params: err.config.params,
          method: err.config.method
        };
      } else {
        obj.err = err;
      }
      return obj;
    }
    var ajaxProxy = function (options, cb, skipCreds) {
      if (skipCreds) {
        realAjax(options, function (err, data) {
          if (err) {
            $krCache.addToCache(fixErr(err), 'errors');
            cb(err);
          } else {
            cb(null, data);
          }
        });
      } else {
        $auth.getCredentials().then(function (credentials) {
          options.params = options.params || {};
          // append the version to every request
          options.params.version = $krConfig.version;
          // token auth wrapper
          if (!options.headers) {
            options.headers = { 'Authorization': 'Bearer ' + credentials.token };
          } else if (!options.headers['X-CSRF-Token']) {
            // only add the auth token if we don't already have a cross domain token.
            options.headers['Authorization'] = 'Bearer ' + credentials.token;
          }
          var time = new Date().getTime();
          realAjax(options, function (err, data) {
            if (err) {
              if (err.status == 401) {
                // logout... unauthorized
                $utils.alert('Your account is not authorized.  Logging out.');
                document.location.href = '#/logout';
              } else {
                $krCache.addToCache(fixErr(err), 'errors');
                cb(err);
              }
            } else {
              cb(null, data);
            }
          });
        }, function (err) {
          cb(err);
        });
      }
    };
    var defaultAjax = function (options, cb) {
      $http(options).success(function (data) {
        cb(null, data);
      }).error(function (data, status, headers, config) {
        var errObj = {
            data: data,
            status: status,
            headers: headers,
            config: config
          };
        cb(errObj);
      });
    };
    var forgeAjax = function (options, cb) {
      var serialize = function (obj) {
        var str = [];
        for (var p in obj) {
          var obStr = _.isObject(obj[p]) ? JSON.stringify(obj[p]) : obj[p];
          str.push(encodeURIComponent(p) + '=' + encodeURIComponent(obStr));
        }
        return str.join('&');
      };
      //var baseUrl = $krConfig.getAjaxBase();
      var params = serialize(options.params);
      var data = options.data ? JSON.stringify(options.data) : {};
      // forge ios ajax insists on data for posts
      if (options.data && options.preserveDataFormat)
        data = options.data;
      forge.request.ajax({
        type: options.method,
        url: options.url + '?' + params,
        data: data,
        dataType: 'json',
        headers: options.headers || {},
        contentType: options.contentType || 'application/x-www-form-urlencoded',
        success: function (data) {
          $rootScope.$apply(cb(null, data));
        },
        error: function (error) {
          cb(error);
        }
      });
    };
    realAjax = typeof forge != 'undefined' ? forgeAjax : defaultAjax;
    return { ajax: ajaxProxy };
  }
]);
'use strict';
angular.module('kidreports2App').factory('$backend', [
  '$rootScope',
  '$q',
  '$location',
  '$utils',
  '$adapter',
  '$ajax',
  '$krConfig',
  '$prefs',
  '$native',
  '$base64',
  '$online',
  '$krCache',
  '$timeout',
  '$localdb',
  '$messageBox',
  '$facility',
  '$progressiveTimeout',
  '$featureFlags',
  function ($rootScope, $q, $location, $utils, $adapter, $ajax, $krConfig, $prefs, $native, $base64, $online, $krCache, $timeout, $localdb, $messageBox, $facility, $progressiveTimeout, $featureFlags) {
    var credentials = {
        username: '',
        password: ''
      };
    var currentUser = null;
    var facility = null;
    var eventTimer;
    var eventCount = 0;
    var eventSaveInProgress = false;
    var delIdxList = [];
    var progressTimeoutScope = 'events';
    var _self = {
        setCredentials: function (username, password, token) {
          credentials.username = username;
          credentials.password = password;
          credentials.token = token;
          $krConfig.token = token;
          $localdb.set('username', username);
          $localdb.set('token', token);
        },
        logout: function () {
          this.setCredentials(null, null, null);
          this.setOfflineMode(false);
          facility = null;
        },
        getCurrentUser: function () {
          // TODO Hack
          return $facility.getSync() ? $facility.getSync().currentUser : null;
        },
        setOfflineMode: function (enabled) {
          if (enabled) {
            $localdb.set('facility', facility);
            $localdb.set('currentUser', currentUser);  //console.log("SET Cached Facility");
                                                       //console.log(facility);
          } else {
            $localdb.set('facility', null);
            $localdb.set('currentUser', null);
          }
        },
        isLoggedIn: function () {
          return facility != null;
        },
        refreshMode: false,
        saveEvent: function (event, isDraft) {
          // Create a list of ONLY child IDs
          var childrenList = [];
          event.observed.forEach(function (child) {
            childrenList.push(child.id);
          });
          var legacyEvent = $adapter.createLegacyEvent(event);
          // create a uuid so that we can tell if this event has been saved or not.
          if (event.clientKey) {
            legacyEvent.clientKey = event.clientKey;
          } else
            legacyEvent.clientKey = $utils.generateUUID();
          event.observed.forEach(function (child) {
            if (!child.events)
              child.events = [];
            var lEvent = $adapter.createLegacyEvent(event);
            lEvent.eventType = $facility.findEventType(lEvent.eventType.id);
            $adapter.processEvent(lEvent);
            lEvent.user = $facility.findUser(lEvent.user.id);
            lEvent.child = child;
            lEvent.clientKey = legacyEvent.clientKey;
            child.events.push(lEvent);
            $rootScope.$broadcast('eventQueued', lEvent);
          });
          if (isDraft)
            return;
          var info = event.eventType.name + ' Event (' + childrenList.length + ' Kids): Added @ ' + $utils.getTimeString($utils.getTimePart(new Date()));
          var params = {
              id: childrenList.toString(),
              body: legacyEvent
            };
          return _self.cacheAjax({
            method: 'POST',
            url: $krConfig.getAjaxBase() + '/data/event',
            params: params
          }, 'eventCB', info);
        },
        cacheAjax: function (evtParams, fire, info) {
          evtParams.fireEvent = fire;
          evtParams.info = info;
          evtParams.status = 'Pending';
          // Queue Events
          $krCache.addToCache(evtParams, null, null, function () {
            if (!eventTimer) {
              eventTimer = $progressiveTimeout.createTimer(progressTimeoutScope, _self.saveEventInternal);
            }
          });
        },
        cancelEventSave: function () {
          eventSaveInProgress = false;
        },
        saveEventInternal: function () {
          if ($online.state() != true) {
            // broadcast that we are complete so the spinner stops.
            $rootScope.$broadcast('ajaxComplete', true);
            return;
          }
          if ($facility.getSync() && $facility.getSync().disableServerPing) {
            _self._realSaveEvent();
          } else {
            // check if we are alive by pinging the server
            var opts = {
                method: 'GET',
                url: $krConfig.getAjaxBase() + '/public/ping.json',
                params: {}
              };
            $ajax.ajax(opts, function (err, data) {
              if (err) {
              } else {
                _self._realSaveEvent();
              }
            });
          }
        },
        _realSaveEvent: function () {
          $krCache.getCache(null, function (eventQueue) {
            eventQueue = eventQueue && _.isArray(eventQueue) ? eventQueue : [];
            eventQueue.forEach(function (evtState, idx) {
              if (evtState.status == 'Deleted') {
                var obj = {
                    info: evtState.info,
                    status: 'Deleted',
                    time: $utils.getTimeString($utils.getTimePart(new Date()))
                  };
                $krCache.addToCache(obj, 'notifications');
                $krCache.getCache(null, function (eventQueue2) {
                  delete eventQueue2[idx];
                  eventQueue2.clean(undefined);
                  $krCache.setCache(eventQueue2);
                });
              } else if (!eventSaveInProgress) {
                //evtState.status=="Pending") {
                eventSaveInProgress = true;
                evtState.status = 'In Progress';
                $krCache.setCache(eventQueue, null, function () {
                  var opts = {
                      method: evtState.method,
                      url: evtState.url,
                      params: evtState.params
                    };
                  if (evtState.data)
                    opts.data = evtState.data;
                  if (evtState.headers)
                    opts.headers = evtState.headers;
                  if (evtState.transformRequest)
                    opts.transformRequest = evtState.transformRequest;
                  if (evtState.preserveDataFormat)
                    opts.preserveDataFormat = evtState.preserveDataFormat;
                  if (evtState.contentType)
                    opts.contentType = evtState.contentType;
                  $ajax.ajax(opts, function (err, data) {
                    if (err) {
                      console.debug('Error making request.  Keeping in queue.');
                      $krCache.addToCache();
                      $rootScope.$broadcast(evtState.fireEvent, err, data);
                    } else {
                      $progressiveTimeout.notifyScopeOfSuccess(progressTimeoutScope);
                      //console.log(data);
                      var obj = {
                          info: evtState.info,
                          status: 'Success',
                          time: $utils.getTimeString($utils.getTimePart(new Date()))
                        };
                      $krCache.addToCache(obj, 'notifications');
                      $rootScope.$broadcast(evtState.fireEvent, err, data);
                      $krCache.getCache(null, function (eventQueue2) {
                        delete eventQueue2[idx];
                        eventQueue2.clean(undefined);
                        $krCache.setCache(eventQueue2);
                      });
                    }
                    eventSaveInProgress = false;
                  });
                });
              }
            });
            $progressiveTimeout.destroyTimer(eventTimer);
            // Don't run the timer if the queue is empty
            if (eventQueue.length == 0) {
              eventTimer = null;
              $rootScope.$broadcast('ajaxComplete', true);
            } else {
              // keep it going since it's a timeout, not an interval
              $rootScope.$broadcast('ajaxRunning', true);
              eventTimer = $progressiveTimeout.createTimer(progressTimeoutScope, _self.saveEventInternal);
            }
          });
        },
        sendMessage: function (message, cb) {
          var params = { body: message };
          return $ajax.ajax({
            method: 'GET',
            url: $krConfig.getAjaxBase() + '/data/message',
            params: params
          }, cb);  //
                   //
                   //        // not online, invoke the callback immediately.
                   //        if ( !$online.state() ) {
                   //          cb(null, {offline:true})
                   //        } else {
                   //          var f = $rootScope.$on("sendMessageCB", function(evt, err, data) {
                   //            f();
                   //            cb(null, {offline:false});
                   //          });
                   //        }
                   //
                   //        var info = "Send Message ("+ids.length+ " Kids): Added @ "+$utils.getTimeString($utils.getTimePart(new Date()));
                   //        return _self.cacheAjax({method: 'GET', url: $krConfig.getAjaxBase() + '/data/message', params: params}, 'sendMessageCB', info);
        },
        moveRoom: function (toRoom, ids, cb, ts) {
          ts = ts || $utils.createGsonCompatibleTimestamp(new Date());
          var params = {
              to: toRoom,
              ids: ids.join(','),
              ts: ts
            };
          // not online, invoke the callback immediately.
          if (!$online.state()) {
            cb(null, { offline: true });
          } else {
            $timeout(function () {
              var f = $rootScope.$on('moveRoomCB', function (evt, err, data) {
                  f();
                });
              cb(null, { offline: false });
            });
          }
          var info = 'Move Room (' + ids.length + ' Kids): Added @ ' + $utils.getTimeString($utils.getTimePart(new Date()));
          return $krConfig.isNZ() ? _self.cacheAjax({
            method: 'POST',
            url: $krConfig.getAjaxBase() + '/data/changeRoom',
            data: params
          }, 'moveRoomCB', info) : _self.cacheAjax({
            method: 'POST',
            url: $krConfig.getAjaxBase() + '/data/handleChangeRoomPost',
            data: params
          }, 'moveRoomCB', info);
        },
        getProfileUrl: function (id, cb) {
          var params = { id: id };
          return $ajax.ajax({
            method: 'GET',
            url: $krConfig.getAjaxBase() + '/data/getProfileUrl',
            params: params
          }, cb);
        },
        saveAttendance: function (children, category, date, cb, isUser, rmId, fromUser) {
          var attendanceUseSeconds = $featureFlags.isFeatureEnabled('attendance-use-seconds');
          var inDates = '', outDates = '';
          var inIds = '', outIds = '';
          var total = 0;
          date = date || new Date();
          children.forEach(function (kid) {
            if (kid.clean) {
              return;
            }
            if (kid.isIn === true) {
              inIds += kid.id + ',';
              if (kid.timeVal) {
                inDates += $utils.createGsonTSFromDTStrings($utils.getDatePart(date), kid.timeVal, attendanceUseSeconds) + ';';
              }
            } else if (kid.isIn === false) {
              outIds += kid.id + ',';
              if (kid.timeVal) {
                outDates += $utils.createGsonTSFromDTStrings($utils.getDatePart(date), kid.timeVal, attendanceUseSeconds) + ';';
              }
            }
            if (!kid.clean) {
              total++;
            }
          });
          inIds = inIds.length > 0 ? inIds.substr(0, inIds.length - 1) : '';
          outIds = outIds.length > 0 ? outIds.substr(0, outIds.length - 1) : '';
          inDates = inDates.length > 0 ? inDates.substr(0, inDates.length - 1) : '';
          outDates = outDates.length > 0 ? outDates.substr(0, outDates.length - 1) : '';
          if (inIds == '' && outIds == '') {
            return;
          }
          var params = {
              inIDs: inIds,
              outIDs: outIds,
              fromUser: fromUser
            };
          var url = $krConfig.isNZ() ? '/data/signInDiscrete' : '/data/handleSignInPost';
          params.inDates = inDates;
          params.outDates = outDates;
          if (isUser) {
            params.isUser = true;
            params.room = rmId;
          }
          if (!$online.state()) {
            cb(null, { offline: true });
          } else {
            $timeout(function () {
              var f = $rootScope.$on('attendanceCB', function (evt, err, data) {
                  cb(err, {
                    offline: false,
                    data: data
                  });
                  f();
                });
            });
          }
          var info = 'Attendance (' + total + ' Kids): Added @ ' + $utils.getTimeString($utils.getTimePart(new Date()));
          return $krConfig.isNZ() ? _self.cacheAjax({
            method: 'GET',
            url: $krConfig.getAjaxBase() + url,
            params: params
          }, 'attendanceCB', info) : _self.cacheAjax({
            method: 'POST',
            url: $krConfig.getAjaxBase() + url,
            data: params
          }, 'attendanceCB', info);
        },
        updateAttendance: function (eventId, newTimeOccurred, callback) {
          callback(null, null);
          var params = {
              eventId: eventId,
              newTimeOccurred: newTimeOccurred
            };
          if (!$online.state()) {
            callback(null, { offline: true });
          } else {
            $timeout(function () {
              var func = $rootScope.$on('updateAttendanceCB', function (event, error, response) {
                  callback(error, response);
                  func();
                });
            });
          }
          var info = 'Update attendance EventId: ' + eventId + ' Added: ' + $utils.getTimeString($utils.getTimePart(new Date()));
          return _self.cacheAjax({
            method: 'PUT',
            url: $krConfig.getAjaxBase() + '/updateAttendance',
            data: params
          }, 'updateAttendanceCB', info);
        }
      };
    return _self;
  }
]);
'use strict';
angular.module('kidreports2App').factory('$native', [
  '$krConfig',
  '$utils',
  '$rootScope',
  '$ajax',
  '$base64',
  '$http',
  '$krCache',
  '$timeout',
  '$online',
  '$q',
  '$messageBox',
  '$model',
  '$translate',
  function ($krConfig, $utils, $rootScope, $ajax, $base64, $http, $krCache, $timeout, $online, $q, $messageBox, $model, $translate) {
    var uploadTimer = null;
    var uploadInProgress = false;
    var firstTime = true;
    var currVidFile = null, currVidUrl = null, currVidCB = null, currFreeSpaceCB = null;
    var _serverPingDisabled = false;
    var useDefaultImage = true;
    var forgeEnabled = typeof forge != 'undefined';
    //        if ( forgeEnabled && forge.is.ios()) {
    //            forge.internal.call("damn_you_form_assist.killBar", {
    //                text: ""
    //            }, (function() {
    //                return $utils.alert("Success!");
    //            }), function(e) {
    //                return $utils.alert("Error: " + e.message);
    //            });
    //        }
    var nativeType = null;
    function iOSversion() {
      if (/iP(hone|od|ad)/.test(navigator.platform)) {
        // supports iOS 2.0 and later: <http://bit.ly/TJjs1V>
        var v = navigator.appVersion.match(/OS (\d+)_(\d+)_?(\d+)?/);
        return [
          parseInt(v[1], 10),
          parseInt(v[2], 10),
          parseInt(v[3] || 0, 10)
        ];
      }
    }
    function updateHeader(t) {
      t = t || 25;
      if (_self.isNative()) {
        var offset = 0;
        var vf = '';
        if (forge.is.ios()) {
          //&& ver && ver.length > 0 && ver[0] >= 5) {
          offset = 20;
          vf = 'viewport-fit=auto, ';
        }
        //<meta name="viewport" content="viewport-fit=cover, width=device-width, initial-scale=1.0, minimum-scale=1.0, maximum-scale=1.0, user-scalable=no">
        $timeout(function () {
          $('head meta[name="viewport"]').attr('content', vf + 'user-scalable=no, minimum-scale=1.0, maximum-scale=1.0, initial-scale=1.0');
        }, t);
      }
    }
    function finishRegistration(un) {
      var d = $q.defer();
      forge.parse.registerForNotifications(function () {
        forge.parse.push.subscribe(un, function () {
          forge.logging.info('subscribed to push');
          d.resolve();
        }, function (err) {
          forge.logging.error('error subscribing to push notifications: ' + JSON.stringify(err));
          d.reject();
        });
      }, function (err) {
        forge.logging.error('Failed to register for notifications: ' + JSON.stringify(err));
        d.reject();
      });
      return d.promise;
    }
    var ver = iOSversion();
    var _platform;
    var _self = {
        setServerPingDisabled: function (p) {
          _serverPingDisabled = p;
        },
        setUseMultiImageSelection: function (b) {
          useDefaultImage = !b;
        },
        resetiOSHeader: updateHeader,
        isNative: function () {
          return forgeEnabled && (forge.is.ios() || forge.is.android());
        },
        getPlatform: function () {
          return _platform;
        },
        isAndroid502: function () {
          return _self.android() && _platform && _platform.osversion == '5.0.2';
        },
        isKindle56: function () {
          return _platform.manufacturer == 'Amazon';
        },
        isIOS1103: function () {
          return _self.iOS() && _platform && _platform.osversion == '11.0.3';
        },
        isGreaterThanIOS1103: function () {
          var maj = parseInt(_platform.osversion.split('.')[0], 10);
          var min = parseInt(_platform.osversion.split('.')[1], 10);
          var bug = parseInt(_platform.osversion.split('.')[2], 10);
          return _self.iOS() && _platform && maj >= 11 && (min == 0 && bug > 3 || min > 0);
        },
        isIOS11: function () {
          var maj = parseInt(_platform.osversion.split('.')[0], 10);
          return _self.iOS() && _platform && maj >= 11;
        },
        isConnected: function () {
          return forgeEnabled && forge.is.connection.connected() || $rootScope.online;
        },
        type: function () {
          return forgeEnabled ? forge.nativeType : '';
        },
        iOS: function () {
          return forgeEnabled && forge.is.ios();
        },
        android: function () {
          return forgeEnabled && forge.is.android();
        },
        customEvent: function (name, params, success, error) {
        },
        anyOrientation: function () {
          if (forgeEnabled && forge.display) {
            forge.display.orientation.allowAny();
          }
        },
        landscape: function () {
          var d = $q.defer();
          if (forgeEnabled && forge.display) {
            forge.display.orientation.forceLandscape(function () {
              d.resolve();
            });
          } else {
            d.resolve();
          }
          return d.promise;
        },
        portrait: function () {
          var d = $q.defer();
          if (forgeEnabled && forge.display) {
            forge.display.orientation.forcePortrait(function () {
              d.resolve();
            });
          } else {
            d.resolve();
          }
          return d.promise;
        },
        removeFile: function (path) {
          var d = $q.defer();
          console.log('Removing file:' + path);
          forge.file.remove(path, function (s) {
            d.resolve(s);
          }, function (err) {
            d.reject(err);
          });
          return d.promise;
        },
        getInfo: function (path) {
          var d = $q.defer();
          console.log('Get info for file:' + path);
          forge.file.info(path, function (b) {
            if (b)
              d.resolve(b);
            else
              d.reject();
          }, function (err) {
            console.log('NO FILE:' + path);
            d.reject(err);
          });
          return d.promise;
        },
        isFile: function (path) {
          var d = $q.defer();
          console.log('Checking file:' + path);
          forge.file.exists(path, function (b) {
            if (b) {
              forge.file.getScriptURL(path, function (_url) {
                d.resolve({
                  url: _url,
                  file: path
                });
              }, function (err) {
                d.reject(err);
              });
            } else
              d.reject();
          }, function (err) {
            console.log('NO FILE:' + path);
            d.reject(err);
          });
          return d.promise;
        },
        cacheURL: function (url) {
          var d = $q.defer();
          //forge.file.saveURL(url, function(file) {
          forge.file.cacheURL(url, function (file) {
            forge.file.getScriptURL(file, function (_url) {
              d.resolve({
                url: _url,
                file: file
              });
            }, function (err) {
              d.reject(err);
            });
          }, function (err) {
            d.reject(err);
          });
          return d.promise;
        },
        setup: function () {
          if (forgeEnabled) {
            console.log('start native setup for ' + forge.nativeType);
            if (forge.topbar)
              forge.topbar.hide();
            forge.event.connectionStateChange.addListener(function () {
              //alert("connStateChange");
              // todo initialization.
              forge.nativeType = forge.is.android() ? 'ANDROID' : 'IOS';
            }, function (err) {
              console.log(err);
            });
            updateHeader();
            if (_self.isNative()) {
              forge.event.orientationChange.addListener(function () {
                //$utils.log("Orientation changed");
                //$utils.alert("new height"+$krConfig.height);
                updateHeader();
                $krConfig.width = window.innerWidth;
                $krConfig.height = window.innerHeight;
                $rootScope.$broadcast('orientationChanged', 'true');  //$utils.alert("new height"+$krConfig.height);
              }, function (err) {
                $utils.log('Orientation change err');
                console.log(err);  //alert(err);
              });
            }
            if (_self.isNative()) {
              forge.reload.updateReady.addListener(function () {
                $rootScope.$broadcast('reload.message', 'KidReports has been updated! Please press OK and wait for the app to refresh');
              }, function (err) {
                console.log('Error checking for update ready');
                console.log(err);
              });
            }
            forge.internal.addEventListener('pgcapture.complete', function (obj) {
              var err = false;
              if (obj && obj.duration && obj.duration > $model.videoDuration + 1) {
                $utils.alert('Video is longer than ' + $model.videoDuration + ' seconds.  Please select a different file.');
                currVidFile = null;
                currVidUrl = null;
                err = true;
              }
              if (currVidCB) {
                currVidCB(err, currVidFile, currVidUrl);
                currVidCB = null;
              }
            });
            forge.internal.addEventListener('deviceinfo.freeSpaceComplete', function (obj) {
              if (currFreeSpaceCB) {
                // offset for iOS is 225mb, offset for Android is 105mb
                var freeSpaceMB = _self.android() ? obj.space / 1024 - 105 : obj.space / (1024 * 1024) - 225;
                currFreeSpaceCB(freeSpaceMB);
                currFreeSpaceCB = null;
              }
            });
            if (forge && forge.pushwoosh)
              forge.pushwoosh.setForegroundAlert({ alert: true });
            // forge.event.messagePushed.addListener(function(data) {
            //   console.log("got message:"+JSON.stringify(data));
            //   if ( data && data.alert ) {
            //     $messageBox.open("Alert", data.alert, { oktext: "Ok"  });
            //   }
            // }, function(err) {
            //   $utils.alert(err);
            // });
            /// todo change to use a configuration defined stream.
            //                    forge.reload.switchStream("KidReports2", function() {
            //                        // now check for an update
            //                      $utils.alert("switched stream!");
            //                    }, function(err) {
            //                      console.log("error switching stream");
            //                      console.log(err);
            //                    });
            //                        forge.reload.updateAvailable(function(ua) {
            //                            if ( ua ) {
            //                                //alert("reload available");
            //                                console.log("reload available");
            //                                forge.reload.update(function(success) {
            //                                  console.log("reloaded new version!");
            //                                  // todo add alert
            //
            //                                    //alert("reload started");
            //                                }, function(err) {
            //                                    console.log("error updating");
            //                                    console.log(err);
            //                                });
            //                            }
            //                        }, function(err) {
            //                            console.log("error checking for update");
            //                            console.log(err);
            //                        });
            if (_self.isNative()) {
              forge.launchimage.hide();
            }
            if (_self.isNative() && forge && forge.topbar) {
            }
          }
          // init the platform
          _platform = { type: 'mobile' };
          if (forgeEnabled) {
            _platform.type = forge.is.ios() ? 'ios' : 'android';
            forge.platform.getVersion(function (v) {
              _platform.osversion = v;
            });
            forge.platform.getModel(function (v) {
              _platform.model = v;
            });
            if (forge.is.android()) {
              forge.platform.getAPILevel(function (v) {
                _platform.apiLevel = v;
                forge.platform.getManufacturer(function (v) {
                  _platform.manufacturer = v;
                });
              });
            }
          } else {
            //_platform.osversion = encodeURI(navigator.appVersion);
            _platform.model = navigator.appName;
            _platform.manufacturer = navigator.platform;
          }
        },
        restart: function () {
          forge.reload.applyAndRestartApp(function () {
          }, function () {
          });
        },
        registerPushwooshTag: function (name, uuid, appID) {
          var obj = {
              request: {
                application: appID,
                hwid: uuid,
                tags: {
                  Username: name,
                  'Parse Channels': name
                }
              }
            };
          var opts = {
              method: 'POST',
              url: 'https://cp.pushwoosh.com/json/1.3/setTags',
              data: obj
            };
          $ajax.ajax(opts, function (err, data) {
            if (err) {
            } else {
              //alert("setTags");
              var obj = {
                  request: {
                    application: appID,
                    hwid: uuid
                  }
                };
              var opts = {
                  method: 'POST',
                  url: 'https://cp.pushwoosh.com/json/1.3/getTags',
                  data: obj
                };
              $ajax.ajax(opts, function (err, data) {
                //alert("Got Tags:"+JSON.stringify(data));
                var opts = {
                    method: 'GET',
                    url: $krConfig.getAjaxBase() + '/data/setAppId',
                    params: { appId: appID }
                  };
                $ajax.ajax(opts, function (err, data) {
                });
              });
            }
          });
        },
        registerUsername: function (name) {
          // pushwoosh...
          var usePushwoosh = true;
          if (_self.isNative() && typeof forge.pushwoosh != 'undefined' && usePushwoosh) {
            var appID = forge.config.modules.parameters.config.PushwooshAppID;
            forge.internal.addEventListener('pushwoosh.registrationSuccess', function (status) {
              console.log('registered with token: ' + status['deviceToken']);
              // now set up the tag
              forge.pushwoosh.getHWID(function (hwid) {
                //alert("Got HWID:"+hwid+", AppID:"+appID);
                forge.pushwoosh.setForegroundAlert({ alert: true });
                //forge.pushwoosh.setMultiNotificationMode();
                _self.registerPushwooshTag(name, hwid, appID);
              });
            });
            forge.internal.addEventListener('pushwoosh.pushReceived', function (obj) {
              var data = JSON.parse(obj);
              if (data && (data.alert || data.title)) {
                var txt = data.alert || data.title;
                $messageBox.open('Alert', txt, { oktext: 'Ok' });
              }  //alert('push received: ' + data);
                 //alert('push received: ' + JSON.stringify(data));
            }, function (err) {
            });
            if (_self.iOS()) {
              forge.pushwoosh.onDeviceReady({ 'pw_appid': appID });
            } else {
              forge.pushwoosh.onDeviceReady({
                'pw_appid': appID,
                'gcm_id': '919511654544'
              });
            }
            forge.pushwoosh.registerDevice();
          }
          // migrate to parse
          if (_self.isNative() && typeof forge.parse != 'undefined' && !usePushwoosh) {
            console.log('checking Parse object');
            var un = name.replace(/[^a-zA-Z0-9]/gi, '');
            finishRegistration(un).then(function () {
              forge.parse.push.subscribedChannels(function (channels) {
                forge.logging.info('subscribed to: ' + JSON.stringify(channels));
                var count = 0;
                if (channels.length && channels.length > 0) {
                  _.each(channels, function (channel) {
                    if (channel != un) {
                      forge.parse.push.unsubscribe(channel, function () {
                      }, function (err) {
                      });
                    }
                  });
                }  //else finishRegistration(un);
              }, function (err) {
              });
            });
          }
        },
        singleimage: function (cb, params) {
          this.oldImage(function (err, files, urls) {
            cb(err, files[0], urls[0]);
          }, params);
        },
        image: function (cb, params) {
          if (useDefaultImage) {
            this.oldImage(cb, params);
          } else {
            var p = params || {};
            //$prefs.current.getMedia = true;
            forge.notification.confirm($translate.instant('imageSourceTitle'), $translate.instant('imageSourceBody'), $translate.instant('imageSourceGallery'), $translate.instant('imageSourceCamera'), function (userClickedYes) {
              if (userClickedYes) {
                forge.file.getImages(function (files) {
                  var total = files.length;
                  var count = 0;
                  var urls = [];
                  _.each(files, function (file) {
                    forge.file.getScriptURL(file, function (url) {
                      url = forge.httpd.normalize(url);
                      urls.push(url);
                      count++;
                      if (count == total) {
                        //alert("calling evdetails cb");
                        cb(null, files, urls);
                      }
                    });
                  });
                });
              } else {
                _self.oldImageFromCamera(cb, params);
              }
            });
          }
        },
        oldImage: function (cb, params) {
          var p = params || {};
          //$prefs.current.getMedia = true;
          forge.notification.confirm($translate.instant('imageSourceTitle'), $translate.instant('imageSourceBody'), $translate.instant('imageSourceGallery'), $translate.instant('imageSourceCamera'), function (userClickedYes) {
            if (userClickedYes) {
              _self.oldImageFromGallery(cb, params);
            } else {
              _self.oldImageFromCamera(cb, params);
            }
          });
        },
        oldImageFromCamera: function (cb, params) {
          params.source = 'camera';
          params.saveLocation = 'file';
          var method = forge.capture.getImage;
          _self._oldImageInternal(cb, params, method);
        },
        oldImageFromGallery: function (cb, params) {
          params.source = 'gallery';
          params.saveLocation = 'file';
          var method = forge.file.getImage;
          _self._oldImageInternal(cb, params, method);
        },
        _oldImageInternal: function (cb, params, method) {
          method(params, function (file) {
            // Get a URL to the returned file object which can be used from the local webview.
            //console.log("got file");
            //console.log(file);
            forge.file.getScriptURL(file, function (url) {
              url = forge.httpd.normalize(url);
              //$prefs.current.getMedia = false;
              //console.log("got url");
              //console.log(url);
              cb(null, [file], [url]);
            }, function (err) {
              console.log(err);
            });
          }, function (error) {
            cb(error);
          });
        },
        video: function (cb, params) {
          var p = params || {};
          var maxFileSize = 20000000;
          p.videoDuration = p.videoDuration || 15;
          forge.notification.confirm($translate.instant('imageSourceTitle'), $translate.instant('imageSourceBody'), $translate.instant('imageSourceGallery'), $translate.instant('imageSourceCamera'), function (userClickedYes) {
            if (userClickedYes) {
              forge.file.getVideo(p, function (file) {
                // Get a URL to the returned file object which can be used from the local webview.
                forge.file.getScriptURL(file, function (url) {
                  url = forge.httpd.normalize(url);
                  //currVidFile = file;
                  //currVidUrl = url;
                  //currVidCB = cb;
                  //forge.internal.call('pgcapture.getFormatData', {text: currVidUrl, file: currVidUrl});
                  forge.file.info(file, function (object) {
                    if (object.size > maxFileSize) {
                      $utils.alert('The video is larger than 20MB, please select a different file.');
                      cb(true);
                    } else {
                      cb(null, file, url);
                    }
                  }, function (e) {
                    cb(e);
                  });
                }, function (err) {
                  cb(err);
                });
              }, function (error) {
                cb(error);
              });
            } else {
              forge.capture.getVideo(p, function (file) {
                // Get a URL to the returned file object which can be used from the local webview.
                forge.file.getScriptURL(file, function (url) {
                  url = forge.httpd.normalize(url);
                  //currVidFile = file;
                  //currVidUrl = url;
                  //currVidCB = cb;
                  //forge.internal.call('pgcapture.getFormatData', {text: currVidUrl, file: currVidUrl});
                  forge.file.info(file, function (object) {
                    if (object.size > maxFileSize) {
                      $utils.alert('The video is larger than 20MB, please select a different file.');
                      cb(true);
                    } else {
                      cb(null, file, url);
                    }
                  }, function (e) {
                    cb(e);
                  });
                }, function (err) {
                  console.log(err);
                  cb(err);
                });
              }, function (error) {
                cb(error);
              });
            }
          });
        },
        getFreeSpace: function (cb) {
          currFreeSpaceCB = cb;
          forge.internal.call('deviceinfo.getFreeSpace', {});
        },
        saveToLibrary: function (url) {
          forge.file.saveURL(url, function (f) {
            console.log('saved file:' + f);
          }, function (err) {
            console.log(err);
          });
        },
        upload: function (file, url, cb, credentials, postCB, params, key) {
          var k = key || $utils.generateUUID();
          var item = {
              file: file,
              params: params,
              key: k,
              credentials: credentials,
              postCB: postCB,
              status: 'Pending'
            };
          var uploadType = item.credentials.indexOf('PHOTO') == -1 ? 'Video' : 'Photo';
          item.info = uploadType + ' upload @ ' + $utils.getTimeString($utils.getTimePart(new Date()));
          $krCache.addToCache(item, 'uploads', null, function (cache) {
            if (!uploadTimer) {
              uploadTimer = $timeout(_self.uploadInternal);
            }
            if (cb)
              cb(null, item);
          });
        },
        uploadMulti: function (files, urls, cb, credentials, postCB, params, keys) {
          var items = [];
          var count = 0, total = files.length;
          _.each(files, function (file) {
            var k = keys[count] || $utils.generateUUID();
            var item = {
                file: file,
                params: params,
                key: k,
                credentials: credentials,
                postCB: postCB,
                status: 'Pending'
              };
            var uploadType = item.credentials.indexOf('PHOTO') == -1 ? 'Video' : 'Photo';
            item.info = uploadType + ' upload @ ' + $utils.getTimeString($utils.getTimePart(new Date()));
            items.push(item);
            count++;
            if (count == total) {
              $krCache.addToCache(items, 'uploads', null, function (cache) {
                if (!uploadTimer) {
                  uploadTimer = $timeout(_self.uploadInternal);
                }
                if (cb)
                  cb(null, item);
              });
            }
          });
        },
        uploadThreadRunning: function () {
          return uploadTimer != null;
        },
        uploadInternal: function () {
          if ($online.state() != true) {
            $rootScope.$broadcast('uploadsComplete', true);
            return;
          }
          // check if we are alive by pinging the server
          if (_serverPingDisabled) {
            _self._realUpload();
          } else {
            var opts = {
                method: 'GET',
                url: $krConfig.getAjaxBase() + '/public/ping.json',
                params: {}
              };
            $ajax.ajax(opts, function (err, data) {
              if (err) {
              } else {
                _self._realUpload();
              }
            });
          }
        },
        _realUpload: function () {
          function fixErr(err) {
            var obj = {
                date: new Date(),
                type: 'ajax'
              };
            if (err.config && err.config.url) {
              obj.err = {
                url: err.config.url,
                params: err.config.params,
                method: err.config.method
              };
            } else {
              obj.err = err;
            }
            return obj;
          }
          function updateCacheItem(idx, status) {
            $krCache.getCache('uploads', function (evtQ) {
              var item = evtQ[idx];
              item.status = status;
              $krCache.setCache(evtQ, 'uploads');
            });
          }
          function uploadSuccess(data, idx, file) {
            $krCache.getCache('uploads', function (evtQ) {
              var item = evtQ[idx];
              console.log('Successful upload');
              // remove old file
              $utils.log('Removing File:' + file);
              forge.file.remove(file, function (success) {
                $utils.log('success removing file' + success);
              }, function (err) {
                $utils.log('Error removing file:' + err);
              });
              var obj = {
                  data: data,
                  params: item.params
                };
              uploadInProgress = false;
              $rootScope.$broadcast('uploadSuccess', obj);
              var obj = {
                  info: item.info,
                  status: 'Success',
                  time: $utils.getTimeString($utils.getTimePart(new Date()))
                };
              $krCache.addToCache(obj, 'notifications');
              //console.log("deleting idx:"+idx+", evtQ[idx]"+evtQ[idx]);
              delete evtQ[idx];
              evtQ.clean(undefined);
              $krCache.setCache(evtQ, 'uploads');
            });
          }
          function uploadError(err, idx) {
            console.log('Failed upload');
            console.log(JSON.stringify(err));
            forge.logging.info('Failed upload');
            forge.logging.info(JSON.stringify(err));
            $krCache.addToCache(fixErr(err), 'errors');
            updateCacheItem(idx, 'Error:' + err.message);
            uploadInProgress = false;
          }
          function uploadProgress(progress, idx) {
            $krCache.getCache('uploads', function (evtQ) {
              var item = evtQ[idx];
              // if we hit an error, reset to pending
              if (item.status.indexOf('Error') != -1) {
                updateCacheItem(idx, 'Pending');
                return;
              }
              var done, total;
              if (progress.done && progress.total) {
                done = progress.done;
                total = progress.total;
              } else {
                // ios native callback - array of done/total
                done = parseInt(progress[0], 10);
                total = parseInt(progress[1], 10);
              }
              var pct = done / total * 100;
              pct = Math.round(pct * 100) / 100;
              item.status = 'In Progress: ' + pct + '%';
              //item.status = "In Progress: "+progress.done+" of "+progress.total;
              $krCache.setCache(evtQ, 'uploads');
            });
          }
          $krCache.getCache('uploads', function (uploadQueue) {
            uploadQueue = uploadQueue && _.isArray(uploadQueue) ? uploadQueue : [];
            var key = null;
            uploadQueue.forEach(function (item, idx) {
              if (item.status == 'Deleted') {
                var obj = {
                    info: item.info,
                    status: 'Deleted',
                    time: $utils.getTimeString($utils.getTimePart(new Date()))
                  };
                $krCache.addToCache(obj, 'notifications');
                delete uploadQueue[idx];
                uploadQueue.clean(undefined);
                $krCache.setCache(uploadQueue, 'uploads');
              }
              // only start it if we aren't in progress.
              if (!uploadInProgress && item.status == 'Pending') {
                uploadInProgress = true;
                item.status = 'In Progress';
                // force a cache broadcast for the settings page
                $krCache.setCache(uploadQueue, 'uploads');
                var uploadURL = $krConfig.getMediaBase() + item.credentials + '&key=' + item.key;
                /// TODO replace this with forge once they add bg uploading for ios.  For now use my plugin
                var forceBackgroundIOSUpload = $model.forceBackgroundIOSUpload;
                /* if (forceBackgroundIOSUpload && forge.is.ios() && (item.credentials.indexOf("Video") > -1 || (!$model.imageScaleParams || !$model.imageScaleParams.width))) {

                if (firstTime) {
                  forge.internal.addEventListener("background_upload.progress", function (obj) {
                    uploadProgress(obj, idx)
                  });
                  forge.internal.addEventListener("background_upload.success", function (obj) {
                    uploadSuccess(obj, idx, item.file)
                  });
                  forge.internal.addEventListener("background_upload.error", function (obj) {
                    uploadError(obj, idx)
                  });
                  firstTime = false;
                }

                var settings = {
                  file: item.file.uri,
                  url: uploadURL,
                  fileName: item.key,
                  chunked: true
                };

                console.log("calling ios bg upload");
                //alert("calling ios bg upload");
                forge.internal.call('background_upload.upload', settings,
                  function () {
                    console.log('Success!')
                  },
                  function (e) {
                    console.log('Error: ' + e.message)
                  }
                );
              } else {//} if ( forge.is.android() ) { */
                ///alert("calling regular upload");
                forge.request.ajax({
                  type: 'POST',
                  url: uploadURL,
                  files: [item.file],
                  dataUploadMethod: 'raw',
                  success: function (obj) {
                    uploadSuccess(obj, idx, item.file);
                  },
                  error: function (obj) {
                    uploadError(obj, idx);
                  },
                  progress: function (obj) {
                    uploadProgress(obj, idx);
                  },
                  timeout: 1800000
                });  //}
              }
            });
            // Don't run the timer if the queue is empty or we went offline.
            $timeout.cancel(uploadTimer);
            if (uploadQueue.length == 0 || $online.state() != true) {
              uploadTimer = null;
              $rootScope.$broadcast('uploadsComplete', true);
            } else {
              // keep it going since it's a timeout, not an interval
              uploadTimer = $timeout(_self._realUpload, 5000);
              $rootScope.$broadcast('uploadsRunning', true);
            }
          });
        },
        enhanceUI: function () {
          if (typeof forge != 'undefined' && forge.is.android()) {
            forge.ui.enhanceAllInputs();
          }
        }
      };
    return _self;
  }
]);
'use strict';
angular.module('kidreports2App').factory('$utils', [
  '$translate',
  function ($translate) {
    function _zeroPad(n) {
      n = isNaN(n) ? 0 : n;
      return n > 9 ? '' + n : '0' + n;
    }
    function _prefix() {
      return 'img/icon/w/';
    }
    function _gsonTs(ts) {
      var head = ts.toString().substring(4, 10);
      // Apr 29
      var h = ts.getHours() > 12 ? ts.getHours() - 12 : ts.getHours();
      var ap = ts.getHours() >= 12 ? 'PM' : 'AM';
      var m = ts.getMinutes();
      var sec = ts.getSeconds();
      var yr = ts.getFullYear();
      // if somehow we got NaN for m and h, just use a new date to set this.
      if (isNaN(m) && isNaN(h)) {
        var d = new Date();
        //console.log("new date: " + d);
        m = d.getMinutes();
        h = d.getHours();
        sec = d.getSeconds();
        ap = h >= 12 ? 'PM' : 'AM';
        yr = d.getFullYear();
        head = d.toString().substring(4, 10);
      }
      var tail = ', ' + yr + ' ' + _zeroPad(h) + ':' + _zeroPad(m) + ':00 ' + ap;
      var s = head + tail;
      return s.trim();
    }
    return {
      isAndroid5: function () {
        var ua = navigator.userAgent;
        if (ua.indexOf('Android') >= 0) {
          //console.log(ua);
          var androidversion = parseFloat(ua.slice(ua.indexOf('Android') + 8));
          //console.log(androidversion);
          if (androidversion >= 5 && androidversion < 5.1) {
            return true;
          }
        }
        return false;
      },
      isAndroid502: function () {
        return true;
        var ua = navigator.userAgent;
        if (ua.indexOf('Android') >= 0) {
          //console.log(ua);
          var androidversion = parseFloat(ua.slice(ua.indexOf('Android') + 8));
          //console.log(androidversion);
          if (androidversion == 5.02) {
            return true;
          }
        }
        return false;
      },
      noop: function () {
      },
      zeroPad: _zeroPad,
      prefix: _prefix,
      getDefault: function (et) {
        var n = et.name.toLowerCase().trim();
        //var prefix = "img/icon/";
        var prefix = _prefix();
        if (n.indexOf($translate.instant('diaper').toLowerCase()) > -1)
          return prefix + 'diaper.png';
        else if (n.indexOf($translate.instant('nap').toLowerCase()) > -1)
          return prefix + 'nap.png';
        else if (n.indexOf($translate.instant('play').toLowerCase()) > -1)
          return prefix + 'play.png';
        else if (n.indexOf($translate.instant('profile planner').toLowerCase()) > -1)
          return 'img/icons/profileplanner.png';
        else if (n.indexOf($translate.instant('activity').toLowerCase()) > -1)
          return prefix + 'play.png';
        else if (n.indexOf($translate.instant('pleasebring').toLowerCase()) > -1)
          return prefix + 'pleasebring.png';
        else if (n.indexOf($translate.instant('bottle').toLowerCase()) > -1)
          return prefix + 'bottle.png';
        else if (n.indexOf($translate.instant('meal').toLowerCase()) > -1)
          return prefix + 'meal.png';
        else if (n.indexOf($translate.instant('snack').toLowerCase()) > -1)
          return prefix + 'fork.png';
        else if (n.indexOf($translate.instant('lessonplan').toLowerCase()) > -1)
          return prefix + 'book1.png';
        else if (n.indexOf($translate.instant('picturevideo').toLowerCase()) > -1)
          return prefix + 'take-picture.png';
        else if (et.iconPath) {
          return prefix + et.iconPath;
        } else
          return prefix + 'activity.png';
      },
      createPhotoUrl: function (base, path, defIcon) {
        var ret = defIcon || 'img/face.png';
        if (path) {
          ret = base + path;
        }
        return ret;
      },
      createGsonCompatibleTimestamp: _gsonTs,
      createGsonTSFromDTStrings: function (date, time, useSeconds) {
        useSeconds = useSeconds || false;
        var ds = date.split('-');
        time = time.getTime ? this.getTimePart(time) : time;
        var ts = time.split(':');
        function getMonth(num) {
          var month = new Array();
          month[0] = 'January';
          month[1] = 'February';
          month[2] = 'March';
          month[3] = 'April';
          month[4] = 'May';
          month[5] = 'June';
          month[6] = 'July';
          month[7] = 'August';
          month[8] = 'September';
          month[9] = 'October';
          month[10] = 'November';
          month[11] = 'December';
          //console.log("Number for month:");
          //console.log(num);
          var mon = num - 1;
          //console.log(mon);
          var d = month[mon];
          //console.log(d);
          return d;
        }
        var monStr = ds[1];
        //console.log(ds[1]);
        var mon = getMonth(ds[1]);
        var head = mon + ' ' + ds[2];
        var tail = ', ' + ds[0] + ' ';
        // time is either 5:14 pm or 17:14 or 17:14:00.  Need to handle all 3
        if (time.indexOf('AM') != -1 || time.indexOf('PM') != -1) {
          var hrs = parseInt(ts[0], 10);
          var minAP = ts[1].split(' ');
          var ap = minAP[1];
          tail += _zeroPad(hrs) + ':' + _zeroPad(parseInt(minAP[0], 10)) + ':' + (useSeconds ? _zeroPad(parseInt(ts[2], 10)) : '00') + ' ' + ap;
        } else {
          var hrs = parseInt(ts[0], 10);
          var h = hrs > 12 ? hrs - 12 : hrs;
          var ap = hrs >= 12 ? 'PM' : 'AM';
          tail += _zeroPad(h) + ':' + _zeroPad(parseInt(ts[1], 10)) + ':' + (useSeconds ? _zeroPad(parseInt(ts[2], 10)) : '00') + ' ' + ap;
        }
        var s = head + tail;
        return s.trim();
      },
      generateUUID: function () {
        var d = new Date().getTime();
        var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
            var r = (d + Math.random() * 16) % 16 | 0;
            d = Math.floor(d / 16);
            return (c == 'x' ? r : r & 7 | 8).toString(16);
          });
        return uuid;
      },
      createDateFromDTStrings: function (date, time) {
        var ds = date.split('-');
        time = time.getTime ? this.getTimePart(time) : time;
        var ts = time.split(':');
        var d = new Date(parseInt(ds[0], 10), parseInt(ds[1], 10) - 1, parseInt(ds[2], 10), parseInt(ts[0], 10), parseInt(ts[1], 10));
        var d2 = new Date();
        d2.setFullYear(parseInt(ds[0], 10), parseInt(ds[1], 10) - 1, parseInt(ds[2], 10));
        d2.setHours(parseInt(ts[0], 10));
        d2.setMinutes(parseInt(ts[1], 10));
        return d2;
      },
      getDatePart: function (date) {
        return date.getFullYear() + '-' + _zeroPad(date.getMonth() + 1) + '-' + _zeroPad(date.getDate());
      },
      getTimePart: function (date) {
        return _zeroPad(date.getHours()) + ':' + _zeroPad(date.getMinutes()) + ':' + _zeroPad(date.getSeconds());
      },
      getTimeString: function (timeStr, useSeconds) {
        useSeconds = useSeconds || false;
        if (!timeStr || timeStr === '')
          return '';
        if (timeStr.getTime)
          return this.getTimeString(this.getTimePart(timeStr), useSeconds);
        var arr = timeStr.split(':');
        var hrStr = arr[0];
        var hrs = parseInt(hrStr, 10);
        var ampm = 'AM';
        if (hrs >= 12) {
          ampm = 'PM';
          hrs = hrs == 12 ? hrs : hrs - 12;
        } else if (hrs == 0) {
          hrs = 12;
        }
        return useSeconds ? hrs + ':' + arr[1] + ':' + arr[2] + ' ' + ampm : hrs + ':' + arr[1] + ' ' + ampm;
      },
      getMilTime: function (milTimeStr) {
        if (milTimeStr.getTime)
          milTimeStr = this.getTimeString(this.getTimePart(milTimeStr));
        // check if mil format though - sometimes it is.  If so just return it.
        var arr = milTimeStr.split(':');
        if (arr.length == 3 && milTimeStr.indexOf('AM') == -1 && milTimeStr.indexOf('PM') == -1) {
          return milTimeStr;
        }
        var hrs = parseInt(arr[0], 10);
        var arr2 = arr[1].split(' ');
        var min = arr2[0];
        var sec = arr2[1];
        var ispm = arr2[1] == 'PM';
        if (ispm && hrs < 12)
          hrs += 12;
        else if (!ispm && hrs == 12)
          hrs = 0;
        return _zeroPad(hrs) + ':' + min + ':00';
      },
      isToday: function (date) {
        if (Object.prototype.toString.call(date) === '[object Date]') {
          var now = new Date();
          return now.getMonth() === date.getMonth() && now.getDate() === date.getDate();
        } else {
          return true;
        }
      },
      isTomorrow: function (t) {
        var sync = false;
        if (!t) {
          //t = new Date().getTime();
          sync = true;
        } else {
          var startOfToday = new Date();
          startOfToday.setHours(0);
          startOfToday.setMinutes(0);
          startOfToday.setSeconds(0);
          // resync if last timestamp was from the prior day.
          sync = t < startOfToday.getTime();
        }
        return sync;
      },
      translateDate: function (datestr) {
        if (typeof datestr === 'object') {
          datestr = _gsonTs(datestr);
        }
        // translate string like 'Mar 11, 2015 10:21:00 AM'
        var month = datestr.substr(0, datestr.indexOf(' '));
        return $translate.instant(month) + ' ' + datestr.substr(datestr.indexOf(' '));
      },
      formatLink: function (link) {
        var re = new RegExp('^(kidreports|http|https)://', 'i');
        if (!re.test(link)) {
          link = 'http://' + link;
        } else if (link.indexOf('kidreports') == 0) {
          link = link.replace('kidreports://', '');
        }
        return link;
      },
      log: function (msg) {
        if (typeof forge != 'undefined') {
          forge.logging.info(msg);
        } else {
          console.log(msg);
        }
      },
      alert: function (msg) {
        if (typeof forge != 'undefined') {
          forge.notification.alert('Alert', msg);
        } else {
          window.alert(msg);
        }
      },
      convertUsersToString: function (users, numToShow) {
        var validUsers = _.filter(users, function (user) {
            return user && user.firstName && user.lastName;
          });
        var fullNames = _.map(validUsers, function (user) {
            return user.firstName + ' ' + user.lastName;
          });
        var displayedNames = numToShow ? fullNames.slice(0, numToShow) : fullNames;
        var appendOthers = '';
        if (fullNames.length !== displayedNames.length) {
          var numOthers = fullNames.length - displayedNames.length;
          appendOthers += ', and ' + numOthers + (numOthers === 1 ? ' other' : ' others');
        }
        return displayedNames.join(', ') + appendOthers;
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').factory('$adapter', [
  '$utils',
  '$eventFactory',
  '$krConfig',
  '$facility',
  '$model',
  '$featureFlags',
  function ($utils, $eventFactory, $krConfig, $facility, $model, $featureFlags) {
    function updateAttendance(item, baseDate) {
      var attendanceUseSeconds = $featureFlags.isFeatureEnabled('attendance-use-seconds');
      if (item.cachedAttendance || (!item.events || item.events.length == 0)) {
        item.originalState = item.isIn;
        return item;
      }
      var lastTime = null, lastEvt = null;
      if (item.events) {
        _.forEach(item.events, function (evt) {
          var evtType = evt.eventType && evt.eventType.id ? $facility.findEventType(evt.eventType.id) : null;
          if (evtType && evtType.name === 'Take Attendance' && (lastTime == null || new Date(lastTime) < new Date(evt.timeOccurred))) {
            lastEvt = evt;
            lastTime = evt.timeOccurred;
          }
        });
      }
      // is attendance already set for today?
      if (lastEvt && lastEvt.eventValues && lastEvt.eventValues.length > 0) {
        item.isIn = lastEvt.eventValues[0].value == 'In' ? true : false;
        item.timeVal = $utils.getTimePart(new Date(lastTime));
        item.time = $utils.getTimeString(item.timeVal, attendanceUseSeconds);
        item.previousTimeOccurred = new Date(lastTime);
        item.timeOccurred = new Date(lastTime);
        item.lastEventId = lastEvt.id;
        item.attendanceNotes = lastEvt.notes;
        item.attendanceSet = true;
      }
      item.originalState = item.isIn;
    }
    ;
    function extractMedia(evt, type) {
      var ret = [];
      if (evt.mediaList) {
        _.forEach(evt.mediaList, function (obj) {
          if (type == obj.mediaType) {
            ret.push({
              'url': $krConfig.s3Base + obj.relativePath,
              'key': obj.key,
              'id': obj.id,
              'thumbnail': obj.thumbnail
            });
          }
        });
      }
      return ret;
    }
    function findEtt(eventType, id) {
      var match = _.filter(eventType.eventTypeTemplates, function (ett) {
          return ett.id == id;
        });
      if (match && match[0])
        return match[0];
      else
        return null;
    }
    var _this = {
        refreshState: function (events, kids, date, preserveSelection) {
          date = date || new Date();
          var timevalue = $utils.getTimePart(date);
          function setItem(item) {
            item.clean = true;
            item.selected = preserveSelection ? item.selected : false;
            _this.findAttendanceStatus(item, timevalue);
          }
          function setEvents(item, evt) {
            item.events.push(evt);
            _this.findAttendanceStatus(item, timevalue);
          }
          var attendanceSet = false;
          _.forEach(kids, function (item, key) {
            setItem(item);
          });
          //var localItems = angular.copy(items);
          var itemLookup = {};
          _.forEach(kids, function (item) {
            itemLookup[item.id] = item;
          });
          // clear out the old ones
          _.forEach(kids, function (item) {
            item.events = [];
          });
          _.forEach(events, function (evt) {
            var lookupId = evt.child ? evt.child.id : evt.subject.id;
            var item = itemLookup[lookupId];
            if (item) {
              setEvents(item, evt);
              // EXPENSIVE
              attendanceSet = attendanceSet || item.isIn != null;
            }
          });
          kids.forEach(function (item) {
            item.attendanceSet = attendanceSet;
          });
        },
        updateNapEndTime: function (event) {
          var currMS = event.properties.nowTime;
          var endTime = null;
          _.forEach(event.eventValues, function (ev) {
            if (ev.type == 'TIME_INPUT' && (ev.name.toLowerCase().indexOf('end') > -1 || ev.name.toLowerCase().indexOf('stop') > -1)) {
              ev.value = $utils.getTimePart(new Date(currMS));  //new Date(new Date(ev.value).getTime() + elapsedMS);
            }
          });
        },
        processEvent: function (evt) {
          var event = angular.copy(evt);
          event.hasPhoto = event.mediaList != null && _.filter(event.mediaList, function (m) {
            return m.mediaType == 'PHOTO';
          }).length > 0;
          event.hasVideo = event.mediaList != null && _.filter(event.mediaList, function (m) {
            return m.mediaType == 'VIDEO';
          }).length > 0;
          // update thumbnails for videos and photos
          if (event.mediaList) {
            _.forEach(event.mediaList, function (m) {
              if (m.mediaType == 'VIDEO') {
                if (m.thumbnailPath && m.thumbnailPath.indexOf('_0000.png') > -1)
                  m.thumbnail = $krConfig.s3Base + m.thumbnailPath;
                else
                  m.thumbnail = 'https://secure.kidreports.com/public/img/kidreports_small.png';
              } else {
                m.thumbnail = $krConfig.s3Base + m.thumbnailPath;
              }
            });
          }
          // save off any local/new images
          var imgs = event.images;
          var vids = event.videos;
          var keys = {};
          event.images = extractMedia(event, 'PHOTO');
          event.videos = extractMedia(event, 'VIDEO');
          _.forEach(event.images, function (k) {
            keys[k.key] = true;
          });
          _.forEach(event.videos, function (k) {
            keys[k.key] = true;
          });
          // Only use "isNew" flag if this event hasn't been saved yet AND there are no images/videos from the server
          //if(!event.id){
          if (imgs) {
            _.forEach(imgs, function (i) {
              if (i.isNew == true && i.url != null && !keys[i.key])
                event.images.push(i);
            });
          }
          if (vids) {
            _.forEach(vids, function (i) {
              if (i.isNew == true && i.url != null && !keys[i.key])
                event.videos.push(i);
            });
          }
          //}
          if ($model.viewPortfolio) {
            event.timeStr = $utils.getDatePart(new Date(event.timeOccurred)) + ' ' + $utils.getTimeString($utils.getTimePart(new Date(event.timeOccurred)));
          } else {
            event.timeStr = $utils.getTimeString($utils.getTimePart(new Date(event.timeOccurred)));
          }
          event.eventValues = _.filter(event.eventValues, function (ev) {
            if (ev.ett && !ev.eventTypeTemplate)
              ev.eventTypeTemplate = {
                id: ev.ett.id,
                name: ev.ett.name
              };
            return ev.eventTypeTemplate || ev.ett;
          });
          event.eventValues.forEach(function (ev) {
            if (ev.eventTypeTemplate && ev.eventTypeTemplate.templateType == 'TIME_INPUT' || ev.ett && ev.ett.templateType == 'TIME_INPUT') {
              ev.formattedValue = $utils.getTimeString(ev.value);
            } else
              ev.formattedValue = ev.value;
            if (!ev.eventTypeTemplate || !ev.eventTypeTemplate.name || ev.eventTypeTemplate.name == '') {
              var ett = findEtt(event.eventType, ev.eventTypeTemplate ? ev.eventTypeTemplate.id : ev.ett.id);
              if (ett && ett.name)
                ev.eventTypeTemplate.name = ett.name;
            }
          });
          event.eventValues.forEach(function (ev) {
            ev.eventTypeTemplate = _.filter($facility.findEventType(event.eventType.id).eventTypeTemplates, function (ett) {
              return ett.id == ev.eventTypeTemplate.id;
            })[0];
          });
          //evt.eventValues = evt.eventValues.reverse();
          event.dateTimeOccurred = new Date(evt.timeOccurred);
          var idx = 0;
          var found = {};
          event.formattedEventValues = [];
          _.forEach(event.eventValues, function (ev) {
            var s = '';
            if (!ev.eventTypeTemplate)
              return;
            if (typeof found[ev.eventTypeTemplate.name] === 'undefined') {
              found[ev.eventTypeTemplate.name] = idx;
              event.formattedEventValues.push({
                name: ev.eventTypeTemplate.name,
                value: ''
              });
              idx++;
            }
            var index = found[ev.eventTypeTemplate.name];
            var val = event.formattedEventValues[index].value;
            if (val.length > 0) {
              val += ', ';
            }
            val += ev.formattedValue;
            event.formattedEventValues[index].value = val;
          });
          return event;
        },
        createNewEvent: function (eventType) {
          var event = $eventFactory.buildEvent(eventType.name);
          function createEventValue(ett) {
            var valueOps = {
                'MULTI_SELECT': function (ret) {
                  ret.value = { choices: [] };
                  _.forEach(ett.templateValues, function (tv) {
                    ret.value.choices.push({
                      name: tv.value,
                      selected: false
                    });
                  });
                },
                'NUMERIC_SELECT': function (ret) {
                  ret.value = {
                    amount: 0,
                    numericUnits: ett.numericUnits,
                    choices: []
                  };
                  _.forEach(ett.templateValues, function (tv) {
                    ret.value.choices.push({ name: tv.value });
                  });
                  ret.value.selected = ret.value.choices[0];
                },
                'SINGLE_SELECT': function (ret) {
                  ret.value = { choices: [] };
                  _.forEach(ett.templateValues, function (tv) {
                    ret.value.choices.push({ name: tv.value });
                  });
                  ret.value.selected = ret.value.choices[0];
                },
                'TIME_INPUT': function (ret) {
                  ret.value = $utils.getTimePart(new Date());
                }
              };
            var valueOp = valueOps[ett.templateType] || $utils.noop;
            var ret = $eventFactory.buildEventValue(ett.name, ett.templateType);
            valueOp(ret);
            // Hang on to legacy template type for now
            ret.ett = ett;
            return ret;
          }
          _.forEach(eventType.eventTypeTemplates, function (ett) {
            var eventVal = createEventValue(ett);
            event.eventValues.push(eventVal);
          });
          // Hang onto legacy event type for now
          event.eventType = eventType;
          return event;
        },
        findAttendanceStatus: updateAttendance,
        filterByPresent: function (items, baseDate) {
          if (!baseDate)
            baseDate = $utils.getTimePart(new Date());
          var attendanceSet = false;
          var arr = _.filter(items, function (item) {
              updateAttendance(item, baseDate);
              attendanceSet = attendanceSet || item.isIn != null;
              return item.isIn;
            });
          _.forEach(items, function (item) {
            item.attendanceSet = attendanceSet;
          });
          return items;
        },
        createEvent: function (legacyEvent, facility) {
          var eventType = null;
          function findEventType() {
            _.forEach(facility.eventTypes, function (et) {
              if (et.id === legacyEvent.eventType.id) {
                eventType = et;
              }
            });
          }
          findEventType();
          var ret = this.createNewEvent(eventType);
          function populateEvent() {
            var ops = {
                'MULTI_SELECT': function (lev) {
                  _.forEach(ret.eventValues, function (ev) {
                    if (ev.type === 'MULTI_SELECT' && lev.eventTypeTemplate.id == ev.ett.id) {
                      var found = false;
                      _.forEach(ev.value.choices, function (choice) {
                        if (choice.name === lev.value) {
                          found = true;
                          choice.selected = true;
                        }
                      });
                      // Add a custom item here
                      if (!found) {
                        ev.value.choices.push({
                          name: lev.value,
                          selected: true
                        });
                      }
                    }
                  });
                },
                'NUMERIC_SELECT': function (lev, ev) {
                  _.forEach(ret.eventValues, function (ev) {
                    if (ev.type === 'NUMERIC_SELECT' && lev.eventTypeTemplate.id == ev.ett.id) {
                      var split = lev.value.split(' ');
                      ev.value.amount = parseFloat(split[0]);
                      ev.value.numericUnits = split[1];
                      var val = split[2];
                      var found = false;
                      _.forEach(ev.value.choices, function (choice) {
                        if (choice.name === val) {
                          found = true;
                          ev.value.selected = choice;
                        }
                      });
                      // Add custom item
                      if (!found) {
                        var newval = { name: val };
                        ev.value.choices.push(newval);
                        ev.value.selected = newval;
                      }
                    }
                  });
                },
                'SINGLE_SELECT': function (lev, ev) {
                  _.forEach(ret.eventValues, function (ev) {
                    if (ev.type === 'SINGLE_SELECT' && lev.eventTypeTemplate.id == ev.ett.id) {
                      var found = false;
                      _.forEach(ev.value.choices, function (choice) {
                        if (choice.name === lev.value) {
                          ev.value.selected = choice;
                          found = true;
                        }
                      });
                      // Add custom item
                      if (!found) {
                        var newval = { name: lev.value };
                        ev.value.choices.push(newval);
                        ev.value.selected = newval;
                      }
                    }
                  });
                },
                'TIME_INPUT': function (lev, ev) {
                  _.forEach(ret.eventValues, function (ev) {
                    if (ev.type === 'TIME_INPUT' && lev.eventTypeTemplate.id == ev.ett.id) {
                      ev.value = lev.value;  //$utils.getTimePart(new Date(lev.value));
                    }
                  });
                }
              };
            _.forEach(legacyEvent.eventValues, function (lev) {
              var op = ops[lev.eventTypeTemplate.templateType] || $utils.noop;
              op(lev);
            });
          }
          populateEvent();
          ret.observed.push(legacyEvent.child);
          ret.observers.push(legacyEvent.user);
          ret.notes = legacyEvent.notes;
          ret.id = legacyEvent.id;
          ret.timestamp = new Date(legacyEvent.timeOccurred);
          ret.links = legacyEvent.links;
          ret.images = extractMedia(legacyEvent, 'PHOTO');
          ret.videos = extractMedia(legacyEvent, 'VIDEO');
          if (legacyEvent.images) {
            _.forEach(legacyEvent.images, function (img) {
              if (_.filter(ret.images, function (i) {
                  return i.key == img.key;
                }).length == 0) {
                ret.images.push(img);
              }
            });
          }
          if (legacyEvent.videos) {
            _.forEach(legacyEvent.videos, function (img) {
              if (_.filter(ret.videos, function (i) {
                  return i.key == img.key;
                }).length == 0) {
                ret.videos.push(img);
              }
            });
          }
          ret.clientKey = legacyEvent.clientKey;
          ret.draftId = legacyEvent.draftId;
          ret.properties = legacyEvent.properties || {};
          ret.___id = legacyEvent.___id;
          if (legacyEvent.hasOwnProperty('stateStandards')) {
            ret.stateStandards = legacyEvent.stateStandards;
          }
          if (legacyEvent.hasOwnProperty('eventStateStandards')) {
            ret.stateStandards = legacyEvent.eventStateStandards;
          }
          if (legacyEvent.hasOwnProperty('showAbsentKids')) {
            ret.showAbsentKids = legacyEvent.showAbsentKids;
          }
          return ret;
        },
        buildEventValues: function (eventValues) {
          var ret = [];
          function buildEventValue(eventValue) {
            var ops = {
                'MULTI_SELECT': function () {
                  _.forEach(eventValue.value.choices, function (tv) {
                    if (tv.selected) {
                      ret.push({
                        eventTypeTemplate: {
                          id: eventValue.ett.id,
                          templateType: eventValue.ett.templateType,
                          name: eventValue.ett.name
                        },
                        value: tv.name
                      });
                    }
                  });
                },
                'NUMERIC_SELECT': function () {
                  var amt = eventValue.value.amount;
                  if (amt && amt != '') {
                    ret.push({
                      eventTypeTemplate: {
                        id: eventValue.ett.id,
                        templateType: eventValue.ett.templateType,
                        name: eventValue.ett.name
                      },
                      value: eventValue.value.amount + ' ' + eventValue.value.numericUnits + ' ' + eventValue.value.selected.name
                    });
                  }
                },
                'SINGLE_SELECT': function () {
                  ret.push({
                    eventTypeTemplate: {
                      id: eventValue.ett.id,
                      templateType: eventValue.ett.templateType,
                      name: eventValue.ett.name
                    },
                    value: eventValue.value.selected.name
                  });
                },
                'TIME_INPUT': function () {
                  ret.push({
                    eventTypeTemplate: {
                      id: eventValue.ett.id,
                      templateType: eventValue.ett.templateType,
                      name: eventValue.ett.name
                    },
                    value: eventValue.value ? eventValue.value.getTime ? $utils.getMilTime($utils.getTimePart(eventValue.value)) : $utils.getMilTime(eventValue.value) : null
                  });
                }
              };
            var f = ops[eventValue.type] || function () {
              };
            f();
          }
          _.forEach(eventValues, buildEventValue);
          return ret;
        },
        createLegacyEvent: function (event, fixUp) {
          function buildMediaKeys(event) {
            var keys = [];
            function extractKey(media) {
              var obj = {};
              // media.new ? {} : { 'id' : media.id };
              keys.push(angular.extend({ 'key': media.key }, obj));
            }
            _.forEach(event.images, extractKey);
            _.forEach(event.videos, extractKey);
            //console.log(keys);
            return keys;
          }
          var obj = {
              notes: event.notes,
              timeOccurred: typeof event.timestamp == Date ? $utils.createGsonCompatibleTimestamp(event.timestamp) : event.timestamp,
              dateOccurred: 'Ignored',
              user: {
                id: event.observers[0].id,
                firstName: event.observers[0].firstName,
                lastName: event.observers[0].lastName
              },
              eventType: {
                id: event.eventType.id,
                name: event.eventType.name
              },
              eventValues: _this.buildEventValues(event.eventValues),
              id: event.id || 0,
              mediaList: buildMediaKeys(event),
              child: { id: event.observed.length > 1 || !event.observed[0] ? 0 : event.observed[0].id },
              images: event.images,
              videos: event.videos,
              links: event.links
            };
          if (event.hasOwnProperty('stateStandards')) {
            obj.stateStandards = event.stateStandards;
          }
          if (event.hasOwnProperty('showAbsentKids')) {
            obj.showAbsentKids = event.showAbsentKids;
          }
          if (event.portfolio) {
            obj.portfolio = true;
          }
          obj.clientKey = event.clientKey;
          obj.draftId = event.draftId;
          obj.properties = event.properties || {};
          return obj;
        },
        stateStandardsToApiModel: function (standards) {
          var retVal = [];
          standards.forEach(function (standard) {
            retVal.push({
              'depth_level': standard.depth_level || standard.depthLevel,
              'domain_guid': standard.domain_guid || standard.domainGuid,
              'domain_number': standard.domain_number || standard.domainNumber,
              'domain_text': standard.domain_text || standard.domainText,
              'subdomain_guid': standard.subdomain_guid || standard.subDomainGuid,
              'subdomain_number': standard.subdomain_number || standard.subDomainNumber,
              'subdomain_text': standard.subdomain_text || standard.subDomainText,
              'depth_three_guid': standard.depth_three_guid || standard.depthThreeGuid,
              'depth_three_number': standard.depth_three_number || standard.depthThreeNumber,
              'depth_three_text': standard.depth_three_text || standard.depthThreeText,
              'set': standard.set,
              'age_group': standard.age_group || standard.ageGroup,
              'guid': standard.guid,
              'standard_text': standard.standard_text || standard.standardText,
              'standard_number': standard.standard_number || standard.standardNumber,
              'adopted': standard.adopted,
              'modified': standard.modified
            });
          });
          return retVal;
        },
        stateStandardsToPlayModel: function (standards) {
          var retVal = [];
          standards.forEach(function (standard) {
            retVal.push({
              'depthLevel': standard.depth_level || standard.depthLevel,
              'domainGuid': standard.domain_guid || standard.domainGuid,
              'domainNumber': standard.domain_number || standard.domainNumber,
              'domainText': standard.domain_text || standard.domainText,
              'subDomainGuid': standard.subdomain_guid || standard.subDomainGuid,
              'subDomainNumber': standard.subdomain_number || standard.subDomainNumber,
              'subDomainText': standard.subdomain_text || standard.subDomainText,
              'depthThreeGuid': standard.depth_three_guid || standard.depthThreeGuid,
              'depthThreeNumber': standard.depth_three_number || standard.depthThreeNumber,
              'depthThreeText': standard.depth_three_text || standard.depthThreeText,
              'set': standard.set,
              'ageGroup': standard.age_group || standard.ageGroup,
              'guid': standard.guid,
              'standardText': standard.standard_text || standard.standardText,
              'standardNumber': standard.standard_number || standard.standardNumber,
              'adopted': standard.adopted,
              'modified': standard.modified
            });
          });
          return retVal;
        },
        computeAttendance: function (kidlist, events) {
          var kids = _this.filterByPresent(kidlist);
          var count = 0;
          var isSet = kids && kids.length > 0 ? kids[0].attendanceSet : false;
          _.forEach(kids, function (kid) {
            var last = null;
            // get the events for this child, then iterate over them
            _.filter(events, function (e) {
              return e.child.id === kid.id;
            }).forEach(function (evt) {
              var evtType = $facility.findEventType(evt.eventType.id);
              if (evtType && evtType.name === 'Take Attendance') {
                if (last == null || last.timeOccurred < evt.timeOccurred) {
                  last = evt;
                }
              }
            });
            // use in mem val first, then check event.
            count += isSet && kid.isIn || last != null && last.eventValues && last.eventValues.length > 0 && last.eventValues[0].value == 'In' ? 1 : 0;
          });
          return {
            count: count,
            isSet: isSet
          };
        }
      };
    return _this;
  }
]);
'use strict';
angular.module('kidreports2App');
angular.module('kidreports2App').factory('$appstate', [
  '$prefs',
  '$timeout',
  '$backend',
  '$events',
  '$localdb',
  '$state',
  '$rootScope',
  '$translate',
  '$utils',
  '$adapter',
  '$krCache',
  '$facility',
  '$auth',
  '$modal',
  '$native',
  '$kidplanner',
  '$krConfig',
  '$ajax',
  '$messageBox',
  '$nav',
  '$eventAdapter',
  '$model',
  '$filters',
  '$msg',
  function ($prefs, $timeout, $backend, $events, $localdb, $state, $rootScope, $translate, $utils, $adapter, $krCache, $facility, $auth, $modal, $native, $kidplanner, $krConfig, $ajax, $messageBox, $nav, $eventAdapter, $model, $filters, $msg) {
    // AngularJS will instantiate a singleton by calling "new" on this function
    var _this = {
        toggleStreamFilter: function () {
          if ($model.streamFilter == 'NONE')
            $model.streamFilter = 'MEDIA';
          else if ($model.streamFilter == 'MEDIA')
            $model.streamFilter = 'ALL';
          else
            $model.streamFilter = 'NONE';
        },
        eventAdapter: $eventAdapter,
        model: $model,
        filters: $filters,
        message: $msg,
        getEventsByGroup: function () {
          return $eventAdapter.getEventsByGroup();
        },
        getEventsByDate: function () {
          return $eventAdapter.getEventsByDate();
        },
        getEventsByChild: function () {
          return $eventAdapter.getEventsByChild();
        },
        refreshEvents: function (kids, date, force, updateAttendance, days, cb, keepOldEvents, filter) {
          return $eventAdapter.refreshEvents(kids, date, force, updateAttendance, days, cb, keepOldEvents, filter);
        },
        fixKid: function (items, kid, onlyCopy) {
          return $eventAdapter.fixKid(items, kid, onlyCopy);
        },
        viewDraft: function (draft, isParent) {
          $eventAdapter.viewDraft(draft, isParent);
        },
        showEvent: function (item) {
          $eventAdapter.showEvent(item);
        },
        canEdit: function (item) {
          return $eventAdapter.canEdit(item);
        },
        viewEvent: function (item, isParent, skipCreate) {
          $eventAdapter.viewEvent(item, isParent, skipCreate);
        },
        viewEventObservations: function (item) {
          $eventAdapter.viewEventObservations(item);
        },
        goToMemories: function () {
          $state.go('memories');
        },
        isParent: function () {
          return $backend.getCurrentUser() && $backend.getCurrentUser().role === 'PARENT_ROLE';
        },
        getHelpPath: function (p) {
          p = p || 'main';
          return 'views/help/' + p + '.html';
        },
        setModalMessage: function (msg) {
          $msg.setModalMessage(msg);
        },
        msg: function (msg, show, msgType, timer) {
          $msg.msg(msg, show, msgType, timer);
        },
        hideMsg: function () {
          $msg.hideMsg();
        },
        isCacheEmpty: function () {
          return !($events.queueSize() > 0 || $model.cachedMedia && $model.cachedMedia.length > 0 || $model.cachedItems && $model.cachedItems.length > 0);
        },
        cacheSize: function () {
          return $events.queueSize() + _this.mediaQueueSize() + _this.backendQueueSize();
        },
        eventQueueSize: function () {
          return $events.queueSize();
        },
        mediaQueueSize: function () {
          return $model.cachedMedia && $model.cachedMedia.length ? $model.cachedMedia.length : 0;
        },
        backendQueueSize: function () {
          return $model.cachedItems && $model.cachedItems.length ? $model.cachedItems.length : 0;
        },
        getCache: function () {
          return $events.queue().concat($model.cachedMedia || []).concat($model.cachedItems || []);
        },
        configureRooms: function () {
          var oldRm = $facility.findRoom($prefs.current.room);
          var rooms = _.filter($facility.getSync().rooms, function (item) {
              return !(item.props && item.props['moveRoomsOnly'] === 'true');
            });
          var rm = oldRm || rooms[0];
          $model.currentRoom = rm.name;
          if (!$model.selectedRoom)
            _this.setCurrentRoom(rm);
          $model.eventList = [];
          var parent = $backend.getCurrentUser().role == 'PARENT_ROLE';
          var kids = parent ? $facility.getSync().children : rm.children;
          if (parent) {
            $model.observed = kids;
          } else {
            $model.observed = rm.children;
          }
        },
        setCurrentRoom: function (item, skipRefresh) {
          if (!item)
            return;
          $facility.getSync().observers = item.teachers;
          $facility.getSync().observed = item.children;
          $facility.getSync().observed.forEach(function (item) {
            var icon = item.iconHtml;
            if (icon && icon != '') {
              icon = icon.match(/src='.*'/)[0];
              icon = icon.substring(5, icon.length - 1);
              item.iconUrl = icon;
            }
          });
          $model.selectedRoom = item;
          $model.selectedCategory = item.category;
          if (!skipRefresh) {
            _this.refreshEvents(item.children, null, false, true);
            $rootScope.$broadcast('selectedRoomChange');
          }
        },
        getVisibleChildren: function (includeAbsentIfSet, includeDashboardedChildren) {
          if (!$facility.getSync() || !$facility.getSync().observed) {
            return [];
          }
          var children = [];
          $facility.getSync().observed.forEach(function (child) {
            var shouldIncludeChild = $model.viewPortfolio || _this.isPresent(child, includeAbsentIfSet) || includeDashboardedChildren && child.showInDashboard;
            if (_this.doesMatchName(child) && shouldIncludeChild) {
              children.push(child);
            }
          });
          return children;
        },
        toggleEventListSort: function () {
          if (!$model.eventListDateSort) {
            $model.eventListSort = [
              'dateTimeOccurred',
              'child.lastName',
              'child.firstName'
            ];
          } else {
            $model.eventListSort = [
              'child.lastName',
              'child.firstName',
              'dateTimeOccurred'
            ];
          }
          $model.eventListDateSort = !$model.eventListDateSort;
        },
        checkEventFilter: function (ele) {
          return $filters.checkEventFilter(ele);
        },
        currentDate: function (evt) {
          return $filters.currentDate(evt);
        },
        isNapEvent: function (evt) {
          var timeCnt = 0;
          evt.eventValues.forEach(function (ev) {
            if (ev.type == 'TIME_INPUT' || ev.eventTypeTemplate && ev.eventTypeTemplate.templateType == 'TIME_INPUT')
              timeCnt++;
          });
          return timeCnt == 2;
        },
        refreshFacility: function () {
          $facility.logout().then($nav.resetToHome);
        },
        doesMatchName: function (item) {
          var m = $model.matchName ? $model.matchName.toUpperCase() : '';
          return m == '' || (item.firstName.toUpperCase().indexOf(m) != -1 || item.lastName.toUpperCase().indexOf(m) != -1);
        },
        isPresent: function (item, showAbsentIfSet) {
          return $model.showAbsentKids && showAbsentIfSet || _this.isParent() || !_this.isAbsent(item);
        },
        isAbsent: function (item) {
          return !((item.attendanceSet || item.cachedAttendance) && item.isIn == true || !item.attendanceSet && !item.cachedAttendance && item.isIn == null);
        },
        checkAttendance: function (roomID, cb, item) {
          var planInSession = $kidplanner.planInSession;
          function hasTakenAttendance(roomId) {
            //var room = $facility.findRoom(roomId);
            var found = false;
            var children = $facility.getSync().observed;
            children.forEach(function (item) {
              found = found || (item.attendanceSet || item.cachedAttendance) && item.isIn == true;
            });
            return found;
          }
          function missingKidsForEvent(et) {
            // Parents are allowed to add activities regardless of required first activities.
            if ($backend.getCurrentUser() && $backend.getCurrentUser().role === 'PARENT_ROLE') {
              return -1;
            }
            //var room = $facility.findRoom(roomId);
            var missing = [];
            var children = $model.observed;
            children.forEach(function (item) {
              var id = lookupRequiredET(item);
              var match = !id || id != et.id || _.filter($eventAdapter.getEventsForChild(item.id), function (i) {
                  return i.eventType.name == et.name;
                }).length;
              if (!match)
                missing.push(item);
            });
            return missing;
          }
          function lookupRequiredET(i) {
            var _id = -1;
            if (i.homeRoom && i.homeRoom.category) {
              _id = i.homeRoom.category.initialRequiredActivityId;
            } else if (i.category != null) {
              _id = i.category.initialRequiredActivityId;
            }
            if (_id == null)
              _id = -1;
            return _id;
          }
          var _cb = function () {
            if (planInSession) {
              cb();
              return;
            }
            var children = $model.observed;
            var requiredEventIds = _.map(children, lookupRequiredET);
            //var requiredEventID = item.categories && item.categories[0] && item.categories[0].initialRequiredActivityId;
            //alert(JSON.stringify(requiredEventIds));
            if (requiredEventIds && requiredEventIds.length) {
              //var found = false;
              for (var i = 0; i < requiredEventIds.length; i++) {
                var requiredEventID = requiredEventIds[i];
                var evtType = $facility.findEventType(requiredEventID);
                if (evtType) {
                  // todo check if kids have this
                  var missingKids = missingKidsForEvent(evtType);
                  if (missingKids.length) {
                    $utils.alert('Please record a ' + evtType.name + ' activity before doing any room movements or recording events.');
                    $model.observed = missingKids;
                    cb(evtType);
                    return;
                  }
                }
              }
            }
            cb();
          };
          var attendanceMode = $facility.getSync().attendanceWarningMode;
          var attendanceTaken = hasTakenAttendance(roomID);
          if (!planInSession && attendanceMode && attendanceMode != 'NO_WARNING' && !attendanceTaken) {
            if (attendanceMode == 'WARN') {
              $messageBox.open($translate.instant('AttendanceWarning'), 'Attendance has not yet been taken. Please take attendance before doing any room movements or recording events.  Continue anyways?', {
                oktext: $translate.instant('Yes'),
                canceltext: $translate.instant('No')
              }).result.then(_cb);
            } else {
              $messageBox.open('Attendance Warning', 'Please take attendance before doing any room movements or recording events.', { oktext: 'Ok' }, null, 'img/icon/flag.svg');
            }
          } else {
            _cb();
          }
        },
        togglePortfolio: function () {
          $model.viewPortfolio = !$model.viewPortfolio;
        },
        profilePlannerEnabled: function () {
          var enabled = false;
          $model.menuItems.forEach(function (mi) {
            enabled = enabled || mi.path.root === 'profilePlanner';
          });
          return enabled;
        },
        goHome: function (directState, wasClick) {
          $nav.goHome(directState, wasClick);
        },
        getClass: function (evt) {
          var et = evt.header ? null : $facility.findEventType(evt.eventType.id);
          if (et && et.actionRequired && et.actionRequired == true && (et.streamColor == null || et.streamColor == ''))
            return 'actionRequired';
          else
            return '';
        },
        getBGStyle: function (evt) {
          //return 'red';
          var et = evt.header ? null : $facility.findEventType(evt.eventType.id);
          if (et && et.actionRequired && et.actionRequired == true && et.streamColor != null && et.streamColor != '')
            return et.streamColor;
          else
            return '';
        },
        getTextStyle: function (evt) {
          //return 'red';
          var et = evt.header ? null : $facility.findEventType(evt.eventType.id);
          if (et && et.actionRequired && et.actionRequired == true && et.streamTextColor != null && et.streamTextColor != '')
            return et.streamTextColor;
          else
            return '';
        },
        getGroupColor: function (group) {
          if (group == 'Administrative')
            return '#D17265';
          else if (group == 'DailyRoutine')
            return '#FFC32D';
          else if (group == 'Meals')
            return '#8AA604';
          else if (group == 'Portfolio')
            return '#886793';
          else
            return '#4864B7';
        },
        getGroupIcon: function (group, circle) {
          var circleStr = circle ? '-circle' : '';
          if (group == 'Administrative')
            return 'img/icon/group/notes' + circleStr + '.svg';
          else if (group == 'DailyRoutine')
            return 'img/icon/group/dailyroutine' + circleStr + '.svg';
          else if (group == 'Meals')
            return 'img/icon/group/meal' + circleStr + '.svg';
          else if (group == 'Portfolio')
            return 'img/icon/group/learning' + circleStr + '.svg';
          else
            return 'img/icon/group/notes.svg';
        },
        setFocused: function (t) {
          $model.focused = t;
        },
        getRatioBackground: function () {
          return 'none';
        },
        hideHeader: function () {
          return this.hideFrame ? 'noPaddingTop' : '';
        },
        nav: $nav
      };
    _this.toggleEventListSort();
    return _this;
  }
]);
'use strict';
angular.module('kidreports2App').directive('customDragArea', function () {
  return {
    restrict: 'AE',
    link: function (scope, element, attrs) {
    }
  };
});
'use strict';
angular.module('kidreports2App').directive('drawing', [
  '$model',
  function ($model) {
    function draw(event) {
    }
    function endDraw(event) {
    }
    return {
      restrict: 'A',
      link: function (scope, element) {
        $model.signaturePad = new SignaturePad(element[0]);
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').directive('appVersion', [
  'version',
  function (version) {
    return function (scope, elm, attrs) {
      elm.text(version);
    };
  }
]);
'use strict';
angular.module('kidreports2App').directive('ngBlur', [
  '$parse',
  function ($parse) {
    return function (scope, element, attr) {
      var fn = $parse(attr['ngBlur']);
      element.bind('blur', function (event) {
        scope.$apply(function () {
          fn(scope, { $event: event });
        });
      });
    };
  }
]);
'use strict';
angular.module('kidreports2App').directive('ngFocus', [
  '$parse',
  function ($parse) {
    return function (scope, element, attr) {
      var fn = $parse(attr['ngFocus']);
      element.bind('focus', function (event) {
        scope.$apply(function () {
          fn(scope, { $event: event });
        });
      });
    };
  }
]);
'use strict';
angular.module('kidreports2App').directive('renderEventRow', [
  '$popup',
  '$prefs',
  '$backend',
  function ($popup, $prefs, $backend) {
    return {
      restrict: 'E',
      scope: {
        item: '=',
        func: '='
      },
      templateUrl: 'views/directives/renderEventRow.html',
      link: function (scope, element, attrs, $scope) {
        scope.invoke = function () {
          scope.func.call(null, scope.item);
        };
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').directive('renderChild', [
  '$popup',
  '$prefs',
  '$facility',
  '$backend',
  '$appstate',
  '$events',
  '$timeout',
  function ($popup, $prefs, $facility, $backend, $appstate, $events, $timeout) {
    return {
      restrict: 'E',
      scope: {
        rooms: '=',
        item: '=',
        func: '=',
        attendanceFunc: '=',
        isShowAll: '='
      },
      templateUrl: 'views/directives/renderChild.html',
      link: function (scope, element, attrs, $scope) {
        $events.getEvents([scope.item], null, function (err, data) {
          if (!err)
            scope.itemevents = data.events;
        });
        scope.invoke = function () {
          if (scope.func && !scope.showAttendance) {
            scope.func.call(null, scope.item);
          }
        };
        scope.invokeAttendanceFunc = function () {
          if (scope.attendanceFunc) {
            scope.attendanceFunc.call(null, scope.item);
            event.preventDefault();
            event.cancelBubble = true;
          }
        };
        scope.showProfile = function () {
          if (!scope.isParent) {
            $popup.showProfile(scope.item);
            event.preventDefault();
            event.cancelBubble = true;
          }
        };
        scope.checkEventFilter = function (ele) {
          var keep = !ele.eventType.hidden && $prefs.matchesQuickViewFilter(ele);
          return keep;
        };
        scope.shared = $appstate;
        scope.item.isInDisplay = scope.item.isIn;
        scope.showAttendance = scope.$eval(attrs['showattendance']);
        scope.prefs = $prefs.current;
        scope.roomname = $facility.findRoom(scope.item.roomId).name;
        scope.isParent = $backend.getCurrentUser().role == 'PARENT_ROLE';
        var selectedRoomId = $facility.getSync().consolidatingRoomId || $appstate.model.selectedRoom.id;
        scope.item.selectedChildRoom = _.filter(scope.rooms, function (rm) {
          return rm.id == selectedRoomId;
        })[0];
        scope.isSigningIn = function () {
          return scope.isShowAll && (!scope.item.current.isIn && (scope.item.isIn == true || scope.item.isIn == null));
        }  //if ( scope.showAttendance ) {
           //  $timeout(function () {
           //    $("[name='toggleCheckbox-" + scope.item.id + "']").bootstrapSwitch();
           //    $("[name='toggleCheckbox-" + scope.item.id + "']").bootstrapSwitch('state', scope.item.isInDisplay);
           //    $("[name='toggleCheckbox-" + scope.item.id + "']").bootstrapSwitch('size', 'large');
           //    $("[name='toggleCheckbox-" + scope.item.id + "']").on('switchChange.bootstrapSwitch', function(event, state) {
           //      $timeout(function() {$appstate.toggleIn(scope.item) }, 500);
           //    });
           //  }, 10);
           //}
;
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').directive('eventDetail', function () {
  return {
    restrict: 'E',
    scope: {
      bind: '=',
      disabled: '='
    },
    link: function (scope, element, attrs) {
      scope.bind(element);
    }
  };
});
'use strict';
angular.module('kidreports2App').directive('ngEnter', function () {
  return function (scope, element, attrs) {
    element.bind('keydown keypress', function (event) {
      if (event.which === 13) {
        scope.$apply(function () {
          scope.$eval(attrs.ngEnter);
        });
        event.preventDefault();
      }
    });
  };
});
'use strict';
angular.module('kidreports2App').directive('continue', function () {
  return {
    restrict: 'A',
    scope: { item: '=' },
    templateUrl: 'views/directives/continue.html',
    link: function ($scope) {
      $scope.onClick = function (choice) {
        $scope.item.execute();
      };
    }
  };
});
'use strict';
angular.module('kidreports2App').directive('singleSelect', [
  '$messageBox',
  '$facility',
  function ($messageBox, $facility) {
    return {
      restrict: 'A',
      scope: {
        item: '=',
        disabled: '='
      },
      templateUrl: 'views/directives/singleSelect.html',
      link: function ($scope) {
        $scope.onAdd = function (name) {
          if (name && name.length > '' && _.filter($scope.item.value.choices, function (c) {
              return c.name == name;
            }).length == 0) {
            if (name.length > $facility.getSync().maxValueLength) {
              $messageBox.open('Warning', 'Additional values should not be longer than ' + $facility.getSync().maxValueLength + ' characters.  Please use the Notes field for longer comments.', { oktext: 'Ok' });
              return;
            }
            var opt = { name: name };
            $scope.item.value.choices.push(opt);
            $scope.item.value.selected = opt;
            $scope.item.additional = '';
          }
        };
        $scope.allowFreeTxt = $scope.item.ett.allowFreeTxt;
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').directive('multiSelect', [
  '$messageBox',
  '$facility',
  function ($messageBox, $facility) {
    return {
      restrict: 'A',
      scope: {
        item: '=',
        disabled: '=',
        focusfunc: '='
      },
      templateUrl: 'views/directives/multiSelect.html',
      link: function ($scope) {
        $scope.onAdd = function (name) {
          if (name && name.length > '' && _.filter($scope.item.value.choices, function (c) {
              return c.name == name;
            }).length == 0) {
            if (name.length > $facility.getSync().maxValueLength) {
              $messageBox.open('Warning', 'Additional values should not be longer than ' + $facility.getSync().maxValueLength + ' characters.  Please use the Notes field for longer comments.', { oktext: 'Ok' });
              return;
            }
            var opt = {
                name: name,
                selected: true
              };
            $scope.item.value.choices.push(opt);
            $scope.item.additional = '';
          }
        };
        $scope.setFocused = function (b) {
          if ($scope.focusfunc)
            $scope.focusfunc.call(null, b);  //            console.log(b);
        };
        $scope.allowFreeTxt = $scope.item.ett.allowFreeTxt;
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').directive('numericSelect', [
  '$messageBox',
  '$facility',
  function ($messageBox, $facility) {
    return {
      restrict: 'A',
      scope: {
        item: '=',
        disabled: '=',
        focusfunc: '='
      },
      templateUrl: 'views/directives/numericSelect.html',
      link: function ($scope) {
        $scope.onAdd = function (name) {
          if (name && name.length > '' && _.filter($scope.item.value.choices, function (c) {
              return c.name == name;
            }).length == 0) {
            if (name.length > $facility.getSync().maxValueLength) {
              $messageBox.open('Warning', 'Additional values should not be longer than ' + $facility.getSync().maxValueLength + ' characters.  Please use the Notes field for longer comments.', { oktext: 'Ok' });
              return;
            }
            var opt = { name: name };
            $scope.item.value.choices.push(opt);
            $scope.item.value.selected = opt;
            $scope.item.additional = '';
          }
        };
        $scope.setFocused = function (b) {
          if ($scope.focusfunc)
            $scope.focusfunc.call(null, b);  //            console.log(b);
        };
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').directive('timeInput', [
  '$utils',
  '$native',
  '$model',
  function ($utils, $native, $model) {
    return {
      restrict: 'A',
      scope: {
        item: '=',
        disabled: '='
      },
      templateUrl: 'views/directives/timeInput.html',
      link: function (scope, element, attrs, ngModel) {
        scope.isAndroid502 = function () {
          return $model.disableNativeTimepicker || $native.isAndroid502();
        };
        var convertToDate = function (value) {
          var d = $utils.getDatePart(new Date());
          var date = $utils.createDateFromDTStrings(d, value || $utils.getTimePart(new Date()));
          date.setSeconds(0);
          date.setMilliseconds(0);
          return date;
        };
        if (scope.isAndroid502()) {
          scope.item.value = convertToDate(scope.item.value);
        }
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').directive('focusMe', [
  '$timeout',
  function ($timeout) {
    return {
      scope: { trigger: '@focusMe' },
      link: function (scope, element) {
        scope.$watch('trigger', function (value) {
          if (value === 'true') {
            $timeout(function () {
              element[0].focus();
            });
          }
        });
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').directive('nativeDateElement', [
  '$utils',
  function ($utils) {
    return {
      restrict: 'A',
      replace: true,
      require: 'ngModel',
      template: '<input type="date"/>',
      link: function (scope, element, attrs, ngModel) {
        //format text going to user (model to view)
        //convert 2013-09-29 to Date() object
        ngModel.$formatters.push(function (value) {
          return $utils.createDateFromDTStrings(value, '00:00:00');
        });
        //format text from the user (view to model)
        ngModel.$parsers.push(function (value) {
          return $utils.getDatePart(value);
        });
        element.bind('blur', function (event) {
          ngModel.$setViewValue(element.val());
        });
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').directive('nativeTimeElement', [
  '$native',
  '$utils',
  '$timeout',
  function ($native, $utils, $timeout) {
    return {
      restrict: 'A',
      replace: true,
      require: 'ngModel',
      template: function (element, attrs) {
        var htmlText = '<input type="time"/>';
        if ($utils.isAndroid5()) {
          htmlText = '<input type="datetime-local"/>';
        }
        return htmlText;
      },
      link: function (scope, element, attrs, ngModel) {
        scope.clear = function () {
          ngModel.$setViewValue(null);
        };
        function fixTimestring(valueFromModel) {
          if (valueFromModel && valueFromModel.indexOf(':') > -1) {
            var arr = valueFromModel.split(':');
            if (arr.length > 3) {
              // error, got a string like this 16:2512:20:00 where the first 5 digits are the true time
              valueFromModel = arr[0] + ':' + arr[1].substring(0, 2) + ':00';
              arr = valueFromModel.split(':');
            }
            if (arr.length <= 3 && arr[0].length == 1) {
              valueFromModel = '0' + valueFromModel;
            }
            // expected
            if (arr.length == 2)
              valueFromModel = valueFromModel + ':00';
          }
          return valueFromModel;
        }
        element.bind('blur', function (event) {
          scope.$apply(function () {
            var newval = fixTimestring(element.val());
            //newval = $utils.getTimeString(newval);
            ngModel.$setViewValue(newval);
          });
        });
        if ($native.android()) {
          ngModel.$formatters.unshift(function (valueFromModel) {
            // TODO update this once android supports AM/PM time.
            //console.log("valueFromModel:"+valueFromModel)
            valueFromModel = fixTimestring(valueFromModel);
            //console.log("fixedValueFromModel:"+valueFromModel)
            return valueFromModel;
          });
        }
        //format text going to user (model to view)
        //convert 2013-09-29 to Date() object
        ngModel.$formatters.push(function (value) {
          var d = $utils.getDatePart(new Date());
          var date = $utils.createDateFromDTStrings(d, value);
          date.setSeconds(0);
          date.setMilliseconds(0);
          //console.log('model to view');
          //console.log(d);
          //console.log(date);
          return date;
        });
        //format text from the user (view to model)
        ngModel.$parsers.push(function (value) {
          var t = $utils.getTimePart(value);
          //console.log('view to model');
          //console.log(value);
          //console.log(t);
          //t = $utils.getTimeString(t);
          return t;
        });
        // A bit of a hack - we need a way to intercept the actual dom attach event
        //if ($native.type() == "ANDROID") {
        $timeout(function () {
          if (!$utils.isAndroid5())
            $native.enhanceUI();
        }, 1000);
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').directive('materialDatepicker', [
  '$utils',
  function ($utils) {
    return {
      restrict: 'AE',
      replace: false,
      require: 'ngModel',
      link: function (scope, element, attrs, ngModel) {
        //format text going to user (model to view)
        //convert 2013-09-29 to Date() object
        ngModel.$formatters.push(function (value) {
          return $utils.createDateFromDTStrings(value, '00:00:00');
        });
        //format text from the user (view to model)
        ngModel.$parsers.push(function (value) {
          return $utils.getDatePart(value);
        });
        element.bind('blur', function (event) {
          ngModel.$setViewValue(element.val());
        });
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').directive('imageElement', function () {
  return {
    restrict: 'A',
    replace: true,
    template: '<img class="invisible"></img>',
    scope: { func: '=' },
    link: function (scope, element, attrs) {
      if (attrs.width) {
        element.attr('width', attrs.width);
      }
      scope.func = function (src) {
        element.removeClass('invisible');
        element[0].src = src;
      };
    }
  };
});
'use strict';
angular.module('kidreports2App').directive('dynamicUrl', function () {
  return {
    restrict: 'A',
    link: function postLink(scope, element, attrs) {
      function update() {
        if (scope.vid && scope.vid.url)
          element.attr('src', scope.vid.url);
      }
      scope.$watch('scope.vid', function () {
        update();
      });
      update();
    }
  };
}).directive('dynamicPoster', function () {
  return {
    restrict: 'A',
    link: function postLink(scope, element, attrs) {
      function update() {
        if (scope.vid && scope.vid.thumbnail)
          element.attr('poster', scope.vid.thumbnail);
      }
      scope.$watch('scope.vid', function () {
        update();
      });
      update();
    }
  };
});
'use strict';
angular.module('kidreports2App').directive('videoElement', function () {
  return {
    restrict: 'A',
    replace: true,
    template: '<video class="invisible" src=""></video>',
    scope: { func: '=' },
    link: function (scope, element, attrs) {
      if (attrs.width) {
        element.attr('width', attrs.width);
      }
      scope.func = function (src) {
        element.removeClass('invisible');
        element[0].src = src;
        element[0].play();
      };
    }
  };
});
'use strict';
angular.module('kidreports2App').factory('$eventFactory', function () {
  return {
    buildEventValue: function (name, type) {
      var obj = {};
      obj.name = name;
      obj.type = type;
      obj.value = null;
      return obj;
    },
    buildEvent: function (name) {
      var obj = {};
      obj.name = name;
      obj.timestamp = new Date();
      obj.notes = '';
      obj.observers = [];
      obj.observed = [];
      obj.eventValues = [];
      obj.images = [];
      obj.videos = [];
      return obj;
    }
  };
});
'use strict';
angular.module('kidreports2App').controller('SideCtrl', [
  '$scope',
  '$appstate',
  '$backend',
  '$state',
  function ($scope, $appstate, $backend, $state) {
    $scope.onClick = function (item) {
      //      snapRemote.getSnapper().then(function(snapper) {
      //        snapper.close();
      //      });
      $appstate.nav.icon = item.iconUrl;
      $appstate.nav.startPath(item.path.root);
    };
    var u = $backend.getCurrentUser();
    $scope.name = u.firstName + ' ' + u.lastName;
    $scope.checkItem = function (item) {
      if ($state.includes('home.dash') && !item.sideWhenDash)
        return false;
      return true;
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('MessageBoxController', MessageBoxController);
MessageBoxController.$inject = [
  '$scope',
  '$modalInstance',
  'model',
  '$appstate'
];
function MessageBoxController($scope, $modalInstance, model, $appstate) {
  $scope.iconPath = model.iconPath || 'img/icon/info.svg';
  $scope.title = model.title;
  $scope.message = model.message;
  $scope.buttons = model.buttons;
  $scope.oktext = model.options.oktext;
  $scope.canceltext = model.options.canceltext || null;
  $scope.showOneTimeCb = model.options.showOneTimeCb;
  $scope.oneTimeCbVal = { val: false };
  $appstate.model.freezeBackButton = true;
  $scope.ok = function (response) {
    $appstate.model.freezeBackButton = false;
    $modalInstance.close(model.options.showOneTimeCb ? $scope.oneTimeCbVal.val : true);
  };
  $scope.cancel = function (response) {
    $appstate.model.freezeBackButton = false;
    $modalInstance.dismiss('cancel');
  };
}
'use strict';
angular.module('kidreports2App').factory('$messageBox', [
  '$modal',
  function ($modal) {
    return {
      open: function (title, message, modalOptions, question, iconPath) {
        var options = {
            templateUrl: 'views/popups/messageBox.html',
            controller: 'MessageBoxController',
            resolve: {
              model: function () {
                return {
                  iconPath: iconPath,
                  title: title,
                  message: {
                    text: message,
                    question: question
                  },
                  options: modalOptions
                };
              }
            }
          };
        if (modalOptions)
          angular.extend(options, modalOptions);
        return $modal.open(options);
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').factory('$pbnj', [
  '$ajax',
  function ($ajax) {
    var queue = [];
    var uri = 'http://cdn.pbnjtv.com/pbjTVminiView.asmx';
    function process(data) {
      var x2js = new X2JS();
      var doc = x2js.parseXmlString(data);
      var cams = [];
      var camURLs = {};
      var camList = doc.getElementsByTagName('Cameras');
      for (var i = 0; i < camList.length; i++) {
        var cam = camList[i];
        var camurl, camname, camthumb;
        for (var j = 0; j < cam.childNodes.length; j++) {
          var el = cam.childNodes[j];
          if (el.nodeName == 'RoomName')
            camname = el.textContent;
          else if (el.nodeName == 'camURL')
            camurl = el.textContent;
          else if (el.nodeName == 'Thumb')
            camthumb = el.textContent;
        }
        if (camurl && camurl.length > 0 && camURLs[camurl] == null) {
          cams.push({
            url: camurl,
            name: camname,
            thumb: camthumb
          });
          camURLs[camurl] = true;
        }
      }
      return cams;
    }
    return {
      getCamList: function (user, pass, cb) {
        // todo pull into 3rd party svcs.
        //
        //        var sr =
        //          '<?xml version="1.0" encoding="utf-8"?>'+
        //            '<soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">   <soap:Body>'+
        //            '<pbjTV xmlns="pbnjtv"><pbjUN>'+user+'</pbjUN><TVpass>'+pass+'</TVpass></pbjTV></soap:Body>'+
        //            '</soap:Envelope>';
        //
        //        var opts = {method: 'POST', url: uri, params: {}, headers:{"Content-Type":'text/xml'}, data: sr, contentType: 'text/xml', preserveDataFormat: true};
        //        $ajax.ajax(opts, function(err, data) {
        //          if (err) {
        //            console.error("Error making request");
        //          } else {
        //            getURL(user, pass);
        //          }
        //        });
        getURL(user, pass);
        function getURL(user, pass) {
          var sr = '<?xml version="1.0" encoding="utf-8"?>' + '<soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"><soap:Body>' + '<GetCams xmlns="pbnjtv"><state>' + user + '</state><location>' + pass + '</location></GetCams></soap:Body>' + '</soap:Envelope>';
          var opts = {
              method: 'POST',
              url: uri,
              params: {},
              headers: { 'Content-Type': 'text/xml' },
              data: sr,
              contentType: 'text/xml',
              preserveDataFormat: true
            };
          $ajax.ajax(opts, function (err, data) {
            if (err) {
              console.error('Error making request');
            } else {
              cb(process(data));
            }
          });
        }
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').filter('isPresent', [
  '$appstate',
  function ($appstate) {
    return function (input) {
      if (input && _.isArray(input)) {
        return input.filter(function (element, index, array) {
          return $appstate.isPresent(element, true);
        });
      } else {
        return '';
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').filter('doesMatchName', [
  '$appstate',
  function ($appstate) {
    return function (input) {
      if (input && _.isArray(input)) {
        return input.filter(function (element, index, array) {
          return $appstate.doesMatchName(element);
        });
      } else {
        return '';
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').filter('startFrom', [
  '$appstate',
  function ($appstate) {
    return function (input, start) {
      return input.slice(start);
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('AttendancedashCtrl', [
  '$scope',
  '$appstate',
  function ($scope, $appstate) {
    $scope.close = function () {
      $appstate.close();
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('HelpCtrl', [
  '$scope',
  '$appstate',
  function ($scope, $appstate) {
    $scope.done = function () {
      $appstate.goHome();
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('ActivitystreamCtrl', [
  '$scope',
  '$appstate',
  '$backend',
  '$utils',
  '$adapter',
  '$state',
  '$facility',
  '$timeout',
  '$native',
  '$krConfig',
  'moment',
  '$rootScope',
  '$localdb',
  function ($scope, $appstate, $backend, $utils, $adapter, $state, $facility, $timeout, $native, $krConfig, moment, $rootScope, $localdb) {
    $scope.preventWriteActivity = $facility.getSync().preventWriteActivity;
    $scope.pageSize = 5;
    $scope.feedSelected = true;
    $scope.selectFeed = function (b) {
      $scope.feedSelected = b;
    };
    $scope.isLite = $facility.isLite;
    var isParent = $backend.getCurrentUser().role == 'PARENT_ROLE';
    $scope.isParent = isParent;
    // remove until we figure out fb issues
    //$scope.isNative = $native.isNative();
    $scope.isNative = $native.isNative();
    $scope.showFb = $facility.getSync().facebookEnabled;
    $scope.videoClick = function () {
      //event.preventDefault();
      //if (event ) event.cancelBubble=true;
      $scope.isVideoClick = true;
    };
    document.addEventListener('DOMContentLoaded', function () {
      $scope.height = window.innerHeight;
    }, false);
    $scope.showDash = $state.includes('home.dash');
    $scope.getClass = function (idx, item) {
      //      var str = idx % 2?'oddRow':'evenRow'
      //      str += " " + $appstate.getClass(item);
      //      return str;
      return $appstate.getClass(item);
    };
    $scope.isCollapsed = {};
    // Pagination in controller
    //var addItems = 30;
    $scope.getItemHeight = function (item, index) {
      //Make evenly indexed items be 10px taller, for the sake of example
      if (item && item.header)
        return 30;
      return 95;  //(index % 2) === 0 ? 50 : 60;
    };
    $scope.setCurrentPage = function (currentPage) {
      $scope.currentPage = currentPage;
    };
    //    setTimeout(function() {
    //      $scope.max = $appstate.eventList.length;
    //      var myEl = angular.element( document.querySelector( '#activityStreamScroll' ) )[0];
    //      $scope.currTop = myEl.scrollTop;
    //      myEl.onscroll = function(evt) {
    ////
    //        var w = myEl.height || myEl.clientHeight;
    //        var offset = parseInt(((w)/2), 10);
    ////
    //        // every time we scroll half page add 10 more
    //        if ( myEl.scrollTop - $scope.currTop > offset ) {
    //          //var add = $appstate.eventList.length - $scope.pageSize < 10 ? 10 : $scope.pageSize/2;
    //          $timeout(function(f) {
    //            $appstate.pageSize += 50;
    //          }, 0);
    //          $scope.currTop = myEl.scrollTop;
    //        }
    //      }
    //    }, 250);
    var today = new Date();
    var dayOffset = 1;
    var dayMultiple = isParent ? 1 : 1;
    var keepOld = !$state.includes('home.dash');
    var isFirstTime = true;
    // needed because for some reason, oldval and newval are equal the first time the below callback is run
    // The date scope variable needs to be watched because it is being modified from the DOM (via the directive)
    $scope.$watch('values.date', function (newval, oldval) {
      var isDateEqual = newval === oldval;
      newval += 'T12:00:00';
      var date = Date.fromISO(newval);
      //new Date(newval);
      if (isNaN(date))
        return;
      $scope.currDate = date;
      $appstate.model.currentDate = $scope.currDate;
      if (!isDateEqual || isFirstTime) {
        if (!isFirstTime)
          $localdb.set('roomMap', []);
        // invalidate roomMap
        isFirstTime = false;
        var kids = getObserved();
        if (kids) {
          $appstate.model.isRefreshingEvents = true;
          $appstate.refreshEvents(kids, date, true, false, null, function () {
            $appstate.model.isRefreshingEvents = false;
          }, false);
        }
      }
    });
    $scope.loadMore = function (future) {
      var m = -1;
      if (future)
        m = 1;
      $scope.pageSize += 5;
      if ($appstate.model.isSettingChild) {
        $timeout(function () {
          $scope.$broadcast('scroll.infiniteScrollComplete');
        }, 100);
        return;
      }
      updateDate(m * dayMultiple * dayMillis);
    };
    function getObserved() {
      if (isParent)
        return $appstate.model.observed;
      else
        return $facility.getSync().observed;
    }
    //    $scope.$on('stateChangeSuccess', function() {
    //      $scope.loadMore();
    //    });
    var dayMillis = 24 * 60 * 60 * 1000;
    $appstate.model.currentDate = $appstate.model.currentDate || new Date();
    function updateDate(offset) {
      $scope.values.date = $utils.getDatePart(new Date($appstate.model.currentDate.setTime($appstate.model.currentDate.getTime() + offset)));
      if ($native.android()) {
        var myEl = angular.element(document.querySelector('#native-date-element1'));
        var myEl2 = angular.element(document.querySelector('#native-date-element2'));
        myEl.removeAttr('disabled');
        myEl2.removeAttr('disabled');
        $timeout(function () {
          myEl.attr('disabled', 'disabled');
          myEl2.attr('disabled', 'disabled');
        });
      }
    }
    $scope.setDate = function (year, month) {
    };
    function updatePortfolio(increment) {
      if (increment)
        $scope.values.portfolioDate = moment($scope.values.portfolioDate).add(1, 'month').toDate();
      else
        $scope.values.portfolioDate = moment($scope.values.portfolioDate).subtract(1, 'month').toDate();
      updatePortfolioEvents();
    }
    function updatePortfolioEvents() {
      var kids = $appstate.model.observed;
      if (kids && kids.length) {
        $appstate.model.isRefreshingEvents = true;
        var startDate = moment($scope.values.portfolioDate).endOf('month').toDate();
        var days = moment(startDate).dayOfYear() - moment(startDate).subtract(1, 'month').dayOfYear();
        // support for crossing year boundary
        if (days < 0)
          days += 365;
        var filter = { 'filter': 'portfolio' };
        $appstate.refreshEvents([kids[0]], startDate, true, false, days - 1 + '', function () {
          $appstate.model.isRefreshingEvents = false;
        }, false, filter);
      }
    }
    $scope.forwardDate = function () {
      if ($appstate.model.viewPortfolio)
        updatePortfolio(true);
      else
        updateDate(dayMillis);
    };
    $scope.forwardWeek = function () {
      updateDate(7 * dayMillis);
    };
    $scope.backDate = function () {
      if ($appstate.model.viewPortfolio)
        updatePortfolio(false);
      else
        updateDate(-dayMillis);
    };
    $scope.backWeek = function () {
      updateDate(-7 * dayMillis);
    };
    $scope.openActivityPickerDialog = function () {
      $state.go('home.dash.activityPickerDialog');
    };
    $scope.model = $appstate.model;
    var f = $rootScope.$on('toggleViewPortfolio', function () {
        $scope.refreshEvents(true);
      });
    $scope.$on('destroy', function () {
      f();
    });
    $scope.refreshEvents = function (force) {
      if ($appstate.model.viewPortfolio) {
        updatePortfolioEvents();
      } else {
        $scope.pageSize = 5;
        //      // purge all events within the dateframe to avoid dupes
        //      var stop = new Date();
        //      var start = new Date();
        //
        //      stop.setHours(23);
        //      stop.setMinutes(59,59,999);
        //      start.setHours(0);
        //      start.setMinutes(0,0,0);
        //
        //      $appstate.model.eventList = _.filter($appstate.model.eventList, function(evt) {
        //        return !(evt.dateTimeOccurred.getTime() >= start.getTime() && evt.dateTimeOccurred.getTime() <= stop.getTime());
        //      });
        $appstate.refreshEvents(getObserved(), $appstate.model.currentDate || new Date(), force, true, null, function () {
          $scope.$broadcast('scroll.refreshComplete');
        }, false);
      }
    };
    function startDrag(item) {
    }
    //$scope.isLongPress = false;
    //$scope.touchStart = function(event) {
    //  console.log("touchStart", event);
    //  $scope.isLongPress = true;
    //  $timeout(function() { startDrag(event) }, 2000);
    //}
    //
    //$scope.touchEnd = function(event, item) {
    //  console.log("touchEnd", event, item);
    //  $scope.isLongPress = false;
    //}
    // set the date after defining the watch
    $scope.values = {};
    $scope.values.portfolioDate = new Date();
    function setDate() {
      if (isParent) {
        $scope.currDate = new Date();
        $appstate.model.currentDate = $scope.currDate;
      } else {
        $scope.currDate = $appstate.model.currentDate || new Date();
      }
    }
    setDate();
    updateDate(0);
    $timeout(function () {
      $native.enhanceUI();
    }, 1000);
  }
]);
'use strict';
angular.module('kidreports2App').factory('$localdb', [
  '$localstorage',
  '$nativestorage',
  function ($localstorage, $nativestorage) {
    // This service will decouple and decide the storage mechanism based on TBD factors
    function iOSversion() {
      if (/iP(hone|od|ad)/.test(navigator.platform)) {
        // supports iOS 2.0 and later: <http://bit.ly/TJjs1V>
        var v = navigator.appVersion.match(/OS (\d+)_(\d+)_?(\d+)?/);
        return [
          parseInt(v[1], 10),
          parseInt(v[2], 10),
          parseInt(v[3] || 0, 10)
        ];
      } else
        return [
          0,
          0,
          0
        ];
    }
    function getDB() {
      // if (typeof(forge) != 'undefined' && forge.is.ios() && iOSversion()[0] >= 11) {
      //   ///alert("iOS 11");
      //   return $nativestorage;
      // } else {
      //alert("not iOS 11");
      return $localstorage;  //}
    }
    // For now, use local storage
    //return $websql;//$localstorage;// getDB();
    return getDB();
  }
]);
'use strict';
angular.module('kidreports2App').factory('$localstorage', [
  '$q',
  function ($q) {
    // We are assuming webviews or modern browsers with local storage support
    var localStorage = localStorage || window.localStorage;
    var _get = function (key) {
      // Does nothing if key is not found
      return localStorage.getItem(key);
    };
    var _set = function (key, value) {
      // Can throw a QuotaExceededError
      localStorage.setItem(key, value);
    };
    var _remove = function (key) {
      // Does nothing if key is not found
      localStorage.removeItem(key);
    };
    function isCyclic(obj) {
      var seenObjects = [];
      function detect(obj) {
        if (obj && typeof obj === 'object') {
          if (seenObjects.indexOf(obj) !== -1) {
            return true;
          }
          seenObjects.push(obj);
          for (var key in obj) {
            if (obj.hasOwnProperty(key) && detect(obj[key])) {
              console.log(obj, 'cycle at ' + key);
              return true;
            }
          }
        }
        return false;
      }
      return detect(obj);
    }
    // Keep async interface
    return {
      get: function (key) {
        var d = $q.defer();
        var json = _get(key);
        if (json) {
          try {
            var obj = JSON.parse(json);
            d.resolve(obj);
          } catch (e) {
            d.reject(e);
          }
        } else {
          d.resolve(null);
        }
        return d.promise;
      },
      set: function (key, value) {
        var d = $q.defer();
        try {
          var str = JSON.stringify(value);
          _set(key, str);
          d.resolve(value);
        } catch (e) {
          console.log(e);
          isCyclic(value);
          d.reject(e);
        }
        return d.promise;
      },
      getKeys: function () {
        var v = [];
        for (var i = 0, len = localStorage.length; i < len; ++i) {
          v.push(localStorage.key(i));
        }
        return v;
      },
      remove: function (key) {
        var d = $q.defer();
        try {
          localStorage.removeItem(key);
          d.resolve();
        } catch (e) {
          d.reject(e);
        }
        return d.promise;
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').factory('$websql', [
  '$q',
  function ($q) {
    // Keep async interface
    return {
      get: function (key) {
        var d = $q.defer();
        __webdb.get(key, function (v) {
          d.resolve(v);
        }, function (err) {
          d.reject(err);
        });
        return d.promise;
      },
      set: function (key, value) {
        var d = $q.defer();
        __webdb.set(key, value, function (v) {
          d.resolve(v);
        }, function (err) {
          d.reject(err);
        });
        return d.promise;
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').factory('$nativestorage', [
  '$q',
  function ($q) {
    // Keep async interface
    return {
      get: function (key) {
        var d = $q.defer();
        forge.prefs.get(key, function (json) {
          if (json) {
            try {
              var obj = JSON.parse(json);
              d.resolve(obj);
            } catch (e) {
              d.reject(e);
            }
          } else {
            d.resolve(null);
          }
        }, function (err) {
          d.reject(err);
        });
        return d.promise;
      },
      set: function (key, value) {
        var d = $q.defer();
        try {
          var str = JSON.stringify(value);
          forge.prefs.set(key, str, function () {
            d.resolve(value);
          }, function (err) {
            d.reject(err);
          });
        } catch (e) {
          d.reject(e);
        }
        return d.promise;
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').factory('$facility', [
  '$q',
  '$localdb',
  '$auth',
  '$krConfig',
  '$ajax',
  '$timeout',
  '$native',
  '$utils',
  '$translate',
  '$image',
  '$model',
  '$msg',
  '$quicklinks',
  '$policies',
  '$messageBox',
  function ($q, $localdb, $auth, $krConfig, $ajax, $timeout, $native, $utils, $translate, $image, $model, $msg, $quicklinks, $policies, $messageBox) {
    // The ONLY facility reference in memory!
    var theFacility;
    var _lsEnabled = false;
    var _sys = false;
    // copy legacy action restrictions to new format
    function _updateRoles(obj) {
      if (!obj.restrictedActions) {
        obj.restrictedActions = [];
      }
      // copy the teacher restriction on sending messages
      if (obj.role == 'TEACHER_ROLE' && obj.canSendMessage === false) {
        obj.restrictedActions.push('sendMessage');
      }
      // if an admin logged in and saved events.
      if (obj.role.indexOf('ADMIN') > -1) {
        obj.legacyRole = obj.role;
        obj.role = 'TEACHER_ROLE';
        obj.canDeleteEvents = true;
        obj.canEditChildren = true;
        obj.canPublishPortfolioItems = true;
      }
    }
    ;
    var keepList = {
        'Diaper/Potty': true,
        'Nap': true,
        'Meal': true,
        'Snack': true,
        'Bottle': true,
        'Activity': true,
        'Daily Report': true,
        'Please Bring': true,
        'Drop-Off Information': true,
        'Picture/Video': true,
        'Diaper/Potty (M)': true,
        'Nap (M)': true,
        'Meal (M)': true,
        'Snack (M)': true,
        'Bottle (M)': true,
        'Activity (M)': true,
        'Daily Report (M)': true,
        'Please Bring (M)': true,
        'Drop-Off Information (M)': true,
        'Picture/Video (M)': true,
        'Diaper/Potty (R)': true,
        'Nap (R)': true,
        'Meal (R)': true,
        'Snack (R)': true,
        'Bottle (R)': true,
        'Activity (R)': true,
        'Daily Report (R)': true,
        'Please Bring (R)': true,
        'Drop-Off Information (R)': true,
        'Picture/Video (R)': true
      };
    function keepEventType(record) {
      var keep = false;
      var currUser = theFacility.currentUser;
      if (theFacility.enableLiteMode && (record.group == 'Portfolio' || !keepList[record.name]))
        return false;
      if (record.allowedRoles == 'ALLOW_ALL' || currUser.role == 'SYSTEM_ADMIN_ROLE' || currUser.role == 'FACILITY_ADMIN_ROLE') {
        keep = true;
      } else if (record.allowedRoles == 'ALLOW_TEACHER' && currUser.role == 'TEACHER_ROLE') {
        keep = true;
      } else if (record.allowedRoles == 'ALLOW_PARENT' && currUser.role == 'PARENT_ROLE') {
        keep = true;
      } else if (!currUser.role || !record.allowedRoles) {
        // fallback for role or allowedRoles not being set
        keep = true;
      } else {
        return false;
      }
      if (keep && record.categories && record.categories.length > 0 && record.enabled == true && record.isTemplate == false) {
        return true;
      }
      return false;
    }
    function updateChildren(facility) {
      var outerPromise = $q.defer();
      var count = 0;
      function updateChildTransients(item) {
        var deferred = $q.defer();
        item.name = item.firstName + ' ' + item.lastName;
        var prefix = $utils.prefix();
        var icon = item.iconHtml;
        var icon2 = item.secondaryIconHtml;
        if (icon && icon != '') {
          icon = icon.match(/src='.*'/)[0];
          icon = icon.substring(5, icon.length - 1);
          if (icon.indexOf('flag') > -1) {
            item.iconUrl = prefix + 'flag.png';
          }  //                item.iconUrl = icon;
        }
        if (icon2 && icon2 != '') {
          icon2 = icon2.match(/src='.*'/)[0];
          icon2 = icon2.substring(5, icon2.length - 1);
          if (icon2.indexOf('exclamation') > -1) {
            item.iconUrl2 = prefix + 'exclamation.png';
          }  //                item.iconUrl2 = icon2;
        }
        var url = $utils.createPhotoUrl($krConfig.s3Base, item.photoPath);
        if (item.photoPath) {
          $image.getImage(item.id, url).then(function (d) {
            item.photoUrl = d;
            count++;
            deferred.resolve();
          }, function (err) {
            console.log('Error retrieving file', err);
            deferred.resolve();
          });
        } else {
          item.photoUrl = url;
          count++;
          deferred.resolve();
        }
        return deferred.promise;
      }
      function fetchImages(kids, stepSize, promise) {
        var promises = [];
        var max = stepSize < kids.length ? stepSize : kids.length;
        for (var i = 0; i < max; i++) {
          promises.push(updateChildTransients(kids[i]));
        }
        $q.all(promises).then(function () {
          var arr = kids.slice(max, kids.length);
          if (arr.length > 0) {
            // recursively fetch images..
            fetchImages(arr, stepSize, promise);
          } else {
            promise.resolve();
          }
        });
      }
      var kids = [];
      facility.children.forEach(function (item) {
        kids.push(item);
      });
      facility.rooms.forEach(function (room) {
        room.children.forEach(function (kid) {
          kids.push(kid);
        });
      });
      fetchImages(kids, 10, outerPromise);
      // DEV-1452: Fetch profile images for every teacher in every room of the facility:
      var teachers = [];
      facility.rooms.forEach(function (room) {
        if (room.teachers) {
          room.teachers.forEach(function (teacher) {
            teachers.push(teacher);
          });
        }
      });
      fetchImages(teachers, 10, outerPromise);
      //return $q.all(promises);
      return outerPromise.promise;
    }
    var localGet = function () {
      return $localdb.get('facility');
    };
    var localSet = function (value) {
      if (value)
        value.lastSaveTime = new Date();
      return $localdb.set('facility', value);
    };
    var remoteGet = function () {
      var d = $q.defer();
      var baseUrl = $krConfig.getAjaxBase();
      var platform = $native.getPlatform();
      platform.reloadversion = $krConfig.reloadVersion;
      $ajax.ajax({
        method: 'GET',
        url: baseUrl + '/api/facility',
        params: platform
      }, function (err, data) {
        if (err) {
          d.reject(err);
        } else {
          if (data.facilities.length > 0) {
            var facility = data.facilities[0];
            theFacility = facility;
            $auth.getCredentials().then(function (credentials) {
              processFacility(facility, credentials).then(function () {
                $quicklinks.setCredentials(credentials).then(function () {
                  $quicklinks.get().then(function (links) {
                    facility.quicklinks = links;
                    facility.updatedTimestamp = new Date();
                    localSet(facility).then(function () {
                      d.resolve(facility);
                    });
                  });
                });
                if (facility.enablePolicyModule) {
                  $policies.setCredentials(credentials).then(function () {
                    $policies.get().then(function (links) {
                      facility.policies = links;
                    });
                  });
                }
              });
            }, function (err) {
            });
          } else {
            d.reject(new Error('Couldn\'t get facilities'));
          }
        }
      });
      return d.promise;
    };
    var localMoveRoom = function (kid, oldRm, newRoom, setHomeRoom, ts) {
      var idx = -1;
      oldRm.children.forEach(function (oldChild, count) {
        if (oldChild.id == kid.id)
          idx = count;
      });
      if (idx != -1)
        oldRm.children.splice(idx, 1);
      //oldRm.children = oldRm.children.remove(kid);
      if (setHomeRoom && !kid.homeRoom) {
        var rm = angular.copy(oldRm);
        rm.children = [];
        kid.homeRoom = rm;
      }
      // first check if kid is in room already
      if (_.filter(newRoom.children, function (k) {
          return k.id == kid.id;
        }).length == 0) {
        newRoom.children.push(kid);
        try {
          newRoom.children.sort(function (x, y) {
            return x.name.localeCompare(y.name);
          });
        } catch (err) {
        }
      }
      kid.category = newRoom.category;
      kid.roomId = newRoom.id;  //kid.lastMoveTimeStamp = ts || new Date().getTime();
    };
    var DAY_MS = 24 * 3600 * 1000;
    var runUpdateStrategy = function () {
      var d = $q.defer();
      // check if we are on a new day from the last time the rooms were saved and update back to home rooms.
      function updateTempRooms(facility) {
        if (facility.lastSaveTime && new Date(theFacility.lastSaveTime).getDate() != new Date().getDate()) {
          var move = [];
          facility.rooms.forEach(function (rm) {
            rm.children.forEach(function (kid) {
              if (kid.homeRoomId && kid.homeRoomId != kid.roomId) {
                move.push({
                  fromRm: rm,
                  toRm: _this.findRoom(kid.homeRoomId),
                  kid: kid
                });
              } else if (kid.homeRoom != null) {
                move.push({
                  fromRm: rm,
                  toRm: _this.findRoom(kid.homeRoom.id),
                  kid: kid
                });
              }
            });
          });
          move.forEach(function (obj) {
            localMoveRoom(obj.kid, obj.fromRm, obj.toRm);
            obj.kid.homeRoom = null;
          });
          //processFacility(facility);
          localSet(facility);
        }
      }
      // Get the latest facility if the local record is too old
      if (!theFacility.updatedTimestamp) {
        d.resolve(remoteGet());
      } else {
        var sync = _checkFacilityTime();
        if (sync) {
          d.resolve(remoteGet());
        } else {
          updateTempRooms(theFacility);
          $auth.getCredentials().then(function (credentials) {
            //processFacility(theFacility, credentials);
            // set up urban airship for push (if on native device)
            $timeout(function () {
              //alert('register uname');
              $native.registerUsername(theFacility.currentUser.username);
            }, 5000);
            d.resolve(theFacility);
          });
        }
      }
      return d.promise;
    };
    var processFacility = function (facility, credentials) {
      var d = $q.defer();
      function findUser(users) {
        facility.currentUser = _.filter(users, function (user) {
          var u = user.username || 'a';
          var e = user.email || 'e';
          return u.toUpperCase() === credentials.username.toUpperCase() || e.toUpperCase() === credentials.username.toUpperCase();
        })[0];
        // update current user role.  For now set to teacher if an admin.
        // todo support more actions if an admin, etc
        if (facility.currentUser.role == 'FACILITY_ADMIN_ROLE' || facility.currentUser.role == 'COMPANY_ADMIN_ROLE' || facility.currentUser.role == 'SYSTEM_ADMIN_ROLE') {
          facility.currentUser.role = 'TEACHER_ROLE';
          facility.currentUser.canDeleteEvents = true;
        }
      }
      function createObservers() {
        // Create an "observers" link in the facility data
        if (facility.currentUser.role === 'PARENT_ROLE') {
          // TODO include both parents as options
          facility.observers = [facility.currentUser];
          facility.observed = _.filter(facility.children, function (child) {
            facility.selectedCategory = child.category;
            for (var i = 0; i < child.parents.length; i++) {
              if (child.parents[i].id === facility.currentUser.id) {
                return true;
              }
            }
            return false;
          });
        } else {
        }
      }
      function updateActivityTypes(facility) {
        var lsETs = _.filter(facility.eventTypes, function (et) {
            return et.name == 'Learning Story' && !et.editable;
          });
        _lsEnabled = lsETs && lsETs.length && lsETs.length > 0;
      }
      function updateRooms() {
        var rooms = _.filter(facility.rooms, function (item) {
            return !(item.props && item.props['moveRoomsOnly'] === 'true');
          });
        var allKids = [];
        rooms.forEach(function (room) {
          allKids = allKids.concat(room.children);
        });
        facility.rooms.push({
          id: 0,
          name: $translate.instant('AllChildren'),
          children: allKids
        });
      }
      /// setup roles from legacy json
      facility.users.forEach(function (obj) {
        _updateRoles(obj);
      });
      var p = updateChildren(facility);
      facility.maxValueLength = facility.maxValueLength || 50;
      facility.maxValueLength = facility.maxValueLength || 50;
      // Need to find the logged-in user
      findUser(facility.users);
      // Create observers based on role
      createObservers();
      updateActivityTypes(facility);
      updateRooms();
      // set up urban airship for push (if on native device)
      $timeout(function () {
        $native.registerUsername(facility.currentUser.username);
      }, 5000);
      //$model.forceBackgroundIOSUpload = false;
      $model.forceBackgroundIOSUpload = facility.forceBackgroundIOSUpload;
      return p;
    };
    function _checkFacilityTime() {
      if (theFacility && theFacility.updatedTimestamp) {
        var ts = new Date(theFacility.updatedTimestamp);
        //        var UPDATE_INTERVAL_DAYS = 7;
        //        var intervalMs = UPDATE_INTERVAL_DAYS * DAY_MS;
        //        var diff = new Date() - ts;
        var startOfToday = new Date();
        startOfToday.setHours(0, 0, 0, 0);
        // resync if last timestamp was from the prior day.
        return ts.getTime() < startOfToday.getTime();
      } else
        return false;
    }
    var _this = {
        needsRefresh: _checkFacilityTime,
        get: function () {
          var d = $q.defer();
          // First, attempt to return the cache
          if (theFacility) {
            d.resolve(theFacility);
          } else {
            // Next try the local db
            localGet().then(function (facility) {
              if (facility) {
                // Cache
                theFacility = facility;
                updateChildren(facility).then(function () {
                  d.resolve(runUpdateStrategy());
                });
              } else {
                // Lastly get,store, and cache the remote
                d.resolve(remoteGet());
              }
            }, function (err) {
              d.reject(err);
            });
          }
          return d.promise;
        },
        save: function () {
          var evtHash = {};
          theFacility.children.forEach(function (c) {
            c.events = null;
          });
          theFacility.rooms.forEach(function (r) {
            r.children.forEach(function (c) {
              c.events = null;
            });
          });
          return localSet(theFacility);
        },
        allowEventType: function (r) {
          return keepEventType(r);
        },
        getSync: function () {
          return theFacility;
        },
        logout: function () {
          theFacility = null;
          return localSet(null);
        },
        findRoom: function (id) {
          if (theFacility != null) {
            return _.filter(theFacility.rooms, function (rm) {
              return parseInt(rm.id, 10) == parseInt(id, 10);
            })[0];
          } else
            return {};
        },
        findUser: function (id) {
          if (theFacility != null) {
            return _.filter(theFacility.users, function (u) {
              return parseInt(u.id, 10) == parseInt(id, 10);
            })[0];
          } else
            return {};
        },
        findEventType: function (id) {
          if (theFacility != null) {
            return _.filter(theFacility.eventTypes, function (u) {
              return parseInt(u.id, 10) == parseInt(id, 10);
            })[0];
          } else
            return {};
        },
        findCategory: function (id) {
          if (theFacility != null) {
            return _.filter(theFacility.categories, function (u) {
              return parseInt(u.id, 10) == parseInt(id, 10);
            })[0];
          } else
            return {};
        },
        findChild: function (id) {
          if (theFacility != null) {
            return _.filter(theFacility.children, function (kid) {
              return parseInt(kid.id, 10) == parseInt(id, 10);
            })[0];
          } else
            return {};
        },
        moveRoom: function (kid, oldRm, newRoom, setHomeRoom, ts) {
          localMoveRoom(kid, oldRm, newRoom, setHomeRoom, ts);
        },
        findEventTypeByName: function (name, catId) {
          if (theFacility != null) {
            return _.filter(theFacility.eventTypes, function (u) {
              return u.name === name && parseInt(u.categories && u.categories.length > 0 ? u.categories[0].id : -1, 10) == parseInt(catId, 10);
            })[0];
          } else
            return {};
        },
        findKidsByParent: function (p) {
          if (p.role != 'PARENT_ROLE')
            return [];
          return _.filter(theFacility.children, function (kid) {
            return _.filter(kid.parents, function (parent) {
              return parent.id === p.id;
            }).length > 0;
          });
        },
        getVideoLength: function () {
          var maxLen = 15;
          if (theFacility && theFacility.maxVideoLength) {
            maxLen = theFacility.maxVideoLength;
          }
          $model.videoDuration = maxLen;
          return maxLen;
        },
        getImageParameters: function () {
          // testing
          //theFacility.imageScale = 'XLARGE';
          console.log('Image Scale:' + theFacility.imageScale);
          var params = {};
          if (theFacility && theFacility.imageScale) {
            var scale = theFacility.imageScale;
            if (scale == 'XSMALL') {
              params = {
                width: 320,
                height: 240
              };
            } else if (scale == 'SMALL') {
              params = {
                width: 640,
                height: 480
              };
            } else if (scale == 'MEDIUM') {
              params = {
                width: 1280,
                height: 960
              };
            } else if (scale == 'LARGE') {
              params = {
                width: 1920,
                height: 1440
              };
            } else if (scale == 'CUSTOM') {
              // Temp for future custom image scaling settings.  Default to MEDIUM if not set.
              var width = facility.imageScaleWidth || 1280;
              var height = facility.imageScaleHeight || 960;
              params = {
                width: width,
                height: height
              };
            }
          }
          $model.imageScaleParams = params;
          return params;
        },
        isLSEnabled: function () {
          return _lsEnabled;
        },
        isLite: function () {
          return theFacility && theFacility.enableLiteMode;
        },
        hideParentRecordEvent: function () {
          return theFacility && theFacility.hideParentRecordEvent;
        },
        isSys: function () {
          return theFacility.currentUser.legacyRole == 'SYSTEM_ADMIN_ROLE';
        },
        isAdmin: function () {
          var usr = theFacility.currentUser;
          return usr.legacyRole == 'FACILITY_ADMIN_ROLE' || usr.legacyRole == 'REGION_ADMIN_ROLE' || usr.legacyRole == 'COMPANY_ADMIN_ROLE' || usr.legacyRole == 'SYSTEM_ADMIN_ROLE';
        }
      };
    return _this;
  }
]);
'use strict';
angular.module('kidreports2App').factory('$auth', [
  '$q',
  '$localdb',
  '$krConfig',
  '$base64',
  '$http',
  '$rootScope',
  '$environmentService',
  function ($q, $localdb, $krConfig, $base64, $http, $rootScope, $environmentService) {
    var theCredentials;
    var localGet = function () {
      return $localdb.get('credentials');
    };
    var localSet = function (value) {
      return $localdb.set('credentials', value);
    };
    var defaultAjax = function (options, cb) {
      $http(options).success(function (data) {
        cb(null, data);
      }).error(function (data, status, headers, config) {
        var errObj = {
            data: data,
            status: status,
            headers: headers,
            config: config
          };
        cb(errObj);
      });
    };
    // TODO need a real unpolluted ajax service
    var forgeAjax = function (options, cb) {
      var serialize = function (obj) {
        var str = [];
        for (var p in obj) {
          var obStr = _.isObject(obj[p]) ? JSON.stringify(obj[p]) : obj[p];
          str.push(encodeURIComponent(p) + '=' + encodeURIComponent(obStr));
        }
        return str.join('&');
      };
      //var baseUrl = $krConfig.getAjaxBase();
      var params = serialize(options.params);
      var url = params ? options.url + '?' + params : options.url;
      forge.request.ajax({
        type: options.method,
        url: url,
        data: options.data,
        dataType: 'json',
        headers: options.headers || {},
        contentType: options.contentType || 'application/x-www-form-urlencoded',
        success: function (data) {
          $rootScope.$apply(cb(null, data));
        },
        error: function (error) {
          cb(error);
        }
      });
    };
    var realAjax = typeof forge != 'undefined' ? forgeAjax : defaultAjax;
    function auth(username, password) {
      var d = $q.defer();
      var authStr = 'Basic ' + $base64.encode($krConfig.clientID + ':' + $krConfig.clientKey);
      // TODO why both url params and header?
      var opts = {
          url: $environmentService.getApiUrl() + '/token',
          method: 'POST',
          data: {
            username: username,
            password: password
          },
          headers: { 'Authorization': authStr }
        };
      realAjax(opts, function (err, data) {
        if (err) {
          d.reject(err);  //          $rootScope.$broadcast("authError", err, expired);
        } else {
          theCredentials = {
            token: data.access_token,
            username: username
          };
          d.resolve(localSet(theCredentials));  //          $rootScope.$broadcast("tokenUpdated", $krConfig.token);
        }
      });
      return d.promise;
    }
    function _lookupUserPass(isUsername, value) {
      var d = $q.defer();
      var authStr = 'Basic ' + $base64.encode($krConfig.clientID + ':' + $krConfig.clientKey);
      // TODO why both url params and header?
      var opts = {
          url: $krConfig.getAjaxBase() + '/utils/lookupUsernamePass?' + 'u=' + isUsername + '&v=' + encodeURIComponent(value),
          method: 'GET',
          headers: { 'Authorization': authStr }
        };
      realAjax(opts, function (err, data) {
        if (err) {
          d.reject(err);
        } else {
          d.resolve(data);
        }
      });
      return d.promise;
    }
    return {
      setCredentials: function (username, token) {
        theCredentials = {
          token: token,
          username: username
        };
        return localSet(theCredentials);
      },
      getCredentialsSync: function () {
        return theCredentials;
      },
      getCredentials: function () {
        var d = $q.defer();
        // First, attempt to return the cache
        if (theCredentials) {
          d.resolve(theCredentials);
        } else {
          // Next try the local db
          localGet().then(function (t) {
            if (t) {
              // Cache
              theCredentials = t;
              d.resolve(t);
            } else {
              d.reject(new Error('No token yet'));
            }
          }, function (err) {
            d.reject(err);
          });
        }
        return d.promise;
      },
      logout: function () {
        theCredentials = null;
        return localSet(null);
      },
      login: function (username, password) {
        return auth(username, password);
      },
      lookupUserPass: function (isUser, val) {
        return _lookupUserPass(isUser, val);
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').service('$events', [
  '$backend',
  '$taffy',
  '$adapter',
  '$utils',
  '$krConfig',
  '$ajax',
  '$rootScope',
  '$krCache',
  '$facility',
  '$online',
  '$stateStandards',
  '$translate',
  '$q',
  function Events($backend, $taffy, $adapter, $utils, $krConfig, $ajax, $rootScope, $krCache, $facility, $online, $stateStandards, $translate, $q) {
    var isUploading = false;
    var uploadStartedAt = null;
    var _disableBulkUpdates = false;
    // cleanup method
    function cleanupCircularRefs() {
      _.each($taffy.events()().get(), function (evt) {
        evt.child = { id: evt.child.id || evt.childId };
        $taffy.events()({ ___id: evt.___id }).update(evt);
      });
    }
    function _setStore(b) {
      if (!b) {
        $taffy.events().delayDiskUpdates();  //$taffy.events().store('events');
      } else {
        $taffy.events().flushDiskUpdates();  //settings( {storageName : null} );
      }
    }
    cleanupCircularRefs();
    function buildInfoObject(event, status, type) {
      var _type = $translate.instant(type || 'Added');
      var kid = $facility.findChild(event.child.id);
      var info = event.eventType.name + ' ' + $translate.instant('Event') + ' ' + $translate.instant('For') + ' ' + kid.firstName + ' ' + kid.lastName + ': ' + _type + ' @ ' + $utils.getTimeString($utils.getTimePart(new Date()));
      return {
        info: info,
        status: $translate.instant(status || 'Success'),
        time: $utils.getTimeString($utils.getTimePart(new Date()))
      };
    }
    function getLocalEvents(kids) {
      var events = [];
      if (_.isArray(kids)) {
        kids.forEach(function (kid) {
          events = events.concat($taffy.events()({ childId: kid.id }).get());
        });
      } else if (kids && kids.id) {
        events = $taffy.events()({ childId: kids.id }).get();
      }
      return _.filter(events, function (evt) {
        return !evt._deleted || evt._deleted === false;
      });
    }
    /**
 * Removes dataUrls from images property on an event
 * @param {object} event
 * @returns {object}
 */
    function removeDataUrlsFromEvent(event) {
      // Make a deep copy:
      var copy = {};
      angular.copy(event, copy);
      // If there is not images property (for some reason), just return:
      if (!copy.images) {
        return copy;
      }
      // Remove any dataURLs from images:
      _.each(copy.images, function (image) {
        // Delete the dataURL
        if (image.url) {
          delete image.url;
        }
      });
      return copy;
    }
    this.removeDataUrlsFromEvent = removeDataUrlsFromEvent;
    /**
     * Removes dataUrls from images for each event in a list
     * @param {Array<object>} events
     */
    function removeDataUrlsFromEvents(events) {
      var clean = [];
      _.each(events, function (event) {
        clean.push(removeDataUrlsFromEvent(event));
      });
      return clean;
    }
    this.removeDataUrlsFromEvents = removeDataUrlsFromEvents;
    /**
     * @function createLegacyEvents()
     * @param {object} event
     * @returns {Array}
       */
    function createLegacyEvents(event) {
      var legacyEvent = $adapter.createLegacyEvent(event);
      var legacyEvents = [];
      event.observed.forEach(function (o) {
        var le = angular.copy(legacyEvent);
        le.child = {};
        le.observed = [];
        le.child.id = o.id;
        // Create need a queryable top-level child id
        le.childId = o.id;
        // Create queryable timestamp
        le.timestamp = new Date(le.timeOccurred).getTime();
        le.child.events = [];
        legacyEvents.push(le);
      });
      return legacyEvents;
    }
    function deleteRemoteEvent(item, cb) {
      var opts = {
          method: 'DELETE',
          url: $krConfig.getAjaxBase() + '/data/event',
          params: { id: item.id }
        };
      //var opts = {method: 'DELETE', url: "http://localhost:3000" + '/data/event', params: {id: item.id}};
      $ajax.ajax(opts, function (err, data) {
        if (err) {
          console.error('Error making request');
          if (cb) {
            cb(err);
          }
        } else {
          if (cb) {
            cb(null, data);
          }
        }
      });
    }
    function getEventById(theId) {
      var baseUrl = $krConfig.getAjaxBase();
      var d = $q.defer();
      var params = { eventId: theId };
      // use a POST in case we are over the request string length (which has happened)
      var opts = {
          method: 'POST',
          url: baseUrl + '/data/queryEvents',
          data: params
        };
      $ajax.ajax(opts, function (err, data) {
        if (data && data.events && data.events.length) {
          //insertEvents(data.events, true);
          d.resolve(data.events[0]);
        } else
          d.reject(err);
      });
      return d.promise;
    }
    function getRemoteEvents(kids, date, cb, isUser, days, room, filter) {
      var baseUrl = $krConfig.getAjaxBase();
      var childrenIds;
      if (_.isArray(kids)) {
        childrenIds = kids.map(function (child) {
          return child ? child.id : '-1';
        });
      } else {
        childrenIds = [kids.id];
      }
      // Java no like 0 hours
      date.setHours(12);
      var params = {
          id: childrenIds.toString(),
          date: $utils.createGsonCompatibleTimestamp(date)
        };
      if (filter) {
        params = _.extend(params, filter);
      }
      if (isUser)
        params.isUser = 'true';
      if (days)
        params.days = days;
      if (room)
        params.roomId = String(room);
      // use a POST in case we are over the request string length (which has happened)
      var opts = {
          method: 'POST',
          url: baseUrl + '/data/queryEvents',
          data: params
        };
      $ajax.ajax(opts, getRemoteStateStandards(baseUrl, cb));
    }
    function getRemoteStateStandards(baseUrl, eventCallback) {
      return function (error, data) {
        // If there's an error, let the original caller handle it.
        if (error) {
          eventCallback(error, data);
          return;
        }
        // If there are no events, forward on the call.
        if (data.events.length === 0) {
          eventCallback(error, data);
          return;
        }
        var eventIds = data.events.map(function (event) {
            return event.id;
          });
        var opts = {
            method: 'POST',
            url: baseUrl + '/api/events/statestandards',
            data: { eventIds: eventIds }
          };
        $ajax.ajax(opts, function (error, allStateStandards) {
          // If there's an error, let the original caller handle it.
          if (error) {
            eventCallback(error, data);
            return;
          }
          if (allStateStandards.length === 0) {
            eventCallback(error, data);
            return;
          }
          for (var idx = 0; idx < data.events.length; idx++) {
            var evt = data.events[idx];
            evt.stateStandards = allStateStandards.filter(function (e) {
              return parseInt(e.event_id, 10) == evt.id;
            });
          }
          eventCallback(error, data);
        });
      };
    }
    function fixChildRef(evt) {
      evt.child = { id: evt.child.id };
    }
    /**
     * @function saveRemoteEvents()
     * @param events
     * @param childrenList
     * @param {Function} callback
     */
    function saveRemoteEvents(events, childrenList, callback) {
      var baseUrl = $krConfig.getAjaxBase();
      childrenList = childrenList || (events.length ? [events[0].child.id] : [events.child.id]);
      var url = events.length ? 'multiEvent' : 'event';
      if (events.length) {
        _.each(events, fixChildRef);
      } else {
        fixChildRef(events);
      }
      var params = {
          id: childrenList.toString(),
          overrideAttendanceCheck: events.showAbsentKids || _.some(events, [
            'showAbsentKids',
            true
          ])
        };
      var removeDataUrls = events.length ? removeDataUrlsFromEvents : removeDataUrlsFromEvent;
      // Removes any data URLs before going to server:
      var opts = {
          method: 'POST',
          url: baseUrl + '/data/' + url,
          params: params,
          data: removeDataUrls(events)
        };
      $ajax.ajax(opts, function (err, data) {
        if (err) {
          if (callback) {
            callback(err);
          }
          return;
        } else {
          if (!events.length && events.hasOwnProperty('stateStandards')) {
            var eventId = _.get(data, 'events[0].id');
            if (!eventId || eventId < 0) {
              callback(null, data);
              return;
            }
            $stateStandards.saveEventStandards(eventId, events.stateStandards).then(function (standards) {
              if (data.events.length) {
                data.events[0].stateStandards = standards;
              }
              callback(null, data);
            }).catch(function (error) {
              callback(error);
            });
          } else {
            if (callback) {
              callback(null, data);
            }
          }
        }
      });
    }
    var bulkSize = 5;
    // Serially push out new events
    function pushNewBulk(cb, doneCb) {
      uploadStartedAt = new Date().getTime();
      var event, events;
      // Find and send an event that doesn't have a server-assigned id
      if (bulkSize > 0) {
        events = $taffy.events()({ id: 0 }).limit(bulkSize).get();
      } else {
        events = $taffy.events()({ id: 0 }).get();
      }
      // all the ones that are in process, set id to -1
      $taffy.events()({
        ___id: _.map(events, function (e) {
          return e.___id;
        })
      }).update({ id: -1 });
      if (events.length > 0) {
        //event = events[0];
        saveRemoteEvents(events, null, function (err, remoteEvent) {
          if (!err) {
            // lookup by taffy id that was previously sent over
            _setStore(false);
            angular.forEach(remoteEvent.events, function (event) {
              var id = event.id;
              if (id != 0) {
                $taffy.events()({ ___id: event.___id }).update({
                  id: id,
                  $$hashKey: id
                });
                $krCache.addToCache(buildInfoObject(event), 'notifications');
              } else {
                $krCache.addToCache(buildInfoObject(event, 'Dropped'), 'notifications');
              }
            });
            // remove all the ones that didn't get saved and still have a -1 id (inprocess)
            $taffy.events()({ id: -1 }).remove();
            _setStore(true);
            // Push the next one (if any)
            pushNewBulk(cb, doneCb);
          } else {
            //console.log(err);
            // reset the in process ones to new.
            $taffy.events()({ id: -1 }).update({ id: 0 });
            if (doneCb)
              doneCb(true, err);
          }
        });
      } else {
        // On to the next callback
        if (cb) {
          cb(null, doneCb);
        } else if (doneCb)
          doneCb(false);
      }
    }
    // Serially push out new events
    function pushNewAsync(cb, doneCb) {
      uploadStartedAt = new Date().getTime();
      var event, events;
      // Find and send an event that doesn't have a server-assigned id
      events = $taffy.events()({ id: 0 }).get();
      angular.forEach(events, function (event) {
        saveRemoteEvents(event, null, function (err, remoteEvent) {
          if (!err && remoteEvent.events) {
            var event = remoteEvents.events[0];
            var id = remoteEvent.events[0].id;
            if (id != 0) {
              $taffy.events()({ ___id: event.___id }).update({
                id: id,
                $$hashKey: id
              });
              $krCache.addToCache(buildInfoObject(remoteEvent.events[0]), 'notifications');
            } else {
              $taffy.events()({ ___id: event.___id }).remove();
              $krCache.addToCache(buildInfoObject(remoteEvent.events[0], 'Dropped'), 'notifications');
            }
          } else if (!err) {
            // event was not saved on server, remove from client
            $taffy.events()({ ___id: event.___id }).remove();
          } else {
            //console.log(err);
            if (doneCb)
              doneCb(true, err);
          }
        });
      });
      // On to the next callback
      if (cb) {
        cb(null, doneCb);
      } else if (doneCb)
        doneCb(false);
    }
    // Serially push out new events
    function pushNew(cb, doneCb) {
      uploadStartedAt = new Date().getTime();
      var event, events;
      // Find and send an event that doesn't have a server-assigned id
      events = $taffy.events()({ id: 0 }).limit(1).get();
      if (events.length > 0) {
        event = events[0];
        saveRemoteEvents(event, null, function (err, remoteEvent) {
          if (!err && remoteEvent.events) {
            var id = remoteEvent.events[0].id;
            if (id != 0) {
              $taffy.events()({ ___id: event.___id }).update({
                id: id,
                $$hashKey: id
              });
              $krCache.addToCache(buildInfoObject(event), 'notifications');
            } else {
              $taffy.events()({ ___id: event.___id }).remove();
              $krCache.addToCache(buildInfoObject(event, 'Dropped'), 'notifications');
            }
            // Push the next one (if any)
            pushNew(cb, doneCb);
          } else if (!err) {
            // event was not saved on server, remove from client
            $taffy.events()({ ___id: event.___id }).remove();
            pushNew(cb, doneCb);
          } else {
            //console.log(err);
            if (doneCb)
              doneCb(true, err);
          }
        });
      } else {
        // On to the next callback
        if (cb) {
          cb(null, doneCb);
        } else if (doneCb)
          doneCb(false);
      }
    }
    // Serially push out changed events
    function pushDirty(cb, doneCb) {
      uploadStartedAt = new Date().getTime();
      var event, events;
      // Find and send a single dirty event
      events = $taffy.events()({ dirty: true }).limit(1).get();
      if (events.length > 0) {
        event = events[0];
        saveRemoteEvents(event, null, function (err, remoteEvent) {
          if (!err) {
            $taffy.events()({ ___id: event.___id }).update({
              dirty: false,
              $$hashKey: event.id
            });
            $krCache.addToCache(buildInfoObject(event, 'Success', 'Edited'), 'notifications');
            // Push the next one (if any)
            pushDirty(cb, doneCb);
          } else {
            if (doneCb)
              doneCb(true, err);
          }
        });
      } else {
        // On to the next callback
        if (cb) {
          cb(null, doneCb);
        } else if (doneCb)
          doneCb(false, null);
      }
    }
    // Serially push out changed events
    function pushDeleted(cb, doneCb) {
      uploadStartedAt = new Date().getTime();
      var event, events;
      // Find and send a single dirty event
      events = $taffy.events()({ _deleted: true }).limit(1).get();
      if (events.length > 0) {
        event = events[0];
        deleteRemoteEvent(event, function (err, data) {
          if (!err && (!data.error || data.error && data.message == 'Event Not Found')) {
            $taffy.events()({ ___id: event.___id }).remove();
            $krCache.addToCache(buildInfoObject(event, 'Success', 'Deleted'), 'notifications');
            // Push the next one (if any)
            pushDeleted(cb, doneCb);
          } else {
            if (doneCb)
              doneCb(true, err);
          }
        });
      } else {
        // On to the next callback
        if (cb) {
          cb(null, doneCb);
        } else if (doneCb)
          doneCb(false);
      }
    }
    function doneWithPush(isError, error) {
      isUploading = false;
      uploadStartedAt = null;
      $rootScope.$broadcast('pushComplete', isError, error);
    }
    // Hacked to get sequential server updates
    function push() {
      if (!$online.state())
        return;
      var t = new Date().getTime();
      var diff = uploadStartedAt != null ? t - uploadStartedAt : 0;
      var oneMinute = 1 * 60 * 1000;
      if (isUploading && diff < oneMinute) {
        //console.log("PushInProgress.");
        return;
      } else {
        isUploading = true;
        uploadStartedAt = new Date().getTime();
        var f = _disableBulkUpdates ? pushNew : pushNewBulk;
        f(function () {
          pushDirty(function () {
            pushDeleted(null, doneWithPush);
          }, doneWithPush);
        }, doneWithPush);
      }
    }
    function insertEvents(events, skipStore) {
      if (!skipStore)
        _setStore(false);
      events.forEach(function (event) {
        event.dirty = false;
        event.child = { id: event.child.id };
        event.childId = event.child.id;
        event.timestamp = new Date(event.timeOccurred).getTime();
        // Give angular the server-supplied unique id to prevent collisions
        event['$$hashKey'] = event.id;
        var theCopy = angular.copy(event);
        // Replace any existing events (server always wins)
        var results = $taffy.events()({ id: theCopy.id }).update(theCopy);
        if (0 === results.count()) {
          // Didn't exist -> insert
          $taffy.events().insert(theCopy);
        }
      });
      if (!skipStore)
        _setStore(true);
    }
    function pull(kids, cb, room, filter) {
      getRemoteEvents(kids, new Date(), function (err, data) {
        if (err && cb) {
          cb(err, null);
        }
        if (!err) {
          // purge kids returned, then insert new ones.
          var childrenIds = _.uniq(_.map(data.events, function (e) {
              return e.child.id;
            }));
          $taffy.events()({
            childId: childrenIds,
            id: { '!is': 0 }
          }).remove();
          insertEvents(data.events);
          if (cb) {
            cb(null, { events: data.events });
          }
        }
      }, null, null, room, filter);
    }
    function _get(event) {
      if (event.id && event.id != 0) {
        return $taffy.events()({ id: event.id }).first() || event;
      } else if (event.___id) {
        return $taffy.events()({ ___id: event.___id }).first() || event;
      } else
        return event;
    }
    function _saveEvent(event, skipPush, skipLocal) {
      var le, legacyEvents;
      // Past events bypass local DB
      if (skipLocal || !$utils.isToday(new Date(event.timestamp))) {
        // don't assume this is a single child event
        le = createLegacyEvents(event)[0];
        var childrenList = [];
        event.observed.forEach(function (child) {
          childrenList.push(child.id);
        });
        console.log('_saveEvents -> saveRemoteEvents');
        saveRemoteEvents(le, childrenList, function () {
          $rootScope.$broadcast('remoteEventSaveComplete');
        });
        return;
      }
      // Check for an update to existing record
      _setStore(false);
      if (event.___id) {
        // check for id just in case it saved while we were viewing.
        var evt = $taffy.events()({ ___id: event.___id }).first();
        if (evt && evt.id != 0)
          event.id = evt.id;
        // now see if they added kids to the list... add any new ones, update the existing one.
        var toAdd = [];
        var foundExisting;
        event.observed.forEach(function (obs) {
          if (obs.id != evt.childId)
            toAdd.push(obs);
          else {
            foundExisting = obs;
          }
        });
        if (foundExisting) {
          event.observed = [foundExisting];
          le = createLegacyEvents(event)[0];
          le.dirty = true;
          $taffy.events()({ ___id: event.___id }).update(le);
        }
        if (toAdd.length > 0) {
          event.observed = toAdd;
          event.___id = null;
          event.id = null;
          event.clientKey = null;
          legacyEvents = createLegacyEvents(event);
          $taffy.events().insert(legacyEvents);
        }
      } else {
        legacyEvents = createLegacyEvents(event);
        $taffy.events().insert(legacyEvents);
      }
      _setStore(true);
      if (!skipPush) {
        // Attempt to push new events to the server
        push();
      }
    }
    function purge() {
      // Get rid of events more than a day old
      var startOfToday = new Date();
      startOfToday.setHours(0);
      startOfToday.setMinutes(0);
      startOfToday.setSeconds(0);
      $taffy.events()({ timestamp: { lt: startOfToday.getTime() } }).remove();
    }
    this.offline = function () {
      isUploading = false;
    };
    this.queueSize = function () {
      return $taffy.events()({ dirty: true }).count() + $taffy.events()({ id: 0 }).count() + $taffy.events()({ id: -1 }).count() + $taffy.events()({ _deleted: true }).count();
    };
    this.queue = function () {
      return $taffy.events()({ dirty: true }).get().concat($taffy.events()({ id: 0 }).get()).concat($taffy.events()({ _deleted: true }).get());
    };
    this.push = function () {
      push();
    };
    /**
     * Return the latest version from the db, or return event if not found.
     * @param event
     */
    this.get = function (event) {
      return _get(event);
    }, this.getById = function (id) {
      return getEventById(id);
    };
    /**
     * Save the given event to the local db/store for offline usage and will eventually synchronize it to the
     * backend at a later time (either timer based or event based)
     * @param event
     */
    this.saveEvent = function (event, skipPush, skipLocal) {
      _saveEvent(event, skipPush, skipLocal);
    };
    /**
     * Save the given draft to the local db/store for offline usage and retrieval.
     * @param draft
     */
    this.saveDraft = function (draft) {
      $backend.saveEvent(draft, true);  //todo
    };
    this.deleteEvent = function (event, cb) {
      //var evt = _get(item);
      if (event.id && event.id != 0) {
        $taffy.events()({ id: event.id }).update({ _deleted: true });
      } else if (event.___id) {
        $taffy.events()({ ___id: event.___id }).update({ _deleted: true });
      }
      push();
      cb();
    };
    /**
     * Given the list of kids and the date, returns the events that match
     *
     * @param kids - Array of child objects
     * @param date - Date to retrieve events for, or today if null
     * @param cb - callback to invoke upon event retrieval (either from server or local)
     */
    this.getEvents = function (kids, date, cb, force, days, room, filter) {
      // Past events bypass local DB
      if ($online.state() && (force || days || date && !$utils.isToday(date))) {
        date = date || new Date();
        if (!days && $utils.isToday(date)) {
          pull(kids, cb, room, filter);
        } else {
          getRemoteEvents(kids, date, cb, false, days, room, filter);
        }
      } else {
        cb(null, { events: getLocalEvents(kids) });
      }
    };
    /**
     * Given the list of users(teachers) and the date, returns the events that match
     *
     * @param users - Array of User objects
     * @param date - Date to retrieve events for, or today if null
     * @param cb - callback to invoke upon event retrieval (either from server or local)
     */
    this.getUserEvents = function (users, date, cb, force, days, room, filter) {
      // Past events bypass local DB
      if ($online.state() && (force || date && !$utils.isToday(date))) {
        return getRemoteEvents(users, date, cb, true, days, room, filter);
      }
      var events = [];
      if (_.isArray(users)) {
        users.forEach(function (u) {
          events = events.concat($taffy.userEvents()({ subjectId: u.id }).get());
        });
      } else {
        events = $taffy.userEvents()({ subjectId: users.id }).get();
      }
      cb(null, { events: events });  //      $backend.getEvents(users, date, cb, true, false);
                                     //cb(null, []);
    };
    this.disableBulkUpdates = function (b) {
      _disableBulkUpdates = b;
    };
    this.bulkUpdatesEnabled = function () {
      return !_disableBulkUpdates;
    };
    this.sendToPortfolio = function (item) {
      var d = $q.defer();
      var copy = angular.copy(item);
      copy.id = 0;
      copy.portfolio = true;
      delete copy.clientKey;
      delete copy.___id;
      _.each(copy.eventValues, function (ev) {
        delete ev.id;
      });
      copy.mediaList = _.map(copy.mediaList, function (m) {
        return { key: m.key };
      });
      saveRemoteEvents(copy, null, function (err, data) {
        if (err)
          d.reject(err);
        else
          d.resolve(data);
      });
      return d.promise;
    };
    this.syncTodayEvents = function (kids, cb) {
      // Remove any old events from the local db
      purge();
      // Get any new events from the server
      pull(kids, cb);
    };
    this.purge = purge;
    this.cleardb = function () {
      $taffy.events()().remove();
    };
    this.store = function (b) {
      _setStore(b);
    };
  }
]);
;
'use strict';
angular.module('kidreports2App').factory('$taffy', function () {
  var events = TAFFY();
  var drafts = TAFFY();
  var userEvents = TAFFY();
  var plans = TAFFY();
  var messages = TAFFY();
  var stories = TAFFY();
  var defaults = TAFFY();
  var reminders = TAFFY();
  var memories = TAFFY();
  // Restore from local storage
  events.store('events');
  drafts.store('drafts');
  userEvents.store('userEvents');
  //plans.store('plans');
  defaults.store('defaults');
  reminders.store('reminders');
  return {
    events: function () {
      return events;
    },
    drafts: function () {
      return drafts;
    },
    userEvents: function () {
      return userEvents;
    },
    plans: function () {
      return plans;
    },
    messages: function () {
      return messages;
    },
    stories: function () {
      return stories;
    },
    memories: function () {
      return memories;
    },
    reminders: function () {
      return reminders;
    },
    defaults: function () {
      return defaults;
    }
  };
});
'use strict';
angular.module('kidreports2App').factory('$bootstrap', [
  '$localdb',
  '$q',
  '$appstate',
  '$featureFlags',
  function ($localdb, $q, $appstate, $featureFlags) {
    // Service logic
    // ...
    function checkDash(isParent, isUpgrade, cb) {
      $localdb.get('dashFlagSet').then(function (v) {
        if (!v) {
          $localdb.get('dashboard').then(function (db) {
            if (db == null) {
              $localdb.set('dashboard', !isParent);
            }
            $localdb.set('240migration', true);
            $localdb.set('showMediaInStream', true);
            $localdb.set('dashFlagSet', true).then(function () {
              cb(true);
            });
          });
        } else {
          // fix for upgrades to 2.4 ignoring dashboard flag and not setting it properly for new users.
          $localdb.get('240migration').then(function (v) {
            if (!v) {
              console.log('2.4 migration');
              // get values from prior localstorage for:
              // dashboard, requestCache, showMediaInStream, notifications, uploads
              var localStorage = localStorage || window.localStorage;
              var dashboard = localStorage.getItem('ls.dashboard');
              var db = dashboard != null && dashboard != 'null' && dashboard != 'false' && dashboard != false;
              // override dash mode if this was a 2.4 install that wasn't set up as a new install correctly.
              var dashNew = localStorage.getItem('dashboard');
              var db2 = dashNew != null && dashNew != 'null' && dashNew != 'false' && dashNew != false;
              if (dashboard == null && db2 == false) {
                db = true;
              }
              $localdb.set('dashboard', db);
              var media = localStorage.getItem('ls.showMediaInStream');
              var showMedia = media === 'false' || media === false ? false : true;
              $localdb.set('showMediaInStream', showMedia);
              var requestCache = localStorage.getItem('ls.requestCache');
              if (requestCache && requestCache.length && requestCache.length > 0) {
                $localdb.set('requestCache', requestCache);
              }
              var notifications = localStorage.getItem('ls.notifications');
              if (notifications && notifications.length && notifications.length > 0) {
                $localdb.set('notifications', notifications);
              }
              var uploads = localStorage.getItem('ls.uploads');
              if (uploads && uploads.length && uploads.length > 0) {
                $localdb.set('uploads', uploads);
              }
              $localdb.set('240migration', true);
            }
            cb(false);
          });
        }
      });
    }
    // Public API here
    return {
      check: function (ver, isUpgrade, user) {
        var q = $q.defer();
        var isParent = user.role == 'PARENT_ROLE';
        $featureFlags.establishCache();
        checkDash(isParent, isUpgrade, function (isDash) {
          q.resolve();
        });
        return q.promise;
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('KidplannerCtrl', [
  '$scope',
  '$facility',
  '$timeout',
  '$appstate',
  '$state',
  '$kidplanner',
  '$auth',
  '$modal',
  'translations',
  '$messageBox',
  '$online',
  '$ajax',
  '$krConfig',
  '$adapter',
  '$events',
  '$utils',
  '$backend',
  '$native',
  '$krCache',
  '$q',
  '$profileplanner',
  '$translate',
  '$eventAdapter',
  '$rootScope',
  function ($scope, $facility, $timeout, $appstate, $state, $kidplanner, $auth, $modal, translations, $messageBox, $online, $ajax, $krConfig, $adapter, $events, $utils, $backend, $native, $krCache, $q, $profileplanner, $translate, $eventAdapter, $rootScope) {
    // DEV-1022: On destroy, move back to portrait mode:
    $scope.$on('$destroy', function () {
      $native.portrait();
    });
    var facility = $facility.getSync();
    $scope.preventWriteActivity = facility.preventWriteActivity;
    $scope.planInSession = $kidplanner.planInSession = !$state.is('home.action.kidPlanner.main');
    //false;
    $scope.isPlanningMode = $kidplanner.planFinished ? $kidplanner.wasPlanning : true;
    $scope.kidPlannerLoading = true;
    $scope.printView = false;
    var isPT = translations.getCurrent().value == 'pt-br';
    var defaultOverrides = null;
    if (isPT) {
      // set fc to pt
      // todo upgrade to fc 2.x
      defaultOverrides = {
        monthNames: [
          'Janeiro',
          'Fevereiro',
          'Mar\xe7o',
          'Abril',
          'Maio',
          'Junho',
          'Julho',
          'Agosto',
          'Septembro',
          'Outubro',
          'Novembro',
          'Dezembro'
        ],
        monthNamesShort: [
          'Jan',
          'Fev',
          'Mar\xe7o',
          'Abril',
          'Maio',
          'Junho',
          'Julho',
          'Agosto',
          'Set',
          'Out',
          'Nov',
          'Dez'
        ],
        dayNames: [
          'Domingo',
          'Segunda-feira',
          'Ter\xe7a-feira',
          'Quarta-feira',
          'Quinta-feira',
          'Sexta-feira',
          'S\xe1bado'
        ],
        dayNamesShort: [
          'Dom',
          '2\xaa',
          '3\xaa',
          '4\xaa',
          '5\xaa',
          '6\xaa',
          'Sab'
        ],
        buttonText: {
          prev: '<span class=\'fc-text-arrow\'>&lsaquo;</span>',
          next: '<span class=\'fc-text-arrow\'>&rsaquo;</span>',
          prevYear: '<span class=\'fc-text-arrow\'>&laquo;</span>',
          nextYear: '<span class=\'fc-text-arrow\'>&raquo;</span>',
          today: 'today',
          month: 'm\xeas',
          week: 'semana',
          day: 'dia'
        }
      };
    }
    var currState = $state.current.name;
    var ppEnabled = $appstate.profilePlannerEnabled();
    /////////Offline Mode Check... Not supported for now///////////
    function goHome() {
      $messageBox.open('Offline Mode', 'Unfortunately we\'re having trouble connecting to the KidReports server.  Please try again  later.', { oktext: 'Ok' }).result.then(function (selectedItem) {
        $appstate.nav.goHome();
      }, function () {
        $appstate.nav.goHome();
      });
    }
    // short circuit for offline mode
    if (!$online.state()) {
      goHome();
    } else {
      var online = true;
      // check if we are "online" but we can't ping the server
      if ($facility.getSync() && $facility.getSync().disableServerPing) {
        var opts = {
            method: 'GET',
            url: $krConfig.getAjaxBase() + '/public/ping.json',
            params: {}
          };
        var pingTrue = false;
        $appstate.model.isRefreshing = true;
        $ajax.ajax(opts, function (err, data) {
          $appstate.model.isRefreshing = false;
          pingTrue = true;
          if (err) {
            goHome();
          } else {
            online = true;
          }
        });
        setTimeout(function () {
          if (!pingTrue) {
            $scope.$apply(function () {
              $appstate.model.isRefreshing = false;
              goHome();
            });
          }
        }, 10000);
      }
    }
    var date, d, m, y;
    function setDate(_date) {
      var oldM = date ? date.getMonth() : undefined;
      var oldY = date ? date.getFullYear() : undefined;
      date = _date;
      d = date.getDate();
      m = date.getMonth();
      y = date.getFullYear();  // check if month OR year are diff...
                               //      if ( oldM && oldY && ( oldM != m || oldY != y)  ) {
                               //        updateItemList();
                               //      }
    }
    var tmpdate = $kidplanner.wasPlanning ? $kidplanner.date : new Date();
    var todaydate = new Date();
    todaydate.setHours(0);
    todaydate.setMinutes(0);
    todaydate.setSeconds(0, 0);
    tmpdate.setHours(0);
    tmpdate.setMinutes(0);
    tmpdate.setSeconds(0, 0);
    setDate(tmpdate);
    $scope.usePlanDate = todaydate.toLocaleDateString();
    $scope.events = [];
    function updateItemList(selectionChange, startD, stopD) {
      //$scope.kidPlannerLoading = true;
      startD = startD || date;
      stopD = stopD || $scope.endDate || date;
      if (selectionChange) {
        $scope.plannedItems = getPlannedItems($scope.selectedRoom.id, startD, stopD);  //$scope.kidPlannerLoading = false;
      } else {
        // it's a room or month/year change, update the cal to show the full month of events
        //        var d = date;
        //        var d1 = new Date(d.getTime());
        //        d1.setUTCDate(1);
        //
        //        var d2 = new Date(d.getTime());
        //        d2.setUTCMonth(d2.getUTCMonth()+1);
        //        d2.setUTCDate(0);
        setItems(getPlannedItems($scope.selectedRoom.id, null, null));  //startD || d1, stopD || d2));
                                                                        //$scope.kidPlannerLoading = false;
      }
    }
    ;
    function setItems(items) {
      $scope.eventSources[0] = [];
      //{title:"foo", start:date, stop:date}];
      var addedDays = {};
      var extraItems = [];
      var regItems = [];
      items.forEach(function (p) {
        var key = p.start + '-' + p.stop;
        //addedDays[key] = 0;
        if (!addedDays.hasOwnProperty(key)) {
          //regItems.push(buildEvent(p));
          addedDays[key] = 0;
        }
        //else
        if (addedDays.hasOwnProperty(key)) {
          if (addedDays[key] === 0) {
            extraItems.push({
              title: '...',
              start: new Date(p.start),
              end: new Date(p.stop),
              key: key,
              className: getClassName(p)
            });
          }
          addedDays[key]++;
        }
      });
      extraItems.forEach(function (item) {
        item.title = '' + addedDays[item.key] + ' plan' + (addedDays[item.key] > 1 ? 's' : '');
        $scope.eventSources[0].push(item);
      });
      regItems.forEach(function (item) {
        $scope.eventSources[0].push(item);
      });
    }
    function buildEvent(plan) {
      var p = {};
      p.title = plan.template.eventType.name;
      p.start = new Date(plan.start);
      p.end = new Date(plan.stop);
      p.className = getClassName(plan);
      p.item = plan;
      return p;
    }
    function getClassName(plan) {
      var v = 'ui-calendar-day';
      if (plan.stop - plan.start > 7 * 24 * 60 * 60 * 1000) {
        v = 'ui-calendar-month';
      } else if (plan.stop > plan.start) {
        v = 'ui-calendar-week';
      }
      return [v];
    }
    function updateSelectedRoom() {
      $kidplanner.selectedRoom = $scope.selectedRoom;
      if (ppEnabled) {
        updateProfilePlannerEvents();
      }
      updateItemList(false);
      updateItemList(true);
      $timeout(function () {
        if ($appstate.toggle)
          $appstate.toggle(true);
        $appstate.setCurrentRoom($scope.selectedRoom);
      }, 250);
    }
    function buildPPEventValues(event, evtType) {
      var ppEtt;
      evtType.eventTypeTemplates.forEach(function (ett) {
        if (ett.name === 'Activity Details')
          ppEtt = ett;
      });
      var arr = [];
      arr.push({
        eventTypeTemplate: {
          id: ppEtt.id,
          name: ppEtt.name
        },
        value: event.title
      });
      return arr;
    }
    $scope.allSelected = false;
    function getPlannedItems(roomid, start, end) {
      var allEvents = [];
      if ($scope.myCalendar) {
        $scope.myCalendar.fullCalendar('clientEvents', function (event) {
          var s = new Date(event.start), e = new Date(event.end || event.start);
          s.setHours(0, 0, 0, 0);
          e.setHours(0, 0, 0, 0);
          if (s <= start && e >= end && event.lessonplanentryid) {
            // it's a profile planner event, sync up the template.
            var et = $facility.findEventTypeByName('Profile Planner Activity', $scope.selectedRoom.category.id);
            event.template = $adapter.createNewEvent(et);
            event.template.eventValues[0].value.choices.push({
              selected: true,
              name: event.title
            });
            event.template.legacyEventValues = $adapter.buildEventValues(event.template.eventValues);
            allEvents.push(event);
          }
        });
      }
      var items = $kidplanner.get(roomid, start ? start.getTime() : null, end ? end.getTime() : null);
      items.forEach(function (i) {
        i.template.legacyEventValues = $adapter.buildEventValues(i.template.eventValues);
        delete i['$$hashKey'];
        allEvents.push(i);
      });
      return allEvents;
    }
    function saveEvents(evt, isDraft, cache) {
      var newEvt = angular.copy(evt);
      newEvt.id = 0;
      if (isDraft) {
        // generate a uuid for in mem lookups
        newEvt.draftId = $utils.generateUUID();
        // copy by hand
        var cleanDraft = {
            draftId: newEvt.draftId,
            eventValues: angular.copy(newEvt.eventValues),
            id: newEvt.id,
            images: angular.copy(newEvt.images),
            videos: angular.copy(newEvt.videos),
            name: newEvt.name,
            notes: newEvt.notes,
            properties: angular.copy(newEvt.properties),
            timestamp: newEvt.timestamp
          };
        cleanDraft.eventType = { id: newEvt.eventType.id };
        cleanDraft.observed = [];
        newEvt.observed.forEach(function (obs) {
          cleanDraft.observed.push({ id: obs.id });
        });
        cleanDraft.observers = [];
        newEvt.observers.forEach(function (obs) {
          cleanDraft.observers.push({ id: obs.id });
        });
        cache.push(cleanDraft);
        $events.saveDraft(newEvt);
      } else {
        $events.saveEvent(newEvt);
      }
    }
    $scope.height = function () {
      return parseInt(window.innerHeight / 3, 10);
    };
    $scope.getSelectedCount = function () {
      return _.filter($scope.plannedItems, function (p) {
        return p.selected;
      }).length;
    };
    function saveAllEvents(isDraft, observed) {
      var q = $q.defer();
      $krCache.getCache('drafts', function (c) {
        var cache = c && _.isArray(c) ? c : [];
        $scope.plannedItems.forEach(function (p) {
          if (p.selected) {
            p.template.observed = observed;
            p.template.timestamp = $utils.createGsonCompatibleTimestamp(new Date());
            p.template.observers = [$backend.getCurrentUser()];
            // DEV-470: Onload, Play has returned a plan that has state standards in the eventStr property,
            //  however, if the plan was just saved and we haven't gone to play for the updated plan, the
            //  eventStr is undefined and we must rely on the template where KidPlanner saves them. 
            var eventData = p.eventStr ? JSON.parse(p.eventStr) : p.template;
            if (eventData.eventStateStandards) {
              p.template.stateStandards = $adapter.stateStandardsToApiModel(eventData.eventStateStandards);
            }
            saveEvents(p.template, isDraft, cache);
            setPlanSaved(p, new Date());
            if (p.template.eventType.name != 'Profile Planner Activity') {
              $kidplanner.put(p);
            } else {
              var extId = p._id;
              var oldPlan = $kidplanner.findByExternalId(extId);
              p.id = oldPlan ? oldPlan.id : null;
              p.roomId = $scope.selectedRoom.id;
              $kidplanner.put(p, extId);
            }
          }
        });
        $krCache.setCache(cache, 'drafts');
        q.resolve();
      });
      return q.promise;
    }
    $scope.setPickKids = function (b) {
      $scope.isPickKids = b;
    };
    function setPlanSaved(plan, date) {
      //planSavedToday(item) != ''"
      if (!plan.savedDates)
        plan.savedDates = {};
      plan.savedDates[$utils.getDatePart(date)] = true;
    }
    $scope.planSavedToday = function (plan) {
      if (plan.template.eventType.name === 'Profile Planner Activity') {
        var p = $kidplanner.isPlanSaved(plan);
        if (p && p.length > 0)
          p = p[0];
        return p && p.savedDates && p.savedDates[$utils.getDatePart(date)] == true ? 'planSaved' : '';
      } else {
        return plan.savedDates && plan.savedDates[$utils.getDatePart(date)] == true ? 'planSaved' : '';
      }
    };
    $scope.setUseMode = function () {
      $scope.isPlanningMode = false;
      $scope.planInSession = false;
      $scope.isPickKids = false;
      $scope.toggleSelected(true);
      $scope.dayClick(todaydate, true, null, null, true, true);
    };
    $scope.setPlanMode = function () {
      $timeout(function () {
        $scope.isPlanningMode = true;
        $scope.planInSession = false;
      });
    };
    $scope.usePlan = function (isDraft) {
      $scope.processing = true;
      $timeout(function () {
        _usePlan(isDraft);
      }, 200);
    };
    function _usePlan(isDraft) {
      // take the selected kids and selected plans and save all of them.
      var observed = $appstate.model.observed;
      var evts = [];
      var msg = '';
      if ($scope.getSelectedCount() == 0) {
        msg = $translate.instant('ErrorSavingPlannedEvents1');
      } else if (!observed || observed.length == 0) {
        msg = $translate.instant('ErrorSavingPlannedEvents2');
      }
      if (msg != '') {
        $messageBox.open($translate.instant('ErrorSavingPlannedEventsMsg'), msg, { oktext: $translate.instant('Ok') }).result.then();
        $scope.processing = false;
        return;
      }
      var itemCnt = observed.length * $scope.getSelectedCount();
      saveAllEvents(isDraft, observed).then(function () {
        var d = isDraft ? ' Draft' : '';
        if (d != '')
          d = ' ' + $translate.instant('Draft');
        var msg = $translate.instant('PlanSavedMsg').replace('&events&', itemCnt + d).replace('&kids&', $appstate.model.observed.length);
        var keepPlanning = $translate.instant('KeepPlanningMsg');
        if ($appstate.model.hideFrame) {
          $scope.processing = false;
          $scope.isPickKids = false;
          $scope.isPlanningMode = true;
          $appstate.msg(msg, true);
        } else {
          $messageBox.open($translate.instant('EventsSaved'), msg + '  ' + keepPlanning, {
            oktext: $translate.instant('Yes'),
            canceltext: $translate.instant('No')
          }).result.then(function (selectedItem) {
            //$state.go("home.action.selectEvent");
            $scope.processing = false;
            $scope.isPickKids = false;
            $scope.isPlanningMode = true;
            $appstate.msg(msg, true);
          }, function () {
            $scope.processing = false;
            $appstate.msg(msg, true);
            $appstate.goHome();
          });
        }
      });
    }
    $scope.toggleSelected = function (b) {
      $scope.allSelected = b || !$scope.allSelected;
      $scope.plannedItems.forEach(function (p) {
        p.selected = $scope.allSelected;
      });
    };
    /* alert on eventClick */
    $scope.alertOnEventClick = function (calEvent, jsEvent, view) {
      var mousex = jsEvent.pageX;
      var mousey = jsEvent.pageY;
      $('td').each(function (index) {
        var offset = $(this).offset();
        if (offset.left + $(this).outerWidth() > mousex && offset.left < mousex && offset.top + $(this).outerHeight() > mousey && offset.top < mousey) {
          var date = $(this)[0].dataset.date;
          var d = $utils.createDateFromDTStrings(date, '00:00:00');
          d.setHours(0);
          d.setMinutes(0);
          d.setSeconds(0, 0);
          $scope.dayClick(d, true, jsEvent, view);
        }
      });
    };
    /* alert on Drop */
    $scope.alertOnDrop = function (event, dayDelta, minuteDelta, allDay, revertFunc, jsEvent, ui, view) {
      $scope.alertMessage = 'Event Droped to make dayDelta ' + dayDelta;
    };
    /* alert on Resize */
    $scope.alertOnResize = function (event, dayDelta, minuteDelta, revertFunc, jsEvent, ui, view) {
      $scope.alertMessage = 'Event Resized to make dayDelta ' + minuteDelta;
    };
    /* add and removes an event source of choice */
    $scope.addRemoveEventSource = function (sources, source) {
      var canAdd = 0;
      angular.forEach(sources, function (value, key) {
        if (sources[key] === source) {
          sources.splice(key, 1);
          canAdd = 1;
        }
      });
      if (canAdd === 0) {
        sources.push(source);
      }
    };
    /* add custom event*/
    $scope.addEvent = function () {
      $scope.eventSources[0].push({
        title: 'Open Sesame',
        start: new Date(y, m, 28),
        end: new Date(y, m, 29),
        className: ['openSesame']
      });
    };
    /* remove event */
    $scope.remove = function (index) {
      $scope.eventSources[0].splice(index, 1);
    };
    /* Change View */
    $scope.changeView = function (view, calendar) {
      calendar.fullCalendar('changeView', view);
    };
    /* Change View */
    $scope.renderCalender = function (calendar) {
      calendar.fullCalendar('render');
    };
    $scope.lastDay = null;
    $scope.endDate = null;
    $scope.getDateString = function (template) {
      var start = template.start;
      var stop = template.stop;
      if (start == stop || !stop)
        return new Date(start).toLocaleDateString();
      else
        return 'From ' + new Date(start).toLocaleDateString() + ' to ' + new Date(stop).toLocaleDateString();
    };
    $scope.vid = { url: '' };
    $scope.viewPP = function (ppItem) {
      $appstate.model.isRefreshing = true;
      $profileplanner.getEventDetails($facility.getSync().currentUser, ppItem.id).then(function (data) {
        $appstate.model.isRefreshing = false;
        var modalInstance = $modal.open({
            templateUrl: 'views/popups/profilePlannerDetails.html',
            controller: 'ProfileplannerdetailsCtrl',
            resolve: {
              viewPPItem: function () {
                return data;
              }
            }
          });
        modalInstance.result.then(function (d) {
          return;
        });
      });
    };
    $scope.viewLessonPlan = function (item) {
      $eventAdapter.showEvent($adapter.processEvent($adapter.createLegacyEvent(item.template)));
    };
    $scope.deleteItem = function (item) {
      $messageBox.open($translate.instant('ConfirmDelete'), $translate.instant('ConfirmDeletePlanMessage'), {
        oktext: $translate.instant('Yes'),
        canceltext: $translate.instant('No')
      }).result.then(function (selectedItem) {
        $kidplanner.deleteItem(item).then(function (d) {
          $kidplanner.planFinished = true;
          $kidplanner.wasPlanning = true;
          $kidplanner.date = new Date();
          $state.go(currState, {}, { reload: true });
        }, function (err) {
        });
      }, function () {
      });
    };
    var days = [
        'Monday',
        'Tuesday',
        'Wednesday',
        'Thursday',
        'Friday'
      ];
    $scope.print = function () {
      var modalInstance = $modal.open({
          templateUrl: 'views/popups/kidplannerPrintDialog.html',
          controller: 'KidplannerprintCtrl',
          resolve: {
            room: function () {
              return $scope.selectedRoom;
            }
          }
        });
      modalInstance.result.then(function (val) {
        $scope.printParams = {
          date: val.date,
          items: val.items,
          days: days
        };
        $scope.printView = true;
        var dateobj = $utils.createDateFromDTStrings(val.date, '00:00:00');
        $scope.printParams.weekStart = new Date(findWeekStart(dateobj).getTime() + dayMS);
        $scope.printParams.weekStop = new Date(findWeekStop(dateobj).getTime() - dayMS);
      });
    };
    var dayMS = 1000 * 60 * 60 * 24;
    function findWeekStart(date) {
      var d = new Date(date.getTime());
      var day = d.getUTCDay();
      return new Date(d.getTime() - day * dayMS);
    }
    function findWeekStop(date) {
      var d = new Date(date.getTime());
      var day = d.getUTCDay();
      return new Date(d.getTime() + (6 - day) * dayMS);
    }
    $scope.cancelPrint = function () {
      $timeout(function () {
        $scope.printView = false;
      });
    };
    $scope.getItemPrintDetails = function (et, day) {
      var dayIndex = -1;
      _.forEach(days, function (d, idx) {
        if (d == day)
          dayIndex = idx;
      });
      if (dayIndex == -1)
        return '';
      //dayIndex++;
      // find the actual date in question, then look up any activities for the day and event type
      var theDate = new Date($scope.printParams.weekStart.getTime() + dayIndex * 1000 * 60 * 60 * 24);
      theDate.setHours(0, 0, 0, 0);
      var items = getPlannedItems($scope.selectedRoom.id, theDate, theDate);
      var filtered = _.filter(items, function (evt) {
          return evt.template.eventType.id === et.id || evt.template.eventType.name == et.name;
        });
      var str = '';
      _.each(filtered, function (lp) {
        var planStr = '';
        _.each(lp.template.legacyEventValues, function (ev) {
          //if ( ev.value.choices && _.isArray(ev.value.choices) ) {
          //  if ( ev.value.choices.length > 0 && _.filter(ev.value.choices, function(c) { return c.selected }).length > 0 ) {
          //    str += ev.ett.name + ":";
          //    _.each(ev.value.choices, function (choice) {
          //      if ( choice.selected ) str += choice.name + ", ";
          //    })
          //    str = str.substr(0, str.length - 2);
          //    str += "<br/>";
          //  }
          //} else {
          planStr += ev.eventTypeTemplate.name + ':' + ev.value + '<br/>';  //          }
        });
        var standardsByDomain = _.groupBy(lp.template.stateStandards, 'domain_guid');
        _.each(standardsByDomain, function (standards) {
          planStr += 'Domain: ' + standards[0].domain_text + '<br/>';
          _.each(standards, function (standard) {
            planStr += 'Indicator: ' + standard.standard_number + ': ' + standard.standard_text + '<br/>';
          });
        });
        if (lp.template.notes && lp.template.notes.length > 0)
          planStr += 'Notes:&nbsp;' + lp.template.notes + '<br/>';
        if (planStr == '') {
          planStr = 'Planned - No Details<br/>';
        }
        str += planStr;
      });
      return str;
    };
    $scope.copy = function (item) {
      var sDate = new Date(item.start);
      var eDate = new Date(item.stop);
      var diff = item.stop - item.start + 24 * 60 * 60 * 1000;
      sDate.setTime(sDate.getTime() + diff);
      eDate.setTime(eDate.getTime() + diff);
      var modalInstance = $modal.open({
          templateUrl: 'views/popups/dateRangePopup.html',
          controller: 'DaterangeCtrl',
          resolve: {
            startdate: function () {
              return $utils.getDatePart(sDate);
            },
            enddate: function () {
              return $utils.getDatePart(eDate);
            }
          }
        });
      modalInstance.result.then(function (val) {
        //console.log(val.startdate);
        //console.log(val.enddate);
        var d1 = $utils.createDateFromDTStrings(val.startdate, '00:00:00');
        var d2 = $utils.createDateFromDTStrings(val.enddate, '00:00:00');
        d1.setHours(0);
        d1.setMinutes(0);
        d1.setSeconds(0, 0);
        d2.setHours(0);
        d2.setMinutes(0);
        d2.setSeconds(0, 0);
        $appstate.msg('Plan Copied to ' + val.startdate + ' - ' + val.enddate, true);
        // save plan
        var plan = {
            start: d1.getTime(),
            stop: d2.getTime(),
            roomId: $scope.selectedRoom.id
          };
        plan.template = angular.copy(item.template);
        plan.template.observed = [$facility.getSync().children[0]];
        plan.template.timestamp = $utils.createGsonCompatibleTimestamp(new Date());
        $appstate.model.isRefreshing = true;
        $kidplanner.put(plan).then(function () {
          $kidplanner.currentPlan = null;
          $kidplanner.planFinished = true;
          $kidplanner.wasPlanning = true;
          $kidplanner.date = date;
          $state.go(currState, {}, { reload: true });
        });
      }, function () {
      });
    };
    $scope.refreshPlans = function () {
      $scope.isRefreshingPlans = true;
      var date = $scope.myCalendar.fullCalendar('getDate');
      $kidplanner.refreshPlans(date.getTime()).then(function (d) {
        finishKidPlannerSetup();
        $scope.isRefreshingPlans = false;
      });
    };
    $scope.edit = function (item) {
      $kidplanner.planInSession = true;
      $kidplanner.planDate = item.start;
      $appstate.model.currentEvent = item.template;
      $appstate.model.currentEvent.observed = [];
      $appstate.model.currentEvent.observed.push($facility.getSync().children[0]);
      //$appstate.model.currentEvent.observers = [];
      //$appstate.model.currentEvent.observed.push($facility.getSync().currentUser);
      $appstate.model.currentEvent.timestamp = new Date();
      $kidplanner.currentPlan = item;
      $kidplanner.dateRange = {
        start: date,
        stop: $scope.endDate || date
      };
      $kidplanner.nextState = plannerNext;
      $scope.planInSession = false;
      $appstate.nav.startPath('kidPlannerAdd');
      $timeout(function () {
        $scope.planInSession = true;
        $appstate.nav.startPath('kidPlannerEdit');
      });
    };
    $scope.addActivity = function () {
      $kidplanner.planInSession = true;
      $scope.planInSession = true;
      $kidplanner.planDate = $scope.date;
      $appstate.model.observed = [$facility.getSync().children[0]];
      $appstate.nav.nextState = null;
      $kidplanner.currentPlan = {
        start: date.getTime(),
        stop: $scope.endDate ? $scope.endDate.getTime() : date.getTime(),
        roomId: $scope.selectedRoom.id
      };
      $kidplanner.dateRange = {
        start: date,
        stop: $scope.endDate || date
      };
      $kidplanner.nextState = plannerNext;
      $rootScope.$broadcast('kidPlannerDateChange');
      $appstate.nav.startPath('kidPlannerAdd');
    };
    function select(startDate, endDate, type, skipApply) {
      if (skipApply) {
        $timeout(function () {
          applyselect(startDate, endDate, type);
        }, 100);
      } else {
        $scope.$apply(function () {
          applyselect(startDate, endDate, type);
        });
      }
    }
    function applyselect(startDate, endDate, type) {
      $scope.myCalendar.fullCalendar('unselect');
      $scope.myCalendar.fullCalendar('select', startDate, endDate, true);
      $scope.myCalendar.fullCalendar('gotoDate', startDate.getFullYear(), startDate.getMonth(), startDate.getDate());
      if (startDate.getTime() == endDate.getTime())
        $scope.date = $translate.instant('SelectedDay') + ': ' + startDate.toLocaleDateString();
      else {
        $scope.date = $translate.instant('Selected' + type) + ' : ' + startDate.toLocaleDateString() + ' - ' + endDate.toLocaleDateString();
      }
      $scope.endDate = endDate;
      $kidplanner.dateRange = {
        start: date,
        stop: $scope.endDate || date
      };
      updateItemList(true);
    }
    $scope.dayClick = function (d, allDay, jsEvent, view, skipApply, skipWMSelect) {
      // add the event
      var wasPlanInSession = $scope.planInSession;
      $scope.planInSession = false;
      $kidplanner.planInSession = false;
      // update state
      setDate(d);
      if (skipWMSelect || wasPlanInSession) {
        $scope.lastDay = d;
        select(d, d, null, skipApply);
      } else if ($scope.lastDay && d.getTime() == $scope.lastDay.getTime()) {
        // case for first day of month and week on same day.
        $scope.lastDay = null;
        if (d.getUTCDate() == 1 && d.getUTCDay() == 0 && $scope.date.indexOf('Month') > -1) {
          var d2 = new Date(d.getTime());
          d2.setUTCDate(d.getUTCDate() + 6);
          select(d, d2, 'Week', skipApply);
        } else if (d.getUTCDate() == 1) {
          // select the month
          var d2 = new Date(d.getTime());
          d2.setUTCMonth(d2.getUTCMonth() + 1);
          d2.setUTCDate(0);
          select(d, d2, 'Month', skipApply);
        } else if (d.getUTCDay() == 0) {
          var d2 = new Date(d.getTime());
          d2.setUTCDate(d.getUTCDate() + 6);
          select(d, d2, 'Week', skipApply);
        } else {
          select(d, d, null, skipApply);
        }
      } else {
        $scope.lastDay = date;
        select(d, d, null, skipApply);
      }
    };
    function _savePlan(rmId, baseRoomId) {
      // save plan
      if ($kidplanner.currentPlan.id) {
        var plan = $kidplanner.currentPlan;
        plan.template = angular.copy($appstate.model.currentEvent);
        return $kidplanner.put(plan);
      } else {
        var plan = angular.copy($kidplanner.currentPlan);
        plan.roomId = rmId;
        if (baseRoomId)
          plan.baseRoomId = baseRoomId;
        plan.template = angular.copy($appstate.model.currentEvent);
        return $kidplanner.put(plan);
      }
    }
    function plannerNext() {
      //if we're on event details go back to main, otherwise go next and reset
      if ($state.is('home.action.kidPlanner.eventDetails') || $state.is('home.dash.eventDetailsKidPlanner')) {
        // save the event, go home
        $kidplanner.planInSession = false;
        $appstate.model.isRefreshing = true;
        var promises = [];
        // save the original plan with the base room
        var baseRoomId = $kidplanner.currentPlan.roomId;
        promises.push(_savePlan(baseRoomId));
        // save the plan for all extra rooms
        if (!$kidplanner.currentPlan.id && $kidplanner.extraRooms) {
          _.each($kidplanner.extraRooms, function (val, key) {
            if (val) {
              promises.push(_savePlan(parseInt(key, 10), baseRoomId));
            }
          });
        }
        $q.all(promises).then(function () {
          //for a single event
          $kidplanner.currentPlan = null;
          $appstate.model.isRefreshing = false;
          $kidplanner.planFinished = true;
          $kidplanner.wasPlanning = $scope.isPlanningMode;
          $kidplanner.date = date;
          $state.go(currState, {}, { reload: true });
          var msg = $translate.instant('PlanSaved') + ' ' + $translate.instant('For') + ' ' + $appstate.model.currentEvent.eventType.name;
          $appstate.msg(msg, true);
          $messageBox.open($translate.instant('PlanSaved') + '!', msg + ' - ' + $scope.date, { oktext: $translate.instant('Ok') }).result.then();
        });
      } else {
        $appstate.nav.next();
        $appstate.nav.nextState = plannerNext;
      }
    }
    $scope.plannerBack = function () {
      if ($state.is('home.action.kidPlanner.main') && !$scope.isPlanningMode) {
        $scope.isPlanningMode = true;
      } else
        $appstate.nav.back();
    };
    $appstate.nav.backState = $scope.plannerBack;
    $scope.select = function (startDate, endDate, allDay, jsEvent, view) {
    };
    $scope.unselect = function (view, jsEvent) {
    };
    $scope.viewRender = function (view, element) {
      if (!$scope.kidPlannerLoading)
        $scope.dayClick(view.start, true, null, element, true, true);
    };
    $native.landscape().then(function () {
      $timeout(function () {
        var height = window.innerHeight - 150;
        /* config object */
        $scope.uiConfig = {
          calendar: {
            height: height,
            editable: true,
            selectable: true,
            header: {
              left: 'prev,next',
              center: 'title',
              right: 'month,basicWeek,basicDay'
            },
            eventClick: $scope.alertOnEventClick,
            eventDrop: $scope.alertOnDrop,
            eventResize: $scope.alertOnResize,
            dayClick: $scope.dayClick,
            select: $scope.select,
            unselect: $scope.unselect,
            viewRender: $scope.viewRender
          }
        };
        if (defaultOverrides) {
          $scope.uiConfig.calendar = _.extend($scope.uiConfig.calendar, defaultOverrides);
        }
      });
    });
    // todo inject this
    $scope.rooms = _.filter($facility.getSync().rooms, function (r) {
      return r.name != $translate.instant('AllChildren') && !(r.props && r.props['moveRoomsOnly'] === 'true');
    });
    $scope.selectedRoom = $kidplanner.selectedRoom || $scope.rooms[0];
    //$scope.changeRoom($scope.selectedRoom);
    var f1 = $scope.$watch('isPlanningMode', function () {
        $kidplanner.isPlanningMode = $scope.isPlanningMode;
        $kidplanner.planningModeSet = true;
      });
    var f2 = $scope.$watch('selectedRoom', function () {
        if ($scope.planInSession && $state.is('home.action.kidPlanner.eventDetails')) {
          $scope.planInSession = false;
        }
        if (ppEnabled) {
          // change room to current... then update
          $appstate.msg('Querying Profile Planner...', true, 'info', 10000);
          var id = $scope.selectedRoom.props['profileplanner.classroom_id'];
          $profileplanner.setRoom($facility.getSync().currentUser, id).then(function (data) {
            updateSelectedRoom();
          }, function (reason) {
            updateSelectedRoom();  //alert('Failed: ' + reason);
          });
        } else {
          updateSelectedRoom();
        }
      });
    var func4 = $rootScope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams) {
        $timeout(function () {
          $scope.planInSession = toState.name != 'home.action.kidPlanner.main';
        });
      });
    var f3 = $scope.$on('destroy', function () {
        f1();
        f2();
        f3();
        func4();
      });
    var uid = $facility.getSync().currentUser.props['profileplanner.uid'];
    var url = $profileplanner.endpoint;
    var lastSource = null;
    function updateProfilePlannerEvents() {
      $appstate.msg('Loading Profile Planner Events...', true, 'info', 10000);
      $timeout(function () {
        if (lastSource) {
          $scope.myCalendar.fullCalendar('removeEventSource', lastSource);
        }
        var opts = {
            method: 'GET',
            url: url + '/drupal/planner_files/udp/events.php?user_uid=' + uid,
            params: {}
          };
        $ajax.ajax(opts, function (err, data) {
          $appstate.msg('Added Profile Planner Events!', true, 'success', 2000);
          lastSource = {
            events: data,
            className: 'profilePlannerEvent'
          };
          $scope.myCalendar.fullCalendar('addEventSource', lastSource);
          $timeout(function () {
            $scope.dayClick(date, true, null, null, true, true);
          }, 500);
        });
      }, 10);
    }
    $scope.eventSources = [$scope.events];
    // set up credentials
    var token = $auth.getCredentialsSync().token;
    var creds = { 'Authorization': 'Bearer ' + token };
    $appstate.model.isRefreshing = true;
    $scope.kidPlannerLoading = true;
    if (!$kidplanner.planFinished) {
      $kidplanner.setCredentials(creds).then(finishKidPlannerSetup, function (err) {
        console.log(err);
      });
    } else {
      $kidplanner.planFinished = false;
      date = $kidplanner.dateRange.start;
      $scope.endDate = $kidplanner.dateRange.stop;
      $timeout(finishKidPlannerSetup, 100);
    }
    function finishKidPlannerSetup() {
      setItems(getPlannedItems($scope.selectedRoom.id));
      $timeout(function () {
        $scope.kidPlannerLoading = false;
        //applyselect(date, $scope.endDate);
        $scope.dayClick(date, true, null, null, true, true);
      }, 500);
      $appstate.model.isRefreshing = false;
      if ($native.isNative())
        $native.resetiOSHeader();
    }
    $scope.isNative = $native.isNative();
  }
]);
'use strict';
angular.module('kidreports2App').factory('$kidplanner', [
  '$taffy',
  '$resource',
  '$ajax',
  '$adapter',
  '$utils',
  '$facility',
  '$q',
  '$krConfig',
  function ($taffy, $resource, $ajax, $adapter, $utils, $facility, $q, $krConfig) {
    // Service logic
    var credentials = {};
    var LessonPlan = null;
    //$resource('/lessonplan/:id');
    var url = function (ts) {
      var str = $krConfig.getAjaxBase() + '/lessonplan';
      if (ts)
        str += '?startDate=' + ts;
      return str;
    };
    //
    //      template : angular.copy($appstate.model.currentEvent),
    //      start : date.getTime(),
    //      stop : $scope.endDate ? $scope.endDate.getTime() : date.getTime(),
    //      room_id : $scope.selectedRoom.id
    function synchDB(d, timestamp) {
      $taffy.plans()().remove();
      getPlans(d, timestamp);
    }
    function filterTemplate(event) {
      return $adapter.createLegacyEvent(event);
    }
    function savePlan(plan, d, externalPlanId) {
      var p = {};
      //new LessonPlan();
      var oldId = plan.___id;
      p.event = filterTemplate(plan.template);
      p.start = $utils.createGsonCompatibleTimestamp(new Date(plan.start));
      p.stop = $utils.createGsonCompatibleTimestamp(new Date(plan.stop));
      p.roomId = plan.roomId;
      p.savedDates = plan.savedDates;
      p.externalId = externalPlanId || null;
      p.autoApply = plan.autoApply;
      // Align angular model to the play model.
      if (!p.event.eventStateStandards) {
        p.event.eventStateStandards = $adapter.stateStandardsToPlayModel(p.event.stateStandards);
      }
      var u = url() + (plan.id ? '/' + plan.id : '');
      var opts = {
          method: 'POST',
          url: u,
          headers: credentials,
          data: p
        };
      $ajax.ajax(opts, function (err, data) {
        if (data && data.id) {
          var extId = data.externalId || '-1';
          $taffy.plans()({ ___id: oldId }).update({
            id: data.id,
            externalId: extId
          });
          d.resolve(data);
        } else
          d.reject(err);
      });
    }
    function deletePlan(plan, d) {
      //      var p = new LessonPlan();
      //      p.$delete({id:plan.id});
      var opts = {
          method: 'DELETE',
          url: new String(url() + '/' + plan.id),
          headers: credentials
        };
      $ajax.ajax(opts, function (err, data) {
        if (err) {
          console.log(err);
          d.reject(err);
        } else {
          if (plan.___id) {
            $taffy.plans()(plan.___id).remove();
          } else if (plan.id) {
            $taffy.plans()({ id: plan.id }).remove();
          }
          d.resolve(data);
        }
      });
    }
    function getPlans(d, ts) {
      var fac = $facility.getSync();
      var opts = {
          method: 'GET',
          url: url(ts),
          headers: credentials
        };
      $ajax.ajax(opts, function (err, plans) {
        if (!err) {
          if (plans && plans.length) {
            plans.forEach(function (p) {
              if (p.eventStr && p.eventStr.length && p.eventStr.length > 0) {
                try {
                  p.template = $adapter.createEvent(JSON.parse(p.eventStr), fac);
                  if (p.template.stateStandards) {
                    p.template.stateStandards = $adapter.stateStandardsToApiModel(p.template.stateStandards);
                  }
                  p.start = new Date(p.start).getTime();
                  //Date.fromISO(p.start).getTime();
                  p.stop = new Date(p.stop).getTime();
                  //Date.fromISO(p.stop).getTime();
                  p.externalId = p.externalId || '-1';
                  $taffy.plans().insert(p);
                } catch (e) {
                  console.log(e);
                }
              }
            });
          }
          d.resolve();
        } else {
          console.log(err);
          d.reject(err);
        }
      });
    }
    return {
      setCredentials: function (creds) {
        var d = $q.defer();
        credentials = creds;
        LessonPlan = $resource($krConfig.getAjaxBase() + '/lessonplan/:id', {}, {
          'get': {
            method: 'GET',
            headers: creds
          },
          'save': {
            method: 'POST',
            headers: creds
          },
          'query': {
            method: 'GET',
            isArray: true,
            headers: creds
          },
          'delete': {
            method: 'DELETE',
            headers: creds
          }
        });
        synchDB(d);
        return d.promise;
      },
      refreshPlans: function (timestamp) {
        var d = $q.defer();
        synchDB(d, timestamp);
        return d.promise;
      },
      put: function (plan, externalPlanId) {
        var d = $q.defer();
        // Replace any existing events (server always wins)
        var results = $taffy.plans()({ id: plan.id }).update(plan);
        if (!plan.id || 0 === results.count()) {
          // Didn't exist -> insert
          $taffy.plans().insert(plan);
        }
        savePlan(plan, d, externalPlanId);
        return d.promise;
      },
      deleteItem: function (item) {
        var d = $q.defer();
        deletePlan(item, d);
        return d.promise;
      },
      get: function (room_id, start, end) {
        var params = {};
        if (room_id) {
          params.roomId = room_id;
        }
        if (start && end) {
          if (start === end) {
            params.start = { 'lt': start + 1 };
            params.stop = { 'gt': end - 1 };
          } else {
            params.start = { 'gt': start + 1 };
            params.stop = { 'lt': end - 1 };
          }
        } else
          params.externalId = { 'is': '-1' };
        return $taffy.plans()(params).get();
      },
      isPlanSaved: function (ppPlan) {
        var params = { externalId: { 'is': ppPlan._id } };
        return $taffy.plans()(params).get();
      },
      findByExternalId: function (extId) {
        var params = { externalId: { 'is': extId } };
        return $taffy.plans()(params).get()[0];
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').service('$defaults', [
  '$taffy',
  function Defaults($taffy) {
    function getId(event) {
      return event.observed[0].id + '_' + event.eventType.id;
    }
    this.get = function (event) {
      var id = getId(event);
      return $taffy.defaults()({ id: id }).first() || event;
    };
    this.save = function (event) {
      event.id = getId(event);
      // Replace any existing events (server always wins)
      var results = $taffy.defaults()({ id: event.id }).update(event);
      if (0 === results.count()) {
        // Didn't exist -> insert
        $taffy.defaults().insert(event);
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').factory('$facebook', [
  '$krConfig',
  '$facility',
  '$appstate',
  '$messageBox',
  '$timeout',
  '$q',
  '$native',
  '$modal',
  '$translate',
  '$utils',
  '$ajax',
  function ($krConfig, $facility, $appstate, $messageBox, $timeout, $q, $native, $modal, $translate, $utils, $ajax) {
    function err(msg) {
      $timeout(function () {
        forge.notification.alert($translate.instant('AnErrorOccurred'), msg);
      });
    }
    function ok(msg, subj) {
      subj = subj || $translate.instant('ContentPostedSuccessfully');
      $timeout(function () {
        forge.notification.alert(subj, msg);
      });
    }
    var actions = ['publish_actions'];
    function _login() {
    }
    function _logout() {
    }
    function handleShareSuccess(response, msg) {
      console.log($translate.instant('SharedSuccessfully'));
      $timeout(function () {
        $appstate.msg($translate.instant('SharedSuccessfully'));
        ok($translate.instant('SharedDetails'), $translate.instant('ContentPostedSuccessfully'));
      }, 100);
    }
    // GRAPH API (OPEN GRAPH)
    function handleOGSuccess(response, share, errorHandler, url) {
      function krShare(id) {
      }
      // handle the response
      if (response.error) {
        var e = response.error;
        $utils.alert('Error saving media: ' + e.message + ' - ' + e.code + ' : ' + e.type);
      } else if (share) {
        // share it
        krShare(response.id);
      } else
        handleShareSuccess(response);
      console.log('[handleOGSuccess] done.');
    }
    function handleGenericError(e) {
      console.log('Error ...' + JSON.stringify(e));
      forge.notification.create('Error posting to Facebook');
    }
    function handlePublishOGError(e) {
      console.log('Error publishing ...' + JSON.stringify(e));
      var errorCode = e.code;
      console.log('Error code ...' + errorCode);
      /* TRIGGER.IO FORGE notification module:

       http://current-docs.trigger.io/modules/notification.html

       */
      forge.notification.create('Error posting to Facebook - Error code: ' + errorCode);
    }
    function _authorize() {
      var d = $q.defer();
      // forge.facebook.hasAuthorized(actions, 'friends', function(token_information) {
      //   d.resolve();
      // }, function(content) {
      //   console.log("Not Authorized:"+JSON.stringify(content));
      //   forge.facebook.authorize(actions, 'friends', function(token_information) {
      d.resolve();
      //   }, function(err) {
      //     console.log("Error authorizing:"+JSON.stringify(content));
      //     $utils.alert("Error:"+err.message);
      //     d.reject();
      //     //forge.notification.alert("An Error Occurred", err);
      //   });
      // });
      return d.promise;
    }
    function _publishOGAction(response, params, url) {
      var errorHandler = null;
      // Handle if we came in via a reauth.
      // Also avoid loops, set generic error
      // handler if already reauthed.
      if (!response || response.error) {
        errorHandler = handlePublishOGError;
      } else {
        errorHandler = handleGenericError;
      }
      function publish() {
        forge.oauth.authorize('facebook', function (endpoint) {
          forge.oauth.actionWithToken(endpoint, function (token) {
            $utils.log('Got token:' + token);
            var opts = {
                headers: { 'Authorization': 'Bearer ' + token.access },
                url: 'https://graph.facebook.com/v2.10/' + url,
                params: params,
                method: 'POST'
              };
            $ajax.ajax(opts, function (err, data) {
              $appstate.model.isRefreshing = false;
              console.log(response);
              if (!err) {
                handleOGSuccess(data, false, errorHandler, params.url);
              } else {
                errorHandler(err);
              }
            }, true);
          }, function (error) {
            forge.logging.error('forge.oauth.actionWithToken failed: ' + JSON.stringify(error));
            errorHandler(error);
          });
        }, function (error) {
          forge.logging.error('forge.oauth.authorize failed: ' + JSON.stringify(error));
          errorHandler(error);
        });
      }
      publish();
    }
    function shareVidUpload(url, msg, thumb) {
      getMessage(function (msg) {
        //$appstate.model.isRefreshing = true;
        if (msg !== null) {
          $appstate.model.isRefreshing = true;
          _authorize().then(function () {
            msg = updateMessage(msg);
            var params = {
                source: url,
                picture: thumb,
                file_url: url,
                description: msg
              };
            _publishOGAction(null, params, 'me/videos');
          }, function (err) {
            console.log(err);
            $appstate.model.isRefreshing = false;
          });
        }
      });
    }
    function updateMessage(msg) {
      return msg;  // todo fix message or figure out way to embed link to KR and school.
                   //if ( msg && msg.length > 0 ) msg += "\r\n\r\n";
                   //msg += $translate.instant("MemorySharedFrom")+" "+$facility.getSync().name;
                   //if ( $facility.getSync().website ) {
                   //  msg += "\r\n - "+$facility.getSync().website
                   //}
                   //
                   //return msg;
    }
    function photoShare(url) {
      getMessage(function (msg) {
        //$appstate.model.isRefreshing = true;
        if (msg !== null) {
          $appstate.model.isRefreshing = true;
          _authorize().then(function () {
            msg = updateMessage(msg);
            var params = {
                url: url,
                caption: msg
              };
            _publishOGAction(msg, params, 'me/photos');  //_sharePhoto(url);
          }, function (err) {
            $appstate.model.isRefreshing = false;
            forge.notification.alert($translate.instant('Error'), $translate.instant('FacebookLoginError'));
          });
        }
      });
    }
    function getMessage(cb) {
      var modalInstance = $modal.open({
          templateUrl: 'views/popups/facebookStatusPopup.html',
          controller: 'FacebookstatusCtrl'
        });
      modalInstance.result.then(function (msg) {
        cb(msg);
      });
    }
    // Public API here
    return {
      logout: function () {
        _logout();
      },
      sharePhoto: function (url, msg) {
        photoShare(url, msg);
      },
      shareVideo: function (url, msg, thumb) {
        shareVidUpload(url, msg, thumb);
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').service('$attendance', [
  '$appstate',
  '$events',
  '$adapter',
  '$facility',
  '$utils',
  '$q',
  '$localdb',
  '$online',
  '$eventAdapter',
  function attendance($appstate, $events, $adapter, $facility, $utils, $q, $localdb, $online, $eventAdapter) {
    function getAllChildren() {
      if ($facility.getSync().syncAllRooms) {
        var arr = _.filter(_.flatten(_.map($facility.getSync().rooms, function (room) {
            return room.children;
          })), function (k) {
            return k && k.id;
          });
        found = {};
        return _.filter(arr, filterFunc);
      } else {
        return $appstate.model.selectedRoom.children;
      }
    }
    var found;
    function getAllTeachers() {
      if ($facility.getSync().syncAllRooms) {
        var arr = _.filter(_.flatten(_.map($facility.getSync().rooms, function (room) {
            return room.teachers;
          })), function (k) {
            return k && k.id;
          });
        found = {};
        return _.filter(arr, filterFunc);
      } else {
        return $appstate.model.selectedRoom.teachers;
      }
    }
    function filterFunc(item) {
      if (!found[item.id]) {
        found[item.id] = true;
        return true;
      } else
        return false;
    }
    function _refresh(items, d, isTeacher, b, hideLoading, hideRoomId) {
      if (!hideLoading) {
        $appstate.model.isRefreshing = true;
      } else {
        $appstate.model.dashLoading = true;
      }
      var roomId = hideRoomId ? null : $appstate.model.selectedRoom.id;
      return $eventAdapter.getEvents(items, d, b, null, roomId, isTeacher, true, false);
    }
    this.refreshAll = function (hideLoading) {
      var d = $q.defer();
      if ($online.state()) {
        //_refresh(getAllChildren(), new Date(), false, true).then(function ()
        _refresh(getAllChildren(), $appstate.model.currentDate || new Date(), false, true).then(function () {
          //_refresh(getAllTeachers(), new Date(), true, true).then(function ()
          _refresh(getAllTeachers(), $appstate.model.currentDate, true, true).then(function () {
            $appstate.model.dashLoading = false;
            $appstate.model.isRefreshing = false;
            d.resolve();
            if (!hideLoading) {
              //              $appstate.setCurrentRoom($appstate.model.selectedRoom);
              $appstate.nav.goHome();
            }
          });
        });
      } else {
        d.resolve({ offline: true });
      }
      return d.promise;
    };
    ///////////////////////////////////////
    this.refresh = function (items, d, isTeacher, b, hideLoading, hideRoomId) {
      return _refresh(items, d, isTeacher, b, hideLoading, hideRoomId).then(function () {
        $appstate.model.isRefreshing = false;
      });
    };
    this.computeAttendance = function (kidlist, events) {
      return $adapter.computeAttendance(kidList, events);
    };
  }
]);
'use strict';
angular.module('kidreports2App').service('$reminders', [
  '$appstate',
  '$taffy',
  '$timeout',
  '$messageBox',
  '$utils',
  '$translate',
  function Reminders($appstate, $taffy, $timeout, $messageBox, $utils, $translate) {
    var db = $taffy.reminders();
    var intervals = [];
    function setupIntervals() {
      intervals = [];
      var hour = $translate.instant('Hour');
      var minutes = $translate.instant('Minutes');
      for (var i = 15; i < 60; i = i + 15) {
        intervals.push({
          'v': i,
          l: i + ' ' + minutes
        });
      }
      for (var i = 1; i < 3; i = i + 1) {
        intervals.push({
          'v': i * 60,
          l: i + ' ' + hour + (i > 1 ? 's' : '')
        });
        intervals.push({
          'v': i * 60 + 15,
          l: i + ' ' + hour + (i > 1 ? 's' : '') + ', 15 ' + minutes
        });
        intervals.push({
          'v': i * 60 + 30,
          l: i + ' ' + hour + (i > 1 ? 's' : '') + ', 30 ' + minutes
        });
        intervals.push({
          'v': i * 60 + 45,
          l: i + ' ' + hour + (i > 1 ? 's' : '') + ', 45 ' + minutes
        });
      }
      intervals.push({
        'v': 180,
        'l': '3 ' + hour + 's'
      });
      intervals.push({
        'v': 240,
        'l': '4 ' + hour + 's'
      });
    }
    //
    function start() {
      var ts = new Date().getTime();
      var reminders = db({ timestamp: { 'lt': ts } }).get();
      if (reminders && reminders.length > 0) {
        // block until we show the reminder.
        trigger(reminders[0]);
      } else {
        $timeout(start, 60000);
      }
    }
    function trigger(r) {
      var now = new Date().getTime();
      var msg = $translate.instant('ReminderDetails') + ': ' + r.message;
      var offset = r.timestamp - r.createTime;
      var newTS = new Date().getTime() + offset;
      var newTime = $utils.getTimeString($utils.getTimePart(new Date(newTS)));
      var question = $translate.instant('RepeatReminderTime').replace('&time&', newTime);
      $messageBox.open($translate.instant('ReminderNotification'), msg, {
        oktext: $translate.instant('Yes'),
        canceltext: $translate.instant('No')
      }, question).result.then(function (selectedItem) {
        // yes
        _remove(r);
        _add(r.message, newTS, now);
        // 60 second countdown
        $timeout(start, 60000);
      }, function () {
        // no
        _remove(r);
        // 60 second countdown
        $timeout(start, 60000);
      });
    }
    setupIntervals();
    function _add(str, time, now) {
      var obj = {
          message: str,
          timestamp: time,
          createTime: now
        };
      db.insert(obj);
    }
    function _remove(r) {
      if (r.___id) {
        db(r.___id).remove();
      }
    }
    this.start = function () {
      start();
    };
    this.addReminder = function (str, time, now) {
      _add(str, time, now);
    };
    this.deleteReminder = function (r) {
      _remove(r);
    };
    this.deleteAll = function () {
      db().remove();
    };
    this.getReminders = function () {
      return db().get();
    };
    this.getIntervals = function () {
      setupIntervals();
      return intervals;
    };
  }
]);
'use strict';
angular.module('kidreports2App').factory('$profileplanner', [
  '$ajax',
  '$q',
  '$native',
  '$cookies',
  function ($ajax, $q, $native, $cookies) {
    // Service logic
    // ...
    var endpoint = 'https://www.profileplanner.com';
    var config = {
        loginUrl: '/drupal/api/planner/user/login.json',
        tokenUrl: '/drupal/services/session/token',
        logoutUrl: '/drupal/logout',
        viewUrl: '/drupal/planner_files/event_qtip.php?id={{id}}&title={{title}}&etype=activity',
        roomChangeUrl: '/drupal/lessonplan/classrooms/current/{{roomId}}',
        plannerUrl: '/drupal/lessonplan?theme=mobile',
        eventDetailsUrl: '/drupal/planner_files/udp/event.php?id={{id}}',
        connectUrl: '/drupal/api/planner/system/connect.json'
      };
    var token;
    function _getToken(u, p) {
      var d = $q.defer();
      // try to get token, if no token, try to login.  If that fails, logout then login again.
      var opts = {
          method: 'GET',
          url: endpoint + config.tokenUrl,
          headers: {
            'X-CSRF-Token': token || '__dummyToken',
            'Content-Type': 'application/json'
          },
          contentType: 'application/json'
        };
      $ajax.ajax(opts, function (err, data) {
        if (err) {
          d.resolve(_login(u, p));
        } else {
          token = data;
          d.resolve(_connect(u, p));
        }
      }, true);
      return d.promise;
    }
    function _connect(u, p) {
      var d = $q.defer();
      var opts = {
          method: 'POST',
          url: endpoint + config.connectUrl,
          data: '',
          headers: {
            'X-CSRF-Token': token || '__dummyToken',
            'Content-Type': 'application/json'
          },
          contentType: 'application/json'
        };
      $ajax.ajax(opts, function (err, data) {
        if (err || data.user.uid === 0) {
          d.resolve(_login(u, p));
        } else {
          d.resolve(data);
        }
      }, true);
      return d.promise;
    }
    function _login(u, p) {
      var d = $q.defer();
      var params = {
          username: u,
          password: p
        };
      var opts = {
          method: 'POST',
          url: endpoint + config.loginUrl,
          data: params,
          contentType: 'application/json',
          headers: { 'Content-Type': 'application/json' }
        };
      $ajax.ajax(opts, function (err, data) {
        if (err) {
          // logout then login
          $ajax.ajax({
            method: 'GET',
            url: endpoint + config.logoutUrl,
            params: {}
          }, function (err, data) {
            if (err) {
              d.reject(err);
            } else {
              d.resolve(_login(u, p));
            }
          });
        } else {
          if (data.constructor === String) {
            data = JSON.parse(data);
          }
          token = data.token;
          var cName = data.session_name;
          var cValue = data.sessid;
          $cookies[cName] = cValue;
          //document.cookie = cName + "=" + cValue;
          d.resolve(data);
        }
      }, true);
      return d.promise;
    }
    function _getDetails(evtId) {
      var url = endpoint + config.eventDetailsUrl;
      url = url.replace('{{id}}', evtId);
      var d = $q.defer();
      var opts = {
          method: 'GET',
          url: url,
          contentType: 'application/json',
          data: '',
          headers: {
            'X-CSRF-Token': token || '__dummyToken',
            'Content-Type': 'application/json'
          }
        };
      $ajax.ajax(opts, function (err, data) {
        if (err) {
          d.reject(err);
        } else {
          d.resolve(data);
        }
      }, true);
      return d.promise;
    }
    function _logout() {
      var d = $q.defer();
      var opts = {
          method: 'POST',
          url: endpoint + config.logoutUrl,
          data: '',
          headers: {
            'X-CSRF-Token': token,
            'Content-Type': 'application/json'
          },
          contentType: 'application/json'
        };
      $ajax.ajax(opts, function (err, data) {
        if (err) {
          d.reject(err);
        } else {
          d.resolve(data);
        }
      }, true);
      return d.promise;
    }
    function _setRoom(rmId, cb) {
      var d = $q.defer();
      var opts = {
          method: 'GET',
          url: endpoint + config.roomChangeUrl.replace('{{roomId}}', rmId),
          data: '',
          headers: { 'X-CSRF-Token': token }
        };
      opts.withCredentials = true;
      $ajax.ajax(opts, function (err, data) {
        if (err) {
          d.reject(err);
        } else {
          d.resolve(data);
        }
      });
      return d.promise;
    }
    // Public API here
    return {
      login: function (user) {
        var u = user.props['profileplanner.user'];
        var p = user.props['profileplanner.pass'];
        return _getToken(u, p);
      },
      getEventDetails: function (user, evtId) {
        return this.login(user).then(function () {
          return _getDetails(evtId);
        });
      },
      setRoom: function (user, roomId) {
        return this.login(user).then(function () {
          return _setRoom(roomId);
        });
      },
      config: config,
      endpoint: endpoint
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('ProfileplannerCtrl', [
  '$scope',
  '$profileplanner',
  '$appstate',
  '$facility',
  '$ajax',
  '$native',
  '$timeout',
  function ($scope, $profileplanner, $appstate, $facility, $ajax, $native, $timeout) {
    $scope.title = 'Profile Planner';
    $scope.user = {};
    $scope.err = false;
    $scope.isNative = $native.isNative();
    $scope.update = function () {
      $profileplanner.updateAccount($scope.user);
    };
    var usr = $facility.getSync().currentUser;
    var __ppCBPointer = null;
    function login(retry) {
      $profileplanner.login(usr).then(function (data) {
        $appstate.model.isRefreshing = false;
        var ppUrl = $profileplanner.endpoint + $profileplanner.config.plannerUrl;
        if ($native.isNative()) {
          //forge.tabs.open(ppUrl);
          forge.tabs.openAdvanced({
            url: ppUrl,
            title: 'Profile Planner'
          }, function (modal) {
            modal.closed.addListener(function () {
              // go back home when closed
              $appstate.goHome();
            });
          });
        } else {
          if (__ppCBPointer)
            __ppCBPointer(data.session_name, data.sessid);
          $('#ppIframe')[0].src = ppUrl;
        }
      }, function (err) {
        handleErr(err);
      });
    }
    function handleErr(err) {
      console.log(err);
      $appstate.model.isRefreshing = false;
      $scope.err = true;
    }
    $scope.doLogin = function (cb) {
      if (cb)
        __ppCBPointer = cb;
      $('#ppIframe')[0].src = $profileplanner.endpoint;
      $appstate.model.isRefreshing = true;
      $timeout(function () {
        login(true);
      }, 500);
    };
    if ($native.isNative()) {
      // autologin since we don't have iframe issue
      // check if android and we are in portrait orientation
      if ($native.android() && window.innerWidth < window.innerHeight) {
        forge.notification.alert('Orientation Warning', 'For Best Results with Profile Planner, please rotate your device to a Landscape orientation.', $scope.doLogin, $scope.doLogin);
      } else {
        $scope.doLogin();
      }
    } else {
      var u = usr.props['profileplanner.user'];
      var p = usr.props['profileplanner.pass'];
      // global for iframe parent ref.
      __ppLoadPointer = webLogin;
      __ppCredentials = {
        user: u,
        pass: p
      };
    }
    function webLogin() {
      $scope.$apply(function () {
        $appstate.model.isRefreshing = true;
        $timeout(function () {
          var ppUrl = $profileplanner.endpoint + $profileplanner.config.plannerUrl;
          $('#ppIframe')[0].src = ppUrl;
          $appstate.model.isRefreshing = false;
        }, 2000);
      });
    }
  }
]);
'use strict';
angular.module('kidreports2App').controller('ProfileplannerdetailsCtrl', [
  '$scope',
  '$appstate',
  '$native',
  '$modalInstance',
  'viewPPItem',
  '$utils',
  function ($scope, $appstate, $native, $modalInstance, viewPPItem, $utils) {
    $scope.viewPPItem = viewPPItem;
    $scope.isNative = $native.isNative();
    $scope.vid = { url: viewPPItem.Movie_Link };
    $scope.showPDF = function (pdfLink) {
      forge.tabs.open(pdfLink);
    };
    $scope.ok = function () {
      //console.log($scope.vals.date);
      $modalInstance.close();
    };
    $scope.cancel = function () {
      $modalInstance.dismiss('cancel');
    };
    $scope.setVidUrl = function (url) {
      forge.media.videoPlay(url, function () {
      }, function (err) {
        $utils.alert('Error Playing Video');
      });
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('RemindersController', RemindersController);
RemindersController.$inject = [
  '$scope',
  '$reminders',
  '$messageBox',
  '$utils',
  '$translate'
];
/**
 * @function RemindersController
 * @author Joshua Belden <jcbelden@procaresoftware.com>
 * @param {*} $scope 
 * @param {*} $reminders 
 * @param {*} $messageBox 
 * @param {*} $utils 
 * @param {*} $translate 
 */
function RemindersController($scope, $reminders, $messageBox, $utils, $translate) {
  $scope.getTime = function (ts) {
    return $utils.getTimeString($utils.getTimePart(new Date(ts)));
  };
  $scope.deleteAll = function () {
    $messageBox.open($translate.instant('ConfirmDelete'), $translate.instant('DeleteAllRemindersConfirm'), {
      oktext: $translate.instant('Yes'),
      canceltext: $translate.instant('No')
    }, '', 'img/icon/delete.svg').result.then(function (selectedItem) {
      $scope.items = [];
      $reminders.deleteAll();
    }, function () {
    });
  };
  $scope.deleteItem = function (item) {
    $messageBox.open($translate.instant('ConfirmDelete'), $translate.instant('DeleteReminderConfirm'), {
      oktext: $translate.instant('Yes'),
      canceltext: $translate.instant('No')
    }, '', 'img/icon/delete.svg').result.then(function (selectedItem) {
      $scope.items = _.filter($scope.items, function (i) {
        return item.___id != i.___id;
      });
      $reminders.deleteReminder(item);
    }, function () {
    });
  };
  $scope.items = $reminders.getReminders();
}
'use strict';
angular.module('kidreports2App').controller('KioskCtrl', [
  '$scope',
  '$appstate',
  '$localdb',
  '$state',
  '$kiosk',
  '$facility',
  '$messageBox',
  '$krConfig',
  '$auth',
  '$translate',
  '$ajax',
  '$q',
  '$featureFlags',
  function ($scope, $appstate, $localdb, $state, $kiosk, $facility, $messageBox, $krConfig, $auth, $translate, $ajax, $q, $featureFlags) {
    // todo
    $localdb.set('kioskMode', 'true');
    $kiosk.set(null);
    $scope.passBox = '';
    $scope.fname = $facility.getSync().name;
    $scope.icon = $krConfig.s3Base + $facility.getSync().logoPath;
    $scope.numbers = [
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ];
    if ($featureFlags.getAllFlags().length === 0) {
      $featureFlags.establishCache();
    }
    $scope.closeKiosk = function () {
      $scope.showPassBox = true;
    };
    $scope.submitClose = function () {
      var password = $scope.passBox;
      // get username
      var username = $auth.getCredentialsSync().username;
      $auth.login(username, password).then(function (credentials) {
        $scope.showPassBox = false;
        $localdb.set('kioskMode', 'false');
        $appstate.nav.logout();
      }, function (err) {
        $scope.showPassBox = false;
        $appstate.msg('Incorrect Password!', true, 'warning');
      });
    };
    $scope.doLogin = function () {
      var usr = $kiosk.login(Number($scope.pin), $scope.initials, 'PARENT_ROLE');
      var queryLoginPin = $featureFlags.isFeatureEnabled('query-login-pin');
      if (!usr && queryLoginPin) {
        getParents().then(function (response) {
          finishLogin(response.parent);
        });
      } else {
        finishLogin(usr);
      }
    };
    function finishLogin(usr) {
      if (usr) {
        $kiosk.set(usr);
        var kids = $facility.findKidsByParent(usr);
        var pname = $translate.instant('ParentsChildren').replace('&name&', usr.firstName);
        $appstate.model.selectedRoom = {
          name: pname,
          children: kids
        };
        $facility.getSync().observed = kids;
        $facility.getSync().observer = usr;
        $state.go('home.action.kioskMain');
      } else {
        $messageBox.open($translate.instant('Error'), $translate.instant('InvalidPINInitials'), { oktext: 'Ok' });
        $scope.initials = '';
        $scope.pin = '';
      }
    }
    function getParents() {
      var d = $q.defer();
      var options = {
          method: 'Get',
          url: $krConfig.getAjaxBase() + '/api/pin',
          headers: { 'Authorization': 'Bearer ' + $auth.getCredentialsSync().token },
          params: {
            pin: $scope.pin,
            initials: $scope.initials
          }
        };
      $ajax.ajax(options, function (err, response) {
        if (err) {
          d.reject(err);
        } else {
          d.resolve(response);
        }
      });
      return d.promise;
    }
    $scope.addNumber = function (n) {
      if (!$scope.pin)
        $scope.pin = 0;
      $scope.pin = Number($scope.pin.toString() + n.toString());
    };
    $scope.removeNumber = function () {
      var str = $scope.pin.toString();
      $scope.pin = Number(str.substring(0, str.length - 1));
      if ($scope.pin === 0)
        $scope.pin = '';
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('KioskmainCtrl', [
  '$scope',
  '$appstate',
  '$state',
  '$localdb',
  '$kiosk',
  '$facility',
  '$krConfig',
  function ($scope, $appstate, $state, $localdb, $kiosk, $facility, $krConfig) {
    // todo
    $localdb.set('kioskMode', 'true');
    $scope.fname = $facility.getSync().name;
    $scope.icon = $krConfig.s3Base + $facility.getSync().logoPath;
    $scope.dirty = $appstate.model.dirty;
    var dirty = $scope.$on('attendanceDirty', function (event, args) {
        $scope.dirty = $appstate.model.dirty;
      });
    $scope.$on('$destroy', function () {
      dirty();
    });
    $scope.user = $kiosk.get();
    $scope.done = function () {
      $state.go('home.action.signInMain');
    };
    $scope.save = function () {
      $appstate.nav.nextState();
      $scope.done();
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('LearningStoriesCtrl', [
  '$scope',
  '$learningstories',
  '$auth',
  '$appstate',
  '$model',
  '$facility',
  '$modal',
  '$native',
  '$utils',
  '$msg',
  '$sce',
  '$state',
  '$timeout',
  '$messageBox',
  'moment',
  '$stateParams',
  function ($scope, $learningstories, $auth, $appstate, $model, $facility, $modal, $native, $utils, $msg, $sce, $state, $timeout, $messageBox, moment, $stateParams) {
    $scope.getItemHeight = function (item, index) {
      //Make evenly indexed items be 10px taller, for the sake of example
      return 105;  //(index % 2) === 0 ? 50 : 60;
    };
    $scope.allowParentsToMessageTeachers = $facility.getSync().allowParentsToMessageTeachers;
    $scope.disableTeacherToTeacherMessaging = $facility.getSync().disableTeacherToTeacherMessaging;
    var pageSize = 20;
    var page = 1;
    $scope.isNative = $native.isNative();
    $scope.isDash = $state.includes('home.dash');
    $scope.getRefreshClass = function () {
      return $scope.refreshing ? 'ion-refreshing' : 'ion-refresh';
    };
    $scope.openModal = function () {
      if ($scope.modal)
        return;
      $appstate.model.backButtonFunction = $scope.closeModal;
      $scope.modal = $modal.open({
        templateUrl: 'views/popups/viewStoryModal.html',
        controller: 'ViewstoryCtrl',
        resolve: {
          _scope: function () {
            return $scope;
          }
        }
      });
    };
    $scope.closeModal = function () {
      $scope.modal.close();
      $scope.modal = null;
      $scope.mainMessageScrollDivClass = 'scrollable';
      $appstate.model.backButtonFunction = null;
    };
    var userId = $facility.getSync().currentUser.id;
    var isParent = $facility.getSync().currentUser.role == 'PARENT_ROLE';
    $scope.isParent = isParent;
    $scope.currUser = $facility.getSync().currentUser;
    $scope.doRefresh = function () {
      $scope.refreshing = true;
      page = 1;
      $learningstories.sync().then(function () {
        $learningstories.get().then(setupstories);
        $scope.refreshing = false;
      });
    };
    $scope.getName = function (item) {
      var tch = $facility.findUser(item.user.id) || item.user;
      return tch.firstName + ' ' + tch.lastName;
    };
    $scope.addComment = function (commentId) {
      $scope.commenting = true;
      if (commentId)
        $scope.commentId = commentId;
      $timeout(function () {
        var ele = angular.element(document.querySelector('.modal'))[0];
        ele.scrollTop = ele.scrollHeight;
      });
    };
    $scope.toggleShowAll = function () {
      $scope.showAllMode = !$scope.showAllMode;
    };
    $scope.filterStories = function (s) {
      return $scope.showAllMode || $scope.isParent ? true : s.newCommentCount > 0;
    };
    $scope.cancelComment = function () {
      $scope.commenting = false;
      $scope.comment = '';
    };
    $scope.getChildNames = function (item) {
      var str = '';
      _.each(item.children, function (k) {
        var kid = $facility.findChild(k.id);
        if (str != '')
          str += ', ';
        str += kid.firstName + ' ' + kid.lastName;
      });
      return str;
    };
    $scope.view = function (item) {
      //alert(item);
      //$scope.showMessage(item);
      $scope.mainMessageScrollDivClass = '';
      $scope.story = item;
      $scope.mode = 'view';
      $learningstories.updateStoryStatus(item.id, 1).then(finishSetup);
      $timeout($scope.openModal, 10);
    };
    $scope.getDate = function (item) {
      if (item.toDate == null || item.fromDate == item.toDate)
        return moment(item.fromDate).format('DD-MM-YYYY');
      else
        return moment(item.fromDate).format('DD-MM-YYYY') + ' - ' + moment(item.toDate).format('DD-MM-YYYY');
    };
    var storyId = null;
    function doSetup() {
      // set up credentials
      var token = $auth.getCredentialsSync().token;
      var creds = { 'Authorization': 'Bearer ' + token };
      storyId = $stateParams.storyId;
      //$appstate.model.isRefreshing = true;
      $learningstories.setCredentials(creds).then(finishSetup);
    }
    function finishSetup() {
      $learningstories.get().then(setupstories);
    }
    function setupstories(stories) {
      $scope.stories = stories;
      var story = null;
      $appstate.model.storyBadge = _.filter($scope.stories, function (m) {
        return m.status === 0;
      }).length;
      if (storyId != null) {
        story = _.filter($scope.stories, function (s) {
          return s.id == storyId;
        });
        if (story && story.length) {
          story = story[0];
          $timeout(function () {
            $scope.view(story);
            storyId = null;
          });
        }
      }
    }
    $scope.getMore = function () {
      page++;
      $scope.refreshing = true;
      $learningstories.get(null, null, page, pageSize).then(function (messages) {
        setupstories(messages);
        $scope.refreshing = false;
      });
    };
    $scope.getCommentName = function (item) {
      var usr = $facility.findUser(item.fromUser.id);
      return usr.firstName + ' ' + usr.lastName;
    };
    $scope.getCommentDate = function (item) {
      return moment(item.sentDate).format('DD-MM-YYYY h:mm a');
    };
    $scope.mainMessageScrollDivClass = 'scrollable';
    $scope.saveComment = function (body) {
      $scope.isSending = true;
      $timeout(function () {
        realSaveMsg(body);
      }, 0);
    };
    function realSaveMsg(item) {
      //alert('saving message'+item);
      $learningstories.addComment($scope.commentId, $scope.story.id, item).then(function (m) {
        //alert('saved');
        $msg.msg('Comment Saved!');
        $scope.isSending = false;
        $scope.story = m;
        $scope.cancelComment();
      });
    }
    doSetup();
  }
]);
;
'use strict';
angular.module('kidreports2App').controller('MessagesController', MessagesController);
MessagesController.$inject = [
  '$scope',
  '$messages',
  '$auth',
  '$appstate',
  '$facility',
  '$native',
  '$utils',
  '$state',
  '$timeout',
  '$translate',
  '$krConfig',
  '$uiAlert',
  '$messageBox',
  'moment'
];
/**
* @function MessagesController
* @author Joshua Belden <jcbelden@procaresoftware.com>
* @param $scope
* @param $messages
* @param $auth
* @param $appstate
* @param $facility
* @param $native
* @param $utils
* @param $state
* @param $timeout
* @param $translate
* @param $krConfig
* @param $uiAlert
* @param moment
* @constructor
 */
function MessagesController($scope, $messages, $auth, $appstate, $facility, $native, $utils, $state, $timeout, $translate, $krConfig, $uiAlert, $messageBox, moment) {
  // Local Declarations
  var currentFacility = $facility.getSync();
  var facilityIsLite = $facility.isLite();
  var pageSize = 20;
  var page = 1;
  var userId = currentFacility.currentUser.id;
  $appstate.inboxMode = $appstate.inboxMode !== undefined ? $appstate.inboxMode : true;
  $scope.shared = $appstate;
  $scope.preventWriteActivity = currentFacility.preventWriteActivity;
  $scope.loading = true;
  $scope.isParent = currentFacility.currentUser.role == 'PARENT_ROLE';
  $scope.isNative = $native.isNative();
  $scope.disableTeacherToTeacherMessaging = currentFacility.disableTeacherToTeacherMessaging;
  $scope.allowParentsToMessageTeachers = currentFacility.allowParentsToMessageTeachers;
  $scope.currUser = currentFacility.currentUser;
  $scope.toOptions = [];
  $scope.mainMessageScrollDivClass = 'scrollable';
  $scope.isAdminIntegration = $appstate.isAdminIntegration;
  $scope.getCurrentUserPhotoPath = function () {
    var currentUser = currentFacility.currentUser;
    return $utils.createPhotoUrl($krConfig.s3Base, currentUser.photoPath);
  };
  $scope.navigateHome = function () {
    $appstate.goHome();
  };
  $scope.navigateNewMessage = function () {
    $state.go('messageNew');
  };
  $scope.clearFile = function () {
    angular.forEach(angular.element('input[type=\'file\']'), function (inputElem) {
      if (inputElem.name == 'messageFile') {
        angular.element(inputElem).val(null);
        $scope.messageFile = null;
      }
    });
    return false;
  };
  $scope.picture = function () {
    $messages.uploadPhoto().then(function (res) {
      $timeout(function () {
        var o = JSON.parse(res);
        $scope.image = { realURL: o.link };
      });
    }, function (err) {
    });
  };
  $scope.showFileButton = function () {
    return !$native.isNative() || $scope.sendTo.type != 'PARENT';
  };
  $scope.allowNewMessage = function () {
    if ($scope.preventWriteActivity) {
      return false;
    }
    // Admin integration requires users to go through the admin to create new messages.
    if ($scope.isAdminIntegration) {
      return false;
    }
    // Parents can always send new messages.
    if ($scope.isParent) {
      return true;
    }
    // Admins can always send new messages. Note that this admin check is necessary
    //  before the current user's canSendMessage property, because admins will have
    //  that set to true, but the facility sending for teachers will override it.
    if ($facility.isAdmin()) {
      return true;
    }
    // Determine if teachers are allowed to send messages
    return currentFacility.allowAllTeacherFamilyMessaging === undefined ? currentFacility.currentUser.canSendMessage : currentFacility.allowAllTeacherFamilyMessaging;
  };
  function loadMessages() {
    $scope.loading = true;
    $messages.get(null, null, page, pageSize).then(function (messages) {
      initMessages(messages);
      $scope.loading = false;
    });
    $messages.getUnreadMessageCount().then(function (response) {
      $appstate.model.messageBadge = response.count;
    });
  }
  $scope.getMore = function () {
    $scope.loading = true;
    page++;
    $messages.get(null, null, page, pageSize).then(function (messages) {
      initMessages(messages);
      $scope.loading = false;
    });
  };
  $scope.toggleInbox = function () {
    $appstate.inboxMode = !$appstate.inboxMode;
    $scope.mainlist = $appstate.inboxMode ? $scope.received : $scope.sent;
  };
  $scope.confirmDelete = function (message) {
    $messageBox.open('Confirm Delete', '', {
      oktext: 'Yes',
      canceltext: 'No'
    }, 'Are you sure you want to delete this message?', 'img/icon/delete.svg').result.then(function () {
      $messages.put(message, 3).then(loadMessages);
    }, function () {
    });
  };
  var loadMessageDelay;
  $scope.confirmMarkAllRead = function () {
    $messageBox.open('Confirm Mark Messages Read', '', {
      oktext: 'Yes',
      canceltext: 'No'
    }, 'Are you sure you want to mark ' + $scope.mainlist.length + ' message(s) read?', 'img/icon/message-2.svg').result.then(function () {
      _.each($scope.mainlist, function (message) {
        $messages.put(message, 1).then(function () {
          if (loadMessageDelay) {
            clearTimeout(loadMessageDelay);
          }
          loadMessageDelay = setTimeout(loadMessages, 100);
        });
      });
    });
  };
  $scope.confirmDeleteAll = function () {
    $messageBox.open('Confirm Delete Messages', '', {
      oktext: 'Yes',
      canceltext: 'No'
    }, 'Are you sure you want to delete ' + $scope.mainlist.length + ' message(s)?', 'img/icon/delete.svg').result.then(function () {
      _.each($scope.mainlist, function (message) {
        $messages.put(message, 3).then(function () {
          if (loadMessageDelay) {
            clearTimeout(loadMessageDelay);
          }
          loadMessageDelay = setTimeout(loadMessages, 100);
        });
      });
    });
  };
  $scope.viewMessage = function (message) {
    $state.go('message', { messageId: message.id });
  };
  $scope.close = function () {
    $messages.getUnreadMessageCount().then(function (response) {
      $appstate.model.messageBadge = response.count;
    });
    if ($scope.isParent) {
      $state.go('parent-home');
    } else {
      $state.go('home.dash');
    }
  };
  $scope.getAlertClass = function (alert) {
    if (alert.type === $uiAlert.ALERT_TYPE_SUCCESS) {
      return 'alert alert-success';
    } else if (alert.type === $uiAlert.ALERT_TYPE_WARNING) {
      return 'alert alert-warning';
    } else if (alert.type === $uiAlert.ALERT_TYPE_ERROR) {
      return 'alert alert-danger';
    } else {
      return 'alert';
    }
  };
  function displayAlerts() {
    var alerts = $uiAlert.popAlertsFor('MESSAGE_CENTER');
    if (alerts) {
      $scope.alerts = alerts;
      $timeout(function () {
        $scope.alerts = null;
      }, 3000);
    }
  }
  function configureMessageCredentials() {
    var token = $auth.getCredentialsSync().token;
    var creds = { 'Authorization': 'Bearer ' + token };
    $messages.setCredentials(creds).then(loadMessages, function (err) {
      console.log(err);
    });
  }
  function initMessages(messages) {
    $scope.messages = messages;
    $scope.messages.forEach(function (message) {
      var user = $facility.findUser(message.fromUserID);
      message.from = user;
      message.toUsers = _.filter(_.map(message.toUserIds, function (id) {
        return $facility.findUser(id);
      }), function (i) {
        return i;
      });
      message.sentDate = moment(new Date(message.sentDate)).format('MM/DD/YYYY h:mm A');
      if (!message.subject || !message.subject.trim()) {
        message.subject = 'No subject';
      }
    });
    $scope.convertUsersToString = $utils.convertUsersToString;
    $scope.sent = _.filter($scope.messages, function (msg) {
      return msg.fromUserID == userId;
    });
    $scope.received = _.filter($scope.messages, function (msg) {
      return msg.fromUserID != userId;
    });
    $scope.mainlist = $appstate.inboxMode ? $scope.received : $scope.sent;
  }
  function init() {
    // Message controller requires the observed children to be an array. In some cases, when users load the message center
    //  from the admin portal via an iframe, observed is not defined. This fixes it so it at least won't throw an error.
    if (!$appstate.model.observed) {
      $appstate.model.observed = [];
    }
    if (currentFacility.currentUser.linkedIds && currentFacility.currentUser.linkedIds.length) {
      $scope.toOptions.push({
        name: $translate.instant('SchoolDirector') + $translate.instant('OriginalParens'),
        type: 'FACILITY_ADMIN:' + currentFacility.id
      });
      $scope.toOptions.push({
        name: $translate.instant('SchoolDirector') + $translate.instant('LinkedParens'),
        type: 'FACILITY_ADMIN:' + currentFacility.currentUser.linkedIds[0]
      });
    } else {
      $scope.toOptions.push({
        name: $translate.instant('SchoolDirector'),
        type: 'FACILITY_ADMIN'
      });
    }
    if ($scope.isParent) {
      if ($scope.allowParentsToMessageTeachers && !facilityIsLite) {
        $appstate.model.observed.forEach(function (child) {
          var name = child.firstName;
          var extra = '';
          if (currentFacility.currentUser.linkedIds.length > 0) {
            var isOrig = $facility.findChild(child.id).facilityId == currentFacility.id;
            extra = isOrig ? $translate.instant('OriginalParens') : $translate.instant('LinkedParens');
          }
          $scope.toOptions.push({
            name: $translate.instant('ChildsTeachers').replace('&child&', name) + extra,
            type: 'TEACHER',
            value: child.id
          });
        });
      }
      if (currentFacility.enableSendToIndividualDirector) {
        currentFacility.users.forEach(function (c) {
          if (c.legacyRole == 'FACILITY_ADMIN_ROLE' && c.enabled) {
            $scope.toOptions.push({
              name: c.firstName + ' ' + c.lastName,
              type: 'ADMINDIRECT',
              value: c.id
            });
          }
        });
      }
    } else {
      // add teachers/roms in here if they have access.
      if (!$scope.disableTeacherToTeacherMessaging && !facilityIsLite) {
        // add options for all kids that this teacher has access to, as well as system admin
        $scope.toOptions.push({
          name: $translate.instant('Room'),
          type: 'ROOM'
        });
        $scope.toRoomOptions = [];
        currentFacility.rooms.forEach(function (c) {
          if (c.name != $translate.instant('AllChildren'))
            $scope.toRoomOptions.push({
              name: c.name,
              type: 'ROOM',
              value: c.id
            });
        });
        $scope.toOptions.push({
          name: $translate.instant('Teacher'),
          type: 'TEACHERDIRECT'
        });
        $scope.toTeacherOptions = [];
        var foundTeacher = {};
        var teachers = _.sortBy(_.filter(currentFacility.users, function (t) {
            return t.role == 'TEACHER_ROLE';
          }), 'firstName');
        teachers.forEach(function (c) {
          if (c.role == 'TEACHER_ROLE' && c.enabled && !foundTeacher[c.id]) {
            $scope.toTeacherOptions.push({
              name: c.firstName + ' ' + c.lastName,
              type: 'TEACHERDIRECT',
              value: c.id
            });
            foundTeacher[c.id] = true;
          }
        });
        $scope.sendToTeacher = $scope.toTeacherOptions[0];
        $scope.sendToRoom = $scope.toRoomOptions[0];
      }
      if (currentFacility.allowAllTeacherFamilyMessaging === undefined ? currentFacility.currentUser.canSendMessage : currentFacility.allowAllTeacherFamilyMessaging) {
        // add options for all kids that this teacher has access to, as well as system admin
        $appstate.model.observed.forEach(function (child) {
          $scope.toOptions.push({
            name: $translate.instant('ChildsParents').replace('&child&', child.firstName + ' ' + child.lastName),
            type: 'PARENT',
            value: child.id
          });
        });
      }
    }
    $scope.sendTo = $scope.toOptions[0];
  }
  init();
  configureMessageCredentials();
  displayAlerts();
}
'use strict';
angular.module('kidreports2App').controller('MessageController', MessageController);
MessageController.$inject = [
  '$scope',
  '$state',
  '$stateParams',
  '$messages',
  '$facility',
  '$utils',
  '$appstate'
];
/**
* @function MessageController
* @author Joshua Belden <jcbelden@procaresoftware.com>
* @param $scope
* @param $state
* @param $stateParams
* @param $messages
* @param $facility
* @param $utils
* @param $appstate
* @constructor
*/
function MessageController($scope, $state, $stateParams, $messages, $facility, $utils, $appstate) {
  var currentFacility = $facility.getSync();
  var messageId = parseInt($stateParams.messageId);
  $scope.preventWriteActivity = currentFacility.preventWriteActivity;
  $scope.currentUser = currentFacility.currentUser;
  $scope.isParent = $scope.currentUser.role == 'PARENT_ROLE';
  $scope.allowParentsToMessageTeachers = currentFacility.allowParentsToMessageTeachers;
  $scope.allowAllTeacherFamilyMessaging = currentFacility.allowAllTeacherFamilyMessaging === undefined ? currentFacility.currentUser.canSendMessage : currentFacility.allowAllTeacherFamilyMessaging;
  $scope.message = $messages.find(messageId);
  $scope.isSentFromCurrentUser = $scope.message.fromUserID === $scope.currentUser.id;
  $scope.isAdminIntegration = $appstate.isAdminIntegration;
  markMessageRead($scope.message);
  /**
   * Marks the specified message as read.
   * @param {Message to mark read.} message 
   */
  function markMessageRead(message) {
    $messages.put(message, 1).then(function () {
    });
  }
  $scope.convertUsersToString = $utils.convertUsersToString;
  /**
   * Formats a url to be displayed in the message list of urls.
   */
  $scope.formatUrl = function (link) {
    var result = link.substr(link.lastIndexOf('/') + 1);
    if (result.length > 15) {
      result = result.substr(0, 13) + '...';
    }
    return result;
  };
  /**
   * Navigates back to the message list.
   */
  $scope.navigateBack = function () {
    $state.go('messages');
  };
  /**
   * Determines if the current message is allowed to be replied to.
   */
  $scope.allowReply = function (message) {
    if ($scope.preventWriteActivity) {
      return false;
    }
    if ($scope.message.fromUserID === $scope.currentUser.id) {
      return false;
    }
    var fromAdmin = message && message.from && (message.from.legacyRole == 'FACILITY_ADMIN_ROLE' || message.from.legacyRole == 'REGION_ADMIN_ROLE' || message.from.legacyRole == 'COMPANY_ADMIN_ROLE' || message.from.legacyRole == 'SYSTEM_ADMIN_ROLE');
    // Allow reply when user is an admin or the email is from an admin.
    if (fromAdmin || $facility.isAdmin()) {
      return true;
    }
    if ($scope.isParent) {
      return $scope.allowParentsToMessageTeachers;
    }
    return $scope.allowAllTeacherFamilyMessaging;
  };
  /** 
   * Navigates to the reply message screen.
   */
  $scope.reply = function (messageId) {
    $state.go('messageReply', { messageId: messageId });
  };
}
'use strict';
angular.module('kidreports2App').controller('MessageNewController', MessageNewController);
MessageNewController.$inject = [
  '$scope',
  '$state',
  '$appstate',
  '$facility',
  '$messages',
  '$krConfig',
  '$utils',
  '$translate',
  '$native',
  '$uiAlert',
  '$timeout'
];
/**
* @function MessageNewController
* @author Joshua Belden <jcbelden@procaresoftware.com>
* @param $scope
* @param $state
* @param $appstate
* @param $facility
* @param $messages
* @param $krConfig
* @param $utils
* @param $translate
* @param $native
* @param $uiAlert
* @param $timeout
* @constructor
 */
function MessageNewController($scope, $state, $appstate, $facility, $messages, $krConfig, $utils, $translate, $native, $uiAlert, $timeout) {
  // Local Declarations
  var currentFacility = $facility.getSync();
  var facilityIsLite = $facility.isLite();
  if (currentFacility.preventWriteActivity) {
    $scope.navigateBack();
  }
  // Scope Declarations
  $scope.isParent = currentFacility.currentUser.role == 'PARENT_ROLE';
  $scope.isNative = $native.isNative();
  $scope.disableTeacherToTeacherMessaging = currentFacility.disableTeacherToTeacherMessaging;
  $scope.allowParentsToMessageTeachers = currentFacility.allowParentsToMessageTeachers;
  $scope.toOptions = [];
  $scope.toRoomOptions = [];
  $scope.toTeacherOptions = [];
  $scope.toParentOptions = [];
  $scope.sendTo = [];
  $scope.sendToTeacher = [];
  $scope.sendToParent = [];
  $scope.sendToRoom = [];
  $scope.message = {
    sendTo: { value: '' },
    subject: ''
  };
  var options = buildSendToOptions(currentFacility, facilityIsLite, $translate, $scope.isParent, $scope.disableTeacherToTeacherMessaging, $scope.allowParentsToMessageTeachers);
  $scope.toOptions = options.toOptions;
  $scope.sendTo = options.toOptions[0];
  $scope.toRoomOptions = options.toRoomOptions;
  $scope.sendToRoom = options.toRoomOptions[0];
  $scope.toTeacherOptions = options.toTeacherOptions;
  $scope.sendToTeacher = options.toTeacherOptions[0];
  $scope.toParentOptions = options.toParentOptions;
  $scope.sendToParent = options.toParentOptions[0];
  $scope.messageIsValid = false;
  /**
   * Builds an object containing options to send the message to.
   * @param {The current facility.} facility 
   * @param {A translation service.} translateService 
   * @param {Whether or not the current user is a parent.} isParent 
   * @param {Whether or not teachers are allowed to send messages to other teachers.} disableTeacherToTeacherMessaging 
   */
  function buildSendToOptions(facility, facilityIsLite, translateService, isParent, disableTeacherToTeacherMessaging, allowParentsToMessageTeachers) {
    var result = {
        toOptions: [],
        toRoomOptions: [],
        toTeacherOptions: [],
        toParentOptions: []
      };
    if (facility.currentUser.linkedIds && facility.currentUser.linkedIds.length) {
      result.toOptions.push({
        name: translateService.instant('SchoolDirector') + translateService.instant('OriginalParens'),
        type: 'FACILITY_ADMIN:' + facility.id
      });
      result.toOptions.push({
        name: translateService.instant('SchoolDirector') + translateService.instant('LinkedParens'),
        type: 'FACILITY_ADMIN:' + facility.currentUser.linkedIds[0]
      });
    } else {
      result.toOptions.push({
        name: translateService.instant('SchoolDirector'),
        type: 'FACILITY_ADMIN'
      });
    }
    if (isParent) {
      if (allowParentsToMessageTeachers && !$facility.isLite()) {
        $appstate.model.observed.forEach(function (child) {
          var name = child.firstName;
          var extra = '';
          if (facility.currentUser.linkedIds.length > 0) {
            var isOrig = $facility.findChild(child.id).facilityId == facility.id;
            extra = isOrig ? $translate.instant('OriginalParens') : $translate.instant('LinkedParens');
          }
          result.toOptions.push({
            name: $translate.instant('ChildsTeachers').replace('&child&', name) + extra,
            type: 'TEACHER',
            value: child.id
          });
        });
      }
      if (facility.enableSendToIndividualDirector) {
        facility.users.forEach(function (user) {
          if (user.legacyRole == 'FACILITY_ADMIN_ROLE' && user.enabled && !user.archived && !user.deleted) {
            result.toOptions.push({
              name: user.firstName + ' ' + user.lastName,
              type: 'ADMINDIRECT',
              value: user.id
            });
          }
        });
      }
    } else {
      // add teachers/roms in here if they have access.
      if (!disableTeacherToTeacherMessaging && !facilityIsLite) {
        result.toOptions.push({
          name: translateService.instant('Room'),
          type: 'ROOM'
        });
        result.toRoomOptions = [];
        facility.rooms.forEach(function (room) {
          if (room.name != translateService.instant('AllChildren'))
            result.toRoomOptions.push({
              name: room.name,
              type: 'ROOM',
              value: room.id
            });
        });
        result.toOptions.push({
          name: translateService.instant('Teacher'),
          type: 'TEACHERDIRECT'
        });
        result.toTeacherOptions = [];
        var foundTeacher = {};
        var teachers = _.sortBy(_.filter(facility.users, function (teacher) {
            return teacher.role == 'TEACHER_ROLE';
          }), 'firstName');
        teachers.forEach(function (teacher) {
          if (teacher.role == 'TEACHER_ROLE' && teacher.enabled && !teacher.archived && !teacher.deleted && !foundTeacher[teacher.id]) {
            result.toTeacherOptions.push({
              name: teacher.firstName + ' ' + teacher.lastName,
              type: 'TEACHERDIRECT',
              value: teacher.id
            });
            foundTeacher[teacher.id] = true;
          }
        });
      }
      if (facility.allowAllTeacherFamilyMessaging === undefined ? facility.currentUser.canSendMessage : facility.allowAllTeacherFamilyMessaging) {
        result.toOptions.push({
          name: translateService.instant('Parent'),
          type: 'PARENT'
        });
        facility.children.forEach(function (child) {
          result.toParentOptions.push({
            name: translateService.instant('ChildsParents').replace('&child&', child.firstName + ' ' + child.lastName),
            type: 'PARENT',
            value: child.id
          });
        });
      }
    }
    return result;
  }
  /**
   * Gets the current users profile photo's path.
   */
  $scope.getCurrentUserPhotoPath = function () {
    var currentUser = currentFacility.currentUser;
    return $utils.createPhotoUrl($krConfig.s3Base, currentUser.photoPath);
  };
  /**
   * Attaches the first file found in the specified element to the message.
   */
  $scope.setFile = function (element) {
    $scope.$apply(function () {
      $scope.messageFile = _.first(element.files);
    });
  };
  /**
   * Removes all attached files from the message.
   */
  $scope.clearFile = function () {
    angular.forEach(angular.element('input[type=\'file\']'), function (inputElem) {
      if (inputElem.name == 'messageFile') {
        angular.element(inputElem).val(null);
        $scope.messageFile = null;
      }
    });
    return false;
  };
  $scope.setPicture = function () {
    $messages.uploadPhoto().then(function (res) {
      $timeout(function () {
        var o = JSON.parse(res);
        $scope.image = { realURL: o.link };  //$scope.isSending=false;
      });
    }, function (err) {
    });
  };
  $scope.clearPicture = function () {
    $scope.image = null;
  };
  /**
   * Validates that the specified message is good to send.
   */
  $scope.validateMessage = function (message) {
    $scope.messageIsValid = message.body && message.body.length > 0 && message.subject && message.subject.length > 0;
    return $scope.messageIsValid;
  };
  /**
   * Sends the specified message.
   */
  $scope.sendMessage = function (message) {
    if (!$scope.validateMessage(message)) {
      $uiAlert.pushErrorAlert('Could not send message without a body or subject.', 'MESSAGE_CENTER');
      $scope.navigateBack();
    }
    var sendTo = $scope.sendTo.type;
    var sendToValue = $scope.sendTo.value;
    if (sendTo == 'ROOM') {
      sendToValue = $scope.sendToRoom.value;
      // "ROOM" is a legacy recipient type for parents to message teachers
      // If the sender is a teacher, target parents
      if (!$scope.isParent) {
        sendTo = 'ROOM_PARENTS';
      }
    } else if (sendTo == 'TEACHERDIRECT') {
      sendToValue = $scope.sendToTeacher.value;
    } else if (sendTo == 'PARENT') {
      sendToValue = $scope.sendToParent.value;
    }
    if (sendToValue) {
      sendTo += ':' + sendToValue;
    }
    if ($scope.messageFile) {
      message.attachment = $scope.messageFile;
      $scope.messageFile = null;
    }
    if ($scope.image) {
      message.image = $scope.image;
      $scope.image = null;
    }
    $messages.put(message, null, sendTo).then(function (m) {
      $uiAlert.pushSuccessAlert('Message Successfully Sent!', 'MESSAGE_CENTER');
      $scope.navigateBack();
    }, function () {
      $uiAlert.pushErrorAlert('There was an issue sending your message.', 'MESSAGE_CENTER');
      $scope.navigateBack();
    });
  };
  /**
   * Navigates back to the message list.
   */
  $scope.navigateBack = function () {
    $state.go('messages');
  };
}
'use strict';
angular.module('kidreports2App').controller('MessageReplyController', MessageReplyController);
MessageReplyController.$inject = [
  '$scope',
  '$state',
  '$stateParams',
  '$facility',
  '$messages',
  '$krConfig',
  '$utils',
  '$native',
  '$uiAlert',
  '$timeout',
  '$appstate'
];
/**
* @function MessageReplyController
* @author Joshua Belden <jcbelden@procaresoftware.com>
* @param $scope
* @param $state
* @param $stateParams
* @param $facility
* @param $messages
* @param $krConfig
* @param $utils
* @param $native
* @param $uiAlert
* @param $timeout
* @param $appstate
* @constructor
 */
function MessageReplyController($scope, $state, $stateParams, $facility, $messages, $krConfig, $utils, $native, $uiAlert, $timeout, $appstate) {
  // Local Declarations
  var currentFacility = $facility.getSync();
  var currentUser = currentFacility.currentUser;
  var messageId = parseInt($stateParams.messageId);
  if (currentFacility.preventWriteActivity) {
    $scope.navigateBack();
  }
  // Scope Declarations
  $scope.isParent = isUserParent(currentUser);
  $scope.isNative = $native.isNative();
  $scope.originalMessage = $messages.find(messageId);
  $scope.message = buildReply($scope.originalMessage, currentUser, $scope.isParent);
  $scope.messageIsValid = true;
  $scope.isAdminIntegration = $appstate.isAdminIntegration;
  /**
   * Builds a new message in reply to the specified original message.
   * @param {The message to reply to.} originalMessage 
   * @param {The user that is replying to the original message.} currentUser 
   * @param {Whether or not the user replying is a parent.} isParent 
   */
  function buildReply(originalMessage, currentUser, isParent) {
    var reply = {
        fromUser: { id: currentUser.id },
        from: currentUser,
        subject: 'Re: ' + originalMessage.subject,
        body: '\r\n\r\n>>>>>>Original Message>>>>>\r\n\r\n' + originalMessage.body,
        recipientList: [{ id: originalMessage.fromUserID }]
      };
    // Only teachers reply to all
    if (!isParent) {
      originalMessage.toUsers.forEach(function (user) {
        if (user && user.id != currentUser.id) {
          reply.recipientList.push({ id: user.id });
        }
      });
    }
    reply.toUsers = _.map(reply.recipientList, function (user) {
      return $facility.findUser(user.id);
    });
    return reply;
  }
  /**
   * Determines whether or not the current user is a parent.
   * @param {The currently authenticated user.} user 
   */
  function isUserParent(user) {
    return user.role == 'PARENT_ROLE';
  }
  $scope.convertUsersToString = $utils.convertUsersToString;
  /**
   * Gets the current users profile photo's path.
   */
  $scope.getCurrentUserPhotoPath = function () {
    var currentUser = currentFacility.currentUser;
    return $utils.createPhotoUrl($krConfig.s3Base, currentUser.photoPath);
  };
  /**
   * Attaches the first file found in the specified element to the message.
   */
  $scope.setFile = function (element) {
    $scope.$apply(function () {
      $scope.messageFile = _.first(element.files);
    });
  };
  /**
   * Removes all attached files from the message.
   */
  $scope.clearFile = function () {
    angular.forEach(angular.element('input[type=\'file\']'), function (inputElem) {
      if (inputElem.name == 'messageFile') {
        angular.element(inputElem).val(null);
        $scope.messageFile = null;
      }
    });
    return false;
  };
  $scope.setPicture = function () {
    $messages.uploadPhoto().then(function (res) {
      $timeout(function () {
        var o = JSON.parse(res);
        $scope.image = { realURL: o.link };
      });
    }, function (err) {
    });
  };
  $scope.clearPicture = function () {
    $scope.image = null;
  };
  /**
   * Validates that the specified message is good to send.
   */
  $scope.validateMessage = function (message) {
    $scope.messageIsValid = message.body && message.body.length > 0 && message.subject && message.subject.length > 0;
    return $scope.messageIsValid;
  };
  /**
   * Sends the specified message.
   */
  $scope.sendMessage = function (message) {
    if (!$scope.validateMessage(message)) {
      $uiAlert.pushErrorAlert('Could not send message without a body or subject.', 'MESSAGE_CENTER');
      $scope.navigateBack();
    }
    if ($scope.messageFile) {
      message.attachment = $scope.messageFile;
      $scope.messageFile = null;
    }
    if ($scope.image) {
      message.image = $scope.image;
      $scope.image = null;
    }
    $messages.put(message, null, null).then(function (m) {
      $uiAlert.pushSuccessAlert('Message Successfully Sent!', 'MESSAGE_CENTER');
      $scope.navigateHome();
    }, function () {
      $uiAlert.pushErrorAlert('There was an issue sending your message.', 'MESSAGE_CENTER');
      $scope.navigateHome();
    });
  };
  /**
   * Navigates back to the main page.
   */
  $scope.navigateHome = function () {
    $state.go('messages');
  };
  /**
   * Navigates back to the message list.
   */
  $scope.navigateBack = function (messageId) {
    $state.go('message', { messageId: messageId });
  };
}
'use strict';
angular.module('kidreports2App').factory('$kiosk', [
  '$facility',
  function ($facility) {
    var _user = null;
    var _this = {
        get: function () {
          return _user;
        },
        set: function (u) {
          _user = u;
        },
        login: function (pin, initials, myRole) {
          if (!pin || pin < 1000 || !initials || initials.length < 2)
            return null;
          var _usr = null;
          $facility.getSync().users.forEach(function (usr) {
            if (usr.role === myRole && usr.pin === pin && usr.initials && usr.initials.toLowerCase() === initials.toLowerCase()) {
              _usr = usr;
            }
          });
          return _usr ? $facility.findUser(_usr.id) : null;
        }
      };
    return _this;
  }
]);
'use strict';
angular.module('kidreports2App').factory('$messages', [
  '$taffy',
  '$ajax',
  '$utils',
  '$facility',
  '$q',
  '$krConfig',
  '$timeout',
  '$rootScope',
  '$native',
  '$online',
  '$messageBox',
  '$auth',
  function ($taffy, $ajax, $utils, $facility, $q, $krConfig, $timeout, $rootScope, $native, $online, $messageBox, $auth) {
    var credentials = {};
    function url() {
      return $krConfig.getAjaxBase() + '/messages';
    }
    function synchDB(d) {
      $taffy.messages()().remove();
      getMessages(d);
    }
    function upload(file) {
      var formData = new FormData();
      formData.append('file', file);
      var hType = undefined;
      var opts = {
          method: 'POST',
          url: url() + '/uploadAttachment',
          data: formData,
          contentType: hType,
          headers: { 'Content-Type': hType },
          transformRequest: angular.identity
        };
      var d = $q.defer();
      $ajax.ajax(opts, function (err, data) {
        if (data && data.url) {
          d.resolve(data);
        } else
          d.reject(err);
      });
      return d.promise;  //return $http(opts);
    }
    function saveMessage(msg, d, status, sendTo) {
      var oldId = msg.___id;
      var u = url() + (msg.id ? '/' + msg.id : '');
      var params = {};
      if (status)
        //u = u + '?status=' + status;
        params.status = status;
      else if (sendTo)
        //u = u + '?sendTo=' + sendTo;
        params.sendTo = sendTo;
      var opts = {
          method: 'POST',
          url: u,
          headers: credentials,
          data: msg,
          params: params
        };
      $ajax.ajax(opts, function (err, data) {
        if (data && data.id) {
          $taffy.messages()({ ___id: oldId }).update({
            id: data.id,
            status: data.status
          });
          d.resolve(data);
        } else
          d.reject(err);
      });
    }
    function _uploadPhoto() {
      var d = $q.defer();
      var token = $auth.getCredentialsSync();
      var creds = '/data/uploadMedia?msg=true&type=LSPHOTO&token=' + token.token;
      // short circuit for offline mode
      if (!$online.state()) {
        $messageBox.open('Offline Mode', 'Please try this attaching a photo when you are online.', { oktext: 'Ok' });
        return;
      }
      $native.singleimage(function (err, file, url) {
        if (err) {
        } else {
          var func = $rootScope.$on('uploadSuccess', function (evt, resp) {
              func();
              d.resolve(resp.data);
            });
          $native.upload(file, url, null, creds, null);
        }
      }, $facility.getImageParameters());
      return d.promise;
    }
    function getUnreadMessageCount(deferred) {
      var opts = {
          method: 'GET',
          url: $krConfig.getAjaxBase() + '/messagecount',
          params: { 'token': $auth.getCredentialsSync().token }
        };
      $ajax.ajax(opts, function (err, response) {
        if (!err) {
          deferred.resolve(response);
        } else {
          console.log(err);
          deferred.reject(err);
        }
      });
    }
    /**
     *  array of messages like this.  If fromUserId != currentUser.id then this goes into inbox.
     * body: "this is a sample message"
     * id: 131037
     * sentDate: "Aug 6, 2012 12:36:43 AM"
     * subject: "test"
     * fromUserId: 12345
     * */
    function getMessages(d, page, pageSize) {
      var fac = $facility.getSync();
      var params = {
          page: page || 1,
          pageSize: pageSize || 20
        };
      var opts = {
          method: 'GET',
          url: url(),
          headers: credentials,
          params: params
        };
      $ajax.ajax(opts, function (err, messages) {
        if (!err) {
          if (messages && messages.length) {
            messages.forEach(function (msg) {
              // remove existing ones if present so we don't have dupes.
              $taffy.messages()({ id: msg.id }).remove();
              $taffy.messages().insert(msg);
            });
          }
          d.resolve();
        } else {
          console.log(err);
          d.reject(err);
        }
      });
    }
    return {
      setCredentials: function (creds) {
        var d = $q.defer();
        // only set if they are different.
        if (credentials.Authorization && credentials.Authorization === creds.Authorization) {
          $timeout(function () {
            d.resolve();
          }, 0);
        } else {
          credentials = creds;
          d.resolve();
        }
        return d.promise;
      },
      put: function (message, status, sendTo) {
        var d = $q.defer();
        // Replace any existing events (server always wins)
        var results;
        if (message.id)
          results = $taffy.messages()({ id: message.id }).update(message);
        if (!message.id || 0 === results.count()) {
          // Didn't exist -> insert
          $taffy.messages().insert(message);
        }
        if (message.attachment) {
          var p = upload(message.attachment);
          p.then(function (a) {
            delete message.attachment;
            if (a.url && a.url != '') {
              message.urlList = message.urlList || [];
              message.urlList.push(a.url);
            }
            saveMessage(message, d, status, sendTo);
          }, function (err) {
            $utils.alert('Error saving attachment.  Please try again.');
          });
        } else if (message.image) {
          if (message.image.realURL) {
            message.urlList = message.urlList || [];
            message.urlList.push(message.image.realURL);
          } else {
            message.keyList = [message.image.key];
            delete message.image;
          }
          saveMessage(message, d, status, sendTo);
        } else {
          saveMessage(message, d, status, sendTo);
        }
        return d.promise;
      },
      getUnreadMessageCount: function () {
        var deferred = $q.defer();
        getUnreadMessageCount(deferred);
        return deferred.promise;
      },
      get: function (start, end, page, pageSize) {
        var params = { status: { 'lt': 3 } };
        if (start && end) {
          params.start = { 'lt': start + 1 };
          params.stop = { 'gt': end - 1 };
        }
        var d = $q.defer();
        getMessages(d, page, pageSize);
        return d.promise.then(function () {
          return $taffy.messages()(params).get();
        });
      },
      find: function (msgId) {
        var id = msgId && Number(msgId) || -1;
        var arr = $taffy.messages()({ id: id }).get();
        return arr && arr[0] ? arr[0] : {};
      },
      sync: function () {
        var d = $q.defer();
        synchDB(d);
        return d.promise;
      },
      uploadPhoto: function () {
        return _uploadPhoto();
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').factory('$learningstories', [
  '$taffy',
  '$resource',
  '$ajax',
  '$adapter',
  '$utils',
  '$facility',
  '$q',
  '$krConfig',
  '$http',
  '$timeout',
  function ($taffy, $resource, $ajax, $adapter, $utils, $facility, $q, $krConfig, $http, $timeout) {
    // Service logic
    var credentials = {};
    var url = function () {
      return $krConfig.getAjaxBase() + '/learningStories';
    };
    function synchDB(d) {
      $taffy.messages()().remove();
      getLearningStories(d);
    }
    function recurse(newOnly) {
      return function (num, r) {
        if (r.replies && r.replies.length) {
          num += _.reduce(r.replies, recurse(newOnly), 0);
        }
        if (!newOnly || r.status === 0 && newOnly)
          return num + 1;
        else
          return num;
      };
    }
    function precompute(s) {
      s.newCommentCount = _.reduce(s.replies, recurse(true), 0);
      s.totalCommentCount = _.reduce(s.replies, recurse(false), 0);
      return s;
    }
    function updateStoryStatus(storyId, status, d) {
      var u = url() + '/updateStory';
      var params = {
          id: storyId,
          status: status
        };
      var opts = {
          method: 'POST',
          url: u,
          headers: credentials,
          params: params
        };
      $ajax.ajax(opts, function (err, data) {
        if (data && data.id) {
          $taffy.stories()({ id: data.id }).remove();
          $taffy.stories().insert(data);
          d.resolve(data);
        } else
          d.reject(err);
      });
    }
    function updateCommentStatus(commentId, status, d) {
      var u = url() + '/updateComment';
      var params = {
          id: commentId,
          status: status
        };
      var opts = {
          method: 'POST',
          url: u,
          headers: credentials,
          params: params
        };
      $ajax.ajax(opts, function (err, data) {
        if (data && data.id) {
          $taffy.stories()({ id: data.id }).remove();
          $taffy.stories().insert(data);
          if (d)
            d.resolve(data);
        } else if (d)
          d.reject(err);
      });
    }
    function saveComment(lsid, parentComment, content, d) {
      var u = url() + '/saveComment';
      var params = {
          lsaddCommentTo: parentComment,
          content: content,
          lsID: lsid
        };
      var opts = {
          method: 'POST',
          url: u,
          headers: credentials,
          params: params
        };
      $ajax.ajax(opts, function (err, data) {
        if (data && data.id) {
          $taffy.stories()({ id: data.id }).remove();
          $taffy.stories().insert(data);
          d.resolve(data);
        } else
          d.reject(err);
      });
    }
    // array of messages like this.  If fromUserId != currentUser.id then this goes into inbox.
    //    body: "this is a sample message"
    //    id: 131037
    //    sentDate: "Aug 6, 2012 12:36:43 AM"
    //    subject: "test"
    //    fromUserId: 12345
    function getLearningStories(d, page, pageSize) {
      var fac = $facility.getSync();
      var params = {
          page: page || 1,
          pageSize: pageSize || 20
        };
      var opts = {
          method: 'GET',
          url: url(),
          headers: credentials,
          params: params
        };
      $ajax.ajax(opts, function (err, stories) {
        if (!err) {
          if (stories && stories.length) {
            stories.forEach(function (s) {
              // remove existing ones if present so we don't have dupes.
              $taffy.stories()({ id: s.id }).remove();
              s = precompute(s);
              $taffy.stories().insert(s);
            });
          }
          d.resolve();
        } else {
          console.log(err);
          d.reject(err);
        }
      });
    }
    function updateComments(replies) {
      _.each(replies, function (r) {
        updateCommentStatus(r.id, 1);
        if (r.replies && r.replies.length)
          updateComments(r.replies);
      });
    }
    return {
      setCredentials: function (creds) {
        var d = $q.defer();
        // only set if they are different.
        if (credentials.Authorization && credentials.Authorization === creds.Authorization) {
          $timeout(function () {
            d.resolve();
          }, 0);
        } else {
          credentials = creds;
          synchDB(d);
        }
        return d.promise;
      },
      addComment: function (pComment, story, content) {
        var d = $q.defer();
        // Replace any existing events (server always wins)
        var results;
        saveComment(story, pComment, content, d);
        return d.promise;
      },
      get: function (start, end, page, pageSize) {
        var params = { status: { 'lt': 3 } };
        if (start && end) {
          params.start = { 'lt': start + 1 };
          params.stop = { 'gt': end - 1 };
        }
        var d = $q.defer();
        getLearningStories(d, page, pageSize);
        return d.promise.then(function () {
          return $taffy.stories()(params).get();
        });
      },
      updateStoryStatus: function (storyID, status) {
        var d = $q.defer();
        updateStoryStatus(storyID, status, d);
        return d.promise.then(function (s) {
          updateComments(s.replies);
          return s;
        });
      },
      updateCommentStatus: function (commentId, status) {
        var d = $q.defer();
        updateCommentStatus(commentId, status, d);
        return d.promise;
      },
      sync: function () {
        var d = $q.defer();
        synchDB(d);
        return d.promise;
      },
      update: function (model) {
        this.get().then(function (messages) {
          model.storyBadge = _.filter(messages, function (m) {
            return m.status === 0;
          }).length;
          model.newCommentCount = _.filter(messages, function (m) {
            return m.newCommentCount > 0;
          }).length;
        });
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').factory('$memories', [
  '$taffy',
  '$resource',
  '$ajax',
  '$adapter',
  '$utils',
  '$facility',
  '$q',
  '$krConfig',
  '$timeout',
  function ($taffy, $resource, $ajax, $adapter, $utils, $facility, $q, $krConfig, $timeout) {
    // Service logic
    var cache = {};
    var credentials = {};
    var url = function () {
      return $krConfig.getAjaxBase() + '/api/media';
    };
    var s3Base = $krConfig.s3Base;
    function synchDB(d) {
      $taffy.memories()().remove();
      cache = {};  //return getPhotos(null, null, null, null, null, d);
    }
    function getPhotos(start, end, page, pagesize, type, d) {
      d = d || $q.defer();
      var params = {
          pagesize: pagesize,
          page: page,
          start: start,
          end: end,
          type: type
        };
      var opts = {
          url: url(),
          method: 'GET',
          params: params,
          headers: credentials
        };
      $ajax.ajax(opts, function (err, data) {
        if (!err && data && data.media) {
          var photos = _.map(data.media, function (photo) {
              var vid;
              if (photo.mediaType == 'PHOTO') {
                vid = s3Base + (photo.thumbnailPath || photo.relativePath);
              } else if (photo.thumbnailPath && photo.thumbnailPath.indexOf('.png') > -1) {
                vid = s3Base + photo.thumbnailPath;
              } else {
                vid = '/public/img/kidreports_tiny.png';
              }
              return {
                id: photo.id,
                title: photo.childName + ' - ' + photo.createDate,
                thumb_src: vid,
                src: s3Base + photo.relativePath,
                type: photo.mediaType,
                createDate: new Date(photo.createDate),
                name: photo.childName,
                bgStyle: {
                  'background-image': 'url(\'' + vid + '\')',
                  'background-size': 'cover'
                }
              };
            });
          d.resolve(photos);
        } else {
          d.reject(err);
        }
      });
      return d.promise;
    }
    return {
      setCredentials: function (creds, clean) {
        //var d = $q.defer();
        credentials = creds;
        if (clean)
          synchDB();  //return d.promise;
      },
      getPhotos: function (start, end, page, pagesize, type) {
        return getPhotos(start, end, page, pagesize, type);  // todo filter.
                                                             //return $taffy.memories()().get();
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').factory('locale', function () {
  return {
    getTimeZone: function () {
      var tz = jstz.determine();
      return tz.name();
    },
    getLanguage: function () {
      //console.log(navigator.language);
      //console.log(navigator.languages);
      //console.log(JSON.stringify(navigator.languages));
      //console.log(navigator.userLanguage);
      var language = navigator.languages && navigator.languages.length > 0 ? navigator.languages[0] : navigator.language || navigator.userLanguage;
      return language ? language.toLowerCase() : 'en';
    }
  };
});
'use strict';
angular.module('kidreports2App').factory('popover', [
  '$appstate',
  function ($appstate) {
    var theScope;
    function showPopup() {
      if (theScope)
        theScope.popup.isOpen = true;
      $appstate.model.showBackdrop = true;
      $appstate.backdropClick = hideBackdrop;
    }
    function hideBackdrop() {
      if (theScope)
        theScope.popup.isOpen = false;
      $appstate.model.showBackdrop = false;
      $appstate.backdropClick = null;
    }
    return {
      show: function (_scope) {
        theScope = _scope;
        showPopup();
      },
      hide: function () {
        hideBackdrop();
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').factory('$templateMgr', function () {
  var map = {};
  return {
    setTemplate: function (key, template) {
      map[key] = template;
    },
    getTemplate: function (key) {
      return map[key];
    }
  };
});
'use strict';
angular.module('kidreports2App').factory('$nav', [
  '$q',
  '$localdb',
  '$auth',
  '$events',
  '$facility',
  '$state',
  '$native',
  '$model',
  '$krCache',
  '$kidplanner',
  '$rootScope',
  '$translate',
  '$messageBox',
  '$learningstories',
  function ($q, $localdb, $auth, $events, $facility, $state, $native, $model, $krCache, $kidplanner, $rootScope, $translate, $messageBox, $learningstories) {
    var pathRedirects = [{
          oldPath: 'messageCenter',
          newPath: 'messages'
        }];
    var _this = {
        togglePortfolio: function () {
          $model.viewPortfolio = !$model.viewPortfolio;
          $rootScope.$broadcast('toggleViewPortfolio');
        },
        toggleDrawer: function () {
          $model.viewDrawer = !$model.viewDrawer;
        },
        setTrackingSheet: function (idx) {
          $model.trackingSheetIndex = idx;
        },
        menuHash: [],
        current: null,
        currentIndex: 0,
        isDashboard: true,
        _goHome: function (directState, wasClick) {
          // just in case... catch all.
          $kidplanner.planInSession = false;
          $native.anyOrientation();
          if ($facility.isLSEnabled())
            $learningstories.update($model);
          _this.isDashboard = false;
          _this.current = null;
          _this.nextState = null;
          _this.backState = null;
          var fac = $facility.getSync();
          if (typeof fac == 'undefined') {
            $state.go('login');
            return;
          }
          var isParent = fac.currentUser.role == 'PARENT_ROLE';
          if (!isParent) {
            _this.isDashboard = true;
            if (!$state.is('home.dash') || !wasClick) {
              $state.go('home.dash');  //, {}, {reload:true});
            }
          } else if ($state.is('home.main') && _this.showEventStream && !isParent) {
            _this.showEventStream = false;
          } else if (!$state.is('home.main')) {
            $state.go('home.main');
          }
        },
        gotoState: function (mi, idx, manualState) {
          if (mi && mi.script) {
            eval(mi.script);
            if (_this.current.path.states.length == 0) {
              _this.current = null;
              return;
            }
          }
          if (!_this.current || (idx < 0 || idx >= _this.current.path.states.length)) {
            _this.currentIndex = 0;
            if (_this.current && _this.current.path.backState) {
              _this.startPath(_this.current.path.backState);
            } else {
              _this._goHome(true);
            }
          } else {
            _this.currentIndex = idx;
            var s = _this.current.path.states[_this.currentIndex];
            _this.dashboardMode = false;
            if (!manualState) {
              // if the current state path contains a home.action or home.dash then go directly to the link
              //  otherwise, assume home.action is correct.
              var redirect = _.find(pathRedirects, function (redirect) {
                  return redirect.oldPath === s;
                });
              if (redirect) {
                $state.go(redirect.newPath, {}, { reload: !$kidplanner.planInSession });
              } else {
                $state.go('home.action.' + s, {}, { reload: !$kidplanner.planInSession });
              }
            }
          }
        },
        back: function () {
          _this.loading = true;
          _this.nextState = null;
          if (_this.backState) {
            var backState = _this.backState;
            _this.backState = null;
            backState();
          } else {
            _this.gotoState(_this.current, _this.currentIndex - 1);
          }
        },
        next: function (skipState) {
          _this.loading = true;
          if (_this.nextState && skipState != true) {
            var nextState = _this.nextState;
            _this.nextState = null;
            nextState();
          } else {
            _this.gotoState(_this.current, _this.currentIndex + 1);
          }
        },
        start: function () {
          _this.gotoState(_this.current, _this.currentIndex);
        },
        dialog: function () {
          _this.loading = false;
          _this.backDisabled = true;
        },
        startPath: function (newPath, manualState) {
          $native.customEvent('PathStart: ' + newPath);
          // $spMenu.hide();
          if (newPath == 'logout') {
            _this.logout();
          } else {
            _this.current = _this.menuHash[newPath];
            if (!_this.current && $model.menuItems) {
              // rehash
              _this.menuHash = [];
              $model.menuItems.forEach(function (mi) {
                _this.menuHash[mi.path.root] = mi;
              });
              _this.current = _this.menuHash[newPath];
            }
            if (_this.current && ($state.includes('home.dash') || _this.current.thirdParty) && _this.current.sref != null) {
              var mi = _this.current;
              if (mi && mi.script) {
                eval(mi.script);
              }
              var sref = _this.current.sref;
              _this.current = null;
              $state.go(sref);
            } else {
              _this.gotoState(_this.current, 0, manualState);
            }
          }
        },
        getPath: function () {
          return _this.current ? _this.current.path.root : '';
        },
        getCurrentStep: function () {
          return _this.current ? 'home-action-' + _this.current.path.states[_this.currentIndex] : '';
        },
        isScrollable: function () {
          return $state.includes('home.dash') || $state.is('home.action.kidPlanner.main') ? '' : 'scrollable';
        },
        isLoading: false,
        goHome: function (directState, wasClick) {
          _this._goHome(directState, wasClick);
        },
        onClick: function (item) {
          _this.icon = item.iconUrl;
          _this.startPath(item.path.root);
        },
        logout: function () {
          $krCache.getCache('drafts', function (d) {
            if (d && d.length) {
              var subj = $translate.instant('DeleteAllDrafts');
              var msg = $translate.instant('ConfirmDeleteAllDraftsOnLogout').replace('&drafts&', d.length);
              $messageBox.open(subj, msg, {
                oktext: $translate.instant('Yes'),
                canceltext: $translate.instant('No')
              }).result.then(finishLogout, function (err) {
              });
            } else {
              finishLogout();
            }
          });
          var finishLogout = function () {
            $auth.logout().then(function () {
              // Preserve user role and date to see if clearing db should be done on login
              var facility = $facility.getSync();
              $localdb.set('formerUserRole', facility.currentUser.role);
              $localdb.set('formerUserDate', moment());
              $krCache.setCache([], 'drafts', function () {
                $facility.logout().then(_this.resetToHome);
              });
            });
          };
        },
        resetToHome: function (v) {
          $localdb.set('lastSyncTime', null);
          $localdb.set('lastEventSynchTime', null);
          document.location.reload({ forceGet: true });
        }
      };
    return _this;
  }
]);
'use strict';
angular.module('kidreports2App').factory('$model', [
  '$prefs',
  function ($prefs) {
    return {
      showQuickView: $prefs.current.showQuickView || false,
      quickViewFilter: [],
      selectAll: false,
      showEventStream: false,
      totalAttendance: '--',
      currentRoomAttendance: '--',
      currentRoom: '--',
      isRefreshing: false,
      isUploading: false,
      imageurl: 'img/empty.png',
      streamFilter: 'NONE',
      hideMessage: true,
      eventListDateSort: false,
      showDash: false,
      focused: false,
      eventList: []
    };
  }
]);
'use strict';
angular.module('kidreports2App').factory('$filters', [
  '$model',
  '$state',
  '$facility',
  '$prefs',
  '$utils',
  function Filter($model, $state, $facility, $prefs, $utils) {
    function isParent() {
      var facility = $facility.getSync();
      if (!facility) {
        return false;
      }
      return facility.currentUser.role == 'PARENT_ROLE';
    }
    var _this = {
        filterParentKid: function (evt) {
          if (!isParent() || evt.header)
            return true;
          else {
            return evt.child && $model.observed[$model.currChild].id == evt.child.id;
          }
        },
        checkEventFilter: function (ele) {
          // check for LS and hide from teachers
          // todo use a flag on the eventtype to determine if parents or teachers should see an event
          if (!isParent() && ele.eventType && ele.eventType.name == 'Learning Story' && !ele.eventType.editable)
            return false;
          if ($model.viewPortfolio) {
            return ele && (ele.eventType && ele.eventType.group && ele.eventType.group.toLowerCase() == 'portfolio' || ele.portfolio);
          }
          if (isParent() && _.get(ele, 'eventType.name') === 'Take Attendance') {
            return true;
          }
          var keep = ele.header || ele.eventType && !ele.eventType.hidden && $prefs.matchesQuickViewFilter(ele);
          return keep;
        },
        currentDate: function (evt) {
          if ($model.viewPortfolio) {
            return true;
          } else {
            return $utils.getDatePart(new Date(evt.timeOccurred)) == $utils.getDatePart(new Date($model.currentDate));
          }
        },
        filterTemplateEvents: function (evt) {
          return evt.header || evt.eventType && !evt.eventType.hidden;
        },
        filterSelectedKids: function (evt) {
          if ($state.is('home.main') || !$model.observed || $model.observed.length == 0 || evt.header) {
            return true;
          }
          for (var idx = 0; idx < $model.observed.length; idx++) {
            if ($model.observed[idx] && $model.observed[idx].id == evt.child.id) {
              return true;
            }
          }
          return false;
        }
      };
    return _this;
  }
]);
'use strict';
angular.module('kidreports2App').factory('$eventAdapter', [
  '$events',
  '$facility',
  '$adapter',
  '$utils',
  '$krCache',
  '$state',
  '$filters',
  '$translate',
  '$modal',
  '$backend',
  '$nav',
  '$prefs',
  '$q',
  '$online',
  '$localdb',
  '$model',
  '$timeout',
  function EventAdapter($events, $facility, $adapter, $utils, $krCache, $state, $filters, $translate, $modal, $backend, $nav, $prefs, $q, $online, $localdb, $model, $timeout) {
    function cb(err, data, refresh, items, d, isTeacher) {
      if (err) {
        console.error(err);
        return [];
      } else {
        var ids = _.map(items, function (item) {
            return item.id;
          });
        var joinedIds = _.union(_.uniq(_.map(data.events, function (e) {
            return isTeacher ? e.subject.id : e.child.id;
          })), ids);
        items = _.map(joinedIds, function (childId) {
          return isTeacher ? $facility.findUser(childId) : $facility.findChild(childId);
        });
        items = _.filter(items, function (i) {
          return i;
        });
        if (refresh) {
          //update state for all items since we did a server refresh
          items.forEach(function (item) {
            item.isIn = null;
            item.attendanceNotes = null;
            item.cachedAttendance = null;
            item.attendanceSet = true;
          });
        }
        if (!isTeacher) {
          // KR-892 Check for any room transitions that have occurred today to update the in-memory room prior to attendance.
          _.each(data.events, function (evt) {
            var evtType = $facility.findEventType(evt.eventType.id);
            if (evtType && evtType.name === 'Room Transition') {
              // todo
              var kid = $facility.findChild(evt.child.id);
              var currRoom = $facility.findRoom(kid.roomId);
              var toRoom = null;
              _.each(evt.eventValues, function (m, n) {
                var intoRoom = $translate.instant('movedIntoRoom').toLowerCase();
                if (m && m.value && m.value.toLowerCase().indexOf(intoRoom) > -1) {
                  // workaround for bad data
                  var idx = m.value.indexOf('::') > -1 ? m.value.indexOf('::') + 2 : m.value.indexOf(':') + 1;
                  var s = m.value.substr(idx, m.value.length).trim();
                  _.forEach($facility.getSync().rooms, function (r) {
                    if (r.name === s)
                      toRoom = r;
                  });
                }
              });
              if (toRoom && parseInt(kid.roomId, 10) != toRoom.id && (refresh || !kid.cachedRoom)) {
                $facility.moveRoom(kid, currRoom, toRoom, true);
                kid.cachedRoom = null;
                $facility.save();
              }
            }
          });
        }
        var t4 = new Date().getTime();
        if (!isTeacher) {
          // sort items into map grouped by room so attendance is set up correctly, then refresh state by rooms.
          var myRooms = $facility.getSync().rooms;
          _.each(myRooms, function (r) {
            $adapter.refreshState(data.events, _.filter(items, function (i) {
              return parseInt(i.roomId, 10) === parseInt(r.id, 10);
            }), d, true);
          });
        } else {
          $adapter.refreshState(data.events, items, d, true);  // EXPENSIVE
        }
        _this.updateAttendanceState(items, isTeacher, false);
        // EXPENSIVE
        return items;
      }
    }
    function _updateLists(data, keepOldEvents, updateAttendance, date, kids) {
      var oldEvts = {};
      var retList = [];
      var tmpEventList = [];
      //angular.copy($model.eventList || []);
      if (!keepOldEvents && !data.offline) {
        tmpEventList = [];
      } else {
        _.forEach($model.eventList, function (item, idx) {
          var k = item.id || item.___id || item.draftId;
          oldEvts[k] = idx;
          tmpEventList.push(item);
        });
      }
      var newList = _.filter(data.events, function (evt) {
          // throw out any that don't have associated eventType or child
          if (!evt.eventType || !evt.child)
            return false;
          var eventType = $facility.findEventType(evt.eventType.id);
          var child = $facility.findChild(evt.child.id);
          if (!child)
            return false;
          if (!eventType)
            return false;
          evt.eventType.name = eventType.name;
          evt.eventType.group = eventType.group;
          evt.eventType.hidden = eventType.hidden;
          evt.child.firstName = child.firstName;
          evt.child.lastName = child.lastName;
          return true;
        });
      _.each(newList, function (event) {
        var evt = $adapter.processEvent(event);
        var k = evt.id || evt.___id || evt.draftId;
        if (typeof oldEvts[k] === 'number') {
          tmpEventList.splice(oldEvts[k], 1, evt);
        } else {
          tmpEventList.push(evt);
        }
        retList.push(evt);
      });
      // compute attendance
      if (!date || $utils.isToday(date)) {
        var count = $adapter.computeAttendance(kids, tmpEventList);
        if (updateAttendance) {
          $model.currentRoomAttendance = count.isSet ? count.count : '--';
          $model.currentRoomRatio = '';
          //_this.selectedCategory.ratio ? ' (' + _this.selectedCategory.ratio + ')' : '';
          $model.currentAttendanceSet = count.isSet;
        }
      }
      // add in drafts only if it's today
      $krCache.getCache('drafts', function (c) {
        var items = c && _.isArray(c) ? c : [];
        if ($utils.isToday(date || new Date())) {
          _.forEach(items, function (item) {
            item.observed.forEach(function (child) {
              // check if we have this draft in the list, if not lets add it
              if (_.filter(tmpEventList, function (e) {
                  if (!e.child)
                    return false;
                  return e.draftId == item.draftId && e.child.id == child.id;
                }).length == 0) {
                var lEvent = $adapter.createLegacyEvent(item);
                lEvent.eventType = $facility.findEventType(lEvent.eventType.id);
                if (lEvent.eventType) {
                  lEvent = $adapter.processEvent(lEvent);
                  lEvent.eventValues.forEach(function (ev) {
                    ev.eventTypeTemplate = _.filter(lEvent.eventType.eventTypeTemplates, function (ett) {
                      return ett.id == ev.eventTypeTemplate.id;
                    })[0];
                  });
                  lEvent.user = $facility.findUser(lEvent.user.id);
                  lEvent.child = $facility.findChild(child.id);
                  lEvent.___id = lEvent.draftId + '_' + lEvent.child.id;
                  tmpEventList.push(lEvent);
                }
              }
            });
          });
        }
        // force update.
        $model.eventList = tmpEventList;
        _this.rebuildLists();
      });
      return retList;
    }
    ;
    var _this = {
        rebuildLists: function () {
          //$model.eventList = _this.getEventList();
          $model.eventListByChild = _this.getEventsByChild();
          $model.eventListByGroup = _this.getEventsByGroup();
          $model.eventListByDate = _this.getEventsByDate();
        },
        updateAttendanceState: function (items, isTeacher, propagateState) {
          if (isTeacher) {
            _.forEach(items, function (t) {
              _this.fixKid(items, $facility.findUser(t.id), propagateState);
            });
          } else {
            // now propogate attendance state changes to in  mem children
            var observed = $facility.getSync().observed;
            if (observed) {
              _.forEach($facility.getSync().observed, function (k) {
                _this.fixKid(items, $facility.findChild(k.id), propagateState);
              });
              $facility.getSync().observed = _.map($facility.getSync().observed, function (k) {
                return $facility.findChild(k.id);
              });
            }
          }
          $facility.save();
        },
        getEvents: function (items, date, force, days, roomId, isTeacher, updateAttendance, keepOldEvents, filter) {
          var d = $q.defer();
          //$events.store(false);
          $model.isRefreshingEvents = true;
          if (isTeacher) {
            $events.getUserEvents(items, date, function (err, data) {
              $model.isRefreshingEvents = false;
              cb(err, data, $online.state() && force, items, date, isTeacher);
              //$timeout(function() {
              //  $events.store(true);
              //}, 1000);
              if (err)
                d.reject(err);
              else
                d.resolve(data);
            }, force, days, roomId, filter);
          } else {
            $events.getEvents(items, date, function (err, data) {
              $model.isRefreshingEvents = false;
              $timeout(function () {
                var time = new Date().getTime();
                if ($online.state()) {
                  // only update our synch times if we're online
                  $model.lastSynchTime = $utils.getTimeString($utils.getTimePart(new Date()));
                  $localdb.set('lastEventSynchTime', $model.lastSynchTime);
                }
                var list = data;
                var kids = cb(err, data, $online.state() && force, items, date, isTeacher);
                if (kids && kids.length) {
                  list = { events: _updateLists(data, keepOldEvents, updateAttendance, date, kids) };
                }
                //$timeout(function() {
                //  $events.store(true);
                //}, 1000);
                if (err)
                  d.reject(err);
                else
                  d.resolve(list);
              });
            }, force, days, roomId, filter);
          }
          return d.promise;
        },
        refreshEvents: function (kids, date, force, updateAttendance, days, cb, keepOldEvents, filter) {
          _this.getEvents(kids, date, force, days, null, false, updateAttendance, keepOldEvents, filter).then(function (data) {
            if (cb)
              cb();
          });
        },
        fixKid: function (items, kid, propogateState) {
          var match = null;
          _.forEach(items, function (copyKid) {
            if (copyKid.id == kid.id)
              match = copyKid;
          });
          if (match != null) {
            //kid.events = match.events;
            kid.timeVal = match.timeVal;
            kid.time = match.time;
            kid.isIn = match.isIn;
            kid.attendanceNotes = match.attendanceNotes;
            if (propogateState) {
              kid.cachedAttendance = true;
              kid.attendanceSet = true;
            }
          }
          if (propogateState) {
            if ($model.observed && $state.includes('home.dash')) {
              var remove = -1;
              _.forEach($model.observed, function (selkid, idx) {
                if (kid.id == selkid.id)
                  remove = idx;
              });
              if (remove != -1) {
                if (kid.isIn != true) {
                  $model.observed.splice(remove, 1);
                } else {
                  kid.selected = true;
                }
              }
            }
          }
        },
        eventListByChild: {},
        getEventsForChild: function (kidId) {
          var arr = [];
          if ($model.eventList) {
            _.forEach($model.eventList, function (evt) {
              if ($filters.checkEventFilter(evt)) {
                if (kidId != evt.child.id)
                  return;
                arr.push(evt);
              }
            });
          }
          return arr;
        },
        getEventsByChild: function () {
          var arr = {};
          if ($model.eventList) {
            _.forEach($model.eventList, function (evt) {
              if ($filters.checkEventFilter(evt)) {
                var kid = $facility.findChild(evt.child.id);
                if (!kid)
                  return;
                var name = kid.firstName + ' ' + kid.lastName;
                //var name = evt.eventType.groupName;
                if (!arr[name]) {
                  arr[name] = [];
                }
                arr[name].push(evt);
              }
            });
          }
          return arr;
        },
        eventListByDate: {},
        getEventsByDate: function () {
          var arr = [];
          var found = {};
          if ($model.eventList) {
            _.forEach($model.eventList, function (evt) {
              if ($filters.checkEventFilter(evt)) {
                //var name = evt.child.firstName + " " + evt.child.lastName;
                var date = new Date(evt.timeOccurred);
                var day = date.toDateString();
                if (evt.draftId != null) {
                  day = 'Drafts';
                  date = date.getTime() * 2;
                } else {
                  date = date.getTime();
                }
                if (!found[day]) {
                  found[day] = {
                    day: day,
                    date: date,
                    values: []
                  };
                  arr.push(found[day]);
                }
                found[day]['values'].push(evt);
              }
            });
          }
          var sorted = [];
          _.sortBy(arr, 'date').forEach(function (el) {
            var d = new Date(el.date);
            d.setHours(23);
            d.setMinutes(59, 59, 999);
            sorted.push({
              header: true,
              day: el.day,
              dateTimeOccurred: d,
              id: el.date
            });
            _.forEach(el.values, function (v) {
              sorted.push(v);
            });
          });
          //        return arr;
          return sorted;
        },
        getEventList: function () {
          var arr = [];
          if ($model.eventList) {
            $model.eventList.forEach(function (evt) {
              if ($filters.filterParentKid(evt) && $filters.filterTemplateEvents(evt) && $filters.filterSelectedKids(evt) && $filters.checkEventFilter(evt)) {
                arr.push(evt);
              }
            });
          }
          return arr;
        },
        eventListByGroup: {},
        getEventsByGroup: function () {
          var arr = {};
          if ($model.eventList) {
            _.forEach($model.eventList, function (evt) {
              if ($filters.checkEventFilter(evt)) {
                //var name = evt.child.firstName + " " + evt.child.lastName;
                var evtType = $facility.findEventType(evt.eventType.id);
                var name = evtType.groupName;
                if (!arr[name]) {
                  arr[name] = {
                    _groupname: evtType.baseGroupName.replace(' ', ''),
                    _values: {}
                  };
                }
                var etName = evtType.name;
                if (!arr[name]['_values'][etName]) {
                  arr[name]['_values'][etName] = [];
                }
                arr[name]['_values'][etName].push(evt);
              }
            });
          }
          return arr;
        },
        viewDraft: function (draft, isParent) {
          draft.eventType = $facility.findEventType(draft.eventType.id);
          var obs = [];
          _.forEach(draft.observed, function (c) {
            obs.push($facility.findChild(c.id));
          });
          draft.observed = obs;
          if (!(draft.timestamp instanceof Date)) {
            draft.timestamp = new Date(draft.timestamp);
          }
          draft.eventValues.forEach(function (ev) {
            if (ev.value.selected) {
              ev.value.selected = _.filter(ev.value.choices, function (ch) {
                return ch.name == ev.value.selected.name;
              })[0] || ev.value.selected;
            }
          });
          //$apps
          $model.currentEvent = draft;  //_this.viewEvent(draft, isParent, true);
        },
        showEvent: function (item) {
          var modalInstance = $modal.open({
              templateUrl: 'views/popups/parentProfile.html',
              controller: 'ParentPopupController',
              resolve: {
                event: function () {
                  return item;
                }
              }
            });
          modalInstance.result.then(function (d) {
            return;
          });
        },
        canEdit: function (item) {
          var usr = $facility.findUser($backend.getCurrentUser().id);
          var evtUsr = $facility.findUser(item.user.id);
          return evtUsr && usr.role == evtUsr.role;
        },
        viewEvent: function (item, isParent, skipCreate) {
          // skip viewing/editing if we're a parent and we didn't create this event.
          if (isParent && item.user.id != $backend.getCurrentUser().id) {
            //** show parent popup
            _this.showEvent(item);
            return;
          }
          item = $events.get(item);
          if (!skipCreate)
            $model.currentEvent = $adapter.createEvent(item, $facility.getSync());
          if ($state.includes('home.dash')) {
            $state.go('home.dash.event', {}, {
              location: true,
              inherit: true,
              relative: 'home.dash',
              notify: true
            });
          } else {
            $nav.startPath('viewEventDirect');
          }
        },
        viewEventObservations: function (item) {
          item = $events.get(item);
          $model.currentEvent = item;
          if ($state.includes('home.dash')) {
            $state.go('home.dash.eventObservations', {}, {
              location: true,
              inherit: true,
              relative: 'home.dash',
              notify: true
            });
          } else {
            $nav.startPath('viewEventObservations');
          }
        }
      };
    $localdb.get('lastEventSynchTime').then(function (t) {
      if (t)
        $model.lastSynchTime = t;
    });
    return _this;
  }
]);
'use strict';
angular.module('kidreports2App').filter('slice', function () {
  return function (arr, start, end) {
    return (arr || []).slice(start, end);
  };
});
angular.module('kidreports2App').filter('reverse', function () {
  return function (items) {
    return items.slice().reverse();
  };
});
'use strict';
angular.module('kidreports2App').controller('AlbumCtrl', [
  '$scope',
  '$http',
  '$location',
  '$timeout',
  '$auth',
  '$memories',
  '$appstate',
  '$q',
  '$translate',
  '$native',
  '$facility',
  '$backend',
  '$facebook',
  '$utils',
  function ($scope, $http, $location, $timeout, $auth, $memories, $appstate, $q, $translate, $native, $facility, $backend, $facebook, $utils) {
    var per_page;
    per_page = 10000;
    $scope.page = 1;
    $scope.isParent = $backend.getCurrentUser().role == 'PARENT_ROLE';
    $scope.isNative = $native.isNative();
    // comment out for now.
    $scope.showFb = $facility.getSync().facebookEnabled;
    $scope.fb = function (item) {
      $facebook.sharePhoto(item);
    };
    $scope.fbVid = function (item) {
      $facebook.shareVideo(item);
    };
    var d = new Date();
    $scope.setYear = function (evt) {
      $scope.thumbyear = evt;
      return evt;
    };
    $scope.isSelected = function (yr, mth) {
      return yr === $scope.year && mth === $scope.month ? 'positive' : '';
    };
    $scope.getTitle = function () {
      return $scope.month + ' ' + $scope.year;
    };
    $scope.checkMonth = function (evt) {
      var monthIdx = -1;
      $scope.months.forEach(function (m, i) {
        if (m === evt)
          monthIdx = i;
      });
      if ($scope.thumbyear == d.getYear() && monthIdx > d.getMonth())
        return false;
      return true;
    };
    $scope.translateHeader = function (date) {
      var month = date.substr(0, date.indexOf(' '));
      var yr = date.substr(date.indexOf(' '));
      return $translate.instant(month) + ' ' + yr;
    };
    var photos = [];
    $scope.months = [
      'January',
      'February',
      'March',
      'April',
      'May',
      'June',
      'July',
      'August',
      'September',
      'October',
      'November',
      'December'
    ];
    $scope.types = [
      {
        'label': 'All Memories',
        'value': 'ALL'
      },
      {
        'label': 'Just Pictures',
        'value': 'PHOTO'
      },
      {
        'label': 'Just Videos',
        'value': 'VIDEO'
      }
    ];
    $scope.lastMonths = $scope.months.slice(0, d.getMonth() + 1);
    $scope.year = d.getFullYear();
    //2014;
    $scope.years = [];
    for (var yr = 2012; yr <= $scope.year; yr++) {
      $scope.years[$scope.years.length] = yr;
    }
    $scope.month = $scope.months[new Date().getMonth()];
    $scope.mediaType = $scope.types[0];
    $scope.$watch('year', function (newValue, oldValue) {
      $scope.dateChange();
    });
    $scope.$watch('month', function (newValue, oldValue) {
      $scope.dateChange();
    });
    $scope.$watch('mediaType', function (newValue, oldValue) {
      $scope.dateChange();
    });
    $scope.step = function (i) {
      var monthIdx = -1;
      $scope.months.forEach(function (m, i) {
        if (m == $scope.month)
          monthIdx = i;
      });
      var yearIdx = -1;
      $scope.years.forEach(function (m, i) {
        if (m == $scope.year)
          yearIdx = i;
      });
      if (yearIdx === 0 && monthIdx === 0 && i === -1) {
      } else if (yearIdx === $scope.years.length - 1 && monthIdx === $scope.months.length - 1 && i === 1) {
      } else {
        monthIdx = monthIdx + i;
        if (monthIdx === 12) {
          // incremented to next year
          $scope.month = $scope.months[0];
          $scope.year = $scope.years[yearIdx + 1];
        } else if (monthIdx === -1) {
          // incremented to next year
          $scope.month = $scope.months[$scope.months.length - 1];
          $scope.year = $scope.years[yearIdx - 1];
        } else {
          $scope.month = $scope.months[monthIdx];
        }
        $scope.page = 1;
      }
    };
    $scope.set_current_photo = function (photo) {
      if (photo) {
        $scope.title = photo.title;
        $scope.hideVideo = true;
        $timeout(function () {
          $scope.current_photo = photo;
          $scope.hideVideo = false;
          if ($scope.current_photo.type === 'VIDEO') {
            var player = document.getElementById('theVidTag');
            var mp4Vid = document.getElementById('theVidSrc');
            player.pause();
            // Now simply set the 'src' attribute of the mp4Vid variable!!!!
            // (...using the jQuery library in this case)
            $(player).attr('poster', photo.thumb_src);
            $(mp4Vid).attr('src', photo.src);
            player.load();
            player.play();
          }
        }, 0);
      }
    };
    // dummy up
    var defaultPhoto = {
        title: 'No Media Found',
        thumb_src: '/public/img/kidreports_tiny.png',
        src: '/public/img/kidreports_tiny.png',
        type: 'PHOTO'
      };
    $scope.set_current_photo(defaultPhoto);
    $scope.next_page = function () {
      if ($scope.page >= $scope.pages) {
        return;
      }
      $scope.page++;
      updatePages();
    };
    $scope.prev_page = function () {
      if ($scope.page <= 1) {
        return;
      }
      $scope.page--;
      updatePages();
    };
    function updatePages() {
      if ($scope.photos.length > per_page) {
        if ($scope.page * per_page < $scope.photos.length)
          $scope.end = $scope.page * per_page;
        else
          $scope.end = $scope.photos.length;
      } else {
        $scope.end = $scope.photos.length;
      }
      $scope.start = $scope.page * per_page - per_page;
      $scope.set_current_photo($scope.photos[$scope.start]);
    }
    $scope.next_photo = function () {
      var current_photo, next_photo;
      current_photo = $('.thumbnail.active');
      next_photo = current_photo.parent().next('li');
      if (next_photo.length) {
        return $scope.set_current_photo($scope.photos[$('.thumbnails li').index(next_photo)]);
      } else {
        return $scope.next_page();
      }
    };
    $scope.prev_photo = function () {
      var current_photo, prev_photo;
      current_photo = $('.thumbnail.active');
      prev_photo = current_photo.parent().prev('li');
      console.log(prev_photo);
      if (prev_photo.length) {
        return $scope.set_current_photo($scope.photos[$('.thumbnails li').index(prev_photo)]);
      } else {
        return $scope.prev_page();
      }
    };
    var func = $scope.$on('$viewContentLoaded', function () {
        func();
        return $('#listing').focus();
      });
    $scope.dateChange = function () {
      $scope.page = 1;  //getPhotos();
    };
    function doSetup(force) {
      // set up credentials
      var d = $q.defer();
      var token = $auth.getCredentialsSync().token;
      var creds = { 'Authorization': 'Bearer ' + token };
      //$appstate.model.isRefreshing = true;
      $memories.setCredentials(creds, force);  //$scope.setDate($scope.year, $scope.month);
    }
    $scope.shareNative = function (media) {
      forge.share.item({
        image: media.src,
        text: 'This is a placeholder. Ain\'t it cool?',
        url: 'https://www.kidreports.com'
      }, function (success) {
        // success
        console.log(success);
      }, function (error) {
        // handle any errors
        console.log(error);
      });
    };
    $scope.setDate = function (year, month) {
      $scope.year = year;
      $scope.month = month;
      //$ionicNavBarDelegate.setTitle(month + ' ' + year);
      getPhotos();
    };
    var firstTime = true;
    $scope.loadMore = function () {
      if (firstTime)
        firstTime = false;
      else
        $scope.step(-1);
      getPhotos(true);
    };
    $scope.doRefresh = function () {
      $scope.refreshing = true;
      var start = new Date();
      $scope.year = start.getFullYear();
      $scope.month = $scope.months[start.getMonth()];
      var yearIdx = -1;
      $scope.years.forEach(function (m, i) {
        if (m == $scope.year)
          yearIdx = i;
      });
      // $scope.year = $scope.years[yearIdx];
      doSetup(true);
      $scope.photos = [];
      getPhotos(true);
      $scope.refreshing = false;
    };
    function getPhotos(keepOld) {
      var start = new Date();
      start.setYear($scope.year);
      var idx = -1;
      $scope.months.forEach(function (m, i) {
        if (m == $scope.month)
          idx = i;
      });
      start.setMonth(idx);
      start.setDate(1);
      start.setHours(0, 0, 0, 0);
      var end = new Date(start.getTime());
      end.setMonth(idx + 1);
      $scope.loading = true;
      var arr = keepOld && $scope.photos ? $scope.photos : [];
      //handle($memories.getPhotos(start, end, $scope.page, per_page, $scope.mediaType.value));
      $memories.getPhotos(start, end, $scope.page, per_page, $scope.mediaType.value).then(function (images) {
        handle(images, arr);
        $scope.$broadcast('scroll.refreshComplete');
        $scope.$broadcast('scroll.infiniteScrollComplete');
      }, function (err) {
        //alert("An Error Occurred");
        $scope.loading = false;
        $appstate.model.isRefreshing = false;
        $scope.$broadcast('scroll.refreshComplete');
        $scope.$broadcast('scroll.infiniteScrollComplete');
      });
    }
    $scope.photos = [];
    function handle(images, old) {
      $scope.photos = old;
      var date = $scope.month + ' ' + $scope.year;
      var idx = -1;
      var replace = _.filter(old, function (p, i) {
          if (p.date == date) {
            idx = i;
            return true;
          } else
            return false;
        });
      if (replace && replace[0] && idx > -1) {
        $scope.photos[idx].media = images;
      } else {
        $scope.photos.push({
          date: date,
          media: images
        });
      }
      $scope.loading = false;
      $appstate.model.isRefreshing = false;
    }
    $scope.saveToAlbum = function (uri, type) {
      if (forge && forge.internal) {
        $appstate.model.isRefreshing = true;
        $appstate.model.loadingMessage = 'Downloading...';
        if (type == 'video' && forge.is.android()) {
          forge.internal.call('pgcapture.saveToLibrary', {
            text: 'some msg here',
            uri: f.uri,
            type: type.toLowerCase(),
            src: uri
          }, function (data) {
            $utils.alert('Saved to Photo Library!');
            finishSave();
          }, function (e) {
            //console.log('Error: ' + e.message)
            $utils.alert('Error Saving To Gallery');
            finishSave();
          });
          return;
        } else {
          forge.file.saveURL(uri, function (f) {
            $appstate.model.loadingMessage = 'Saving to Library...';
            forge.internal.call('pgcapture.saveToLibrary', {
              text: 'some msg here',
              uri: f.uri,
              type: type.toLowerCase(),
              src: uri
            }, function (data) {
              $utils.alert('Saved to Photo Library!');
              finishSave();
            }, function (e) {
              //console.log('Error: ' + e.message)
              $utils.alert('Error Saving To Gallery');
              finishSave();
            });
          }, function (err) {
            $utils.alert('Error Downloading.  Please try again later.');
            finishSave();
          });
        }
      }
    };
    function finishSave() {
      $scope.$apply(function () {
        $appstate.model.isRefreshing = false;
        $appstate.model.loadingMessage = null;
      });
    }
    //doSetup();
    $scope.doRefresh();
  }
]);
'use strict';
angular.module('kidreports2App').controller('ViewstoryCtrl', [
  '$scope',
  '$appstate',
  '$sce',
  '$modalInstance',
  '_scope',
  function ($scope, $appstate, $sce, $modalInstance, _scope) {
    $scope.parent = _scope;
    $scope.formatLink = function (link) {
      return link.substr(link.lastIndexOf('/') + 1);
    };
    $scope.trustAsHtml = function (string) {
      return $sce.trustAsHtml(string);
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('ViewpolicyCtrl', [
  '$scope',
  '$appstate',
  '$sce',
  '$modalInstance',
  '_scope',
  function ($scope, $appstate, $sce, $modalInstance, _scope) {
    $scope.parent = _scope;
    $scope.formatLink = function (link) {
      return link.substr(link.lastIndexOf('/') + 1);
    };
    $scope.trustAsHtml = function (string) {
      return $sce.trustAsHtml(string);
    };
  }
]);
'use strict';
function printDirective() {
  var printSection = document.getElementById('printSection');
  // if there is no printing section, create one
  if (!printSection) {
    printSection = document.createElement('div');
    printSection.id = 'printSection';
    document.body.appendChild(printSection);
  }
  function link(scope, element, attrs) {
    element.on('click', function () {
      var elemToPrint = document.getElementById(attrs.printElementId);
      if (elemToPrint) {
        printNewPage(elemToPrint);
      }
    });
    window.onafterprint = function () {
      // clean the print section before adding new content
      printSection.innerHTML = '';
    };
  }
  function printElement(elem) {
    // clones the element you want to print
    var domClone = elem.cloneNode(true);
    printSection.appendChild(domClone);
    window.print();
  }
  function printNewPage(elem) {
    var title = 'Print View';
    var mywindow = window.open('', title);
    mywindow.document.write('<html  style="overflow:initial;"><head><title>' + title + '</title>');
    mywindow.document.write('<link href="/m/css/bootstrap.css"  media="all" rel="stylesheet"/>');
    mywindow.document.write('<link href="/m/css/googlewebfonts.css" rel=\'stylesheet\' type=\'text/css\'/>');
    mywindow.document.write('<link rel="stylesheet" href="/m/css/app.css">');
    mywindow.document.write('</head><body style="overflow:auto;padding:10px;"><div>');
    mywindow.document.write(elem.innerHTML);
    mywindow.document.write('</div></body></html>');
    mywindow.document.close();
    // necessary for IE >= 10
    mywindow.focus();
    // necessary for IE >= 10
    setTimeout(function () {
      mywindow.print();  //mywindow.close();
    }, 1000);
    return true;
  }
  return {
    link: link,
    restrict: 'A'
  };
}
angular.module('kidreports2App').directive('ngPrint', [printDirective]);
'use strict';
angular.module('kidreports2App').directive('renderComments', [
  '$popup',
  '$facility',
  'moment',
  '$backend',
  'RecursionHelper',
  function ($popup, $facility, moment, $backend, RecursionHelper) {
    return {
      restrict: 'E',
      template: '<ul>' + '<li ng-repeat="child in comments">' + '<i>{{parent.getCommentName(child)}} - {{parent.getCommentDate(child)}}</i><br/><b>{{child.body}}</b>' + '<br/><a  class="btn btn-success" ng-click="parent.addComment(child.id)">Reply</a>' + '<a  class="btn btn-danger" ng-if="parent.isAdmin()" ng-click="parent.deleteComment(child.id)">X</a>' + '<br/>' + '<render-comments comments="child.replies" parent="parent"></render-comments>' + '</li>' + '</ul>',
      compile: function (element) {
        // Use the compile function from the RecursionHelper,
        // And return the linking function(s) which it returns
        return RecursionHelper.compile(element);
      },
      scope: {
        comments: '=',
        parent: '='
      },
      link: function (scope, element, attrs, $scope) {
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').directive('kidFlags', [
  '$popup',
  '$facility',
  'moment',
  '$backend',
  'RecursionHelper',
  function ($popup, $facility, moment, $backend, RecursionHelper) {
    return {
      restrict: 'E',
      templateUrl: 'views/directives/kidflags.html',
      scope: {
        item: '=',
        clazz: '='
      }
    };
  }
]);
angular.module('kidreports2App').factory('RecursionHelper', [
  '$compile',
  function ($compile) {
    return {
      compile: function (element, link) {
        // Normalize the link parameter
        if (angular.isFunction(link)) {
          link = { post: link };
        }
        // Break the recursion loop by removing the contents
        var contents = element.contents().remove();
        var compiledContents;
        return {
          pre: link && link.pre ? link.pre : null,
          post: function (scope, element) {
            // Compile the contents
            if (!compiledContents) {
              compiledContents = $compile(contents);
            }
            // Re-add the compiled contents to the element
            compiledContents(scope, function (clone) {
              element.append(clone);
            });
            // Call the post-linking function, if any
            if (link && link.post) {
              link.post.apply(null, arguments);
            }
          }
        };
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('ActivitystreaminlineCtrl', [
  '$scope',
  '$appstate',
  '$backend',
  '$native',
  '$facebook',
  '$facility',
  '$events',
  '$messageBox',
  function ($scope, $appstate, $backend, $native, $facebook, $facility, $events, $messageBox) {
    $scope.shared = $appstate;
    var usr = $backend.getCurrentUser();
    $scope.isParent = usr.role == 'PARENT_ROLE';
    $scope.isNative = $native.isNative();
    $scope.canEdit = function (item) {
      return item && $appstate.canEdit(item);
    };
    $scope.showEditDeletePopover = function (evt, item) {
      $scope.popupItem = item;  //alert('show popover');
                                //$scope.popover.show(evt);
    };
    $scope.getEvtVals = function (item) {
      if (!item.evtVals) {
        item.evtVals = {};
        item.eventValues.forEach(function (ev) {
          if (!item.evtVals[ev.eventTypeTemplate.name]) {
            item.evtVals[ev.eventTypeTemplate.name] = [];
          }
          item.evtVals[ev.eventTypeTemplate.name].push(ev.formattedValue);
        });
      }
      return item.evtVals;
    };
    var delMode = $facility.getSync().deleteMode || '';
    // only allow delete if: not a draft, and we can edit (same role created both) and not parent
    // and delete allows all or delete allows by teacher
    // if parent, only allow if a parent created the eventand not a draft
    $scope.canDelete = function (item) {
      if (!item)
        return false;
      if ($scope.isParent) {
        return item.draftId == null && $scope.canEdit(item);
      } else {
        return item.draftId == null && canEdit(item) && (delMode == 'ALL' || delMode == 'BY_TEACHER' && usr.canDeleteEvents);
      }
    };
    $scope.viewItem = function (item) {
      $scope.popover.hide();
      $appstate.showEvent(item);
    };
    $scope.editItem = function (item) {
      $scope.popover.hide();
      $appstate.viewEvent(item, $scope.isParent);
    };
    $scope.deleteItem = function (item) {
      $scope.popover.hide();
      //$appstate.viewEvent(item, isParent);
      if ($scope.canDelete) {
        $messageBox.open('Delete Item', 'This will permanently delete the event.  This cannot be undone.  Are you sure you wish to continue?', {
          oktext: 'Yes',
          canceltext: 'No'
        }).result.then(function (selectedItem) {
          $events.deleteEvent(item, function (err, data) {
            if (err) {
              console.log(err);
              $messageBox.open('Error', 'An error occurred, the event was not deleted!', { oktext: 'Ok' });
            } else {
              $messageBox.open('Item Deleted', 'Event Deleted Successfully!', { oktext: 'Ok' });
              $appstate.refreshEvents($appstate.model.observed);
            }
          });
        }, function () {
          $appstate.msg('Delete Cancelled');
        });
      }
    };
    $scope.getWidth = function (item) {
      return $scope.canEdit(item) && $scope.canDelete(item) ? '33%' : $scope.canEdit(item) || $scope.canDelete(item) ? '50%' : '100%';
    };
    // comment out for now.
    $scope.showFb = $facility.getSync().facebookEnabled;
    $scope.setImageUrl = function (url) {
      $appstate.model.imageurl = url;
    };
    $scope.setVidUrl = function (url) {
      if ($native.isNative()) {
        forge.media.videoPlay(url, function () {
        }, function (err) {
        });
      } else {
      }
    };
    $scope.fb = function (item, evt) {
      $facebook.sharePhoto(item.url);
    };
    $scope.fbVid = function (item, evt) {
      $facebook.shareVideo(item.url, null, item.thumbnail);
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('NametofaceCtrl', [
  '$scope',
  '$rootScope',
  '$facility',
  '$state',
  '$utils',
  '$appstate',
  '$krConfig',
  '$events',
  '$adapter',
  '$timeout',
  '$messageBox',
  '$translate',
  '$eventAdapter',
  '$modal',
  '$attendance',
  '$localdb',
  'moment',
  '$drawerService',
  '$trackingService',
  '$uiAlert',
  function ($scope, $rootScope, $facility, $state, $utils, $appstate, $krConfig, $events, $adapter, $timeout, $messageBox, $translate, $eventAdapter, $modal, $attendance, $localdb, moment, $drawerService, $trackingService, $uiAlert) {
    var facility = $facility.getSync();
    $scope.menuItems = $trackingService.populateMenuItems();
    $scope.menuItemSelectedIndex = _.findIndex($scope.menuItems, function (item) {
      return item.name === $translate.instant('NameToFace');
    });
    // For mobile - keep headers aligned with grid
    $('#gridScroll').scroll(function () {
      $('#headerScroll').scrollLeft($(this).scrollLeft());
      $('#childScroll').scrollTop($(this).scrollTop());
    });
    // Date for display purposes
    $scope.today = moment(new Date()).format('MMMM Do, YYYY');
    $scope.shared = $appstate;
    $scope.currentTime = new Date().toTimeString();
    $scope.isDash = $state.includes('home.dash');
    $scope.fac = facility;
    $scope.rooms = facility.rooms;
    $scope.state = $state;
    var labels = null, sublabels = null;
    if ($facility.getSync().n2fCustomLabels) {
      labels = $facility.getSync().n2fCustomLabels;
      sublabels = $facility.getSync().n2fCustomSublabels;
    }
    $scope.times = [];
    $scope.start = 7;
    $scope.stop = 18;
    var timeMap = {};
    var oldTimeMap = {};
    // just get the two times closes to the current one.
    var d = new Date();
    var h = d.getHours();
    var m = d.getMinutes();
    var ap;
    var interval = 30;
    if ($facility.getSync().n2fFrequency) {
      interval = $facility.getSync().n2fFrequency;
    }
    //$scope.$watch('hour', function() {
    //  updateCurrTime(null);
    //});
    $scope.nav = function (location) {
      $state.go(location);
    };
    $scope.openChildProfile = function (childId, $event) {
      $event.stopPropagation();
      $drawerService.openChildProfile(childId);
    };
    function updateCurrTime(prior) {
      $scope.currtimes = [];
      var lastH;
      var start = prior ? prior + $scope.minute : 0;
      for (var i = start; i < start + interval * 4; i += interval) {
        //'8:15', '8:30', '8:45', '9:00'
        ap = 'AM';
        var h = $scope.hour;
        var val = i;
        if (i < 0) {
          var hrs = (i % 60 == 0 ? 0 : 1) + Math.floor(i / -60);
          val = i + 60 * hrs;
          h -= hrs;
        } else {
          h += parseInt(val / 60, 10);
        }
        var m = val % 60;
        if (h >= 12) {
          ap = 'PM';
          h = h > 12 ? h - 12 : h;
        } else if (h == 0)
          h = 12;
        var st = new String(h + ':' + (m == 0 ? '00' : m < 10 ? '0' + m : m) + ' ' + ap);
        $scope.currtimes.push(st);
        //$scope.minute = val;
        lastH = h;
      }
      if ($scope.currtimes[0]) {
        var st = $scope.currtimes[0].split(':');
        $scope.hour = parseInt(st[0], 10);
        $scope.minute = parseInt(st[1].split(' ')[0], 10);
        if (st[1].split(' ')[1] == 'PM' && $scope.hour < 12)
          $scope.hour += 12;
      }
    }
    $scope.hour = $appstate.model.nameToFaceHour || h;
    $scope.minute = 0;
    $scope.currtimes = [];
    updateCurrTime(null);
    $scope.time = d.toTimeString();
    $scope.setCurrTime = function (inc) {
      var timeOffset = inc > 0 ? interval * 4 : interval * -4;
      updateCurrTime(timeOffset);
    };
    function _close() {
      if (dirty) {
        $messageBox.open($translate.instant('UnsavedChanges'), $translate.instant('UnsavedNameToFaceMessage'), {
          canceltext: $translate.instant('YesDiscardChanges'),
          oktext: $translate.instant('Cancel')
        }).result.then(function (selectedItem) {
          //$scope.save();
          $timeout(function () {
            $appstate.canClose = _close;
          }, 50);
        }, function () {
          $appstate.goHome();
        });
      } else {
        $appstate.goHome();
      }
    }
    $appstate.canClose = _close;
    $scope.close = _close;
    $scope.getCurrentUserPhotoPath = function () {
      var currentUser = $facility.getSync().currentUser;
      return $utils.createPhotoUrl($krConfig.s3Base, currentUser.photoPath);
    };
    $scope.timeSelected = function (kid, time) {
      return timeMap[kid.id] && timeMap[kid.id][time] && timeMap[kid.id][time].isSet && timeMap[kid.id][time].room.id == $appstate.model.selectedRoom.id;
    };
    $scope.oldTimeSelected = function (kid, time) {
      return timeMap[kid.id] && timeMap[kid.id][time] && timeMap[kid.id][time].room.id != $appstate.model.selectedRoom.id;
    };
    $scope.selectedColor = function (kid, time) {
      if ($scope.oldTimeSelected(kid, time))
        return 'red';
      else if ($scope.timeSelected(kid, time))
        return 'blue';
    };
    $scope.getRoomName = function (kid, time) {
      if ($scope.oldTimeSelected(kid, time))
        //|| $scope.timeSelected(kid, time))
        return timeMap[kid.id][time].room.name;
    };
    $scope.getTeacherName = function (kid, time) {
      if ($scope.oldTimeSelected(kid, time) || $scope.timeSelected(kid, time) && timeMap[kid.id][time].teacher.id != loggedInUser.id) {
        return timeMap[kid.id][time].teacher.firstName + ' ' + timeMap[kid.id][time].teacher.lastName.substr(0, 1) + '.';
      }
    };
    var dirty = false;
    var loggedInUser = $facility.getSync().currentUser;
    $scope.values = {};
    $scope.values.currTeacher = loggedInUser;
    $scope.teachers = $appstate.model.selectedRoom.teachers;
    $scope.teachers.forEach(function (evt) {
      if (evt.id === $scope.values.currTeacher.id)
        $scope.values.currTeacher = evt;
    });
    $scope.setTimeSelected = function (kid, time) {
      // can't change values that were set in a diff room.
      if ($scope.oldTimeSelected(kid, time))
        return;
      if (!timeMap[kid.id])
        timeMap[kid.id] = {};
      var m = timeMap[kid.id];
      if (m[time] && m[time].isSet) {
        m[time].isSet = false;
      } else {
        m[time] = {
          isSet: true,
          room: $appstate.model.selectedRoom
        };
      }
      m[time].teacher = $scope.values.currTeacher;
      dirty = true;
      $appstate.model.dirty = true;
    };
    /**
     * Checks if all values return falsy
     * @param room
     * @param times
     * @param teacher
     * @returns {*}
     */
    function checkEventValues(room, times, teacher) {
      return room && times && teacher;
    }
    var buildTimeMap = function () {
      $eventAdapter.getEvents($appstate.getVisibleChildren(), d, false, null, null, false, false, true).then(function (data) {
        if (data.events) {
          data.events.forEach(function (ev) {
            if (ev.eventType.name === 'Name To Face' && ev.notes !== 'TrackingSheet' && ev.eventValues && ev.eventValues.length > 0) {
              var room, times, teacher;
              // Parses event values:
              ev.eventValues.forEach(function (ev) {
                if (ev.eventTypeTemplate.name === 'Room')
                  room = ev.value;
                if (ev.eventTypeTemplate.name === 'Times')
                  times = ev.value;
                if (ev.eventTypeTemplate.name === 'Teacher')
                  teacher = ev.value;
              });
              // KRT3-287: Return out of loop if any bad values, preventing creation of bad map:
              if (!checkEventValues(room, times, teacher)) {
                return false;
              }
              // each event has three event values - times, room, teacher.  Pull these out and set on the timemap
              var rooms = room.split(';');
              var teachers = teacher.split(';');
              times.split(';').forEach(function (t, idx) {
                if (t.trim().length > 0) {
                  if (!timeMap[ev.child.id]) {
                    timeMap[ev.child.id] = {
                      evt: ev,
                      arr: []
                    };
                  }
                  timeMap[ev.child.id][t] = {
                    isSet: true,
                    room: $facility.findRoom(rooms[idx]),
                    teacher: $facility.findUser(teachers[idx])
                  };
                }
              });
            }
          });
        }
      });
    };
    buildTimeMap();
    // execute on opening of view
    var refreshNameToFace = function (forceUpdate) {
      var roomMap = $localdb.get('roomMap').$$state.value;
      var firstTime = !(roomMap.indexOf($appstate.model.selectedRoom.id) > -1);
      if (firstTime || forceUpdate) {
        $attendance.refreshAll(true).then(function (result) {
          buildTimeMap();
        }, function (err) {
          console.log(err);  // Error: "It broke"
        });
      } else {
        buildTimeMap();
      }
    };
    /**
     * Refreshes Name To Face data - should be called on every initialization to keep information fresh
     */
    $scope.refreshNameToFace = function () {
      refreshNameToFace(true);
    };
    // KRT3-287: Refreshes Name To Face data:
    $scope.refreshNameToFace();
    function buildValue(str) {
      // take a string like 7:45 AM;8:15 AM; etc and parse it into an array of objects
      var arr = [];
      //str.split(";").forEach(function(s) {
      arr.push({
        selected: true,
        name: str
      });
      //});
      return arr;
    }
    var buildMoment = function (time) {
      var h = parseInt(time.split(':')[0], 10), m = time.split(':')[1].split(' ')[0], ap = time.split(':')[1].split(' ')[1];
      var d = new Date();
      if (ap == 'PM' && h != 12)
        h += 12;
      d.setHours(h);
      d.setMinutes(parseInt(m, 10));
      return d.getTime();
    };
    $scope.save = function () {
      // convert timemap to events
      // if events exist update
      // if not save new oness
      $appstate.getVisibleChildren().forEach(function (ob) {
        var kid = $facility.findChild(ob.id);
        var m = timeMap[ob.id];
        if (!m)
          return;
        var str = '';
        var rms = '';
        var tchs = '';
        var arr = [];
        for (var p in m) {
          if (m.hasOwnProperty(p) && (p.indexOf('AM') > -1 || p.indexOf('PM') > -1) && m[p] && m[p].isSet === true) {
            m[p].moment = buildMoment(p);
            m[p].time = p;
            arr.push(m[p]);
          }
        }
        arr = _.sortBy(arr, 'moment');
        // KRT3-287: If there is no valid events to attempt, skip:
        if (!arr.length) {
          return false;
        }
        _.each(arr, function (m) {
          var p = m.time;
          str += p + ';';
          rms += m.room.id + ';';
          tchs += m.teacher.id + ';';
        });
        var event;
        var timestamp;
        if (m.evt) {
          event = m.evt;
          var room, times, teacher;
          event.eventValues.forEach(function (ev, idx) {
            if (ev.eventTypeTemplate.name === 'Room')
              room = idx;
            if (ev.eventTypeTemplate.name === 'Times')
              times = idx;
            if (ev.eventTypeTemplate.name === 'Teacher')
              teacher = idx;
          });
          event.eventValues[times].value = { choices: buildValue(str) };
          event.eventValues[times].ett = event.eventValues[times].eventTypeTemplate;
          event.eventValues[times].type = 'MULTI_SELECT';
          event.eventValues[room].value = { selected: { name: rms } };
          event.eventValues[room].ett = event.eventValues[room].eventTypeTemplate;
          event.eventValues[room].type = 'SINGLE_SELECT';
          event.eventValues[teacher].value = { selected: { name: tchs } };
          event.eventValues[teacher].ett = event.eventValues[teacher].eventTypeTemplate;
          event.eventValues[teacher].type = 'SINGLE_SELECT';
          timestamp = new Date(event.timestamp);
        } else {
          var items = _.filter($facility.getSync().eventTypes, function (et) {
              return et.categories[0].id === $facility.findRoom(kid.roomId).category.id && et.name === 'Name To Face';
            });
          if (items && items.length && items.length > 0) {
            event = $adapter.createNewEvent(items[0]);
            var room, times, teacher;
            items[0].eventTypeTemplates.forEach(function (eventTypeTemplate, idx) {
              if (eventTypeTemplate.name === 'Room')
                room = idx;
              if (eventTypeTemplate.name === 'Times')
                times = idx;
              if (eventTypeTemplate.name === 'Teacher')
                teacher = idx;
            });
            event.eventValues = [];
            timestamp = new Date();
            event.eventValues.push({
              ett: items[0].eventTypeTemplates[times],
              value: { choices: buildValue(str) },
              type: 'MULTI_SELECT'
            });
            event.eventValues.push({
              ett: items[0].eventTypeTemplates[room],
              value: { selected: { name: rms } },
              type: 'SINGLE_SELECT'
            });
            event.eventValues.push({
              ett: items[0].eventTypeTemplates[teacher],
              value: { selected: { name: tchs } },
              type: 'SINGLE_SELECT'
            });
          }
        }
        if (event) {
          event.observed = [ob];
          event.observers = [$facility.getSync().currentUser];
          var day = $utils.getDatePart(timestamp);
          var time = $utils.getTimePart(timestamp);
          event.timestamp = $utils.createGsonTSFromDTStrings(day, time);
          $events.saveEvent(event, true);
        } else {
          $utils.alert('Name To Face Not Found!');
        }
      });
      $events.push();
      var eventListener = $rootScope.$on('pushComplete', function (scope, hasError, error) {
          if (!hasError) {
            $uiAlert.pushSuccessAlert($translate.instant('NameToFaceSaved'), 'NAMETOFACE');
          } else {
            $uiAlert.pushErrorAlert('There was an error saving Name To Face.', 'NAMETOFACE');
          }
          displayAlerts();
        });
      $scope.$on('$destroy', eventListener);
    };
    // open kiosk PIN modal if collectTeacherPin is set to true
    var collectTeacherPin = $facility.getSync().collectTeacherPins;
    var isAdmin = $facility.isAdmin();
    if (collectTeacherPin && !isAdmin) {
      var kioskModal = $modal.open({
          templateUrl: 'views/popups/kioskModal.html',
          controller: 'KioskModalCtrl',
          resolve: {
            items: function () {
              return {
                'pin': $scope.pin,
                'initials': $scope.initials
              };
            }
          }
        });
      // When modal is dismissed close this window and return to main
      var doDismissFn = function (reason) {
        $appstate.goHome();
      };
      // when modal is closed without being dismissed.
      var doClosureFn = function (teacher) {
        var matchingTeacher = $appstate.model.selectedRoom.teachers.filter(function (t) {
            return t.id === teacher.id;
          });
        $scope.values.currTeacher = matchingTeacher[0];
        $scope.values.isTeacherSelectDisabled = true;
      };
      // chain functionality onto result promise
      kioskModal.result.then(doClosureFn, doDismissFn);
    }
    function displayAlerts() {
      var alerts = $uiAlert.popAlertsFor('NAMETOFACE');
      if (alerts) {
        $scope.alerts = alerts;
        $timeout(function () {
          $scope.alerts = null;
        }, 5000);
      }
    }
    $scope.getAlertClass = function (alert) {
      if (alert.type === $uiAlert.ALERT_TYPE_SUCCESS) {
        return 'alert alert-success';
      } else if (alert.type === $uiAlert.ALERT_TYPE_WARNING) {
        return 'alert alert-warning';
      } else if (alert.type === $uiAlert.ALERT_TYPE_ERROR) {
        return 'alert alert-danger';
      } else {
        return 'alert';
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('CWNametofaceCtrl', [
  '$scope',
  '$rootScope',
  '$facility',
  '$state',
  '$utils',
  '$appstate',
  '$events',
  '$adapter',
  '$timeout',
  '$messageBox',
  '$translate',
  '$eventAdapter',
  '$modal',
  '$krConfig',
  '$drawerService',
  '$trackingService',
  '$uiAlert',
  function ($scope, $rootScope, $facility, $state, $utils, $appstate, $events, $adapter, $timeout, $messageBox, $translate, $eventAdapter, $modal, $krConfig, $drawerService, $trackingService, $uiAlert) {
    var facility = $facility.getSync();
    $scope.menuItems = $trackingService.populateMenuItems();
    $scope.menuItemSelectedIndex = _.findIndex($scope.menuItems, function (item) {
      return item.name === $translate.instant('TrackingSheet');
    });
    // For mobile - keep headers aligned with grid
    $('#gridScroll').scroll(function () {
      $('#headerScroll').scrollLeft($(this).scrollLeft());
      $('#childScroll').scrollTop($(this).scrollTop());
    });
    $scope.state = $state;
    $scope.shared = $appstate;
    $scope.currentTime = new Date().toTimeString();
    $scope.today = moment(new Date()).format('MMMM Do, YYYY');
    $scope.fac = facility;
    $scope.rooms = facility.rooms;
    $scope.isDash = $state.includes('home.dash');
    $scope.openChildProfile = function (childId, $event) {
      $event.stopPropagation();
      $drawerService.openChildProfile(childId);
    };
    $scope.getCurrentUserPhotoPath = function () {
      var currentUser = $facility.getSync().currentUser;
      return $utils.createPhotoUrl($krConfig.s3Base, currentUser.photoPath);
    };
    // todo drive from config for cw?
    function parseLabels(s, defaults) {
      var arr = [];
      if (s) {
        arr = s.split(';');
      }
      if (arr.length > 0)
        return arr;
      return defaults;
    }
    $scope.labels = parseLabels($scope.fac.trackingSheetLabels, [
      'AM Outside',
      'PM Outside'
    ]);
    $scope.sublabels = parseLabels($scope.fac.trackingSheetSublabels, [
      'Lined Up At Door',
      'Arrived',
      'Line Up to Leave',
      'Back in Classroom'
    ]);
    var labelMap = [];
    _.each($scope.labels, function (l) {
      _.each($scope.sublabels, function (sl) {
        labelMap[l + ':' + sl] = true;
      });
    });
    $scope.currIndex = 0;
    var timeMap = {};
    var oldTimeMap = {};
    $scope.setCurrTime = function (inc) {
      if (inc > 0) {
        $scope.currIndex++;
      } else {
        $scope.currIndex--;
      }
      if ($scope.currIndex >= $scope.labels.length)
        $scope.currIndex = 0;
      else if ($scope.currIndex < 0)
        $scope.currIndex = $scope.labels.length - 1;
    };
    function _close() {
      if (dirty) {
        $messageBox.open($translate.instant('UnsavedChanges'), $translate.instant('UnsavedNameToFaceMessage'), {
          canceltext: $translate.instant('YesDiscardChanges'),
          oktext: $translate.instant('Cancel')
        }).result.then(function (selectedItem) {
          //$scope.save();
          $timeout(function () {
            $appstate.canClose = _close;
          }, 50);
        }, function () {
          $appstate.goHome();
        });
      } else {
        $appstate.goHome();
      }
    }
    $appstate.canClose = _close;
    $scope.close = _close;
    $scope.timeSelected = function (kid, time) {
      return timeMap[kid.id] && timeMap[kid.id][time] && timeMap[kid.id][time].isSet && timeMap[kid.id][time].room.id == $appstate.model.selectedRoom.id;
    };
    $scope.oldTimeSelected = function (kid, time) {
      return timeMap[kid.id] && timeMap[kid.id][time] && timeMap[kid.id][time].room.id != $appstate.model.selectedRoom.id;
    };
    $scope.selectedColor = function (kid, time) {
      if ($scope.oldTimeSelected(kid, time))
        return 'red';
      else if ($scope.timeSelected(kid, time))
        return 'blue';
    };
    $scope.getRoomName = function (kid, time) {
      if ($scope.oldTimeSelected(kid, time))
        //|| $scope.timeSelected(kid, time))
        return timeMap[kid.id][time].room.name;
    };
    $scope.getTeacherName = function (kid, time) {
      if ($scope.oldTimeSelected(kid, time) || $scope.timeSelected(kid, time) && timeMap[kid.id][time].teacher.id != loggedInUser.id) {
        return timeMap[kid.id][time].teacher.firstName + ' ' + timeMap[kid.id][time].teacher.lastName.substr(0, 1) + '.';
      }
    };
    $scope.refreshAttendance = function (b) {
      $appstate.refresh(b);
    };
    var dirty = false;
    var loggedInUser = $facility.getSync().currentUser;
    $scope.values = {};
    $scope.values.currTeacher = loggedInUser;
    $scope.teachers = $appstate.model.selectedRoom.teachers;
    $scope.teachers.forEach(function (evt) {
      if (evt.id === $scope.values.currTeacher.id)
        $scope.values.currTeacher = evt;
    });
    var d = new Date();
    $scope.getLookup = function (sublabel) {
      return $scope.labels[$scope.currIndex] + ':' + sublabel;
    };
    $scope.setTimeSelected = function (kid, time) {
      // can't change values that were set in a diff room.
      if ($scope.oldTimeSelected(kid, time))
        return;
      if (!timeMap[kid.id])
        timeMap[kid.id] = {};
      var m = timeMap[kid.id];
      if (m[time] && m[time].isSet) {
        m[time].isSet = false;
      } else {
        m[time] = {
          isSet: true,
          room: $appstate.model.selectedRoom
        };
      }
      m[time].teacher = $scope.values.currTeacher;
      m[time].timestamp = $utils.getTimeString($utils.getTimePart(new Date()));
      dirty = true;
      $appstate.model.dirty = true;
    };
    $eventAdapter.getEvents($appstate.getVisibleChildren(), d, false, null, null, false, false, true).then(function (data) {
      if (data.events) {
        data.events.forEach(function (ev) {
          if (ev.eventType.name == 'Name To Face' && ev.notes == 'TrackingSheet' && ev.eventValues && ev.eventValues.length > 0) {
            var room, times, teacher, timestamp;
            ev.eventValues.forEach(function (ev) {
              if (ev.eventTypeTemplate.name === 'Room')
                room = ev.value;
              if (ev.eventTypeTemplate.name === 'Times')
                times = ev.value;
              if (ev.eventTypeTemplate.name === 'Timestamp')
                timestamp = ev.value;
              if (ev.eventTypeTemplate.name === 'Teacher')
                teacher = ev.value;
            });
            // each event has three event values - times, room, teacher.  Pull these out and set on the timemap
            var rooms = room.split(';');
            var teachers = teacher.split(';');
            var timestamps = timestamp.split(';');
            times.split(';').forEach(function (t, idx) {
              if (t.trim().length > 0) {
                if (!timeMap[ev.child.id]) {
                  timeMap[ev.child.id] = {
                    evt: ev,
                    arr: []
                  };
                }
                timeMap[ev.child.id][t] = {
                  isSet: true,
                  room: $facility.findRoom(rooms[idx]),
                  teacher: $facility.findUser(teachers[idx]),
                  timestamp: timestamps[idx]
                };
              }
            });
          }
        });
      }
    });
    function buildValue(str) {
      // take a string like 7:45 AM;8:15 AM; etc and parse it into an array of objects
      var arr = [];
      //str.split(";").forEach(function(s) {
      arr.push({
        selected: true,
        name: str
      });
      //});
      return arr;
    }
    $scope.save = function () {
      // convert timemap to events
      // if events exist update
      // if not save new ones
      $appstate.getVisibleChildren().forEach(function (ob) {
        var kid = $facility.findChild(ob.id);
        var m = timeMap[ob.id];
        if (!m)
          return;
        var str = '';
        var rms = '';
        var tchs = '';
        var timestmps = '';
        for (var p in m) {
          if (m.hasOwnProperty(p) && labelMap[p] && m[p] && m[p].isSet === true) {
            str += p + ';';
            rms += m[p].room.id + ';';
            tchs += m[p].teacher.id + ';';
            timestmps += m[p].timestamp + ';';
          }
        }
        var event;
        var timestamp;
        if (m.evt) {
          event = m.evt;
          var room, times, teacher, tstamp;
          event.eventValues.forEach(function (ev, idx) {
            if (ev.eventTypeTemplate.name === 'Room')
              room = idx;
            if (ev.eventTypeTemplate.name === 'Times')
              times = idx;
            if (ev.eventTypeTemplate.name === 'Timestamp')
              tstamp = idx;
            if (ev.eventTypeTemplate.name === 'Teacher')
              teacher = idx;
          });
          event.eventValues[times].value = { choices: buildValue(str) };
          event.eventValues[times].ett = event.eventValues[times].eventTypeTemplate;
          event.eventValues[times].type = 'MULTI_SELECT';
          event.eventValues[room].value = { selected: { name: rms } };
          event.eventValues[room].ett = event.eventValues[room].eventTypeTemplate;
          event.eventValues[room].type = 'SINGLE_SELECT';
          event.eventValues[teacher].value = { selected: { name: tchs } };
          event.eventValues[teacher].ett = event.eventValues[teacher].eventTypeTemplate;
          event.eventValues[teacher].type = 'SINGLE_SELECT';
          event.eventValues[tstamp].value = { selected: { name: timestmps } };
          event.eventValues[tstamp].ett = event.eventValues[tstamp].eventTypeTemplate;
          event.eventValues[tstamp].type = 'SINGLE_SELECT';
          timestamp = new Date(event.timestamp);
        } else {
          var items = _.filter($facility.getSync().eventTypes, function (et) {
              return et.categories[0].id === $facility.findRoom(kid.roomId).category.id && et.name === 'Name To Face';
            });
          if (items && items.length && items.length > 0) {
            event = $adapter.createNewEvent(items[0]);
            var room, times, teacher, tstamp;
            items[0].eventTypeTemplates.forEach(function (eventTypeTemplate, idx) {
              if (eventTypeTemplate.name === 'Room')
                room = idx;
              if (eventTypeTemplate.name === 'Times')
                times = idx;
              if (eventTypeTemplate.name === 'Teacher')
                teacher = idx;
              if (eventTypeTemplate.name === 'Timestamp')
                tstamp = idx;
            });
            event.eventValues = [];
            timestamp = new Date();
            event.eventValues.push({
              ett: items[0].eventTypeTemplates[times],
              value: { choices: buildValue(str) },
              type: 'MULTI_SELECT'
            });
            event.eventValues.push({
              ett: items[0].eventTypeTemplates[room],
              value: { selected: { name: rms } },
              type: 'SINGLE_SELECT'
            });
            event.eventValues.push({
              ett: items[0].eventTypeTemplates[teacher],
              value: { selected: { name: tchs } },
              type: 'SINGLE_SELECT'
            });
            event.eventValues.push({
              ett: items[0].eventTypeTemplates[tstamp],
              value: { selected: { name: timestmps } },
              type: 'SINGLE_SELECT'
            });
          }
        }
        if (event) {
          event.observed = [ob];
          event.observers = [$facility.getSync().currentUser];
          // to distinguish between normal N2F and tracking N2F
          event.notes = 'TrackingSheet';
          var day = $utils.getDatePart(timestamp);
          var time = $utils.getTimePart(timestamp);
          event.timestamp = $utils.createGsonTSFromDTStrings(day, time);
          $events.saveEvent(event, true);
        } else {
          $utils.alert('Name To Face Not Found!');
        }
      });
      $events.push();
      var eventListener = $rootScope.$on('pushComplete', function (scope, hasError, error) {
          if (!hasError) {
            $uiAlert.pushSuccessAlert($translate.instant('NameToFaceSaved'), 'NAMETOFACE');
          } else {
            $uiAlert.pushErrorAlert('There was an error saving Name To Face.', 'NAMETOFACE');
          }
          displayAlerts();
        });
      $scope.$on('$destroy', eventListener);
    };
    // open kiosk PIN modal if collectTeacherPin is set to true
    var collectTeacherPin = $facility.getSync().collectTeacherPins;
    var isAdmin = $facility.isAdmin();
    if (collectTeacherPin && !isAdmin) {
      var kioskModal = $modal.open({
          templateUrl: 'views/popups/kioskModal.html',
          controller: 'KioskModalCtrl',
          resolve: {
            items: function () {
              return {
                'pin': $scope.pin,
                'initials': $scope.initials
              };
            }
          }
        });
      // When modal is dismissed close this window and return to main
      var doDismissFn = function (reason) {
        $appstate.goHome();
      };
      // when modal is closed without being dismissed.
      var doClosureFn = function (teacher) {
        var matchingTeacher = $appstate.model.selectedRoom.teachers.filter(function (t) {
            return t.id === teacher.id;
          });
        $scope.values.currTeacher = matchingTeacher[0];
        $scope.values.isTeacherSelectDisabled = true;
      };
      // chain functionality onto result promise
      kioskModal.result.then(doClosureFn, doDismissFn);
    }
    function displayAlerts() {
      var alerts = $uiAlert.popAlertsFor('NAMETOFACE');
      if (alerts) {
        $scope.alerts = alerts;
        $timeout(function () {
          $scope.alerts = null;
        }, 5000);
      }
    }
    $scope.getAlertClass = function (alert) {
      if (alert.type === $uiAlert.ALERT_TYPE_SUCCESS) {
        return 'alert alert-success';
      } else if (alert.type === $uiAlert.ALERT_TYPE_WARNING) {
        return 'alert alert-warning';
      } else if (alert.type === $uiAlert.ALERT_TYPE_ERROR) {
        return 'alert alert-danger';
      } else {
        return 'alert';
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('TrackingChartCtrl', [
  '$scope',
  '$rootScope',
  '$facility',
  '$state',
  '$utils',
  '$appstate',
  '$events',
  '$adapter',
  '$timeout',
  '$messageBox',
  '$translate',
  '$eventAdapter',
  '$modal',
  '$krConfig',
  '$stateParams',
  '$drawerService',
  '$trackingService',
  '$uiAlert',
  function ($scope, $rootScope, $facility, $state, $utils, $appstate, $events, $adapter, $timeout, $messageBox, $translate, $eventAdapter, $modal, $krConfig, $stateParams, $drawerService, $trackingService, $uiAlert) {
    var facility = $facility.getSync();
    var trackingChartId = parseInt($stateParams.trackingChartId);
    var currentTrackingChart = _.find(facility.trackingCharts, function (trackingChart) {
        return trackingChart.id === trackingChartId;
      });
    $scope.trackingChartName = currentTrackingChart.name;
    $scope.menuItems = $trackingService.populateMenuItems();
    $scope.menuItemSelectedIndex = _.findIndex($scope.menuItems, function (item) {
      return item.name === $scope.trackingChartName.substring(0, 16);
    });
    // For mobile - keep headers aligned with grid
    $('#gridScroll').scroll(function () {
      $('#headerScroll').scrollLeft($(this).scrollLeft());
      $('#subheaderScroll').scrollLeft($(this).scrollLeft());
      $('#childScroll').scrollTop($(this).scrollTop());
    });
    $scope.shared = $appstate;
    $scope.state = $state;
    $scope.currentTime = new Date().toTimeString();
    $scope.today = moment(new Date()).format('MMMM Do, YYYY');
    $scope.fac = facility;
    $scope.isDash = $state.includes('home.dash');
    $scope.openChildProfile = function (childId, $event) {
      $event.stopPropagation();
      $drawerService.openChildProfile(childId);
    };
    $scope.getCurrentUserPhotoPath = function () {
      var currentUser = $facility.getSync().currentUser;
      return $utils.createPhotoUrl($krConfig.s3Base, currentUser.photoPath);
    };
    $scope.isActiveTrackingChart = function (trackingChart) {
      if (trackingChart === currentTrackingChart) {
        return 'menu-item-selected';
      } else {
        return '';
      }
    };
    // todo drive from config for cw?
    function parseLabels(s, defaults) {
      var arr = [];
      if (s) {
        arr = s.split(';');
      }
      if (arr.length > 0)
        return arr;
      return defaults;
    }
    $scope.sublabels = parseLabels(currentTrackingChart.subHeaders, [
      'Lined Up At Door',
      'Arrived',
      'Line Up to Leave',
      'Back in Classroom'
    ]);
    $scope.widthStyle = 'width:' + (66 + 2 / 3) / $scope.sublabels.length + '% !important;';
    var markedAll = {};
    $scope.markAllSelected = function (item) {
      return markedAll[$scope.getLookup(item)] == true;
    };
    $scope.markAllToggle = function (item) {
      var lookup = $scope.getLookup(item);
      var b = !markedAll[lookup];
      markedAll[lookup] = b;
      _.each($scope.shared.getVisibleChildren(), function (kid) {
        $scope.setTimeSelected(kid, lookup, b);
      });
    };
    $scope.currIndex = 0;
    $scope.times = [];
    $scope.start = 7;
    $scope.stop = 18;
    var timeMap = {};
    var oldTimeMap = {};
    // just get the two times closes to the current one.
    var d = new Date();
    var h = d.getHours();
    var m = d.getMinutes();
    var ap;
    var interval = 30;
    if (currentTrackingChart.chartFrequency) {
      interval = currentTrackingChart.chartFrequency;
    }
    //
    //function getTimes() {
    //  var arr = [];
    //  for ( var h = 0; h < 24; h++ ) {
    //
    //  }
    //  return arr;
    //}
    //
    //var labelMap = [];
    //_.each(getTimes(), function(l) {
    //  _.each($scope.sublabels, function (sl) {
    //    labelMap[l+""+sl] = true;
    //  })
    //});
    $scope.getCurrentTime = function () {
      var h = $scope.hour, m = $scope.minute, ap = 'AM';
      if (h >= 12) {
        ap = 'PM';
        h = h > 12 ? h - 12 : h;
      } else if (h == 0)
        h = 12;
      return new String(h + ':' + (m == 0 ? '00' : m < 10 ? '0' + m : m) + ' ' + ap);
    };
    function updateCurrTime(prior) {
      if ($scope.minute + prior >= 60) {
        $scope.hour++;
        $scope.minute = $scope.minute + prior - 60;
      } else if ($scope.minute + prior < 0) {
        $scope.hour--;
        $scope.minute = $scope.minute + prior + 60;
      } else {
        $scope.minute += prior;
      }
      if ($scope.minute > 59 || $scope.minute < 0) {
        $scope.hour += Math.floor($scope.minute / 60);
        $scope.minute = $scope.minute % 60;
      }
      $scope.currentTime = $scope.getCurrentTime();
    }
    $scope.hour = h;
    $scope.minute = Math.floor(m / interval) * interval;
    updateCurrTime(0);
    $scope.time = d.toTimeString();
    $scope.setCurrTime = function (inc) {
      var timeOffset = inc > 0 ? interval * 1 : interval * -1;
      updateCurrTime(timeOffset);
    };
    function _close() {
      if (dirty) {
        $messageBox.open($translate.instant('UnsavedChanges'), $translate.instant('UnsavedNameToFaceMessage'), {
          canceltext: $translate.instant('YesDiscardChanges'),
          oktext: $translate.instant('Cancel')
        }).result.then(function (selectedItem) {
          //$scope.save();
          $timeout(function () {
            $appstate.canClose = _close;
          }, 50);
        }, function () {
          $appstate.goHome();
        });
      } else {
        $appstate.goHome();
      }
    }
    $appstate.canClose = _close;
    $scope.close = _close;
    $scope.timeSelected = function (kid, time) {
      return timeMap[kid.id] && timeMap[kid.id][time] && timeMap[kid.id][time].isSet && timeMap[kid.id][time].room.id == $appstate.model.selectedRoom.id;
    };
    $scope.oldTimeSelected = function (kid, time) {
      return timeMap[kid.id] && timeMap[kid.id][time] && timeMap[kid.id][time].room.id != $appstate.model.selectedRoom.id;
    };
    $scope.selectedColor = function (kid, time) {
      if ($scope.oldTimeSelected(kid, time))
        return 'red';
      else if ($scope.timeSelected(kid, time))
        return 'blue';
    };
    $scope.getRoomName = function (kid, time) {
      if ($scope.oldTimeSelected(kid, time))
        //|| $scope.timeSelected(kid, time))
        return timeMap[kid.id][time].room.name;
    };
    $scope.getTeacherName = function (kid, time) {
      if ($scope.oldTimeSelected(kid, time) || $scope.timeSelected(kid, time) && timeMap[kid.id][time].teacher.id != loggedInUser.id) {
        return timeMap[kid.id][time].teacher.firstName + ' ' + timeMap[kid.id][time].teacher.lastName.substr(0, 1) + '.';
      }
    };
    var dirty = false;
    var loggedInUser = $facility.getSync().currentUser;
    $scope.values = {};
    $scope.values.currTeacher = loggedInUser;
    $scope.teachers = $appstate.model.selectedRoom.teachers;
    $scope.teachers.forEach(function (evt) {
      if (evt.id === $scope.values.currTeacher.id)
        $scope.values.currTeacher = evt;
    });
    var d = new Date();
    $scope.getLookup = function (sublabel) {
      return $scope.getCurrentTime() + ':' + sublabel;
    };
    $scope.setTimeSelected = function (kid, time, newval) {
      // can't change values that were set in a diff room.
      if ($scope.oldTimeSelected(kid, time))
        return;
      if (!timeMap[kid.id])
        timeMap[kid.id] = {};
      var m = timeMap[kid.id];
      if (m[time] && m[time].isSet) {
        m[time].isSet = false;
      } else {
        m[time] = {
          isSet: true,
          room: $appstate.model.selectedRoom
        };
      }
      if (typeof newval != 'undefined')
        m[time].isSet = newval;
      m[time].teacher = $scope.values.currTeacher;
      m[time].timestamp = $utils.getTimeString($utils.getTimePart(new Date()));
      dirty = true;
      $appstate.model.dirty = true;
    };
    $eventAdapter.getEvents($appstate.getVisibleChildren(), d, false, null, null, false, false, true).then(function (data) {
      if (data.events) {
        data.events.forEach(function (ev) {
          if (ev.eventType.name == 'Name To Face' && ev.notes == 'TrackingChart:' + currentTrackingChart.name && ev.eventValues && ev.eventValues.length > 0) {
            var room, times, teacher, timestamp;
            ev.eventValues.forEach(function (ev) {
              if (ev.eventTypeTemplate.name === 'Room')
                room = ev.value;
              if (ev.eventTypeTemplate.name === 'Times')
                times = ev.value;
              if (ev.eventTypeTemplate.name === 'Timestamp')
                timestamp = ev.value;
              if (ev.eventTypeTemplate.name === 'Teacher')
                teacher = ev.value;
            });
            // each event has three event values - times, room, teacher.  Pull these out and set on the timemap
            var rooms = room.split(';');
            var teachers = teacher.split(';');
            var timestamps = timestamp.split(';');
            times.split(';').forEach(function (t, idx) {
              if (t.trim().length > 0) {
                if (!timeMap[ev.child.id]) {
                  timeMap[ev.child.id] = {
                    evt: ev,
                    arr: []
                  };
                }
                timeMap[ev.child.id][t] = {
                  isSet: true,
                  room: $facility.findRoom(rooms[idx]),
                  teacher: $facility.findUser(teachers[idx]),
                  timestamp: timestamps[idx]
                };
              }
            });
          }
        });
      }
    });
    function buildValue(str) {
      // take a string like 7:45 AM;8:15 AM; etc and parse it into an array of objects
      var arr = [];
      //str.split(";").forEach(function(s) {
      arr.push({
        selected: true,
        name: str
      });
      //});
      return arr;
    }
    $scope.save = function () {
      // convert timemap to events
      // if events exist update
      // if not save new ones
      $appstate.getVisibleChildren().forEach(function (ob) {
        var kid = $facility.findChild(ob.id);
        var m = timeMap[ob.id];
        if (!m)
          return;
        var str = '';
        var rms = '';
        var tchs = '';
        var timestmps = '';
        for (var p in m) {
          if (m.hasOwnProperty(p) && m[p] && m[p].isSet === true) {
            str += p + ';';
            rms += m[p].room.id + ';';
            tchs += m[p].teacher.id + ';';
            timestmps += m[p].timestamp + ';';
          }
        }
        var event;
        var timestamp;
        if (m.evt) {
          event = m.evt;
          var room, times, teacher, tstamp;
          event.eventValues.forEach(function (ev, idx) {
            if (ev.eventTypeTemplate.name === 'Room')
              room = idx;
            if (ev.eventTypeTemplate.name === 'Times')
              times = idx;
            if (ev.eventTypeTemplate.name === 'Timestamp')
              tstamp = idx;
            if (ev.eventTypeTemplate.name === 'Teacher')
              teacher = idx;
          });
          event.eventValues[times].value = { choices: buildValue(str) };
          event.eventValues[times].ett = event.eventValues[times].eventTypeTemplate;
          event.eventValues[times].type = 'MULTI_SELECT';
          event.eventValues[room].value = { selected: { name: rms } };
          event.eventValues[room].ett = event.eventValues[room].eventTypeTemplate;
          event.eventValues[room].type = 'SINGLE_SELECT';
          event.eventValues[teacher].value = { selected: { name: tchs } };
          event.eventValues[teacher].ett = event.eventValues[teacher].eventTypeTemplate;
          event.eventValues[teacher].type = 'SINGLE_SELECT';
          event.eventValues[tstamp].value = { selected: { name: timestmps } };
          event.eventValues[tstamp].ett = event.eventValues[tstamp].eventTypeTemplate;
          event.eventValues[tstamp].type = 'SINGLE_SELECT';
          timestamp = new Date(event.timestamp);
        } else {
          var items = _.filter($facility.getSync().eventTypes, function (et) {
              return et.categories[0].id === $facility.findRoom(kid.roomId).category.id && et.name === 'Name To Face';
            });
          if (items && items.length && items.length > 0) {
            event = $adapter.createNewEvent(items[0]);
            var room, times, teacher, tstamp;
            items[0].eventTypeTemplates.forEach(function (eventTypeTemplate, idx) {
              if (eventTypeTemplate.name === 'Room')
                room = idx;
              if (eventTypeTemplate.name === 'Times')
                times = idx;
              if (eventTypeTemplate.name === 'Teacher')
                teacher = idx;
              if (eventTypeTemplate.name === 'Timestamp')
                tstamp = idx;
            });
            event.eventValues = [];
            timestamp = new Date();
            event.eventValues.push({
              ett: items[0].eventTypeTemplates[times],
              value: { choices: buildValue(str) },
              type: 'MULTI_SELECT'
            });
            event.eventValues.push({
              ett: items[0].eventTypeTemplates[room],
              value: { selected: { name: rms } },
              type: 'SINGLE_SELECT'
            });
            event.eventValues.push({
              ett: items[0].eventTypeTemplates[teacher],
              value: { selected: { name: tchs } },
              type: 'SINGLE_SELECT'
            });
            event.eventValues.push({
              ett: items[0].eventTypeTemplates[tstamp],
              value: { selected: { name: timestmps } },
              type: 'SINGLE_SELECT'
            });
          }
        }
        if (event) {
          event.observed = [ob];
          event.observers = [$facility.getSync().currentUser];
          // to distinguish between tracking charts
          event.notes = 'TrackingChart:' + currentTrackingChart.name;
          var day = $utils.getDatePart(timestamp);
          var time = $utils.getTimePart(timestamp);
          event.timestamp = $utils.createGsonTSFromDTStrings(day, time);
          $events.saveEvent(event, true);
        } else {
          $utils.alert('Name To Face Not Found!');
        }
      });
      $events.push();
      var eventListener = $rootScope.$on('pushComplete', function (scope, hasError, error) {
          if (!hasError) {
            $uiAlert.pushSuccessAlert($translate.instant('NameToFaceSaved'), 'NAMETOFACE');
          } else {
            $uiAlert.pushErrorAlert('There was an error saving Name To Face.', 'NAMETOFACE');
          }
          displayAlerts();
        });
      $scope.$on('$destroy', eventListener);
    };
    //$appstate.nav.next = $scope.save;
    $appstate.nav.nextState = $scope.save;
    // open kiosk PIN modal if collectTeacherPin is set to true
    var collectTeacherPin = $facility.getSync().collectTeacherPins;
    var isAdmin = $facility.isAdmin();
    if (collectTeacherPin && !isAdmin) {
      var kioskModal = $modal.open({
          templateUrl: 'views/popups/kioskModal.html',
          controller: 'KioskModalCtrl',
          resolve: {
            items: function () {
              return {
                'pin': $scope.pin,
                'initials': $scope.initials
              };
            }
          }
        });
      // When modal is dismissed close this window and return to main
      var doDismissFn = function (reason) {
        $appstate.goHome();
      };
      // when modal is closed without being dismissed.
      var doClosureFn = function (teacher) {
        var matchingTeacher = $appstate.model.selectedRoom.teachers.filter(function (t) {
            return t.id === teacher.id;
          });
        $scope.values.currTeacher = matchingTeacher[0];
        $scope.values.isTeacherSelectDisabled = true;
      };
      // chain functionality onto result promise
      kioskModal.result.then(doClosureFn, doDismissFn);
    }
    function displayAlerts() {
      var alerts = $uiAlert.popAlertsFor('NAMETOFACE');
      if (alerts) {
        $scope.alerts = alerts;
        $timeout(function () {
          $scope.alerts = null;
        }, 5000);
      }
    }
    $scope.getAlertClass = function (alert) {
      if (alert.type === $uiAlert.ALERT_TYPE_SUCCESS) {
        return 'alert alert-success';
      } else if (alert.type === $uiAlert.ALERT_TYPE_WARNING) {
        return 'alert alert-warning';
      } else if (alert.type === $uiAlert.ALERT_TYPE_ERROR) {
        return 'alert alert-danger';
      } else {
        return 'alert';
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('EditmyprofileCtrl', [
  '$scope',
  '$facility',
  '$image',
  '$utils',
  '$krConfig',
  '$appstate',
  '$native',
  '$auth',
  '$online',
  '$messageBox',
  '$backend',
  '$rootScope',
  '$profile',
  '$translate',
  function ($scope, $facility, $image, $utils, $krConfig, $appstate, $native, $auth, $online, $messageBox, $backend, $rootScope, $profile, $translate) {
    $scope.teacher = $facility.getSync().currentUser;
    $scope.setPhoto = true;
    $scope.userparams = {
      username: new String($scope.teacher.username),
      password: '',
      verifyPassword: ''
    };
    function getImg(force) {
      $image.getImage($scope.teacher.id, $utils.createPhotoUrl($krConfig.s3Base, $scope.teacher.photoPath), force).then(function (d) {
        $scope.teacher.photoUrl = d;
      });
    }
    getImg(false);
    $scope.native = $native.isNative();
    $scope.setProfilePhoto = function (usr) {
      $profile.setProfilePhoto(usr).then(function (ret) {
        $facility.getSync().currentUser.photoPath = ret.photoPath;
        $facility.getSync().currentUser.photoUrl = ret.photoUrl;
        $facility.save();
        getImg(true);
        console.log('got profile photo callback');
        $messageBox.open($translate.instant('Success'), $translate.instant('ProfilePhotoUpdated'), { oktext: $translate.instant('Ok') }).result.then(function () {
          $appstate.nav.goHome();
        });
      });
    };
    $scope.saveUserName = function () {
      //alert("in save username");
      if ($scope.userparams.username == $scope.teacher.username) {
        return;
      }
      //user name blank -> error and show message
      if ($scope.userparams.username == '') {
        //no special characters
        $messageBox.open($translate.instant('Error'), $translate.instant('UsernameFieldBlank'), { oktext: $translate.instant('Ok') });
        return;
      }
      $profile.updateProfile({ username: $scope.userparams.username }).then(function (data) {
        //console.log(data);
        if (data.success) {
          //set on the current user and user in fac
          $facility.getSync().currentUser.username = $scope.userparams.username;
          var usr = $facility.findUser($facility.getSync().currentUser.id);
          usr.username = $scope.userparams.username;
          $facility.save();
          // update auth creds
          //getcredentialssync
          var creds = $auth.getCredentialsSync();
          console.log(creds);
          $auth.setCredentials($scope.userparams.username, creds.token);
          //auth.setcredentials -> current username and token
          //identify all text and put in translation: language.en.json, languages.pt-br.json
          $messageBox.open($translate.instant('Success'), $translate.instant('UsernameUpdated'), { oktext: $translate.instant('Ok') }).result.then(function () {
            $appstate.nav.goHome();
          });
        } else {
          $messageBox.open($translate.instant('Error'), data.message, { oktext: $translate.instant('Ok') });
        }
      }, function (err) {
        $messageBox.open($translate.instant('Error'), $translate.instant('CommunicationError'), { oktext: $translate.instant('Ok') });
      });
    };
    $scope.savePassword = function () {
      if ($scope.userparams.password == '' && $scope.userparams.verifyPassword == '') {
        return;
      }
      //user name blank -> error and show message
      if ($scope.userparams.password != $scope.userparams.verifyPassword) {
        $messageBox.open($translate.instant('Error'), $translate.instant('PasswordsNoMatch'), { oktext: $translate.instant('Ok') });
        return;
      }
      //in app -> update cred if password update was successful
      $profile.updateProfile({
        password: $scope.userparams.password,
        verifyPassword: $scope.userparams.verifyPassword
      }).then(function (data) {
        console.log(data);
        if (data.success) {
          $messageBox.open($translate.instant('Success'), $translate.instant('PasswordUpdated'), { oktext: $translate.instant('Ok') }).result.then(function () {
            $appstate.nav.goHome();
          });
          var creds = $auth.getCredentialsSync();
          console.log(creds);
          $auth.setCredentials(creds.username, data.message);
        } else {
          //data.message is going to be new token
          $messageBox.open($translate.instant('Error'), data.message, { oktext: $translate.instant('Ok') });
        }
      }, function (err) {
        $messageBox.open($translate.instant('Error'), $translate.instant('CommunicationError'), { oktext: $translate.instant('Ok') });
      });
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('AbcCtrl', [
  '$scope',
  '$appstate',
  '$native',
  function ($scope, $appstate, $native) {
  }
]);
'use strict';
angular.module('kidreports2App').controller('LogoutCtrl', [
  '$appstate',
  function ($appstate) {
    $appstate.nav.logout();
  }
]);
'use strict';
angular.module('kidreports2App').factory('$image', [
  '$localdb',
  '$q',
  '$native',
  '$timeout',
  '$model',
  function ($localdb, $q, $native, $timeout, $model) {
    var svc = {};
    var PROFILE_PIC = 'profilePic-';
    function cacheFile(url) {
      if ($native.isNative() && !$model.disableProfilePictureCaching) {
        return $native.cacheURL(url);
      } else {
        var d = $q.defer();
        $timeout(function () {
          d.resolve({
            file: url,
            url: url
          });
        });
        return d.promise;
      }
    }
    function cache(url, d, id, returnFile, retry) {
      // DEV-1262: Prevent default profile image(s) (and local images) from being cached:
      var pattern = /^((http|https):\/\/)/;
      if (!pattern.test(url)) {
        return false;
      }
      cacheFile(url).then(function (file) {
        var obj = {
            url: url,
            path: file.url,
            file: file.file
          };
        $localdb.set(id, obj).then(function () {
          if (returnFile)
            d.resolve(file.file);
          else {
            var url = typeof forge != 'undefined' && forge.httpd ? forge.httpd.normalize(file.url) : file.url;
            d.resolve(url);
          }
        });
      }, function (err) {
        console.log(err);
        // retry
        if (retry)
          cache(url, d, id, returnFile, true);
        else
          d.reject(err);
      });
    }
    function verifyFile(obj) {
      if ($native.isNative() && !$model.disableProfilePictureCaching) {
        return $native.isFile(obj.file);
      } else {
        var d = $q.defer();
        $timeout(function () {
          d.resolve(obj);
        });
        return d.promise;
      }
    }
    // TODO reexamine and use forge local storage?
    function getFromCache(url, id, fetch, returnFile) {
      var d = $q.defer();
      $localdb.get(id).then(function (obj) {
        if (obj && obj.path && (returnFile || url == obj.url) && !fetch) {
          verifyFile(obj).then(function (ret) {
            //alert("got:"+ret.uri);
            if (returnFile)
              d.resolve(obj.file);
            else {
              var url = typeof forge != 'undefined' && forge.httpd ? forge.httpd.normalize(ret.url) : ret.url;
              d.resolve(url);
            }
          }, function (err) {
            cache(url, d, id, returnFile);
          });
        } else {
          cache(url, d, id, returnFile);
        }
      });
      return d.promise;
    }
    svc.getImage = function (id, url, clean) {
      return getFromCache(url, PROFILE_PIC + id, clean);
    };
    svc.getProfilePicKeys = function () {
      var pic = [];
      _.each($localdb.getKeys(), function (c) {
        if (c.indexOf(PROFILE_PIC) > -1)
          pic.push(c);
      });
      return pic;
    };
    svc.getProfilePicCount = function () {
      return svc.getProfilePicKeys().length;
    };
    svc.getProfilePicSize = function () {
      var d = $q.defer();
      if ($native.isNative()) {
        var promises = _.map(svc.getProfilePicKeys(), function (key) {
            return getFromCache(null, key, false, true);
          });
        $q.all(promises).then(function (arr) {
          promises = _.map(arr, function (file) {
            return $native.getInfo(file);
          });
          $q.all(promises).then(function (arr) {
            var size = 0;
            console.log('Arr:', JSON.stringify(arr));
            _.each(arr, function (s) {
              console.log('S:', s);
              size += s.size;
            });
            var sizeStr = '0';
            if (size > 0 && size < 1000000) {
              sizeStr = Math.round(size / 1000) + 'KB';
            } else if (size >= 1000000) {
              sizeStr = Math.round(size / 10000) / 100 + 'MB';
            }
            d.resolve(sizeStr);
          });
        });
      } else
        d.resolve(0);
      return d.promise;
    };
    svc.removeAllProfilePics = function () {
      var d = $q.defer();
      if ($native.isNative()) {
        var promises = _.map(svc.getProfilePicKeys(), function (key) {
            return getFromCache(null, key, false, true);
          });
        $q.all(promises).then(function (arr) {
          promises = _.map(arr, function (file) {
            return $native.removeFile(file);
          });
          $q.all(promises).then(function () {
            promises = _.map(svc.getProfilePicKeys(), $localdb.remove);
            d.resolve($q.all(promises));
          }, function (err) {
            d.reject(err);
          });
        });
      } else
        d.resolve();
      return d.promise;
    };
    return svc;
  }
]);
'use strict';
angular.module('kidreports2App').factory('$msg', [
  '$q',
  '$modal',
  '$model',
  '$timeout',
  function ($q, $modal, $model, $timeout) {
    var _timeout;
    var _this = {
        setModalMessage: function (msg) {
          $model.modalMsg = msg;
        },
        msg: function (msg, show, msgType, timer) {
          if (_timeout)
            $timeout.cancel(_timeout);
          _this.alert = show ? {
            message: msg,
            type: msgType ? msgType : 'success'
          } : {};
          var time = timer || 3000;
          $model.hideMessage = !show;
          if (show) {
            _timeout = $timeout(function () {
              _timeout = null;
              _this.msg(null, false);
            }, time);
          }
        },
        hideMsg: function () {
          $model.alert = {};
          $model.hideMessage = true;
        }
      };
    return _this;
  }
]);
'use strict';
angular.module('kidreports2App').factory('$profile', [
  '$localdb',
  '$q',
  '$native',
  '$timeout',
  '$facility',
  '$image',
  '$utils',
  '$krConfig',
  '$appstate',
  '$auth',
  '$online',
  '$messageBox',
  '$backend',
  '$rootScope',
  '$ajax',
  function ($localdb, $q, $native, $timeout, $facility, $image, $utils, $krConfig, $appstate, $auth, $online, $messageBox, $backend, $rootScope, $ajax) {
    var svc = {};
    function _setProfilePhoto(obj) {
      var d = $q.defer();
      var token = $auth.getCredentialsSync();
      var creds = '/data/uploadMedia?type=PHOTO&profileId=' + obj.id + '&token=' + token.token;
      // short circuit for offline mode
      if (!$online.state()) {
        $messageBox.open('Offline Mode', 'Please try this again when you are online.', { oktext: 'Ok' });
        return;
      }
      $native.singleimage(function (err, file, url) {
        if (err) {
        } else {
          $timeout(function () {
            obj.photoUrl = url;  //"img/loading.gif";
          }, 10);
          var func = $rootScope.$on('uploadSuccess', function (evt, resp) {
              func();
              var params = resp.params;
              //console.log(resp);
              if (params && params.id == obj.id) {
                //console.log("calling getprofileUrl");
                $backend.getProfileUrl(params.id, function (evt, data) {
                  var data = angular.fromJson(data);
                  $rootScope.$apply(function () {
                    console.log(data);
                    console.log(JSON.stringify(data));
                    obj.photoPath = data.path;
                    $image.getImage(obj.id, $utils.createPhotoUrl($krConfig.s3Base, obj.photoPath)).then(function (d) {
                      obj.photoUrl = d;
                      $rootScope.$broadcast('profileImageUpdate', obj);
                      $facility.save();
                      var retObj = {
                          photoUrl: d,
                          photoPath: data.path
                        };
                      var kid = $facility.findChild(obj.id);
                      if (kid) {
                        kid.photoPath = data.path;
                        kid.photoUrl = obj.photoUrl;
                        $facility.save();
                      } else {
                        var parent = $facility.findUser(obj.id);
                        if (parent) {
                          parent.photoPath = data.path;
                          parent.photoUrl = obj.photoUrl;
                          $facility.save();
                        }
                      }
                      d.resolve(retObj);
                    });
                    //alert('Upload successful');
                    $appstate.msg('Profile Upload Successful!', true);
                  });
                });
              }
            });
          $native.upload(file, url, null, creds, null, { id: obj.id });
        }
      }, $facility.getImageParameters());
      return d.promise;
    }
    function _updateProfile(userparams) {
      //alert('button clicked');
      var url = $krConfig.getAjaxBase() + '/data/updateProfile';
      var opts = {
          method: 'POST',
          url: url,
          data: userparams
        };
      var d = $q.defer();
      $ajax.ajax(opts, function (err, data) {
        if (err)
          d.reject(err);
        else
          d.resolve(data);
      });
      return d.promise;
    }
    svc.setProfilePhoto = _setProfilePhoto;
    svc.updateProfile = _updateProfile;
    svc.native = $native.isNative();
    return svc;
  }
]);
'use strict';
angular.module('kidreports2App').factory('$pdf', [
  '$localdb',
  '$q',
  '$native',
  '$model',
  '$timeout',
  function ($localdb, $q, $native, $model, $timeout) {
    var svc = {};
    svc.open = function (url) {
      if ($native.isNative()) {
        _open(url);
      } else {
        window.open(url);
      }
    };
    function _open(baseurl) {
      var pdfurl = baseurl.replace('https://s3.amazonaws.com/kidreports-media/public/pdfjsweb/viewer.html?file=', '');
      var page = '';
      var pageidx = pdfurl.indexOf('#page=');
      if (pageidx > -1) {
        page = pdfurl.substr(pageidx);
        pdfurl = pdfurl.substr(0, pageidx);
      }
      //test url with no spaces
      //pdfurl = "https://s3.amazonaws.com/kidreports-media/documents/af890cqw48jcmclahkluxdt89ujnpy8h7s4m0aweq-k490um-4qfAWZKZDKJ89/cni/x39b8vaj3cxh934jgszkldjg89/2021/March/T/MAR2021_T_LEB_Combo.pdf";
      getFromCache(pdfurl, pdfurl).then(function (obj) {
        forge.file.getFileFromSourceDirectory('pdfjsweb/viewer.html', function (f) {
          forge.file.getScriptURL(f, function (uri) {
            _openPDF(uri, obj, page);
          });
        });
      });
    }
    function _openPDF(uri1, uri2, page) {
      openAdvanced2(uri1 + '?file=' + uri2 + page);
    }
    function getFromCache(url, id) {
      var d = $q.defer();
      $localdb.get(id).then(function (obj) {
        if (obj && obj.path) {
          verifyFile(obj).then(function (ret) {
            d.resolve(obj.path);
          }, function (err) {
            cache(url, d, id);
          });
        } else {
          cache(url, d, id);
        }
      });
      return d.promise;
    }
    function verifyFile(obj) {
      return $native.isFile(obj.file);
    }
    function cacheFile(url) {
      return $native.cacheURL(url);
    }
    function cache(url, d, id) {
      cacheFile(url).then(function (file) {
        var obj = {
            url: url,
            path: file.url,
            file: file.file
          };
        $localdb.set(id, obj).then(function () {
          d.resolve(obj.path);
        });
      }, function (err) {
        forge.logging.info(err);
        d.reject(err);
      });
    }
    function openAdvanced2(theUrl) {
      forge.logging.info('OPEN:  ' + theUrl);
      forge.tabs.openAdvanced({ url: theUrl }, function (modal) {
        modal.loadFinished.addListener(function (obj) {
        });
        modal.loadError.addListener(function (obj) {
        });
        modal.closed.addListener(function () {
        });
      });
    }
    return svc;
  }
]);
'use strict';
angular.module('kidreports2App').factory('$quicklinks', [
  '$krConfig',
  '$ajax',
  '$timeout',
  '$q',
  function ($krConfig, $ajax, $timeout, $q) {
    var url = function () {
      return $krConfig.getAjaxBase() + '/quicklinks';
    };
    var credentials = {};
    var _linkCache = null;
    function getQuickLinks() {
      var d = $q.defer();
      if (_linkCache) {
        d.resolve(_linkCache);
      } else {
        var opts = {
            method: 'GET',
            url: url(),
            headers: credentials,
            params: {}
          };
        $ajax.ajax(opts, function (err, links) {
          if (!err) {
            _linkCache = links;
            d.resolve(links);
          } else {
            console.log(err);
            d.resolve([]);
          }
        });
      }
      return d.promise;
    }
    return {
      setCredentials: function (creds) {
        var d = $q.defer();
        // only set if they are different.
        if (credentials.Authorization && credentials.Authorization === creds.Authorization) {
          $timeout(function () {
            d.resolve();
          }, 0);
        } else {
          credentials = creds;
          d.resolve();
        }
        return d.promise;
      },
      get: function () {
        return getQuickLinks();
      },
      getMenuItem: function () {
        return {
          allowedRoles: [
            'PARENT_ROLE',
            'TEACHER_ROLE'
          ],
          dash: false,
          icon: 'edit.png',
          main: false,
          name: 'QuickLinks',
          optional: false,
          path: {
            root: 'quicklinks',
            states: ['quicklinks']
          },
          side: true,
          sideWhenDash: true,
          thirdParty: false,
          id: 'quicklinksItem'
        };
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('QuicklinksController', QuicklinksController);
QuicklinksController.$inject = [
  '$scope',
  '$quicklinks',
  '$facility',
  '$linkClickTracker'
];
/**
 * @function QuicklinksController
 * @author Joshua Belden <jcbelden@procaresoftware.com>
 * @param {*} $scope 
 * @param {*} $quicklinks 
 * @param {*} $facility 
 * @param {*} $linkClickTracker 
 * @constructor
 */
function QuicklinksController($scope, $quicklinks, $facility, $linkClickTracker) {
  var facility = $facility.getSync();
  var userId = facility.currentUser.id;
  var foldersShowing = {};
  $scope.folderCount = 0;
  $scope.showFolder = function (name) {
    return foldersShowing[name];
  };
  $scope.toggleShowFolder = function (name) {
    foldersShowing[name] = !foldersShowing[name];
  };
  $scope.trackLinkClick = function (url) {
    $linkClickTracker.persistLinkClick(userId, facility.id, null, url);
  };
  $quicklinks.get().then(function (links) {
    var items = [];
    var folders = {};
    _.each(links, function (link) {
      if (link.folderName) {
        if (!folders[link.folderName]) {
          folders[link.folderName] = [];
          items.push({
            label: link.folderName,
            items: folders[link.folderName]
          });
        }
        folders[link.folderName].push(link);
      } else {
        items.push(link);
      }
    });
    $scope.folderCount = Object.keys(folders).length;
    $scope.items = items;
  });
}
'use strict';
angular.module('kidreports2App').factory('$policies', [
  '$krConfig',
  '$ajax',
  '$timeout',
  '$q',
  function ($krConfig, $ajax, $timeout, $q) {
    // Service logic
    var credentials = {};
    var url = function () {
      return $krConfig.getAjaxBase() + '/policies';
    };
    // array of messages like this.  If fromUserId != currentUser.id then this goes into inbox.
    //    body: "this is a sample message"
    //    id: 131037
    //    sentDate: "Aug 6, 2012 12:36:43 AM"
    //    subject: "test"
    //    fromUserId: 12345
    var _links = null;
    function getPolicies() {
      var d = $q.defer();
      if (_links) {
        d.resolve(_links);
      } else {
        var opts = {
            method: 'GET',
            url: url(),
            headers: credentials,
            params: {}
          };
        $ajax.ajax(opts, function (err, links) {
          if (!err) {
            _links = links;
            d.resolve(links);
          } else {
            console.log(err);
            d.resolve([]);
          }
        });
      }
      return d.promise;
    }
    function saveComment(lsid, parentComment, content, d) {
      var u = url() + '/saveComment';
      var params = {
          lsaddCommentTo: parentComment,
          content: content,
          lsID: lsid
        };
      var opts = {
          method: 'POST',
          url: u,
          headers: credentials,
          params: params
        };
      $ajax.ajax(opts, function (err, data) {
        if (data && data.id) {
          _links = null;
          getPolicies().then(function (f) {
            d.resolve(data);
          });
        } else
          d.reject(err);
      });
    }
    function deleteComment(id, d) {
      var u = url() + '/deleteComment';
      var params = { commentID: id };
      var opts = {
          method: 'POST',
          url: u,
          headers: credentials,
          params: params
        };
      $ajax.ajax(opts, function (err, data) {
        if (!err) {
          _links = null;
          getPolicies().then(function (policies) {
            d.resolve(policies);
          });
        } else
          d.reject(err);
      });
    }
    var _menuItem = {
        allowedRoles: [
          'PARENT_ROLE',
          'TEACHER_ROLE'
        ],
        dash: false,
        icon: 'edit.png',
        main: false,
        name: 'My Policies',
        optional: false,
        path: {
          root: 'policies',
          states: ['policies']
        },
        side: true,
        sideWhenDash: true,
        thirdParty: false,
        id: 'policyItem',
        sref: 'home.dash.policies'
      };
    return {
      setCredentials: function (creds) {
        var d = $q.defer();
        // only set if they are different.
        if (credentials.Authorization && credentials.Authorization === creds.Authorization) {
          $timeout(function () {
            d.resolve();
          }, 0);
        } else {
          credentials = creds;
          d.resolve();
        }
        return d.promise;
      },
      addComment: function (pComment, story, content) {
        var d = $q.defer();
        // Replace any existing events (server always wins)
        var results;
        saveComment(story, pComment, content, d);
        return d.promise;
      },
      deleteComment: function (id) {
        var d = $q.defer();
        // Replace any existing events (server always wins)
        var results;
        deleteComment(id, d);
        return d.promise;
      },
      get: function () {
        return getPolicies();
      },
      getMenuItem: function () {
        return _menuItem;
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('PoliciesCtrl', [
  '$scope',
  '$appstate',
  '$krCache',
  '$policies',
  '$modal',
  '$timeout',
  '$msg',
  '$facility',
  function ($scope, $appstate, $krCache, $policies, $modal, $timeout, $msg, $facility) {
    function setup() {
      $policies.get().then(function (links) {
        $scope.items = links;
      });
    }
    $scope.isAdmin = function () {
      var usr = $facility.getSync().currentUser;
      return usr.legacyRole == 'FACILITY_ADMIN_ROLE' || usr.legacyRole == 'REGION_ADMIN_ROLE' || usr.legacyRole == 'COMPANY_ADMIN_ROLE' || usr.legacyRole == 'SYSTEM_ADMIN_ROLE';
    };
    $scope.view = function (item) {
      //alert(item);
      //$scope.showMessage(item);
      $scope.mainMessageScrollDivClass = '';
      $scope.policy = item;
      $scope.mode = 'view';
      $timeout($scope.openModal, 10);
    };
    $scope.openModal = function () {
      if ($scope.modal)
        return;
      $appstate.model.backButtonFunction = $scope.closeModal;
      $scope.modal = $modal.open({
        templateUrl: 'views/popups/viewPolicyModal.html',
        controller: 'ViewpolicyCtrl',
        resolve: {
          _scope: function () {
            return $scope;
          }
        }
      });
    };
    $scope.closeModal = function () {
      $scope.modal.close();
      $scope.modal = null;
      $scope.mainMessageScrollDivClass = 'scrollable';
      $appstate.model.backButtonFunction = null;
    };
    $scope.addComment = function (commentId) {
      $scope.commenting = true;
      if (commentId)
        $scope.commentId = commentId;
      $timeout(function () {
        var ele = angular.element(document.querySelector('.modal'))[0];
        ele.scrollTop = ele.scrollHeight;
      });
    };
    $scope.cancelComment = function () {
      $scope.commenting = false;
      $scope.comment = '';
    };
    $scope.getCommentName = function (item) {
      var usr = $facility.findUser(item.fromUser.id);
      if (usr)
        return usr.firstName + ' ' + usr.lastName;
      else
        return '--';
    };
    $scope.getCommentDate = function (item) {
      return moment(item.sentDate).format('DD-MM-YYYY h:mm a');
    };
    $scope.mainMessageScrollDivClass = 'scrollable';
    $scope.deleteComment = function (id) {
      $scope.isSending = true;
      $timeout(function () {
        realDeleteComment(id);
      }, 0);
    };
    $scope.saveComment = function (body) {
      $scope.isSending = true;
      $timeout(function () {
        realSaveMsg(body);
      }, 0);
    };
    function realDeleteComment(id) {
      //alert('saving message'+item);
      $policies.deleteComment(id).then(function (policies) {
        //alert('saved');
        $msg.msg('Comment Deleted!');
        $scope.isSending = false;
        _.each(policies, function (p) {
          if (p.id == $scope.policy.id) {
            $scope.policy = p;
          }
        });
        $timeout(setup);
      });
    }
    function realSaveMsg(item) {
      //alert('saving message'+item);
      $policies.addComment($scope.commentId, $scope.policy.id, item).then(function (m) {
        //alert('saved');
        $msg.msg('Comment Saved!');
        $scope.isSending = false;
        $scope.policy = m;
        $scope.cancelComment();
        $timeout(setup);
      });
    }
    setup();
  }
]);
'use strict';
angular.module('kidreports2App').directive('renderLink', [
  '$backend',
  '$native',
  '$location',
  '$state',
  '$utils',
  '$pdf',
  '$facility',
  '$linkClickTracker',
  '$appstate',
  function ($backend, $native, $location, $state, $utils, $pdf, $facility, $linkClickTracker, $appstate) {
    return {
      templateUrl: 'views/directives/renderLink.html',
      restrict: 'E',
      scope: { ev: '=' },
      link: function (scope, element, attrs, $scope) {
        scope.isParent = $backend.getCurrentUser().role == 'PARENT_ROLE';
        scope.isNative = $native.isNative();
        scope.getTarget = function (url) {
          if (url.indexOf('kidreports') == 0) {
            return '';
          } else
            return '_blank';
        };
        scope.format = $utils.formatLink;
        scope.isPDFandNative = function (ev) {
          return ev.url.toLowerCase().indexOf('.pdf') > -1 && scope.isNative && !$native.iOS();
        };
        scope.showPDF = function (ev) {
          $pdf.open(ev.url);
        };
        scope.goTo = function (url) {
          if (typeof forge != 'undefined') {
            // if ( url.indexOf(pdfStr) == 0 ) {
            //   $state.go("main.action.pdf", {file:url.replace(pdfStr, "")})
            // } else {
            $location.path(url.replace('#', ''));  //}
          } else
            document.location.href = url;
        };
        element.bind('click', function () {
          var userId = $facility.getSync().currentUser.id;
          var role = $facility.getSync().currentUser.role;
          var url = scope.ev.url;
          var facilityId = $facility.id;
          var roomId = $appstate.model.selectedRoom.id;
          $linkClickTracker.persistLinkClick(userId, facilityId, roomId, url);
        });
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').factory('$linkClickTracker', [
  '$ajax',
  '$krConfig',
  '$q',
  '$timeout',
  function ($ajax, $krConfig, $q, $timeout) {
    var credentials = {};
    return {
      persistLinkClick: function (teacherId, facilityId, roomId, url) {
        var baseUrl = $krConfig.getAjaxBase();
        var lickClickData = {
            'roomId': roomId,
            'linkUrl': url
          };
        var opts = {
            method: 'POST',
            url: baseUrl + '/trackLinkClick',
            data: lickClickData
          };
        $ajax.ajax(opts, function (err, data, d) {
          if (err) {
            console.error('Error making request: linkClickTracker:' + url);
          }
        });
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('KioskModalCtrl', [
  '$scope',
  '$kiosk',
  '$messageBox',
  '$translate',
  '$modalInstance',
  '$appstate',
  'items',
  function ($scope, $kiosk, $messageBox, $translate, $modalInstance, $appstate, items) {
    $scope.numbers = [
      1,
      2,
      3,
      4,
      5,
      6,
      7,
      8,
      9
    ];
    $scope.value = {
      pin: '',
      initials: ''
    };
    $scope.cancel = function () {
      $modalInstance.dismiss('cancel');
    };
    $scope.doLogin = function () {
      var usr = $kiosk.login(Number($scope.value.pin), $scope.value.initials, 'TEACHER_ROLE');
      if (usr) {
        $modalInstance.close(usr);
      } else {
        $scope.modalDisplay = 'none';
        var messageBox = $messageBox.open($translate.instant('Error'), $translate.instant('InvalidPINInitials'), { oktext: 'Ok' });
        messageBox.result.then(function () {
          $scope.modalDisplay = 'block';
        });
        $scope.value.initials = '';
        $scope.value.pin = '';
      }
    };
    $scope.addNumber = function (n) {
      if (!$scope.value.pin)
        $scope.value.pin = 0;
      $scope.value.pin = Number($scope.value.pin.toString() + n.toString());
    };
    $scope.removeNumber = function () {
      var str = $scope.value.pin.toString();
      $scope.value.pin = Number(str.substring(0, str.length - 1));
      if ($scope.value.pin === 0)
        $scope.value.pin = '';
    };
  }
]);
'use strict';
angular.module('kidreports2App').factory('$stateStandards', stateStandardsService);
stateStandardsService.$inject = [
  '$ajax',
  '$krConfig',
  '$q'
];
/**
 * @function stateStandardsService()
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @param $ajax
 * @param $krConfig
 * @param $q
 * @returns {{getDomain: (function(string): *), getFacility: (function(): *)}}
 */
function stateStandardsService($ajax, $krConfig, $q) {
  // Caching variables:
  var facility = false;
  var stateStandards = false;
  var performanceRatingLabels = false;
  return {
    createFullStandard: createFullStandard,
    deepFindFirst: deepFindFirst,
    getFacility: getFacility,
    getStandards: getStandards,
    getPerformanceRatingLabels: getPerformanceRatingLabels,
    saveEventStandards: saveEventStandards
  };
  /**
   * @function getFullStandard()
   * @description Creates an object that has additional domain and subdomain information to be used for saving
   * @param {number} depthLevel
   * @param {object} domain
   * @param {object} subdomain
   * @param {object} standard
   * @param {object|null} depthThree
   * @param {object|null} depthFour
   * @returns {{}}
   */
  function createFullStandard(depthLevel, domain, subdomain, standard, depthThree, depthFour, ageGroup) {
    return {
      'depth_level': depthLevel,
      'domain_guid': _.get(domain, 'guid'),
      'domain_number': _.get(domain, 'standard_number'),
      'domain_text': _.get(domain, 'standard_text'),
      'subdomain_guid': _.get(subdomain, 'guid'),
      'subdomain_number': _.get(subdomain, 'standard_number'),
      'subdomain_text': _.get(subdomain, 'standard_text'),
      'set': _.get(standard, 'set'),
      'guid': _.get(standard, 'guid'),
      'standard_text': _.get(standard, 'standard_text'),
      'standard_number': _.get(standard, 'standard_number'),
      'adopted': _.get(standard, 'adopted'),
      'modified': _.get(standard, 'modified'),
      'depth_three_guid': _.get(depthThree, 'guid') || null,
      'depth_three_number': _.get(depthThree, 'standard_number') || null,
      'depth_three_text': _.get(depthThree, 'standard_text') || null,
      'depth_four_guid': _.get(depthFour, 'guid') || null,
      'depth_four_number': _.get(depthFour, 'standard_number') || null,
      'depth_four_text': _.get(depthFour, 'standard_text') || null,
      'age_group': ageGroup.label
    };
  }
  /**
   * @function deepFindFirst()
   * @description Used to deep find a nested standard (regardless of depth level)
   * @param {object} tree
   * @param {string} childrenKey
   * @param {object} objToFindBy
   * @returns {*}
   */
  function deepFindFirst(tree, childrenKey, objToFindBy) {
    var treeToReturn = tree;
    var found = false;
    var findKeys = Object.keys(objToFindBy);
    findKeys.forEach(function (key) {
      _.isEqual(tree[key], objToFindBy[key]) ? found = true : found = false;
    });
    if (found) {
      return tree;
    }
    var findInChildren = function (obj, childrenKey, objToFindBy) {
      var foundInChild = false;
      if (obj.hasOwnProperty(childrenKey)) {
        for (var i = 0; i < obj[childrenKey].length; i++) {
          findKeys.forEach(function (key) {
            _.isEqual(obj[childrenKey][i][key], objToFindBy[key]) ? foundInChild = true : foundInChild = false;
          });
          if (foundInChild) {
            found = true;
            treeToReturn = obj[childrenKey][i];
            break;
          }
        }
        if (!foundInChild && !found) {
          obj[childrenKey].forEach(function (child) {
            findInChildren(child, childrenKey, objToFindBy);
          });
        }
      }
      return obj;
    };
    findInChildren(tree, childrenKey, objToFindBy);
    return found ? treeToReturn : false;
  }
  /**
   * @function getFacility()
   * @description Retrieves state standards information for a given facility
   * @returns {Promise}
   */
  function getFacility() {
    var baseUrl = $krConfig.getAjaxBase();
    var url = '/api/facility/state_standards';
    var deferred = $q.defer();
    if (facility) {
      deferred.resolve(facility);
    } else {
      $ajax.ajax({
        method: 'GET',
        url: baseUrl + url,
        params: null
      }, function (error, data) {
        if (error) {
          deferred.reject(error);
        } else {
          facility = data;
          deferred.resolve(data);
        }
      });
    }
    return deferred.promise;
  }
  /**
   * @function getStandards()
   * @param {string} state
   * @returns {*}
   */
  function getStandards(state) {
    var baseUrl = $krConfig.getAjaxBase();
    var url = '/api/state_standards';
    var deferred = $q.defer();
    if (stateStandards) {
      deferred.resolve(stateStandards);
    } else {
      $ajax.ajax({
        method: 'GET',
        url: baseUrl + url,
        params: { state: state }
      }, function (error, data) {
        if (error) {
          deferred.reject(error);
        } else {
          stateStandards = data;
          deferred.resolve(data);
        }
      });
    }
    return deferred.promise;
  }
  /**
   * @function getPerformanceRatingLabels()
   * @param {string} facilityId
   * @returns {*}
   */
  function getPerformanceRatingLabels(facilityId) {
    var baseUrl = $krConfig.getAjaxBase();
    var url = '/api/state_standards';
    var deferred = $q.defer();
    if (performanceRatingLabels) {
      deferred.resolve(performanceRatingLabels);
    } else {
      $ajax.ajax({
        method: 'GET',
        url: baseUrl + url,
        params: {
          listperformanceratinglabels: true,
          facilityid: facilityId
        }
      }, function (error, data) {
        if (error) {
          deferred.reject(error);
        } else {
          performanceRatingLabels = data;
          deferred.resolve(data);
        }
      });
    }
    return deferred.promise;
  }
  /**
   * @function saveEventStandards()
   * @param {number} eventId
   * @param {Array<object>} standards
   */
  function saveEventStandards(eventId, standards) {
    var _standards = angular.copy(standards);
    var baseUrl = $krConfig.getAjaxBase();
    var url = '/api/events/' + eventId + '/statestandards';
    var deferred = $q.defer();
    $ajax.ajax({
      method: 'POST',
      url: baseUrl + url,
      data: { 'state_standards': standards }
    }, function (error, data) {
      if (error) {
        deferred.reject(error);
      } else {
        deferred.resolve(_standards);
      }
    });
    return deferred.promise;
  }
}
'use strict';
angular.module('kidreports2App').controller('EventobservationsCtrl', EventObservationsController);
function EventObservationsController($scope, $appstate, $events, $adapter, $facility, $timeout, $rootScope, $kidplanner, $state, $stateStandards) {
  var event = $appstate.model.currentEvent;
  $scope.clearSingleObservation = clearSingleObservation;
  $scope.saveSingleObservation = saveSingleObservation;
  $scope.saveAllObservations = saveAllObservations;
  $scope.cancel = cancel;
  $scope.save = save;
  var facility = $facility.getSync();
  $scope.child = _.find(facility.children, function (child) {
    return child.id === event.child.id;
  });
  $scope.eventTypeName = event.eventType.name;
  $scope.eventDate = moment(new Date(event.timeOccurred)).format('MMM D, YYYY');
  $scope.eventTime = moment(new Date(event.timeOccurred)).format('h:mm A');
  $stateStandards.getPerformanceRatingLabels(facility.id).then(function (result) {
    $scope.progressLevels = [
      result['statestandards.labels.not-observed'],
      result['statestandards.labels.emerging'],
      result['statestandards.labels.progressing'],
      result['statestandards.labels.skilled']
    ];
  });
  $scope.domain = event.stateStandards[0].domain_text;
  $scope.stateStandards = prepareCustomStandardsModel(event.stateStandards);
  $scope.itemExpanded = [[true]];
  $scope.$on('pushComplete', stopAllProgressSpinners);
  function beginProgressSpinner(item) {
    item.showSpinner = true;
    var el = $('#applying' + item.guid);
    $(el).prev('textarea').css('color', '#d3d3d3');
    $('img', el).show();
  }
  function stopAllProgressSpinners() {
    var standards = _.flatten(_.values($scope.stateStandards));
    for (var i in standards) {
      if (standards[i].showSpinner) {
        standards[i].showSpinner = false;
        var el = $('#applying' + standards[i].guid);
        $('img', el).hide();
        $('span', el).show().delay(800).fadeOut(1200);
        setTimeout(function () {
          $(el).prev('textarea').css('color', '');
        }, 2000);
      }
    }
  }
  /**
     * Clears the observation inputs for a single state standard
     * @param item
     */
  function clearSingleObservation(item) {
    item.progress = null;
    item.notes = '';
  }
  /**
     * Saves the observation inputs for a single state standard
     * @param item
     */
  function saveSingleObservation(item) {
    beginProgressSpinner(item);
    // locate original item this one was duped from & overwrite it
    var originalItem = getStandardFromEvent(item.guid);
    _.merge(originalItem, item);
    save(false);
  }
  /**
     * Saves the observation inputs for all state standards associated with the event
     */
  function saveAllObservations() {
    event.stateStandards = convertToSaveableModel();
    save(true);
  }
  /**
     * Saves the observation inputs for all state standards associated with the event, as a draft
     */
  function save(closeScreen) {
    // The previous event object is slightly different than what $events.saveEvent() is expecting so we're reusing the adapters
    //  create event to get the right type. Unfortunately, the timestamp from this event is a DateTime that gets cleaned up in
    //  another area of code that we're not reusing. We need to convert it back to a string in the correct format so saveEvent
    //  will handle it correctly.
    var saveEvent = $adapter.createEvent(event, $facility.getSync());
    saveEvent.timestamp = moment(saveEvent.timestamp).format('MMM D, YYYY h:mm:ss A');
    $events.saveEvent(saveEvent, false, closeScreen && $appstate.model.closeOnEventComplete);
    if (closeScreen) {
      $appstate.msg('Observation Saved', true);
      $appstate.nav.next();
    }
  }
  /**
     * Cancel the observation inputs for all state standards associated with the event
     */
  function cancel() {
    $appstate.dash.closeDialog();
  }
  /**
     * Duplicates standards list for editing (so that individual save buttons don't affect other sections),
     * groups them by subdomain + depth, and creates an array of headers on the first standard of each group
     * @param originalList
     * @returns {any}
     */
  function prepareCustomStandardsModel(originalList) {
    var stateStandards = JSON.parse(JSON.stringify(originalList));
    stateStandards = _.groupBy(stateStandards, function (standard) {
      return [
        standard.subdomain_guid,
        standard.depth_three_guid,
        standard.depth_four_guid
      ].join('_');
    });
    _.each(stateStandards, function (group) {
      var headers = [];
      var standard = group[0];
      if (standard.subdomain_text)
        headers.push(standard.subdomain_text);
      if (standard.depth_three_text)
        headers.push(standard.depth_three_text);
      if (standard.depth_four_text)
        headers.push(standard.depth_four_text);
      standard.headers = headers;
    });
    return stateStandards;
  }
  function convertToSaveableModel() {
    var arr = [];
    _.each($scope.stateStandards, function (list) {
      _.each(list, function (standard) {
        arr.push(standard);
      });
    });
    return arr;
  }
  function getStandardFromEvent(guid) {
    return _.find(event.stateStandards, function (i) {
      return i.guid === guid;
    });
  }
  function getStandardFromUiModel(guid) {
    return _.find(_.flatten(_.values($scope.stateStandards)), function (i) {
      return i.guid === guid;
    });
  }
}
'use strict';
angular.module('kidreports2App').controller('KidplannerprintCtrl', [
  '$scope',
  '$appstate',
  '$modalInstance',
  '$utils',
  'room',
  '$facility',
  function ($scope, $appstate, $modalInstance, $utils, room, $facility) {
    $scope.vals = { startdate: $utils.getDatePart(new Date()) };
    var cat = _.filter($facility.getSync().categories, function (cat) {
        return cat.id == room.category.id;
      })[0];
    $scope.items = [];
    $facility.getSync().eventTypes.forEach(function (et) {
      if (et.categories[0].id === cat.id && et.hidden != true && et.enabled != false) {
        $scope.items.push(et);
      }
    });
    $scope.ok = function () {
      var vals = {
          date: $scope.vals.startdate,
          items: _.filter($scope.items, function (item) {
            return item.selected;
          })
        };
      $modalInstance.close(vals);
    };
    $scope.cancel = function () {
      $modalInstance.dismiss('cancel');
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('BulkplannerCtrl', [
  '$scope',
  '$ajax',
  '$location',
  '$timeout',
  '$filter',
  '$facility',
  '$utils',
  '$modal',
  '$messageBox',
  function ($scope, $ajax, $location, $timeout, $filter, $facility, $utils, $modal, $messageBox) {
    $scope.loading = true;
    $scope.isCollapsed = false;
    $scope.planningMode = false;
    $scope.linkdata = {};
    $scope.isSys = function () {
      return $facility.isSys();
    };
    // $scope.$watch("planfilterdate", updatePriorDate);
    function updatePriorDate() {
      var d = new Date(Date.parse($scope.planfilterdate) + tsOffset);
      $scope.planpriordate = $filter('date')(d.getTime() - 24 * 60 * 60 * 1000, 'yyyy-MM-dd');
    }
    $scope.planfilterdate = $filter('date')(Date.now(), 'yyyy-MM-dd');
    var tsOffset = new Date().getTime() - Date.parse($scope.planfilterdate);
    updatePriorDate();
    $scope.editPlan = function (idx) {
      $scope.current = $scope.plans[idx];
    };
    $scope.addRow = function () {
      $scope.plans.push(angular.copy($scope.current));  //var cat = $scope.current.category;
                                                        //$scope.current = { startDate: date, endDate: date, details: [], category: cat };
    };
    $scope.addDetailsRow = function () {
      if (!$scope.current.ettValue2 && !$scope.current.ettValue)
        return;
      var val = $scope.current.ettValue2 != null ? $scope.current.ettValue2 : $scope.current.ettValue.value;
      $scope.current.ettValue2 = null;
      $scope.current.details.push({
        eventTypeTemplate: $scope.current.ett,
        value: val
      });
    };
    $scope.deleteDetailsRow = function (i) {
      $scope.current.details.splice(i, 1);
    };
    $scope.deleteRow = function (i) {
      $scope.plans.splice(i, 1);
    };
    $scope.applyPlan = function () {
      if ($scope.isEditing) {
        _delete($scope.editId, function () {
          applyPlan();
          $scope.isEditing = false;
        });
      } else {
        applyPlan();
      }
    };
    function _delete(id, cb) {
      $ajax.ajax({
        method: 'GET',
        url: '/bulkplans/deleteBulkPlan',
        params: { 'id': id }
      }, function (err, data) {
        if (data.status && !cb) {
          $utils.alert('Deleted');
        } else if (!data.status || err)
          $utils.alert('Error Deleting');
        $scope.savedPlans = _.filter($scope.savedPlans, function (f) {
          return f.id != id;
        });
        if (cb)
          cb();
      });
    }
    $scope.resetValues = function () {
      $scope.current.category = $scope.facility.categories[0];
      initCenters([]);
      $scope.linkdata = {};
      $scope.current.notes = '';
      $scope.current.autoApply = $scope.facility.enableAutoApplyPlans;
      //false;
      $scope.current.links = [];
      $scope.isEditing = false;
    };
    function _matchCategory(category) {
      return function (et) {
        return (et.categories && et.categories[0].id === category.id || et.category && et.category.id == category.id) && (et.name.indexOf('-Inactive') == -1 && !et.inactive && !et.deleted && !et.hidden && et.editable);
      };
    }
    ;
    $scope.facility = {};
    // 2014-09-26
    var date = $filter('date')(Date.now(), 'yyyy-MM-dd');
    //var date2 = $filter("date")(Date.now(), 'yyyy-MM-dd');
    //var dateStr = date.getYear() + "-" + (date.getMonth()+1) + "-" + date.getDate();
    $scope.current = {
      startDate: date,
      endDate: date,
      details: []
    };
    $scope.plans = [];
    var updateActivities = function (category) {
      if (!category)
        return;
      $scope.acts = _.filter($scope.facility.eventTypes, _matchCategory(category));
      if ($scope.editSetup)
        return;
      $scope.current.activity = $scope.acts[0];
    };
    $scope.$watch('current.category', function () {
      updateActivities($scope.current.category);
    });
    $scope.$watch('categoryFilter', function () {
      updateActivities($scope.categoryFilter);
    });
    $scope.$watch('current.activity', function () {
      $scope.current.ett = $scope.current.activity && $scope.current.activity.eventTypeTemplates && $scope.current.activity.eventTypeTemplates.length ? $scope.current.activity.eventTypeTemplates[0] : null;
      if ($scope.editSetup)
        return;
      $scope.current.details = [];
    });
    $scope.$watch('current.ett', function () {
      $scope.current.ettValue = $scope.current.ett && $scope.current.ett.templateValues && $scope.current.ett.templateValues.length ? $scope.current.ett.templateValues[0] : null;
    });
    var allCenters;
    var centerCategoryCount = 0;
    var centerIds = {};
    var categoryIds = {};
    var roomIds = {};
    function initCenters(centers) {
      var checkedCenters = {};
      angular.forEach(centers, function (c) {
        checkedCenters[c] = true;
      });
      $scope.availableCenters = {
        id: -1,
        'name': 'All Categories/Centers',
        'checked': checkedCenters[-1]
      };
      $scope.availableCenters['items'] = _.map(allCenters, function (c) {
        return {
          'id': c.id,
          'name': c.name,
          'checked': checkedCenters[c.id],
          'items': _.map(c.categories, function (cat) {
            return {
              'id': cat.id,
              'name': cat.name,
              'checked': checkedCenters[cat.id]
            };
          })
        };
      });
      $scope.current.selectedCenters = centers;
    }
    $scope.initialized = false;
    function init() {
      $scope.isRefreshing = true;
      _getPlans();
    }
    function _getPlans() {
      var params = {
          stopdate: Date.parse($scope.planfilterdate) + tsOffset,
          startdate: Date.parse($scope.planpriordate) + tsOffset
        };
      $ajax.ajax({
        method: 'GET',
        url: '/bulkplans/getBulkPlans',
        params: params
      }, function (err, data) {
        $scope.isRefreshing = false;
        $scope.savedPlans = data && data.length ? data[0].plans : [];
        angular.forEach($scope.savedPlans, initRooms);
        if (!$scope.initialized) {
          allCenters = data[0].centers;
          angular.forEach(allCenters, function (c) {
            centerIds[c.id] = c;
            centerCategoryCount++;
            angular.forEach(c.categories, function (cat) {
              categoryIds[cat.id] = cat;
              cat.facId = c.id;
              centerCategoryCount++;
              angular.forEach(cat.rooms, function (r) {
                roomIds[r.id] = r;
                r.facId = c.id;
                r.catId = cat.id;
              });
            });
          });
          $scope.initialized = true;
          initCenters([]);
        }
      });
      $scope.refreshPlans = function () {
        $scope.isRefreshing = true;
        _getPlans();
      };
      $scope.facility = $facility.getSync();
      $scope.cats = _.filter($scope.facility.categories, function (c) {
        return !c.archived && !c.deleted;
      });
      $scope.current.category = $scope.facility.categories[0];
      $scope.loading = false;
    }
    $scope.updatePlanAcrossCenters = function (plan) {
      $scope.isRefreshing = true;
      var params = { id: plan.id };
      $ajax.ajax({
        method: 'GET',
        url: '/bulkplans/updatePlanAcrossCenters',
        params: params
      }, function (err, data) {
        $scope.isRefreshing = false;
        if (err) {
          $utils.alert(JSON.stringify(err));
        } else {
          $utils.alert('Updated Plan!  Added:' + data.added);  //$timeout($scope.refreshPlans);
        }
      });
    };
    $scope.updateAllPlansAcrossCenters = function () {
      $scope.isRefreshing = true;
      var params = {
          stopdate: Date.parse($scope.planfilterdate) + tsOffset,
          startdate: Date.parse($scope.planpriordate) + tsOffset
        };
      $ajax.ajax({
        method: 'GET',
        url: '/bulkplans/updateAllPlansAcrossCenters',
        params: params
      }, function (err, data) {
        $scope.isRefreshing = false;
        if (err) {
          $utils.alert(JSON.stringify(err));
        } else {
          $utils.alert('Updated All Plans!  Added:' + data.added);
          $timeout($scope.refreshPlans);
        }
      });
    };
    $scope.getCenterName = function (cId) {
      var c = centerIds[cId];
      if (c && c.name)
        return c.name;
      else
        return 'Center Not Found';
    };
    $scope.getCenterText = function (cId, plan) {
      var c = centerIds[cId];
      var str = '';
      if (c && c.name)
        str += '<b><u>' + c.name + '</u></b><br/>';
      angular.forEach(plan.categoryIds, function (catId) {
        var cat = categoryIds[catId];
        if (cat && cat.facId == cId) {
          str += '&nbsp;&nbsp;&nbsp;&nbsp;<b>' + cat.name + '</b><br/>';
          angular.forEach(plan.roomIds, function (roomId) {
            var room = roomIds[roomId];
            if (room && room.catId == catId) {
              str += '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<i>' + room.name + '</i><br/>';
            }
          });
        }
      });
      return str;
    };
    $scope.getCategory = function (catId) {
      // return the name of the category if it is part ofthe given center id, else return empty string.
      var cat = categoryIds[catId];
      if (cat)
        return cat.name + ' (' + centerIds[cat.facId].name + ')';
      else
        return '';
    };
    $scope.savedPlans = [];
    function matchCenter(c) {
      return centerIds[c];
    }
    function matchCategory(c) {
      return categoryIds[c];
    }
    function matchRoom(c) {
      return roomIds[c];
    }
    function initRooms(plan) {
    }
    //produce date like 2016-02-01T00:00:00.000Z
    var parseDate = function (d) {
      return d.getFullYear() + '-' + $utils.zeroPad(d.getMonth() + 1) + '-' + $utils.zeroPad(d.getDate() + 1) + 'T' + $utils.zeroPad(d.getHours()) + ':' + $utils.zeroPad(d.getMinutes()) + ':' + $utils.zeroPad(d.getSeconds()) + '.000Z';
    };
    function applyPlan() {
      var d = angular.copy($scope.current);
      d.startDate = new Date(Date.parse(d.startDate));
      d.endDate = new Date(Date.parse(d.endDate));
      if (d.startDate.getTime() > d.endDate.getTime()) {
        $utils.alert('Start Date must be before End Date');
        return;
      }
      if (d.startDate.getTime() != d.endDate.getTime()) {
        var days = (d.endDate.getTime() - d.startDate.getTime()) / (24 * 60 * 60 * 1000);
        $messageBox.open('Multi-Day Plan Warning', 'This Plan spans ' + (Math.round(days) + 1) + ' days.  Continue?', {
          oktext: 'Yes',
          canceltext: 'No'
        }).result.then(function (selectedItem) {
          _applyPlan(d);
        });
      } else {
        _applyPlan(d);
      }
    }
    function _applyPlan(d) {
      d.centers = _.map(_.filter(d.selectedCenters, function (r) {
        return matchCenter(r);
      }), function (c) {
        return { id: c };
      });
      d.categories = _.map(_.filter(d.selectedCenters, function (r) {
        return matchCategory(r);
      }), function (c) {
        return { id: c };
      });
      //      d.rooms = _.map(_.filter(d.rooms, function(r) {return r.checked}), function(c) {
      //        return { id: c.id };
      //      });
      if (d.categories.length === 0) {
        $utils.alert('Must select at least one Category!');
        return;
      }
      $scope.loading = true;
      d.startDate = parseDate(d.startDate);
      d.endDate = parseDate(d.endDate);
      $ajax.ajax({
        method: 'post',
        url: '/bulkplans/saveBulkPlan',
        data: d
      }, function (err, data) {
        $scope.loading = false;
        if (data) {
          // ok
          $scope.savedPlans.push(data);
          $utils.alert('Plan Saved!');
          $scope.planningMode = false;
          initRooms(d);
        }
      });
    }
    $scope.hideHiddenPlans = true;
    $scope.isHiddenPlan = function (p) {
      return !$scope.hideHiddenPlans || !(p.isHidden && p.isHidden === true);
    };
    $scope.categoryFilter = null;
    $scope.activityFilter = null;
    $scope.categoryAndActivity = function (p) {
      var matchCat = true, matchAct = true;
      if ($scope.categoryFilter != null)
        matchCat = p.category.id === $scope.categoryFilter.id;
      if ($scope.activityFilter != null)
        matchAct = p.activity.id === $scope.activityFilter.id;
      return matchCat && matchAct;
    };
    $scope.hide = function (plan, b) {
      plan.isHidden = b;
      $ajax.ajax({
        method: 'GET',
        url: '/bulkplans/updateBulkPlan',
        params: {
          'id': plan.id,
          'hidden': b
        }
      }, function (err, data) {
        if (data.status) {
        } else
          $utils.alert('Error Hiding Plan');
      });
    };
    var dayMillis = 24 * 60 * 60 * 1000;
    $scope.copy = function (plan) {
      //alert("copy plan");
      var sDate = new Date();
      var eDate = new Date(sDate.getTime() + dayMillis);
      var modalInstance = $modal.open({
          templateUrl: 'views/popups/dateRangePopup.html',
          controller: 'DaterangeCtrl',
          resolve: {
            startdate: function () {
              return $utils.getDatePart(sDate);
            },
            enddate: function () {
              return $utils.getDatePart(eDate);
            }
          }
        });
      modalInstance.result.then(function (val) {
        //console.log(val.startdate);
        //console.log(val.enddate);
        var d1 = $utils.createDateFromDTStrings(val.startdate, '00:00:00');
        var d2 = $utils.createDateFromDTStrings(val.enddate, '00:00:00');
        d1.setHours(12);
        d1.setMinutes(0);
        d1.setSeconds(0, 0);
        d2.setHours(12);
        d2.setMinutes(0);
        d2.setSeconds(0, 0);
        d1.setTime(d1.getTime() - dayMillis);
        d2.setTime(d2.getTime() - dayMillis);
        //alert("Copy To:"+d1+", "+d2);
        $scope.loading = true;
        plan.startDate = d1;
        plan.endDate = d2;
        $scope.current = angular.copy(plan);
        delete $scope.current.id;
        delete $scope.current.createDate;
        $scope.current.category = $facility.findCategory(plan.category.id);
        $scope.current.activity = $facility.findEventType(plan.activity.id);
        $scope.current.notes = plan.notes;
        $scope.current.autoApply = plan.autoApply;
        $scope.current.links = _.map(plan.links, function (d) {
          return {
            label: d.label,
            url: d.url
          };
        });
        $scope.current.details = _.map(plan.details, function (d) {
          return {
            value: d.value,
            eventTypeTemplate: _.filter($scope.current.activity.eventTypeTemplates, function (ett) {
              return ett.id == d.eventTypeTemplate.id;
            })[0]
          };
        });
        $scope.current.selectedCenters = [];
        angular.forEach(plan.centerIds, function (c) {
          $scope.current.selectedCenters.push(c);
        });
        angular.forEach(plan.categoryIds, function (c) {
          $scope.current.selectedCenters.push(c);
        });
        applyPlan();
      });
    };
    $scope.edit = function (plan) {
      $scope.isEditing = true;
      $scope.editId = plan.id;
      $scope.planningMode = true;
      // set these in the timeout to avoid angular's digest
      //setTimeout(function() {
      $scope.editSetup = true;
      $scope.current.category = $facility.findCategory(plan.category.id);
      $scope.current.activity = $facility.findEventType(plan.activity.id);
      $scope.current.notes = plan.notes;
      $scope.current.autoApply = plan.autoApply;
      $scope.current.links = plan.links;
      var date = $filter('date')(Date.now(), 'yyyy-MM-dd');
      $scope.current.startDate = $filter('date')(Date.parse(plan.startDate), 'yyyy-MM-dd');
      $scope.current.endDate = $filter('date')(Date.parse(plan.endDate), 'yyyy-MM-dd');
      $scope.current.details = _.map(plan.details, function (d) {
        return {
          value: d.value,
          eventTypeTemplate: _.filter($scope.current.activity.eventTypeTemplates, function (ett) {
            return ett.id == d.eventTypeTemplate.id;
          })[0]
        };
      });
      $scope.current.selectedCenters = [];
      angular.forEach(plan.centerIds, function (c) {
        $scope.current.selectedCenters.push(c);
      });
      angular.forEach(plan.categoryIds, function (c) {
        $scope.current.selectedCenters.push(c);
      });
      if ($scope.current.selectedCenters.length === centerCategoryCount)
        $scope.current.selectedCenters.push(-1);
      initCenters($scope.current.selectedCenters);
      $timeout(function () {
        $scope.editSetup = false;  //        $scope.current.rooms = _.map($scope.current.rooms, function(c) {
                                   //          c.checked = _.filter(plan.rooms, function(r) {return r.id==c.id;}).length > 0;
                                   //          return c;
                                   //        });
      }, 250);  //  $scope.$apply();
                //});
    };
    $scope.delete = function (plan) {
      $messageBox.open('Delete Bulk Plan', 'This will delete the bulk plan and ALL associated Lesson Plans.  Are you sure?', {
        oktext: 'Yes',
        canceltext: 'Cancel'
      }).result.then(function (selectedItem) {
        // do delete
        _delete(plan.id);
      }, function () {
      });
    };
    init();
  }
]);
'use strict';
angular.module('kidreports2App').controller('OverviewCtrl', [
  '$scope',
  '$appstate',
  '$krConfig',
  '$modal',
  '$utils',
  '$backend',
  '$state',
  '$adapter',
  '$localdb',
  '$messages',
  '$q',
  '$messageBox',
  '$events',
  '$facility',
  '$attendance',
  '$rootScope',
  '$timeout',
  '$translate',
  'translations',
  '$krCache',
  function ($scope, $appstate, $krConfig, $modal, $utils, $backend, $state, $adapter, $localdb, $messages, $q, $messageBox, $events, $facility, $attendance, $rootScope, $timeout, $translate, translations, $krCache) {
    var isTeacher = false;
    var title = $translate.instant('Overview');
    $scope.title = title;
    $appstate.model.title = title;
    // Need a copy since we're modifying the objects
    //$scope.items = angular.copy($facility.getSync().observed);
    var d = new Date();
    $scope.value = $utils.getTimePart(d);
    $scope.goToDash = function (item, idx) {
      $appstate.goHome();
      $timeout(function () {
        $appstate.pickKid(item, idx);
      });
    };
    var roomId = $appstate.model.selectedRoom.id;
    function setupItems() {
      $scope.items = angular.copy($appstate.getVisibleChildren());
      _.each($scope.items, function (item) {
        item.clean = true;
        item.selected = false;
      });
    }
    var oldRefresh = $appstate.refresh;
    $appstate.refresh = function (b) {
      _.each($scope.items, function (item) {
        item.current = {};
      });
      $appstate.model.inVal = 'In';
      $attendance.refresh($scope.items, d, isTeacher, b).then(updateItemState);
    };
    $scope.$on('$destroy', function () {
      $appstate.refresh = oldRefresh;
    });
    //$scope.hasMessage = {};
    function setupMessages() {
      var d = $q.defer();
      $scope.hasMessage = {};
      var kidHash = {};
      _.each($scope.items, function (k) {
        kidHash[k.id] = true;
      });
      $messages.get().then(function (messages) {
        _.each(messages, function (m) {
          if (m.status === 0) {
            var kids = $facility.findKidsByParent($facility.findUser(m.fromUserID));
            _.each(kids, function (k) {
              if (kidHash[k.id]) {
                $scope.hasMessage[k.id] = true;
              }
            });
          }
        });
        d.resolve();
      });
      return d.promise;
    }
    function updateItemState() {
      setupItems();
      setupMessages().then(buildAllIcons);
    }
    function buildAllIcons() {
      _.each($scope.items, buildIcons);
    }
    var usMatchList = [
        'Please Bring',
        'Athletics'
      ];
    var ptMatchList = [
        'Aviso para a escola',
        'Sa\xfade',
        'Drop-Off Information'
      ];
    function buildIcons(item) {
      var isPT = translations.getCurrent().value == 'pt-br';
      var matchList = $facility.getSync().overviewItems || (isPT ? ptMatchList : usMatchList);
      $events.getEvents([item], d, function (err, data) {
        item.icons = [];
        if ($scope.hasMessage[item.id]) {
          item.icons.push('img/icons/send-message.png');
        }
        var found = {};
        _.each(data.events, function (evt) {
          //console.log(data.events);
          _.each(matchList, function (match) {
            if (evt.eventType.name.indexOf(match) > -1 && !found[match]) {
              found[match] = true;
              var et = $facility.findEventType(evt.eventType.id);
              var icon = $utils.createPhotoUrl($krConfig.s3Base, et.icon, $utils.getDefault(et));
              item.icons.push(icon);
            }
          });
        });
        // now check for messages and drafts
        $krCache.getCache('drafts', function (drafts) {
          var foundDraft = false;
          drafts.forEach(function (d) {
            d.observed.forEach(function (c) {
              if (c.id == item.id && !foundDraft) {
                foundDraft = true;
                item.icons.push('img/icons/edit.png');
              }
            });
          });
        });
      });
    }
    function hasDraft(item) {
    }
    function hasMessageFromParent(item) {
    }
    setupItems();
    $appstate.refresh(isTeacher);
    $appstate.model.dirty = false;
    function _close() {
      $appstate.goHome();
    }
    $appstate.canClose = _close;
    $scope.close = _close;
    $appstate.setTime = $scope.setTime;
    $appstate.toggleIn = $scope.toggleIn;
    $appstate.getSelectedText = $scope.getSelectedText;
    $appstate.close = $scope.close;
  }
]);
'use strict';
angular.module('kidreports2App').controller('OverviewdashCtrl', [
  '$scope',
  '$appstate',
  function ($scope, $appstate) {
    $scope.close = function () {
      $appstate.close();
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('OverviewheaderCtrl', [
  '$scope',
  '$state',
  '$facility',
  '$appstate',
  function ($scope, $state, $facility, $appstate) {
    if ($state.is('home.action.teacherSignIn') || $state.is('home.dash.teacherSignIn')) {
      $scope.canSetTime = !$facility.getSync().disableTeachersFromSettingTime;
    } else {
      $scope.canSetTime = true;
    }
    $scope.toggleIn = function (item) {
      $appstate.toggleIn(item);
    };
    $scope.setTime = function (item) {
      $appstate.setTime();
    };
    $scope.getSelectedText = function () {
      return $appstate.getSelectedText();
    };
    $scope.refreshAttendance = function (b) {
      $appstate.refresh(b);
    };
  }
]);
'use strict';
angular.module('kidreports2App').controller('ParentprofileCtrl', [
  '$scope',
  '$drawerService',
  '$timeout',
  '$facility',
  '$nav',
  '$native',
  '$utils',
  '$krConfig',
  '$image',
  '$profile',
  '$appstate',
  function ($scope, $drawerService, $timeout, $facility, $nav, $native, $utils, $krConfig, $image, $profile, $appstate) {
    $scope.native = $native.isNative();
    var facility = $facility.getSync();
    $scope.preventWriteActivity = facility.preventWriteActivity;
    var isTeacher = facility.currentUser.role === 'TEACHER_ROLE';
    var parents = _.flatMap(facility.children, 'parents');
    $scope.parent = _.find(parents, function (p) {
      return p.id === parseInt($drawerService.params.parentId);
    });
    $scope.children = _.filter(facility.children, function (child) {
      return _.find(child.parents, [
        'id',
        $scope.parent.id
      ]);
    });
    $scope.showChildProfileDrawer = function (child) {
      $drawerService.openChildProfile(child.id);
    };
    $image.getImage($scope.parent.id, $utils.createPhotoUrl($krConfig.s3Base, $scope.parent.photoPath)).then(function (url) {
      $scope.parent.photoUrl = url;
    });
    $scope.closeDrawer = $drawerService.closeDrawer;
    $scope.setProfilePhoto = $profile.setProfilePhoto;
    $scope.shared = $appstate;
    $scope.showContactInfo = $facility.isAdmin() || isTeacher && facility.showParentContactToTeachers || facility.currentUser.id === $scope.parent.id;
  }
]);
'use strict';
angular.module('kidreports2App').factory('$uiAlert', uiAlertService);
uiAlertService.$inject = ['$rootScope'];
/**
 * @function uiAlertService()
 * @author Joshua Belden <jcbelden@procaresoftware.com>
 * @returns {{
 *  ALERT_TYPE_SUCCESS: number,
 *  ALERT_TYPE_WARNING: number,
 *  ALERT_TYPE_ERROR: number,
 *  pushSuccessAlert: (function(number, string, string): *),
 *  pushWarningAlert: (function(number, string, string): *),
 *  pushErrorAlert: (function(number, string, string): *),
 *  pushSuccessAlert: (function(number, string, string): *),
 *  popAllAlerts: (function(): *),
 *  popAlertsFor: (function(string): *),
 *  clearAllAlerts: (function(): *)
 * }}
 */
function uiAlertService($rootScope) {
  var ALERT_TYPE_SUCCESS = 0;
  var ALERT_TYPE_WARNING = 1;
  var ALERT_TYPE_ERROR = 2;
  var allAlerts = [];
  return {
    ALERT_TYPE_SUCCESS: ALERT_TYPE_SUCCESS,
    ALERT_TYPE_WARNING: ALERT_TYPE_WARNING,
    ALERT_TYPE_ERROR: ALERT_TYPE_ERROR,
    pushSuccessAlert: pushSuccessAlert,
    pushWarningAlert: pushWarningAlert,
    pushErrorAlert: pushErrorAlert,
    popAllAlerts: popAllAlerts,
    popAlertsFor: popAlertsFor,
    clearAllAlerts: clearAllAlerts
  };
  /**
     * @function pushAlert()
     * @description Creates a new alert using the specified message.
     * @param {number} type The type of alert to create.
     * @param {string} message The message of the alert.
     * @param {string} target The target consumer this alert is intended for.
     * @returns {void}
     */
  function pushAlert(type, message, target) {
    allAlerts.push({
      type: type,
      message: message,
      target: target
    });
    $rootScope.$broadcast('uiAlertNew');
  }
  /**
     * @function pushSuccessAlert()
     * @description Creates a new success alert using the specified message.
     * @param {string} message The message of the alert.
     * @param {string} target The target consumer this alert is intended for.
     * @returns {void}
     */
  function pushSuccessAlert(message, target) {
    pushAlert(ALERT_TYPE_SUCCESS, message, target);
  }
  /**
     * @function pushWarningAlert()
     * @description Creates a new warning alert using the specified message.
     * @param {string} message The message of the alert.
     * @param {string} target The target consumer this alert is intended for.
     * @returns {void}
     */
  function pushWarningAlert(message, target) {
    pushAlert(ALERT_TYPE_WARNING, message, target);
  }
  /**
     * @function pushErrorAlert()
     * @description Creates a new error alert using the specified message.
     * @param {string} message The message of the alert.
     * @param {string} target The target consumer this alert is intended for.
     * @returns {void}
     */
  function pushErrorAlert(message, target) {
    pushAlert(ALERT_TYPE_ERROR, message, target);
  }
  /**
     * @function popAllAlerts()
     * @description Retrieves all alerts that are currently being tracked.
     * @returns {*}
     */
  function popAllAlerts() {
    if (!allAlerts.length) {
      return [];
    }
    // Return all alerts that we have and clear the main collection.
    var matchedAlerts = allAlerts;
    clearAllAlerts();
    return matchedAlerts;
  }
  /**
     * @function popAlertsFor()
     * @description Retrieves all alerts that are currently being tracked for the specified target.
     * @param {string} target The target to filter alerts by. This is a case-sensitive match.
     * @returns {*}
     */
  function popAlertsFor(target) {
    if (!allAlerts.length) {
      return [];
    }
    // Find alerts for the specified target.
    var matchedAlerts = window._.filter(allAlerts, function (alert) {
        return alert.target === target;
      });
    // Remove the matched alerts from the main collection.
    allAlerts = window._.difference(allAlerts, matchedAlerts);
    return matchedAlerts;
  }
  /**
     * @function clearAllAlerts()
     * @description Dumps all alerts being tracked regardless of target.
     * @returns {void}
     */
  function clearAllAlerts() {
    allAlerts = [];
  }
}
'use strict';
angular.module('kidreports2App').controller('TeacherprofileCtrl', [
  '$drawerService',
  '$backend',
  '$scope',
  '$timeout',
  '$facility',
  '$nav',
  '$state',
  '$ajax',
  '$appstate',
  '$utils',
  '$msg',
  '$translate',
  '$krConfig',
  '$image',
  '$profile',
  '$native',
  function ($drawerService, $backend, $scope, $timeout, $facility, $nav, $state, $ajax, $appstate, $utils, $msg, $translate, $krConfig, $image, $profile, $native) {
    $scope.native = $native.isNative();
    var user = $backend.getCurrentUser();
    $image.getImage(user.id, $utils.createPhotoUrl($krConfig.s3Base, user.photoPath)).then(function (url) {
      user.photoUrl = url;
    });
    $scope.user = user;
    $scope.closeDrawer = $drawerService.closeDrawer;
    $scope.setProfilePhoto = $profile.setProfilePhoto;
  }
]);
'use strict';
angular.module('kidreports2App').factory('$appService', appService);
appService.$inject = ['$facility'];
/**
 * @function appService
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @param $facility
 */
function appService($facility) {
  var _ = window._;
  return {
    getCurrentUser: getCurrentUser,
    canSendItemsToPortfolio: canSendItemsToPortfolio,
    getUserRole: getUserRole
  };
  function canSendItemsToPortfolio() {
    // First, check Facility attributes:
    if (_.get($facility, 'hideSendToPortfolio') || $facility.isLite()) {
      return false;
    }
    // Next, check user's individual permission:
    var currentUser = getCurrentUser();
    if (getUserRole() === 'TEACHER_ROLE') {
      return _.get(currentUser, 'canPublishPortfolioItems');
    }
    // Otherwise, just return false:
    return false;
  }
  /**
   * Retrieves current user from Facility service
   * @returns {*}
   */
  function getCurrentUser() {
    return _.get($facility.getSync(), 'currentUser');
  }
  /**
   * Retrieves role from Facility service
   * @returns {*}
   */
  function getUserRole() {
    return _.get($facility.getSync(), 'currentUser.role');
  }
}
'use strict';
angular.module('kidreports2App').factory('$attendanceService', attendanceService);
attendanceService.$inject = [
  '$facility',
  '$appstate',
  '$backend',
  '$eventAdapter',
  '$utils',
  '$attendance',
  '$events',
  '$q',
  'moment',
  '$featureFlags'
];
/**
 * @function attendanceService
 * @author Erik Johnson <eajohnson@procaresoftware.com> & Joshua Belden <jcbelden@procaresoftware.com>
 * @param $facility
 * @param $appstate
 * @param $backend
 * @param $eventAdapter
 * @param $utils
 * @param $attendance
 * @param $events
 * @param $q
 * @param moment
 * @param $featureFlags
 * @returns {{getChildren: (function(): *), getTeachers: (function(): {children: []})}}
 */
function attendanceService($facility, $appstate, $backend, $eventAdapter, $utils, $attendance, $events, $q, moment, $featureFlags) {
  var _ = window._;
  return {
    completeEvent: completeEvent,
    getAllChildren: getAllChildren,
    getChildren: getChildren,
    getSelectedRoom: getSelectedRoom,
    getTeachersAsync: getTeachersAsync,
    refreshAttendance: refreshAttendance
  };
  /**
   * Completes check-in of list of people (Teachers, Children) via service
   * @param {Array<object>} people
   * @param {boolean} isTeacher
   * @param {Function} callback
   */
  function completeEvent(people, isTeacher, roomId, callback) {
    var attendanceUseSeconds = $featureFlags.isFeatureEnabled('attendance-use-seconds');
    var datetime = new Date();
    var selectedCategory = _.get($appstate, 'model.selectedCategory');
    var attendanceRoomId = roomId || _.get($appstate, 'model.selectedRoom.id');
    var forId = $facility.getSync().observer && $facility.getSync().observer.id;
    $backend.saveAttendance(setProperties(people, attendanceUseSeconds), selectedCategory, datetime, function (error, response) {
      if (error) {
        callback(error, null);
      } else {
        if (isTeacher) {
          doTeacherAttendance(response.data);
        }
        updateApplicationState(people, isTeacher);
        callback(null, response);
      }
    }, isTeacher, attendanceRoomId, forId);
  }
  //only check in and check out. so only one event per teacher
  function doTeacherAttendance(data) {
    if (data != null) {
      _.forEach(data, function (event) {
        var temp = [];
        _.forEach($facility.getSync().users, function (user) {
          if (event.subject.id === user.id) {
            _.forEach(user.events, function (userEvent) {
              temp.push(userEvent);
            });
            temp.push(event);
            user.events = temp;
            $facility.save();
          }
        });
      });
    }
  }
  /**
   * Sets properties to model used by backend service
   * @param people
   */
  function setProperties(people, useSeconds) {
    useSeconds = useSeconds || false;
    return _.each(people, function (person) {
      person.isIn = !!person.pendingIn;
      person.clean = false;
      person.timeVal = $utils.getTimePart(new Date());
      person.time = $utils.getTimeString(person.timeVal, useSeconds);
    });
  }
  /**
   * Retrieves list of all children in a facility from facility service
   * @returns {*}
   */
  function getAllChildren() {
    var children = _.sortBy(_.get($facility.getSync(), 'children'), 'firstName', 'lastName');
    return angular.copy(children);
  }
  /**
   * Retrieves list of children from facility service
   * @returns {*}
   */
  function getChildren() {
    var children = _.get($facility.getSync(), 'observed');
    return angular.copy(children);
  }
  /**
   * Retrieves the selected room from the appstate service
   * @returns {*}
   */
  function getSelectedRoom() {
    return $appstate.model.selectedRoom;
  }
  /**
   * Retrieves list of teachers from the appstate service
   * @returns {*}
   */
  function getTeachersAsync() {
    var q = $q.defer();
    var attendanceUseSeconds = $featureFlags.isFeatureEnabled('attendance-use-seconds');
    // Filter out teachers that are not allowed to sign-in.
    var teachers = _.filter($appstate.model.selectedRoom.teachers, function (teacher) {
        return !_.get(teacher, 'removeFromSignIn');
      });
    // Review teacher events to calculate their current attendance status. Find events for
    //  the entire collection of teachers as individual lookups will be expensive.
    var date = new Date();
    var force = true;
    var days = 1;
    var roomId = $appstate.model.selectedRoom.id;
    var filter = null;
    $events.getUserEvents(teachers, date, function (err, result) {
      if (err) {
        q.reject(err);
        return;
      }
      _.forEach(teachers, function (teacher) {
        var teacherEvents = _.filter(result.events, function (event) {
            return teacher.id === event.subject.id;
          });
        applyAttendanceValues(teacher, teacherEvents, roomId, attendanceUseSeconds);
      });
      q.resolve(teachers);
    }, force, days, roomId, filter);
    return q.promise;
  }
  /**
   * Applies attendance values
   * @param {object} teacher
   * @param {Array<object>} events
   * @returns {boolean}
   */
  function applyAttendanceValues(teacher, events, currentRoomId, showSeconds) {
    showSeconds = showSeconds || false;
    if (!teacher || !events || events.length < 1) {
      return false;
    }
    var lastTime = null, lastEvent = null;
    var attendanceEvents = _.filter(events, function (event) {
        return event.eventType && event.eventType.name === 'Take Attendance';
      });
    _.forEach(attendanceEvents, function (event) {
      if (lastTime === null || new Date(lastTime) < new Date(event.timeOccurred)) {
        lastEvent = event;
        lastTime = event.timeOccurred;
      }
    });
    if (!lastEvent || !lastEvent.eventValues || lastEvent.eventValues.length < 1) {
      teacher.isIn = false;
      return false;
    }
    if (lastEvent.eventValues[0].value === 'In') {
      if (lastEvent.notes.indexOf('(' + currentRoomId + ')') > -1) {
        teacher.isIn = true;
        teacher.time = showSeconds ? moment(lastEvent.timeOccurred).format('hh:mm A') : moment(lastEvent.timeOccurred).format('hh:mm:ss A');
      } else {
        teacher.isIn = false;
        teacher.isInOtherRoom = true;
        teacher.isInOtherRoomName = lastEvent.notes.replace(/\(.*\)/, '').trim();
      }
    } else {
      teacher.isIn = false;
      teacher.isInOtherRoom = false;
      teacher.isInOtherRoomName = '';
    }
  }
  /**
   * Refreshes attendance
   */
  function refreshAttendance(items, isTeacher) {
    return $attendance.refresh(items, new Date(), isTeacher, true, false, false);
  }
  /**
   * Updates application state on check-in/check-out
   * @param {Array<object>} people
   * @param {boolean} isTeacher
   */
  function updateApplicationState(people, isTeacher) {
    if (!isTeacher) {
      $appstate.model.currentRoomAttendance = _.filter(people, function (i) {
        return i.isIn === true;
      }).length;
    }
    $eventAdapter.updateAttendanceState(people, isTeacher, true);
    $facility.save();
  }
}
'use strict';
angular.module('kidreports2App').factory('$roomService', roomService);
roomService.$inject = [
  '$appstate',
  '$backend',
  '$facility'
];
/**
 * @function roomService
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @param $appstate
 * @param $backend
 * @param $facility
 * @returns {object}
 */
function roomService($appstate, $backend, $facility) {
  return {
    getRooms: getRooms,
    getAllRooms: getAllRooms,
    getConsolidationRoomId: getConsolidationRoomId,
    moveRoom: moveRoom
  };
  /**
   * Retrieves available rooms
   */
  function getRooms(ignoreCurrentlySelected) {
    return _.filter($facility.getSync().rooms, function (item) {
      return item.name !== 'All Children' && item.id !== $appstate.model.selectedRoom.id;
    });
  }
  /**
   * Retrieves available rooms
   */
  function getAllRooms() {
    return _.filter($facility.getSync().rooms, function (item) {
      return item.name !== 'All Children';
    });
  }
  /**
   * Retrieves the configured consolidation room's id
   * @returns number
   */
  function getConsolidationRoomId() {
    return $facility.getSync().consolidatingRoomId;
  }
  /**
   * Performs room movement across back-end services for a list of people (children, teachers)
   * @param {number} fromRoomId
   * @param {number} toRoomId
   * @param {Array<object>} people
   * @param {Function} callback
   */
  function moveRoom(fromRoomId, toRoomId, people, callback) {
    var peopleIds = _.map(people, function (peep) {
        return peep.id;
      });
    $backend.moveRoom(toRoomId, peopleIds, function (error, results) {
      if (!error) {
        _.each(people, function (person) {
          // Move room for each person - in facility:
          var fromRoom = $facility.findRoom(fromRoomId);
          var toRoom = $facility.findRoom(toRoomId);
          $facility.moveRoom(person, fromRoom, toRoom, true);
          person.cachedRoom = true;
        });
        // Save facility:
        $facility.save().then(function () {
          console.log('Facility saved...');
          $appstate.setCurrentRoom($appstate.model.selectedRoom);
          callback(null, results);
        });
      } else {
        console.log(error);
        callback(error, null);
      }
    });
  }
}
'use strict';
angular.module('kidreports2App').factory('$memoriesService', memoriesService);
memoriesService.$inject = [
  '$facility',
  '$memories',
  'moment'
];
/**
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @param $facility
 * @param $memories
 * @param moment
 * @returns {{getChildren: (function(): []), getMedia: (function(string): *), sortMediaToChildren: (function(Array<Object>, Array<Object>): Array<Object>), shareMedia: shareMedia}}
 */
function memoriesService($facility, $memories, moment) {
  var _ = window._;
  return {
    getChildren: getChildren,
    getMedia: getMedia,
    isNative: isNative,
    shareMedia: shareMedia,
    sortMediaToChildren: sortMediaToChildren
  };
  /**
   *
   * @param {string} date
   * @returns {Promise<Array<object>>}
   */
  function getMedia(date) {
    var start = moment(date).startOf('month').toDate();
    var end = moment(start).add(1, 'month').toDate();
    return $memories.getPhotos(start, end, 1, 10000, 'ALL');
  }
  /**
   * Returns appropriate sharing options for Forge, based on media type
   * @param media
   * @returns {{}}
   */
  function getShareItem(media) {
    var item = {};
    if (media.type === 'VIDEO') {
      item.url = media.src;
    } else {
      item.image = media.src;
    }
    return item;
  }
  /**
   * Retrieves children for a given parent from Facility service
   * @returns {*}
   */
  function getChildren() {
    var children = [];
    var currentUser = $facility.getSync().currentUser;
    var facilityChildren = $facility.getSync().children;
    _.each(facilityChildren, function (child) {
      var parents = _.get(child, 'parents');
      if (_.find(parents, { id: currentUser.id })) {
        children.push(child);
      }
    });
    return children;
  }
  /**
   * Checks if user has Forge (is native)
   * @returns {boolean}
   */
  function isNative() {
    return typeof window.forge !== 'undefined';
  }
  /**
   * Shares media - native or web
   * @param {object} media
   * @param {Function} callback
   * @returns {*}
   */
  function shareMedia(media, callback) {
    var type = _.get(media, 'type');
    var item = getShareItem(media);
    if (isNative() && type !== 'VIDEO') {
      return window.forge.share.item(item, function (success) {
        callback(null, success);
      }, function (error) {
        callback(error);
      });
    } else if (isNative() && type === 'VIDEO') {
      return window.forge.tabs.openWithOptions({
        url: media.src,
        navigationToolbar: true
      }, function (success) {
        callback(null, success);
      }, function (error) {
        callback(error);
      });
    } else {
      window.open(media.src);
      callback(null, null);
    }
  }
  /**
   * Sorts media to given child for a list of children
   * @param {Array<object>} children
   * @param {Array<object>} media
   * @returns {*}
   */
  function sortMediaToChildren(children, media) {
    _.each(children, function (child) {
      child.media = _.filter(media, function (item) {
        // Parse Facility ID for the media and ensure it matches:
        return item.name === child.firstName && checkFacilityIdOnMedia(item.src, _.get(child, 'facilityId'));
      });
      child.media = _.reverse(child.media);
    });
    return children;
  }
  /**
   * Checks to see if Facility ID is found in href of media
   * @param {string} href
   * @param {number} facilityId
   * @returns {boolean}
   */
  function checkFacilityIdOnMedia(href, facilityId) {
    var idx = href.indexOf('/' + facilityId + '/');
    return idx > -1;
  }
}
'use strict';
angular.module('kidreports2App').factory('$activitiesService', activitiesService);
activitiesService.$inject = [
  '$events',
  '$facility',
  '$utils',
  '$messageBox',
  '$appstate',
  '$appService',
  'popover',
  '$filters'
];
/**
 * Service for Activities
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @param $events
 * @param $facility
 * @param $utils
 * @param $messageBox
 * @param $appstate
 * @param $appService
 * @param popover
 */
function activitiesService($events, $facility, $utils, $messageBox, $appstate, $appService, popover, $filters) {
  var _ = window._;
  return {
    canDelete: canDelete,
    canEdit: canEdit,
    deleteActivity: deleteActivity,
    editActivity: editActivity,
    getActivities: getActivities,
    getCategoryList: getCategoryList,
    getChildren: getChildren,
    getEventTypeName: getEventTypeName,
    getGroupIcon: getGroupIcon,
    hasActivities: hasActivities,
    sortActivitiesToChildren: sortActivitiesToChildren,
    viewActivity: viewActivity
  };
  /**
   * Checks if user can delete given activity
   * @param {object} activity
   * @returns {boolean|*}
   */
  function canDelete(activity) {
    return !activity.draftId && canEdit(activity);
  }
  /**
   * Checks if user can
   * @param activity
   * @returns {*}
   */
  function canEdit(activity) {
    return $appstate.canEdit(activity);
  }
  /**
   * Prompts user to confirm deletion and deletes event
   * @param {object} activity
   */
  function deleteActivity(activity) {
    if (canDelete(activity)) {
      $messageBox.open('Delete Item', 'This will permanently delete the event. This cannot be undone. Are you sure you wish to continue?', {
        oktext: 'Yes',
        canceltext: 'No'
      }).result.then(function () {
        $events.deleteEvent(activity, function (error) {
          if (error) {
            console.log(error);
            $messageBox.open('Error', 'An error occurred, the event was not deleted!', { oktext: 'Ok' });  // TODO: Add callback
          } else {
            $messageBox.open('Item Deleted', 'Event Deleted Successfully!', { oktext: 'Ok' });
            $appstate.refreshEvents($appstate.model.observed);  // TODO: Add callback
          }
        });
      }, function () {
        $appstate.msg('Delete Cancelled');
      });
    }
  }
  /**
   * Opens edit modal if applicable to role
   * @param {object} activity
   */
  function editActivity(activity) {
    var isParent = $appService.getUserRole() === 'PARENT_ROLE';
    $appstate.model.isDashActivityEdit = true;
    $appstate.viewEvent(activity, isParent);
  }
  /**
   * Format event value label for either time or action
   * @param eventValue
   * @returns {*}
   */
  function formatEventValue(eventValue) {
    if (eventValue.eventTypeTemplate && eventValue.eventTypeTemplate.templateType === 'TIME_INPUT' || eventValue.ett && eventValue.ett.templateType === 'TIME_INPUT') {
      return $utils.getTimeString(eventValue.value);
    } else {
      return eventValue.value;
    }
  }
  /**
   * Formats and adds 'formattedEventValues' property to a given activity
   * @param {Array<object>} eventValues
   * @returns {object}
   */
  function formatEventValues(eventValues) {
    var idx = 0;
    var found = {};
    var formattedEventValues = [];
    _.forEach(eventValues, function (eventValue) {
      // No template, skip:
      if (!eventValue.eventTypeTemplate) {
        return;
      }
      // The name is not previous found, create new entry:
      if (typeof found[eventValue.eventTypeTemplate.name] === 'undefined') {
        found[eventValue.eventTypeTemplate.name] = idx;
        formattedEventValues.push({
          name: eventValue.eventTypeTemplate.name,
          value: ''
        });
        idx++;
      }
      var index = found[eventValue.eventTypeTemplate.name];
      var val = formattedEventValues[index].value;
      if (val.length > 0) {
        val += ', ';
      }
      val += formatEventValue(eventValue);
      formattedEventValues[index].value = val;
      formattedEventValues[index].formattedValue = val;
    });
    return formattedEventValues;
  }
  /**
   * Creates category list to be used by filter
   * @returns {Array<object>}
   */
  function getCategoryList() {
    return [
      {
        id: 'All',
        value: 'All Categories'
      },
      {
        id: 'Administrative',
        value: 'Administrative'
      },
      {
        id: 'DailyRoutine',
        value: 'Daily Routine'
      },
      {
        id: 'Meals',
        value: 'Meals'
      },
      {
        id: 'Portfolio',
        value: 'Portfolio'
      },
      {
        id: 'Notes',
        value: 'Notes'
      }
    ];
  }
  /**
   * Retrieves children for a given parent from Facility service
   * @returns {*}
   */
  function getChildren() {
    var children = [];
    var currentUser = $facility.getSync().currentUser;
    var facilityChildren = $facility.getSync().children;
    _.each(facilityChildren, function (child) {
      var parents = _.get(child, 'parents');
      if (_.find(parents, { id: currentUser.id })) {
        children.push(child);
      }
    });
    return children;
  }
  /**
   * Retrieves activities (events) by date
   */
  function getActivities(date, callback) {
    var children = getChildren();
    $events.getEvents(children, date, function (error, response) {
      if (error) {
        callback(error, null);
      } else {
        var allActivities = _.get(response, 'events');
        var activities = _.filter(allActivities, $filters.checkEventFilter);
        activities = _.sortBy(activities, ['date']).reverse();
        // Format event values:
        activities.forEach(function (activity) {
          activity.formattedEventValues = formatEventValues(activity.eventValues);
        });
        callback(null, activities);
      }
    }, true);
  }
  /**
   * Retrieves the event type 'group name'
   * @param {number} id
   * @returns {string}
   */
  function getEventTypeName(id) {
    var eventType = $facility.findEventType(id);
    return _.get(eventType, 'groupName') || 'Activity';
  }
  /**
   * Retrieves group icon
   * @param {string} group
   * @param {boolean} circle
   * @returns {string}
   */
  function getGroupIcon(group, circle) {
    var circleStr = circle ? '-circle' : '';
    if (group === 'Administrative') {
      return 'img/icon/group/notes' + circleStr + '.svg';
    }
    if (group === 'DailyRoutine') {
      return 'img/icon/group/dailyroutine' + circleStr + '.svg';
    }
    if (group === 'Meals') {
      return 'img/icon/group/meal' + circleStr + '.svg';
    }
    if (group === 'Portfolio') {
      return 'img/icon/group/learning' + circleStr + '.svg';
    }
    return 'img/icon/group/notes.svg';
  }
  /**
   * Checks if a list of children has any activities
   * @param {Array<object>} children
   */
  function hasActivities(children) {
    var hasActivities = false;
    _.each(children, function (child) {
      if (child && child.activities && child.activities.length) {
        hasActivities = true;
        return false;
      }
    });
    return hasActivities;
  }
  /**
   * Sorts media to given child for a list of children
   * @param {Array<object>} activities
   * @param {string|boolean} category
   * @returns {*}
   */
  function sortActivitiesToChildren(activities, category) {
    var children = getChildren();
    _.each(children, function (child) {
      child.activities = _.filter(activities, function (item) {
        if (category) {
          return _.get(item, 'child.id') === child.id && _.get(item, 'eventType.group') === category;
        } else {
          return _.get(item, 'child.id') === child.id;
        }
      });
    });
    return children;
  }
  /**
   * Shows activity details modal via $appstate service
   * @param {object} activity
   * @param {object} child
   */
  function viewActivity(activity, child) {
    popover.hide();
    activity.child.firstName = _.get(activity, 'child.firstName') || _.get(child, 'firstName');
    _.each(activity.eventValues, function (eventValue) {
      eventValue.formattedValue = formatEventValue(eventValue);
    });
    $appstate.showEvent(activity, true);
  }
}
'use strict';
angular.module('kidreports2App').factory('$drawerService', drawerService);
function drawerService($q, $http, $templateCache, $timeout) {
  var _this = {
      openChildProfile: openChildProfile,
      openParentProfile: openParentProfile,
      openTeacherProfile: openTeacherProfile,
      closeDrawer: closeDrawer,
      params: {},
      _delegates: {
        openDrawer: null,
        closeDrawer: null
      }
    };
  var templatesToFetch = [
      'views/drawers/childProfile.html',
      'views/drawers/parentProfile.html',
      'views/drawers/teacherProfile.html'
    ];
  // Wait a few seconds to let essential requests finish first (since
  // this is running on initial app load) and then pre-fetch templates
  $timeout(function () {
    for (var i = 0; i < templatesToFetch.length; i++) {
      $http.get(templatesToFetch[i], { cache: $templateCache });
    }
  }, 3000);
  function openChildProfile(childId) {
    closeDrawer().then(function () {
      _this.params = { childId: childId };
      _this._delegates.openDrawer('views/drawers/childProfile.html');
    });
  }
  function openParentProfile(parentId) {
    closeDrawer().then(function () {
      _this.params = { parentId: parentId };
      _this._delegates.openDrawer('views/drawers/parentProfile.html');
    });
  }
  function openTeacherProfile() {
    closeDrawer().then(function () {
      _this.params = {};
      _this._delegates.openDrawer('views/drawers/teacherProfile.html');
    });
  }
  /**
   * @returns {Promise} resolved when the close animation finishes
   */
  function closeDrawer() {
    return _this._delegates.closeDrawer();
  }
  return _this;
}
'use strict';
angular.module('kidreports2App').factory('$trackingService', trackingService);
trackingService.$inject = [
  '$facility',
  '$translate',
  '$state'
];
/**
 * Service that provides functionality and state for Tracking - Tracking Sheet, Name To Face, etc.
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @param $facility
 * @param $translate
 * @param $state
 * @returns {{populateMenuItems: (function(): [])}}
 */
function trackingService($facility, $translate, $state) {
  var _ = window._;
  var items = _.get($facility.getSync(), 'menuItems');
  return {
    hasNameToFace: hasNameToFace,
    hasTrackingSheet: hasTrackingSheet,
    populateMenuItems: populateMenuItems
  };
  /**
   * Checks if Facility has Name To Face enabled
   * @returns {*}
   */
  function hasNameToFace() {
    return _.find(items, { name: 'Name To Face' });
  }
  /**
   * Checks if Facility has Tracking Sheet enabled
   * @returns {*}
   */
  function hasTrackingSheet() {
    return _.find(items, { name: 'Tracking Sheet' });
  }
  function hasCustomTrackingCharts() {
    return _.get($facility.getSync(), 'trackingCharts');
  }
  /**
   * Populates menu items by checking if items are found in Facility menu items
   * @returns {[]}
   */
  function populateMenuItems() {
    var menuItems = [];
    // Add Tracking Sheet, if found:
    if (hasNameToFace()) {
      menuItems.push({
        name: $translate.instant('NameToFace'),
        onClick: function () {
          $state.go('home.dash.nameToFace');
        }
      });
    }
    // Add Name To Face,  if found:
    if (hasTrackingSheet()) {
      menuItems.push({
        name: $translate.instant('TrackingSheet'),
        onClick: function () {
          $state.go('home.dash.trackingSheet');
        }
      });
    }
    // Add Custom Tracking Charts, if found
    if (hasCustomTrackingCharts()) {
      var trackingCharts = $facility.getSync().trackingCharts;
      for (var i = 0; i < trackingCharts.length; i++) {
        menuItems.push({
          id: trackingCharts[i].id,
          name: trackingCharts[i].name.substring(0, 16),
          onClick: function () {
            $state.go('home.dash.trackingChart', { trackingChartId: this.id });
          }
        });
      }
    }
    // Where Is is always present
    menuItems.push({
      name: $translate.instant('WhereIs'),
      onClick: function () {
        $state.go('home.dash.whereIs');
      }
    });
    return menuItems;
  }
}
'use strict';
angular.module('kidreports2App').factory('$refreshService', refreshService);
refreshService.$inject = [
  '$appstate',
  '$events'
];
/**
 * Functions and state relating to Configuration/Facility Refresh
 * @param {object} $appstate
 * @param {object} $events
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @returns {object}
 */
function refreshService($appstate, $events) {
  var _ = window._;
  return {
    checkAllStorage: checkAllStorage,
    checkStorage: checkStorage,
    getConfirmationMessage: getConfirmationMessage,
    getConfirmationSubject: getConfirmationSubject,
    getStorage: getStorage,
    logStorageKeys: logStorageKeys
  };
  /**
   * Checks all known caching keys used in Local Storage for items -
   * Drafts, Events, Uploads, etc. - and returns item if found
   * @returns {Array<object>}
   */
  function checkAllStorage() {
    // Iterate through list of Local Storage keys:
    var keys = [{
          key: 'drafts',
          label: 'Drafts'
        }];
    var mapped = _.map(keys, function (key) {
        var items = checkStorage(key.key) ? getStorage(key.key) : [];
        if (items.length) {
          return {
            key: key.key,
            label: key.label,
            items: items
          };
        }
      });
    var list = _.compact(mapped);
    // Check the custom cache for Media:
    var media = $appstate.model.cachedMedia;
    if (media && media.length) {
      list.push({
        key: 'media',
        label: 'Media',
        items: media
      });
    }
    // Check the custom cache for Attendance Events:
    var attendance = $appstate.model.cachedItems;
    if (attendance && attendance.length) {
      list.push({
        key: 'attendance',
        label: 'Attendance Events',
        items: attendance
      });
    }
    // Check the custom cache for Events/Activities:
    if ($events.queue() && $events.queue().length) {
      list.push({
        key: 'events',
        label: 'Activities',
        items: $events.queue()
      });
    }
    return list;
  }
  /**
   * Checks to see if storage item is found
   * @param {string} key
   * @returns {boolean}
   */
  function checkStorage(key) {
    var cache = JSON.parse(window.localStorage.getItem(key));
    return !!(cache && cache.length);
  }
  /**
   * Retrieves the message for Refresh confirmation
   * @returns {string}
   */
  function getConfirmationMessage() {
    var message = 'Some items have not been saved:<br /><br />';
    var items = checkAllStorage();
    _.each(items, function (item) {
      var listItem = item.label + ': ' + item.items.length + ' item(s)<br />';
      message += listItem;
    });
    message += '<br />By refreshing, you will lose all pending items. Do you want to continue?';
    return message;
  }
  /**
   * Retrieves the subject of Refresh confirmation
   * @returns {string}
   */
  function getConfirmationSubject() {
    return 'Remove all pending items?';
  }
  /**
   * Finds and retrieves parsed JSON for item from Local Storage by key
   * @param {string} key
   * @returns {any}
   */
  function getStorage(key) {
    return JSON.parse(window.localStorage.getItem(key));
  }
  /**
   * Used for debugging purposes - logs the keys currently in Local Storage
   * @returns {void}
   */
  function logStorageKeys() {
    // Observe all items in local storage:
    for (var i = 0; i < window.localStorage.length; i++) {
      var key = window.localStorage.key(i);
      if (!key.includes('profilePic-')) {
        console.log(key, localStorage.getItem(key));
      }
    }
  }
}
'use strict';
angular.module('kidreports2App').factory('$environmentService', environmentService);
environmentService.$inject = ['$krConfig'];
/**
 * @function environmentService
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 */
function environmentService($krConfig) {
  var _ = window._;
  var mapping = [
      {
        url: 'https://stage.kidreports.com',
        hostname: 'stage.kidreports.com',
        api: 'https://express-api-stage.kidreports.com'
      },
      {
        url: 'https://secure.kidreports.com',
        hostname: 'secure.kidreports.com',
        api: 'https://express-api.kidreports.com'
      },
      {
        url: 'https://test.kidreports.com',
        hostname: 'test.kidreports.com',
        api: 'https://express-api-stage.kidreports.com'
      },
      {
        url: 'http://localhost',
        hostname: 'localhost',
        api: 'http://localhost'
      }
    ];
  /**
   * Retrieves the Express API URL prefix based on environment
   * @returns {*}
   */
  function getApiUrl() {
    if (isNative()) {
      var cachedServerUrl = $krConfig.getAjaxBase();
      // If native, ensure fall back to production Express server if nothing is found:
      return getApiUrlFromCached(cachedServerUrl) || 'https://express-api.kidreports.com';
    } else {
      var hostname = _.get(window, 'location.hostname');
      var match = _.find(mapping, { hostname: hostname });
      return match ? _.get(match, 'api') : '';
    }
  }
  /**
   * Retrieves the correct API URL prefix from the cached
   * @param cachedServerUrl
   * @returns {*}
   */
  function getApiUrlFromCached(cachedServerUrl) {
    var match = _.find(mapping, { url: cachedServerUrl });
    return match ? _.get(match, 'api') : 'https://express-api.kidreports.com';
  }
  /**
   * Checks if Forge is available to determine if application is running as native mobile
   * @returns {boolean}
   */
  function isNative() {
    return typeof forge !== 'undefined';
  }
  return { getApiUrl: getApiUrl };
}
'use strict';
angular.module('kidreports2App').factory('$featureFlags', featureFlagService);
featureFlagService.$inject = [
  '$environmentService',
  '$q',
  '$ajax',
  '$facility'
];
/**
 * @function featureFlagService()
 * @author Joshua Belden <jcbelden@procaresoftware.com>
 * @returns {{
 *  getAllFeatures: (function(): object),
 *  isFeatureEnabled: (function(string): boolean),
 * }}
 */
function featureFlagService($environmentService, $q, $ajax, $facility) {
  var _ = window._;
  var heartbeatTtl = 60;
  // The first heartbeat should tick after 60 seconds. It will then retrieve the real value from the server.
  var heartbeatTimer;
  var cacheTtl;
  var cacheTimer;
  var cachedFlags;
  return {
    establishCache: establishCache,
    getAllFlags: getAllFlags,
    isFeatureEnabled: isFeatureEnabled
  };
  function establishCache() {
    console.debug('feature-flag.service :: Establishing cache');
    // Retrieve flags immediately before setting up the intervals
    retrieveAllFlagsAsync();
    heartbeatTimer = createHeartbeatTimer();
  }
  function createHeartbeatTimer() {
    // Create an infinite timer to continually check for the cache ttl every 1 hour
    return setInterval(function () {
      console.debug('feature-flag.service :: Heartbeat tick');
      getCacheTtlAsync().then(function (response) {
        // The cache ttl has changed, setup the cache timer.
        if (response.cacheTtl !== cacheTtl) {
          // We have a new value for the cache ttl, setup a timer to invalidate the cache and rebuild it.
          cacheTtl = response.cacheTtl;
          console.debug('feature-flag.service :: Cache ttl set to', cacheTtl);
          // Create a timer to invalidate the cache and rebuild it.
          if (cacheTimer) {
            clearInterval(cacheTimer);
          }
          cacheTimer = setInterval(retrieveAllFlagsAsync, cacheTtl * 1000);
        }
        // The heartbeat ttl has changed, setup the heartbeat timer.
        if (response.heartbeatTtl !== heartbeatTtl) {
          // We have a new value for the cache ttl, setup a timer to invalidate the cache and rebuild it.
          heartbeatTtl = response.heartbeatTtl;
          console.debug('feature-flag.service :: Heartbeat ttl set to', heartbeatTtl);
          // Create a timer to check for the cache ttl.
          if (heartbeatTimer) {
            clearInterval(heartbeatTimer);
          }
          heartbeatTimer = createHeartbeatTimer();
        }
      }).catch(function (error) {
        console.debug('feature-flag.service::createHeartbeatTimer - Service is down.');
      });
    }, heartbeatTtl * 1000);
  }
  function getCacheTtlAsync() {
    return new Promise(function (resolve, reject) {
      try {
        var endpoint = $environmentService.getApiUrl() + '/featureflag/options';
        var options = {
            method: 'GET',
            url: endpoint
          };
        console.debug('feature-flag.service::getCacheTtlAsync endpoint, options', endpoint, options);
        $ajax.ajax(options, function (error, response) {
          if (error) {
            resolve();
          } else {
            // If the feature flag service is accidentally set to a value that is converted to milliseconds
            //  and greater than a 32-bit integer, the setinterval method will blow up and continually run.
            var heartbeatTtlNewValue = response.heartbeatTtl;
            if (heartbeatTtlNewValue > 2147483) {
              heartbeatTtlNewValue = 2147483;
            }
            var cacheTtlNewValue = response.cacheTtl;
            if (cacheTtlNewValue > 2147483) {
              cacheTtlNewValue = 2147483;
            }
            resolve({
              heartbeatTtl: heartbeatTtlNewValue,
              cacheTtl: cacheTtlNewValue
            });
          }
        });
      } catch (error) {
        console.debug('feature-flag.service::getCacheTtlAsync - Service is down.');
        resolve();
      }
    });
  }
  /**
     * @function @cacheAllFlags
     * @author Joshua Belden <jcbelden@procaresoftware.com>
     * @description Forces a call to the server to retrieve all flags and caches them locally.
     */
  function retrieveAllFlagsAsync() {
    return new Promise(function (resolve, reject) {
      try {
        var endpoint = $environmentService.getApiUrl() + '/featureflag';
        var options = {
            method: 'GET',
            url: endpoint
          };
        $ajax.ajax(options, function (error, response) {
          if (error) {
            resolve();
          } else {
            cachedFlags = response.featureFlags;
            console.debug('feature-flag.service::retrieveAllFlagsAsync - Feature flags retrieved and cached');
            resolve();
          }
        });
      } catch (error) {
        console.debug('feature-flag.service :: Service is down.');
        resolve();
      }
    });
  }
  /**
     * @function getAllFlags
     * @author Joshua Belden <jcbelden@procaresoftware.com>
     * @returns {Array} An array of feature flags.
     */
  function getAllFlags() {
    if (!cachedFlags) {
      console.debug('You must first cache all flags before retrieving them.');
      return [];
    }
    return cachedFlags;
  }
  /**
     * @function isFeatureEnabled
     * @author Joshua Belden <jcbelden@procaresoftware.com>
     * @description Checks whether or not the specified feature is enabled. The current user's facility will be used when
     *  evaluating facility targets.
     * @param {string} featureKey Unique identifier of the feature to check.
     * @returns {Promise} A promise representing whether or not the feature is enabled.
     */
  function isFeatureEnabled(featureKey) {
    if (!cachedFlags) {
      console.debug('You must first get all flags before checking for individual features.');
      return false;
    }
    try {
      var flag = _.find(cachedFlags, function (f) {
          return f.featureKey == featureKey;
        });
      // If the flag wasn't found, just return false, otherwise run through the enable logic, including targets.
      if (!flag) {
        return false;
      } else {
        // If the flag isn't enabled, bail immediately with a false and skip any more logic.
        if (!flag.enabled) {
          return false;
        }
        if (flag.targets && flag.targets.length > 0) {
          // Facility query is here since we need it for context to check targets.
          var facilityId = $facility.getSync().id;
          for (let targetIdx = 0; targetIdx < flag.targets.length; targetIdx++) {
            const target = flag.targets[targetIdx];
            switch (target.type) {
            case 'facility':
              if (_.toNumber(target.matchValue) == facilityId) {
                // We have a valid match, this feature is enabled.
                return true;
              }
              break;
            }
          }
          // If the flag was enabled and no targets matched, this feature is disabled.
          return false;
        } else {
          // If the flag was enabled and we weren't evaluating targets, this feature is enabled.
          return true;
        }
      }
    } catch (error) {
      // If there is any error at all, consider the feature disabled.
      return false;
    }
  }
}
'use strict';
angular.module('kidreports2App').factory('$progressiveTimeout', progressiveTimeoutService);
progressiveTimeoutService.$inject = ['$timeout'];
/**
 * Progressive Timeout Service
 * @author Joshua Belden <jcbelden@procaresoftware.com>
 * @param $timeout
 * @description The progressive timeout service is used with a retry mechanism
 * and provides an increasing interval to relieve pressure on the server in an
 * outage. Scopes are used to create different interval timers.
 */
function progressiveTimeoutService($timeout) {
  // The interval curve represents the increased delay in seconds. Each
  //  scope will be assigned in index to the internalCurve array. Each
  //  call to create a timer will increase the delay by incrementing the
  //  index along the array.
  var intervalCurve = [
      1,
      2,
      3,
      5,
      10,
      20,
      30,
      60,
      2 * 60,
      5 * 60
    ];
  var scopedIndex = {};
  var toMilliseconds = 1000;
  return {
    createTimer: createTimer,
    destroyTimer: destroyTimer,
    notifyScopeOfSuccess: notifyScopeOfSuccess
  };
  function createTimer(scope, handler) {
    var interval = getInterval(scope);
    var timer = $timeout(handler, interval);
    incrementScopedIndex(scope);
    console.debug('timeout::interval-update', interval);
    return timer;
  }
  function destroyTimer(timer) {
    $timeout.cancel(timer);
  }
  function notifyScopeOfSuccess(scope) {
    if (!scopedIndex[scope]) {
      return;
    }
    scopedIndex[scope] = 0;
    console.debug('timeout::notifyScopeOfSuccess', scope, scopedIndex);
  }
  function getInterval(scope) {
    if (!scopedIndex[scope]) {
      scopedIndex[scope] = 0;
    }
    return intervalCurve[scopedIndex[scope]] * toMilliseconds;
  }
  function incrementScopedIndex(scope) {
    if (scopedIndex[scope] < intervalCurve.length - 1) {
      scopedIndex[scope]++;
    }
  }
}
'use strict';
/**
 * The AttendanceController manages attendance data, behavior and its views. Behavior not specifically
 * tied to a view can be found in the attendance service. Attendance is responsible for marking children
 * as 'in' or 'out' from the current classroom. Attendance can also move children from room to room if
 * the child is currently marked as 'out'.
 * 
 * Room Consolidation and Batch Mode - If a center has setup a room as a consolidation room in the admin
 * portal, then 'Batch Mode' is enabled, allowing a teacher to view all children in a facility and move
 * them in one batch operation. When batch mode is on, a dropdown of available rooms is displayed next
 * to each child. The drop down will default to the consolidation room that was configured, but the teacher
 * can always override that setting.
 */
angular.module('kidreports2App').controller('AttendanceController', AttendanceController);
AttendanceController.$inject = [
  '$state',
  '$attendanceService',
  '$roomService',
  '$stateParams',
  '$appstate',
  '$uiAlert',
  '$rootScope',
  '$scope',
  '$timeout',
  '$backend',
  '$facility'
];
/**
 * @function AttendanceController
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @param $state
 * @param $attendanceService
 * @param $roomService
 * @param $stateParams
 * @param $appstate
 * @param $uiAlert
 * @param $rootScope
 * @param $scope
 * @param $timeout
 * @param $backend
 *
 * @constructor
 */
function AttendanceController($state, $attendanceService, $roomService, $stateParams, $appstate, $uiAlert, $rootScope, $scope, $timeout, $backend, $facility) {
  var vm = this;
  var _ = window._;
  var htmlEntities = window.he;
  // Public functions:
  vm.close = close;
  vm.completeEvent = completeEvent;
  vm.getCheckedIn = getCheckedIn;
  vm.getCheckedOut = getCheckedOut;
  vm.getInCount = getInCount;
  vm.getPendingOut = getPendingOut;
  vm.getRoomName = getRoomName;
  vm.goToOptions = goToOptions;
  vm.isDisabled = isDisabled;
  vm.canToggleBatchMode = canToggleBatchMode;
  vm.toggleAllSelected = toggleAllSelected;
  vm.togglePersonType = togglePersonType;
  vm.toggleViewState = toggleViewState;
  vm.toggleBatchMode = toggleBatchMode;
  vm.showUpdateAttendance = showUpdateAttendance;
  vm.updateAttendance = updateAttendance;
  // DEV-1475: Disable refresh for timeout period to prevent rapid requests
  vm.refresh = refresh;
  vm.refreshDisabled = false;
  vm.refreshTimeout = 3000;
  vm.preventWriteActivity = $facility.getSync().preventWriteActivity;
  initialize();
  /**
   * Determines whether or not the user can go into batch edit mode.
   */
  function canToggleBatchMode() {
    return vm.type === 'Children' && vm.consolidationRoomId && !vm.preventWriteActivity;
  }
  /**
   * Clears pending (out & in) properties for all selected
   */
  function clearPending() {
    vm.selected = _.each(vm.selected, function (item) {
      item.pendingOut = false;
      item.pendingIn = false;
    });
  }
  /**
   * Clears people who have left the room (due to a successful Move Room event)
   */
  function clearPendingMoveRoom() {
    // Remove from selected list, and from general list:
    var list = vm.type === 'Children' ? vm.children : vm.teachers;
    _.each(peoplePendingMoveRoom, function (person) {
      // Remove from selected:
      _.remove(vm.selected, { id: person.id });
      // Remove from general list:
      _.remove(list, { id: person.id });
    });
  }
  /**
   * Navigates back to Dashboard
   */
  function close() {
    $state.go('home.dash');
  }
  /**
   * Completes check in via service
   */
  function completeEvent() {
    var isTeacher = vm.type === 'Teachers';
    var items = !vm.checkedIn ? getPendingIn() : getPendingOut();
    if (!items.length) {
      return false;
    }
    // Set pending success message:
    vm.successMessage = getSuccessMessage(items);
    if (vm.isBatchMode) {
      /*
      If the teacher has selected batch mode, than we need to group children by the 
      destination room chosen and for each room, execute both a move room and attendance update for
      each child under that room.
      */
      var consolidationRoomIds = _.groupBy(items, 'consolidationRoomId');
      _.each(_.keys(consolidationRoomIds), function (consolidationRoomId) {
        var peopleInRoom = consolidationRoomIds[consolidationRoomId];
        var fromRoom = vm.selectedRoom.id;
        var toRoom = consolidationRoomId;
        if (consolidationRoomId === 'undefined') {
          toRoom = vm.consolidationRoomId;
        }
        // Sets people pending to be used by 'move room' event listener:
        peoplePendingMoveRoom = peopleInRoom;
        $roomService.moveRoom(fromRoom, toRoom, peopleInRoom, function (error, response) {
          if (isOfflineEvent(response)) {
            $uiAlert.pushWarningAlert(getPendingMessage(), 'ATTENDANCE');
          }
          $attendanceService.completeEvent(items, isTeacher, null, function (error, attendanceResponse) {
            _.forEach(items, function (item) {
              var childEvent = _.find(attendanceResponse.data, function (ce) {
                  return ce.childId === item.id;
                });
              item.lastEventId = childEvent.eventId;
              item.timeOccurred = new Date(childEvent.timeOccurred);
            });
            if (isOfflineEvent(attendanceResponse)) {
              $uiAlert.pushWarningAlert(getPendingMessage(), 'ATTENDANCE');
            }
          });
        });
      });
    } else {
      $attendanceService.completeEvent(items, isTeacher, null, function (error, response) {
        _.forEach(items, function (item) {
          if (response != null) {
            var childEvent = _.find(response.data, function (ce) {
                return ce.childId === item.id;
              });
            if (!isTeacher) {
              item.lastEventId = childEvent.eventId;
              item.timeOccurred = new Date(childEvent.timeOccurred);
            }
          }
        });
        if (isOfflineEvent(response)) {
          $uiAlert.pushWarningAlert(getPendingMessage(), 'ATTENDANCE');
        }
      });
    }
  }
  /**
   * Creates a room list (for transfer usage)
   * @param {Array<object>} rooms
   * @returns {Array<string>}
   */
  function createRoomList(rooms) {
    var list = [];
    _.each(rooms, function (room) {
      list.push(htmlEntities.encode(room.name));
    });
    return list;
  }
  /**
   * Retrieves room name for a given children
   * @param {object} child
   * @returns {string|*}
   */
  function getRoomName(child) {
    if (vm.type === 'Teachers') {
      return '';
    }
    var room = _.find(vm.rooms, function (room) {
        return child.roomId === room.id;
      });
    if (!room) {
      return '';
    }
    return room.name;
  }
  /**
   * Retrieves list of people checked out for selected type
   * @returns {*}
   */
  function getCheckedOut() {
    return _.filter(vm.selected, function (item) {
      return !item.isIn;
    });
  }
  /**
   * Retrieves list of people checked in for selected type
   * @returns {*}
   */
  function getCheckedIn() {
    return _.filter(vm.selected, function (item) {
      return item.isIn;
    });
  }
  /**
   * Retrieves count of checked in by type
   * @param {string} type
   * @returns {*}
   */
  function getInCount(type) {
    var items = type === 'Children' ? vm.children : vm.teachers;
    return _.filter(items, function (item) {
      return item.isIn;
    }).length;
  }
  /**
   * Retrieves list of pending checked in for selected type
   * @returns {*}
   */
  function getPendingIn() {
    var items = vm.type === 'Children' ? vm.children : vm.teachers;
    return _.filter(items, function (item) {
      return item.pendingIn && !item.isInOtherRoom;
    });
  }
  /**
   * Retrieves list of pending checked out for selected type
   * @returns {*}
   */
  function getPendingOut() {
    var items = vm.type === 'Children' ? vm.children : vm.teachers;
    return _.filter(items, function (item) {
      return item.pendingOut;
    });
  }
  /**
   * Retrieves the message used for a success alert
   * @param {Array<object>} items
   * @returns {string}
   */
  function getSuccessMessage(items) {
    if (items.length > 1) {
      return items.length + ' ' + vm.type.toLowerCase() + ' have been successfully checked ' + (!vm.checkedIn ? 'in' : 'out');
    } else {
      var label = vm.type.toLowerCase() === 'children' ? 'child' : 'teacher';
      return items.length + ' ' + label + ' has been successfully checked ' + (!vm.checkedIn ? 'in' : 'out');
    }
  }
  /**
   * Retrieves the message used for a pending alert
   * @returns {string}
   */
  function getPendingMessage() {
    return 'You are currently in Offline Mode. These records are currently pending.';
  }
  /**
   * Retrieves the message used for a success alert for a Transfer action
   * @param {Array<object>} items
   * @param {string} roomName
   * @returns {string}
   */
  function getSuccessMessageForTransfer(items, roomName) {
    if (items.length > 1) {
      return items.length + ' ' + vm.type.toLowerCase() + ' have been successfully transferred to ' + roomName;
    } else {
      var label = vm.type.toLowerCase() === 'children' ? 'child' : 'teacher';
      return items.length + ' ' + label + ' has been successfully transferred to ' + roomName;
    }
  }
  /**
   * Move to Attendance Options view - currently unused
   */
  function goToOptions() {
    $state.go('attendanceOptions', {
      people: getPendingOut(),
      room: vm.selectedRoom
    });
  }
  /**
   * Initialize controller state - retrieve from service, set default values
   */
  function initialize() {
    setListeners();
    vm.teachers = [];
    // Must retrieve additional information for teachers:
    $attendanceService.getTeachersAsync().then(function (teachers) {
      // KRT3-327 - Only show enabled teachers:
      vm.teachers = _.filter(teachers, function (teacher) {
        return teacher.enabled;
      });
    }).catch(function () {
      // DEV-1475: Alert user of error retrieving teacher information:
      $uiAlert.pushErrorAlert('An error has occurred retrieving teacher information. Refresh to try again.', 'ATTENDANCE');
    });
    vm.children = $attendanceService.getChildren();
    vm.selectedRoom = $attendanceService.getSelectedRoom();
    vm.selected = vm.children;
    vm.allSelected = false;
    vm.checkedIn = false;
    vm.rooms = $roomService.getRooms();
    vm.type = 'Children';
    vm.typeImageSrc = 'img/svg/child-staff-left.svg';
    vm.headerTitle = vm.isBatchMode ? 'All Rooms' : vm.selectedRoom.name;
    initializeSelectRoom();
    initializeConsolidationRoom(vm.children);
    if ($stateParams.tab === 'In') {
      toggleViewState('In');
    }
    // Remove the Check Out ability and Child/Teacher toggle if Move Room:
    if ($stateParams.mode === 'moveRoom') {
      vm.checkOutDisabled = true;
      vm.teacherToggleDisabled = true;
    }
  }
  /**
   * Initialize the consolidation room feature
   */
  function initializeConsolidationRoom() {
    vm.isBatchMode = false;
    vm.rooms = $roomService.getAllRooms();
    vm.consolidationRoomId = $roomService.getConsolidationRoomId();
  }
  /**
   * Initializes room select element with title, room list, etc.
   */
  function initializeSelectRoom() {
    new window.MobileSelect({
      trigger: '#roomSelect',
      triggerDisplayData: false,
      title: 'Move Room',
      wheels: [{ data: createRoomList(vm.rooms) }],
      position: [2],
      callback: moveRoom
    });
  }
  /**
   * Checks if main action (Check In, Check Out) is disabled by pending counts
   * @returns {boolean}
   */
  function isDisabled() {
    return vm.preventWriteActivity || (vm.checkedIn ? getPendingOut().length < 1 : getPendingIn().length < 1);
  }
  /**
   * Checks to see if response from back-end server denotes Offline Mode
   * @param {object} response
   * @returns {*}
   */
  function isOfflineEvent(response) {
    return _.get(response, 'offline');
  }
  /**
   * Performs move room - taking in selection from room select callback
   * @param {Array<number>} index
   * @param {Array<string>} data
   */
  function moveRoom(index, data) {
    $appstate.checkAttendance($appstate.model.selectedRoom.id, function () {
      var toRoom = _.find(vm.rooms, { name: htmlEntities.decode(data[0]) });
      var fromRoom = vm.selectedRoom;
      var people = getPendingOut();
      vm.successMessage = getSuccessMessageForTransfer(people, toRoom.name);
      // Sets people pending to be used by 'move room' event listener:
      peoplePendingMoveRoom = people;
      // Make call to service:
      $roomService.moveRoom(fromRoom.id, toRoom.id, people, function (error, response) {
        if (isOfflineEvent(response)) {
          $uiAlert.pushWarningAlert(getPendingMessage(), 'ATTENDANCE');
        }  // Event subscription is used for Move Room event...
      });
    });
  }
  function showUpdateAttendance(item) {
    var user = $backend.getCurrentUser();
    if (!user) {
      return false;
    }
    if (!item) {
      return false;
    }
    if (!item.lastEventId || !item.timeOccurred) {
      return false;
    }
    return user.legacyRole === 'ADMIN_ROLE' || user.legacyRole === 'FACILITY_ADMIN_ROLE' || user.legacyRole === 'SYSTEM_ADMIN_ROLE' || user.legacyRole === 'ADMIN_ROLE';
  }
  function updateAttendance(item) {
    if (!item) {
      return;
    }
    var eventId = item.lastEventId;
    if (!eventId || eventId <= 0) {
      return;
    }
    try {
      var newTimeOccurred = moment(item.timeOccurred).format('YYYY-MM-DDTHH:mm:ss');
      if (!newTimeOccurred) {
        return;
      }
      $backend.updateAttendance(eventId, newTimeOccurred, function (error, response) {
        if (!response) {
          return;
        }
        if (isOfflineEvent(response)) {
          $uiAlert.pushWarningAlert(getPendingMessage(), 'ATTENDANCE');
        }
        if (response.success == false) {
          $uiAlert.pushErrorAlert('There was an error saving attendance.', 'ATTENDANCE');
          item.timeOccurred = item.previousTimeOccurred;
        } else {
          item.previousTimeOccurred = item.timeOccurred;
        }
      });
    } catch (error) {
      console.error('Error occurred updating attendance.');
    }
  }
  /**
   * Refreshes attendance via service
   */
  function refresh() {
    vm.refreshDisabled = true;
    var isTeacher = vm.type === 'Teachers' || false;
    $attendanceService.refreshAttendance(vm.selected, isTeacher).then(function () {
      vm.children = vm.isBatchMode ? $attendanceService.getAllChildren() : $attendanceService.getChildren();
      vm.selected = vm.type === 'Children' ? vm.children : vm.teachers;
      return $attendanceService.getTeachersAsync();
    }).then(function (teachers) {
      // KRT3-327 - Only show enabled teachers:
      vm.teachers = _.filter(teachers, function (teacher) {
        return teacher.enabled;
      });
      $uiAlert.pushSuccessAlert('Attendance refreshed successfully.', 'ATTENDANCE');
      $timeout(function () {
        vm.refreshDisabled = false;
      }, vm.refreshTimeout);
    }).catch(function () {
      $uiAlert.pushErrorAlert('An error has occurred refreshing attendance information. Please try again.', 'ATTENDANCE');
      $timeout(function () {
        vm.refreshDisabled = false;
      }, vm.refreshTimeout);
    });
  }
  /**
   * Sets up listener handlers on $rootScope for Attendance and Move Room "callbacks"
   */
  function setListeners() {
    // Listener for Attendance action:
    var attendanceListener = $rootScope.$on('attendanceCB', function (event, error) {
        if (error) {
          $uiAlert.pushErrorAlert('An error has occurred. Attempting retry...', 'ATTENDANCE');
        } else {
          if (vm.successMessage) {
            $uiAlert.pushSuccessAlert(vm.successMessage, 'ATTENDANCE');
          }
          clearPending();
        }
      });
    var updateAttendanceListener = $rootScope.$on('updateAttendanceCB', function (event, error) {
        if (error) {
          $uiAlert.pushErrorAlert('An error has occurred. Attempting retry...', 'ATTENDANCE');
        } else {
          if (vm.successMessage) {
            $uiAlert.pushSuccessAlert(vm.successMessage, 'ATTENDANCE');
          }
          clearPending();
        }
      });
    // Listener for Move Room action:
    var moveRoomListener = $rootScope.$on('moveRoomCB', function (event, error) {
        if (error) {
          $uiAlert.pushErrorAlert('An error has occurred. Attempting retry...', 'ATTENDANCE');
        } else {
          if (vm.successMessage) {
            $uiAlert.pushSuccessAlert(vm.successMessage, 'ATTENDANCE');
          }
          clearPending();
          clearPendingMoveRoom();
        }
      });
    // Remove listeners when destroyed:
    $scope.$on('$destroy', attendanceListener);
    $scope.$on('$destroy', updateAttendanceListener);
    $scope.$on('$destroy', moveRoomListener);
  }
  /**
   * Toggles or un-toggles all 'pendingIn' for the selected list
   */
  function toggleAllSelected() {
    vm.selected = _.each(vm.selected, function (item) {
      if (!vm.checkedIn) {
        item.pendingIn = !item.isIn && !item.isInOtherRoom && vm.allSelected;
      } else {
        item.pendingOut = item.isIn && !item.isInOtherRoom && vm.allSelected;
      }
    });
  }
  /**
   * Toggles the type of person - 'Children' or 'Teachers'
   */
  function togglePersonType() {
    vm.isBatchMode = false;
    vm.type = vm.type === 'Children' ? 'Teachers' : 'Children';
    vm.typeImageSrc = vm.type === 'Children' ? 'img/svg/child-staff-left.svg' : 'img/svg/child-staff-right.svg';
    vm.selected = vm.type === 'Children' ? vm.children : vm.teachers;
  }
  /**
   * Toggles the view state between 'In' and 'Out'
   * @param {string} state
   */
  function toggleViewState(state) {
    vm.isBatchMode = false;
    vm.checkedIn = state === 'In';
  }
  /**
   * Toggles whether or not we're in batch mode.
   */
  function toggleBatchMode() {
    vm.isBatchMode = !vm.isBatchMode;
    vm.allSelected = false;
    vm.children = vm.isBatchMode ? $attendanceService.getAllChildren() : $attendanceService.getChildren();
    vm.selected = vm.type === 'Children' ? vm.children : vm.teachers;
    clearPending();
  }
}
'use strict';
angular.module('kidreports2App').controller('AttendanceOptionsController', AttendanceOptionsController);
AttendanceOptionsController.$inject = [
  '$state',
  '$roomService',
  'moment'
];
/**
 * @function AttendanceOptionsController
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @param $state
 * @param $roomService
 * @param moment
 * @constructor
 */
function AttendanceOptionsController($state, $roomService, moment) {
  var vm = this;
  // Public functions:
  vm.close = close;
  initialize();
  /**
   * Navigates back to Dashboard
   */
  function close() {
    $state.go('home.dash');
  }
  function getDefaultTime() {
    var date = moment().toArray();
    return new Date(date[0], date[1], date[2], date[3], date[4]);
  }
  /**
   * On initialization, console.log some stuff
   */
  function initialize() {
    vm.transferActive = false;
    vm.people = _.get($state, 'params.people');
    vm.room = _.get($state, 'params.room');
    vm.defaultTime = getDefaultTime();
    console.log(vm.people);
    console.log(vm.defaultTime);
  }
}
'use strict';
angular.module('kidreports2App').controller('MemoriesController', MemoriesController);
MemoriesController.$inject = [
  '$scope',
  '$memoriesService',
  '$uiAlert'
];
/**
 * @function MemoriesController
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @param $scope
 * @param $memoriesService
 * @param $uiAlert
 * @constructor
 */
function MemoriesController($scope, $memoriesService, $uiAlert) {
  var vm = this;
  // Public functions:
  vm.close = close;
  vm.hideMedia = hideMedia;
  vm.share = share;
  vm.showMedia = showMedia;
  initialize();
  /**
   * Retrieves media from service using any date
   * @param {string} date
   */
  function getMedia(date) {
    $memoriesService.getMedia(date).then(function (result) {
      vm.mediaFound = !!result.length;
      vm.loading = false;
      vm.children = sortMediaToChildren(vm.children, result);
    }, function (error) {
      vm.loading = false;
      console.log(error);
      $uiAlert.pushErrorAlert('There was an error loading media. Please try again.', 'MEMORIES');
    });
  }
  /**
   * Initializes controller state
   */
  function initialize() {
    watchDate();
    watchChildren();
    vm.children = $memoriesService.getChildren();
    vm.maxHeight = window.innerHeight - 200 + 'px';
  }
  /**
   * Hides media via closing modal
   */
  function hideMedia() {
    vm.modalShown = false;
  }
  /**
   * Initiates native share via Forge/Trigger.io
   * @param {object} event
   * @param {object} media
   */
  function share(event, media) {
    event.stopPropagation();
    $memoriesService.shareMedia(media, function (error) {
      if (error) {
        $uiAlert.pushErrorAlert('There was an error sharing media. Please try again.', 'MEMORIES');
      }
    });
  }
  /**
   * Sets selected media and enables/shows modal
   * @param {object} media
   */
  function showMedia(media) {
    if ($memoriesService.isNative() && media.type === 'VIDEO') {
      // If native and video, open up video in Forge tabs:
      return window.forge.tabs.openWithOptions({
        url: media.src,
        navigationToolbar: true
      }, function (success) {
      }, function (error) {
        if (error) {
          $uiAlert.pushErrorAlert('There was an error sharing media. Please try again.', 'MEMORIES');
        }
      });
    } else if (!$memoriesService.isNative() && media.type === 'VIDEO') {
      // If video on web:
      window.open(media.src);
    } else {
      // If photo - all platforms:
      vm.selectedMedia = media;
      vm.modalShown = true;
    }
  }
  /**
   * Sorts media to children
   * @param {Array<object>} children
   * @param {Array<object>} media
   */
  function sortMediaToChildren(children, media) {
    return $memoriesService.sortMediaToChildren(children, media);
  }
  /**
   * Sets watch on 'vm.children' - when a change comes back from selector, filter selected
   */
  function watchChildren() {
    $scope.$watch('vm.children', function (newValue, oldValue) {
      if (newValue !== oldValue) {
        vm.selected = vm.children.filter(function (child) {
          return child.selected === true;
        });
      }
    }, true);
  }
  /**
   * Sets up watcher for changes on 'vm.date' - get new media for a change in date
   */
  function watchDate() {
    $scope.$watch('vm.date', function (newValue, oldValue) {
      if (newValue !== oldValue) {
        vm.loading = true;
        getMedia(newValue);
      }
    }, true);
  }
}
'use strict';
angular.module('kidreports2App').controller('ParentHomeController', ParentHomeController);
ParentHomeController.$inject = [
  'moment',
  '$activitiesService',
  '$appstate',
  '$uiAlert',
  '$drawerService',
  '$facility',
  '$messages',
  '$messageBox',
  '$translate'
];
/**
 * @function ParentHomeController
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @constructor
 */
function ParentHomeController(moment, $activitiesService, $appstate, $uiAlert, $drawerService, $facility, $messages, $messageBox, $translate) {
  var vm = this;
  var _ = window._;
  var facility = $facility.getSync();
  vm.preventWriteActivity = facility.preventWriteActivity;
  vm.navigateAddActivity = navigateAddActivity;
  vm.formatTime = formatTime;
  vm.getEventTypeName = getEventTypeName;
  vm.getGroupIcon = getGroupIcon;
  vm.openChildProfile = openChildProfile;
  vm.refreshActivities = refreshActivities;
  vm.sendArrivalMessage = sendArrivalMessage;
  initialize();
  /**
   * Formats date to time-based format (Example: '10:35 AM')
   * @param date
   * @returns {*}
   */
  function formatTime(date) {
    return moment(new Date(date)).format('h:mm A');
  }
  /**
   * Retrieves activities and sorts to children
   * @param {Moment} date
   */
  function getActivities(date) {
    $activitiesService.getActivities(date.toDate(), function (error, response) {
      if (error) {
        $uiAlert.pushErrorAlert('There was an error loading activities. Please try again.', 'PARENT-HOME');
      } else {
        vm.activitiesFound = !!response.length;
        vm.children = $activitiesService.sortActivitiesToChildren(response);
        vm.selectedChild = vm.children[0];
      }
    });
  }
  /**
   * Retrieves the event type name from activity id
   * @param {number} id
   * @returns {string}
   */
  function getEventTypeName(id) {
    return $activitiesService.getEventTypeName(id);
  }
  /**
   * Retrieves group icon from service
   * @param {object} group
   * @returns {*}
   */
  function getGroupIcon(group) {
    return $activitiesService.getGroupIcon(group, false);
  }
  /**
   * Initializes controller
   */
  function initialize() {
    vm.showSendArrivalMessage = $facility.getSync().showArrivalMessage;
    var date = moment();
    vm.date = date.format('MMMM D, YYYY');
    getActivities(date);
  }
  /**
   * Navigates to add activity screen
   */
  function navigateAddActivity() {
    $appstate.nav.startPath('parentChildEvent', true);
    $appstate.nav.nextState = null;
    $appstate.nav.backState = $appstate.goHome;
    $appstate.nav.start();
  }
  /**
   * Opens child profile drawer
   * @param {object} child
   */
  function openChildProfile(child, $event) {
    $event.stopPropagation();
    $drawerService.openChildProfile(child.id);
  }
  /**
   * Refreshes activities by retrieving media for the currently set date
   */
  function refreshActivities() {
    getActivities(vm.date);
  }
  function sendArrivalMessage(child) {
    var currentUser = $facility.getSync().currentUser;
    var subject = $translate.instant('arrivalMessage').replace('&childname&', child.firstName).replace('&parentname&', currentUser.firstName);
    var sendTo = 'TEACHER:' + child.id;
    var item = { fromUser: { id: currentUser.id } };
    item.body = subject;
    item.subject = subject;
    $messages.put(item, null, sendTo).then(function (m) {
      $messageBox.open('Message was sent.', null, { oktext: 'OK' }, null, 'img/icon/send.svg');
    }, function (err) {
      $messageBox.open('Message could not be sent sent.', err.message, { oktext: 'OK' }, null, 'img/icon/send.svg');
    });
  }
}
'use strict';
angular.module('kidreports2App').controller('ActivitiesController', ActivitiesController);
ActivitiesController.$inject = [
  '$scope',
  'moment',
  '$activitiesService',
  '$uiAlert',
  '$appstate',
  '$krConfig'
];
/**
 * @function ActivitiesController
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @param $scope
 * @param moment
 * @param $activitiesService
 * @param $uiAlert
 * @param $appstate
 * @param $krConfig
 * @constructor
 */
function ActivitiesController($scope, moment, $activitiesService, $uiAlert, $appstate, $krConfig) {
  var vm = this;
  var _ = window._;
  vm.children = $activitiesService.getChildren();
  vm.activitiesFound = false;
  vm.loading = false;
  vm.streamFilter = _.get($appstate, 'model.streamFilter');
  // Public functions:
  vm.close = close;
  vm.clickable = true;
  vm.formatTime = formatTime;
  vm.formatTeacher = formatTeacher;
  vm.getEventTypeName = getEventTypeName;
  vm.getGroupIcon = getGroupIcon;
  vm.showActivityDetails = showActivityDetails;
  vm.isNumber = function (test) {
    return typeof test === 'number';
  };
  vm.canEdit = $activitiesService.canEdit;
  vm.canDelete = $activitiesService.canDelete;
  vm.editActivity = editActivity;
  vm.deleteActivity = deleteActivity;
  vm.hasMedia = hasMedia;
  initialize();
  /**
   * Prompts deletion confirmation via service
   * @param {object} activity
   * @param {object} event
   */
  function deleteActivity(activity, event) {
    $activitiesService.deleteActivity(activity);
    event.stopPropagation();
  }
  /**
   * Opens edit window via service
   * @param {object} activity
   * @param {object} event
   */
  function editActivity(activity, event) {
    $activitiesService.editActivity(activity);
    event.stopPropagation();
  }
  /**
   * Adds 'images' properties to be used
   * @param {Array<object>} activities
   */
  function createMediaProperties(activities) {
    _.each(activities, function (activity) {
      if (hasMedia(activity)) {
        activity.images = [];
        activity.videos = [];
        _.each(activity.mediaList, function (media) {
          var type = _.get(media, 'mediaType') === 'PHOTO' ? 'images' : 'videos';
          activity[type].push({
            url: $krConfig.s3Base + _.get(media, 'relativePath'),
            key: _.get(media, 'key'),
            id: _.get(media, 'id'),
            thumbnail: getThumbnailPath(media)
          });
        });
      }
    });
    return activities;
  }
  /**
   * Filters activities for children by category
   * @param {Array<number>} index
   * @param {Array<object>} data
   */
  function filterCategory(index, data) {
    var categoryId = _.get(data, '[0].id') === 'All' ? false : _.get(data, '[0].id');
    vm.selectedCategory = _.get(data, '[0].value');
    vm.children = $activitiesService.sortActivitiesToChildren(vm.activities, categoryId);
    vm.activitiesFound = $activitiesService.hasActivities(vm.children);
    $scope.$apply();
  }
  /**
   * Formats teacher's name
   * @param {object} activity
   * @returns {string}
   */
  function formatTeacher(activity) {
    var lastName = _.get(activity, 'user.lastName');
    var firstName = _.get(activity, 'user.firstName');
    return firstName + ' ' + lastName[0] + '.';
  }
  /**
   * Formats date to time-based format (Example: '10:35 AM')
   * @param date
   * @returns {*}
   */
  function formatTime(date) {
    return moment(new Date(date)).format('h:mm A');
  }
  /**
   * Retrieves activities and sorts to children
   * @param {Moment} date
   */
  function getActivities(date) {
    $activitiesService.getActivities(date.toDate(), function (error, response) {
      vm.loading = false;
      if (error) {
        $uiAlert.pushErrorAlert('There was an error loading activities. Please try again.', 'ACTIVITIES');
      } else {
        vm.activities = createMediaProperties(response);
        vm.children = $activitiesService.sortActivitiesToChildren(createMediaProperties(response), false);
        vm.activitiesFound = $activitiesService.hasActivities(vm.children);
      }
    });
  }
  /**
   * Retrieves the event type name from activity id
   * @param {number} id
   * @returns {string}
   */
  function getEventTypeName(id) {
    return $activitiesService.getEventTypeName(id);
  }
  /**
   * Retrieves group icon from service
   * @param {object} group
   * @returns {*}
   */
  function getGroupIcon(group) {
    return $activitiesService.getGroupIcon(group, false);
  }
  /**
   * Retrieves thumbnail path for image or video
   * @param {object} media
   * @returns {*}
   */
  function getThumbnailPath(media) {
    var thumbnailPath = _.get(media, 'thumbnailPath');
    var type = _.get(media, 'mediaType') === 'PHOTO' ? 'images' : 'videos';
    // Images:
    if (type === 'images') {
      return $krConfig.s3Base + _.get(media, 'thumbnailPath');
    }
    // Videos:
    if (thumbnailPath && thumbnailPath.indexOf('_0000.png') > -1) {
      return $krConfig.s3Base + _.get(media, 'thumbnailPath');
    } else {
      return 'https://secure.kidreports.com/public/img/kidreports_small.png';
    }
  }
  /**
   * Checks if an activity has a list of media
   * @param {object} activity
   */
  function hasMedia(activity) {
    return activity.mediaList && activity.mediaList.length;
  }
  /**
   * Initializes controller state
   */
  function initialize() {
    vm.selectedCategory = 'All Activities';
    var date = moment();
    vm.date = date.format('YYYY-MM-DD');
    getActivities(date);
    watchDate();
    watchChildren();
    initializeCategoryFilter();
  }
  /**
   * Initializes category filter
   */
  function initializeCategoryFilter() {
    var mobileSelect = new window.MobileSelect({
        trigger: '#categorySelect',
        triggerDisplayData: false,
        title: 'Category',
        wheels: [{ data: $activitiesService.getCategoryList() }],
        position: [0],
        callback: filterCategory
      });
  }
  /**
   * Sets selected activity and enables details modal
   * @param {object} activity
   * @param {object} child
   */
  function showActivityDetails(activity, child) {
    $activitiesService.viewActivity(activity, child);
  }
  /**
   * Sets up watcher for changes on 'vm.children' - when a change comes back from selector, filter for selected
   */
  function watchChildren() {
    $scope.$watch('vm.children', function (newValue, oldValue) {
      if (newValue !== oldValue) {
        vm.selected = vm.children.filter(function (child) {
          return child.selected === true;
        });
      }
    }, true);
  }
  /**
   * Sets up watcher for changes on 'vm.date' - get new activities for a change in date
   */
  function watchDate() {
    $scope.$watch('vm.date', function (newValue, oldValue) {
      if (newValue !== oldValue) {
        vm.loading = true;
        getActivities(moment(vm.date));
      }
    }, true);
  }
}
'use strict';
angular.module('kidreports2App').controller('DrawerController', [
  '$scope',
  '$drawerService',
  '$q',
  function ($scope, $drawerService, $q) {
    $scope.template = { url: '' };
    $scope.drawerOpen = false;
    $scope.onClose = onClose;
    var unresolvedPromises = [];
    var isClosing = false;
    $scope.$watch('drawerOpen', function (value) {
      if (value) {
        isClosing = value;
      }
    });
    /**
     * Open right-hand drawer and load the specified template
     * @param templateUrl
     */
    function openDrawer(templateUrl) {
      $scope.template.url = templateUrl;
      $scope.drawerOpen = true;
    }
    /**
     * @returns {Promise}
     */
    function closeDrawer() {
      var deferred = $q.defer();
      if ($scope.drawerOpen || isClosing) {
        $scope.drawerOpen = false;
        unresolvedPromises.push(deferred);
      } else {
        deferred.resolve();
      }
      return deferred.promise;
    }
    /**
     * Resolve promises made by closeDrawer() when pageslide's CSS transition ends
     */
    function onClose() {
      var promise;
      isClosing = false;
      // unload the ng-include, needed if we end up loading the same view next time
      $scope.template.url = 'views/empty.html';
      $scope.$apply();
      while (promise = unresolvedPromises.pop()) {
        promise.resolve();
      }
    }
    $drawerService._delegates.openDrawer = openDrawer;
    $drawerService._delegates.closeDrawer = closeDrawer;
  }
]);
'use strict';
angular.module('kidreports2App').controller('ChooseMediaSourceControlller', ChooseMediaSourceControlller);
function ChooseMediaSourceControlller($modal, $rootScope, $scope, $facility, $messageBox, $translate, $appstate, $state, $adapter, $modalInstance, $timeout, $native) {
  var choice = null;
  $scope.choosePhoto = function ($event) {
    $event.stopPropagation();
    choice = 1;
    createMediaEvent();
  };
  $scope.chooseVideo = function ($event) {
    $event.stopPropagation();
    choice = 2;
    createMediaEvent();
  };
  var destroyWatcher = $rootScope.$on('EventDetailsRendered', function () {
      $modalInstance.close();
      $timeout(function () {
        $(getInputId()).click();
        choice = null;
      }, 250);
      destroyWatcher();
    });
  function getInputId() {
    if (choice === 1) {
      if ($native.isNative())
        return '#NativeAddPhoto';
      else
        return '#fileimg';
    } else if (choice === 2) {
      if ($native.isNative())
        return '#NativeAddVideo';
      else
        return '#filevid';
    }
  }
  function createMediaEvent() {
    var cat = $appstate.model.selectedRoom.category;
    var eventType = _.filter($facility.getSync().eventTypes, function (item) {
        return item.name.indexOf('Picture/Video') > -1 && item.categories[0].id == cat.id;
      })[0];
    newEvent(eventType, 'home.dash.event', true);
  }
  // Ripped from dashboard
  function newEvent(item, state, isPicVideo) {
    $appstate.checkAttendance($appstate.model.selectedRoom.id, function (_item) {
      __finishEvent(_item || item, state, isPicVideo);
    }, item);
  }
  function __finishEvent(item, state, isPicVideo) {
    $appstate.model.currentEvent = $adapter.createNewEvent(item);
    $appstate.model.currentEvent.observed = $appstate.model.observed;
    if (isPicVideo && item.name.indexOf('Picture/Video') == -1) {
      state = 'home.dash.event';
    }
    if ($state.includes('home.dash')) {
      $state.go(state, {}, {
        location: true,
        inherit: true,
        relative: 'home.dash',
        notify: true
      });
    }
  }
}
'use strict';
angular.module('kidreports2App').controller('HeaderController', HeaderController).directive('krHeader', krHeader);
/**
 * @function krHeader()
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @description Directive that creates the 'kr-header' component
 * @returns {object}
 */
function krHeader() {
  return {
    restrict: 'EA',
    require: 'ngModel',
    templateUrl: 'views/directives/kr-header.directive.html',
    scope: {
      title: '=',
      hideHome: '='
    },
    transclude: true,
    controller: HeaderController,
    controllerAs: 'vm',
    bindToController: true,
    compile: function (element, attrs) {
      if (!attrs.title) {
        attrs.title = 'Title';
      }
      if (!attrs.hideHome) {
        attrs.hideHome = false;
      }
    }
  };
}
HeaderController.$inject = [
  '$state',
  '$appService'
];
/**
 * @function HeaderController
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @param $state
 * @param $appService
 */
function HeaderController($state, $appService) {
  var vm = this;
  // Public Functions:
  vm.close = close;
  /**
   * Navigates back to Dashboard
   */
  function close() {
    if ($appService.getUserRole() === 'PARENT_ROLE') {
      $state.go('parent-home');
    } else {
      $state.go('home.dash');
    }
  }
}
'use strict';
angular.module('kidreports2App').controller('AvatarDropdownController', AvatarDropdownController).directive('krAvatarDropdown', krAvatarDropdown);
/**
 * @function krAvatarDropdown()
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @description Directive that creates the 'kr-avatar-dropdown' component
 * @returns {object}
 */
function krAvatarDropdown() {
  return {
    restrict: 'EA',
    require: 'ngModel',
    templateUrl: 'views/directives/kr-avatar-dropdown.directive.html',
    scope: { title: '=' },
    controller: AvatarDropdownController,
    controllerAs: 'vm',
    bindToController: true
  };
}
AvatarDropdownController.$inject = [
  '$scope',
  '$appstate',
  '$facility',
  '$krConfig',
  '$state',
  '$utils',
  '$drawerService',
  '$rootScope'
];
/**
 * @function AvatarDropdownController
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 */
function AvatarDropdownController($scope, $appstate, $facility, $krConfig, $state, $utils, $drawerService, $rootScope) {
  var vm = this;
  // Public Functions:
  vm.navigateToMyProfile = navigateToMyProfile;
  vm.navigateToEditDrafts = navigateToEditDrafts;
  vm.navigateToViewReminders = navigateToViewReminders;
  vm.navigateToQuickLinks = navigateToQuickLinks;
  vm.signOut = signOut;
  initialize();
  /**
   * Parses image URL from user
   * @param {object} user
   * @returns {*}
   */
  function getImageUrlFromUser(user) {
    return $utils.createPhotoUrl($krConfig.s3Base, user.photoPath);
  }
  /**
   * Retrieves user information from Facility service
   * @returns {*}
   */
  function getUser() {
    return $facility.getSync().currentUser;
  }
  /**
   * Initializes states (on construction)
   */
  function initialize() {
    vm.user = getUser();
    vm.image = getImageUrlFromUser(vm.user);
    vm.role = _.get(vm.user, 'role');
    onProfileImageUpdate();
  }
  /**
   * Navigates to my profile
   */
  function navigateToMyProfile($event) {
    $event.stopPropagation();
    if (vm.role === 'PARENT_ROLE') {
      $drawerService.openParentProfile(vm.user.id);
    } else {
      $drawerService.openTeacherProfile();
    }
  }
  /**
   * Navigates to edit drafts
   */
  function navigateToEditDrafts() {
    $state.go('home.dash.pickDraft');
  }
  /**
   * Navigates to view reminders
   */
  function navigateToViewReminders() {
    $state.go('home.dash.reminders');
  }
  /**
   * Navigates to quick links
   */
  function navigateToQuickLinks() {
    $state.go('home.dash.quicklinks');
  }
  /**
   * Subscribes to 'profileImageUpdate' changes, and updates image URL in handler
   */
  function onProfileImageUpdate() {
    $rootScope.$on('profileImageUpdate', function (event, data) {
      vm.image = _.get(data, 'photoUrl') || 'img/face.png';
    });
  }
  /**
   * Signs user out of KidReports
   */
  function signOut() {
    $appstate.nav.logout();
  }
}
'use strict';
angular.module('kidreports2App').controller('DatepickerController', DatepickerController).directive('krDatepicker', krDatepicker);
/**
 * @function krDatepicker()
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @description Directive that creates the 'kr-datepicker' component
 * @returns {object}
 */
function krDatepicker() {
  return {
    restrict: 'EA',
    require: 'ngModel',
    templateUrl: 'views/directives/kr-datepicker.directive.html',
    scope: {
      period: '=',
      date: '=',
      disabled: '='
    },
    transclude: true,
    controller: DatepickerController,
    controllerAs: 'vm',
    bindToController: true,
    compile: function (element, attrs) {
      if (!attrs.period) {
        attrs.period = 'day';
      }
      if (!attrs.disabled) {
        attrs.disabled = false;
      }
    }
  };
}
DatepickerController.$inject = [
  '$utils',
  'moment'
];
/**
 * @function DatepickerController
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 */
function DatepickerController($utils, moment) {
  var vm = this;
  initialize();
  // Public Functions:
  vm.backDate = backDate;
  vm.forwardDate = forwardDate;
  /**
   * Moves date back by offset based on period (day, month)
   */
  function backDate() {
    vm.date = modifyDate(vm.date, 'subtract', vm.period);
  }
  /**
   * Moves date forward by offset based on period (day, month)
   */
  function forwardDate() {
    vm.date = modifyDate(vm.date, 'add', vm.period);
  }
  /**
   * On initialization, retrieve date and format
   */
  function initialize() {
    vm.dateFormat = getDateFormat(vm.period);
    vm.date = vm.date || getInitialDate();
  }
  /**
   * Retrieves initial date as string (example: '2020-02-17')
   */
  function getInitialDate() {
    return $utils.getDatePart(new Date());
  }
  /**
   * Retrieves date format for date filter by the period (day, month)
   * @param {string} period
   */
  function getDateFormat(period) {
    return period.toLowerCase() === 'day' ? 'EEE MMM d, yyyy' : 'MMMM, yyyy';
  }
  /**
   * Modifies date (back/forward) for a given period
   * @param {string} date
   * @param {string} direction
   * @param {string} period
   */
  function modifyDate(date, direction, period) {
    return moment(date)[direction](1, period).format('YYYY-MM-DD');
  }
}
'use strict';
angular.module('kidreports2App').controller('PeopleSelectController', PeopleSelectController).directive('krPeopleSelect', krPeopleSelect);
/**
 * @function krPeopleSelect()
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @description Directive that creates the 'kr-people-select' component
 * @returns {object}
 */
function krPeopleSelect() {
  return {
    restrict: 'EA',
    require: 'ngModel',
    templateUrl: 'views/directives/kr-people-select.directive.html',
    scope: { people: '=' },
    transclude: true,
    controller: PeopleSelectController,
    controllerAs: 'vm',
    bindToController: true,
    compile: function (element, attrs) {
      if (!attrs.people) {
        attrs.people = [];
      }
    }
  };
}
PeopleSelectController.$inject = [];
/**
 * @function PeopleSelectController
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 */
function PeopleSelectController() {
  var _ = window._;
  var vm = this;
  // Public Functions:
  vm.getPersonImage = getPersonImage;
  vm.isSelected = isSelected;
  vm.select = select;
  /**
   * Retrieves a person's image in background-image format
   * @param {object} person
   * @returns {{"background-image": string}}
   */
  function getPersonImage(person) {
    var url = _.get(person, 'photoUrl') ? _.get(person, 'photoUrl') : 'image/face.png';
    return { 'background-image': 'url(' + url + ')' };
  }
  /**
   * Retrieves selected status for a given person
   * @param {object} person
   * @returns {*}
   */
  function isSelected(person) {
    return _.get(person, 'selected');
  }
  /**
   * Sets 'selected' to true for a given person, and un-selects any others
   * @param {object} person
   */
  function select(person) {
    if (!person) {
      return false;
    }
    if (_.get(person, 'selected')) {
      person.selected = false;
    } else {
      // Unselect everyone else:
      _.map(vm.people, function (person) {
        if (person) {
          person.selected = false;
          return person;
        }
      });
      person.selected = true;
    }
  }
}
'use strict';
angular.module('kidreports2App').controller('ChildCarouselController', ChildcarouselController).directive('krChildCarousel', krChildCarousel);
function krChildCarousel() {
  return {
    restrict: 'E',
    templateUrl: 'views/directives/kr-child-carousel.directive.html',
    scope: {
      outerId: '@',
      children: '=',
      childSelectHeaderColor: '=?'
    },
    controller: ChildcarouselController,
    controllerAs: 'vm',
    bindToController: true
  };
}
function ChildcarouselController($timeout, $appstate, $rootScope, $scope) {
  var vm = this;
  vm.editChildSelection = editChildSelection;
  vm.scrollCarousel = scrollCarousel;
  vm.isLeftScrollVisible = false;
  vm.isRightScrollVisible = false;
  // Re-evaluate visibility bools upon child selection change
  $rootScope.$on('observedChange', function () {
    $timeout(evaluateScrollVisibilities, 0);
  });
  // Re-evaluate visibility bools upon render & manual (mobile) scrolling
  $timeout(function () {
    evaluateScrollVisibilities();
    document.querySelector('#' + vm.outerId).onscroll = function () {
      if (vm.isScrolling) {
        return;
      }
      evaluateScrollVisibilities();
      $scope.$apply();
    };
  }, 0);
  function editChildSelection() {
    $appstate.dash.innerSelect = true;
    $appstate.dash.setSelectChild(true, vm.childSelectHeaderColor);
  }
  /**
   * Scrolls the carousel left or right
   * @param {Integer} direction - -1 or 1 for left or right
   */
  function scrollCarousel(direction) {
    var containerWidth = $('#' + vm.outerId).width();
    var photoWidth = $('#' + vm.outerId + ' .kr-child-carousel-photo:first').outerWidth(true);
    var numChildrenAbleToDisplay = Math.floor(containerWidth / photoWidth);
    var currentScrollLeft = $('#' + vm.outerId).scrollLeft();
    var targetScrollLeft = Math.round(currentScrollLeft / photoWidth) * photoWidth + direction * numChildrenAbleToDisplay * photoWidth;
    var maxScrollLeft = vm.children.length * photoWidth - containerWidth;
    if (targetScrollLeft < 0) {
      targetScrollLeft = 0;
    }
    if (targetScrollLeft > maxScrollLeft) {
      targetScrollLeft = maxScrollLeft;
    }
    vm.isScrolling = true;
    $('#' + vm.outerId).animate({ scrollLeft: targetScrollLeft }, 'fast', 'swing', function () {
      vm.isScrolling = false;
    });
    evaluateScrollVisibilities(targetScrollLeft);
  }
  /**
   * Set whether left and right scroll buttons should be visible
   * @param scrollLeft (optional) - value to be used other than the true 'scrollLeft' property - in case of ongoing animation
   */
  function evaluateScrollVisibilities(scrollLeft) {
    if (!vm.children.length) {
      vm.isRightScrollVisible = false;
      vm.isLeftScrollVisible = false;
      return;
    }
    var containerWidth = $('#' + vm.outerId).width();
    var photoWidth = $('#' + vm.outerId + ' .kr-child-carousel-photo:first').outerWidth(true);
    var currentScrollLeft = typeof scrollLeft !== 'undefined' ? scrollLeft : $('#' + vm.outerId).scrollLeft();
    var maxScrollLeft = vm.children.length * photoWidth - containerWidth;
    vm.isLeftScrollVisible = currentScrollLeft > 0;
    vm.isRightScrollVisible = currentScrollLeft < maxScrollLeft;
  }
}
'use strict';
angular.module('kidreports2App').controller('ChildMultiSelectController', ChildMultiSelectController).directive('krChildMultiSelect', krChildMultiSelect);
function krChildMultiSelect() {
  return {
    restrict: 'E',
    templateUrl: 'views/directives/kr-child-multi-select.directive.html',
    scope: { hideButtons: '=' },
    controller: ChildMultiSelectController,
    bindToController: true
  };
}
'use strict';
angular.module('kidreports2App').controller('ModalController', ModalController).directive('krModal', krModal);
/**
 * @function krModal()
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @description Directive that creates the 'kr-modal' component
 * @returns {object}
 */
function krModal() {
  return {
    restrict: 'EA',
    require: 'ngModel',
    templateUrl: 'views/directives/kr-modal.directive.html',
    scope: { enabled: '=' },
    transclude: true,
    controller: ModalController,
    controllerAs: 'vm',
    bindToController: true,
    compile: function (element, attrs) {
      if (!attrs.enabled) {
        attrs.enabled = false;
      }
    }
  };
}
ModalController.$inject = ['$scope'];
/**
 * @function ModalController
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 */
function ModalController($scope) {
  var vm = this;
  // Public Functions:
  vm.close = close;
  initialize();
  /**
   * Sets enabled to false
   */
  function close() {
    vm.enabled = false;
  }
  function initialize() {
    watchEnabled();
  }
  function watchEnabled() {
    $scope.$watch('vm.enabled', function (newValue) {
      if (newValue) {
        window.$('body').css('overflow', 'hidden');
      } else {
        window.$('body').css('overflow', 'auto');
      }
    });
  }
}
'use strict';
angular.module('kidreports2App').controller('AlertController', AlertController).directive('krAlert', krAlert);
/**
 * @function krAlert()
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @description Directive that creates the 'kr-alert' component
 * @returns {object}
 */
function krAlert() {
  return {
    restrict: 'EA',
    require: 'ngModel',
    templateUrl: 'views/directives/kr-alert.directive.html',
    scope: {
      target: '=',
      ttl: '='
    },
    transclude: true,
    controller: AlertController,
    controllerAs: 'vm',
    bindToController: true,
    compile: function (element, attrs) {
      if (!attrs.target) {
        attrs.target = null;
      }
      if (!attrs.ttl) {
        attrs.ttl = 3000;
      }
    }
  };
}
AlertController.$inject = [
  '$timeout',
  '$uiAlert',
  '$rootScope',
  '$scope'
];
/**
 * @function AlertController
 * @author Joshua Belden <jcbelden@procaresoftware.com> & Erik Johnson <eajohnson@procaresoftware.com>
 */
function AlertController($timeout, $uiAlert, $rootScope, $scope) {
  var vm = this;
  // Public:
  vm.getAlertClass = getAlertClass;
  initialize();
  /**
   * Displays alerts from service and clear on 'vm.ttl' (milliseconds)
   */
  function displayAlert() {
    if (vm.target) {
      var alerts = $uiAlert.popAlertsFor(vm.target);
      if (alerts) {
        vm.alerts = alerts;
        $timeout(function () {
          vm.alerts = null;
        }, vm.ttl);
      }
    }
  }
  /**
   * Helper function used to retrieve the correct CSS classes for each alert type
   * @param {object} alert
   * @returns {string}
   */
  function getAlertClass(alert) {
    if (alert.type === $uiAlert.ALERT_TYPE_SUCCESS) {
      return 'alert alert-success';
    } else if (alert.type === $uiAlert.ALERT_TYPE_WARNING) {
      return 'alert alert-warning';
    } else if (alert.type === $uiAlert.ALERT_TYPE_ERROR) {
      return 'alert alert-danger';
    } else {
      return 'alert';
    }
  }
  /**
   * On initialization
   */
  function initialize() {
    displayAlert();
    // Sets up event listener for new UI alerts:
    var uiAlertNew = $rootScope.$on('uiAlertNew', function () {
        displayAlert();
      });
    // Clean up listener on destroy:
    $scope.$on('$destroy', uiAlertNew);
  }
}
'use strict';
angular.module('kidreports2App').controller('PillMenuController', PillmenuController).directive('krPillMenu', krPillMenu);
function krPillMenu() {
  return {
    restrict: 'E',
    templateUrl: 'views/directives/kr-pill-menu.directive.html',
    scope: {
      outerId: '@',
      menuItems: '=',
      selectedIndex: '='
    },
    controller: PillmenuController,
    controllerAs: 'vm',
    bindToController: true
  };
}
function PillmenuController($timeout, $appstate, $rootScope, $scope, $window) {
  var vm = this;
  vm.scrollLeft = scrollLeft;
  vm.scrollRight = scrollRight;
  vm.isLeftScrollEnabled = false;
  vm.isRightScrollEnabled = false;
  var $container;
  $timeout(function () {
    $container = $('#' + vm.outerId);
    // Ensure initially selected item is visible
    if (!isItemFullyVisible(vm.selectedIndex)) {
      scrollTo(getRelativeOffset(vm.selectedIndex));
    }
    // Re-evaluate visibility bools upon render & manual (mobile) scrolling
    evaluateScrollStates();
    $container.scroll(function () {
      if (vm.isScrolling) {
        return;
      }
      evaluateScrollStates();
    });
  }, 0);
  angular.element($window).bind('resize', evaluateScrollStates);
  /**
   * Find the right-most menu item that has an negative relative offset and scroll to it
   */
  function scrollLeft() {
    var i = vm.menuItems.length - 1;
    var offset;
    while (i >= 0 && (offset = getRelativeOffset(i)) >= 0) {
      i--;
    }
    if (i < 0) {
      return;
    }
    var targetScrollLeft = $container.scrollLeft() + offset;
    scrollTo(targetScrollLeft);
  }
  /**
   * Find the left-most menu item that has a positive relative offset but is not fully visible and scroll to it
   */
  function scrollRight() {
    var i = 0;
    var offset;
    while (i < vm.menuItems.length && ((offset = getRelativeOffset(i)) < 0 || isItemFullyVisible(i))) {
      i++;
    }
    if (i >= vm.menuItems.length) {
      return;
    }
    var targetScrollLeft = $container.scrollLeft() + offset;
    scrollTo(targetScrollLeft);
  }
  /**
   * Begins scrolling the scroll container to the specified x coordinate
   * @param targetScrollLeft
   */
  function scrollTo(targetScrollLeft) {
    vm.isScrolling = true;
    $container.animate({ scrollLeft: targetScrollLeft }, 'fast', 'swing', function () {
      vm.isScrolling = false;
      evaluateScrollStates();
    });
  }
  /**
   * Set whether left and right scroll buttons should be enabled
   */
  function evaluateScrollStates() {
    vm.isLeftScrollEnabled = !isItemFullyVisible(0);
    vm.isRightScrollEnabled = vm.menuItems && !isItemFullyVisible(vm.menuItems.length - 1);
    $scope.$apply();
  }
  /**
   * Determines whether both left and right edges of a specified menu item are in the scrolled region
   * @param menuItemIndex
   * @returns {boolean}
   */
  function isItemFullyVisible(menuItemIndex) {
    var $targetItem = $('.kr-pill-menu-item', $container).eq(menuItemIndex);
    var relativeOffset = $targetItem.offset() ? $targetItem.offset().left - $container.offset().left : 0;
    // Allowing this function to err by 1px because of a -1px margin elsewhere
    // which fixed a visual glitch but technically partially hides menu items
    var marginOfError = 1;
    return relativeOffset >= 0 - marginOfError && relativeOffset + $targetItem.width() <= $container.width() + marginOfError;
  }
  /**
   * Gets the left offset of a given menu item, relative to the scrolling container
   * @param menuItemIndex
   * @returns {number}
   */
  function getRelativeOffset(menuItemIndex) {
    var $menuItem = $('.kr-pill-menu-item', $container).eq(menuItemIndex);
    return $menuItem.offset().left - $container.offset().left;
  }
}
'use strict';
angular.module('kidreports2App').controller('NavigationFooterController', NavigationFooterController).directive('krNavigationFooter', krNavigationFooter);
/**
 * @function krNavigationFooter()
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @description Directive that creates the 'kr-navigation-footer' component
 * @returns {object}
 */
function krNavigationFooter() {
  return {
    restrict: 'EA',
    require: 'ngModel',
    templateUrl: 'views/directives/kr-navigation-footer.directive.html',
    scope: { title: '=' },
    transclude: true,
    controller: NavigationFooterController,
    controllerAs: 'vm',
    bindToController: true
  };
}
NavigationFooterController.$inject = [
  '$appService',
  '$state',
  '$appstate'
];
/**
 * @function NavigationFooterController
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 */
function NavigationFooterController($appService, $state, $appstate) {
  var vm = this;
  vm.shared = $appstate;
  // Public:
  vm.navigateToMedia = navigateToMedia;
  vm.navigateToMessages = navigateToMessages;
  vm.navigateToMoveRoom = navigateToMoveRoom;
  vm.navigateToActivities = navigateToActivities;
  vm.navigateToMemories = navigateToMemories;
  vm.navigateToAttendance = navigateToAttendance;
  initialize();
  function initialize() {
    vm.role = $appService.getUserRole();
    vm.isParentRole = isParentRole;
  }
  /**
   * Checks if parent role
   * @returns {boolean}
   */
  function isParentRole() {
    return vm.role === 'PARENT_ROLE';
  }
  /**
   * Navigates to Activities
   */
  function navigateToActivities() {
    $state.go('activities');
  }
  /**
   * Navigates to Attendance
   */
  function navigateToAttendance() {
    $appstate.checkAttendance($appstate.model.selectedRoom.id, function () {
      $state.go('attendance');
    });
  }
  /**
   * Navigates to Media
   */
  function navigateToMedia() {
  }
  /**
   * Navigates to Memories
   */
  function navigateToMemories() {
    $state.go('memories');
  }
  /**
   * Navigates to Messages
   */
  function navigateToMessages() {
    $state.go('messages');
  }
  /**
   * Navigates to Move Room
   */
  function navigateToMoveRoom() {
    $state.go('attendance', {
      tab: 'In',
      mode: 'moveRoom'
    });
  }
}
/*!
 * mobileSelect.js
 * (c) 2017-present onlyhom
 * Released under the MIT License.
 */
(function () {
  function getClass(dom, string) {
    return dom.getElementsByClassName(string);
  }
  //构造器
  function MobileSelect(config) {
    this.mobileSelect;
    this.wheelsData = config.wheels;
    this.jsonType = false;
    this.cascadeJsonData = [];
    this.displayJson = [];
    this.curValue = null;
    this.curIndexArr = [];
    this.cascade = false;
    this.startY;
    this.moveEndY;
    this.moveY;
    this.oldMoveY;
    this.offset = 0;
    this.offsetSum = 0;
    this.oversizeBorder;
    this.curDistance = [];
    this.clickStatus = false;
    this.isPC = true;
    this.init(config);
  }
  MobileSelect.prototype = {
    constructor: MobileSelect,
    init: function (config) {
      var _this = this;
      _this.keyMap = config.keyMap ? config.keyMap : {
        id: 'id',
        value: 'value',
        childs: 'childs'
      };
      _this.checkDataType();
      _this.renderWheels(_this.wheelsData, config.cancelBtnText, config.ensureBtnText);
      _this.trigger = document.querySelector(config.trigger);
      if (!_this.trigger) {
        console.error('mobileSelect has been successfully installed, but no trigger found on your page.');
        return false;
      }
      _this.wheel = getClass(_this.mobileSelect, 'wheel');
      _this.slider = getClass(_this.mobileSelect, 'selectContainer');
      _this.wheels = _this.mobileSelect.querySelector('.wheels');
      var li = _this.mobileSelect.querySelector('li');
      if (li) {
        _this.liHeight = li.offsetHeight;
      } else {
        _this.liHeight = 40;  // based on what I found testing in a browser.
      }
      _this.ensureBtn = _this.mobileSelect.querySelector('.ensure');
      _this.cancelBtn = _this.mobileSelect.querySelector('.cancel');
      _this.grayLayer = _this.mobileSelect.querySelector('.grayLayer');
      _this.popUp = _this.mobileSelect.querySelector('.content');
      _this.callback = config.callback || function () {
      };
      _this.transitionEnd = config.transitionEnd || function () {
      };
      _this.onShow = config.onShow || function () {
      };
      _this.onHide = config.onHide || function () {
      };
      _this.initPosition = config.position || [];
      _this.titleText = config.title || '';
      _this.connector = config.connector || ' ';
      _this.triggerDisplayData = !(typeof config.triggerDisplayData == 'undefined') ? config.triggerDisplayData : true;
      _this.trigger.style.cursor = 'pointer';
      _this.setStyle(config);
      _this.setTitle(_this.titleText);
      _this.checkIsPC();
      _this.checkCascade();
      _this.addListenerAll();
      if (_this.cascade) {
        _this.initCascade();
      }
      //定位 初始位置
      if (_this.initPosition.length < _this.slider.length) {
        var diff = _this.slider.length - _this.initPosition.length;
        for (var i = 0; i < diff; i++) {
          _this.initPosition.push(0);
        }
      }
      _this.setCurDistance(_this.initPosition);
      //按钮监听
      _this.cancelBtn.addEventListener('click', function () {
        _this.hide();
      });
      _this.ensureBtn.addEventListener('click', function () {
        _this.hide();
        if (!_this.liHeight) {
          _this.liHeight = _this.mobileSelect.querySelector('li').offsetHeight;
        }
        var tempValue = '';
        for (var i = 0; i < _this.wheel.length; i++) {
          i == _this.wheel.length - 1 ? tempValue += _this.getInnerHtml(i) : tempValue += _this.getInnerHtml(i) + _this.connector;
        }
        if (_this.triggerDisplayData) {
          _this.trigger.innerHTML = tempValue;
        }
        _this.curIndexArr = _this.getIndexArr();
        _this.curValue = _this.getCurValue();
        _this.callback(_this.curIndexArr, _this.curValue);
      });
      _this.trigger.addEventListener('click', function () {
        _this.show();
      });
      _this.grayLayer.addEventListener('click', function () {
        _this.hide();
      });
      _this.popUp.addEventListener('click', function () {
        event.stopPropagation();
      });
      _this.fixRowStyle();  //修正列数
    },
    setTitle: function (string) {
      var _this = this;
      _this.titleText = string;
      _this.mobileSelect.querySelector('.title').innerHTML = _this.titleText;
    },
    setStyle: function (config) {
      var _this = this;
      if (config.ensureBtnColor) {
        _this.ensureBtn.style.color = config.ensureBtnColor;
      }
      if (config.cancelBtnColor) {
        _this.cancelBtn.style.color = config.cancelBtnColor;
      }
      if (config.titleColor) {
        _this.title = _this.mobileSelect.querySelector('.title');
        _this.title.style.color = config.titleColor;
      }
      if (config.textColor) {
        _this.panel = _this.mobileSelect.querySelector('.panel');
        _this.panel.style.color = config.textColor;
      }
      if (config.titleBgColor) {
        _this.btnBar = _this.mobileSelect.querySelector('.btnBar');
        _this.btnBar.style.backgroundColor = config.titleBgColor;
      }
      if (config.bgColor) {
        _this.panel = _this.mobileSelect.querySelector('.panel');
        _this.shadowMask = _this.mobileSelect.querySelector('.shadowMask');
        _this.panel.style.backgroundColor = config.bgColor;
        _this.shadowMask.style.background = 'linear-gradient(to bottom, ' + config.bgColor + ', rgba(255, 255, 255, 0), ' + config.bgColor + ')';
      }
      if (!isNaN(config.maskOpacity)) {
        _this.grayMask = _this.mobileSelect.querySelector('.grayLayer');
        _this.grayMask.style.background = 'rgba(0, 0, 0, ' + config.maskOpacity + ')';
      }
    },
    checkIsPC: function () {
      var _this = this;
      var sUserAgent = navigator.userAgent.toLowerCase();
      var bIsIpad = sUserAgent.match(/ipad/i) == 'ipad';
      var bIsIphoneOs = sUserAgent.match(/iphone os/i) == 'iphone os';
      var bIsMidp = sUserAgent.match(/midp/i) == 'midp';
      var bIsUc7 = sUserAgent.match(/rv:1.2.3.4/i) == 'rv:1.2.3.4';
      var bIsUc = sUserAgent.match(/ucweb/i) == 'ucweb';
      var bIsAndroid = sUserAgent.match(/android/i) == 'android';
      var bIsCE = sUserAgent.match(/windows ce/i) == 'windows ce';
      var bIsWM = sUserAgent.match(/windows mobile/i) == 'windows mobile';
      if (bIsIpad || bIsIphoneOs || bIsMidp || bIsUc7 || bIsUc || bIsAndroid || bIsCE || bIsWM) {
        _this.isPC = false;
      }
    },
    show: function () {
      this.mobileSelect.classList.add('mobileSelect-show');
      if (typeof this.onShow === 'function') {
        this.onShow(this);
      }
    },
    hide: function () {
      this.mobileSelect.classList.remove('mobileSelect-show');
      if (typeof this.onHide === 'function') {
        this.onHide(this);
      }
    },
    renderWheels: function (wheelsData, cancelBtnText, ensureBtnText) {
      var _this = this;
      // var cancelText = cancelBtnText ? cancelBtnText : 'Cancel';
      // var ensureText = ensureBtnText ? ensureBtnText : 'Select';
      _this.mobileSelect = document.createElement('div');
      _this.mobileSelect.className = 'mobileSelect';
      _this.mobileSelect.innerHTML = '<div class="grayLayer"></div>' + '<div class="content">' + '<div class="btnBar">' + '<div class="fixWidth">' + '<div class="cancel">' + '<span class="glyphicon glyphicon-remove"></span>' + '</div>' + '<div class="title"></div>' + '<div class="ensure">' + '<span class="glyphicon glyphicon-ok"></span>' + '</div>' + '</div>' + '</div>' + '<div class="panel">' + '<div class="fixWidth">' + '<div class="wheels">' + '</div>' + '<div class="selectLine"></div>' + '<div class="shadowMask"></div>' + '</div>' + '</div>' + '</div>';
      document.body.appendChild(_this.mobileSelect);
      //根据数据长度来渲染
      var tempHTML = '';
      for (var i = 0; i < wheelsData.length; i++) {
        //列
        tempHTML += '<div class="wheel"><ul class="selectContainer">';
        if (_this.jsonType) {
          for (var j = 0; j < wheelsData[i].data.length; j++) {
            //行
            tempHTML += '<li data-id="' + wheelsData[i].data[j][_this.keyMap.id] + '">' + wheelsData[i].data[j][_this.keyMap.value] + '</li>';
          }
        } else {
          for (var j = 0; j < wheelsData[i].data.length; j++) {
            //行
            tempHTML += '<li>' + wheelsData[i].data[j] + '</li>';
          }
        }
        tempHTML += '</ul></div>';
      }
      _this.mobileSelect.querySelector('.wheels').innerHTML = tempHTML;
    },
    addListenerAll: function () {
      var _this = this;
      for (var i = 0; i < _this.slider.length; i++) {
        //手势监听
        (function (i) {
          _this.addListenerWheel(_this.wheel[i], i);
        }(i));
      }
    },
    addListenerWheel: function (theWheel, index) {
      var _this = this;
      theWheel.addEventListener('touchstart', function () {
        _this.touch(event, this.firstChild, index);
      }, false);
      theWheel.addEventListener('touchend', function () {
        _this.touch(event, this.firstChild, index);
      }, false);
      theWheel.addEventListener('touchmove', function () {
        _this.touch(event, this.firstChild, index);
      }, false);
      if (_this.isPC) {
        //如果是PC端则再增加拖拽监听 方便调试
        theWheel.addEventListener('mousedown', function () {
          _this.dragClick(event, this.firstChild, index);
        }, false);
        theWheel.addEventListener('mousemove', function () {
          _this.dragClick(event, this.firstChild, index);
        }, false);
        theWheel.addEventListener('mouseup', function () {
          _this.dragClick(event, this.firstChild, index);
        }, true);
      }
    },
    checkDataType: function () {
      var _this = this;
      if (typeof _this.wheelsData[0].data[0] == 'object') {
        _this.jsonType = true;
      }
    },
    checkCascade: function () {
      var _this = this;
      if (_this.jsonType) {
        var node = _this.wheelsData[0].data;
        for (var i = 0; i < node.length; i++) {
          if (_this.keyMap.childs in node[i] && node[i][_this.keyMap.childs].length > 0) {
            _this.cascade = true;
            _this.cascadeJsonData = _this.wheelsData[0].data;
            break;
          }
        }
      } else {
        _this.cascade = false;
      }
    },
    generateArrData: function (targetArr) {
      var tempArr = [];
      var keyMap_id = this.keyMap.id;
      var keyMap_value = this.keyMap.value;
      for (var i = 0; i < targetArr.length; i++) {
        var tempObj = {};
        tempObj[keyMap_id] = targetArr[i][this.keyMap.id];
        tempObj[keyMap_value] = targetArr[i][this.keyMap.value];
        tempArr.push(tempObj);
      }
      return tempArr;
    },
    initCascade: function () {
      var _this = this;
      _this.displayJson.push(_this.generateArrData(_this.cascadeJsonData));
      if (_this.initPosition.length > 0) {
        _this.initDeepCount = 0;
        _this.initCheckArrDeep(_this.cascadeJsonData[_this.initPosition[0]]);
      } else {
        _this.checkArrDeep(_this.cascadeJsonData[0]);
      }
      _this.reRenderWheels();
    },
    initCheckArrDeep: function (parent) {
      var _this = this;
      if (parent) {
        if (_this.keyMap.childs in parent && parent[_this.keyMap.childs].length > 0) {
          _this.displayJson.push(_this.generateArrData(parent[_this.keyMap.childs]));
          _this.initDeepCount++;
          var nextNode = parent[_this.keyMap.childs][_this.initPosition[_this.initDeepCount]];
          if (nextNode) {
            _this.initCheckArrDeep(nextNode);
          } else {
            _this.checkArrDeep(parent[_this.keyMap.childs][0]);
          }
        }
      }
    },
    checkArrDeep: function (parent) {
      //检测子节点深度  修改 displayJson
      var _this = this;
      if (parent) {
        if (_this.keyMap.childs in parent && parent[_this.keyMap.childs].length > 0) {
          _this.displayJson.push(_this.generateArrData(parent[_this.keyMap.childs]));
          //生成子节点数组
          _this.checkArrDeep(parent[_this.keyMap.childs][0]);  //检测下一个子节点
        }
      }
    },
    checkRange: function (index, posIndexArr) {
      var _this = this;
      var deleteNum = _this.displayJson.length - 1 - index;
      for (var i = 0; i < deleteNum; i++) {
        _this.displayJson.pop();  //修改 displayJson
      }
      var resultNode;
      for (var i = 0; i <= index; i++) {
        if (i == 0)
          resultNode = _this.cascadeJsonData[posIndexArr[0]];
        else {
          resultNode = resultNode[_this.keyMap.childs][posIndexArr[i]];
        }
      }
      _this.checkArrDeep(resultNode);
      //console.log(_this.displayJson);
      _this.reRenderWheels();
      _this.fixRowStyle();
      _this.setCurDistance(_this.resetPosition(index, posIndexArr));
    },
    resetPosition: function (index, posIndexArr) {
      var _this = this;
      var tempPosArr = posIndexArr;
      var tempCount;
      if (_this.slider.length > posIndexArr.length) {
        tempCount = _this.slider.length - posIndexArr.length;
        for (var i = 0; i < tempCount; i++) {
          tempPosArr.push(0);
        }
      } else if (_this.slider.length < posIndexArr.length) {
        tempCount = posIndexArr.length - _this.slider.length;
        for (var i = 0; i < tempCount; i++) {
          tempPosArr.pop();
        }
      }
      for (var i = index + 1; i < tempPosArr.length; i++) {
        tempPosArr[i] = 0;
      }
      return tempPosArr;
    },
    reRenderWheels: function () {
      var _this = this;
      //删除多余的wheel
      if (_this.wheel.length > _this.displayJson.length) {
        var count = _this.wheel.length - _this.displayJson.length;
        for (var i = 0; i < count; i++) {
          _this.wheels.removeChild(_this.wheel[_this.wheel.length - 1]);
        }
      }
      for (var i = 0; i < _this.displayJson.length; i++) {
        //列
        (function (i) {
          var tempHTML = '';
          if (_this.wheel[i]) {
            //console.log('插入Li');
            for (var j = 0; j < _this.displayJson[i].length; j++) {
              //行
              tempHTML += '<li data-id="' + _this.displayJson[i][j][_this.keyMap.id] + '">' + _this.displayJson[i][j][_this.keyMap.value] + '</li>';
            }
            _this.slider[i].innerHTML = tempHTML;
          } else {
            var tempWheel = document.createElement('div');
            tempWheel.className = 'wheel';
            tempHTML = '<ul class="selectContainer">';
            for (var j = 0; j < _this.displayJson[i].length; j++) {
              //行
              tempHTML += '<li data-id="' + _this.displayJson[i][j][_this.keyMap.id] + '">' + _this.displayJson[i][j][_this.keyMap.value] + '</li>';
            }
            tempHTML += '</ul>';
            tempWheel.innerHTML = tempHTML;
            _this.addListenerWheel(tempWheel, i);
            _this.wheels.appendChild(tempWheel);
          }  //_this.·(i);
        }(i));
      }
    },
    updateWheels: function (data) {
      var _this = this;
      if (_this.cascade) {
        _this.cascadeJsonData = data;
        _this.displayJson = [];
        _this.initCascade();
        if (_this.initPosition.length < _this.slider.length) {
          var diff = _this.slider.length - _this.initPosition.length;
          for (var i = 0; i < diff; i++) {
            _this.initPosition.push(0);
          }
        }
        _this.setCurDistance(_this.initPosition);
        _this.fixRowStyle();
      }
    },
    updateWheel: function (sliderIndex, data) {
      var _this = this;
      var tempHTML = '';
      if (_this.cascade) {
        return false;
      } else if (_this.jsonType) {
        for (var j = 0; j < data.length; j++) {
          tempHTML += '<li data-id="' + data[j][_this.keyMap.id] + '">' + data[j][_this.keyMap.value] + '</li>';
        }
        _this.wheelsData[sliderIndex] = { data: data };
      } else {
        for (var j = 0; j < data.length; j++) {
          tempHTML += '<li>' + data[j] + '</li>';
        }
        _this.wheelsData[sliderIndex] = data;
      }
      _this.slider[sliderIndex].innerHTML = tempHTML;
    },
    fixRowStyle: function () {
      var _this = this;
      var width = (100 / _this.wheel.length).toFixed(2);
      for (var i = 0; i < _this.wheel.length; i++) {
        _this.wheel[i].style.width = width + '%';
      }
    },
    getIndex: function (distance) {
      return Math.round((2 * this.liHeight - distance) / this.liHeight);
    },
    getIndexArr: function () {
      var _this = this;
      var temp = [];
      for (var i = 0; i < _this.curDistance.length; i++) {
        temp.push(_this.getIndex(_this.curDistance[i]));
      }
      return temp;
    },
    getCurValue: function () {
      var _this = this;
      var temp = [];
      var positionArr = _this.getIndexArr();
      if (_this.cascade) {
        for (var i = 0; i < _this.wheel.length; i++) {
          temp.push(_this.displayJson[i][positionArr[i]]);
        }
      } else if (_this.jsonType) {
        for (var i = 0; i < _this.curDistance.length; i++) {
          temp.push(_this.wheelsData[i].data[_this.getIndex(_this.curDistance[i])]);
        }
      } else {
        for (var i = 0; i < _this.curDistance.length; i++) {
          temp.push(_this.getInnerHtml(i));
        }
      }
      return temp;
    },
    getValue: function () {
      return this.curValue;
    },
    calcDistance: function (index) {
      return 2 * this.liHeight - index * this.liHeight;
    },
    setCurDistance: function (indexArr) {
      var _this = this;
      var temp = [];
      for (var i = 0; i < _this.slider.length; i++) {
        temp.push(_this.calcDistance(indexArr[i]));
        _this.movePosition(_this.slider[i], temp[i]);
      }
      _this.curDistance = temp;
    },
    fixPosition: function (distance) {
      return -(this.getIndex(distance) - 2) * this.liHeight;
    },
    movePosition: function (theSlider, distance) {
      theSlider.style.webkitTransform = 'translate3d(0,' + distance + 'px, 0)';
      theSlider.style.transform = 'translate3d(0,' + distance + 'px, 0)';
    },
    locatePosition: function (index, posIndex) {
      var _this = this;
      this.curDistance[index] = this.calcDistance(posIndex);
      this.movePosition(this.slider[index], this.curDistance[index]);
      if (_this.cascade) {
        _this.checkRange(index, _this.getIndexArr());
      }
    },
    updateCurDistance: function (theSlider, index) {
      if (theSlider.style.transform) {
        this.curDistance[index] = parseInt(theSlider.style.transform.split(',')[1]);
      } else {
        this.curDistance[index] = parseInt(theSlider.style.webkitTransform.split(',')[1]);
      }
    },
    getDistance: function (theSlider) {
      if (theSlider.style.transform) {
        return parseInt(theSlider.style.transform.split(',')[1]);
      } else {
        return parseInt(theSlider.style.webkitTransform.split(',')[1]);
      }
    },
    getInnerHtml: function (sliderIndex) {
      var _this = this;
      var index = _this.getIndex(_this.curDistance[sliderIndex]);
      return _this.slider[sliderIndex].getElementsByTagName('li')[index].innerHTML;
    },
    touch: function (event, theSlider, index) {
      var _this = this;
      event = event || window.event;
      switch (event.type) {
      case 'touchstart':
        _this.startY = event.touches[0].clientY;
        _this.startY = parseInt(_this.startY);
        _this.oldMoveY = _this.startY;
        break;
      case 'touchend':
        _this.moveEndY = parseInt(event.changedTouches[0].clientY);
        _this.offsetSum = _this.moveEndY - _this.startY;
        _this.oversizeBorder = -(theSlider.getElementsByTagName('li').length - 3) * _this.liHeight;
        if (_this.offsetSum == 0) {
          var clickOffetNum = parseInt((document.documentElement.clientHeight - _this.moveEndY) / 40);
          if (clickOffetNum != 2) {
            var offset = clickOffetNum - 2;
            var newDistance = _this.curDistance[index] + offset * _this.liHeight;
            if (newDistance <= 2 * _this.liHeight && newDistance >= _this.oversizeBorder) {
              _this.curDistance[index] = newDistance;
              _this.movePosition(theSlider, _this.curDistance[index]);
              _this.transitionEnd(_this.getIndexArr(), _this.getCurValue());
            }
          }
        } else {
          //修正位置
          _this.updateCurDistance(theSlider, index);
          _this.curDistance[index] = _this.fixPosition(_this.curDistance[index]);
          _this.movePosition(theSlider, _this.curDistance[index]);
          //反弹
          if (_this.curDistance[index] + _this.offsetSum > 2 * _this.liHeight) {
            _this.curDistance[index] = 2 * _this.liHeight;
            setTimeout(function () {
              _this.movePosition(theSlider, _this.curDistance[index]);
            }, 100);
          } else if (_this.curDistance[index] + _this.offsetSum < _this.oversizeBorder) {
            _this.curDistance[index] = _this.oversizeBorder;
            setTimeout(function () {
              _this.movePosition(theSlider, _this.curDistance[index]);
            }, 100);
          }
          _this.transitionEnd(_this.getIndexArr(), _this.getCurValue());
        }
        if (_this.cascade) {
          _this.checkRange(index, _this.getIndexArr());
        }
        break;
      case 'touchmove':
        event.preventDefault();
        _this.moveY = event.touches[0].clientY;
        _this.offset = _this.moveY - _this.oldMoveY;
        _this.updateCurDistance(theSlider, index);
        _this.curDistance[index] = _this.curDistance[index] + _this.offset;
        _this.movePosition(theSlider, _this.curDistance[index]);
        _this.oldMoveY = _this.moveY;
        break;
      }
    },
    dragClick: function (event, theSlider, index) {
      var _this = this;
      event = event || window.event;
      switch (event.type) {
      case 'mousedown':
        _this.startY = event.clientY;
        _this.oldMoveY = _this.startY;
        _this.clickStatus = true;
        break;
      case 'mouseup':
        _this.moveEndY = event.clientY;
        _this.offsetSum = _this.moveEndY - _this.startY;
        _this.oversizeBorder = -(theSlider.getElementsByTagName('li').length - 3) * _this.liHeight;
        if (_this.offsetSum == 0) {
          var clickOffetNum = parseInt((document.documentElement.clientHeight - _this.moveEndY) / 40);
          if (clickOffetNum != 2) {
            var offset = clickOffetNum - 2;
            var newDistance = _this.curDistance[index] + offset * _this.liHeight;
            if (newDistance <= 2 * _this.liHeight && newDistance >= _this.oversizeBorder) {
              _this.curDistance[index] = newDistance;
              _this.movePosition(theSlider, _this.curDistance[index]);
              _this.transitionEnd(_this.getIndexArr(), _this.getCurValue());
            }
          }
        } else {
          //修正位置
          _this.updateCurDistance(theSlider, index);
          _this.curDistance[index] = _this.fixPosition(_this.curDistance[index]);
          _this.movePosition(theSlider, _this.curDistance[index]);
          //反弹
          if (_this.curDistance[index] + _this.offsetSum > 2 * _this.liHeight) {
            _this.curDistance[index] = 2 * _this.liHeight;
            setTimeout(function () {
              _this.movePosition(theSlider, _this.curDistance[index]);
            }, 100);
          } else if (_this.curDistance[index] + _this.offsetSum < _this.oversizeBorder) {
            _this.curDistance[index] = _this.oversizeBorder;
            setTimeout(function () {
              _this.movePosition(theSlider, _this.curDistance[index]);
            }, 100);
          }
          _this.transitionEnd(_this.getIndexArr(), _this.getCurValue());
        }
        _this.clickStatus = false;
        if (_this.cascade) {
          _this.checkRange(index, _this.getIndexArr());
        }
        break;
      case 'mousemove':
        event.preventDefault();
        if (_this.clickStatus) {
          _this.moveY = event.clientY;
          _this.offset = _this.moveY - _this.oldMoveY;
          _this.updateCurDistance(theSlider, index);
          _this.curDistance[index] = _this.curDistance[index] + _this.offset;
          _this.movePosition(theSlider, _this.curDistance[index]);
          _this.oldMoveY = _this.moveY;
        }
        break;
      }
    }
  };
  if (typeof exports == 'object') {
    module.exports = MobileSelect;
  } else if (typeof define == 'function' && define.amd) {
    define([], function () {
      return MobileSelect;
    });
  } else {
    window.MobileSelect = MobileSelect;
  }
}());
'use strict';
angular.module('kidreports2App').controller('EventStateStandardsController', EventStateStandardsController).directive('eventStateStandards', eventStateStandards);
/**
 * @function eventStateStandards()
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @description Directive that creates the event-state-standards component
 * @returns {object}
 */
function eventStateStandards() {
  return {
    restrict: 'EA',
    require: 'ngModel',
    templateUrl: 'views/directives/event-state-standards.directive.html',
    scope: {
      eventStandardSelections: '=',
      eventType: '=',
      existingStandards: '=',
      facility: '='
    },
    controller: EventStateStandardsController,
    controllerAs: 'vm',
    bindToController: true
  };
}
EventStateStandardsController.$inject = [
  '$scope',
  '$stateStandards',
  '$appstate'
];
/**
 * @function EventStateStandardsController()
 * @author Erik Johnson <eajohnson@procaresoftware.com>
 * @param $scope
 * @param $stateStandards
 * @constructor
 */
function EventStateStandardsController($scope, $stateStandards, $appstate) {
  var vm = this;
  vm.on = _.get($appstate, 'model.currentEvent.eventType.stateStandardsDisplay') === 'SHOW_DEFAULT_ON';
  vm.selectedAgeGroup = false;
  vm.selectedSubdomains = [];
  vm.selectedDomain = false;
  //vm.selectedStandards = [];
  vm.standards = [];
  vm.isEditMode = !(vm.existingStandards && vm.existingStandards.length > 0);
  vm.selectAgeGroup = selectAgeGroup;
  vm.selectDomain = selectDomain;
  vm.toggleEditMode = toggleEditMode;
  $scope.$watch('vm.facility', function () {
    if (vm.facility.state) {
      getStandards(vm.facility.state);
    }
  });
  function toggleEditMode() {
    if (vm.isEditMode) {
      vm.isEditMode = false;
    }
  }
  /**
   * @function getStandards()
   * @param {string} state
   */
  function getStandards(state) {
    $stateStandards.getStandards(state).then(function (standards) {
      vm.standards = standards.standards;
      if (vm.existingStandards && vm.existingStandards.length > 0) {
        populateExistingStandards();
      } else {
        resetExistingStandards();
      }
    });
  }
  /**
   * @function populateExistingAgeGroupAndDomain()
   * @description If in edit mode, sets the state standard dropdowns
   */
  function populateExistingAgeGroupAndDomain() {
    var first = _.get(vm.existingStandards, '[0]');
    // TODO: When age group is added to standard [ANGULAR-291] remove the nested foreach
    _.each(vm.standards, function (standard) {
      var ageGroup = _.get(standard, 'ageGroup');
      _.each(standard.standards, function (standard) {
        if (standard.guid === first.domain_guid) {
          vm.selectedAgeGroup = _.find(vm.facility.ageGroups, { label: ageGroup });
          selectAgeGroup();
          vm.selectedDomain = _.find(vm.domains, { guid: standard.guid });
          selectDomain();
          return false;
        }
      });
    });
  }
  /**
   * @function resetExistingStandards()
   * @description Reset all the expanded and selected attributes of the standards tree
   *
   */
  function resetExistingStandards() {
    // Next, toggle any necessary subdomains and standards:
    _.each(vm.standards, function (age) {
      _.each(age.standards, function (domain) {
        delete domain.expanded;
        resetChildrenRecursive(domain);
      });
    });
  }
  /**
   * @function resetChildrenRecursive()
   * @description Reset all children down the chain
   *
   */
  function resetChildrenRecursive(parent) {
    if (parent.children && parent.children.length) {
      _.each(parent.children, function (child) {
        delete child.expanded;
        delete child.selected;
        resetChildrenRecursive(child);
      });
    }
  }
  /**
   * @function populateExistingStandards()
   * @description Populate existing standards if it exists for the purposes of Edit Mode
   *
   */
  function populateExistingStandards() {
    // First re-add standards to selections:
    _.each(vm.existingStandards, function (s) {
      vm.eventStandardSelections.push(s);
    });
    //vm.eventStandardSelections = vm.existingStandards;//angular.copy(vm.existingStandards);
    // Turn standards on:
    vm.on = true;
    // From first standard, populate age group and domain:
    populateExistingAgeGroupAndDomain();
    // Next, toggle any necessary subdomains and standards:
    _.each(vm.existingStandards, function (existing) {
      // Expand the subdomain:
      var subdomain = _.find(vm.selectedDomain.children, { guid: existing.subdomain_guid });
      subdomain.expanded = true;
      // Select the standard:
      var standard = $stateStandards.deepFindFirst(subdomain, 'children', { guid: existing.guid });
      standard.selected = true;
    });
  }
  /**
   * @function selectAgeGroup()
   * @description On select of age group, retrieve appropriate domains
   */
  function selectAgeGroup() {
    vm.selectedSubdomains = [];
    vm.domains = _.find(vm.standards, { ageGroup: vm.selectedAgeGroup.label }).standards;
  }
  /**
   * @function selectDomain()
   * @description On select of domain - select concepts
   */
  function selectDomain() {
    vm.selectedSubdomains = vm.selectedDomain.children;
  }
}
'use strict';
angular.module('kidreports2App').controller('EventStateStandardsSelectorController', EventStateStandardsSelectorController).directive('eventStateStandardsSelector', eventStateStandardsSelector);
/**
 * @function eventStateStandardsSelector()
 * @author Will Adams & Erik Johnson
 * @description Directive that creates the event-state-standards-selector component
 * @returns {object}
 */
function eventStateStandardsSelector() {
  return {
    restrict: 'EA',
    require: 'ngModel',
    templateUrl: 'views/directives/event-state-standards-selector.directive.html',
    scope: {
      subdomains: '=',
      domain: '=',
      eventStandardSelections: '=',
      toggleEditMode: '=',
      ageGroup: '='
    },
    controller: EventStateStandardsSelectorController,
    controllerAs: 'vm',
    bindToController: true
  };
}
EventStateStandardsSelectorController.$inject = ['$stateStandards'];
/**
 * @function EventStateStandardsSelectorController()
 * @author Will Adams & Erik Johnson
 */
function EventStateStandardsSelectorController($stateStandards) {
  var vm = this;
  vm.depthOfChildren = depthOfChildren;
  vm.resetSelection = resetSelection;
  vm.toggleStandard = toggleStandard;
  /**
   * @function depthOfChildren()
   * @description Determines the depth of children
   * @param standards
   * @param depth
   * @returns {*}
   */
  function depthOfChildren(standards, depth) {
    if (standards.children && standards.children.length > 0) {
      depth++;
      return depthOfChildren(standards.children[0], depth);
    } else {
      return depth;
    }
  }
  /**
   * @function resetSelection()
   * @description Resets all standards selections made, regardless of depth level
   * @param {any} target
   */
  function resetSelection(target) {
    if (typeof target === 'object') {
      for (var key in target) {
        if (key === 'selected') {
          target[key] = false;
        }
        resetSelection(target[key]);
      }
    }
  }
  /**
   * @function toggleStandard()
   * @param {number} depthLevel
   * @param {object} standard
   * @param {object} subdomain
   * @param {object|null} depthThree
   * @param {object|null} depthFour
   */
  function toggleStandard(depthLevel, standard, subdomain, depthThree, depthFour) {
    if (standard.selected) {
      vm.eventStandardSelections.push($stateStandards.createFullStandard(depthLevel, vm.domain, subdomain, standard, depthThree, depthFour, vm.ageGroup));
    } else {
      _.remove(vm.eventStandardSelections, { guid: standard.guid });
    }
  }
}
'use strict';
angular.module('kidreports2App').factory('optionParser', [
  '$parse',
  function ($parse) {
    //                      00000111000000000000022200000000000000003333333333333330000000000044000
    var TYPEAHEAD_REGEXP = /^\s*(.*?)(?:\s+as\s+(.*?))?\s+for\s+(?:([\$\w][\$\w\d]*))\s+in\s+(.*)$/;
    return {
      parse: function (input) {
        var match = input.match(TYPEAHEAD_REGEXP), modelMapper, viewMapper, source;
        if (!match) {
          throw new Error('Expected typeahead specification in form of \'_modelValue_ (as _label_)? for _item_ in _collection_\'' + ' but got \'' + input + '\'.');
        }
        return {
          itemName: match[3],
          source: $parse(match[4]),
          viewMapper: $parse(match[2] || match[1]),
          modelMapper: $parse(match[1])
        };
      }
    };
  }
]).directive('searchableMultiselect', [
  '$parse',
  '$document',
  '$compile',
  'optionParser',
  function ($parse, $document, $compile, optionParser) {
    return {
      restrict: 'E',
      require: 'ngModel',
      link: function (originalScope, element, attrs, modelCtrl) {
        var exp = attrs.options, parsedResult = optionParser.parse(exp), isMultiple = attrs.multiple ? true : false, required = false, scope = originalScope.$new(), changeHandler = attrs.change || angular.noop;
        scope.items = [];
        scope.header = 'Sort by Concept';
        scope.multiple = isMultiple;
        scope.disabled = false;
        originalScope.$on('$destroy', function () {
          scope.$destroy();
        });
        var popUpEl = angular.element('<multiselect-popup></multiselect-popup>');
        //required validator
        if (attrs.required || attrs.ngRequired) {
          required = true;
        }
        attrs.$observe('required', function (newVal) {
          required = newVal;
        });
        //watch disabled state
        scope.$watch(function () {
          return $parse(attrs.disabled)(originalScope);
        }, function (newVal) {
          scope.disabled = newVal;
        });
        //watch single/multiple state for dynamically change single to multiple
        scope.$watch(function () {
          return $parse(attrs.multiple)(originalScope);
        }, function (newVal) {
          isMultiple = newVal || false;
        });
        //watch option changes for options that are populated dynamically
        scope.$watch(function () {
          return parsedResult.source(originalScope);
        }, function (newVal) {
          if (angular.isDefined(newVal))
            parseModel();
        });
        //watch model change
        scope.$watch(function () {
          return modelCtrl.$modelValue;
        }, function (newVal, oldVal) {
          //when directive initialize, newVal usually undefined. Also, if model value already set in the controller
          //for preselected list then we need to mark checked in our scope item. But we don't want to do this every time
          //model changes. We need to do this only if it is done outside directive scope, from controller, for example.
          if (angular.isDefined(newVal)) {
            markChecked(newVal);
            scope.$eval(changeHandler);
          }
          getHeaderText();
          modelCtrl.$setValidity('required', scope.valid());
        }, true);
        function parseModel() {
          scope.items.length = 0;
          var model = parsedResult.source(originalScope);
          for (var i = 0; i < model.length; i++) {
            var local = {};
            local[parsedResult.itemName] = model[i];
            scope.items.push({
              label: parsedResult.viewMapper(local),
              model: model[i],
              checked: false
            });
          }
        }
        parseModel();
        element.append($compile(popUpEl)(scope));
        function getHeaderText() {
          if (!modelCtrl.$modelValue || !modelCtrl.$modelValue.length) {
            scope.header = 'Sort by Concept';
            return scope.header;
          }
          if (isMultiple) {
            scope.header = modelCtrl.$modelValue.length + ' ' + 'selected';
          } else {
            var local = {};
            local[parsedResult.itemName] = modelCtrl.$modelValue;
            scope.header = parsedResult.viewMapper(local);
          }
        }
        scope.valid = function validModel() {
          if (!required)
            return true;
          var value = modelCtrl.$modelValue;
          return angular.isArray(value) && value.length > 0 || !angular.isArray(value) && value != null;
        };
        function selectSingle(item) {
          if (item.checked) {
            scope.uncheckAll();
          } else {
            scope.uncheckAll();
            item.checked = !item.checked;
          }
          setModelValue(false);
        }
        function selectMultiple(item) {
          item.checked = !item.checked;
          setModelValue(true);
        }
        function setModelValue(isMultiple) {
          var value;
          if (isMultiple) {
            value = [];
            angular.forEach(scope.items, function (item) {
              if (item.checked)
                value.push(item.model);
            });
          } else {
            angular.forEach(scope.items, function (item) {
              if (item.checked) {
                value = item.model;
                return false;
              }
            });
          }
          modelCtrl.$setViewValue(value);
        }
        function markChecked(newVal) {
          if (!angular.isArray(newVal)) {
            angular.forEach(scope.items, function (item) {
              if (angular.equals(item.model, newVal)) {
                item.checked = true;
                return false;
              }
            });
          } else {
            angular.forEach(newVal, function (i) {
              angular.forEach(scope.items, function (item) {
                if (angular.equals(item.model, i)) {
                  item.checked = true;
                }
              });
            });
          }
        }
        scope.checkAll = function () {
          if (!isMultiple)
            return;
          angular.forEach(scope.items, function (item) {
            item.checked = true;
          });
          setModelValue(true);
        };
        scope.uncheckAll = function () {
          angular.forEach(scope.items, function (item) {
            item.checked = false;
          });
          setModelValue(true);
        };
        scope.select = function (item) {
          if (isMultiple === false) {
            selectSingle(item);
            scope.toggleSelect();
          } else {
            selectMultiple(item);
          }
        };
      }
    };
  }
]).directive('multiselectPopup', [
  '$document',
  function ($document) {
    return {
      restrict: 'E',
      scope: false,
      replace: true,
      templateUrl: 'views/directives/searchable-multiselect.directive.html',
      link: function (scope, element) {
        scope.isVisible = false;
        scope.toggleSelect = function (event) {
          event.stopPropagation();
          element.addClass('open');
          scope.focus();  //$document.bind('click', clickHandler);
        };
        scope.close = function () {
          element.removeClass('open');
        };
        function clickHandler(event) {
          if (elementMatchesAnyInArray(event.target, element.find(event.target.tagName))) {
            return;
          }
          element.removeClass('open');
          $document.unbind('click', clickHandler);
          scope.$digest();
        }
        scope.focus = function focus() {
          var searchBox = element.find('input')[0];
          searchBox.focus();
        };
        var elementMatchesAnyInArray = function (element, elementArray) {
          for (var i = 0; i < elementArray.length; i++) {
            if (element === elementArray[i]) {
              return true;
            }
          }
          return false;
        };
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').directive('streamItem', [
  '$appstate',
  '$backend',
  '$native',
  '$facebook',
  '$facility',
  '$location',
  '$translate',
  '$events',
  '$messageBox',
  '$modal',
  'popover',
  '$utils',
  '$timeout',
  '$krConfig',
  '$stateStandards',
  '$appService',
  function ($appstate, $backend, $native, $facebook, $facility, $location, $translate, $events, $messageBox, $modal, popover, $utils, $timeout, $krConfig, $stateStandards, $appService) {
    return {
      templateUrl: 'views/directives/streamItem.html',
      restrict: 'E',
      scope: { item: '=' },
      link: function (scope) {
        var usr = $backend.getCurrentUser();
        var facility = $facility.getSync();
        var delMode = facility.deleteMode || '';
        // Variables
        // Comments: Following boolean variables for permissions checking - edit, delete, send to portfolio
        scope.preventWriteActivity = facility.preventWriteActivity;
        scope.shared = $appstate;
        scope.isParent = usr.role === 'PARENT_ROLE';
        scope.showLinks = !facility.disableParentLinks || !scope.isParent;
        scope.isNative = $native.isNative();
        scope.canEdit = $appstate.canEdit(scope.item);
        scope.canSendToPortfolio = $appService.canSendItemsToPortfolio() && !scope.item.portfolio;
        scope.canAddObservations = !scope.isParent;
        // Get a display-friendly group name for the header
        scope.item.eventType.groupName = _.find(facility.eventTypes, function (et) {
          return scope.item.eventType.group === et.group;
        }).groupName;
        $stateStandards.getPerformanceRatingLabels(facility.id).then(function (result) {
          scope.progressLevels = [
            result['statestandards.labels.not-observed'],
            result['statestandards.labels.emerging'],
            result['statestandards.labels.progressing'],
            result['statestandards.labels.skilled']
          ];
        });
        scope.showFb = true;
        scope.isCollapsed = true;
        $appstate.currentPDF = '';
        // TODO: Refactor this into a function that is called - also: harden logic if necessary
        if (scope.isParent) {
          scope.canDelete = !scope.item.draftId && scope.canEdit;
        } else {
          var isAdminMode = delMode === 'DISABLED' && $facility.isAdmin();
          scope.canDelete = !scope.item.draftId && scope.canEdit && (delMode === 'ALL' || delMode === 'BY_TEACHER' && usr.canDeleteEvents || isAdminMode);
        }
        // Functions
        /**
         * @function deleteItem()
         * @param item
         * @returns {void}
         */
        scope.deleteItem = function (item, $event) {
          if (scope.canDelete) {
            $messageBox.open('Delete Item', 'This will permanently delete the event. This cannot be undone. Are you sure you wish to continue?', {
              oktext: 'Yes',
              canceltext: 'No'
            }).result.then(function () {
              $events.deleteEvent(item, function (error) {
                if (error) {
                  console.log(error);
                  $messageBox.open('Error', 'An error occurred, the event was not deleted!', { oktext: 'Ok' });
                } else {
                  $messageBox.open('Item Deleted', 'Event Deleted Successfully!', { oktext: 'Ok' });
                  $appstate.refreshEvents($appstate.model.observed);
                }
              });
            }, function () {
              $appstate.msg('Delete Cancelled');
            });
          }
          $event.stopPropagation();
        };
        /**
         * @function editItem()
         * @description Opens Edit Item model for item
         * @param {Object} item
         * @returns {void}
         */
        scope.editItem = function (item, $event) {
          $appstate.model.isDashActivityEdit = true;
          $appstate.viewEvent(item, scope.isParent);
          $event.stopPropagation();
        };
        /**
         * @function fb()
         * @param {Object} item
         * @returns {void}
         */
        scope.fb = function (item) {
          $facebook.sharePhoto(item.url);
        };
        /**
         * @function fbVid()
         * @param {Object} item
         * @returns {void}
         */
        scope.fbVid = function (item) {
          $facebook.shareVideo(item.url, null, item.thumbnail);
        };
        /**
         * @function getPortfolioText()
         * @returns {string}
         */
        scope.getPortfolioText = function () {
          if ($krConfig.isNZ()) {
            return 'Observations';
          } else {
            return $translate.instant('Portfolio');
          }
        };
        /**
         * @function sendToPortfolio()
         * @param {Object} item
         * @returns {void}
         */
        scope.sendToPortfolio = function (item, $event) {
          $events.sendToPortfolio(item).then(function () {
            $appstate.msg('Saved to Portfolio!', true);
            $timeout(function () {
              $appstate.refreshEvents($appstate.model.observed, null, true);
            }, 2000);
          });
          $event.stopPropagation();
        };
        /**
         * @function setImageUrl()
         * @param {string} url
         * @returns {void}
         */
        scope.setImageUrl = function (url) {
          $appstate.model.imageurl = url;
        };
        /**
         * @function setVidUrl()
         * @param {string} url
         */
        scope.setVidUrl = function (url) {
          if ($native.isNative()) {
            forge.media.videoPlay(url, function () {
            }, function () {
            });
          }
        };
        /**
         * @function viewItem()
         * @deprecated
         * @param item
         * @returns {void}
         */
        scope.viewItem = function (item) {
          popover.hide();
          $appstate.showEvent(item);
        };
        /**
         * @function addObservations()
         * @param item
         * @returns {void}
         */
        scope.addObservations = function (item, $event) {
          $appstate.viewEventObservations(item);
          $event.stopPropagation();
        };
        scope.isNumber = function (test) {
          return typeof test === 'number';
        };
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').directive('renderItemMedia', [
  '$backend',
  '$native',
  '$facebook',
  '$model',
  '$facility',
  '$activitiesService',
  '$memoriesService',
  '$uiAlert',
  function ($backend, $native, $facebook, $model, $facility, $activitiesService, $memoriesService, $uiAlert) {
    return {
      templateUrl: 'views/directives/renderItemMedia.html',
      restrict: 'E',
      scope: {
        item: '=',
        showlinks: '@',
        clickable: '@'
      },
      link: function (scope, element, attrs, $scope) {
        var _ = window._;
        var facility = $facility.getSync();
        scope.currentUser = $backend.getCurrentUser();
        scope.isParent = scope.currentUser.role == 'PARENT_ROLE';
        scope.isNative = $native.isNative();
        // comment out for now.
        scope.showFb = $facility.getSync().facebookEnabled;
        scope.setImageUrl = function (url) {
          if (scope.clickable) {
            var item = _.get(scope, 'item');
            $activitiesService.viewActivity(item, _.get(item, 'child'));
          }
        };
        scope.setVidUrl = function (url) {
          if ($native.isNative()) {
            forge.media.videoPlay(url, function () {
            }, function (err) {
            });
          } else {
          }
        };
        scope.fb = function (item, evt) {
          $facebook.sharePhoto(item.url);
        };
        scope.fbVid = function (item, evt) {
          $facebook.shareVideo(item.url, null, item.thumbnail);
        };
        /**
         * Allows viewing if facility matches - or the media is new and local
         * @param {object} media
         * @returns {boolean}
         */
        scope.allowView = function (media) {
          var local = _.get(media, 'local');
          var facilityIdx = media.url.indexOf('/' + facility.id + '/');
          return facilityIdx > -1 || local;
        };
        scope.share = function (event, src, type) {
          event.stopPropagation();
          $memoriesService.shareMedia({
            type: type,
            src: src
          }, function (error) {
            if (error) {
              $uiAlert.pushErrorAlert('There was an error sharing media. Please try again.', 'MEMORIES');
            }
          });
        };
      }
    };
  }
]);
'use strict';
angular.module('kidreports2App').directive('krSvg', [
  '$cacheFactory',
  '$rootScope',
  function ($cacheFactory, $rootScope) {
    return {
      restrict: 'E',
      template: '<img></img>',
      link: function (scope, element, attrs) {
        var svgRequestCompleteEvent = 'svgRequestComplete';
        var svgRequestPendingStr = 'PENDING';
        var svgCache = $cacheFactory.get('svg') || $cacheFactory('svg');
        var cachedSvg = svgCache.get(attrs.src);
        if (cachedSvg) {
          if (cachedSvg === svgRequestPendingStr) {
            // cache miss but the AJAX request was already fired off by another element -- wait for it
            var killListener = $rootScope.$on(svgRequestCompleteEvent, function (event, src, svg) {
                if (src === attrs.src) {
                  replaceElement(element, svg.cloneNode(true), attrs);
                  killListener();
                }
              });
          } else {
            // cache hit
            replaceElement(element, JSON.parse(cachedSvg), attrs);
          }
        } else {
          // cache miss
          // mark as pending to prevent redundant requests
          svgCache.put(attrs.src, svgRequestPendingStr);
          jQuery.get(attrs.src, function (data) {
            var svg = data.querySelector('svg');
            svgCache.put(attrs.src, JSON.stringify(svg.outerHTML));
            replaceElement(element, svg, attrs);
            $rootScope.$broadcast(svgRequestCompleteEvent, attrs.src, svg);
          }, 'xml');
        }
      }
    };
    function replaceElement(img, svg, attrs) {
      var $svg = jQuery(svg);
      // Add replaced image's ID to the new SVG
      if (typeof attrs.id !== 'undefined') {
        $svg = $svg.attr('id', attrs.id);
      }
      // Add replaced image's classes to the new SVG
      if (typeof attrs.class !== 'undefined') {
        $svg = $svg.attr('class', attrs.class + ' kr-svg');
      }
      if (typeof attrs.tabindex !== 'undefined') {
        $svg = $svg.attr('tabindex', attrs.tabindex);
      }
      // Remove any invalid XML tags as per http://validator.w3.org
      $svg = $svg.removeAttr('xmlns:a');
      // Check if the viewport is set, else we gonna set it if we can.
      if (!$svg.attr('viewBox') && $svg.attr('height') && $svg.attr('width')) {
        $svg.attr('viewBox', '0 0 ' + $svg.attr('height') + ' ' + $svg.attr('width'));
      }
      // Replace image with new SVG
      jQuery(img).replaceWith($svg);
    }
  }
]);
'use strict';
angular.module('kidreports2App').provider('translations', function () {
  // To add a new language, simply add the file in language.code.json, and add information row here.
  var languages = [
      {
        label: 'English',
        value: 'en'
      },
      {
        label: 'Portuguese',
        value: 'pt-br'
      }
    ];
  var _default = languages[0];
  this.configure = function (translateProvider) {
    translateProvider.useLoader('customLoader');
    translateProvider.preferredLanguage('en');
  };
  this.$get = [
    '$translate',
    function ($translate) {
      var translations = {};
      translations.setLanguage = function (lang) {
        $translate.use(lang.value);
        _default = lang;
      };
      translations.getLanguages = function () {
        return languages;
      };
      translations.getCurrent = function () {
        return _default;
      };
      return translations;
    }
  ];
}).factory('customLoader', [
  '$http',
  '$q',
  'translations',
  function ($http, $q, translations) {
    // return loaderFn
    return function (options) {
      var defer = $q.defer();
      $http.get('scripts/i18n/language.' + options.key + '.json').success(function (l) {
        defer.resolve(l);
      });
      return defer.promise;
    };
  }
]);